/*
		                 LCRZOEX
                             Network toolbox
             Copyright(c) 1999,2000,2001,2002 Laurent Constantin
                                  -----

  Main server    : http://www.laurentconstantin.com/
  Backup servers : http://go.to/laurentconstantin/
                   http://laurentconstantin.est-la.com/
                   http://laurentconstantin.free.fr/
                   http://membres.lycos.fr/lauconstantin/
  [my current email address is on the web servers]

                                  -----
  This file is part of Lcrzoex.

  Lcrzoex is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Lcrzoex is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details (http://www.gnu.org/).

------------------------------------------------------------------------
*/

#include <lcrzo.h>
#include "lcrzoex.h"
#include "shared/shared_toolbox.h"

/*-------------------------------------------------------------*/
/* function declaration */
int main(int argc, char* argv[]);
int lcrzoexmain(int argc, char* argv[]);
int lcrzoex_check_ending_number(lcrzo_conststring progname,
                                lcrzo_bool *phasanendingnumber,
                                lcrzo_uint32 *ptoolnumber);
int lcrzoex_helpchoice(lcrzo_conststring progname);
int lcrzoex_helpchoice_tool_cat(lcrzo_int16 language);
int lcrzoex_helpchoice_tool_subcat(lcrzo_int16 language);
int lcrzoex_helpchoice_tool_choose(lcrzo_int16 language,
				   lcrzo_string levels,
				   lcrzo_int16 level,
				   lcrzo_uint32 minnum,
				   lcrzo_uint32 maxnum,
				   lcrzo_uint32 *ptoolnumber);
int lcrzoex_helpchoice_tool_list(lcrzo_string levels, 
				 lcrzo_uint32 *ptoolnumber);
int lcrzoex_helpchoice_subcat(lcrzo_int16 language);
int lcrzoex_helpchoice_subcat_choose(lcrzo_int16 language,
				     lcrzo_string levels,
				     lcrzo_int16 level,
				     char lastchar,
				     lcrzo_uint8 *pchoice);

/*-------------------------------------------------------------*/
/* main function */
int main(int argc, char* argv[])
{ int ret;

  /* initialize lcrzo */
  lcrzo_init(); 

  /* to check lcrzo compilation parameters*/
  lcrzo_epr(lcrzo_compilparam_check());

  /* if you want to change language */
  /*lcrzo_epr(lcrzo_global_set_language(LCRZO_GLOBAL_LANG_FR));*/
  /*lcrzo_epr(lcrzo_global_set_language(LCRZO_GLOBAL_LANG_EN));*/

  /* the real main function */
  ret=lcrzoexmain(argc, argv);
  
#if defined LCRZODEF_SYSTEM_Unix
  if (ret>255)
  { /* system will truncate it */
    ret=LCRZOEX_ERR_MISC_ERRTOOBIG;
  }
#endif

  /* close lcrzo */
  lcrzo_close();
  return(ret);
}

/*-------------------------------------------------------------*/
int lcrzoexmain(int argc, char* argv[])
{ lcrzo_uint32 toolnumber;
  lcrzo_int16 language;
  lcrzo_bool hasanendingnumber;
  int ac, ret;
  char **av;
  
  /* add error displaying functions */
  lcrzo_epr(lcrzo_global_set_err_displayfunction(&shared_lcrzoex_err_print));

  /* get the current language */
  lcrzo_epr(lcrzo_global_get_language(&language));

  /* check if the program contains an ending number */
  lcrzo_epr(lcrzoex_check_ending_number(argv[0], &hasanendingnumber,
					&toolnumber));

  /* if it does not end with a number, read it from the command line */
  if (!hasanendingnumber)
  { /* we help user to choose the tool he wants */
    if ( argc==1 )
    { return(lcrzoex_helpchoice(argv[0]));
    }
    lcrzo_epr(lcrzo_uint32_init_text(argv[1], &toolnumber));
    ac=argc-1;
    av=argv+1;
  }
  else
  { ac=argc;
    av=argv;
  }

  /* call the wanted tool */
  ret=shared_toolbox__x_main(ac, av, toolnumber);
  if (ret==LCRZOEX_ERR_MISC_BADTOOLNUMBER)
  { lcrzo_epr(shared_toolbox_tool_unknown_print(toolnumber));
  }

  /* reinit lcrzo because tools might have closed it */
  lcrzo_init();
  lcrzo_epr(lcrzo_global_set_err_displayfunction(&shared_lcrzoex_err_print));
  lcrzo_ep(ret);

  return(ret);
}

/*-------------------------------------------------------------*/
int lcrzoex_check_ending_number(lcrzo_conststring progname,
                                lcrzo_bool *phasanendingnumber,
                                lcrzo_uint32 *ptoolnumber)
{ lcrzo_conststring pc;
  char c;
  int ret, i;

  if (phasanendingnumber!=NULL) *phasanendingnumber=LCRZO_FALSE;

  /* program name is too short */
  if (!strlen(progname)) return(LCRZO_ERR_OK);

  /* does not end with a digit, so exit now */
  c=progname[strlen(progname)-1];
  if ( c<'0' || c>'9' ) return(LCRZO_ERR_OK);

  /* find the position of the first non digit character */
  pc=progname;
  for ( i=strlen(progname)-1 ; i>=0 ; i-- )
  { c=progname[i];
    if ( c<'0' || c>'9' )
    { pc=progname+i+1;
      break;
    }
  }
  
  /* now, remove leading zeros */
  while ( strlen(pc) )
  { if ( *pc=='0' ) pc++;
    else break;
  }

  /* try to convert to a number */
  ret=lcrzo_uint32_init_text(pc, ptoolnumber);
  if ( ret==LCRZO_ERR_OK)
  { if (phasanendingnumber!=NULL) *phasanendingnumber=LCRZO_TRUE;
  }

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_helpchoice_tool_list(lcrzo_string levels, 
				 lcrzo_uint32 *ptoolnumber)
{ lcrzo_uint32 toolnumber, maxtoolnumber;
  lcrzo_int16 language, level;
  shared_toolbox_helpmode_cat cat;
  shared_toolbox_helpmode_catheader catheader;

  /* get the current language */
  lcrzo_epr(lcrzo_global_get_language(&language));

  level=1;
  toolnumber=0;
  switch(levels[level]) { 
  case '\0':
    lcrzo_epr(shared_toolbox_helpmode_mainmenu(language, levels));
    break;

  case 'a' :
    lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
    lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 157));
    lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 155));
    lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 2));
    lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 274));
    lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 29));
    lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 32));
    lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 37));
    lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 40));
    lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 48));
    lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 51));
    lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 65));
    lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 68));
    lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 170));
    lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 92));
    lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 168));
    lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 186));
    lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 116));
    lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 182));
    lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 141));
    lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 151));
    lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 172));
    lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels, &toolnumber));
    break;
    
  case 'b' :
    level++;
    switch(levels[level]) { 
    case '\0':
      lcrzo_epr(shared_toolbox_helpmode_catheader_init(&catheader,
						       language, level));
#define C_H(f,e) shared_toolbox_helpmode_catheader_add(&catheader,f,e)
      lcrzo_epr(C_H("informations sur cette machine",
		    "information on this computer"));
      lcrzo_epr(C_H("informations sur une machine distante",
		    "information on other computers"));
      lcrzo_epr(C_H("informations sur un reseau",
		    "information on this network"));
      lcrzo_epr(shared_toolbox_helpmode_catheader_close(&catheader,
							levels));
      break;
    case 'a' :
      lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 1));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 5));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 6));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 154));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 155));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 157));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 158));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 159));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 160));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 173));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 174));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 175));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 278));
      lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels, &toolnumber));
      break;
    case 'b' :
      lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 2));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 156));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 3));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 4));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 176));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 379));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 380));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 381));
      lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels, &toolnumber));
      break;
    case 'c' :
      lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 410));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 134));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 209));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 210));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 211));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 212));
      lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels, &toolnumber));
      break;
    }
    break;
    
  case 'c' :
    level++;
    switch(levels[level]) { 
    case '\0':
      lcrzo_epr(shared_toolbox_helpmode_catheader_init(&catheader,
						       language, level));
      lcrzo_epr(C_H("sniffe des paquets et les affiche",
		    "sniff packets and print them"));
      lcrzo_epr(C_H("sniffe des paquets et les sauve dans un record",
		    "sniff packets and save them in a record"));
      lcrzo_epr(C_H("sniffe des paquets et verifie leur checksum",
		    "sniff packets and verify checksum"));
      lcrzo_epr(C_H("sniffe des paquets et les compte", 
		    "sniff packets and count them"));
      lcrzo_epr(C_H("sniffe des paquets et cherche des donnees", 
		    "sniff packets and search data"));
      lcrzo_epr(C_H("sniffe des paquets et execute une commande", 
		    "sniff packets and execute a command"));
      lcrzo_epr(shared_toolbox_helpmode_catheader_close(&catheader,
							levels));
      break;
    case 'a' :
      lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 7));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 274));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 275));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 8));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 9));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 295));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 296));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 297));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 364));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 370));
      lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels, &toolnumber));
      break;
    case 'b' :
      lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 10));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 11));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 12));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 13));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 14));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 15));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 365));
      lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels, &toolnumber));
      break;
    case 'c' :
      lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 16));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 17));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 18));
      lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels, &toolnumber));
      break;
    case 'd' :
      lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 19));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 20));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 21));
      lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels, &toolnumber));
      break;
    case 'e' :
      lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 353));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 354));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 355));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 356));
      lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels, &toolnumber));
      break;
    case 'f' :
      lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 383));
      lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels, &toolnumber));
      break;
    }
    break;
    
  case 'd' :
    level++;
    switch(levels[level]) { 
    case '\0': 
      lcrzo_epr(shared_toolbox_helpmode_catheader_init(&catheader,
						       language, level));
      lcrzo_epr(C_H("spoof ethernet",
		    "ethernet spoof"));
      lcrzo_epr(C_H("spoof ip",
		    "ip spoof"));
      lcrzo_epr(C_H("spoof udp",
		    "udp spoof"));
      lcrzo_epr(C_H("spoof tcp",
		    "tcp spoof"));
      lcrzo_epr(C_H("spoof icmp",
		    "icmp spoof"));
      lcrzo_epr(C_H("spoof arp/rarp",
		    "arp/rarp spoof"));
      lcrzo_epr(shared_toolbox_helpmode_catheader_close(&catheader,
							levels));
      break;
    case 'a' :
      lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 24));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 25));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 26));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 136));
      lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels, &toolnumber));
      break;
    case 'b' :
      lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 29));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 298));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 30));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 31));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 164));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 32));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 299));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 33));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 34));
      lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels, &toolnumber));
      break;
    case 'c' :
      level++;
      switch(levels[level]) { 
      case '\0':
	lcrzo_epr(shared_toolbox_helpmode_catheader_init(&catheader,
							 language, level));
	lcrzo_epr(C_H("spoof udp generique",
		      "generic udp spoof"));
	lcrzo_epr(C_H("spoof udp syslog",
		      "syslog udp spoof"));
	lcrzo_epr(C_H("spoof udp dns",
		      "dns udp spoof"));
	lcrzo_epr(shared_toolbox_helpmode_catheader_close(&catheader,
							  levels));
	break;
      case 'a' :
	lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 37));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 300));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 38));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 39));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 40));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 301));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 41));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 42));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 135));
	lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels,&toolnumber));
	break;
      case 'b' :
	lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 43));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 44));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 45));
	lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels,&toolnumber));
	break;
      case 'c' :
	lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 327));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 328));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 329));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 330));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 331));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 332));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 333));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 334));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 335));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 336));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 337));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 338));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 339));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 340));
	lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels,&toolnumber));
	break;
      }
      break;
    case 'd' :
      lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 48));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 302));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 49));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 306));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 204));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 50));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 205));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 51));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 303));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 52));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 307));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 206));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 53));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 207));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 54));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 55));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 56));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 57));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 58));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 59));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 60));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 61));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 62));
      lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels, &toolnumber));
      break;
    case 'e' :
      lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 65));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 304));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 66));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 67));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 68));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 305));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 69));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 70));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 71));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 72));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 73));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 74));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 75));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 76));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 77));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 78));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 79));
      lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels, &toolnumber));
      break;
    case 'f' :
      lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 80));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 81));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 82));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 83));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 208));
      lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels, &toolnumber));
      break;
    }
    break;
    
  case 'e' :
    level++;
    switch(levels[level]) { 
    case '\0':
      lcrzo_epr(shared_toolbox_helpmode_catheader_init(&catheader,
						       language, level));
      lcrzo_epr(C_H("Eth record",
		    "Eth record"));
      lcrzo_epr(C_H("IP record",
		    "IP record"));
      lcrzo_epr(C_H("UDP record",
		    "UDP record"));
      lcrzo_epr(C_H("TCP record",
		    "TCP record"));
      lcrzo_epr(C_H("ICMP record",
		    "ICMP record"));
      lcrzo_epr(C_H("Data record",
		    "Data record"));
      lcrzo_epr(C_H("Conversion de fichier pcap",
		    "Conversion of pcap file"));
      lcrzo_epr(shared_toolbox_helpmode_catheader_close(&catheader, levels));
      break;
    case 'a' :
      /*eth*/
      level++;
      switch(levels[level]) { 
      case '\0':
        lcrzo_epr(shared_toolbox_helpmode_catheader_init(&catheader,
							 language, level));
	lcrzo_epr(C_H("Emploie les donnees exactes du record",
		      "Use exact data from record"));
	lcrzo_epr(C_H("Modifie les adresses/ports et recalcule les checksum",
		      "Change addresses/ports and recompute checksums"));
	lcrzo_epr(C_H("Recalcule les tailles/checksum du record",
		      "Recompute sizes/checksums of record"));
	lcrzo_epr(shared_toolbox_helpmode_catheader_close(&catheader, levels));
	break;
      case 'a' :
	lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 22));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 23));
	lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels,&toolnumber));
	break;
      case 'b' :
	lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 221));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 222));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 223));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 224));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 225));
	lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels,&toolnumber));
	break;
      case 'c' :
	lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 400));
	lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels,&toolnumber));
	break;
      }
      break;
    case 'b' :
      /*ip*/
      level++;
      switch(levels[level]) { 
      case '\0':
        lcrzo_epr(shared_toolbox_helpmode_catheader_init(&catheader,
							 language, level));
	lcrzo_epr(C_H("Emploie les donnees exactes du record",
		      "Use exact data from record"));
	lcrzo_epr(C_H("Modifie les adresses/ports et recalcule les checksum",
		      "Change addresses/ports and recompute checksums"));
	lcrzo_epr(C_H("Recalcule les tailles/checksum du record",
		      "Recompute sizes/checksums of record"));
	lcrzo_epr(shared_toolbox_helpmode_catheader_close(&catheader, levels));
	break;
      case 'a' :
	lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 27));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 28));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 230));
	lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels,&toolnumber));
	break;
      case 'b' :
	lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 226));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 227));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 228));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 229));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 231));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 232));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 233));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 234));
	lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels,&toolnumber));
	break;
      case 'c' :
	lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 401));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 402));
	lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels,&toolnumber));
	break;
      }
      break;
    case 'c' :
      /*udp*/
      level++;
      switch(levels[level]) { 
      case '\0':
        lcrzo_epr(shared_toolbox_helpmode_catheader_init(&catheader,
							 language, level));
	lcrzo_epr(C_H("Emploie les donnees exactes du record",
		      "Use exact data from record"));
	lcrzo_epr(C_H("Modifie les adresses/ports et recalcule les checksum",
		      "Change addresses/ports and recompute checksums"));
	lcrzo_epr(C_H("Recalcule les tailles/checksum du record",
		      "Recompute sizes/checksums of record"));
	lcrzo_epr(shared_toolbox_helpmode_catheader_close(&catheader, levels));
	break;
      case 'a' :
	lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 35));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 36));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 236));
	lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels,&toolnumber));
	break;
      case 'b' :
	lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 235));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 237));
	lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels,&toolnumber));
	break;
      case 'c' :
	lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 403));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 404));
	lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels,&toolnumber));
	break;
      }
      break;
    case 'd' :
      /*tcp*/
      level++;
      switch(levels[level]) { 
      case '\0':
        lcrzo_epr(shared_toolbox_helpmode_catheader_init(&catheader,
							 language, level));
	lcrzo_epr(C_H("Emploie les donnees exactes du record",
		      "Use exact data from record"));
	lcrzo_epr(C_H("Modifie les adresses/ports et recalcule les checksum",
		      "Change addresses/ports and recompute checksums"));
	lcrzo_epr(C_H("Recalcule les tailles/checksum du record",
		      "Recompute sizes/checksums of record"));
	lcrzo_epr(shared_toolbox_helpmode_catheader_close(&catheader, levels));
	break;
      case 'a' :
	lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 46));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 47));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 239));
	lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels,&toolnumber));
	break;
      case 'b' :
	lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 238));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 240));
	lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels,&toolnumber));
	break;
      case 'c' :
	lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 405));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 406));
	lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels,&toolnumber));
	break;
      }
      break;
    case 'e' :
      /*icmp*/
      level++;
      switch(levels[level]) { 
      case '\0':
        lcrzo_epr(shared_toolbox_helpmode_catheader_init(&catheader,
							 language, level));
	lcrzo_epr(C_H("Emploie les donnees exactes du record",
		      "Use exact data from record"));
	lcrzo_epr(C_H("Modifie les adresses/ports et recalcule les checksum",
		      "Change addresses/ports and recompute checksums"));
	lcrzo_epr(C_H("Recalcule les tailles/checksum du record",
		      "Recompute sizes/checksums of record"));
	lcrzo_epr(shared_toolbox_helpmode_catheader_close(&catheader, levels));
	break;
      case 'a' :
	lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 63));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 64));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 242));
	lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels,&toolnumber));
	break;
      case 'b' :
	lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 241));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 243));
	lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels,&toolnumber));
	break;
      case 'c' :
	lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 407));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 408));
	lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels,&toolnumber));
	break;
      }
      break;
    case 'f' :
      /*data*/
      lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 390));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 391));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 392));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 393));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 394));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 395));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 396));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 397));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 398));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 399));
      lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels, &toolnumber));
      break;
    case 'g' :
      /*pcap*/
      lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 258));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 259));
      lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels, &toolnumber));
      break;
    }
    break;
    
  case 'f' :
    level++;
    switch(levels[level]) { 
    case '\0':
      lcrzo_epr(shared_toolbox_helpmode_catheader_init(&catheader,
						       language, level));
      lcrzo_epr(C_H("clients udp interactifs",
		    "udp interactive clients"));
      lcrzo_epr(C_H("clients syslog",
		    "syslog clients"));
      lcrzo_epr(C_H("clients dns",
		    "dns clients"));
      lcrzo_epr(shared_toolbox_helpmode_catheader_close(&catheader, levels));
      break;
    case 'a' :
      lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 84));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 308));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 85));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 168));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 309));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 178));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 86));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 87));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 169));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 179));
      lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels, &toolnumber));
      break;
    case 'b' :
      lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 88));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 89));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 90));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 91));
      lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels, &toolnumber));
      break;
    case 'c' :
      lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 341));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 344));
      lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels, &toolnumber));
      break;
      /* jump to this section */
      /*strcpy(levels, "-ecc");
	level=3;
	break;*/
    }
    break;

  case 'g' :
    level++;
    switch(levels[level]) { 
    case '\0':
      lcrzo_epr(shared_toolbox_helpmode_catheader_init(&catheader,
						       language, level));
      lcrzo_epr(C_H("clients tcp interactifs",
		    "tcp interactive clients"));
      lcrzo_epr(C_H("clients tcp affichant la banniere d'accueil du serveur",
		    "clients displaying servers' banner"));
      lcrzo_epr(C_H("clients HTTP (web)",
		    "HTTP (web) clients"));
      lcrzo_epr(C_H("clients FTP",
		    "FTP clients"));
      lcrzo_epr(C_H("clients SMTP (mail)",
		    "SMTP (email) clients"));
      lcrzo_epr(C_H("clients NNTP (newsgroup)",
		    "NNTP (newsgroup) clients"));
      lcrzo_epr(C_H("clients TELNET",
		    "TELNET clients"));
      lcrzo_epr(C_H("clients DNS",
		    "DNS clients"));
      lcrzo_epr(shared_toolbox_helpmode_catheader_close(&catheader, levels));
      break;
    case 'a' :
      lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 92));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 312));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 93));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 167));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 170));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 313));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 180));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 94));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 95));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 171));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 181));
      lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels, &toolnumber));
      break;
    case 'b' :
      lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 100));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 101));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 102));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 103));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 137));
      lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels, &toolnumber));
      break;
    case 'c' :
      level++;
      switch(levels[level]) {
      case '\0':
        lcrzo_epr(shared_toolbox_helpmode_catheader_init(&catheader,
							 language, level));
	lcrzo_epr(C_H("affiche une page HTML",
		      "display an HTML page"));
	lcrzo_epr(C_H("telecharge une page HTML",
		      "download an HTML page"));
	lcrzo_epr(C_H("telecharge un site (web spider)",
		      "download a web site (web spider)"));
	lcrzo_epr(C_H("divers",
		      "miscellaneous"));
	lcrzo_epr(shared_toolbox_helpmode_catheader_close(&catheader, levels));
	break;
      case 'a' :
	lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 96));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 97));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 190));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 194));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 98));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 99));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 191));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 195));
	lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat,levels,&toolnumber));
	break;
      case 'b' :
	lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 294));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 192));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 196));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 193));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 197));
	lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat,levels,&toolnumber));
	break;
      case 'c' :
	lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 360));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 361));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 362));
	lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat,levels,&toolnumber));
	break;
      case 'd' :
	lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 198));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 200));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 202));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 199));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 201));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 203));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 359));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 382));
	lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat,levels,&toolnumber));
	break;
      }
      break;
    case 'd' :
      lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 104));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 105));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 279));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 281));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 283));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 285));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 287));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 106));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 107));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 280));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 282));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 284));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 286));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 409));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 288));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 165));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 268));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 289));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 290));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 291));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 292));
      lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels, &toolnumber));
      break;
    case 'e' :
      lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 162));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 166));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 261));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 262));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 264));
      lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels, &toolnumber));
      break;
    case 'f' :
      lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 163));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 263));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 265));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 266));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 267));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 293));
      lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels, &toolnumber));
      break;
    case 'g' :
      lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 270));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 271));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 272));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 273));
      lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels, &toolnumber));
      break;
    case 'h' :
      lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 342));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 343));
      lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels, &toolnumber));
      break;
    }
    break;
    
  case 'h' :
    level++;
    switch(levels[level]) {
    case '\0':
      lcrzo_epr(shared_toolbox_helpmode_catheader_init(&catheader,
						       language, level));
      lcrzo_epr(C_H("serveurs udp interactifs",
		    "udp interactive servers"));
      lcrzo_epr(C_H("serveurs udp monoclient",
		    "udp monoclient servers"));
      lcrzo_epr(C_H("serveurs udp multiclient",
		    "udp multiclient servers"));
      lcrzo_epr(C_H("serveurs DNS",
		    "DNS servers"));
      lcrzo_epr(shared_toolbox_helpmode_catheader_close(&catheader, levels));
      break;
    case 'a' :
      lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 108));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 310));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 109));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 182));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 311));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 183));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 110));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 111));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 184));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 185));
      lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels, &toolnumber));
      break;
    case 'b' :
      lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 112));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 113));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 114));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 115));
      lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels, &toolnumber));
      break;
    case 'c' :
      lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 124));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 125));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 346));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 126));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 347));
      lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels, &toolnumber));
      break;
    case 'd' :
      lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 345));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 348));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 349));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 350));
      lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels, &toolnumber));
      break;
    }
    break;
    
  case 'i' :
    level++;
    switch(levels[level]) {
    case '\0':
      lcrzo_epr(shared_toolbox_helpmode_catheader_init(&catheader,
						       language, level));
      lcrzo_epr(C_H("serveurs tcp interactifs",
		    "tcp interactive servers"));
      lcrzo_epr(C_H("serveurs tcp renvoyant les donnees recues",
		    "tcp servers echoing received data"));
      lcrzo_epr(C_H("serveurs web",
		    "web servers"));
      lcrzo_epr(shared_toolbox_helpmode_catheader_close(&catheader, levels));
      break;
    case 'a' :
      lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 116));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 314));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 117));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 186));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 315));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 187));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 118));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 119));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 188));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 189));
      lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels, &toolnumber));
      break;
    case 'b' :
      lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 120));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 121));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 122));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 123));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 127));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 128));
      lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels, &toolnumber));
      break;
    case 'c' :
      lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 260));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 277));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 358));
      lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels, &toolnumber));
      break;
    }
    break;
    
  case 'j' :
    lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
    lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 213));
    lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 214));
    lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 352));
    lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels, &toolnumber));
    break;
    
  case 'k' :
    level++;
    switch(levels[level]) {
    case '\0':
      lcrzo_epr(shared_toolbox_helpmode_catheader_init(&catheader,
						       language, level));
      lcrzo_epr(C_H("backdoor sur TCP",
		    "backdoor over TCP"));
      lcrzo_epr(C_H("backdoor sur TCP (ancienne methode d'authentification)",
		    "backdoor over TCP (old authentication method)"));
      lcrzo_epr(C_H("backdoor web",
		    "web backdoor"));
      lcrzo_epr(shared_toolbox_helpmode_catheader_close(&catheader, levels));
      break;
    case 'a' :
      lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 384));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 385));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 386));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 387));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 388));
      lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels, &toolnumber));
      break;
    case 'b' :
      lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 216));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 276));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 217));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 218));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 219));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 220));
      lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels, &toolnumber));
      break;
    case 'c' :
      lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 260));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 277));
      lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels, &toolnumber));
      break;
    }
    break;

  case 'l' :
    level++;
    switch(levels[level]) {
    case '\0':
      lcrzo_epr(shared_toolbox_helpmode_catheader_init(&catheader,
						       language, level));
      lcrzo_epr(C_H("ping",
		    "ping"));
      lcrzo_epr(C_H("traceroute",
		    "traceroute"));
      lcrzo_epr(C_H("divers",
		    "miscellaneous"));
      lcrzo_epr(shared_toolbox_helpmode_catheader_close(&catheader, levels));
      break;
    case 'a' :
      lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 208));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 248));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 249));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 319));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 250));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 251));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 320));
      lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels, 
						  &toolnumber));
      break;
    case 'b' :
      lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 138));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 172));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 316));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 244));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 245));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 317));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 246));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 247));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 318));
      lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels, 
						  &toolnumber));
      break;
    case 'c' :
      lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 131));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 177));
      lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels, &toolnumber));
      break;
    }
    break;
    
  case 'm' :
    level++;
    switch(levels[level]) {
    case '\0':
      lcrzo_epr(shared_toolbox_helpmode_catheader_init(&catheader,
						       language, level));
      lcrzo_epr(C_H("scanning",
		    "scanning"));
      lcrzo_epr(C_H("attaque reseau",
		    "network attacks"));
      lcrzo_epr(C_H("email usurpe",
		    "spoofed email"));
      lcrzo_epr(C_H("attaque brute force",
		    "brute force attack"));
      lcrzo_epr(shared_toolbox_helpmode_catheader_close(&catheader, levels));
      break;
    case 'a' :
      lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 252));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 253));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 322));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 254));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 255));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 323));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 256));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 257));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 324));
      lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels, 
						  &toolnumber));
      break;
    case 'b' :
      level++;
      switch(levels[level]) {
      case '\0':
	lcrzo_epr(shared_toolbox_helpmode_catheader_init(&catheader,
							 language, level));
	lcrzo_epr(C_H("attaques Ethernet",
		      "Ethernet attacks"));
	lcrzo_epr(C_H("attaques ARP",
		      "ARP attacks"));
	lcrzo_epr(C_H("attaques ICMP",
		      "ICMP attacks"));
	lcrzo_epr(C_H("attaques TCP",
		      "TCP attacks"));
	lcrzo_epr(C_H("attaques DNS",
		      "DNS attacks"));
	lcrzo_epr(shared_toolbox_helpmode_catheader_close(&catheader, levels));
	break;
      case 'a' :
	lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 357));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 369));
	lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels, 
						    &toolnumber));
	break;
      case 'b' :
	lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 130));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 131));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 215));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 81));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 367));
	lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels, 
						    &toolnumber));
	break;
      case 'c' :
	lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 374));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 375));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 376));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 377));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 378));
	lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels, 
						    &toolnumber));
	break;
      case 'd' :
	lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 129));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 132));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 371));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 269));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 133));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 321));
	lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels, 
						    &toolnumber));
	break;
      case 'e' :
	lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 345));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 349));
	lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 368));
	lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels, 
						    &toolnumber));
	break;
      }
      break;
    case 'c' :
      lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 166));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 262));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 264));
      lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels, &toolnumber));
      break;
    case 'd' :
      lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 202));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 203));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 268));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 273));
      lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels, &toolnumber));
      break;
    }
    break;
    
  case 'n' :
    level++;
    switch(levels[level]) { 
    case '\0':
      lcrzo_epr(shared_toolbox_helpmode_catheader_init(&catheader,
						       language, level));
      lcrzo_epr(C_H("conversion de nombres et caracteres",
		    "conversion of numbers and characters"));
      lcrzo_epr(C_H("conversion d'adresse IP",
		    "conversion of IP addresses"));
      lcrzo_epr(C_H("conversion de chaines",
		    "conversion of strings"));
      lcrzo_epr(C_H("manipulation de fichiers",
		    "conversion and dump of files"));
      lcrzo_epr(C_H("specifique a lcrzoex",
		    "specific to lcrzoex"));
      lcrzo_epr(shared_toolbox_helpmode_catheader_close(&catheader, levels));
      break;
    case 'a' :
      lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 363));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 139));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 140));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 141));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 142));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 143));
      lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels, &toolnumber));
      break;
    case 'b' :
      lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 144));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 161));
      lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels, &toolnumber));
      break;
    case 'c' :
      lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 145));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 146));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 147));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 148));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 152));
      lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels, &toolnumber));
      break;
    case 'd' :
      lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 149));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 150));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 151));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 325));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 326));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 359));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 366));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 372));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 373));
      lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels, &toolnumber));
      break;
    case 'e' :
      lcrzo_epr(shared_toolbox_helpmode_cat_init(&cat, language, level));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 153));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 351));
      lcrzo_epr(shared_toolbox_helpmode_cat_add(&cat, 389));
      lcrzo_epr(shared_toolbox_helpmode_cat_close(&cat, levels, &toolnumber));
      break;
    }
    break;
    
  case 'o' :
    lcrzo_epr(shared_toolbox_tool_search());
    /* to stay in the main loop */
    levels[0]='-';
    levels[1]='\0';
    toolnumber=0;
    break;
 
  case 'p' :
    lcrzo_epr(shared_toolbox_tool_get_max(&maxtoolnumber));
    lcrzo_epr(lcrzo_uint32_init_stdin((language==LCRZO_GLOBAL_LANG_FR)?
				      "Entrez votre numero d'outil":
				      "Enter your tool number", 
				      1, maxtoolnumber, 1, &toolnumber));
    break;
    
  case 'q' :
    lcrzo_epr(shared_toolbox_tool_savefile());
    /* to stay in the main loop */
    levels[0]='-';
    levels[1]='\0';
    toolnumber=0;
    break;
 
  }

  if (ptoolnumber!=NULL) *ptoolnumber=toolnumber;
  return(LCRZO_ERR_OK);
}


/*-------------------------------------------------------------*/
/* this function classify and describes tools.
   this helps users to choose an tool */
int lcrzoex_helpchoice(lcrzo_conststring progname)
{ lcrzo_int8 levels[10];
  lcrzo_uint32 toolnumber;
  lcrzo_int16 language;

  /* get the current language */
  lcrzo_epr(lcrzo_global_get_language(&language));

  /* check if user has admin privileges */
  lcrzo_epr(shared_misc_checkadmin(language));

  if ( language==LCRZO_GLOBAL_LANG_FR )
  { puts("Ces outils ont ete crees avec la bibliotheque lcrzo.");
  }
  else
  { puts("These tools were created with the network library lcrzo.");
  }

  /* loop through menus */
  levels[0]='-';
  levels[1]='\0';
  toolnumber=0;
  while ( levels[0] && (toolnumber==0) )
  { lcrzo_epr(lcrzoex_helpchoice_tool_list(levels, &toolnumber));
  }
  if (levels[0]=='\0') return(LCRZO_ERR_OK);

  /* display */
  lcrzo_epr(shared_toolbox_tool_usage_print(progname, toolnumber));

  return(LCRZO_ERR_OK);
}
