
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000045=
{ /*reference*/     "lcrzoex", 45,
  /*version*/       1, 0,
  /*title*/         "{IP,UDP} spoof of syslog data (source routing)",
  /*french title*/  "spoof {IP,UDP} de donnees syslog (source routing)",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "ip_source_address ip_dest_address source_port router",
  /*french usage*/  "adresse_ip_source adresse_ip_dest port_source routeur",
  /*usage example*/ "1.2.3.4 5.6.7.8 1234 1.2.3.5",
  /*fr. usage ex.*/ "1.2.3.4 5.6.7.8 1234 1.2.3.5", 
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000045_main(int argc, char *argv[]);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000045_main(int argc, char *argv[])
{ lcrzo_data data;
  lcrzo_int32 datasize;
  lcrzo_spoof spoof;
  lcrzo_hdrlip hdrlip;
  lcrzo_hdrludp hdrludp;
  lcrzo_ipopt ipopt;
  lcrzo_uint8 ipoptsize;
  lcrzo_time currenttime;
  lcrzo_ipl ipldest, iplnull;
  lcrzo_uint32 sec, usec;

  lcrzo_init();
  if ( argc<5 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000045, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }

  /* initialize IP header */
  lcrzo_epr(lcrzo_hdrlip_initdefault(&hdrlip));
  lcrzo_epr(shared_para_ipl_init_src(argv[1], &(hdrlip.saddr)));
  lcrzo_epr(shared_para_ipl_init_dst(argv[4], &(hdrlip.daddr)));

  /* initialize IP options */
  lcrzo_epr(lcrzo_ipl_init_hs(argv[2], &ipldest));
  lcrzo_epr(lcrzo_ipl_initdefault(&iplnull));
  lcrzo_epr(lcrzo_ipopt_init_ssrr(ipldest, iplnull, iplnull, iplnull, iplnull,
				  iplnull, iplnull, iplnull, iplnull,
				  1, ipopt, &ipoptsize));

  /* initialize UDP header */
  lcrzo_epr(lcrzo_hdrludp_initdefault(&hdrludp));
  lcrzo_epr(shared_para_port_init_src(argv[3], &hdrludp.sport));
  hdrludp.dport=514;

  /* initialize data */
  lcrzo_epr(lcrzo_data_initm_text("<17>Laurent was ", &data, &datasize));
  lcrzo_epr(lcrzo_time_init_currenttime(&currenttime));
  lcrzo_epr(lcrzo_time_get_secusec(&currenttime, &sec, &usec));
  lcrzo_epr(lcrzo_data_appendm_text("here at ", datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_uint32(sec, "%08X", datasize, 
				      &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text(".", datasize, 
				    &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_uint32(usec, "%08X", datasize,
				      &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text(".", datasize, &data, &datasize));

  /* spoof */
  lcrzo_epr(lcrzo_spoof_init(&spoof));
  lcrzo_epr(lcrzo_spoof_set_printbeforesending(&spoof, LCRZO_TRUE));
  lcrzo_epr(lcrzo_spoof_ipoptudpdata(&spoof, hdrlip, ipopt, ipoptsize,
				     hdrludp, data, (lcrzo_uint16)datasize));
  lcrzo_epr(lcrzo_spoof_close(&spoof));

  lcrzo_er(lcrzo_data_free2(&data));
  lcrzo_close();
  return(LCRZO_ERR_OK);
}
