
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000049=
{ /*reference*/     "lcrzoex", 49,
  /*version*/       1, 1,
  /*title*/         "{IP,TCP} spoof specified by command line (IP options)",
  /*french title*/  "spoof {IP,TCP} specifie sur la ligne de commande (options IP)",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "ip_source_address ip_dest_address source_port dest_port syn ack rst seqnum acknum",
  /*french usage*/  "adresse_ip_source adresse_ip_dest port_source port_dest syn ack rst seqnum acknum",
  /*usage example*/ "1.2.3.4 5.6.7.8 1234 21 1 1 0 43223 493484",
  /*fr. usage ex.*/ "1.2.3.4 5.6.7.8 1234 21 1 1 0 43223 493484", 
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000049_main(int argc, char *argv[]);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000049_main(int argc, char *argv[])
{ lcrzo_data data;
  lcrzo_int32 datasize;
  lcrzo_spoof spoof;
  lcrzo_hdrlip hdrlip;
  lcrzo_hdrltcp hdrltcp;
  lcrzo_ipopt ipopt;
  lcrzo_tcpopt tcpopt;
  lcrzo_uint8 ipoptsize, tcpoptsize;
  lcrzo_int16 language;

  lcrzo_init();
  if ( argc<10 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000049, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }

  /* get language */
  lcrzo_epr(lcrzo_global_get_language(&language));

  /* initialize IP header */
  lcrzo_epr(lcrzo_hdrlip_initdefault(&hdrlip));
  lcrzo_epr(shared_para_ipl_init_src(argv[1], &(hdrlip.saddr)));
  lcrzo_epr(shared_para_ipl_init_dst(argv[2], &(hdrlip.daddr)));

  /* initialize IP options */
  lcrzo_epr(lcrzo_ipopt_init_stdin((language==LCRZO_GLOBAL_LANG_FR)?
			      "options IP":"IP options", "",
			      ipopt, &ipoptsize));

  /* initialize TCP header */
  lcrzo_epr(lcrzo_hdrltcp_initdefault(&hdrltcp));
  lcrzo_epr(shared_para_port_init_src(argv[3], &hdrltcp.sport));
  lcrzo_epr(shared_para_port_init_dst(argv[4], 0, &hdrltcp.dport));
  lcrzo_epr(shared_para_bool_init_tcpsyn(argv[5], &hdrltcp.syn));
  lcrzo_epr(shared_para_bool_init_tcpack(argv[6], &hdrltcp.ack));
  lcrzo_epr(shared_para_bool_init_tcprst(argv[7], &hdrltcp.rst));
  lcrzo_epr(shared_para_uint32_init_tcpseqnum(argv[8], &hdrltcp.seqnum));
  lcrzo_epr(shared_para_uint32_init_tcpacknum(argv[9], &hdrltcp.acknum));

  /* initialize TCP options */
  lcrzo_epr(lcrzo_tcpopt_init_stdin((language==LCRZO_GLOBAL_LANG_FR)?
			       "options TCP":"TCP options", "",
			       tcpopt, &tcpoptsize));

  /* initialize data */
  lcrzo_epr(lcrzo_data_initm_mixed_stdin((language==LCRZO_GLOBAL_LANG_FR)?
					 "donnees du paquet":"packet's data",
					 "'hello' 0D0A", &data, &datasize));

  /* spoof */
  lcrzo_epr(lcrzo_spoof_init(&spoof));
  lcrzo_epr(lcrzo_spoof_set_printbeforesending(&spoof, LCRZO_TRUE));
  lcrzo_epr(lcrzo_spoof_ipopttcpoptdata(&spoof, hdrlip, ipopt, ipoptsize,
					hdrltcp, tcpopt, tcpoptsize,
					data, (lcrzo_uint16)datasize));
  lcrzo_epr(lcrzo_spoof_close(&spoof));

  lcrzo_er(lcrzo_data_free2(&data));
  lcrzo_close();
  return(LCRZO_ERR_OK);
}
