
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000052=
{ /*reference*/     "lcrzoex", 52,
  /*version*/       1, 1,
  /*title*/         "{ETH,IP,TCP} spoof specified by command line (IP/TCP options)",
  /*french title*/  "spoof {ETH,IP,TCP} specifie sur la ligne de commande (options IP/TCP)",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "device src_eth dst_eth src_ip dst_ip port_src port_dst syn ack rst seqnum acknum",
  /*french usage*/  "device eth_src eth_dst ip_src ip_dst src_port dst_port syn ack rst seqnum acknum",
  /*usage example*/ "eth0 aa:bb:cc:dd:ee:ff 11:22:33:44:55:66 1.2.3.4 5.6.7.8 1234 21 1 1 0 43223 493484",
  /*fr. usage ex.*/ "eth0 aa:bb:cc:dd:ee:ff 11:22:33:44:55:66 1.2.3.4 5.6.7.8 1234 21 1 1 0 43223 493484", 
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000052_main(int argc, char *argv[]);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000052_main(int argc, char *argv[])
{ lcrzo_data data;
  lcrzo_int32 datasize;
  lcrzo_spoof spoof;
  lcrzo_hdrleth hdrleth;
  lcrzo_hdrlip hdrlip;
  lcrzo_hdrltcp hdrltcp;
  lcrzo_ipopt ipopt;
  lcrzo_tcpopt tcpopt;
  lcrzo_uint8 ipoptsize, tcpoptsize;
  lcrzo_device device;
  lcrzo_int16 language;

  lcrzo_init();
  if ( argc<13 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000052, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }

  /* get language */
  lcrzo_epr(lcrzo_global_get_language(&language));

  /* initialize device */
  lcrzo_epr(shared_para_device_init_src(argv[1], device));

  /* initialize Ethernet header */
  lcrzo_epr(lcrzo_hdrleth_initdefault(&hdrleth));
  lcrzo_epr(shared_para_etha_init_src(argv[2], hdrleth.src));
  lcrzo_epr(shared_para_etha_init_dst(argv[3], hdrleth.dst));

  /* initialize IP header */
  lcrzo_epr(lcrzo_hdrlip_initdefault(&hdrlip));
  lcrzo_epr(shared_para_ipl_init_src(argv[4], &(hdrlip.saddr)));
  lcrzo_epr(shared_para_ipl_init_dst(argv[5], &(hdrlip.daddr)));

  /* initialize IP options */
  lcrzo_epr(lcrzo_ipopt_init_stdin((language==LCRZO_GLOBAL_LANG_FR)?
			      "options IP":"IP options", "",
			      ipopt, &ipoptsize));

  /* initialize TCP header */
  lcrzo_epr(lcrzo_hdrltcp_initdefault(&hdrltcp));
  lcrzo_epr(shared_para_port_init_src(argv[6], &hdrltcp.sport));
  lcrzo_epr(shared_para_port_init_dst(argv[7], 0, &hdrltcp.dport));
  lcrzo_epr(shared_para_bool_init_tcpsyn(argv[8], &hdrltcp.syn));
  lcrzo_epr(shared_para_bool_init_tcpack(argv[9], &hdrltcp.ack));
  lcrzo_epr(shared_para_bool_init_tcprst(argv[10], &hdrltcp.rst));
  lcrzo_epr(shared_para_uint32_init_tcpseqnum(argv[11], &hdrltcp.seqnum));
  lcrzo_epr(shared_para_uint32_init_tcpacknum(argv[12], &hdrltcp.acknum));

  /* initialize TCP options */
  lcrzo_epr(lcrzo_tcpopt_init_stdin((language==LCRZO_GLOBAL_LANG_FR)?
			       "options TCP":"TCP options", "",
			       tcpopt, &tcpoptsize));

  /* initialize data */
  lcrzo_epr(lcrzo_data_initm_mixed_stdin((language==LCRZO_GLOBAL_LANG_FR)?
					 "donnees du paquet":"packet's data",
					 "'hello' 0D0A", &data, &datasize));

  /* spoof */
  lcrzo_epr(lcrzo_spoof_init(&spoof));
  lcrzo_epr(lcrzo_spoof_set_printbeforesending(&spoof, LCRZO_TRUE));
  lcrzo_epr(lcrzo_spoof_ethipopttcpoptdata(&spoof, device, hdrleth,
					   hdrlip, ipopt, ipoptsize,
					   hdrltcp, tcpopt, tcpoptsize,
					   data, (lcrzo_uint16)datasize));
  lcrzo_epr(lcrzo_spoof_close(&spoof));

  lcrzo_epr(lcrzo_data_free2(&data));
  lcrzo_close();
  return(LCRZO_ERR_OK);
}
