
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000075=
{ /*reference*/     "lcrzoex", 75,
  /*version*/       1, 1,
  /*title*/         "{ETH,IP,ICMP} spoof of echo request (ssrr option)",
  /*french title*/  "spoof {ETH,IP,ICMP} d'un echo request (option ssrr)",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "device src_eth dst_eth src_ip dst_ip router",
  /*french usage*/  "device eth_src eth_dst ip_src ip_dst routeur",
  /*usage example*/ "eth0 aa:bb:cc:dd:ee:ff 11:22:33:44:55:66 1.2.3.4 5.6.7.8 1.2.3.5",
  /*fr. usage ex.*/ "eth0 aa:bb:cc:dd:ee:ff 11:22:33:44:55:66 1.2.3.4 5.6.7.8 1.2.3.5",
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000075_main(int argc, char *argv[]);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000075_main(int argc, char *argv[])
{ lcrzo_data data;
  lcrzo_int32 datasize;
  lcrzo_spoof spoof;
  lcrzo_hdrleth hdrleth;
  lcrzo_hdrlip hdrlip;
  lcrzo_hdrlicmp hdrlicmp;
  lcrzo_ipopt ipopt;
  lcrzo_uint8 ipoptsize;
  lcrzo_device device;
  lcrzo_ipl ipldest, iplnull;

  lcrzo_init();
  if ( argc<7 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000075, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }

  /* initialize device */
  lcrzo_epr(shared_para_device_init_src(argv[1], device));

  /* initialize Ethernet header */
  lcrzo_epr(lcrzo_hdrleth_initdefault(&hdrleth));
  lcrzo_epr(shared_para_etha_init_src(argv[2], hdrleth.src));
  lcrzo_epr(shared_para_etha_init_dst(argv[3], hdrleth.dst));

  /* initialize IP header */
  lcrzo_epr(lcrzo_hdrlip_initdefault(&hdrlip));
  lcrzo_epr(shared_para_ipl_init_src(argv[4], &(hdrlip.saddr)));
  lcrzo_epr(shared_para_ipl_init_dst(argv[6], &(hdrlip.daddr)));

  /* initialize IP options */
  lcrzo_epr(shared_para_ipl_init_src(argv[5], &ipldest));
  lcrzo_epr(lcrzo_ipl_initdefault(&iplnull));
  lcrzo_epr(lcrzo_ipopt_init_ssrr(ipldest, iplnull, iplnull, iplnull, iplnull,
                                  iplnull, iplnull, iplnull, iplnull,
                                  1, ipopt, &ipoptsize));

  /* initialize ICMP header */
  lcrzo_epr(lcrzo_hdrlicmp_initdefault(&hdrlicmp));
  hdrlicmp.type=8;

  /* initialize data */
  lcrzo_epr(lcrzo_data_initm_mixed("'hello' 0D0A", &data, &datasize));

  /* spoof */
  lcrzo_epr(lcrzo_spoof_init(&spoof));
  lcrzo_epr(lcrzo_spoof_set_printbeforesending(&spoof, LCRZO_TRUE));
  lcrzo_epr(lcrzo_spoof_ethipopticmpdata(&spoof, device, hdrleth,
					 hdrlip, ipopt, ipoptsize,
					 hdrlicmp, data, (lcrzo_uint16)datasize));
  lcrzo_epr(lcrzo_spoof_close(&spoof));

  lcrzo_epr(lcrzo_data_free2(&data));
  lcrzo_close();
  return(LCRZO_ERR_OK);
}
