
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000081=
{ /*reference*/     "lcrzoex", 81,
  /*version*/       1, 0,
  /*title*/         "{ETH,ARP} spoof of an ARP reply",
  /*french title*/  "spoof {ETH,ARP} d'une reponse ARP",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "device ip_asked eth_asked [eth_to_which_answer [ip_to_which_answer]",
  /*french usage*/  "device ip_demandee eth_reponse [eth_a_qui_repond [ip_a_qui_repond]]",
  /*usage example*/ "eth0 1.2.3.5 aa:bb:cc:dd:ee:ff 11:22:33:44:55:66",
  /*fr. usage ex.*/ "eth0 1.2.3.5 aa:bb:cc:dd:ee:ff 11:22:33:44:55:66",
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000081_main(int argc, char *argv[]);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000081_main(int argc, char *argv[])
{ lcrzo_spoof spoof;
  lcrzo_hdrleth hdrleth;
  lcrzo_hdrlarp hdrlarp;
  lcrzo_device device;
  lcrzo_ipl ipl;

  lcrzo_init();
  if ( argc<4 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000081, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }

  /* initialize device */
  lcrzo_epr(shared_para_device_init_src(argv[1], device));

  /* initialize Ethernet header */
  lcrzo_epr(lcrzo_hdrleth_initdefault(&hdrleth));
  lcrzo_epr(shared_para_etha_init_src(argv[3], hdrleth.src));
  if (argc>=5)
  { lcrzo_epr(shared_para_etha_init_dst(argv[4], hdrleth.dst));
  }
  else
  { lcrzo_epr(lcrzo_etha_init(0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
			      hdrleth.dst));
  }
  hdrleth.type=LCRZO_HDRLETH_TYPE_ARP;

  /* initialize ARP header */
  lcrzo_epr(lcrzo_hdrlarp_initdefault(&hdrlarp));
  hdrlarp.op = 2;
  memcpy(hdrlarp.hw_src, hdrleth.src, LCRZO_ETHA_MAXBYTES);
  lcrzo_epr(shared_para_ipl_init_src(argv[2], &ipl));
  lcrzo_epr(lcrzo_ipa_init_ipl(ipl, hdrlarp.prot_src));
  memcpy(hdrlarp.hw_dst, hdrleth.dst, LCRZO_ETHA_MAXBYTES);
  if (argc==6)
  { lcrzo_epr(shared_para_ipl_init_src(argv[5], &ipl));
    lcrzo_epr(lcrzo_ipa_init_ipl(ipl, hdrlarp.prot_dst));
  }
  else
  { lcrzo_epr(lcrzo_ipa_init(0,0,0,0, hdrlarp.prot_dst));
  }

  /* spoof */
  lcrzo_epr(lcrzo_spoof_init(&spoof));
  lcrzo_epr(lcrzo_spoof_set_printbeforesending(&spoof, LCRZO_TRUE));
  lcrzo_epr(lcrzo_spoof_etharp(&spoof, device, hdrleth, hdrlarp));
  lcrzo_epr(lcrzo_spoof_close(&spoof));

  lcrzo_close();
  return(LCRZO_ERR_OK);
}
