
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000087=
{ /*reference*/     "lcrzoex", 87,
  /*version*/       1, 0,
  /*title*/         "virtual udp client [stdin] (IP option)",
  /*french title*/  "client udp virtuel [stdin] (option IP)",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "device clieth serveth cliname servname cliport servport",
  /*french usage*/  "device ethcli ethserv nomcli nomserv portcli portserv",
  /*usage example*/ "eth0 aa:bb:cc:dd:ee:ff 11:22:33:44:55:66 1.2.3.4 1.2.3.5 4444 21",
  /*fr. usage ex.*/ "eth0 aa:bb:cc:dd:ee:ff 11:22:33:44:55:66 1.2.3.4 1.2.3.5 4444 21",
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000087_main(int argc, char *argv[]);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000087_main(int argc, char *argv[])
{ lcrzo_etha servereth, clienteth;
  lcrzo_ipl serveripl, clientipl;
  lcrzo_uint16 serverport, clientport;
  lcrzo_sock sock;
  lcrzo_ipopt ipopt;
  lcrzo_uint8 ipoptsize;
  lcrzo_device device;
  lcrzo_int16 language;

  lcrzo_init();
  if ( argc<8 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000087, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }

  lcrzo_epr(shared_para_device_init_src(argv[1], device));

  /* initialize Ethernet addresses */
  lcrzo_epr(shared_para_etha_init_src(argv[2], clienteth));
  lcrzo_epr(shared_para_etha_init_dst(argv[3], servereth));

  /* initialize IP addresses */
  lcrzo_epr(shared_para_ipl_init_src(argv[4], &clientipl));
  lcrzo_epr(shared_para_ipl_init_dst(argv[5], &serveripl));

  /* initialize IP options */
  lcrzo_epr(lcrzo_global_get_language(&language));
  lcrzo_epr(lcrzo_ipopt_init_stdin((language==LCRZO_GLOBAL_LANG_FR)?
				   "options IP":"IP options", "",
				   ipopt, &ipoptsize));

  /* initialize port numbers */
  lcrzo_epr(shared_para_port_init_src(argv[6], &clientport));
  lcrzo_epr(shared_para_port_init_dst(argv[7], 0, &serverport));

  /* initialize the socket */
  lcrzo_epr(lcrzo_sock_udpcli_virt2(device,
				    clienteth, servereth,
				    clientipl, serveripl, 
				    clientport, serverport,
				    ipopt, ipoptsize,
				    &sock));
 
  /* call the example working on the socket */
  lcrzo_epr(shared_sock_stdin(&sock, LCRZO_FALSE));

  lcrzo_close();
  return(LCRZO_ERR_OK);
}
