
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000097=
{ /*reference*/     "lcrzoex", 97,
  /*version*/       1, 1,
  /*title*/         "real tcp client [web head] (local ip/port and IP option)",
  /*french title*/  "client tcp reel [web head] (ip/port local et option IP)",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "client_name server_name client_port server_port url [proxyusername [siteusername [proxypasswd [sitepasswd]]]]",
  /*french usage*/  "nom_client nom_serveur port_client port_serveur url [utilisateurproxy [utilisateursite [passwdproxy [passwdsite]]]]",
  /*usage example*/ "1.2.3.5 1.2.3.4 5555 4444 /index.html joe \"\" hello",
  /*fr. usage ex.*/ "1.2.3.5 1.2.3.4 5555 4444 /index.html laurent \"\" bonjour",
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000097_main(int argc, char *argv[]);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000097_main(int argc, char *argv[])
{ lcrzo_ipl serveripl, clientipl;
  lcrzo_hs hs;
  lcrzo_uint16 serverport, clientport;
  lcrzo_sock sock;
  lcrzo_ipopt ipopt;
  lcrzo_uint8 ipoptsize;
  lcrzo_string proxyuser, proxypasswd, siteuser, sitepasswd, file;
  lcrzo_uint16 errornumber;
  lcrzo_string errormsg, servertype, contenttype, location;
  lcrzo_data header;
  lcrzo_int32 headersize, contentlength;
  lcrzo_int16 language;

  lcrzo_init();
  if ( argc<6 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000097, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }

  /* the default resolver timeout is too short if it's long to reach
     webserver's DNS */
  lcrzo_epr(shared_http_set_resolver_timeout());

  /* initialize the needed variables */
  lcrzo_epr(shared_para_ipl_init_src(argv[1], &clientipl));
  lcrzo_epr(shared_para_hs_init_dst(argv[2], hs));
  lcrzo_epr(lcrzo_ipl_init_hs(hs, &serveripl));
  lcrzo_epr(shared_para_port_init_src(argv[3], &clientport));
  lcrzo_epr(shared_para_port_init_dst(argv[4], 80, &serverport));
  lcrzo_epr(shared_para_string_init_filedistant(argv[5], &file));
  lcrzo_epr(lcrzo_global_get_language(&language));
  lcrzo_epr(lcrzo_ipopt_init_stdin((language==LCRZO_GLOBAL_LANG_FR)?
				   "options IP":"IP options", "",
				   ipopt, &ipoptsize));

  /* init username and passwords */
  lcrzo_epr(shared_http_set_proxysite_userpass(argc, argv, 6,
					       &proxyuser, &proxypasswd, 
					       &siteuser, &sitepasswd))

  /* open the connection */
  lcrzo_epr(lcrzo_sock_tcpcli_real2(clientipl, serveripl,
				    clientport, serverport,
				    ipopt, ipoptsize,
				    &sock));
    
  /* send a request */
  lcrzo_epr(shared_http_request_head(&sock, file, hs, serverport,
				     proxyuser, proxypasswd, 
				     siteuser, sitepasswd));
  /* free variables */
  lcrzo_epr(lcrzo_string_free2(&file));
  if (proxyuser!=NULL) { lcrzo_epr(lcrzo_string_free2(&proxyuser)); }
  if (proxypasswd!=NULL) { lcrzo_epr(lcrzo_string_free2(&proxypasswd)); }
  if (siteuser!=NULL) { lcrzo_epr(lcrzo_string_free2(&siteuser)); }
  if (sitepasswd!=NULL) { lcrzo_epr(lcrzo_string_free2(&sitepasswd)); }

  /* analyze the answer */
  lcrzo_epr(shared_http_response_readheader(&sock,
					    &errornumber,
					    &errormsg,
					    &servertype,
					    &contenttype,
					    &contentlength,
					    &location,
					    &header, &headersize));
  printf("Return %d (%s)\n", errornumber, errormsg);
  printf("Server: %s\n", servertype);
  printf("Content-Type: %s\n", contenttype);
  printf("Location: %s\n", location);
  lcrzo_epr(lcrzo_int32_print("Content-length: ", contentlength, "%d", "\n"));
  lcrzo_epr(lcrzo_data_print(header, headersize, LCRZO_PRINTTYPE_DUMP));
  /* free variables */
  lcrzo_epr(lcrzo_string_free2(&errormsg));
  lcrzo_epr(lcrzo_string_free2(&servertype));
  lcrzo_epr(lcrzo_string_free2(&contenttype));
  lcrzo_epr(lcrzo_string_free2(&location));
  lcrzo_epr(lcrzo_data_free2(&header));

  lcrzo_epr(lcrzo_sock_close(&sock));

  lcrzo_close();
  return(LCRZO_ERR_OK);
}
