
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000104=
{ /*reference*/     "lcrzoex", 104,
  /*version*/       1, 2,
  /*title*/         "active ftp displaying a file (real tcp client)",
  /*french title*/  "ftp actif affichant un fichier (client tcp reel)",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "cliname sername cliport serport clidataport file",
  /*french usage*/  "nomcli nomserv portcli portserv portdonneescli fichier",
  /*usage example*/ "1.2.3.4 1.2.3.5 6666 21 7777 /etc/hosts",
  /*fr. usage ex.*/ "1.2.3.4 1.2.3.5 6666 21 7777 /etc/hosts",
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000104_main(int argc, char *argv[]);
int lcrzoex_000104_child(lcrzo_ptr infos, 
			 lcrzo_int32 infossize, 
			 lcrzo_process *pparentprocess);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000104_main(int argc, char *argv[])
{ shared_sock_address ssa;
  lcrzo_sock sock;
  lcrzo_process child;
  lcrzo_string filedistant;
  int ret;

  lcrzo_init();
  if ( argc<7 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000104, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }

  /* initialize IP addresses */
  lcrzo_epr(shared_para_ipl_init_src(argv[1], &ssa.clientipl));
  lcrzo_epr(shared_para_ipl_init_dst(argv[2], &ssa.serveripl));

  /* initialize ports */
  lcrzo_epr(shared_para_port_init_src(argv[3], &ssa.clientport));
  lcrzo_epr(shared_para_port_init_dst(argv[4], 21, &ssa.serverport));
  lcrzo_epr(shared_para_port_init_ftpdatalocal(argv[5], &ssa.clientport2));

  /* initialize filename */
  lcrzo_epr(shared_para_string_init_filedistant(argv[6], &filedistant));

  /* initialize control connection, and send control commands */
  lcrzo_epr(lcrzo_sock_tcpcli_real2(ssa.clientipl, ssa.serveripl,
				    ssa.clientport, ssa.serverport, 
				    NULL, 0, &sock));

  /* we connect */
  lcrzo_epr(shared_ftp_login_anonymous(&sock));

  /* initialize data connection, and listen (in a child process) */
  lcrzo_epr(lcrzo_process_other_init(&lcrzoex_000104_child, 
				     (lcrzo_data)&ssa, sizeof(ssa), &child));
  lcrzo_epr(lcrzo_time_sleep_msec(500));

  /* then, we send a 'PORT' */
  lcrzo_epr(shared_ftp_cmd_port(&sock, ssa.clientipl, ssa.clientport2));

  /* then, we send a 'RETR' */
  lcrzo_epr(shared_ftp_cmd_retr(&sock, filedistant));
  lcrzo_epr(lcrzo_string_free2(&filedistant));

  /* wait the child */
  lcrzo_epr(lcrzo_process_other_wait_end(&child, LCRZO_WAIT_INFINITE, 
					 NULL, &ret));
  lcrzo_epr(lcrzo_process_close(&child));
  lcrzo_epr(ret);
  lcrzo_epr(shared_ftp_cmd_datasock_waitend(&sock));

  /* we send a 'QUIT' */
  lcrzo_epr(shared_ftp_cmd_quit(&sock));

  /* close the socket */
  lcrzo_epr(lcrzo_sock_close(&sock));

  lcrzo_close();
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000104_child(lcrzo_ptr infos, 
			 lcrzo_int32 infossize, 
			 lcrzo_process *pparentprocess)
{ shared_sock_address *pssa=(shared_sock_address *)infos;
  lcrzo_sock sock;
  lcrzo_data data;
  lcrzo_int32 datasize;
  int ret;

  /* check for correct usage of this function */
  if (infossize!=sizeof(shared_sock_address))
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);

  /* close parent process handle because we do not need it */
  lcrzo_epr(lcrzo_process_close(pparentprocess));

  /* create the listening data port */  
  lcrzo_epr(lcrzo_sock_tcpser_real2(pssa->clientipl, pssa->clientport2, NULL,0,
				    &sock));

  /* display received data */
  ret=LCRZO_ERR_OK;
  while(LCRZO_TRUE)
  { ret=lcrzo_sock_readm(&sock, LCRZO_WAIT_INFINITE,
			 &data, &datasize);
    if ( ret!=LCRZO_ERR_OK ) break;
    lcrzo_epr(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_TEXT));
    lcrzo_epr(lcrzo_data_free2(&data));
  }
  lcrzo_epr(lcrzo_sock_close(&sock));
  
  /* this is the normal error */
  if (ret==LCRZO_ERR_OKDATAEND) ret=LCRZO_ERR_OK;
  return(ret);
}
