
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000105=
{ /*reference*/     "lcrzoex", 105,
  /*version*/       1, 2,
  /*title*/         "active ftp displaying a file (virtual tcp client)",
  /*french title*/  "ftp actif affichant un fichier (client tcp virtuel)",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "device clieth sereth cliname sername cliport serport clidataport file",
  /*french usage*/  "device ethcli ethserv nomcli nomserv portcli portserv portdonneescli fichier",
  /*usage example*/ "eth0 aa:bb:cc:dd:ee:ff 11:22:33:44:55:66 1.2.3.4 1.2.3.5 6666 21 7777 /etc/hosts",
  /*fr. usage ex.*/ "eth0 aa:bb:cc:dd:ee:ff 11:22:33:44:55:66 1.2.3.4 1.2.3.5 6666 21 7777 /etc/hosts",
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000105_main(int argc, char *argv[]);
int lcrzoex_000105_child(lcrzo_ptr infos, 
			 lcrzo_int32 infossize, 
			 lcrzo_process *pparentprocess);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000105_main(int argc, char *argv[])
{ shared_sock_address ssa;
  lcrzo_sock sock;
  lcrzo_process child;
  lcrzo_string file;
  int ret;

  lcrzo_init();
  if ( argc<10 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000105, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }

  /* initialize device */
  lcrzo_epr(shared_para_device_init_src(argv[1], ssa.device));

  /* initialize Ethernet addresses */
  lcrzo_epr(shared_para_etha_init_src(argv[2], ssa.clientetha));
  lcrzo_epr(shared_para_etha_init_dst(argv[3], ssa.serveretha));

  /* initialize IP addresses */
  lcrzo_epr(shared_para_ipl_init_src(argv[4], &ssa.clientipl));
  lcrzo_epr(shared_para_ipl_init_dst(argv[5], &ssa.serveripl));

  /* initialize ports */
  lcrzo_epr(shared_para_port_init_src(argv[6], &ssa.clientport));
  lcrzo_epr(shared_para_port_init_dst(argv[7], 21, &ssa.serverport));
  lcrzo_epr(shared_para_port_init_ftpdatalocal(argv[8], &ssa.clientport2));

  /* initialize filename */
  lcrzo_epr(shared_para_string_init_filedistant(argv[9], &file));

  /* initialize control connection, and send control commands */
  ret=lcrzo_sock_tcpcli_virt(ssa.device, ssa.clientetha, ssa.serveretha, 
			     ssa.clientipl, ssa.serveripl,
			     ssa.clientport, ssa.serverport, 
			     &sock);

  /* we connect */
  lcrzo_epr(shared_ftp_login_anonymous(&sock));

  /* initialize data connection, and listen (in a child process) */
  lcrzo_epr(lcrzo_process_other_init(&lcrzoex_000105_child, 
				     (lcrzo_data)&ssa, sizeof(ssa), &child));
  lcrzo_epr(lcrzo_time_sleep_msec(500));

  /* then, we send a 'PORT' */
  lcrzo_epr(shared_ftp_cmd_port(&sock, ssa.clientipl, ssa.clientport2));

  /* then, we send a 'RETR' */
  lcrzo_epr(shared_ftp_cmd_retr(&sock, file));
  lcrzo_epr(lcrzo_string_free2(&file));

  /* wait the child */
  lcrzo_epr(lcrzo_process_other_wait_end(&child, LCRZO_WAIT_INFINITE, 
					 NULL, &ret));
  lcrzo_epr(lcrzo_process_close(&child));
  lcrzo_epr(ret);
  lcrzo_epr(shared_ftp_cmd_datasock_waitend(&sock));

  /* we send a 'QUIT' */
  lcrzo_epr(shared_ftp_cmd_quit(&sock));

  /* close the socket */
  lcrzo_epr(lcrzo_sock_close(&sock));

  lcrzo_close();
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000105_child(lcrzo_ptr infos, 
			 lcrzo_int32 infossize, 
			 lcrzo_process *pparentprocess)
{ shared_sock_address *pssa=(shared_sock_address *)infos;
  lcrzo_sock sock;
  lcrzo_data data;
  lcrzo_int32 datasize;
  int ret;

  /* check for correct usage of this function */
  if (infossize!=sizeof(shared_sock_address))
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);

  /* close parent process handle because we do not need it */
  lcrzo_epr(lcrzo_process_close(pparentprocess));

  /* create the listening data port */  
  lcrzo_epr(lcrzo_sock_tcpser_virt(pssa->device, 
				   pssa->clientetha, pssa->clientipl, 
				   pssa->clientport2, 
				   &sock));

  /* display received data */
  ret=LCRZO_ERR_OK;
  while(LCRZO_TRUE)
  { ret=lcrzo_sock_readm(&sock, LCRZO_WAIT_INFINITE,
			 &data, &datasize);
    if ( ret!=LCRZO_ERR_OK ) break;
    lcrzo_epr(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_TEXT));
    lcrzo_epr(lcrzo_data_free2(&data));
  }
  lcrzo_epr(lcrzo_sock_close(&sock));
  
  /* this is the normal error */
  if (ret==LCRZO_ERR_OKDATAEND) ret=LCRZO_ERR_OK;
  return(ret);
}
