
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000106=
{ /*reference*/     "lcrzoex", 106,
  /*version*/       1, 2,
  /*title*/         "passive ftp displaying a file (real tcp client)",
  /*french title*/  "ftp passif affichant un fichier (client tcp reel)",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "cliname sername cliport serport clidataport file",
  /*french usage*/  "nomcli nomserv portcli portserv portdonneescli fichier",
  /*usage example*/ "1.2.3.4 1.2.3.5 6666 21 7777 /etc/hosts",
  /*fr. usage ex.*/ "1.2.3.4 1.2.3.5 6666 21 7777 /etc/hosts",
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000106_main(int argc, char *argv[]);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000106_main(int argc, char *argv[])
{ shared_sock_address ssa;
  lcrzo_sock sockcontrol, sockdata;
  lcrzo_data data;
  lcrzo_int32 datasize;
  lcrzo_string filedistant;
  int ret;

  lcrzo_init();
  if ( argc<7 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000106, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }

  /* initialize IP addresses */
  lcrzo_epr(shared_para_ipl_init_src(argv[1], &ssa.clientipl));
  lcrzo_epr(shared_para_ipl_init_dst(argv[2], &ssa.serveripl));

  /* initialize ports */
  lcrzo_epr(shared_para_port_init_src(argv[3], &ssa.clientport));
  lcrzo_epr(shared_para_port_init_dst(argv[4], 21, &ssa.serverport));
  lcrzo_epr(shared_para_port_init_ftpdatalocal(argv[5], &ssa.clientport2));

  /* initialize filename */
  lcrzo_epr(shared_para_string_init_filedistant(argv[6], &filedistant));

  /* initialize control connection */
  lcrzo_epr(lcrzo_sock_tcpcli_real2(ssa.clientipl, ssa.serveripl,
				    ssa.clientport, ssa.serverport, 
				    NULL, 0, &sockcontrol));

  /* first, we connect */
  lcrzo_epr(shared_ftp_login_anonymous(&sockcontrol));

  /* then, we send a 'PASV' */
  lcrzo_epr(shared_ftp_cmd_pasv(&sockcontrol, &ssa.serveripl,
				&ssa.serverport2));
  lcrzo_epr(lcrzo_ipl_print("Server IP : ", ssa.serveripl, "\n"));
  printf("Server port : %d\n", ssa.serverport2);

  /* initialize data connection */
  lcrzo_epr(lcrzo_sock_tcpcli_real2(ssa.clientipl, ssa.serveripl, 
				    ssa.clientport2, ssa.serverport2,
				    NULL, 0,
				    &sockdata));

  /* we send a 'RETR' */
  lcrzo_epr(shared_ftp_cmd_retr(&sockcontrol, filedistant));
  lcrzo_epr(lcrzo_string_free2(&filedistant));

  /* display received data */
  while(LCRZO_TRUE)
  { ret=lcrzo_sock_readm(&sockdata, LCRZO_WAIT_INFINITE,
			 &data, &datasize);
    if ( ret!=LCRZO_ERR_OK ) break;
    lcrzo_epr(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_TEXT));
    lcrzo_epr(lcrzo_data_free2(&data));
  }
  lcrzo_epr(lcrzo_sock_close(&sockdata));

  lcrzo_epr(shared_ftp_cmd_datasock_waitend(&sockcontrol));

  /* then, we send a 'QUIT' */
  lcrzo_epr(shared_ftp_cmd_quit(&sockcontrol));

  /* close the socket */
  lcrzo_epr(lcrzo_sock_close(&sockcontrol));

  lcrzo_close();
  return (LCRZO_ERR_OK);
}



