
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000107=
{ /*reference*/     "lcrzoex", 107,
  /*version*/       1, 3,
  /*title*/         "passive ftp displaying a file (virtual tcp client)",
  /*french title*/  "ftp passif affichant un fichier (client tcp virtuel)",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "device clieth sereth cliname sername cliport serport clidataport file",
  /*french usage*/  "device ethcli ethserv nomcli nomserv portcli portserv portdonneescli fichier",
  /*usage example*/ "eth0 aa:bb:cc:dd:ee:ff 11:22:33:44:55:66 1.2.3.4 1.2.3.5 6666 21 7777 /etc/hosts",
  /*fr. usage ex.*/ "eth0 aa:bb:cc:dd:ee:ff 11:22:33:44:55:66 1.2.3.4 1.2.3.5 6666 21 7777 /etc/hosts",
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000107_main(int argc, char *argv[]);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000107_main(int argc, char *argv[])
{ shared_sock_address ssa;
  lcrzo_sock sockcontrol, sockdata;
  lcrzo_data data;
  lcrzo_int32 datasize;
  lcrzo_string file;
  int ret;

  lcrzo_init();
  if ( argc<10 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000107, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }

  /* initialize device */
  lcrzo_epr(shared_para_device_init_src(argv[1], ssa.device));

  /* initialize Ethernet addresses */
  lcrzo_epr(shared_para_etha_init_src(argv[2], ssa.clientetha));
  lcrzo_epr(shared_para_etha_init_dst(argv[3], ssa.serveretha));

  /* initialize IP addresses */
  lcrzo_epr(shared_para_ipl_init_src(argv[4], &ssa.clientipl));
  lcrzo_epr(shared_para_ipl_init_dst(argv[5], &ssa.serveripl));

  /* initialize ports */
  lcrzo_epr(shared_para_port_init_src(argv[6], &ssa.clientport));
  lcrzo_epr(shared_para_port_init_dst(argv[7], 21, &ssa.serverport));
  lcrzo_epr(shared_para_port_init_ftpdatalocal(argv[8], &ssa.clientport2));

  /* initialize filename */
  lcrzo_epr(shared_para_string_init_filedistant(argv[9], &file));

  /* initialize control connection */
  lcrzo_epr(lcrzo_sock_tcpcli_virt(ssa.device, ssa.clientetha, ssa.serveretha,
				   ssa.clientipl, ssa.serveripl,
				   ssa.clientport, ssa.serverport, 
				   &sockcontrol));

  /* first, we connect */
  lcrzo_epr(shared_ftp_login_anonymous(&sockcontrol));

  /* then, we send a 'PASV' */
  lcrzo_epr(shared_ftp_cmd_pasv(&sockcontrol, &ssa.serveripl,
				&ssa.serverport2));
  lcrzo_epr(lcrzo_ipl_print("Server IP : ", ssa.serveripl, "\n"));
  printf("Server port : %d\n", ssa.serverport2);

  /* initialize data connection */
  lcrzo_epr(lcrzo_sock_tcpcli_virt(ssa.device, 
				   ssa.clientetha, ssa.serveretha,
				   ssa.clientipl, ssa.serveripl, 
				   ssa.clientport2, ssa.serverport2,
				   &sockdata));

  /* we send a 'RETR' */
  lcrzo_epr(shared_ftp_cmd_retr(&sockcontrol, file));
  lcrzo_epr(lcrzo_string_free2(&file));

  /* display received data */
  while(LCRZO_TRUE)
  { ret=lcrzo_sock_readm(&sockdata, LCRZO_WAIT_INFINITE,
			 &data, &datasize);
    if ( ret!=LCRZO_ERR_OK ) break;
    lcrzo_epr(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_TEXT));
    lcrzo_epr(lcrzo_data_free2(&data));
  }
  lcrzo_epr(lcrzo_sock_close(&sockdata));

  lcrzo_epr(shared_ftp_cmd_datasock_waitend(&sockcontrol));

  /* then, we send a 'QUIT' */
  lcrzo_epr(shared_ftp_cmd_quit(&sockcontrol));

  /* close the socket */
  lcrzo_epr(lcrzo_sock_close(&sockcontrol));

  lcrzo_close();
  return (LCRZO_ERR_OK);
}
