
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000119=
{ /*reference*/     "lcrzoex", 119,
  /*version*/       1, 0,
  /*title*/         "virtual tcp server [stdin] (IP option)",
  /*french title*/  "server tcp virtuel [stdin] (option IP)",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "device serveth servname servport",
  /*french usage*/  "device ethserv nomserv portserv",
  /*usage example*/ "eth0 aa:bb:cc:dd:ee:ff 1.2.3.4 5566",
  /*fr. usage ex.*/ "eth0 aa:bb:cc:dd:ee:ff 1.2.3.4 5566",
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000119_main(int argc, char *argv[]);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000119_main(int argc, char *argv[])
{ lcrzo_etha servereth;
  lcrzo_ipl serveripl;
  lcrzo_uint16 serverport;
  lcrzo_sock sock;
  lcrzo_ipopt ipopt;
  lcrzo_uint8 ipoptsize;
  lcrzo_int16 language;
  lcrzo_device device;

  lcrzo_init();
  if ( argc<5 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000119, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }

  lcrzo_epr(shared_para_device_init_listen(argv[1], device));

  /* initialize Ethernet addresses */
  lcrzo_epr(shared_para_etha_init_listen(argv[2], servereth));

  /* initialize IP addresses */
  lcrzo_epr(shared_para_ipl_init_listen(argv[3], &serveripl));

  /* initialize IP options */
  lcrzo_epr(lcrzo_global_get_language(&language));
  lcrzo_epr(lcrzo_ipopt_init_stdin((language==LCRZO_GLOBAL_LANG_FR)?
				   "options IP":"IP options", "",
				   ipopt, &ipoptsize));

  /* initialize port numbers */
  lcrzo_epr(shared_para_port_init_listen(argv[4], &serverport));

  /* initialize the socket */
  lcrzo_epr(lcrzo_sock_tcpser_virt2(device,
				    servereth, serveripl, serverport,
				    ipopt, ipoptsize,
				    &sock));
 
  /* call the example working on the socket */
  lcrzo_epr(shared_sock_stdin(&sock, LCRZO_FALSE));

  lcrzo_close();
  return(LCRZO_ERR_OK);
}
