
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000130=
{ /*reference*/     "lcrzoex", 130,
  /*version*/       1, 0,
  /*title*/         "answer to every ARP/ping request",
  /*french title*/  "repond a toutes les requetes ARP/ping",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "device spoofed_ethernet_address [\"bpf_filter\"]]",
  /*french usage*/  "device adresse_ethernet_usurpee [\"filtre_bpf\"]]",
  /*usage example*/ "eth0 aa:bb:cc:dd:ee:ff \"net 1.2.3.0/24\"",
  /*fr. usage ex.*/ "eth0 aa:bb:cc:dd:ee:ff \"net 1.2.3.0/24\"",
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declaration */
int lcrzoex_000130_main(int argc, char *argv[]);
int lcrzoex_000130_answer_arp(lcrzo_data data, lcrzo_int32 datasize,
			      lcrzo_device device, lcrzo_etha ethtospoof);
int lcrzoex_000130_answer_ping(lcrzo_data data, lcrzo_int32 datasize,
			       lcrzo_device device, lcrzo_etha ethtospoof);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000130_main(int argc, char *argv[])
{ lcrzo_device device;
  lcrzo_etha ethtospoof;
  lcrzo_uint32 mtu;
  lcrzo_sniff sniff;
  lcrzo_data sniffdata;
  lcrzo_int32 sniffdatasize;
  lcrzo_string snifffilter;
  int ret;

  lcrzo_init();
  if ( argc<3 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000130, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }

  /* initialize device, ethernet address and mtu */
  lcrzo_epr(shared_para_device_init_src(argv[1], device));
  lcrzo_epr(shared_para_etha_init_src(argv[2], ethtospoof));
  lcrzo_epr(lcrzo_mtu_init_device(device, &mtu));

  /* ask user if he really wants to continue */
  lcrzo_epr(shared_misc_dangerousexample());

  /* init sniff */
  if (argc<=3)
  { lcrzo_epr(lcrzo_sniff_init(device, mtu, LCRZO_SNIFF_TYPE_NORMAL, "", &sniff));
  }
  else
  { lcrzo_epr(shared_para_string_init_snifffilter(argv[3], &snifffilter));
    lcrzo_epr(lcrzo_sniff_init(device, mtu, LCRZO_SNIFF_TYPE_NORMAL,
			       snifffilter, &sniff));
    lcrzo_epr(lcrzo_string_free2(&snifffilter));
  }

  /* main loop */
  ret=lcrzo_sniff_nextm(&sniff, LCRZO_WAIT_INFINITE, 
			&sniffdata, &sniffdatasize);
  while (ret==LCRZO_ERR_OK)
  { /*lcrzo_packet_print_eth(sniffdata, sniffdatasize,
                           LCRZO_PRINTPROFILE_SYNTHSYNTH);*/
    /* now, we answer to ARP request or ICMP echo request */
    lcrzo_epr(lcrzoex_000130_answer_arp(sniffdata, sniffdatasize,
					device, ethtospoof));
    lcrzo_epr(lcrzoex_000130_answer_ping(sniffdata, sniffdatasize,
					 device, ethtospoof));
    lcrzo_epr(lcrzo_data_free2(&sniffdata));
    /* sniff next packet */
    ret=lcrzo_sniff_nextm(&sniff, LCRZO_WAIT_INFINITE,
			  &sniffdata, &sniffdatasize);
  }

  /* close everything */
  lcrzo_epr(lcrzo_sniff_close(&sniff));
  lcrzo_epr(lcrzo_err_print(ret));
  lcrzo_close();
  return (ret);
}

/*-------------------------------------------------------------*/
int lcrzoex_000130_answer_arp(lcrzo_data data, lcrzo_int32 datasize,
			      lcrzo_device device, lcrzo_etha ethtospoof)
{ lcrzo_hdrleth recv_hdreth, send_hdreth;
  lcrzo_hdrlarp recv_hdrarp, send_hdrarp;
  lcrzo_spoof spoof;
  int ret;
  
  /* try to decode arp */
  ret=lcrzo_packet_decode_etharp(data, datasize, &recv_hdreth, &recv_hdrarp);
  if ( ret!=LCRZO_ERR_OK )
    return(LCRZO_ERR_OK);
  if ( recv_hdreth.type != LCRZO_HDRLETH_TYPE_ARP )
    return(LCRZO_ERR_OK);
  if ( recv_hdrarp.op!=1 )
    return(LCRZO_ERR_OK);

  /* create the new Ethernet header */
  lcrzo_epr(lcrzo_hdrleth_initdefault(&send_hdreth));
  memcpy(send_hdreth.src, ethtospoof, LCRZO_ETHA_MAXBYTES);
  memcpy(send_hdreth.dst, recv_hdrarp.hw_src, LCRZO_ETHA_MAXBYTES);
  send_hdreth.type=LCRZO_HDRLETH_TYPE_ARP;
    
  /* create the new ARP header */
  lcrzo_epr(lcrzo_hdrlarp_initdefault(&send_hdrarp));
  send_hdrarp.op=2;
  memcpy(send_hdrarp.hw_src, ethtospoof, LCRZO_ETHA_MAXBYTES);
  memcpy(send_hdrarp.hw_dst, recv_hdrarp.hw_src, LCRZO_ETHA_MAXBYTES);
  memcpy(send_hdrarp.prot_src, recv_hdrarp.prot_dst, LCRZO_IPA_MAXBYTES);
  memcpy(send_hdrarp.prot_dst, recv_hdrarp.prot_src, LCRZO_IPA_MAXBYTES);
    
  /* send packet */
  lcrzo_epr(lcrzo_spoof_init(&spoof));
  lcrzo_epr(lcrzo_spoof_set_printbeforesending(&spoof, LCRZO_TRUE));
  lcrzo_epr(lcrzo_spoof_set_printprofile(&spoof, 
					 LCRZO_PRINTPROFILE_SYNTHSYNTH));
  lcrzo_epr(lcrzo_spoof_etharp(&spoof, device, send_hdreth, send_hdrarp));
  lcrzo_epr(lcrzo_spoof_close(&spoof));
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000130_answer_ping(lcrzo_data data, lcrzo_int32 datasize,
			       lcrzo_device device, lcrzo_etha ethtospoof)
{ lcrzo_hdrleth  recv_hdreth,  send_hdreth;
  lcrzo_hdrlip   recv_hdrip,   send_hdrip;
  lcrzo_hdrlicmp recv_hdricmp, send_hdricmp;
  lcrzo_data icmpdata;
  lcrzo_uint16 icmpdatasize;
  lcrzo_spoof spoof;
  int ret;

  /* try to decode icmp */
  ret=lcrzo_packet_decodem_ethipopticmpdata(data, datasize, &recv_hdreth,
					    &recv_hdrip, NULL, NULL,
					    &recv_hdricmp, 
					    &icmpdata, &icmpdatasize);
  if ( ret!=LCRZO_ERR_OK )
    return(LCRZO_ERR_OK);
  if ( recv_hdricmp.type != 8 )
    return(LCRZO_ERR_OK);

  /* create the new Ethernet header */
  lcrzo_epr(lcrzo_hdrleth_initdefault(&send_hdreth));
  memcpy(send_hdreth.src, ethtospoof, LCRZO_ETHA_MAXBYTES);
  memcpy(send_hdreth.dst, recv_hdreth.src, LCRZO_ETHA_MAXBYTES);

  /* create the new IP header */
  lcrzo_epr(lcrzo_hdrlip_initdefault(&send_hdrip));
  send_hdrip.saddr=recv_hdrip.daddr;
  send_hdrip.daddr=recv_hdrip.saddr;

  /* create the new ICMP header */
  lcrzo_epr(lcrzo_hdrlicmp_initdefault(&send_hdricmp));
  send_hdricmp.type=0;

  /* send packet */
  lcrzo_epr(lcrzo_spoof_init(&spoof));
  lcrzo_epr(lcrzo_spoof_set_printbeforesending(&spoof, LCRZO_TRUE));
  lcrzo_epr(lcrzo_spoof_set_printprofile(&spoof, 
					 LCRZO_PRINTPROFILE_SYNTHSYNTH));
  lcrzo_epr(lcrzo_spoof_ethipopticmpdata(&spoof, device, send_hdreth, 
					 send_hdrip, NULL, 0,
					 send_hdricmp,icmpdata, icmpdatasize));
  lcrzo_epr(lcrzo_data_free2(&icmpdata));
  lcrzo_epr(lcrzo_spoof_close(&spoof));
  return (LCRZO_ERR_OK);
}
