
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000133=
{ /*reference*/     "lcrzoex", 133,
  /*version*/       1, 4,
  /*title*/         "check if TCP seqnum can be predicted (IP spoof)",
  /*french title*/  "verifie si les numeros de sequence TCP peuvent etre predits (spoof IP)",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "device client_name server_name server_port",
  /*french usage*/  "device nom_client nom_serveur port_serveur",
  /*usage example*/ "eth0 1.2.3.4 1.2.3.5 80",
  /*fr. usage ex.*/ "eth0 1.2.3.4 1.2.3.5 80",
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000133_main(int argc, char *argv[]);
int lcrzoex_000133_child_spoof(lcrzo_ptr infos, 
			       lcrzo_int32 infossize, 
			       lcrzo_process *pparentprocess);
int lcrzoex_000133_child_sniff(lcrzo_ptr infos, 
			       lcrzo_int32 infossize, 
			       lcrzo_process *pparentprocess);
int lcrzoex_000133_print(lcrzo_device device,
			 lcrzo_ipl clientipl, 
			 lcrzo_ipl serveripl, 
			 lcrzo_uint16 serverport,
			 lcrzo_uint16 startingclientport,
			 lcrzo_uint32 startingclientseqnum);
int lcrzoex_000133_printseqnum(lcrzo_data sniffdata,
			       lcrzo_int32 sniffdatasize,
			       lcrzo_ipl clientipl, 
			       lcrzo_ipl serveripl, 
			       lcrzo_uint16 serverport,
			       lcrzo_uint16 startingclientport,
			       lcrzo_uint32 startingclientseqnum);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000133_main(int argc, char *argv[])
{ shared_sock_address ssa;
  lcrzo_process childspoof, childsniff;
  lcrzo_wait waitspoof, waitsniff;
  lcrzo_bool eventspoof, eventsniff;
  int ret;
  
  lcrzo_init();
  if ( argc<5 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000133, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }
  lcrzo_epr(shared_para_device_init_src(argv[1], ssa.device));
  lcrzo_epr(shared_para_ipl_init_src(argv[2], &ssa.clientipl));
  lcrzo_epr(shared_para_ipl_init_dst(argv[3], &ssa.serveripl));
  lcrzo_epr(shared_para_port_init_dst(argv[4], 80, &ssa.serverport));

  /* determine starting values */
  lcrzo_epr(lcrzo_uint16_init_rand(5000, 30000, &ssa.clientport2));
  lcrzo_epr(lcrzo_uint32_init_rand(0, 0xEFFFFFFFU, &ssa.seqnum));

  /* this process sends a SYN (we left to our computer the job to send
     a RST) */
  lcrzo_epr(lcrzo_process_other_init(&lcrzoex_000133_child_spoof, 
                                     (lcrzo_data)&ssa, sizeof(ssa), 
				     &childspoof));

  /* this process sniff and prints sequence numbers */
  ret=lcrzo_process_other_init(&lcrzoex_000133_child_sniff, 
			       (lcrzo_data)&ssa, sizeof(ssa),
			       &childsniff);
  if ( ret!=LCRZO_ERR_OK)
  { /* kill the first child */
    lcrzo_epr(shared_lcrzoex_debug_processotherexit_wanttowarn(LCRZO_FALSE));
    lcrzo_epr(lcrzo_process_other_exit(&childspoof));
    lcrzo_epr(shared_lcrzoex_debug_processotherexit_wanttowarn(LCRZO_TRUE));
    lcrzo_epr(lcrzo_process_other_wait_end(&childspoof, LCRZO_WAIT_INFINITE, 
					   NULL, NULL));
    /* exit */
    return(ret);
  }

  /* wait for the end of one process */
  lcrzo_epr(lcrzo_wait_init_process_end(&childspoof, &waitspoof));
  lcrzo_epr(lcrzo_wait_init_process_end(&childsniff, &waitsniff));
  lcrzo_epr(lcrzo_wait_wait2(&waitspoof, &waitsniff, LCRZO_WAIT_INFINITE,
			     &eventspoof, &eventsniff));
  lcrzo_epr(lcrzo_wait_close(&waitspoof));
  lcrzo_epr(lcrzo_wait_close(&waitsniff));

  /* kill childs */
  if (eventspoof)
  { lcrzo_epr(shared_lcrzoex_debug_processotherexit_wanttowarn(LCRZO_FALSE));
    lcrzo_epr(lcrzo_process_other_exit(&childsniff));
    lcrzo_epr(shared_lcrzoex_debug_processotherexit_wanttowarn(LCRZO_TRUE));
    lcrzo_epr(lcrzo_process_other_wait_end(&childsniff, LCRZO_WAIT_INFINITE, 
					   NULL, NULL));
    lcrzo_epr(lcrzo_process_other_wait_end(&childspoof, LCRZO_WAIT_INFINITE, 
					   NULL, &ret));
    lcrzo_epr(lcrzo_process_close(&childspoof));
    lcrzo_epr(lcrzo_process_close(&childsniff));
    return(ret);
  }
  if (eventsniff)
  { lcrzo_epr(shared_lcrzoex_debug_processotherexit_wanttowarn(LCRZO_FALSE));
    lcrzo_epr(lcrzo_process_other_exit(&childspoof));
    lcrzo_epr(shared_lcrzoex_debug_processotherexit_wanttowarn(LCRZO_TRUE));
    lcrzo_epr(lcrzo_process_other_wait_end(&childspoof, LCRZO_WAIT_INFINITE, 
					   NULL, NULL));
    lcrzo_epr(lcrzo_process_other_wait_end(&childsniff, LCRZO_WAIT_INFINITE, 
					   NULL, &ret));
    lcrzo_epr(lcrzo_process_close(&childspoof));
    lcrzo_epr(lcrzo_process_close(&childsniff));
    return(ret);
  }

  /* should not be reached */
  lcrzo_close();
  return (LCRZO_ERR_IEINTERNALERROR);
}

/*-------------------------------------------------------------*/
int lcrzoex_000133_child_spoof(lcrzo_ptr infos, 
			       lcrzo_int32 infossize, 
			       lcrzo_process *pparentprocess)
{ shared_sock_address *pssa=(shared_sock_address *)infos;
  lcrzo_hdrlip hdrlip;
  lcrzo_hdrltcp hdrltcp;
  lcrzo_spoof spoof;
  int ret;

  /* check for correct usage of this function */
  if (infossize!=sizeof(shared_sock_address))
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);

  /* close parent process handle because we do not need it */
  lcrzo_epr(lcrzo_process_close(pparentprocess));

  /* initialize IP header */
  lcrzo_epr(lcrzo_hdrlip_initdefault(&hdrlip));
  hdrlip.saddr=pssa->clientipl;
  hdrlip.daddr=pssa->serveripl;

  /* initialize TCP header */
  lcrzo_epr(lcrzo_hdrltcp_initdefault(&hdrltcp));
  hdrltcp.sport=pssa->clientport2;
  hdrltcp.dport=pssa->serverport;
  hdrltcp.syn=1;
  hdrltcp.seqnum=pssa->seqnum;

  /* little sleep to be sure sniff process is ready (should be done with
     an IPC) */
  lcrzo_epr(lcrzo_time_sleep_msec(300));

  /* spoof */
  lcrzo_epr(lcrzo_spoof_init(&spoof));
  ret=LCRZO_ERR_OK;
  while(LCRZO_TRUE)
  { ret=lcrzo_spoof_ipopttcpoptdata(&spoof,
				    hdrlip, NULL, 0,
				    hdrltcp, NULL, 0,
				    NULL, 0);
    if (ret!=LCRZO_ERR_OK) break;
    hdrlip.id++;
    hdrltcp.sport++;
    hdrltcp.seqnum+=10;
    lcrzo_epr(lcrzo_time_sleep_usec(100000));
  }
  lcrzo_epr(lcrzo_spoof_close(&spoof));
  return(ret);
}

/*-------------------------------------------------------------*/
int lcrzoex_000133_child_sniff(lcrzo_ptr infos, 
			       lcrzo_int32 infossize, 
			       lcrzo_process *pparentprocess)
{ shared_sock_address *pssa=(shared_sock_address *)infos;
  lcrzo_uint32 mtu;
  lcrzo_sniff sniff;
  lcrzo_data data;
  lcrzo_int32 datasize;
  int ret;

  /* check for correct usage of this function */
  if (infossize!=sizeof(shared_sock_address))
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);

  /* close parent process handle because we do not need it */
  lcrzo_epr(lcrzo_process_close(pparentprocess));

  /* init sniff */
  lcrzo_epr(lcrzo_mtu_init_device(pssa->device, &mtu));
  lcrzo_epr(lcrzo_sniff_init(pssa->device, mtu, LCRZO_SNIFF_TYPE_NORMAL, 
			     "", &sniff));

  /* main loop */
  ret=LCRZO_ERR_OK;
  while (LCRZO_TRUE)
  { ret=lcrzo_sniff_nextm(&sniff, LCRZO_WAIT_INFINITE, &data, &datasize);
    if (ret!=LCRZO_ERR_OK) break;
	  /*lcrzo_packet_print_eth(data, datasize, LCRZO_PRINTPROFILE_ARRAYDUMP);*/
    /* decode TCP and print seqnum */
    ret=lcrzoex_000133_printseqnum(data, datasize,
				   pssa->clientipl, pssa->serveripl,
				   pssa->serverport,
				   pssa->clientport2, 
				   pssa->seqnum);
    lcrzo_epr(lcrzo_data_free2(&data));
    if (ret!=LCRZO_ERR_OK) break;
  }

  lcrzo_epr(lcrzo_sniff_close(&sniff));
  return(ret);
}

/*-------------------------------------------------------------*/
int lcrzoex_000133_printseqnum(lcrzo_data sniffdata,
			       lcrzo_int32 sniffdatasize,
			       lcrzo_ipl clientipl, 
			       lcrzo_ipl serveripl, 
			       lcrzo_uint16 serverport,
			       lcrzo_uint16 startingclientport,
			       lcrzo_uint32 startingclientseqnum)
{ static lcrzo_uint32 oldseqnum=0;
  lcrzo_hdrlip hdrlip;
  lcrzo_hdrltcp hdrltcp;
  int ret;
  lcrzo_uint32 diff1, diff2;

  /* try to decode it */
  ret=lcrzo_packet_decodem_ethipopttcpoptdata(sniffdata, sniffdatasize,
					      NULL, &hdrlip, NULL, NULL,
					      &hdrltcp, NULL, NULL,
					      NULL, NULL);
  if ( ret!=LCRZO_ERR_OK )
    return(LCRZO_ERR_OK);
  /* only count synack */
  if ( ! (hdrltcp.syn && hdrltcp.ack) )
    return(LCRZO_ERR_OK);
  /* check if the addresses are the one we are looking for */
  if ( hdrlip.saddr!=serveripl )
    return(LCRZO_ERR_OK);
  if ( hdrlip.daddr!=clientipl )
    return(LCRZO_ERR_OK);
  /* check server port */
  if ( hdrltcp.sport!=serverport )
    return(LCRZO_ERR_OK);

  /* now this seems ok, but we verify with starting values */
  diff1=hdrltcp.dport-startingclientport;
  diff2=(hdrltcp.acknum-1-startingclientseqnum)/10;
  if ( diff1!=diff2 )
    return(LCRZO_ERR_OK);

  /* now, it's OK, so we print the seqnum */
  lcrzo_epr(lcrzo_uint32_print("", diff1, "% 6u", " : "));
  lcrzo_epr(lcrzo_uint32_print("", hdrltcp.seqnum, "%.10u", "\t[diff "));
  if (hdrltcp.seqnum>=oldseqnum)
  { lcrzo_epr(lcrzo_uint32_print("+", hdrltcp.seqnum-oldseqnum,
				 "%.10u", "]\n"));
  }
  else
  { lcrzo_epr(lcrzo_uint32_print("-", oldseqnum-hdrltcp.seqnum,
				 "%.10u", "]\n"));
  }
  oldseqnum=hdrltcp.seqnum;
 
  return(LCRZO_ERR_OK);
}
