
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000135=
{ /*reference*/     "lcrzoex", 135,
  /*version*/       1, 0,
  /*title*/         "randomly fragmented {ETH,IP,UDP} spoof",
  /*french title*/  "spoof {ETH,IP,UDP} aleatoirement fragmente",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "device src_eth dst_eth src_ip dst_ip src_port dst_port",
  /*french usage*/  "device eth_src eth_dst ip_src ip_dst port_src port_dst",
  /*usage example*/ "eth0 aa:bb:cc:dd:ee:ff 11:22:33:44:55:66 1.2.3.4 5.6.7.8 4444 5555",
  /*fr. usage ex.*/ "eth0 aa:bb:cc:dd:ee:ff 11:22:33:44:55:66 1.2.3.4 5.6.7.8 4444 5555",
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000135_main(int argc, char *argv[]);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000135_main(int argc, char *argv[])
{ lcrzo_data data;
  lcrzo_int32 datasize;
  lcrzo_spoof spoof;
  lcrzo_hdrleth hdrleth;
  lcrzo_hdrlip hdrlip;
  lcrzo_hdrludp hdrludp;
  lcrzo_device device;

  lcrzo_init();
  if ( argc<7 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000135, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }

  /* initialize device */
  lcrzo_epr(shared_para_device_init_src(argv[1], device));

  /* initialize Ethernet header */
  lcrzo_epr(lcrzo_hdrleth_initdefault(&hdrleth));
  lcrzo_epr(shared_para_etha_init_src(argv[2], hdrleth.src));
  lcrzo_epr(shared_para_etha_init_dst(argv[3], hdrleth.dst));

  /* initialize IP header */
  lcrzo_epr(lcrzo_hdrlip_initdefault(&hdrlip));
  lcrzo_epr(shared_para_ipl_init_src(argv[4], &(hdrlip.saddr)));
  lcrzo_epr(shared_para_ipl_init_dst(argv[5], &(hdrlip.daddr)));
  hdrlip.protocol=LCRZO_HDRLIP_PROTOCOL_UDP;

  /* initialize UDP header */
  lcrzo_epr(lcrzo_hdrludp_initdefault(&hdrludp));
  lcrzo_epr(shared_para_port_init_src(argv[6], &hdrludp.sport));
  lcrzo_epr(shared_para_port_init_dst(argv[7], 53, &hdrludp.dport));

  /* main loop */
  lcrzo_epr(lcrzo_spoof_init(&spoof));
  while(1)
  { /* initialize data */
    lcrzo_epr(lcrzo_data_initm_rand(0, 255, 100, &data, &datasize));
    /* change header values */
    lcrzo_epr(lcrzo_uint16_init_rand(0, 20, &(hdrlip.id)));
    lcrzo_epr(lcrzo_uint16_init_rand(0, 5, &(hdrlip.offsetfrag)));
    if (hdrlip.offsetfrag)
    { lcrzo_epr(lcrzo_uint16_init_rand(1, 0x1FFF, &(hdrlip.offsetfrag)));
    }
    lcrzo_epr(lcrzo_bool_init_rand(&(hdrlip.morefrag)));
    /* spoof */
    lcrzo_epr(lcrzo_spoof_ethipoptudpdata(&spoof, device, hdrleth,
					  hdrlip, NULL, 0, hdrludp,
					  data, (lcrzo_uint16)datasize));
    lcrzo_epr(lcrzo_data_free2(&data));
  }
  lcrzo_epr(lcrzo_spoof_close(&spoof));

  lcrzo_close();
  return(LCRZO_ERR_OK);
}
