
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/* set information about this example */
shared_lcrzoex_infos lcrzoex_000149=
{ /*reference*/     "lcrzoex", 149, 
  /*version*/       1, 1, 
  /*title*/         "convert a file from unix to dos", 
  /*french title*/  "convertit un fichier de unix a dos", 
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "input_file output_file",
  /*french usage*/  "fichier_entree fichier_sortie", 
  /*usage example*/ "/tmp/file /tmp/fileout",
  /*fr. usage ex.*/ "/tmp/fichier /tmp/fichiersortie",
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000149_main(int argc, char *argv[]);

/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000149_main(int argc, char *argv[])
{ lcrzo_file inputfile, outputfile;
  lcrzo_string tempofile, fin, fout;
  lcrzo_data data;
  lcrzo_int32 datasize;
  lcrzo_uint32 r;
  int ret;

  lcrzo_init();
  if ( argc<3 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000149, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }
  
  lcrzo_epr(shared_para_string_init_filein(argv[1], &fin));
  lcrzo_epr(shared_para_string_init_fileout(argv[2], &fout));

  /* open the files */
  lcrzo_epr(lcrzo_string_initm_text(fout, &tempofile));
  lcrzo_epr(lcrzo_uint32_init_rand(1, 0xFFFFFFFFu, &r));
  lcrzo_epr(lcrzo_string_appendm_text(".lcrzoex", &tempofile));
  lcrzo_epr(lcrzo_string_appendm_uint32(r, "%08X", &tempofile));
  lcrzo_epr(lcrzo_file_open_write(tempofile, LCRZO_FALSE, LCRZO_TRUE,
				  &outputfile));
  lcrzo_epr(lcrzo_file_open_read(fin, &inputfile));

  /* copy file content */
  ret=LCRZO_ERR_OK;
  while (LCRZO_TRUE)
  { /* read a line */
    ret=lcrzo_file_readm_line(&inputfile, LCRZO_WAIT_INFINITE,
			      &data, &datasize);
    if (ret!=LCRZO_ERR_OK)
    { break;
    }
    ret=lcrzo_file_write(&outputfile, data, datasize);
    if (ret!=LCRZO_ERR_OK) break;
    lcrzo_epr(lcrzo_data_free2(&data));
    /* add a newline */
    ret=lcrzo_file_write(&outputfile, (lcrzo_constdata)"\r\n", 2);
    if (ret!=LCRZO_ERR_OK) break;        
  }
  /* end of file is reached */
  if (ret==LCRZO_ERR_OKDATAEND)
  { /* end of file : this is normal */
    ret=LCRZO_ERR_OK;
  }
  else if (ret==LCRZO_ERR_OKAVAILDATAOTHTYPE)
  { /* a chunk is left */
    ret=lcrzo_file_readm(&inputfile, LCRZO_WAIT_INFINITE, &data, &datasize);
    if (ret==LCRZO_ERR_OK)
    { ret=lcrzo_file_write(&outputfile, data, datasize);
      if (ret==LCRZO_ERR_OK)
      { /* add a newline */
        ret=lcrzo_file_write(&outputfile, (lcrzo_constdata)"\r\n", 2);
      }
      lcrzo_epr(lcrzo_data_free2(&data));
    }
  }
      
  /* close files */
  lcrzo_epr(lcrzo_file_close(&inputfile));
  lcrzo_epr(lcrzo_file_close(&outputfile));

  /* eventually rename temporary file */
  if ( ret!=LCRZO_ERR_OK )
  { lcrzo_epr(lcrzo_err_print(ret));
    lcrzo_epr(lcrzo_filename_remove(tempofile));
  }
  else
  { lcrzo_epr(lcrzo_filename_rename(tempofile, fout));
  }
  lcrzo_epr(lcrzo_string_free2(&tempofile));
  lcrzo_epr(lcrzo_string_free2(&fin));
  lcrzo_epr(lcrzo_string_free2(&fout));

  lcrzo_close();
  return(LCRZO_ERR_OK);
}


