/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000165=
{ /*reference*/     "lcrzoex", 165,
  /*version*/       1, 1,
  /*title*/         "passive ftp uploading a directory (different command line)",
  /*french title*/  "ftp passif uploadant(put) un repertoire (ligne de commande differente)",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "server_name server_port login local_dir distant_dir",
  /*french usage*/  "nom_serveur port_serveur login rep_local rep_distant",
  /*usage example*/ "1.2.3.4 21 bob /tmp/here .",
  /*fr. usage ex.*/ "1.2.3.4 21 bob /tmp/ici .",
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000165_main(int argc, char *argv[]);
int lcrzoex_000165_recursiveftpput(lcrzo_ipl serverip,
				   lcrzo_uint16 serverport,
				   lcrzo_conststring login,
				   lcrzo_conststring localdir,
				   lcrzo_conststring distantdir);
int lcrzoex_000165_recursiveput(lcrzo_sock *psock, 
				lcrzo_conststring abslocaldir,
				lcrzo_conststring rellocaldir);
int lcrzoex_000165_putfile(lcrzo_sock *psock, 
			   lcrzo_conststring absfilename, 
			   lcrzo_int32 filesize,
			   lcrzo_conststring relfilename);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000165_main(int argc, char *argv[])
{ lcrzo_ipl ipl;
  lcrzo_uint16 port;
  lcrzo_int16 language;
  lcrzo_string login, dirlocal, dirdistant;

  lcrzo_init();
  if ( argc<6 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000165, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }

  lcrzo_epr(shared_para_ipl_init_dst(argv[1], &ipl));
  lcrzo_epr(shared_para_port_init_dst(argv[2], 21, &port));
  lcrzo_epr(shared_para_string_init(argv[3], "Login:", "", &login));
  lcrzo_epr(shared_para_string_init_dirlocal(argv[4], &dirlocal));
  lcrzo_epr(shared_para_string_init_dirdistant(argv[5], &dirdistant));
  lcrzo_epr(lcrzoex_000165_recursiveftpput(ipl, port,
					   login, dirlocal, dirdistant));
  lcrzo_epr(lcrzo_string_free2(&login));
  lcrzo_epr(lcrzo_string_free2(&dirlocal));
  lcrzo_epr(lcrzo_string_free2(&dirdistant));
  lcrzo_epr(lcrzo_global_get_language(&language));
  if ( language==LCRZO_GLOBAL_LANG_FR )
    puts("Fin");
  else
    puts("End");
  lcrzo_close();
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000165_recursiveftpput(lcrzo_ipl serverip,
				   lcrzo_uint16 serverport,
				   lcrzo_conststring login,
				   lcrzo_conststring localdir,
				   lcrzo_conststring distantdir)
{ lcrzo_sock sock;
  lcrzo_string pwd;

  /* initialize the client */
  lcrzo_epr(lcrzo_sock_tcpcli_real(serverip, serverport, &sock));

  /* first, we connect */
  lcrzo_epr(lcrzo_string_initm_text_passwd("Passwd", "", &pwd));
  lcrzo_epr(shared_ftp_login(&sock, login, pwd));
  lcrzo_epr(lcrzo_string_free2(&pwd));

  /* then, we send a 'TYPE I' */
  lcrzo_epr(shared_ftp_cmd_type_bin(&sock));

  /* treat every file */  
  lcrzo_epr(lcrzoex_000165_recursiveput(&sock, localdir, distantdir));

  /* then, we send a 'QUIT' */
  lcrzo_epr(shared_ftp_cmd_quit(&sock));

  /* close the socket */
  lcrzo_epr(lcrzo_sock_close(&sock));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000165_recursiveput(lcrzo_sock *psock, 
				lcrzo_conststring abslocaldir,
				lcrzo_conststring rellocaldir)
{ lcrzo_int32 dircount, i, filesize;
  lcrzo_string localfile, filename;
  lcrzo_int16 language, filetype;

  lcrzo_epr(lcrzo_global_get_language(&language));

  /* we send a 'MKD dir' */
  lcrzo_epr(shared_ftp_cmd_mkd(psock, rellocaldir));

  /* we send a 'CWD dir' */
  lcrzo_epr(shared_ftp_cmd_cwd(psock, rellocaldir));

  /* go through the directory */
  lcrzo_epr(lcrzo_dir_count(abslocaldir, &dircount));
  for ( i=dircount ; i>=1; i-- )
  { lcrzo_epr(lcrzo_dir_value_pos(abslocaldir, i, &filename));
    /* ignore . and .. */
    if ( !strcmp(filename, ".") || !strcmp(filename, "..") )
    { lcrzo_epr(lcrzo_string_free2(&filename));
      continue;
    }
    /* set the local file */
    lcrzo_epr(lcrzo_string_initm_text(abslocaldir, &localfile));
    if ( abslocaldir[strlen(abslocaldir)-1] != '/' )
    { lcrzo_epr(lcrzo_string_appendm_char('/', 1, &localfile));
    }
    lcrzo_epr(lcrzo_string_appendm_text(filename, &localfile));
    /* stat it */
    lcrzo_epr(lcrzo_path_type(localfile, &filetype));
    if ( filetype==LCRZO_PATH_TYPE_DIR )
    { if ( language==LCRZO_GLOBAL_LANG_FR )
        printf("%s est un repertoire\n", localfile);
      else
        printf("%s is a directory\n", localfile);
      lcrzo_epr(lcrzoex_000165_recursiveput(psock, localfile, 
					    filename));
      /* this is very DANGEROUS to uncomment this : it DELETEs the local dir*/
      /*lcrzo_epr(lcrzo_dir_remove(localfile));*/
    }
    else if ( filetype==LCRZO_PATH_TYPE_REG )
    { if ( language==LCRZO_GLOBAL_LANG_FR )
        printf("%s est un fichier\n", localfile);
      else
	printf("%s is regular\n", localfile);
      lcrzo_epr(lcrzo_filename_size(localfile, &filesize));
      lcrzo_epr(lcrzoex_000165_putfile(psock, localfile, filesize,
				       filename));
      /* this is very DANGEROUS to uncomment this : it DELETEs the local file*/
      /*lcrzo_epr(lcrzo_filename_remove(localfile));*/
    }
    lcrzo_epr(lcrzo_string_free2(&filename));
    lcrzo_epr(lcrzo_string_free2(&localfile));
  }

  /* we send a 'CWD ..' */
  lcrzo_epr(shared_ftp_cmd_cwd(psock, ".."));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000165_putfile(lcrzo_sock *psock, 
			   lcrzo_conststring absfilename,
			   lcrzo_int32 filesize,
			   lcrzo_conststring relfilename)
{ lcrzo_data data;
  lcrzo_int32 datasize, servsize;
  lcrzo_ipl pasvipl;
  lcrzo_uint16 pasvport;
  lcrzo_sock datasock;
  lcrzo_file inputfile;
  lcrzo_int16 language;
  int ret;

  /* get the port to which connect */
  lcrzo_epr(shared_ftp_cmd_pasv(psock, &pasvipl, &pasvport));
  /*lcrzo_ipl_print("IP : ", pasvipl, ":");
    printf("%d\n", pasvport);*/

  lcrzo_epr(lcrzo_global_get_language(&language));

  /* initialize the data connexion */
  lcrzo_epr(lcrzo_sock_tcpcli_real(pasvipl, pasvport, &datasock));
  
  /* we send a 'STOR file' */
  lcrzo_epr(shared_ftp_cmd_stor(psock, relfilename));

  /* write data to the file */
  lcrzo_epr(lcrzo_file_open_read(absfilename, &inputfile));
  /* copy file content */
  ret=LCRZO_ERR_OK;
  while (LCRZO_TRUE)
  { /* read a line */
    ret=lcrzo_file_readm(&inputfile, LCRZO_WAIT_INFINITE, &data, &datasize);
    if (ret!=LCRZO_ERR_OK)
    { if (ret==LCRZO_ERR_OKDATAEND) ret=LCRZO_ERR_OK;
      break;
    }
    ret=lcrzo_sock_write(&datasock, data, datasize);
    lcrzo_epr(lcrzo_data_free2(&data));
    if (ret!=LCRZO_ERR_OK) break;
  }
  /* close file */
  lcrzo_epr(lcrzo_file_close(&inputfile));
  lcrzo_epr(ret);

  /* close the data connexion */
  lcrzo_epr(lcrzo_sock_close(&datasock));
  lcrzo_epr(shared_ftp_cmd_datasock_waitend(psock));

  /* we send a 'SIZE file' */
  lcrzo_epr(shared_ftp_cmd_size(psock, relfilename, &servsize));
  /* verify size */
  if ( filesize!=servsize )
  { if ( language==LCRZO_GLOBAL_LANG_FR )
    { lcrzo_epr(lcrzo_int32_print("Les tailles sont differentes : ", filesize, 
				  "%d", " "));
      lcrzo_epr(lcrzo_int32_print("", servsize, "%d", " je reessaie\n"));
    }
    else
    { lcrzo_epr(lcrzo_int32_print("Sizes are differents : ", filesize, 
				  "%d", " "));
      lcrzo_epr(lcrzo_int32_print("", servsize, "%d", " we retry\n"));
    }
    /* retry */
    lcrzo_epr(lcrzoex_000165_putfile(psock, absfilename, filesize,
				     relfilename));
  }

  return(LCRZO_ERR_OK);
}
