
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000177=
{ /*reference*/     "lcrzoex", 177,
  /*version*/       1, 2,
  /*title*/         "virtual tcp computer (answer arp, rarp, ping, udpecho, ftp, smtp)",
  /*french title*/  "machine tcp virtuelle (repond a arp, rarp, ping, udpecho, ftp, smtp)",
  /*description*/   "Note: protocol are not really implemented",
  /*french desc.*/  "Note: les protocoles ne sont pas rellement implementes",
  /*usage*/         "device serveth servname",
  /*french usage*/  "device ethserv nomserv",
  /*usage example*/ "eth0 aa:bb:cc:dd:ee:ff 1.2.3.4",
  /*fr. usage ex.*/ "eth0 aa:bb:cc:dd:ee:ff 1.2.3.4",
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000177_main(int argc, char *argv[]);
int lcrzoex_000177_echo(lcrzo_sock *psock,
			lcrzo_bool printbanner);
int lcrzoex_000177_child_ftp(lcrzo_ptr infos, 
			     lcrzo_int32 infossize, 
			     lcrzo_process *pparentprocess);
int lcrzoex_000177_child_smtp(lcrzo_ptr infos, 
			      lcrzo_int32 infossize, 
			      lcrzo_process *pparentprocess);
int lcrzoex_000177_child_udp(lcrzo_ptr infos, 
			     lcrzo_int32 infossize, 
			     lcrzo_process *pparentprocess);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000177_main(int argc, char *argv[])
{ shared_sock_address ssa;
  lcrzo_process childftp, childsmtp, childudp;
  lcrzo_wait waitftp, waitsmtp, waitudp;
  lcrzo_bool eventftp, eventsmtp, eventudp;
  lcrzo_ips ips;
  lcrzo_int16 language;
  int ret;

  lcrzo_init();
  if ( argc<4 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000177, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }

  /* initialize the device */
  lcrzo_epr(shared_para_device_init_listen(argv[1], ssa.device));

  /* initialize Ethernet address */
  lcrzo_epr(shared_para_etha_init_listen(argv[2], ssa.serveretha));

  /* initialize IP address */
  lcrzo_epr(shared_para_ipl_init_listen(argv[3], &ssa.serveripl));

  /* first, set up ftp server */
  lcrzo_epr(lcrzo_process_other_init(&lcrzoex_000177_child_ftp, 
                                     (lcrzo_data)&ssa, sizeof(ssa), 
                                     &childftp));
  /* from now on, only the ftp server answers to icmp, arp and rarp */
  /* we have to wait for windows, to left time to the child process
     to copy the current environment before modifying it */
  lcrzo_epr(lcrzo_time_sleep_msec(100));
  lcrzo_epr(lcrzo_global_set_sock_virt_answeralive(LCRZO_FALSE));
  /* start other processes */
  ret=lcrzo_process_other_init(&lcrzoex_000177_child_smtp, 
                               (lcrzo_data)&ssa, sizeof(ssa),
                               &childsmtp);
  if ( ret!=LCRZO_ERR_OK)
  { lcrzo_epr(shared_lcrzoex_debug_processotherexit_wanttowarn(LCRZO_FALSE));
    lcrzo_epr(lcrzo_process_other_exit(&childftp));
    lcrzo_epr(shared_lcrzoex_debug_processotherexit_wanttowarn(LCRZO_TRUE));
    lcrzo_epr(lcrzo_process_other_wait_end(&childftp, LCRZO_WAIT_INFINITE, 
                                           NULL, NULL));
    return(ret);
  }
  ret=lcrzo_process_other_init(&lcrzoex_000177_child_udp, 
                               (lcrzo_data)&ssa, sizeof(ssa),
                               &childudp);
  if ( ret!=LCRZO_ERR_OK)
  { lcrzo_epr(shared_lcrzoex_debug_processotherexit_wanttowarn(LCRZO_FALSE));
    lcrzo_epr(lcrzo_process_other_exit(&childftp));
    lcrzo_epr(lcrzo_process_other_exit(&childsmtp));
    lcrzo_epr(shared_lcrzoex_debug_processotherexit_wanttowarn(LCRZO_TRUE));
    lcrzo_epr(lcrzo_process_other_wait_end(&childftp, LCRZO_WAIT_INFINITE, 
                                           NULL, NULL));
    lcrzo_epr(lcrzo_process_other_wait_end(&childsmtp, LCRZO_WAIT_INFINITE, 
                                           NULL, NULL));
    return(ret);
  }

  /* print what user can do */
  lcrzo_epr(lcrzo_global_get_language(&language));
  lcrzo_epr(lcrzo_ips_init_ipl(ssa.serveripl, ips));
  if ( language==LCRZO_GLOBAL_LANG_FR )
    printf("Vous pouvez maintenant entrer, depuis une autre machine :\n");
  else
    printf("You can now enter, from another computer :\n");
  printf(" ping %s\n", ips);
  printf(" telnet %s 21\n", ips);
  printf(" telnet %s 25\n", ips);
  printf(" lcrzoex 168 %s 7\n", ips);

  /* wait for the end of one process */
  lcrzo_epr(lcrzo_wait_init_process_end(&childftp, &waitftp));
  lcrzo_epr(lcrzo_wait_init_process_end(&childsmtp, &waitsmtp));
  lcrzo_epr(lcrzo_wait_init_process_end(&childudp, &waitudp));
  lcrzo_epr(lcrzo_wait_wait3(&waitftp, &waitsmtp, &waitudp,
			     LCRZO_WAIT_INFINITE,
                             &eventftp, &eventsmtp, &eventudp));
  lcrzo_epr(lcrzo_wait_close(&waitftp));
  lcrzo_epr(lcrzo_wait_close(&waitsmtp));
  lcrzo_epr(lcrzo_wait_close(&waitudp));

  /* kill childs */
  if (!eventftp) 
  { lcrzo_epr(shared_lcrzoex_debug_processotherexit_wanttowarn(LCRZO_FALSE));
    lcrzo_epr(lcrzo_process_other_exit(&childftp)); 
    lcrzo_epr(shared_lcrzoex_debug_processotherexit_wanttowarn(LCRZO_TRUE));
  }
  if (!eventsmtp) 
  { lcrzo_epr(shared_lcrzoex_debug_processotherexit_wanttowarn(LCRZO_FALSE));
    lcrzo_epr(lcrzo_process_other_exit(&childsmtp));  
    lcrzo_epr(shared_lcrzoex_debug_processotherexit_wanttowarn(LCRZO_TRUE));
  }
  if (!eventudp) 
  { lcrzo_epr(shared_lcrzoex_debug_processotherexit_wanttowarn(LCRZO_FALSE));
    lcrzo_epr(lcrzo_process_other_exit(&childudp));  
    lcrzo_epr(shared_lcrzoex_debug_processotherexit_wanttowarn(LCRZO_TRUE));
  }
  lcrzo_epr(lcrzo_process_other_wait_end(&childftp, LCRZO_WAIT_INFINITE, 
					 NULL, &ret));
  lcrzo_epr(lcrzo_err_print(ret));
  lcrzo_epr(lcrzo_process_other_wait_end(&childsmtp, LCRZO_WAIT_INFINITE, 
					 NULL, &ret));
  lcrzo_epr(lcrzo_err_print(ret));
  lcrzo_epr(lcrzo_process_other_wait_end(&childudp, LCRZO_WAIT_INFINITE, 
					 NULL, &ret));
  lcrzo_epr(lcrzo_err_print(ret));
  lcrzo_epr(lcrzo_process_close(&childftp));
  lcrzo_epr(lcrzo_process_close(&childsmtp));
  lcrzo_epr(lcrzo_process_close(&childudp));
  
  lcrzo_close();
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000177_echo(lcrzo_sock *psock,
			lcrzo_bool printbanner)
{ lcrzo_data data;
  lcrzo_int32 datasize;
  lcrzo_bool leave;
  lcrzo_int16 language;
  int ret;

  /* we eventually print a banner */
  lcrzo_er(lcrzo_global_get_language(&language));
  if (printbanner)
  { if ( language==LCRZO_GLOBAL_LANG_FR )
    { lcrzo_er(lcrzo_data_initm_text(
	       "Bonjour, ce serveur n'implemente pas le protocole.\r\n",
	       &data, &datasize));
      lcrzo_er(lcrzo_data_appendm_text(
	       "Les donnees entrees sont simplement retournees.\r\n", 
	       datasize, &data, &datasize));
      lcrzo_er(lcrzo_data_appendm_text(
	       "Entrez 'QUIT' pour quitter.\r\n", datasize, &data, &datasize));
    }
    else
    { lcrzo_er(lcrzo_data_initm_text(
	       "Hello, this server doesn't really implement the protocol.\r\n",
	       &data, &datasize));
      lcrzo_er(lcrzo_data_appendm_text(
	       "Received data is simply echoed back.\r\n", 
	       datasize, &data, &datasize));
      lcrzo_er(lcrzo_data_appendm_text(
	       "Enter 'QUIT' to leave.\r\n", datasize, &data, &datasize));
    }
    lcrzo_er(lcrzo_sock_write(psock, data, datasize));
    lcrzo_er(lcrzo_data_free2(&data));
  }

  /* read data and simply echo it back */
  leave=LCRZO_FALSE;
  while(!leave)
  { lcrzo_er(lcrzo_sock_readm(psock, LCRZO_WAIT_INFINITE, &data, &datasize));
    if ( datasize!=0 )
    { /* display data on stdout */
      puts("");
      lcrzo_er(lcrzo_sock_print_infos(psock));
      if ( language==LCRZO_GLOBAL_LANG_FR )
        puts("On vient de lire :");
      else
        puts("We read :");
      lcrzo_er(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_DUMP));
      /* write data back */
      lcrzo_er(lcrzo_sock_write(psock, data, datasize));
      /* check for a 'QUIT' */
      ret=lcrzo_data_search_re(data, datasize, LCRZO_FALSE, +1, -1,
			       "^ *[Qq][Uu][Ii][Tt]",
			       NULL, NULL, NULL, NULL);
      if (ret==LCRZO_ERR_OK) leave=LCRZO_TRUE;
    }
    lcrzo_er(lcrzo_data_free2(&data));
  }

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000177_child_ftp(lcrzo_ptr infos, 
			     lcrzo_int32 infossize, 
			     lcrzo_process *pparentprocess)
{ shared_sock_address *pssa=(shared_sock_address *)infos;
  lcrzo_sock sock;
  int ret;

  /* check for correct usage of this function */
  if (infossize!=sizeof(shared_sock_address))
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);

  /* close parent process handle because we do not need it */
  lcrzo_epr(lcrzo_process_close(pparentprocess));

  /* actually, virtual tcp servers of lcrzo does not support
     multiclient servers. So we can only have a client at
     a given time */
  while(LCRZO_TRUE) 
  { /* initialize the socket */
    lcrzo_epr(lcrzo_sock_tcpser_virt(pssa->device, pssa->serveretha, 
				     pssa->serveripl, 21, &sock));
    /* echo every data */
    ret=lcrzoex_000177_echo(&sock, LCRZO_TRUE);
    /* close the socket */
    lcrzo_epr(lcrzo_sock_close(&sock));
    lcrzo_epr(ret);
  }
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000177_child_smtp(lcrzo_ptr infos, 
			      lcrzo_int32 infossize, 
			      lcrzo_process *pparentprocess)
{ shared_sock_address *pssa=(shared_sock_address *)infos;
  lcrzo_sock sock;
  int ret;

  /* check for correct usage of this function */
  if (infossize!=sizeof(shared_sock_address))
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);

  /* close parent process handle because we do not need it */
  lcrzo_epr(lcrzo_process_close(pparentprocess));

  /* actually, virtual tcp servers of lcrzo does not support
     multiclient servers. So we can only have a client at
     a given time */
  while(LCRZO_TRUE) 
  { /* initialize the socket */
    lcrzo_epr(lcrzo_sock_tcpser_virt(pssa->device, pssa->serveretha, 
				     pssa->serveripl, 25, &sock));
    /* echo every data */
    ret=lcrzoex_000177_echo(&sock, LCRZO_TRUE);
    /* close the socket */
    lcrzo_epr(lcrzo_sock_close(&sock));
    lcrzo_epr(ret);
  }
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000177_child_udp(lcrzo_ptr infos, 
			     lcrzo_int32 infossize, 
			     lcrzo_process *pparentprocess)
{ shared_sock_address *pssa=(shared_sock_address *)infos;
  lcrzo_sock sock;
  int ret;

  /* check for correct usage of this function */
  if (infossize!=sizeof(shared_sock_address))
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);

  /* close parent process handle because we do not need it */
  lcrzo_epr(lcrzo_process_close(pparentprocess));

  /* actually, virtual tcp servers of lcrzo does not support
     multiclient servers. So we can only have a client at
     a given time */
  while(LCRZO_TRUE) 
  { /* initialize the socket */
    lcrzo_epr(lcrzo_sock_udpser_virt(pssa->device, pssa->serveretha, 
				     pssa->serveripl, 7, &sock));
    /* echo every data */
    ret=lcrzoex_000177_echo(&sock, LCRZO_FALSE);
    /* close the socket */
    lcrzo_epr(lcrzo_sock_close(&sock));
    lcrzo_epr(ret);
  }
  return(LCRZO_ERR_OK);
}
