
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000190=
{ /*reference*/     "lcrzoex", 190,
  /*version*/       1, 1,
  /*title*/         "real tcp client [web get]",
  /*french title*/  "client tcp reel [web get]",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "server_name server_port url [proxyusername [siteusername [proxypasswd [sitepasswd]]]]",
  /*french usage*/  "nom_serveur port_serveur url [utilisateurproxy [utilisateursite [passwdproxy [passwdsite]]]]",
  /*usage example*/ "1.2.3.5 4444 /index.html joe \"\" hello",
  /*fr. usage ex.*/ "1.2.3.5 4444 /index.html laurent \"\" bonjour",
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000190_main(int argc, char *argv[]);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000190_main(int argc, char *argv[])
{ lcrzo_ipl ipl;
  lcrzo_sock sock;
  lcrzo_uint16 port;
  lcrzo_string proxyuser, proxypasswd, siteuser, sitepasswd, url;
  lcrzo_uint16 errornumber;
  lcrzo_string errormsg, servertype, contenttype;
  lcrzo_data header, body;
  lcrzo_int32 headersize, contentlength, bodysize;
  lcrzo_hs hs;

  lcrzo_init();
  if ( argc<4 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000190, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }

  /* the default resolver timeout is too short if it's long to reach
     webserver's DNS */
  lcrzo_epr(shared_http_set_resolver_timeout());

  /* initialize the needed variables */
  lcrzo_epr(shared_para_hs_init_dst(argv[1], hs));
  lcrzo_epr(lcrzo_ipl_init_hs(hs, &ipl));
  lcrzo_epr(shared_para_port_init_dst(argv[2], 80, &port));
  lcrzo_epr(shared_para_string_init_filedistant(argv[3], &url));

  /* init usernames and passwords */
  lcrzo_epr(shared_http_set_proxysite_userpass(argc, argv, 4,
					       &proxyuser, &proxypasswd, 
					       &siteuser, &sitepasswd))

  /* open the connection */
  lcrzo_epr(lcrzo_sock_tcpcli_real(ipl, port, &sock));
  
  /* send a request */
  lcrzo_epr(shared_http_request_get(&sock, url, hs, port, 
				     proxyuser, proxypasswd, 
				     siteuser, sitepasswd));
  /* free variables */
  lcrzo_epr(lcrzo_string_free2(&url));
  if (proxyuser!=NULL) { lcrzo_epr(lcrzo_string_free2(&proxyuser)); }
  if (proxypasswd!=NULL) { lcrzo_epr(lcrzo_string_free2(&proxypasswd)); }
  if (siteuser!=NULL) { lcrzo_epr(lcrzo_string_free2(&siteuser)); }
  if (sitepasswd!=NULL) { lcrzo_epr(lcrzo_string_free2(&sitepasswd)); }

  /* analyze the answer */
  lcrzo_epr(shared_http_response_readheader(&sock,
					    &errornumber,
					    &errormsg,
					    &servertype,
					    &contenttype,
					    &contentlength, NULL,
					    &header, &headersize));
  printf("Return %d (%s)\n", errornumber, errormsg);
  printf("Server: %s\n", servertype);
  printf("Content-Type: %s\n", contenttype);
  lcrzo_epr(lcrzo_int32_print("Content-length: ", contentlength, "%d", "\n"));
  lcrzo_epr(lcrzo_data_print(header, headersize, LCRZO_PRINTTYPE_DUMP));
  /* free variables */
  lcrzo_epr(lcrzo_string_free2(&errormsg));
  lcrzo_epr(lcrzo_string_free2(&servertype));
  lcrzo_epr(lcrzo_string_free2(&contenttype));
  lcrzo_epr(lcrzo_data_free2(&header));

  /* get the body */
  lcrzo_epr(shared_http_response_readbody(&sock, LCRZO_WAIT_INFINITE,
					  contentlength, 
					  &body, &bodysize));
  lcrzo_epr(lcrzo_data_print(body, bodysize, LCRZO_PRINTTYPE_DUMP));
  lcrzo_epr(lcrzo_data_free2(&body));

  lcrzo_epr(lcrzo_sock_close(&sock));

  lcrzo_close();
  return(LCRZO_ERR_OK);
}
