
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000197=
{ /*reference*/     "lcrzoex", 197,
  /*version*/       1, 2,
  /*title*/         "virtual tcp client [web post and save]",
  /*french title*/  "client tcp virtuel [web post et sauve]",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "device clieth serveth cliname servname cliport servport url param file ",
  /*french usage*/  "device ethcli ethserv nomcli nomserv portcli portserv url param fichier ",
  /*usage example*/ "eth0 aa:bb:cc:dd:ee:ff 11:22:33:44:55:66 1.2.3.5 1.2.3.4 5555 4444 /cgi-bin/t.sh \"v1=a&v2=b&v3=c\" /tmp/file joe \"\" hello",
  /*fr. usage ex.*/ "eth0 aa:bb:cc:dd:ee:ff 11:22:33:44:55:66 1.2.3.5 1.2.3.4 5555 4444 /cgi-bin/t.sh \"v1=a&v2=b&v3=c\" /tmp/fich laurent \"\" bonjour",
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000197_main(int argc, char *argv[]);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000197_main(int argc, char *argv[])
{ lcrzo_etha servereth, clienteth;
  lcrzo_ipl serveripl, clientipl;
  lcrzo_uint16 serverport, clientport;
  lcrzo_sock sock;
  lcrzo_string proxyuser, proxypasswd, siteuser, sitepasswd, url;
  lcrzo_uint16 errornumber;
  lcrzo_string errormsg, servertype, contenttype;
  lcrzo_data header, body;
  lcrzo_int32 headersize, contentlength, bodysize;
  lcrzo_device device;
  lcrzo_hs hs;

  lcrzo_init();
  if ( argc<11 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000197, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }

  /* the default resolver timeout is too short if it's long to reach
     webserver's DNS */
  lcrzo_epr(shared_http_set_resolver_timeout());

  /* initialize the needed variables */
  lcrzo_epr(shared_para_device_init_src(argv[1], device));
  lcrzo_epr(shared_para_etha_init_src(argv[2], clienteth));
  lcrzo_epr(shared_para_etha_init_dst(argv[3], servereth));
  lcrzo_epr(shared_para_ipl_init_src(argv[4], &clientipl));
  lcrzo_epr(shared_para_hs_init_dst(argv[5], hs));
  lcrzo_epr(lcrzo_ipl_init_hs(hs, &serveripl));
  lcrzo_epr(shared_para_port_init_src(argv[6], &clientport));
  lcrzo_epr(shared_para_port_init_dst(argv[7], 80, &serverport));
  lcrzo_epr(shared_para_string_init_filedistant(argv[8], &url));

  /* init usernames and passwords */
  lcrzo_epr(shared_http_set_proxysite_userpass(argc, argv, 11,
					       &proxyuser, &proxypasswd, 
					       &siteuser, &sitepasswd))

  /* open the connection */
  lcrzo_epr(lcrzo_sock_tcpcli_virt(device,
				   clienteth, servereth,
				   clientipl, serveripl, 
				   clientport, serverport,
				   &sock));
 
  /* send a request */
  lcrzo_epr(shared_http_request_post(&sock, url, hs, serverport, 
				     proxyuser, proxypasswd, 
				     siteuser, sitepasswd, argv[9]));
  /* free variables */
  lcrzo_epr(lcrzo_string_free2(&url));
  if (proxyuser!=NULL) { lcrzo_epr(lcrzo_string_free2(&proxyuser)); }
  if (proxypasswd!=NULL) { lcrzo_epr(lcrzo_string_free2(&proxypasswd)); }
  if (siteuser!=NULL) { lcrzo_epr(lcrzo_string_free2(&siteuser)); }
  if (sitepasswd!=NULL) { lcrzo_epr(lcrzo_string_free2(&sitepasswd)); }

  /* analyze the answer */
  lcrzo_epr(shared_http_response_readheader(&sock,
					    &errornumber,
					    &errormsg,
					    &servertype,
					    &contenttype,
					    &contentlength, NULL,
					    &header, &headersize));
  printf("Return %d (%s)\n", errornumber, errormsg);
  printf("Server: %s\n", servertype);
  printf("Content-Type: %s\n", contenttype);
  lcrzo_epr(lcrzo_int32_print("Content-length: ", contentlength, "%d", "\n"));
  lcrzo_epr(lcrzo_data_print(header, headersize, LCRZO_PRINTTYPE_DUMP));
  /* free variables */
  lcrzo_epr(lcrzo_string_free2(&errormsg));
  lcrzo_epr(lcrzo_string_free2(&servertype));
  lcrzo_epr(lcrzo_string_free2(&contenttype));
  lcrzo_epr(lcrzo_data_free2(&header));

  /* get the body */
  if (contentlength)
  { lcrzo_epr(shared_http_response_readbody(&sock, LCRZO_WAIT_INFINITE,
					    contentlength, 
					    &body, &bodysize));
    lcrzo_epr(lcrzo_filename_write(argv[10], LCRZO_FALSE, LCRZO_FALSE,
				   body, bodysize));
    lcrzo_epr(lcrzo_data_free2(&body));
  }

  lcrzo_epr(lcrzo_sock_close(&sock));

  lcrzo_close();
  return(LCRZO_ERR_OK);
}
