
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000208=
{ /*reference*/     "lcrzoex", 208,
  /*version*/       1, 1,
  /*title*/         "ARP ping",
  /*french title*/  "ARP ping",
  /*description*/   "the IP address has to be on the LAN",
  /*french desc.*/  "l'adresse IP doit etre sur le LAN",
  /*usage*/         "ip_to_ping",
  /*french usage*/  "adresse_ip_a_pinguer",
  /*usage example*/ "1.2.3.4",
  /*fr. usage ex.*/ "1.2.3.4",
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000208_main(int argc, char *argv[]);
int lcrzoex_000208_arp_ping(lcrzo_ipl ipl);
int lcrzoex_000208_arp_ping_spoofsniff(lcrzo_device device,
				       lcrzo_ipl ipl,
				       lcrzo_bool *pgotanswer,
				       lcrzo_etha etha);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000208_main(int argc, char *argv[])
{ lcrzo_ipl ipl;

  lcrzo_init();
  if ( argc<2 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000208, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }

  /* initialize infos */
  lcrzo_epr(shared_para_ipl_init_dst(argv[1], &ipl));

  /* try to ARP ping */
  lcrzo_epr(lcrzoex_000208_arp_ping(ipl));

  lcrzo_close();
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000208_arp_ping(lcrzo_ipl ipl)
{ lcrzo_device device;
  lcrzo_int32 count, i;
  lcrzo_bool gotanswer;
  lcrzo_etha etha;

  /* we try on all devices */
  lcrzo_epr(lcrzo_device_count(LCRZO_DEVICE_TYPE_ONLYBOARDS, &count));
  for ( i=1 ; i<=count ; i++ )
  { lcrzo_epr(lcrzo_device_value_pos(LCRZO_DEVICE_TYPE_ONLYBOARDS, i, device));
    printf("Ask on device=%s\n", device);
    lcrzo_epr(lcrzoex_000208_arp_ping_spoofsniff(device, ipl, 
						 &gotanswer, etha));
    if (gotanswer)
    { lcrzo_epr(lcrzo_ipl_print("The IP address ", ipl, " was found at "));
      lcrzo_epr(lcrzo_etha_print("", etha, "\n"));
      return(LCRZO_ERR_OK);
    }
  }
  puts("No arp replies received");
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000208_arp_ping_spoofsniff(lcrzo_device device,
				       lcrzo_ipl ipl,
				       lcrzo_bool *pgotanswer,
				       lcrzo_etha etha)
{ lcrzo_sniff sniff;
  lcrzo_spoof spoof;
  lcrzo_uint32 mtu;
  lcrzo_hdrleth hdrleth;
  lcrzo_hdrlarp hdrlarp;
  lcrzo_data data;
  lcrzo_int32 datasize;
  lcrzo_ipa ipa;
  lcrzo_bool gotanswer, equal;
  lcrzo_time endingtime, currenttime, waitingtime;
  int ret, i;

  /* initialize Ethernet header */
  lcrzo_epr(lcrzo_hdrleth_initdefault(&hdrleth));
  for ( i=0 ; i<LCRZO_ETHA_MAXBYTES ; i++ )
  { lcrzo_epr(lcrzo_uint8_init_rand(1, 0xFE, &(hdrleth.src[i])));
  }
  lcrzo_epr(lcrzo_etha_init(0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, hdrleth.dst));
  hdrleth.type=LCRZO_HDRLETH_TYPE_ARP;

  /* initialize ARP header */
  lcrzo_epr(lcrzo_hdrlarp_initdefault(&hdrlarp));
  hdrlarp.op = 1;
  memcpy(hdrlarp.hw_src, hdrleth.src, LCRZO_ETHA_MAXBYTES);
  for ( i=0 ; i<LCRZO_IPA_MAXBYTES ; i++ )
  { lcrzo_epr(lcrzo_uint8_init_rand(1, 0xFE, &(hdrlarp.prot_src[i])));
  }
  lcrzo_epr(lcrzo_etha_init(0,0,0,0,0,0, hdrlarp.hw_dst));
  lcrzo_epr(lcrzo_ipa_init_ipl(ipl, hdrlarp.prot_dst));

  /* initialize sniff */
  lcrzo_epr(lcrzo_mtu_init_device(device, &mtu));
  lcrzo_epr(lcrzo_sniff_init(device, mtu, LCRZO_SNIFF_TYPE_NORMAL, 
			     "arp", &sniff));

  /* initialize spoof */
  lcrzo_epr(lcrzo_spoof_init(&spoof));

  /* send ARP request*/
  lcrzo_epr(lcrzo_spoof_etharp(&spoof, device, hdrleth, hdrlarp));

  /* send another ARP request, but with IP on the same network 
     because some devices only reply to arp on their network */
  /* set another Ethernet */
  for ( i=0 ; i<LCRZO_ETHA_MAXBYTES ; i++ )
  { lcrzo_epr(lcrzo_uint8_init_rand(1, 0xFE, &(hdrleth.src[i])));
  }
  memcpy(hdrlarp.hw_src, hdrleth.src, LCRZO_ETHA_MAXBYTES);
  /* set IP */
  lcrzo_epr(lcrzo_ipa_init_ipl(ipl, hdrlarp.prot_src));
  if ( hdrlarp.prot_src[3] & 1 )
    hdrlarp.prot_src[3] &= 0xFE;
  else
    hdrlarp.prot_src[3] |= 1;
  /* spoof */
  lcrzo_epr(lcrzo_spoof_etharp(&spoof, device, hdrleth, hdrlarp));

  /* close spoof */
  lcrzo_epr(lcrzo_spoof_close(&spoof));

  /* we compare ipa */
  lcrzo_epr(lcrzo_ipa_init_ipl(ipl, ipa));
  
  /* sniff and analyze answers */  
  gotanswer=LCRZO_FALSE;
  lcrzo_epr(lcrzo_time_init_currenttime(&currenttime));
  lcrzo_epr(lcrzo_time_plus_sec(&currenttime, 1, &endingtime));
  ret=LCRZO_ERR_OK;
  while (LCRZO_TRUE)
  { /* sniff */
    ret=lcrzo_time_init_currenttime(&currenttime);
    if (ret!=LCRZO_ERR_OK) break;
    ret=lcrzo_time_minus_time(&endingtime, &currenttime, &waitingtime);
    if (ret==LCRZO_ERR_SPTIMEDIFFNEG) { ret=LCRZO_ERR_OK; break; }
    if (ret!=LCRZO_ERR_OK) break;
    ret=lcrzo_sniff_nextm(&sniff, &waitingtime, &data, &datasize);
    if (ret==LCRZO_ERR_OKTEMPDATAEND) 
    { ret=LCRZO_ERR_OK; 
      break;
    }
    if (ret!=LCRZO_ERR_OK) break;
    ret=lcrzo_packet_decode_etharp(data, datasize, NULL, &hdrlarp);
    if ( ret==LCRZO_ERR_OK && hdrlarp.op==2 )
    { lcrzo_er(lcrzo_ipa_equal2(hdrlarp.prot_src, ipa, &equal));
      if (equal)
      { /* found */
	memcpy(etha, hdrlarp.hw_src, LCRZO_ETHA_MAXBYTES);
	gotanswer=LCRZO_TRUE;
	lcrzo_epr(lcrzo_data_free2(&data));
	break;
      }
    }
    lcrzo_epr(lcrzo_data_free2(&data));
  }
  lcrzo_epr(lcrzo_sniff_close(&sniff));
  lcrzo_epr(ret);

  if (pgotanswer!=NULL) *pgotanswer=gotanswer;
  return(LCRZO_ERR_OK);
}
