
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000211=
{ /*reference*/     "lcrzoex", 211,
  /*version*/       1, 0,
  /*title*/         "sniff and print open ports",
  /*french title*/  "sniffe et affiche les ports ouverts",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "[device [\"bpf_filter\"]]",
  /*french usage*/  "[device [\"filtre_bpf\"]]",
  /*usage example*/ "eth0 \"net 192.168.1.0/24\"",
  /*fr. usage ex.*/ "eth0 \"net 192.168.1.0/24\"", 
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declaration */
int lcrzoex_000211_main(int argc, char *argv[]);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000211_main(int argc, char *argv[])
{ lcrzo_device device;
  lcrzo_uint32 mtu;
  lcrzo_sniff sniff;
  lcrzo_data sniffdata;
  lcrzo_int32 sniffdatasize;
  lcrzo_list list;
  lcrzo_int16 language;
  lcrzo_string snifffilter;
  int ret;
  
  lcrzo_init();
  /* init device and mtu */
  if (argc>=2)
  { lcrzo_epr(shared_para_device_init_src(argv[1], device));
  }
  else
  { lcrzo_epr(lcrzo_device_value_pos(LCRZO_DEVICE_TYPE_BEST, 1, device));
  }
  lcrzo_epr(lcrzo_mtu_init_device(device, &mtu));

  /* init sniff */
  if (argc<=2)
  { lcrzo_epr(lcrzo_sniff_init(device, mtu, LCRZO_SNIFF_TYPE_NORMAL, 
			       "", &sniff));
  }
  else
  { lcrzo_epr(shared_para_string_init_snifffilter(argv[2], &snifffilter));
    lcrzo_epr(lcrzo_sniff_init(device, mtu, LCRZO_SNIFF_TYPE_NORMAL, 
			       snifffilter, &sniff));
    lcrzo_epr(lcrzo_string_free2(&snifffilter));
  }

  /* init the chained list */
  lcrzo_epr(lcrzo_list_init(&list, sizeof(shared_sniff_ipport), NULL));

  /* print message */
  lcrzo_epr(lcrzo_global_get_language(&language));
  if (language==LCRZO_GLOBAL_LANG_FR)
    puts("Attend que des paquets passent sur le reseau...");
  else
    puts("Waiting for packets on the network...");
  
 /* main loop */
  ret=LCRZO_ERR_OK;
  while(LCRZO_TRUE)
  { ret=lcrzo_sniff_nextm(&sniff, LCRZO_WAIT_INFINITE, 
			  &sniffdata, &sniffdatasize);
    if (ret!=LCRZO_ERR_OK) break;
    /* analyze data in TCP headers */
    lcrzo_epr(shared_sniff_decodetcp_ipport(&list, sniffdata, sniffdatasize));
    /* analyze data in UDP headers */
    lcrzo_epr(shared_sniff_decodeudp_ipport(&list, sniffdata, sniffdatasize));
    lcrzo_epr(lcrzo_data_free2(&sniffdata));
  }

  /* close everything */
  lcrzo_epr(lcrzo_list_close(&list));
  lcrzo_epr(lcrzo_sniff_close(&sniff));
  lcrzo_epr(lcrzo_err_print(ret));
  lcrzo_close();
  return (ret);
}



