
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000212=
{ /*reference*/     "lcrzoex", 212,
  /*version*/       1, 0,
  /*title*/         "sniff and print network statistics",
  /*french title*/  "sniffe et affiche des statistiques reseau",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "[device [\"bpf_filter\"]]",
  /*french usage*/  "[device [\"filtre_bpf\"]]",
  /*usage example*/ "eth0 \"net 192.168.1.0/24\"",
  /*fr. usage ex.*/ "eth0 \"net 192.168.1.0/24\"", 
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* types declarations */
typedef struct {
  lcrzo_list eth;
  lcrzo_list etharp;
  lcrzo_list ethrarp;
  lcrzo_list ethip;
  lcrzo_list ethipudp;
  lcrzo_list ethiptcp;
  lcrzo_list ethipicmp;
} lcrzoex_000212_lists;
typedef struct {
  lcrzo_uint32 type;
  lcrzo_uint32 numberofpackets;
  lcrzo_uint32 totalsize;
} lcrzoex_000202_elem;

/*-------------------------------------------------------------*/
/* function declaration */
int lcrzoex_000212_main(int argc, char *argv[]);
int lcrzoex_000212_analyzedata(lcrzoex_000212_lists *plists, 
			       lcrzo_constdata sniffdata,
			       lcrzo_int32 sniffdatasize);
int lcrzoex_000212_addorupdate(lcrzo_list *plist,
			       lcrzo_uint32 type,
			       lcrzo_uint32 size);
int lcrzoex_000212_search_elem(lcrzo_constptr pelem,
			       lcrzo_ptr pinfos,
			       lcrzo_bool *pfound);
int lcrzoex_000212_sort_type(lcrzo_constptr peleminf, 
			     lcrzo_constptr pelemsup, 
			     lcrzo_ptr pinfos,
			     lcrzo_int8 *pcmp);
int lcrzoex_000212_dispstats(lcrzoex_000212_lists *plists);
int lcrzoex_000212_dispstatseth(lcrzo_constptr pelem, lcrzo_ptr pinfos);
int lcrzoex_000212_dispstatsetharp(lcrzo_constptr pelem, lcrzo_ptr pinfos);
int lcrzoex_000212_dispstatsethrarp(lcrzo_constptr pelem, lcrzo_ptr pinfos);
int lcrzoex_000212_dispstatsethip(lcrzo_constptr pelem, lcrzo_ptr pinfos);
int lcrzoex_000212_dispstatsethipudp(lcrzo_constptr pelem, lcrzo_ptr pinfos);
int lcrzoex_000212_dispstatsethiptcp(lcrzo_constptr pelem, lcrzo_ptr pinfos);
int lcrzoex_000212_dispstatsethipicmp(lcrzo_constptr pelem,lcrzo_ptr pinfos);

#define LCRZOEX_000212_DISPLAYPERIOD 3

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000212_main(int argc, char *argv[])
{ lcrzo_device device;
  lcrzo_uint32 mtu;
  lcrzo_sniff sniff;
  lcrzo_data sniffdata;
  lcrzo_int32 sniffdatasize;
  lcrzoex_000212_lists lists;
  lcrzo_time currenttime;
  lcrzo_uint32 currentsecond, currenttimesec;
  lcrzo_int16 language;
  lcrzo_string snifffilter;
  int ret;
  
  lcrzo_init();
  /* init device and mtu */
  if (argc>=2)
  { lcrzo_epr(shared_para_device_init_src(argv[1], device));
  }
  else
  { lcrzo_epr(lcrzo_device_value_pos(LCRZO_DEVICE_TYPE_BEST, 1, device));
  }
  lcrzo_epr(lcrzo_mtu_init_device(device, &mtu));

  /* init sniff */
  if (argc<=2)
  { lcrzo_epr(lcrzo_sniff_init(device, mtu, LCRZO_SNIFF_TYPE_NORMAL, 
			       "", &sniff));
  }
  else
  { lcrzo_epr(shared_para_string_init_snifffilter(argv[2], &snifffilter));
    lcrzo_epr(lcrzo_sniff_init(device, mtu, LCRZO_SNIFF_TYPE_NORMAL, 
			       snifffilter, &sniff));
    lcrzo_epr(lcrzo_string_free2(&snifffilter));
 }

  /* init the chained lists */
  lcrzo_epr(lcrzo_list_init(&lists.eth, sizeof(lcrzoex_000202_elem), NULL));
  lcrzo_epr(lcrzo_list_init(&lists.etharp, sizeof(lcrzoex_000202_elem), NULL));
  lcrzo_epr(lcrzo_list_init(&lists.ethrarp, sizeof(lcrzoex_000202_elem),NULL));
  lcrzo_epr(lcrzo_list_init(&lists.ethip, sizeof(lcrzoex_000202_elem), NULL));
  lcrzo_epr(lcrzo_list_init(&lists.ethipudp,sizeof(lcrzoex_000202_elem),NULL));
  lcrzo_epr(lcrzo_list_init(&lists.ethiptcp,sizeof(lcrzoex_000202_elem),NULL));
  lcrzo_epr(lcrzo_list_init(&lists.ethipicmp,sizeof(lcrzoex_000202_elem),
			    NULL));

  /* get time */
  lcrzo_epr(lcrzo_time_init_currenttime(&currenttime));
  lcrzo_epr(lcrzo_time_get_sec(&currenttime, &currentsecond));

  /* print message */
  lcrzo_epr(lcrzo_global_get_language(&language));
  if (language==LCRZO_GLOBAL_LANG_FR)
  { puts("Attend que des paquets passent sur le reseau...");
    puts(" protocole           nombre_de_paquets  taille_totale"); 
  }
  else
  { puts("Waiting for packets on the network...");
    puts(" protocol            number_of_packets  total_size");
  }

  /* main loop */
  ret=LCRZO_ERR_OK;
  while (LCRZO_TRUE)
  { ret=lcrzo_sniff_nextm(&sniff, LCRZO_WAIT_INFINITE, 
			  &sniffdata, &sniffdatasize);
    if (ret!=LCRZO_ERR_OK) break;
    /* analyze data */
    lcrzo_epr(lcrzoex_000212_analyzedata(&lists, sniffdata, sniffdatasize));
    lcrzo_epr(lcrzo_data_free2(&sniffdata));
    /* if N seconds has elapsed, we print */
    lcrzo_epr(lcrzo_time_init_currenttime(&currenttime));
    lcrzo_epr(lcrzo_time_get_sec(&currenttime, &currenttimesec));
    if ( currenttimesec >= LCRZOEX_000212_DISPLAYPERIOD + currentsecond )
    { currentsecond=currenttimesec;
      lcrzo_epr(lcrzo_uint32_print("-- ", currentsecond, "%u", "\n"));
      lcrzo_epr(lcrzoex_000212_dispstats(&lists));
    }
  }

  /* close everything */
  lcrzo_epr(lcrzo_sniff_close(&sniff));
  lcrzo_epr(lcrzo_list_close(&lists.ethipicmp));
  lcrzo_epr(lcrzo_list_close(&lists.ethiptcp));
  lcrzo_epr(lcrzo_list_close(&lists.ethipudp));
  lcrzo_epr(lcrzo_list_close(&lists.ethip));
  lcrzo_epr(lcrzo_list_close(&lists.ethrarp));
  lcrzo_epr(lcrzo_list_close(&lists.etharp));
  lcrzo_epr(lcrzo_list_close(&lists.eth));
  lcrzo_epr(lcrzo_err_print(ret));
  lcrzo_close();
  return (ret);
}

/*-------------------------------------------------------------*/
int lcrzoex_000212_analyzedata(lcrzoex_000212_lists *plists, 
			       lcrzo_constdata sniffdata,
			       lcrzo_int32 sniffdatasize)
{ lcrzo_hdrleth hdreth;
  lcrzo_hdrlarp hdrarp;   
  lcrzo_hdrlip hdrip;
  lcrzo_hdrludp hdrudp;
  lcrzo_hdrltcp hdrtcp;
  lcrzo_hdrlicmp hdricmp;
  lcrzo_data ethdata, ipdata;
  lcrzo_uint16 ethdatasize, ipdatasize;
  lcrzo_uint32 sds, port;
  int ret;

  sds=sniffdatasize;
  /* try to decode Ethernet */
  ret=lcrzo_packet_decodem_ethdata(sniffdata, sniffdatasize, 
				   &hdreth, &ethdata, &ethdatasize);
  if (ret!=LCRZO_ERR_OK)
    return(LCRZO_ERR_OK);

  /* set Ethernet statistics */
  if ( hdreth.type>=1 && hdreth.type<=1500 )
  { /* Novell Netware : set type to one */
    hdreth.type=1;
  }
  lcrzo_epr(lcrzoex_000212_addorupdate(&(plists->eth), hdreth.type, sds));

  /* decode Ethernet data */
  switch(hdreth.type) {
  case LCRZO_HDRLETH_TYPE_ARP :
    ret=lcrzo_packet_decode_arp(ethdata, ethdatasize, &hdrarp);
    lcrzo_epr(lcrzo_data_free2(&ethdata));
    if (ret!=LCRZO_ERR_OK) return(LCRZO_ERR_OK);
    lcrzo_epr(lcrzoex_000212_addorupdate(&(plists->etharp), hdrarp.op, sds));
    break;
  case LCRZO_HDRLETH_TYPE_RARP :
    ret=lcrzo_packet_decode_arp(ethdata, ethdatasize, &hdrarp);
    lcrzo_epr(lcrzo_data_free2(&ethdata));
    if (ret!=LCRZO_ERR_OK) return(LCRZO_ERR_OK);
    lcrzo_epr(lcrzoex_000212_addorupdate(&(plists->ethrarp), hdrarp.op,sds));
    break;
  case LCRZO_HDRLETH_TYPE_IP :
    ret=lcrzo_packet_decodem_ipoptdata(ethdata, ethdatasize, 
				       &hdrip, NULL,NULL, &ipdata,&ipdatasize);
    lcrzo_epr(lcrzo_data_free2(&ethdata));
    if (ret!=LCRZO_ERR_OK) return(LCRZO_ERR_OK);
    lcrzo_epr(lcrzoex_000212_addorupdate(&(plists->ethip),hdrip.protocol,sds));
    /* decode IP data */
    switch(hdrip.protocol) {
    case LCRZO_HDRLIP_PROTOCOL_UDP :
      ret=lcrzo_packet_decodem_udpdata(ipdata, ipdatasize, &hdrudp, NULL,NULL);
      lcrzo_epr(lcrzo_data_free2(&ipdata));
      if (ret!=LCRZO_ERR_OK) return(LCRZO_ERR_OK);
      lcrzo_epr(lcrzoex_000212_addorupdate(&(plists->ethipudp),
					   hdrudp.dport, sds));
      break;
    case LCRZO_HDRLIP_PROTOCOL_TCP :
      ret=lcrzo_packet_decodem_tcpoptdata(ipdata, ipdatasize, 
					  &hdrtcp, NULL,NULL, NULL,NULL);
      lcrzo_epr(lcrzo_data_free2(&ipdata));
      if (ret!=LCRZO_ERR_OK) return(LCRZO_ERR_OK);
      port=0xFFFFFFFFu;
      if (hdrtcp.dport<1024) port=hdrtcp.dport;
      if (hdrtcp.sport<1024 && hdrtcp.sport<port) port=hdrtcp.sport;
      lcrzo_epr(lcrzoex_000212_addorupdate(&(plists->ethiptcp), port, sds));
      break;
    case LCRZO_HDRLIP_PROTOCOL_ICMP :
      ret=lcrzo_packet_decodem_icmpdata(ipdata, ipdatasize,&hdricmp,NULL,NULL);
      lcrzo_epr(lcrzo_data_free2(&ipdata));
      if (ret!=LCRZO_ERR_OK) return(LCRZO_ERR_OK);
      lcrzo_epr(lcrzoex_000212_addorupdate(&(plists->ethipicmp),
					   (hdricmp.type<<16)|hdricmp.code,
					   sds));
      break;
    default :
      lcrzo_epr(lcrzo_data_free2(&ipdata));
      break;
    }

    break;
  default :
    /* Novell Netware or unknown */
    lcrzo_epr(lcrzo_data_free2(&ethdata));
    /* we decode nothing else */
    break;
  }

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000212_addorupdate(lcrzo_list *plist,
			       lcrzo_uint32 type,
			       lcrzo_uint32 size)
{ lcrzoex_000202_elem elem, foundelem;
  lcrzo_int32 pos;
  int ret;

  /* search if this type was already saved */
  elem.type=type;
  ret=lcrzo_list_search_all2(plist, &lcrzoex_000212_search_elem,
			     &elem, &pos, &foundelem);
  if (ret==LCRZO_ERR_OK)
  { /* this elem was found, we update it */
    foundelem.numberofpackets++;
    foundelem.totalsize+=size;
    lcrzo_epr(lcrzo_list_replace_pos(plist, pos, &foundelem));
  }
  else
  { /* this elem wasn't found, we add it */
    elem.numberofpackets=1;
    elem.totalsize=size;
    lcrzo_epr(lcrzo_list_add_first(plist, &elem));
  }

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000212_search_elem(lcrzo_constptr pelem,
			       lcrzo_ptr pinfos,
			       lcrzo_bool *pfound)
{ const lcrzoex_000202_elem e1=*(const lcrzoex_000202_elem *)pelem;
  const lcrzoex_000202_elem e2=*(const lcrzoex_000202_elem *)pinfos;
  lcrzo_bool found;
  found=LCRZO_FALSE;
  if (e1.type==e2.type) found=LCRZO_TRUE;
  if (pfound!=NULL) *pfound=found;
  return(LCRZO_ERR_OK);
  pinfos=pinfos; /*just for the compiler warning*/
}

/*-------------------------------------------------------------*/
/* this function sort by types. We could have sorted by numberofpacket
   or totalsize. */
int lcrzoex_000212_sort_type(lcrzo_constptr peleminf, 
			     lcrzo_constptr pelemsup, 
			     lcrzo_ptr pinfos,
			     lcrzo_int8 *pcmp)
{ const lcrzoex_000202_elem e1=*(const lcrzoex_000202_elem *)peleminf;
  const lcrzoex_000202_elem e2=*(const lcrzoex_000202_elem *)pelemsup;
  lcrzo_int8 cmp;

  if ( e1.type==e2.type ) cmp=0;
  else if ( e1.type<e2.type ) cmp=-1;
  else cmp=+1;
  if (pcmp!=NULL) *pcmp=cmp;
  return(LCRZO_ERR_OK);
  pinfos=pinfos; /*just for the compiler warning*/
}

/*-------------------------------------------------------------*/
int lcrzoex_000212_dispstats(lcrzoex_000212_lists *plists)
{
  /* display Ethernet statistics */
  lcrzo_epr(lcrzo_list_sort_all2(&(plists->eth),
				 &lcrzoex_000212_sort_type, NULL));
  lcrzo_epr(lcrzo_list_loop_all(&(plists->eth),
				&lcrzoex_000212_dispstatseth, NULL));
  lcrzo_epr(lcrzo_list_remove_all(&(plists->eth)));
  /* display ARP statistics */
  lcrzo_epr(lcrzo_list_sort_all2(&(plists->etharp),
				 &lcrzoex_000212_sort_type, NULL));
  lcrzo_epr(lcrzo_list_loop_all(&(plists->etharp),
				&lcrzoex_000212_dispstatsetharp, NULL));
  lcrzo_epr(lcrzo_list_remove_all(&(plists->etharp)));
  /* display RARP statistics */
  lcrzo_epr(lcrzo_list_sort_all2(&(plists->ethrarp),
				 &lcrzoex_000212_sort_type, NULL));
  lcrzo_epr(lcrzo_list_loop_all(&(plists->ethrarp),
				&lcrzoex_000212_dispstatsethrarp, NULL));
  lcrzo_epr(lcrzo_list_remove_all(&(plists->ethrarp)));
  /* display IP statistics */
  lcrzo_epr(lcrzo_list_sort_all2(&(plists->ethip),
				 &lcrzoex_000212_sort_type, NULL));
  lcrzo_epr(lcrzo_list_loop_all(&(plists->ethip),
				&lcrzoex_000212_dispstatsethip, NULL));
  lcrzo_epr(lcrzo_list_remove_all(&(plists->ethip)));
  /* display UDP statistics */
  lcrzo_epr(lcrzo_list_sort_all2(&(plists->ethipudp),
				 &lcrzoex_000212_sort_type, NULL));
  lcrzo_epr(lcrzo_list_loop_all(&(plists->ethipudp),
				&lcrzoex_000212_dispstatsethipudp, NULL));
  lcrzo_epr(lcrzo_list_remove_all(&(plists->ethipudp)));
  /* display TCP statistics */
  lcrzo_epr(lcrzo_list_sort_all2(&(plists->ethiptcp),
				 &lcrzoex_000212_sort_type, NULL));
  lcrzo_epr(lcrzo_list_loop_all(&(plists->ethiptcp),
				&lcrzoex_000212_dispstatsethiptcp, NULL));
  lcrzo_epr(lcrzo_list_remove_all(&(plists->ethiptcp)));
  /* display ICMP statistics */
  lcrzo_epr(lcrzo_list_sort_all2(&(plists->ethipicmp),
				 &lcrzoex_000212_sort_type, NULL));
  lcrzo_epr(lcrzo_list_loop_all(&(plists->ethipicmp),
				&lcrzoex_000212_dispstatsethipicmp, NULL));
  lcrzo_epr(lcrzo_list_remove_all(&(plists->ethipicmp)));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000212_dispstatseth(lcrzo_constptr pelem, lcrzo_ptr pinfos)
{ const lcrzoex_000202_elem e=*(const lcrzoex_000202_elem *)pelem;
  
  switch((int)e.type) {
  case 1: 
    printf("Netware\t"); break;
  case LCRZO_HDRLETH_TYPE_IP: 
    printf("IP\t"); break;
  case LCRZO_HDRLETH_TYPE_ARP: 
    printf("ARP\t"); break;
  case LCRZO_HDRLETH_TYPE_RARP: 
    printf("RARP\t"); break;
  default :
    printf("Unknown\t"); break;
  }
  lcrzo_epr(lcrzo_uint32_print("\t\t", e.numberofpackets, "%.10u", ""));
  lcrzo_epr(lcrzo_uint32_print("\t", e.totalsize, "%.10u", "\n"));
  return(LCRZO_ERR_OK);
  pinfos=pinfos; /*just for the compiler warning*/
}

/*-------------------------------------------------------------*/
int lcrzoex_000212_dispstatsetharp(lcrzo_constptr pelem, lcrzo_ptr pinfos)
{ const lcrzoex_000202_elem e=*(const lcrzoex_000202_elem *)pelem;
  
  switch((int)e.type) {
  case 1: 
    printf("ARP:Request\t"); break;
  case 2: 
    printf("ARP:Reply\t"); break;
  default :
    printf("ARP:Unknown\t"); break;
  }
  lcrzo_epr(lcrzo_uint32_print("\t", e.numberofpackets, "%.10u", ""));
  lcrzo_epr(lcrzo_uint32_print("\t", e.totalsize, "%.10u", "\n"));
  return(LCRZO_ERR_OK);
  pinfos=pinfos; /*just for the compiler warning*/
}

/*-------------------------------------------------------------*/
int lcrzoex_000212_dispstatsethrarp(lcrzo_constptr pelem, lcrzo_ptr pinfos)
{ const lcrzoex_000202_elem e=*(const lcrzoex_000202_elem *)pelem;
  
  switch((int)e.type) {
  case 3: 
    printf("RARP:Request\t"); break;
  case 4: 
    printf("RARP:Reply\t"); break;
  default :
    printf("RARP:Unknown\t"); break;
  }
  lcrzo_epr(lcrzo_uint32_print("\t", e.numberofpackets, "%.10u", ""));
  lcrzo_epr(lcrzo_uint32_print("\t", e.totalsize, "%.10u", "\n"));
  return(LCRZO_ERR_OK);
  pinfos=pinfos; /*just for the compiler warning*/
}

/*-------------------------------------------------------------*/
int lcrzoex_000212_dispstatsethip(lcrzo_constptr pelem, lcrzo_ptr pinfos)
{ const lcrzoex_000202_elem e=*(const lcrzoex_000202_elem *)pelem;

  switch((int)e.type) {
  case LCRZO_HDRLIP_PROTOCOL_UDP: 
    printf("IP:UDP\t"); break;
  case LCRZO_HDRLIP_PROTOCOL_TCP: 
    printf("IP:TCP\t"); break;
  case LCRZO_HDRLIP_PROTOCOL_ICMP: 
    printf("IP:ICMP\t"); break;
  default :
    printf("IP:Unknown\t"); break;
  }
  lcrzo_epr(lcrzo_uint32_print("\t\t", e.numberofpackets, "%.10u", ""));
  lcrzo_epr(lcrzo_uint32_print("\t", e.totalsize, "%.10u", "\n"));
  return(LCRZO_ERR_OK);
  pinfos=pinfos; /*just for the compiler warning*/
}

/*-------------------------------------------------------------*/
int lcrzoex_000212_dispstatsethipudp(lcrzo_constptr pelem, lcrzo_ptr pinfos)
{ const lcrzoex_000202_elem e=*(const lcrzoex_000202_elem *)pelem;

  printf("IP:UDP:");
  if (e.type==0xFFFFFFFFu)
    printf(">1024\t");
  else
    printf("%d   \t", (lcrzo_uint16)e.type);
  lcrzo_epr(lcrzo_uint32_print("\t", e.numberofpackets, "%.10u", ""));
  lcrzo_epr(lcrzo_uint32_print("\t", e.totalsize, "%.10u", "\n"));
  return(LCRZO_ERR_OK);
  pinfos=pinfos; /*just for the compiler warning*/
}

/*-------------------------------------------------------------*/
int lcrzoex_000212_dispstatsethiptcp(lcrzo_constptr pelem, lcrzo_ptr pinfos)
{ const lcrzoex_000202_elem e=*(const lcrzoex_000202_elem *)pelem;

  printf("IP:TCP:");
  if (e.type==0xFFFFFFFFu)
    printf(">1024\t");
  else
    printf("%d   \t", (lcrzo_uint16)e.type);
  lcrzo_epr(lcrzo_uint32_print("\t", e.numberofpackets, "%.10u", ""));
  lcrzo_epr(lcrzo_uint32_print("\t", e.totalsize, "%.10u", "\n"));
  return(LCRZO_ERR_OK);
  pinfos=pinfos; /*just for the compiler warning*/
}

/*-------------------------------------------------------------*/
int lcrzoex_000212_dispstatsethipicmp(lcrzo_constptr pelem, lcrzo_ptr pinfos)
{ const lcrzoex_000202_elem e=*(const lcrzoex_000202_elem *)pelem;
  lcrzo_uint8 type, code;

  type=(lcrzo_uint8)(e.type>>16);
  code=(lcrzo_uint8)(e.type&0xFFFF);

  printf("IP:ICMP:%d:%d", type, code);
  lcrzo_epr(lcrzo_uint32_print("\t\t", e.numberofpackets, "%.10u", ""));
  lcrzo_epr(lcrzo_uint32_print("\t", e.totalsize, "%.10u", "\n"));
  return(LCRZO_ERR_OK);
  pinfos=pinfos; /*just for the compiler warning*/
}
