
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000222=
{ /*reference*/     "lcrzoex", 222,
  /*version*/       1, 0,
  /*title*/         "Eth record : modify EthIP fields, spoof",
  /*french title*/  "Eth record : modifie champs EthIP, spoofe",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "file device srceth dsteth srcip dstip [startpacket [endpacket]]",
  /*french usage*/  "nom_fichier device ethsrc ethdst ipsrc ipdst [packet_debut [packet_fin]]",
  /*usage example*/ "/tmp/file eth0 a:a:a:a:a:a b:b:b:b:b:b 1.2.3.4 5.6.7.8 1 5",
  /*fr. usage ex.*/ "/tmp/fichier eth0 a:a:a:a:a:a b:b:b:b:b:b 1.2.3.4 5.6.7.8 1 5", 
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declaration */
int lcrzoex_000222_main(int argc, char *argv[]);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000222_main(int argc, char *argv[])
{ lcrzo_int32 start, end;
  lcrzo_record record;
  shared_record_addmod sram;
  lcrzo_spoof spoof;
  lcrzo_device device;
  lcrzo_hdrleth modhdreth;
  lcrzo_hdrlip modhdrip;

  lcrzo_init();
  if ( argc<7 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000222, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }
  if (argc>=8)
  { lcrzo_epr(lcrzo_int32_init_text(argv[7], &start));
  }
  else 
  { start=+1;
  }
  if (argc>=9)
  { lcrzo_epr(lcrzo_int32_init_text(argv[8], &end));
  }
  else
  { end=-1;
  }

  /* init device */
  lcrzo_epr(shared_para_device_init_src(argv[2], device));

  /* init new addresses */
  lcrzo_epr(lcrzo_hdrleth_initdefault(&modhdreth));
  lcrzo_epr(shared_para_etha_init_src(argv[3], modhdreth.src));
  lcrzo_epr(shared_para_etha_init_dst(argv[4], modhdreth.dst));
  lcrzo_epr(lcrzo_hdrlip_initdefault(&modhdrip));
  lcrzo_epr(shared_para_ipl_init_src(argv[5], &(modhdrip.saddr)));
  lcrzo_epr(shared_para_ipl_init_dst(argv[6], &(modhdrip.daddr)));

  /* init spoof */
  lcrzo_epr(lcrzo_spoof_init(&spoof));
  lcrzo_epr(lcrzo_spoof_set_printbeforesending(&spoof, LCRZO_TRUE));

  /* initialize the sram */
  lcrzo_epr(shared_record_addmod_initdefault(&sram));
  sram.pspoof=&spoof;
  sram.pspoofdev=&device;
  sram.pmodhdreth=&modhdreth;
  sram.pmodhdrip=&modhdrip;

  /* loop through each packet saved in the record */
  lcrzo_epr(lcrzo_record_open(&record, argv[1], LCRZO_RECORD_OPEN_READ));
  lcrzo_epr(lcrzo_record_loop_range(&record, start, end,
				    &shared_record_addmod_eth_spoof, &sram));
  lcrzo_epr(lcrzo_record_close(&record));

  /* close spoof */
  lcrzo_epr(lcrzo_spoof_close(&spoof));

  lcrzo_close();
  return(LCRZO_ERR_OK);
}
