
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000241=
{ /*reference*/     "lcrzoex", 241,
  /*version*/       1, 0,
  /*title*/         "ICMP record : add IP header, modify ICMP fields, spoof",
  /*french title*/  "ICMP record : ajoute entete IP, modifie champs ICMP, spoofe",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "file srcip dstip type code [startpacket [endpacket]]",
  /*french usage*/  "nom_fichier ipsrc ipdst type code [packet_debut [packet_fin]]",
  /*usage example*/ "/tmp/file 1.2.3.4 5.6.7.8 11 12 1 5",
  /*fr. usage ex.*/ "/tmp/fichier 1.2.3.4 5.6.7.8 11 12 1 5", 
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declaration */
int lcrzoex_000241_main(int argc, char *argv[]);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000241_main(int argc, char *argv[])
{ lcrzo_int32 start, end;
  lcrzo_record record;
  shared_record_addmod sram;
  lcrzo_spoof spoof;
  lcrzo_device device;
  lcrzo_hdrlip addhdrip;
  lcrzo_hdrlicmp modhdricmp;

  lcrzo_init();
  if ( argc<6 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000241, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }
  if (argc>=7)
  { lcrzo_epr(lcrzo_int32_init_text(argv[6], &start));
  }
  else 
  { start=+1;
  }
  if (argc>=8)
  { lcrzo_epr(lcrzo_int32_init_text(argv[7], &end));
  }
  else
  { end=-1;
  }

  /* init new addresses */
  lcrzo_epr(lcrzo_hdrlip_initdefault(&addhdrip));
  lcrzo_epr(shared_para_ipl_init_src(argv[2], &(addhdrip.saddr)));
  lcrzo_epr(shared_para_ipl_init_dst(argv[3], &(addhdrip.daddr)));
  addhdrip.protocol=LCRZO_HDRLIP_PROTOCOL_ICMP;
  lcrzo_epr(lcrzo_hdrlicmp_initdefault(&modhdricmp));
  lcrzo_epr(shared_para_uint8_init_icmptype(argv[4], &(modhdricmp.type)));
  lcrzo_epr(shared_para_uint8_init_icmpcode(argv[5], &(modhdricmp.code)));

  /* init spoof */
  lcrzo_epr(lcrzo_spoof_init(&spoof));
  lcrzo_epr(lcrzo_spoof_set_printbeforesending(&spoof, LCRZO_TRUE));

  /* initialize the sram */
  lcrzo_epr(shared_record_addmod_initdefault(&sram));
  sram.pspoof=&spoof;
  sram.pspoofdev=&device;
  sram.paddhdrip=&addhdrip;
  sram.pmodhdricmp=&modhdricmp;
  
  /* loop through the record */
  lcrzo_epr(lcrzo_record_open(&record, argv[1], LCRZO_RECORD_OPEN_READ));
  lcrzo_epr(lcrzo_record_loop_range(&record, start, end,
				    &shared_record_addmod_icmp_spoof, &sram));
  lcrzo_epr(lcrzo_record_close(&record));

  /* close spoof */
  lcrzo_epr(lcrzo_spoof_close(&spoof));

  lcrzo_close();
  return(LCRZO_ERR_OK);
}
