
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000250=
{ /*reference*/     "lcrzoex", 250,
  /*version*/       1, 1,
  /*title*/         "tcp ping (Ethernet and IP spoof)",
  /*french title*/  "ping tcp (spoof Ethernet et IP)",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "device srceth routereth srcip dstip dstport [maxsentpackets]",
  /*french usage*/  "device ethsrc ethrouteur ipsrc ipdst portdst [nbmaxidepaquetsenvoyes]",
  /*usage example*/ "eth0 aa:bb:cc:dd:ee:ff 11:22:33:44:55:66 1.2.3.4 5.6.7.78 80",
  /*fr. usage ex.*/ "eth0 aa:bb:cc:dd:ee:ff 11:22:33:44:55:66 1.2.3.4 5.6.7.78 80",
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000250_main(int argc, char *argv[]);
int lcrzoex_000250_tcpping(lcrzo_device device,
			   lcrzo_etha clientetha,
			   lcrzo_etha routeretha,
			   lcrzo_ipl clientipl, 
			   lcrzo_ipl serveripl, 
			   lcrzo_uint16 serverport,
			   lcrzo_uint8 maxpackets);
int lcrzoex_000250_child_spoof(lcrzo_ptr infos, 
			       lcrzo_int32 infossize, 
			       lcrzo_process *pparentprocess);
int lcrzoex_000250_child_sniff(lcrzo_ptr infos, 
			       lcrzo_int32 infossize, 
			       lcrzo_process *pparentprocess);
int lcrzoex_000250_printroutei(lcrzo_etha routeretha,
			       lcrzo_bool useeth,
			       lcrzo_ipl clientipl, 
			       lcrzo_ipl serveripl, 
			       lcrzo_uint16 serverport,
			       lcrzo_uint16 startingclientport,
			       lcrzo_uint32 startingclientseqnum,
			       lcrzo_data sniffdata,
			       lcrzo_int32 sniffdatasize);
int lcrzoex_000250_printroutet(lcrzo_etha routeretha,
			       lcrzo_bool useeth,
			       lcrzo_ipl clientipl, 
			       lcrzo_ipl serveripl, 
			       lcrzo_uint16 serverport,
			       lcrzo_uint16 startingclientport,
			       lcrzo_uint32 startingclientseqnum,
			       lcrzo_data sniffdata,
			       lcrzo_int32 sniffdatasize);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000250_main(int argc, char *argv[])
{ lcrzo_device device;
  lcrzo_etha clientetha, routeretha;
  lcrzo_ipl clientipl, serveripl;
  lcrzo_uint16 serverport;
  lcrzo_uint8 maxpackets;

  lcrzo_init();
  if ( argc<7 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000250, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }
  lcrzo_epr(shared_para_device_init_src(argv[1], device));
  lcrzo_epr(shared_para_etha_init_src(argv[2], clientetha));
  lcrzo_epr(shared_para_etha_init_dst(argv[3], routeretha));
  lcrzo_epr(shared_para_ipl_init_src(argv[4], &clientipl));
  lcrzo_epr(shared_para_ipl_init_dst(argv[5], &serveripl));
  lcrzo_epr(shared_para_port_init_dst(argv[6], 80, &serverport));
  if ( argc>7 )
  { lcrzo_epr(lcrzo_uint8_init_text(argv[7], &maxpackets));
  }
  else
    maxpackets=0;

  /* call the main function */
  lcrzo_epr(lcrzoex_000250_tcpping(device, clientetha, routeretha,
				   clientipl, serveripl, serverport, 
				   maxpackets));

  lcrzo_close();
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000250_tcpping(lcrzo_device device,
			   lcrzo_etha clientetha,
			   lcrzo_etha routeretha,
			   lcrzo_ipl clientipl, 
			   lcrzo_ipl serveripl, 
			   lcrzo_uint16 serverport,
			   lcrzo_uint8 maxpackets)
{ shared_sock_address ssa;
  lcrzo_process childspoof, childsniff;
  lcrzo_wait waitspoof, waitsniff;
  lcrzo_bool eventspoof, eventsniff;
  int ret;

  /* set values */
  lcrzo_epr(lcrzo_device_init(device, ssa.device));
  memcpy(ssa.clientetha, clientetha, LCRZO_ETHA_MAXBYTES);
  memcpy(ssa.serveretha, routeretha, LCRZO_ETHA_MAXBYTES);
  ssa.clientipl=clientipl;
  ssa.serveripl=serveripl;
  ssa.serverport=serverport;
  ssa.ttl=maxpackets;

  /* determine starting values */
  lcrzo_epr(lcrzo_uint16_init_rand(5000, 30000, &ssa.clientport2));
  lcrzo_epr(lcrzo_uint32_init_rand(0, 0xEFFFFFFFU, &ssa.seqnum));

  /* this process sends */
  lcrzo_epr(lcrzo_process_other_init(&lcrzoex_000250_child_spoof, 
                                     (lcrzo_data)&ssa, sizeof(ssa), 
				     &childspoof));

  /* this process sniff and prints sequence numbers */
  ret=lcrzo_process_other_init(&lcrzoex_000250_child_sniff, 
			       (lcrzo_data)&ssa, sizeof(ssa),
			       &childsniff);
  if ( ret!=LCRZO_ERR_OK)
  { /* kill the first child */
    lcrzo_epr(shared_lcrzoex_debug_processotherexit_wanttowarn(LCRZO_FALSE));
    lcrzo_epr(lcrzo_process_other_exit(&childspoof));
    lcrzo_epr(shared_lcrzoex_debug_processotherexit_wanttowarn(LCRZO_TRUE));
    lcrzo_epr(lcrzo_process_other_wait_end(&childspoof, LCRZO_WAIT_INFINITE, 
					   NULL, NULL));
    /* exit */
    return(ret);
  }

  /* wait for the end of one process */
  lcrzo_epr(lcrzo_wait_init_process_end(&childspoof, &waitspoof));
  lcrzo_epr(lcrzo_wait_init_process_end(&childsniff, &waitsniff));
  lcrzo_epr(lcrzo_wait_wait2(&waitspoof, &waitsniff, LCRZO_WAIT_INFINITE,
			     &eventspoof, &eventsniff));
  lcrzo_epr(lcrzo_wait_close(&waitspoof));
  lcrzo_epr(lcrzo_wait_close(&waitsniff));

  /* kill childs */
  if (eventspoof)
  { lcrzo_epr(lcrzo_time_sleep_sec(1)); /* to left time to the sniff process */
    lcrzo_epr(shared_lcrzoex_debug_processotherexit_wanttowarn(LCRZO_FALSE));
    lcrzo_epr(lcrzo_process_other_exit(&childsniff));
    lcrzo_epr(shared_lcrzoex_debug_processotherexit_wanttowarn(LCRZO_TRUE));
    lcrzo_epr(lcrzo_process_other_wait_end(&childsniff, LCRZO_WAIT_INFINITE, 
					   NULL, NULL));
    lcrzo_epr(lcrzo_process_other_wait_end(&childspoof, LCRZO_WAIT_INFINITE, 
					   NULL, &ret));
    lcrzo_epr(lcrzo_process_close(&childspoof));
    lcrzo_epr(lcrzo_process_close(&childsniff));
    return(ret);
  }
  if (eventsniff)
  { lcrzo_epr(shared_lcrzoex_debug_processotherexit_wanttowarn(LCRZO_FALSE));
    lcrzo_epr(lcrzo_process_other_exit(&childspoof));
    lcrzo_epr(shared_lcrzoex_debug_processotherexit_wanttowarn(LCRZO_TRUE));
    lcrzo_epr(lcrzo_process_other_wait_end(&childspoof, LCRZO_WAIT_INFINITE, 
					   NULL, NULL));
    lcrzo_epr(lcrzo_process_other_wait_end(&childsniff, LCRZO_WAIT_INFINITE, 
					   NULL, &ret));
    lcrzo_epr(lcrzo_process_close(&childspoof));
    lcrzo_epr(lcrzo_process_close(&childsniff));
    return(ret);
  }

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000250_child_spoof(lcrzo_ptr infos, 
			       lcrzo_int32 infossize, 
			       lcrzo_process *pparentprocess)
{ shared_sock_address *pssa=(shared_sock_address *)infos;
  lcrzo_hdrleth hdrleth;
  lcrzo_hdrlip hdrlip;
  lcrzo_hdrltcp hdrltcp;
  lcrzo_spoof spoof;
  lcrzo_bool useeth;
  lcrzo_uint16 numbersent;

  /* check for correct usage of this function */
  if (infossize!=sizeof(shared_sock_address))
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);

  /* close parent process handle because we do not need it */
  lcrzo_epr(lcrzo_process_close(pparentprocess));

  if ( pssa->clientetha[0]==0 && pssa->clientetha[1]==0 &&
       pssa->clientetha[2]==0 && pssa->clientetha[3]==0 &&
       pssa->clientetha[4]==0 && pssa->clientetha[5]==0 )
  { useeth=LCRZO_FALSE;
  }
  else
  { useeth=LCRZO_TRUE;
  }

  /* initialize Ethernet header */
  lcrzo_epr(lcrzo_hdrleth_initdefault(&hdrleth));
  if (useeth)
  { memcpy(hdrleth.src, pssa->clientetha, LCRZO_ETHA_MAXBYTES);
    memcpy(hdrleth.dst, pssa->serveretha, LCRZO_ETHA_MAXBYTES);
  }

  /* initialize IP header */
  lcrzo_epr(lcrzo_hdrlip_initdefault(&hdrlip));
  hdrlip.saddr=pssa->clientipl;
  hdrlip.daddr=pssa->serveripl;
  hdrlip.id=pssa->clientport2;

  /* initialize TCP header */
  lcrzo_epr(lcrzo_hdrltcp_initdefault(&hdrltcp));
  hdrltcp.dport=pssa->serverport;
  hdrltcp.syn=1;
  hdrltcp.sport=pssa->clientport2;
  hdrltcp.seqnum=pssa->seqnum;

 /* little sleep to be sure sniff process is ready (should be done with
     an IPC) */
  lcrzo_epr(lcrzo_time_sleep_msec(300));

   /* spoof */
  lcrzo_epr(lcrzo_spoof_init(&spoof));
  /*lcrzo_epr(lcrzo_spoof_set_printbeforesending(&lcrzoex_000250_spoof,
                                               LCRZO_TRUE));*/
  numbersent=0;
  while(LCRZO_TRUE)
  { if (useeth)
    { lcrzo_epr(lcrzo_spoof_ethipopttcpoptdata(&spoof,
					       pssa->device, hdrleth,
					       hdrlip, NULL, 0,
					       hdrltcp, NULL, 0,
					       NULL, 0));
    }
    else
    { lcrzo_epr(lcrzo_spoof_ipopttcpoptdata(&spoof,
					    hdrlip, NULL, 0,
					    hdrltcp, NULL, 0,
					    NULL, 0));
    }

    hdrlip.id++;
    hdrltcp.sport++;
    hdrltcp.seqnum+=10;
    numbersent++;
    if ( pssa->ttl && numbersent>=pssa->ttl ) break;
    lcrzo_epr(lcrzo_time_sleep_msec(300));
  }
  lcrzo_epr(lcrzo_spoof_close(&spoof));
  
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000250_child_sniff(lcrzo_ptr infos, 
			       lcrzo_int32 infossize, 
			       lcrzo_process *pparentprocess)
{ shared_sock_address *pssa=(shared_sock_address *)infos;
  lcrzo_sniff sniff;
  lcrzo_uint32 mtu;
  lcrzo_data sniffdata;
  lcrzo_int32 sniffdatasize;
  lcrzo_bool useeth;
  int ret;

  /* check for correct usage of this function */
  if (infossize!=sizeof(shared_sock_address))
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);

  /* close parent process handle because we do not need it */
  lcrzo_epr(lcrzo_process_close(pparentprocess));

  if ( pssa->clientetha[0]==0 && pssa->clientetha[1]==0 &&
       pssa->clientetha[2]==0 && pssa->clientetha[3]==0 &&
       pssa->clientetha[4]==0 && pssa->clientetha[5]==0 )
  { useeth=LCRZO_FALSE;
  }
  else
  { useeth=LCRZO_TRUE;
  }

  /* init sniff */
  lcrzo_epr(lcrzo_mtu_init_device(pssa->device, &mtu));
  lcrzo_epr(lcrzo_sniff_init(pssa->device, mtu, LCRZO_SNIFF_TYPE_NORMAL, "",
			     &sniff));

  /* main loop */
  ret=lcrzo_sniff_nextm(&sniff, LCRZO_WAIT_INFINITE, 
			&sniffdata, &sniffdatasize);
  while (ret==LCRZO_ERR_OK)
  { if (useeth)
    { /* answer to requests */
      lcrzo_epr(lcrzo_virtual_answer_alive(sniffdata, sniffdatasize, 
					   pssa->device, pssa->clientetha,
					   pssa->clientipl));
    }
    /* decode an icmp error, and print it */
    lcrzo_epr(lcrzoex_000250_printroutei(pssa->serveretha, useeth,
					 pssa->clientipl, pssa->serveripl, 
					 pssa->serverport,
					 pssa->clientport2, 
					 pssa->seqnum,
					 sniffdata, sniffdatasize));
    /* decode a tcp synack or reset, and print it */
    lcrzo_epr(lcrzoex_000250_printroutet(pssa->serveretha, useeth,
					 pssa->clientipl, pssa->serveripl, 
					 pssa->serverport,
					 pssa->clientport2, 
					 pssa->seqnum,
					 sniffdata, sniffdatasize));
    lcrzo_epr(lcrzo_data_free2(&sniffdata));
    /* sniff next packet */
    ret=lcrzo_sniff_nextm(&sniff, LCRZO_WAIT_INFINITE, 
			  &sniffdata, &sniffdatasize);
  }
  lcrzo_epr(lcrzo_sniff_close(&sniff));

  /*never reached*/
  return(LCRZO_ERR_IEINTERNALERROR);
}

/*-------------------------------------------------------------*/
int lcrzoex_000250_printroutei(lcrzo_etha routeretha,
			       lcrzo_bool useeth,
			       lcrzo_ipl clientipl, 
			       lcrzo_ipl serveripl, 
			       lcrzo_uint16 serverport,
			       lcrzo_uint16 startingclientport,
			       lcrzo_uint32 startingclientseqnum,
			       lcrzo_data sniffdata,
			       lcrzo_int32 sniffdatasize)
{ lcrzo_hdrleth hdrleth;
  lcrzo_hdrlip hdrlip, badhdrip;
  lcrzo_data baddata;
  lcrzo_string string;
  lcrzo_uint16 baddatasize, badtcpsport, badtcpdport;
  lcrzo_uint8 type, code;
  lcrzo_uint32 badtcpseqnum;
  lcrzo_bool ethequal;
  int ret, indexidok, indexidko, indexsport, indexseqnum;

  /*lcrzo_epr(lcrzo_packet_print_eth(sniffdata, sniffdatasize,
				   LCRZO_PRINTPROFILE_ARRAYDUMP));*/

  /* try to decode icmp */
  ret=lcrzo_packet_decodem_ethipopticmperror(sniffdata, sniffdatasize,
					     &hdrleth, &hdrlip, NULL,NULL,
					     &type, &code, NULL, NULL,
					     &baddata, &baddatasize);
  if ( ret!=LCRZO_ERR_OK )
    return(LCRZO_ERR_OK);

  /* check if the addresses are the one we are looking for */
  lcrzo_er(lcrzo_etha_equal2(hdrleth.src, routeretha, &ethequal));
  if ( (useeth&&(!ethequal)) || hdrlip.daddr!=clientipl )
  { lcrzo_epr(lcrzo_data_free2(&baddata));
    return(LCRZO_ERR_OK);
  }

  /* icmp should contain tcp */
  ret=lcrzo_ip64bits_decodem_iptcp(baddata, baddatasize,
				   &badhdrip, &badtcpsport, &badtcpdport,
				   &badtcpseqnum);
  lcrzo_epr(lcrzo_data_free2(&baddata));
  if ( ret!=LCRZO_ERR_OK )
    return(LCRZO_ERR_OK);

  /* check various values in baddata */
  if ( badtcpdport!=serverport ||
       badhdrip.saddr!=clientipl ||
       badhdrip.daddr!=serveripl )
    return(LCRZO_ERR_OK);

  /* compute indexid from the id of the received packet */
  indexidok=badhdrip.id-startingclientport;
  /* this one is because there is a bug in the IP stack of BSD :
     id is reversed :( */
  indexidko= ( (badhdrip.id&0xFF)<<8 ) | ( (badhdrip.id&0xFF00)>>8 );
  indexidko=indexidko-startingclientport;
  /* compute indexsport from the sport of the received packet */
  indexsport=badtcpsport-startingclientport;
  /* compute indexseqnum from the seqnum of the received packet */
  indexseqnum=(badtcpseqnum-startingclientseqnum)/10;

  /* verify that indexes are equal */
  if ( (indexidok!=indexsport&&indexidko!=indexsport) ||
       indexsport!=indexseqnum )
    return(LCRZO_ERR_OK);

  /* display the error type and code */
  printf("% 3d : ", indexsport+1);
  lcrzo_epr(lcrzo_ipl_print("", hdrlip.saddr, ""));
  lcrzo_epr(lcrzo_string_initm_icmptypecode(type, code, &string));
  printf(" : %s\n", string);
  lcrzo_epr(lcrzo_string_free2(&string));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000250_printroutet(lcrzo_etha routeretha,
			       lcrzo_bool useeth,
			       lcrzo_ipl clientipl, 
			       lcrzo_ipl serveripl, 
			       lcrzo_uint16 serverport,
			       lcrzo_uint16 startingclientport,
			       lcrzo_uint32 startingclientseqnum,
			       lcrzo_data sniffdata,
			       lcrzo_int32 sniffdatasize)
{ lcrzo_hdrleth hdrleth;
  lcrzo_hdrlip hdrlip;
  lcrzo_hdrltcp hdrltcp;
  lcrzo_int16 language;
  lcrzo_bool ethequal;
  int ret, theindex1, theindex2;

  /*lcrzo_epr(lcrzo_packet_print_eth(sniffdata, sniffdatasize,
    LCRZO_PRINTPROFILE_ARRAYDUMP));*/

  /* try to decode it */
  ret=lcrzo_packet_decodem_ethipopttcpoptdata(sniffdata, sniffdatasize,
					      &hdrleth, &hdrlip, NULL, NULL,
					      &hdrltcp, NULL, NULL,
					      NULL, NULL);
  if ( ret!=LCRZO_ERR_OK )
    return(LCRZO_ERR_OK);

  /* check if the addresses are the one we are looking for */
  lcrzo_er(lcrzo_etha_equal2(hdrleth.src, routeretha, &ethequal));
  if ( (useeth&&(!ethequal)) || 
       hdrlip.saddr!=serveripl || hdrlip.daddr!=clientipl )
    return(LCRZO_ERR_OK);

  /* check server port */
  if ( hdrltcp.sport!=serverport )
    return(LCRZO_ERR_OK);

  /* compute the theindex from the received packet */
  theindex1=hdrltcp.dport-startingclientport;
  theindex2=(hdrltcp.acknum-1-startingclientseqnum)/10;
  if ( theindex1!=theindex2 )
    return(LCRZO_ERR_OK);

  /* if we received synack or reset, destination is reached */
  if ( ( hdrltcp.syn && hdrltcp.ack ) || hdrltcp.rst )
  { printf("% 3d : ", theindex1+1);
    lcrzo_epr(lcrzo_ipl_print("", hdrlip.saddr, ""));
    if (hdrltcp.rst)
    { lcrzo_epr(lcrzo_global_get_language(&language));
      if ( language==LCRZO_GLOBAL_LANG_FR )
        printf(" (le port TCP %d est ferme)\n", serverport);
      else
        printf(" (the TCP port %d is closed)\n", serverport);
    }
    else
    { printf("\n");
    }
  }

  return(LCRZO_ERR_OK);
}
