
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000254=
{ /*reference*/     "lcrzoex", 254,
  /*version*/       1, 1,
  /*title*/         "scan of IP range, for UDP port range (Ethernet and IP spoof)",
  /*french title*/  "scan d'une plage IP, pour une plage de ports UDP (spoof Ethernet et IP)",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "device srceth srcip dstiprange dstportrange",
  /*french usage*/  "device ethsrc ipsrc plageipdst plageportdst",
  /*usage example*/ "eth0 aa:bb:cc:dd:ee:ff 1.2.3.4 5.6.7.0/24 1-120",
  /*fr. usage ex.*/ "eth0 aa:bb:cc:dd:ee:ff 1.2.3.4 5.6.7.0/24 1-120",
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000254_main(int argc, char *argv[]);
int lcrzoex_000254_scanipudp(lcrzo_device device,
			     lcrzo_etha clientetha,
			     lcrzo_ipl clientipl, 
			     lcrzo_iprange iprange, 
			     lcrzo_portrange portrange);
int lcrzoex_000254_child_spoof(lcrzo_ptr infos, 
			       lcrzo_int32 infossize, 
			       lcrzo_process *pparentprocess);
int lcrzoex_000254_child_sniff(lcrzo_ptr infos, 
			       lcrzo_int32 infossize, 
			       lcrzo_process *pparentprocess);
int lcrzoex_000254_printroutei(lcrzo_etha clientetha,
			       lcrzo_ipl clientipl,
			       lcrzo_ipl serveriplinf,
			       lcrzo_ipl serveriplsup,
			       lcrzo_uint16 serverportinf,
			       lcrzo_uint16 serverportsup,
			       lcrzo_uint16 startingclientport,
			       lcrzo_data sniffdata,
			       lcrzo_int32 sniffdatasize);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000254_main(int argc, char *argv[])
{ lcrzo_device device;
  lcrzo_etha clientetha;
  lcrzo_ipl clientipl;
  lcrzo_iprange iprange;
  lcrzo_portrange portrange;

  lcrzo_init();
  if ( argc<6 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000254, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }
  lcrzo_epr(shared_para_device_init_src(argv[1], device));
  lcrzo_epr(shared_para_etha_init_src(argv[2], clientetha));
  lcrzo_epr(shared_para_ipl_init_src(argv[3], &clientipl));
  lcrzo_epr(lcrzo_string_init_text(argv[4], LCRZO_IPRANGE_MAXBYTES, iprange));
  lcrzo_epr(lcrzo_string_init_text(argv[5], LCRZO_PORTRANGE_MAXBYTES,
				   portrange));

  /* call the main function */
  lcrzo_epr(lcrzoex_000254_scanipudp(device, clientetha, clientipl,
				     iprange, portrange));

  lcrzo_close();
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000254_scanipudp(lcrzo_device device,
			     lcrzo_etha clientetha,
			     lcrzo_ipl clientipl, 
			     lcrzo_iprange iprange, 
			     lcrzo_portrange portrange)
{ shared_sock_address ssa;
  lcrzo_process childspoof, childsniff;
  lcrzo_wait waitspoof, waitsniff;
  lcrzo_bool eventspoof, eventsniff;
  int ret;

  /* set values */
  lcrzo_epr(lcrzo_device_init(device, ssa.device));
  memcpy(ssa.clientetha, clientetha, LCRZO_ETHA_MAXBYTES);
  ssa.clientipl=clientipl;

  /* determine starting values */
  lcrzo_epr(lcrzo_uint16_init_rand(5000, 30000, &ssa.clientport2));

  /* obtain starting and ending values */
  lcrzo_epr(lcrzo_ipl_init_iprange(iprange, &ssa.serveripl, &ssa.serveripl2));
  lcrzo_epr(lcrzo_port_init_portrange(portrange, &ssa.serverport, 
				      &ssa.serverport2));

  /* this process sends */
  lcrzo_epr(lcrzo_process_other_init(&lcrzoex_000254_child_spoof, 
                                     (lcrzo_data)&ssa, sizeof(ssa), 
				     &childspoof));

  /* this process sniff and prints sequence numbers */
  ret=lcrzo_process_other_init(&lcrzoex_000254_child_sniff, 
			       (lcrzo_data)&ssa, sizeof(ssa),
			       &childsniff);
  if ( ret!=LCRZO_ERR_OK)
  { /* kill the first child */
    lcrzo_epr(shared_lcrzoex_debug_processotherexit_wanttowarn(LCRZO_FALSE));
    lcrzo_epr(lcrzo_process_other_exit(&childspoof));
    lcrzo_epr(shared_lcrzoex_debug_processotherexit_wanttowarn(LCRZO_TRUE));
    lcrzo_epr(lcrzo_process_other_wait_end(&childspoof, LCRZO_WAIT_INFINITE, 
					   NULL, NULL));
    /* exit */
    return(ret);
  }

  /* wait for the end of one process */
  lcrzo_epr(lcrzo_wait_init_process_end(&childspoof, &waitspoof));
  lcrzo_epr(lcrzo_wait_init_process_end(&childsniff, &waitsniff));
  lcrzo_epr(lcrzo_wait_wait2(&waitspoof, &waitsniff, LCRZO_WAIT_INFINITE,
			     &eventspoof, &eventsniff));
  lcrzo_epr(lcrzo_wait_close(&waitspoof));
  lcrzo_epr(lcrzo_wait_close(&waitsniff));

  /* kill childs */
  if (eventspoof)
  { lcrzo_epr(lcrzo_time_sleep_sec(1)); /* to left time to the sniff process */
    lcrzo_epr(shared_lcrzoex_debug_processotherexit_wanttowarn(LCRZO_FALSE));
    lcrzo_epr(lcrzo_process_other_exit(&childsniff));
    lcrzo_epr(shared_lcrzoex_debug_processotherexit_wanttowarn(LCRZO_TRUE));
    lcrzo_epr(lcrzo_process_other_wait_end(&childsniff, LCRZO_WAIT_INFINITE, 
					   NULL, NULL));
    lcrzo_epr(lcrzo_process_other_wait_end(&childspoof, LCRZO_WAIT_INFINITE, 
					   NULL, &ret));
    lcrzo_epr(lcrzo_process_close(&childspoof));
    lcrzo_epr(lcrzo_process_close(&childsniff));
    return(ret);
  }
  if (eventsniff)
  { lcrzo_epr(shared_lcrzoex_debug_processotherexit_wanttowarn(LCRZO_FALSE));
    lcrzo_epr(lcrzo_process_other_exit(&childspoof));
    lcrzo_epr(shared_lcrzoex_debug_processotherexit_wanttowarn(LCRZO_TRUE));
    lcrzo_epr(lcrzo_process_other_wait_end(&childspoof, LCRZO_WAIT_INFINITE, 
					   NULL, NULL));
    lcrzo_epr(lcrzo_process_other_wait_end(&childsniff, LCRZO_WAIT_INFINITE, 
					   NULL, &ret));
    lcrzo_epr(lcrzo_process_close(&childspoof));
    lcrzo_epr(lcrzo_process_close(&childsniff));
    return(ret);
  }

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000254_child_spoof(lcrzo_ptr infos, 
			       lcrzo_int32 infossize, 
			       lcrzo_process *pparentprocess)
{ shared_sock_address *pssa=(shared_sock_address *)infos;
  lcrzo_hdrleth hdrleth;
  lcrzo_hdrlip hdrlip;
  lcrzo_hdrludp hdrludp;
  lcrzo_spoof spoof;
  lcrzo_ipl serveripl;
  lcrzo_uint16 serverport;
  lcrzo_device deviceneeded;
  lcrzo_etha routeretha;
  lcrzo_int16 language;
  lcrzo_bool equal;
  int ret;

  /* check for correct usage of this function */
  if (infossize!=sizeof(shared_sock_address))
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);

  /* close parent process handle because we do not need it */
  lcrzo_epr(lcrzo_process_close(pparentprocess));

  /* little sleep to be sure sniff process is ready (should be done with
     an IPC) */
  lcrzo_epr(lcrzo_time_sleep_msec(300));

  /* init spoof */
  lcrzo_epr(lcrzo_spoof_init(&spoof));
  /*lcrzo_epr(lcrzo_spoof_set_printbeforesending(&lcrzoex_000254_spoof,
                                               LCRZO_TRUE));*/

  /* loop through each server */
  for ( serveripl=pssa->serveripl ; serveripl<=pssa->serveripl2 ; serveripl++ )
  { 
    /* obtain routeretha */
    ret=lcrzo_conf_route_to_host(serveripl, deviceneeded, NULL, routeretha,
				 NULL, NULL);
    if (ret!=LCRZO_ERR_OK)
    { lcrzo_epr(lcrzo_ipl_print("", serveripl, " : "));
      lcrzo_epr(lcrzo_err_print(ret));
      continue;
    }
    /* display a warning */
    lcrzo_er(lcrzo_device_equal2(pssa->device, deviceneeded, &equal));
    if ( ! equal )
    { lcrzo_epr(lcrzo_global_get_language(&language));
      if ( language==LCRZO_GLOBAL_LANG_FR )
      { printf("Le device employe (%s) est different du device", pssa->device);
        printf(" necessaire (%s) pour joindre ", deviceneeded);
      }
      else
      { printf("The used device (%s) is different from the necessary", 
	       pssa->device);
        printf(" device (%s) to reach ", deviceneeded);
      }
      lcrzo_epr(lcrzo_ipl_print("", serveripl, "\n"));
    }

    /* initialize Ethernet header */
    lcrzo_epr(lcrzo_hdrleth_initdefault(&hdrleth));
    memcpy(hdrleth.src, pssa->clientetha, LCRZO_ETHA_MAXBYTES);
    memcpy(hdrleth.dst, routeretha, LCRZO_ETHA_MAXBYTES);

    /* initialize IP header */
    lcrzo_epr(lcrzo_hdrlip_initdefault(&hdrlip));
    hdrlip.saddr=pssa->clientipl;
    hdrlip.daddr=serveripl;
    hdrlip.id=pssa->clientport2;

    /* initialize UDP header */
    lcrzo_epr(lcrzo_hdrludp_initdefault(&hdrludp));
    hdrludp.dport=pssa->serverport;
    hdrludp.sport=pssa->clientport2;

    /* spoof */
    for ( serverport=pssa->serverport ; serverport<=pssa->serverport2 ; 
	  serverport++ )
    { lcrzo_epr(lcrzo_spoof_ethipoptudpdata(&spoof,
					    pssa->device, hdrleth,
					    hdrlip, NULL, 0,
					    hdrludp, NULL, 0));
      hdrlip.id++;
      hdrludp.sport++;
      hdrludp.dport++;
      lcrzo_epr(lcrzo_time_sleep_sec(1));
    }
  }

  /* close */
  lcrzo_epr(lcrzo_spoof_close(&spoof));  
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000254_child_sniff(lcrzo_ptr infos, 
			       lcrzo_int32 infossize, 
			       lcrzo_process *pparentprocess)
{ shared_sock_address *pssa=(shared_sock_address *)infos;
  lcrzo_sniff sniff;
  lcrzo_uint32 mtu;
  lcrzo_data sniffdata;
  lcrzo_int32 sniffdatasize;
  int ret;

  /* check for correct usage of this function */
  if (infossize!=sizeof(shared_sock_address))
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);

  /* close parent process handle because we do not need it */
  lcrzo_epr(lcrzo_process_close(pparentprocess));

  /* init sniff */
  lcrzo_epr(lcrzo_mtu_init_device(pssa->device, &mtu));
  lcrzo_epr(lcrzo_sniff_init(pssa->device, mtu, LCRZO_SNIFF_TYPE_NORMAL, "",
			     &sniff));

  /* main loop */
  ret=lcrzo_sniff_nextm(&sniff, LCRZO_WAIT_INFINITE, 
			&sniffdata, &sniffdatasize);
  while (ret==LCRZO_ERR_OK)
  { /* answer to requests */
    lcrzo_epr(lcrzo_virtual_answer_alive(sniffdata, sniffdatasize, 
                                         pssa->device, pssa->clientetha,
					 pssa->clientipl));
    /* decode an icmp error, and print it */
    lcrzo_epr(lcrzoex_000254_printroutei(pssa->clientetha, pssa->clientipl,
					 pssa->serveripl, pssa->serveripl2,
					 pssa->serverport, pssa->serverport2,
					 pssa->clientport2,
					 sniffdata, sniffdatasize));
    lcrzo_epr(lcrzo_data_free2(&sniffdata));
    /* sniff next packet */
    ret=lcrzo_sniff_nextm(&sniff, LCRZO_WAIT_INFINITE, 
			  &sniffdata, &sniffdatasize);
  }
  lcrzo_epr(lcrzo_sniff_close(&sniff));

  return(ret);
}

/*-------------------------------------------------------------*/
int lcrzoex_000254_printroutei(lcrzo_etha clientetha,
			       lcrzo_ipl clientipl,
			       lcrzo_ipl serveriplinf,
			       lcrzo_ipl serveriplsup,
			       lcrzo_uint16 serverportinf,
			       lcrzo_uint16 serverportsup,
			       lcrzo_uint16 startingclientport,
			       lcrzo_data sniffdata,
			       lcrzo_int32 sniffdatasize)
{ lcrzo_hdrleth hdrleth;
  lcrzo_hdrlip hdrlip, badhdrip;
  lcrzo_data baddata;
  lcrzo_string string;
  lcrzo_uint16 baddatasize, badudpsport, badudpdport;
  lcrzo_uint8 type, code;
  lcrzo_int16 language;
  lcrzo_bool ethequal;
  int ret, indexidok, indexidko, indexsport;

  /*lcrzo_epr(lcrzo_packet_print_eth(sniffdata, sniffdatasize,
				   LCRZO_PRINTPROFILE_ARRAYDUMP));*/

  /* try to decode icmp */
  ret=lcrzo_packet_decodem_ethipopticmperror(sniffdata, sniffdatasize,
					     &hdrleth, &hdrlip, NULL,NULL,
					     &type, &code, NULL, NULL,
					     &baddata, &baddatasize);
  if ( ret!=LCRZO_ERR_OK )
    return(LCRZO_ERR_OK);

  /* check if the addresses are the one we are looking for */
  lcrzo_er(lcrzo_etha_equal2(hdrleth.dst, clientetha, &ethequal));
  if ( ! ethequal || hdrlip.daddr!=clientipl )
  { lcrzo_epr(lcrzo_data_free2(&baddata));
    return(LCRZO_ERR_OK);
  }

  /* icmp should contain udp */
  ret=lcrzo_ip64bits_decodem_ipudp(baddata, baddatasize,
                                   &badhdrip, &badudpsport, &badudpdport,
                                   NULL, NULL);
  lcrzo_epr(lcrzo_data_free2(&baddata));
  if ( ret!=LCRZO_ERR_OK )
    return(LCRZO_ERR_OK);

  /* check various values in baddata */
  if ( badudpdport<serverportinf ||
       badudpdport>serverportsup ||
       badhdrip.saddr!=clientipl ||
       badhdrip.daddr<serveriplinf ||
       badhdrip.daddr>serveriplsup )
    return(LCRZO_ERR_OK);

  /* compute indexid from the id of the received packet */
  indexidok=badhdrip.id-startingclientport+1;
  /* this one is because there is a bug in the IP stack of BSD :
     id is reversed :( */
  indexidko= ( (badhdrip.id&0xFF)<<8 ) | ( (badhdrip.id&0xFF00)>>8 );
  indexidko=indexidko-startingclientport+1;
  /* compute indexsport from the sport of the received packet */
  indexsport=badudpsport-startingclientport+1;

  /* verify that indexes are equal */
  if ( (indexidok!=indexsport&&indexidko!=indexsport) )
    return(LCRZO_ERR_OK);

  /* display the error type and code */
  lcrzo_epr(lcrzo_global_get_language(&language));
  lcrzo_epr(lcrzo_ipl_print("", badhdrip.daddr, ""));
  printf(":%d", badudpdport);
  if ( type==LCRZO_ICMPTYPE_DESTUNREACH &&
       code == LCRZO_ICMPCODE_DESTUNREACH_PORT )
  { if ( language==LCRZO_GLOBAL_LANG_FR )
      printf(" - ferme\n");
    else
      printf(" - closed\n");
  }
  else
  { if ( language==LCRZO_GLOBAL_LANG_FR )
      printf(" - error : ");
    else
      printf(" - erreur : ");
    lcrzo_epr(lcrzo_ipl_print("", hdrlip.saddr, ""));
    if ( language==LCRZO_GLOBAL_LANG_FR )
      printf(" a dit : ");
    else
      printf(" said : ");
    lcrzo_epr(lcrzo_string_initm_icmptypecode(type, code, &string));
    printf(" : %s\n", string);
    lcrzo_epr(lcrzo_string_free2(&string));
  }
  return(LCRZO_ERR_OK);
}

