
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000255=
{ /*reference*/     "lcrzoex", 255,
  /*version*/       1, 0,
  /*title*/         "scan of IP range, for UDP port range (true IP, sent at Ethernet level)",
  /*french title*/  "scan d'une plage IP, pour une plage de ports UDP (vrai IP, envoye au niveau Ethernet)",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "dstiprange dstportrange",
  /*french usage*/  "plageipdst plageportdst",
  /*usage example*/ "5.6.7.0/24 1-120",
  /*fr. usage ex.*/ "5.6.7.0/24 1-120",
  /*author*/        "Laurent"
};


/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000255_main(int argc, char *argv[]);
int lcrzoex_000254_scanipudp(lcrzo_device device,
			     lcrzo_etha clientetha,
			     lcrzo_ipl clientipl, 
			     lcrzo_iprange iprange, 
			     lcrzo_portrange portrange);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000255_main(int argc, char *argv[])
{ lcrzo_device device;
  lcrzo_etha clientetha;
  lcrzo_ipl clientipl, serveriplinf, serveriplsup, serverip;
  lcrzo_iprange iprange;
  lcrzo_portrange portrange;
  int ret;

  lcrzo_init();
  if ( argc<3 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000255, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }
  lcrzo_epr(lcrzo_string_init_text(argv[1], LCRZO_IPRANGE_MAXBYTES, iprange));
  lcrzo_epr(lcrzo_string_init_text(argv[2], LCRZO_PORTRANGE_MAXBYTES,
				   portrange));

  /* get necessary addresses */
  lcrzo_epr(lcrzo_ipl_init_iprange(iprange, &serveriplinf, &serveriplsup));
  ret=LCRZOEX_ERR_MISC_BADTOOLUSAGE; clientipl=0; /* for the compiler warning */
  for ( serverip=serveriplinf ; serverip<=serveriplsup ; serverip++ )
  { ret=lcrzo_conf_route_to_host(serverip, device, clientetha, NULL,
				 &clientipl, NULL);
    if (ret==LCRZO_ERR_OK) break;
    lcrzo_epr(lcrzo_ipl_print("", serverip, " : "));
    lcrzo_epr(lcrzo_err_print(ret));
  }

  /* call the main function */
  if (ret==LCRZO_ERR_OK)
  { lcrzo_epr(lcrzoex_000254_scanipudp(device, clientetha, clientipl,
				     iprange, portrange));
  }

  lcrzo_close();
  return (LCRZO_ERR_OK);
}

