
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000256=
{ /*reference*/     "lcrzoex", 256,
  /*version*/       1, 1,
  /*title*/         "scan of IP range, for ICMP ping (Ethernet and IP spoof)",
  /*french title*/  "scan d'une plage IP, pour des ping ICMP (spoof Ethernet et IP)",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "device srceth srcip dstiprange",
  /*french usage*/  "device ethsrc ipsrc plageipdst",
  /*usage example*/ "eth0 aa:bb:cc:dd:ee:ff 1.2.3.4 5.6.7.0/24",
  /*fr. usage ex.*/ "eth0 aa:bb:cc:dd:ee:ff 1.2.3.4 5.6.7.0/24",
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000256_main(int argc, char *argv[]);
int lcrzoex_000256_scanipicmp(lcrzo_device device,
			     lcrzo_etha clientetha,
			     lcrzo_ipl clientipl, 
			     lcrzo_iprange iprange);
int lcrzoex_000256_child_spoof(lcrzo_ptr infos, 
			       lcrzo_int32 infossize, 
			       lcrzo_process *pparentprocess);
int lcrzoex_000256_child_sniff(lcrzo_ptr infos, 
			       lcrzo_int32 infossize, 
			       lcrzo_process *pparentprocess);
int lcrzoex_000256_printroutei(lcrzo_etha clientetha,
			       lcrzo_ipl clientipl,
			       lcrzo_ipl serveriplinf,
			       lcrzo_ipl serveriplsup,
			       lcrzo_uint16 startingid,
			       lcrzo_data sniffdata,
			       lcrzo_int32 sniffdatasize);
int lcrzoex_000256_printroutet(lcrzo_etha clientetha,
			       lcrzo_ipl clientipl,
			       lcrzo_ipl serveriplinf,
			       lcrzo_ipl serveriplsup,
			       lcrzo_uint16 startingid,
			       lcrzo_data sniffdata,
			       lcrzo_int32 sniffdatasize);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000256_main(int argc, char *argv[])
{ lcrzo_device device;
  lcrzo_etha clientetha;
  lcrzo_ipl clientipl;
  lcrzo_iprange iprange;

  lcrzo_init();
  if ( argc<5 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000256, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }
  lcrzo_epr(shared_para_device_init_src(argv[1], device));
  lcrzo_epr(shared_para_etha_init_src(argv[2], clientetha));
  lcrzo_epr(shared_para_ipl_init_src(argv[3], &clientipl));
  lcrzo_epr(lcrzo_string_init_text(argv[4], LCRZO_IPRANGE_MAXBYTES, iprange));

  /* call the main function */
  lcrzo_epr(lcrzoex_000256_scanipicmp(device, clientetha, clientipl, iprange));

  lcrzo_close();
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000256_scanipicmp(lcrzo_device device,
			      lcrzo_etha clientetha,
			      lcrzo_ipl clientipl, 
			      lcrzo_iprange iprange)
{ shared_sock_address ssa;
  lcrzo_process childspoof, childsniff;
  lcrzo_wait waitspoof, waitsniff;
  lcrzo_bool eventspoof, eventsniff;
  int ret;

  /* set values */
  lcrzo_epr(lcrzo_device_init(device, ssa.device));
  memcpy(ssa.clientetha, clientetha, LCRZO_ETHA_MAXBYTES);
  ssa.clientipl=clientipl;

  /* determine starting values */
  lcrzo_epr(lcrzo_uint16_init_rand(5000, 30000, &ssa.clientport2));

  /* obtain starting and ending values */
  lcrzo_epr(lcrzo_ipl_init_iprange(iprange, &ssa.serveripl, &ssa.serveripl2));

  /* this process sends */
  lcrzo_epr(lcrzo_process_other_init(&lcrzoex_000256_child_spoof, 
                                     (lcrzo_data)&ssa, sizeof(ssa), 
				     &childspoof));

  /* this process sniff and prints sequence numbers */
  ret=lcrzo_process_other_init(&lcrzoex_000256_child_sniff, 
			       (lcrzo_data)&ssa, sizeof(ssa),
			       &childsniff);
  if ( ret!=LCRZO_ERR_OK)
  { /* kill the first child */
    lcrzo_epr(shared_lcrzoex_debug_processotherexit_wanttowarn(LCRZO_FALSE));
    lcrzo_epr(lcrzo_process_other_exit(&childspoof));
    lcrzo_epr(shared_lcrzoex_debug_processotherexit_wanttowarn(LCRZO_TRUE));
    lcrzo_epr(lcrzo_process_other_wait_end(&childspoof, LCRZO_WAIT_INFINITE, 
					   NULL, NULL));
    /* exit */
    return(ret);
  }

  /* wait for the end of one process */
  lcrzo_epr(lcrzo_wait_init_process_end(&childspoof, &waitspoof));
  lcrzo_epr(lcrzo_wait_init_process_end(&childsniff, &waitsniff));
  lcrzo_epr(lcrzo_wait_wait2(&waitspoof, &waitsniff, LCRZO_WAIT_INFINITE,
			     &eventspoof, &eventsniff));
  lcrzo_epr(lcrzo_wait_close(&waitspoof));
  lcrzo_epr(lcrzo_wait_close(&waitsniff));

  /* kill childs */
  if (eventspoof)
  { lcrzo_epr(lcrzo_time_sleep_sec(1)); /* to left time to the sniff process */
    lcrzo_epr(shared_lcrzoex_debug_processotherexit_wanttowarn(LCRZO_FALSE));
    lcrzo_epr(lcrzo_process_other_exit(&childsniff));
    lcrzo_epr(shared_lcrzoex_debug_processotherexit_wanttowarn(LCRZO_TRUE));
    lcrzo_epr(lcrzo_process_other_wait_end(&childsniff, LCRZO_WAIT_INFINITE, 
					   NULL, NULL));
    lcrzo_epr(lcrzo_process_other_wait_end(&childspoof, LCRZO_WAIT_INFINITE, 
					   NULL, &ret));
    lcrzo_epr(lcrzo_process_close(&childspoof));
    lcrzo_epr(lcrzo_process_close(&childsniff));
    return(ret);
  }
  if (eventsniff)
  { lcrzo_epr(shared_lcrzoex_debug_processotherexit_wanttowarn(LCRZO_FALSE));
    lcrzo_epr(lcrzo_process_other_exit(&childspoof));
    lcrzo_epr(shared_lcrzoex_debug_processotherexit_wanttowarn(LCRZO_TRUE));
    lcrzo_epr(lcrzo_process_other_wait_end(&childspoof, LCRZO_WAIT_INFINITE, 
					   NULL, NULL));
    lcrzo_epr(lcrzo_process_other_wait_end(&childsniff, LCRZO_WAIT_INFINITE, 
					   NULL, &ret));
    lcrzo_epr(lcrzo_process_close(&childspoof));
    lcrzo_epr(lcrzo_process_close(&childsniff));
    return(ret);
  }

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000256_child_spoof(lcrzo_ptr infos, 
			       lcrzo_int32 infossize, 
			       lcrzo_process *pparentprocess)
{ shared_sock_address *pssa=(shared_sock_address *)infos;
  lcrzo_hdrleth hdrleth;
  lcrzo_hdrlip hdrlip;
  lcrzo_spoof spoof;
  lcrzo_ipl serveripl;
  lcrzo_device deviceneeded;
  lcrzo_etha routeretha;
  lcrzo_data data;
  lcrzo_int32 datasize;
  lcrzo_int16 language;
  lcrzo_bool equal;
  int ret;

  /* check for correct usage of this function */
  if (infossize!=sizeof(shared_sock_address))
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);

  /* close parent process handle because we do not need it */
  lcrzo_epr(lcrzo_process_close(pparentprocess));

  /* little sleep to be sure sniff process is ready (should be done with
     an IPC) */
  lcrzo_epr(lcrzo_time_sleep_msec(300));

  /* init spoof */
  lcrzo_epr(lcrzo_spoof_init(&spoof));
  /*lcrzo_epr(lcrzo_spoof_set_printbeforesending(&spoof,
                                               LCRZO_TRUE));*/

  /* loop through each server */
  for ( serveripl=pssa->serveripl ; serveripl<=pssa->serveripl2 ; serveripl++ )
  { 
    /* obtain routeretha */
    ret=lcrzo_conf_route_to_host(serveripl, deviceneeded, NULL, routeretha,
				 NULL, NULL);
    if (ret!=LCRZO_ERR_OK)
    { lcrzo_epr(lcrzo_ipl_print("", serveripl, " : "));
      lcrzo_epr(lcrzo_err_print(ret));
      continue;
    }
    /* display a warning */
    lcrzo_er(lcrzo_device_equal2(pssa->device, deviceneeded, &equal));
    if ( ! equal )
    { lcrzo_epr(lcrzo_global_get_language(&language));
      if ( language==LCRZO_GLOBAL_LANG_FR )
      { printf("Le device employe (%s) est different du device", pssa->device);
        printf(" necessaire (%s) pour joindre ", deviceneeded);
      }
      else
      { printf("The used device (%s) is different from the necessary", 
	       pssa->device);
        printf(" device (%s) to reach ", deviceneeded);
      }
      lcrzo_epr(lcrzo_ipl_print("", serveripl, "\n"));
    }

    /* initialize Ethernet header */
    lcrzo_epr(lcrzo_hdrleth_initdefault(&hdrleth));
    memcpy(hdrleth.src, pssa->clientetha, LCRZO_ETHA_MAXBYTES);
    memcpy(hdrleth.dst, routeretha, LCRZO_ETHA_MAXBYTES);

    /* initialize IP header */
    lcrzo_epr(lcrzo_hdrlip_initdefault(&hdrlip));
    hdrlip.saddr=pssa->clientipl;
    hdrlip.daddr=serveripl;
    hdrlip.id=pssa->clientport2;

    /* spoof */
    lcrzo_epr(lcrzo_packet_initm_ethipopticmpecho(hdrleth,
                                                  hdrlip, NULL, 0,
                                                  LCRZO_ICMPTYPE_ECHOREQ,
                                                  hdrlip.id, hdrlip.id,
                                                  (lcrzo_constdata)"ICMP scan", 9,
                                                  &data, &datasize));
    lcrzo_epr(lcrzo_spoof_eth(&spoof, pssa->device, data, datasize));
    lcrzo_epr(lcrzo_data_free2(&data));
    hdrlip.id++;
    lcrzo_epr(lcrzo_time_sleep_usec(10000));
  }

  /* close */
  lcrzo_epr(lcrzo_spoof_close(&spoof));  
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000256_child_sniff(lcrzo_ptr infos, 
			       lcrzo_int32 infossize, 
			       lcrzo_process *pparentprocess)
{ shared_sock_address *pssa=(shared_sock_address *)infos;
  lcrzo_sniff sniff;
  lcrzo_uint32 mtu;
  lcrzo_data sniffdata;
  lcrzo_int32 sniffdatasize;
  int ret;

  /* check for correct usage of this function */
  if (infossize!=sizeof(shared_sock_address))
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);

  /* close parent process handle because we do not need it */
  lcrzo_epr(lcrzo_process_close(pparentprocess));

  /* init sniff */
  lcrzo_epr(lcrzo_mtu_init_device(pssa->device, &mtu));
  lcrzo_epr(lcrzo_sniff_init(pssa->device, mtu, LCRZO_SNIFF_TYPE_NORMAL, "",
			     &sniff));

  /* main loop */
  ret=lcrzo_sniff_nextm(&sniff, LCRZO_WAIT_INFINITE, 
			&sniffdata, &sniffdatasize);
  while (ret==LCRZO_ERR_OK)
  { /* answer to requests */
    lcrzo_epr(lcrzo_virtual_answer_alive(sniffdata, sniffdatasize, 
                                         pssa->device, pssa->clientetha, 
					 pssa->clientipl));
    /* decode an icmp error, and print it */
    lcrzo_epr(lcrzoex_000256_printroutei(pssa->clientetha, pssa->clientipl,
					 pssa->serveripl, pssa->serveripl2,
					 pssa->clientport2,
					 sniffdata, sniffdatasize));
    /* decode a icmp synack or reset, and print it */
    lcrzo_epr(lcrzoex_000256_printroutet(pssa->clientetha, pssa->clientipl,
					 pssa->serveripl, pssa->serveripl2,
					 pssa->clientport2,
					 sniffdata, sniffdatasize));
    lcrzo_epr(lcrzo_data_free2(&sniffdata));
    /* sniff next packet */
    ret=lcrzo_sniff_nextm(&sniff, LCRZO_WAIT_INFINITE, 
			  &sniffdata, &sniffdatasize);
  }
  lcrzo_epr(lcrzo_sniff_close(&sniff));

  return(ret);
}

/*-------------------------------------------------------------*/
int lcrzoex_000256_printroutei(lcrzo_etha clientetha,
			       lcrzo_ipl clientipl,
			       lcrzo_ipl serveriplinf,
			       lcrzo_ipl serveriplsup,
			       lcrzo_uint16 startingid,
			       lcrzo_data sniffdata,
			       lcrzo_int32 sniffdatasize)
{ lcrzo_hdrleth hdrleth;
  lcrzo_hdrlip hdrlip, badhdrip;
  lcrzo_data baddata;
  lcrzo_string string;
  lcrzo_uint16 baddatasize;
  lcrzo_uint8 type, code, badicmptype, badicmpcode, b4, b5, b6, b7;
  lcrzo_bool ethequal;
  int ret, indexidok, indexidko, indexechoid, indexechoseqnum;

  /*lcrzo_epr(lcrzo_packet_print_eth(sniffdata, sniffdatasize,
				   LCRZO_PRINTPROFILE_ARRAYDUMP));*/

  /* try to decode icmp */
  ret=lcrzo_packet_decodem_ethipopticmperror(sniffdata, sniffdatasize,
					     &hdrleth, &hdrlip, NULL,NULL,
					     &type, &code, NULL, NULL,
					     &baddata, &baddatasize);
  if ( ret!=LCRZO_ERR_OK )
    return(LCRZO_ERR_OK);

  /* check if the addresses are the one we are looking for */
  lcrzo_er(lcrzo_etha_equal2(hdrleth.dst, clientetha, &ethequal));
  if ( ! ethequal || hdrlip.daddr!=clientipl )
  { lcrzo_epr(lcrzo_data_free2(&baddata));
    return(LCRZO_ERR_OK);
  }

  /* icmp should contain icmp */
  ret=lcrzo_ip64bits_decodem_ipicmp(baddata, baddatasize,
                                    &badhdrip, &badicmptype, &badicmpcode,
                                    NULL, &b4, &b5, &b6, &b7);
  lcrzo_epr(lcrzo_data_free2(&baddata));
  if ( ret!=LCRZO_ERR_OK )
    return(LCRZO_ERR_OK);

  /* check various values in baddata */
  if ( badicmptype!=LCRZO_ICMPTYPE_ECHOREQ ||
       badicmpcode!=0 ||
        badhdrip.saddr!=clientipl ||
       badhdrip.daddr<serveriplinf ||
       badhdrip.daddr>serveriplsup )
    return(LCRZO_ERR_OK);

  /* compute indexid from the id of the received packet */
  indexidok=badhdrip.id-startingid+1;
  /* this one is because there is a bug in the IP stack of BSD :
     id is reversed :( */
  indexidko= ( (badhdrip.id&0xFF)<<8 ) | ( (badhdrip.id&0xFF00)>>8 );
  indexidko=indexidko-startingid+1;
  /* compute indexechoid from the id of the received packet */
  indexechoid=((b4<<8)|b5)-startingid+1;
  /* compute indexechoseqnum from the seqnum of the received packet */
  indexechoseqnum=((b6<<8)|b7)-startingid+1;

  /* verify that indexes are equal */
  if ( (indexidok!=indexechoid&&indexidko!=indexechoid) ||
       indexechoid!=indexechoseqnum )
    return(LCRZO_ERR_OK);

  /* display the error type and code */
  lcrzo_epr(lcrzo_ipl_print("", badhdrip.daddr, " - error : "));
  lcrzo_epr(lcrzo_ipl_print("", hdrlip.saddr, " said "));
  lcrzo_epr(lcrzo_string_initm_icmptypecode(type, code, &string));
  printf(" : %s\n", string);
  lcrzo_epr(lcrzo_string_free2(&string));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000256_printroutet(lcrzo_etha clientetha,
			       lcrzo_ipl clientipl,
			       lcrzo_ipl serveriplinf,
			       lcrzo_ipl serveriplsup,
			       lcrzo_uint16 startingid,
			       lcrzo_data sniffdata,
			       lcrzo_int32 sniffdatasize)
{ lcrzo_hdrleth hdrleth;
  lcrzo_hdrlip hdrlip;
  lcrzo_uint8 type;
  lcrzo_uint16 id, seqnum;
  lcrzo_int16 language;
  lcrzo_bool ethequal;
  int ret, theindex1, theindex2;

  /*lcrzo_epr(lcrzo_packet_print_eth(sniffdata, sniffdatasize,
				   LCRZO_PRINTPROFILE_ARRAYDUMP));*/

  /* try to decode it */
  ret=lcrzo_packet_decodem_ethipopticmpecho(sniffdata, sniffdatasize,
                                            &hdrleth, &hdrlip, NULL, NULL,
                                            &type, &id, &seqnum, NULL, NULL);
  if ( ret!=LCRZO_ERR_OK )
    return(LCRZO_ERR_OK);

  /* check if the addresses are the one we are looking for */
  lcrzo_er(lcrzo_etha_equal2(hdrleth.dst, clientetha, &ethequal));
  if ( ! ethequal || hdrlip.daddr!=clientipl ||
       hdrlip.saddr<serveriplinf || hdrlip.saddr>serveriplsup )
    return(LCRZO_ERR_OK);

  /* check type */
  if ( type!=LCRZO_ICMPTYPE_ECHOREP )
    return(LCRZO_ERR_OK);

  /* compute the theindex from the received packet */
  theindex1=id-startingid+1;
  theindex2=seqnum-startingid+1;
  if ( theindex1!=theindex2 )
    return(LCRZO_ERR_OK);

  /* destination is reached */
  lcrzo_epr(lcrzo_ipl_print("", hdrlip.saddr, ""));
  lcrzo_epr(lcrzo_global_get_language(&language));
  if ( language==LCRZO_GLOBAL_LANG_FR )
  { printf(" - atteint\n");
  }
  else
  { printf(" - reached\n");
  }

  return(LCRZO_ERR_OK);
}





