
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000264=
{ /*reference*/     "lcrzoex", 264,
  /*version*/       1, 1,
  /*title*/         "verify smtp relaying",
  /*french title*/  "verifie le relayage smtp",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "server_name server_port \"relay_domain\" \"dest_email\"",
  /*french usage*/  "nom_serveur port_serveur \"domaine_du_relai\" \"email_dest\"",
  /*usage example*/ "1.2.3.4 25 \"example.com\" \"laurent@s.com\"",
  /*fr. usage ex.*/ "1.2.3.4 25 \"example.com\" \"laurent@s.com\"",
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
#define LCRZOEX_000264_MAXERR 5
lcrzo_string lcrzoex_000264_mailfrom_ok;
lcrzo_string lcrzoex_000264_rcptto_ok;

/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000264_main(int argc, char *argv[]);
int lcrzoex_000264_email_allforms_a(lcrzo_int16 type,
				    lcrzo_conststring dstemail,
				    lcrzo_string *pemail);
int lcrzoex_000264_email_allforms_b(lcrzo_int16 type,
				    lcrzo_conststring dstemail,
				    lcrzo_conststring relay,
				    lcrzo_string *pemail);
int lcrzoex_000264_email_allforms_mailfrom(lcrzo_int16 type,
					   lcrzo_conststring dstemail,
					   lcrzo_consths relaydomain,
					   lcrzo_ipl relayipl,
					   lcrzo_string *pemail);
int lcrzoex_000264_email_allforms_rcptto(lcrzo_int16 type,
					 lcrzo_conststring dstemail,
					 lcrzo_consths relaydomain,
					 lcrzo_ipl relayipl,
					 lcrzo_string *pemail);
int lcrzoex_000264_session_init(lcrzo_ipl relayipl,
				lcrzo_uint16 relayport,
				lcrzo_sock *psock);
int lcrzoex_000264_session_close(lcrzo_sock *psock);
int lcrzoex_000264_session_rst(lcrzo_sock *psock);
int lcrzoex_000264_session_mailfrom_rcptto(lcrzo_sock *psock,
					   lcrzo_conststring mailfrom,
					   lcrzo_conststring rcptto);
int lcrzoex_000264_email_verify_mailfrom(lcrzo_ipl relayipl,
					 lcrzo_uint16 relayport,
					 lcrzo_conststring relaydomain,
					 lcrzo_conststring dstemail);
int lcrzoex_000264_email_verify_rcptto(lcrzo_ipl relayipl,
				       lcrzo_uint16 relayport,
				       lcrzo_conststring relaydomain,
				       lcrzo_conststring dstemail);
int lcrzoex_000264_email_verify(lcrzo_ipl relayipl,
				lcrzo_uint16 relayport,
				lcrzo_conststring relaydomain,
				lcrzo_conststring dstemail);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000264_main(int argc, char *argv[])
{ lcrzo_ipl relayipl;
  lcrzo_uint16 relayport;
  lcrzo_int16 language;

  lcrzo_init();
  if ( argc<5 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000264, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }

  lcrzo_epr(shared_para_ipl_init_dst(argv[1], &relayipl));
  lcrzo_epr(shared_para_port_init_dst(argv[2], 25, &relayport));

  /* init arrays which stores result */
  lcrzo_epr(lcrzo_string_initm_text("", &lcrzoex_000264_mailfrom_ok));
  lcrzo_epr(lcrzo_string_initm_text("", &lcrzoex_000264_rcptto_ok));

  lcrzo_epr(lcrzo_global_get_language(&language));

  if ( language==LCRZO_GLOBAL_LANG_FR )
    puts("\nCherche les 'mail from' valides");
  else
    puts("\nCheck valids 'mail from'");
  lcrzo_epr(lcrzoex_000264_email_verify_mailfrom(relayipl, relayport,
						 argv[3], argv[4]));

  if ( language==LCRZO_GLOBAL_LANG_FR )
    puts("\nCherche les 'rcpt to' valides");
  else
    puts("\nCheck valids 'rcpt to'");
  lcrzo_epr(lcrzoex_000264_email_verify_rcptto(relayipl, relayport,
					       argv[3], argv[4]));

  if ( language==LCRZO_GLOBAL_LANG_FR )
    puts("\nEnvoie les emails");
  else
    puts("\nSend emails");
  lcrzo_epr(lcrzoex_000264_email_verify(relayipl, relayport, argv[3],argv[4]));

  lcrzo_epr(lcrzo_string_free2(&lcrzoex_000264_mailfrom_ok));
  lcrzo_epr(lcrzo_string_free2(&lcrzoex_000264_rcptto_ok));

  lcrzo_close();
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
#define LCRZOEX_000264_ALLFORMS_A_MAX 6
int lcrzoex_000264_email_allforms_a(lcrzo_int16 type,
				    lcrzo_conststring dstemail,
				    lcrzo_string *pemail)
{ lcrzo_regexp re;
  lcrzo_string user, domain;
  lcrzo_int16 language;
  int ret;

  /* dstemail must be "aa@bb.cc" */
  lcrzo_epr(lcrzo_global_get_language(&language));
  ret=lcrzo_string_search_regexp(dstemail, LCRZO_FALSE, +1, -1,"(.+)@(.+)",re);
  if ( ret!=LCRZO_ERR_OK )
  { if ( language==LCRZO_GLOBAL_LANG_FR )
      printf("L'adresse email destinataire doit etre de la forme aa@bb\n"); 
    else
      printf("The email address has not the form aa@bb\n"); 
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }
  lcrzo_epr(lcrzo_string_initm_range(dstemail, re[1].startindex_positive,
                                     re[1].endindex_positive, &user));
  lcrzo_epr(lcrzo_string_initm_range(dstemail, re[2].startindex_positive,
                                     re[2].endindex_positive, &domain));
 
  /* construct all solutions */
  ret=LCRZO_ERR_OK;
  switch(type) {
  case 0:
    lcrzo_epr(lcrzo_string_initm_text(user, pemail));
    lcrzo_epr(lcrzo_string_appendm_text("@", pemail));
    lcrzo_epr(lcrzo_string_appendm_text(domain, pemail));
    break;
  case 1:
    lcrzo_epr(lcrzo_string_initm_text(user, pemail));
    lcrzo_epr(lcrzo_string_appendm_text("%", pemail));
    lcrzo_epr(lcrzo_string_appendm_text(domain, pemail));
    break;
  case 2:
    lcrzo_epr(lcrzo_string_initm_text(domain, pemail));
    lcrzo_epr(lcrzo_string_appendm_text("!", pemail));
    lcrzo_epr(lcrzo_string_appendm_text(user, pemail));
    break;
  case 3:
    lcrzo_epr(lcrzo_string_initm_text("\"", pemail));
    lcrzo_epr(lcrzo_string_appendm_text(user, pemail));
    lcrzo_epr(lcrzo_string_appendm_text("@", pemail));
    lcrzo_epr(lcrzo_string_appendm_text(domain, pemail));
    lcrzo_epr(lcrzo_string_appendm_text("\"", pemail));
    break;
  case 4:
    lcrzo_epr(lcrzo_string_initm_text("\"", pemail));
    lcrzo_epr(lcrzo_string_appendm_text(user, pemail));
    lcrzo_epr(lcrzo_string_appendm_text("%", pemail));
    lcrzo_epr(lcrzo_string_appendm_text(domain, pemail));
    lcrzo_epr(lcrzo_string_appendm_text("\"", pemail));
    break;
  case 5:
    lcrzo_epr(lcrzo_string_initm_text("\"", pemail));
    lcrzo_epr(lcrzo_string_appendm_text(domain, pemail));
    lcrzo_epr(lcrzo_string_appendm_text("!", pemail));
    lcrzo_epr(lcrzo_string_appendm_text(user, pemail));
    lcrzo_epr(lcrzo_string_appendm_text("\"", pemail));
    break;
  case LCRZOEX_000264_ALLFORMS_A_MAX:
  default:
    ret=LCRZO_ERR_IEINTERNALERROR;
    break;
  }

  lcrzo_epr(lcrzo_string_free2(&user));
  lcrzo_epr(lcrzo_string_free2(&domain));
  return(ret);
}

/*-------------------------------------------------------------*/
#define LCRZOEX_000264_ALLFORMS_B_MAX 4
int lcrzoex_000264_email_allforms_b(lcrzo_int16 type,
				    lcrzo_conststring dstemail,
				    lcrzo_conststring relay,
				    lcrzo_string *pemail)
{ int ret;

  /* construct all solutions */
  ret=LCRZO_ERR_OK;
  switch(type) {
  case 0:
    lcrzo_epr(lcrzo_string_initm_text(dstemail, pemail));
    lcrzo_epr(lcrzo_string_appendm_text("@", pemail));
    lcrzo_epr(lcrzo_string_appendm_text(relay, pemail));
    break;
  case 1:
    lcrzo_epr(lcrzo_string_initm_text("\"", pemail));
    lcrzo_epr(lcrzo_string_appendm_text(dstemail, pemail));
    lcrzo_epr(lcrzo_string_appendm_text("@", pemail));
    lcrzo_epr(lcrzo_string_appendm_text(relay, pemail));
    lcrzo_epr(lcrzo_string_appendm_text("\"", pemail));
    break;
  case 2:
    lcrzo_epr(lcrzo_string_initm_text("@", pemail));
    lcrzo_epr(lcrzo_string_appendm_text(relay, pemail));
    lcrzo_epr(lcrzo_string_appendm_text(":", pemail));
    lcrzo_epr(lcrzo_string_appendm_text(dstemail, pemail));
    break;
  case 3:
    lcrzo_epr(lcrzo_string_initm_text("\"@", pemail));
    lcrzo_epr(lcrzo_string_appendm_text(relay, pemail));
    lcrzo_epr(lcrzo_string_appendm_text(":", pemail));
    lcrzo_epr(lcrzo_string_appendm_text(dstemail, pemail));
    lcrzo_epr(lcrzo_string_appendm_text("\"", pemail));
    break;
  case LCRZOEX_000264_ALLFORMS_B_MAX:
  default:
    ret=LCRZO_ERR_IEINTERNALERROR;
    break;
  }

  return(ret);
}

/*-------------------------------------------------------------*/
int lcrzoex_000264_email_allforms_mailfrom(lcrzo_int16 type,
					   lcrzo_conststring dstemail,
					   lcrzo_consths relaydomain,
					   lcrzo_ipl relayipl,
					   lcrzo_string *pemail)
{ lcrzo_string relayips;
  lcrzo_ips ips;

  /* construct all solutions */

  if ( type<1 )
  { lcrzo_epr(lcrzo_string_initm_text("", pemail));
    return(LCRZO_ERR_OK);
  }
  type--;

  if ( type<1 )
  { lcrzo_epr(lcrzo_string_initm_text("relaytest", pemail));
    return(LCRZO_ERR_OK);
  }
  type--;

  if ( type<LCRZOEX_000264_ALLFORMS_A_MAX )
  { lcrzo_epr(lcrzoex_000264_email_allforms_a(type, "relaytest@localhost",
					      pemail));
    return(LCRZO_ERR_OK);
  }
  type=(lcrzo_int16)(type-LCRZOEX_000264_ALLFORMS_A_MAX);

  if ( type<LCRZOEX_000264_ALLFORMS_A_MAX )
  { lcrzo_epr(lcrzoex_000264_email_allforms_a(type, "relaytest@azerfdsq.zzz",
					      pemail));
    return(LCRZO_ERR_OK);
  }
  type=(lcrzo_int16)(type-LCRZOEX_000264_ALLFORMS_A_MAX);

  if ( type<LCRZOEX_000264_ALLFORMS_A_MAX )
  { lcrzo_epr(lcrzoex_000264_email_allforms_a(type, dstemail, pemail));
    return(LCRZO_ERR_OK);
  }
  type=(lcrzo_int16)(type-LCRZOEX_000264_ALLFORMS_A_MAX);

  /* check if have relaydomain */
  if ( relaydomain!=NULL && strlen(relaydomain) )
  { if ( type<LCRZOEX_000264_ALLFORMS_B_MAX )
    { lcrzo_epr(lcrzoex_000264_email_allforms_b(type, "relaytest", 
						relaydomain, pemail));
      return(LCRZO_ERR_OK);
    }
    type=(lcrzo_int16)(type-LCRZOEX_000264_ALLFORMS_B_MAX);
    if ( type<LCRZOEX_000264_ALLFORMS_B_MAX )
    { lcrzo_epr(lcrzoex_000264_email_allforms_b(type, "postmaster", 
						relaydomain, pemail));
      return(LCRZO_ERR_OK);
    }
    type=(lcrzo_int16)(type-LCRZOEX_000264_ALLFORMS_B_MAX);
  }

  /* use relayips */
  if ( type<2*LCRZOEX_000264_ALLFORMS_B_MAX )
  { lcrzo_epr(lcrzo_string_initm_text("[", &relayips));
    lcrzo_epr(lcrzo_ips_init_ipl(relayipl, ips));
    lcrzo_epr(lcrzo_string_appendm_text(ips, &relayips));
    lcrzo_epr(lcrzo_string_appendm_text("]", &relayips));
    if ( type<LCRZOEX_000264_ALLFORMS_B_MAX )
    { lcrzo_epr(lcrzoex_000264_email_allforms_b(type, "relaytest", 
						relayips, pemail));
      lcrzo_epr(lcrzo_string_free2(&relayips));
      return(LCRZO_ERR_OK);
    }
    type=(lcrzo_int16)(type-LCRZOEX_000264_ALLFORMS_B_MAX);
    if ( type<LCRZOEX_000264_ALLFORMS_B_MAX )
    { lcrzo_epr(lcrzoex_000264_email_allforms_b(type, "postmaster", 
						relayips, pemail));
      lcrzo_epr(lcrzo_string_free2(&relayips));
      return(LCRZO_ERR_OK);
    }
    type=(lcrzo_int16)(type-LCRZOEX_000264_ALLFORMS_B_MAX);
    lcrzo_epr(lcrzo_string_free2(&relayips));
    return(LCRZO_ERR_OK);
  }
  type=(lcrzo_int16)(type-2*LCRZOEX_000264_ALLFORMS_B_MAX);

  /* simply add new forms here */

  /* else, error */
  return(LCRZO_ERR_OKDATAEND);
}

/*-------------------------------------------------------------*/
int lcrzoex_000264_email_allforms_rcptto(lcrzo_int16 type,
					 lcrzo_conststring dstemail,
					 lcrzo_consths relaydomain,
					 lcrzo_ipl relayipl,
					 lcrzo_string *pemail)
{ lcrzo_string relayips, email;
  lcrzo_ips ips;
  int a, b, i;
  char c;

  /* construct all solutions */
  if ( type<LCRZOEX_000264_ALLFORMS_A_MAX )
  { lcrzo_epr(lcrzoex_000264_email_allforms_a(type, dstemail, pemail));
    return(LCRZO_ERR_OK);
  }
  type=(lcrzo_int16)(type-LCRZOEX_000264_ALLFORMS_A_MAX);

  /* check if have relaydomain */
  if ( relaydomain!=NULL && strlen(relaydomain) )
  { if ( type<LCRZOEX_000264_ALLFORMS_A_MAX*LCRZOEX_000264_ALLFORMS_B_MAX )
    { a=type%LCRZOEX_000264_ALLFORMS_A_MAX;
      b=type/LCRZOEX_000264_ALLFORMS_A_MAX;
      lcrzo_epr(lcrzoex_000264_email_allforms_a((lcrzo_uint16)a, dstemail,
						&email));
      lcrzo_epr(lcrzoex_000264_email_allforms_b((lcrzo_uint16)b, email,
						relaydomain, pemail));
      lcrzo_epr(lcrzo_string_free2(&email));
      return(LCRZO_ERR_OK);
    }
    type=(lcrzo_int16)(type-LCRZOEX_000264_ALLFORMS_A_MAX*
		       LCRZOEX_000264_ALLFORMS_B_MAX);
  }

  /* use relayips */
  if ( type<LCRZOEX_000264_ALLFORMS_A_MAX*LCRZOEX_000264_ALLFORMS_B_MAX )
  { a=type%LCRZOEX_000264_ALLFORMS_A_MAX;
    b=type/LCRZOEX_000264_ALLFORMS_A_MAX;
    lcrzo_epr(lcrzo_string_initm_text("[", &relayips));
    lcrzo_epr(lcrzo_ips_init_ipl(relayipl, ips));
    lcrzo_epr(lcrzo_string_appendm_text(ips, &relayips));
    lcrzo_epr(lcrzo_string_appendm_text("]", &relayips));
    lcrzo_epr(lcrzoex_000264_email_allforms_a((lcrzo_uint16)a, dstemail,
					      &email));
    lcrzo_epr(lcrzoex_000264_email_allforms_b((lcrzo_uint16)b, email, 
					      relayips, pemail));
    lcrzo_epr(lcrzo_string_free2(&email));
    lcrzo_epr(lcrzo_string_free2(&relayips));
    return(LCRZO_ERR_OK);
  }
  type=(lcrzo_int16)(type-LCRZOEX_000264_ALLFORMS_A_MAX*
		     LCRZOEX_000264_ALLFORMS_B_MAX);

  /* use Exchange bug */
  if ( type<1 )
  { lcrzo_epr(lcrzo_string_initm_text("IMCEASMTP-", pemail));
    for ( i=0 ; i<(int)strlen(dstemail) ; i++ )
    { c=dstemail[i];
      if ( c=='.' )
      { lcrzo_epr(lcrzo_string_appendm_text("+40", pemail));
      }
      else if ( c=='@' )
      { lcrzo_epr(lcrzo_string_appendm_text("+2E", pemail));
      }
      else
      { lcrzo_epr(lcrzo_string_appendm_char(c, 1, pemail));
      }
    }
    lcrzo_epr(lcrzo_string_appendm_text("@", pemail));
    lcrzo_epr(lcrzo_string_appendm_text(relaydomain, pemail));
    return(LCRZO_ERR_OK);
  }
  type--;

  /* simply add new forms here */

  /* else, error */
  return(LCRZO_ERR_OKDATAEND);
}

/*-------------------------------------------------------------*/
int lcrzoex_000264_session_init(lcrzo_ipl relayipl,
				lcrzo_uint16 relayport,
				lcrzo_sock *psock)
{ lcrzo_data data;
  lcrzo_int32 datasize;
  
  /* initialize the client */
  lcrzo_epr(lcrzo_sock_tcpcli_real(relayipl, relayport, psock));

  /* first, we read the banner */
  lcrzo_epr(shared_misc_verifyresponse1(psock, 220));

  /* then, we send a 'HELO' */
  lcrzo_epr(lcrzo_data_initm_text("HELO microsoft.com\r\n", &data, &datasize));
  lcrzo_epr(lcrzo_sock_write(psock, data, datasize));
  lcrzo_epr(lcrzo_data_free2(&data));
  lcrzo_epr(shared_misc_verifyresponse1(psock, 250));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000264_session_close(lcrzo_sock *psock)
{ lcrzo_data data;
  lcrzo_int32 datasize;

  /* we send a 'QUIT' */
  lcrzo_epr(lcrzo_data_initm_text("QUIT\r\n",
                                  &data, &datasize));
  lcrzo_epr(lcrzo_sock_write(psock, data, datasize));
  lcrzo_epr(lcrzo_data_free2(&data));
  lcrzo_epr(shared_misc_verifyresponse1(psock, 221));

  /* close the socket */
  lcrzo_epr(lcrzo_sock_close(psock));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000264_session_rst(lcrzo_sock *psock)
{ lcrzo_data data;
  lcrzo_int32 datasize;

  lcrzo_epr(lcrzo_data_initm_text("RSET\r\n", &data, &datasize));
  lcrzo_epr(lcrzo_sock_write(psock, data, datasize));
  lcrzo_epr(lcrzo_data_free2(&data));
  lcrzo_epr(shared_misc_verifyresponse1(psock, 250));

  return(LCRZO_ERR_OK);
}


/*-------------------------------------------------------------*/
int lcrzoex_000264_session_mailfrom_rcptto(lcrzo_sock *psock,
					   lcrzo_conststring mailfrom,
					   lcrzo_conststring rcptto)
{ lcrzo_data data;
  lcrzo_int32 datasize;
  int ret;

  /* we send a 'MAIL FROM' */
  lcrzo_epr(lcrzo_data_initm_text("Mail From: <", &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text(mailfrom, datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text(">\r\n", datasize, &data, &datasize));
  lcrzo_epr(lcrzo_sock_write(psock, data, datasize));
  lcrzo_epr(lcrzo_data_free2(&data));
  ret=shared_misc_verifyresponse(psock, 250, 250, 250, 250, LCRZO_FALSE,
				 NULL, NULL, NULL);
  if ( ret!=LCRZO_ERR_OK )
  { lcrzo_epr(lcrzoex_000264_session_rst(psock));
    return(ret);
  }

  /* then, we send a 'RCPT TO' (we should not need need, but PostFix
     give an error only after rcptto) */
  lcrzo_epr(lcrzo_data_initm_text("Rcpt to: <", &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text(rcptto, datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text(">\r\n", datasize, &data, &datasize));
  lcrzo_epr(lcrzo_sock_write(psock, data, datasize));
  lcrzo_epr(lcrzo_data_free2(&data));
  ret=shared_misc_verifyresponse(psock, 250, 250, 250, 250, LCRZO_FALSE,
				 NULL, NULL, NULL);
  if ( ret!=LCRZO_ERR_OK )
  { lcrzo_epr(lcrzoex_000264_session_rst(psock));
    return(ret);
  }

  /* we send a 'RSET' */
  lcrzo_epr(lcrzoex_000264_session_rst(psock));    
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000264_email_verify_mailfrom(lcrzo_ipl relayipl,
					 lcrzo_uint16 relayport,
					 lcrzo_conststring relaydomain,
					 lcrzo_conststring dstemail)
{ lcrzo_string mailfrom, rcptto;
  lcrzo_sock sock;
  int ret, i;
  
  /* initialize the client */
  lcrzo_epr(lcrzoex_000264_session_init(relayipl, relayport, &sock));

  /* init rcptto */
  lcrzo_epr(lcrzo_string_initm_text("postmaster@", &rcptto));
  lcrzo_epr(lcrzo_string_appendm_text(relaydomain, &rcptto));

  i=0;
  ret=LCRZO_ERR_OK;
  while (LCRZO_TRUE)
  { /* generate an email */
    ret=lcrzoex_000264_email_allforms_mailfrom((lcrzo_uint16)i, dstemail,
					       relaydomain, 
					       relayipl, &mailfrom);
    if (ret==LCRZO_ERR_OKDATAEND) { ret=LCRZO_ERR_OK; break; }
    else if (ret!=LCRZO_ERR_OK) break;
    /* check it */
    ret=lcrzoex_000264_session_mailfrom_rcptto(&sock, mailfrom, rcptto);
    if (ret==LCRZO_ERR_OK)
    { lcrzo_epr(lcrzo_string_appendm_text("y", &lcrzoex_000264_mailfrom_ok));
      printf("OK : %s\n", mailfrom);
    }
    else if (ret==LCRZOEX_ERR_MISC_BADRESPONSE)
    { lcrzo_epr(lcrzo_string_appendm_text("n", &lcrzoex_000264_mailfrom_ok));
      printf("KO : %s\n", mailfrom);
    }
    else
    { lcrzo_epr(lcrzo_string_free2(&mailfrom));
      break;
    }
    lcrzo_epr(lcrzo_string_free2(&mailfrom));
    /*printf("%s\n", lcrzoex_000264_mailfrom_ok);*/
    /* eventually close and reopen session */
    i++;
    if ( !(i%LCRZOEX_000264_MAXERR) )
    { lcrzo_epr(lcrzoex_000264_session_close(&sock));
      lcrzo_epr(lcrzoex_000264_session_init(relayipl, relayport, &sock));
    }
  }
  lcrzo_epr(lcrzo_string_free2(&rcptto));

  /* close the socket */
  lcrzo_epr(lcrzoex_000264_session_close(&sock));
  return(ret);
}

/*-------------------------------------------------------------*/
int lcrzoex_000264_email_verify_rcptto(lcrzo_ipl relayipl,
				       lcrzo_uint16 relayport,
				       lcrzo_conststring relaydomain,
				       lcrzo_conststring dstemail)
{ int ret, i;
  lcrzo_string mailfrom, rcptto;
  lcrzo_sock sock;
  char *pc;
  
  /* initialize the client */
  lcrzo_epr(lcrzoex_000264_session_init(relayipl, relayport, &sock));

  /* init mailfrom */
  pc=strchr(lcrzoex_000264_mailfrom_ok, 'y');
  if ( pc==NULL )
  { printf("No valid mailfrom address. We cannot continue.\n");
    return(LCRZO_ERR_OK);
  }
  lcrzo_epr(lcrzoex_000264_email_allforms_mailfrom((lcrzo_uint16)(pc -
						   lcrzoex_000264_mailfrom_ok),
						   dstemail, relaydomain, 
						   relayipl, &mailfrom));

  i=0;
  ret=LCRZO_ERR_OK;
  while (LCRZO_TRUE)
  { /* generate an email */
    ret=lcrzoex_000264_email_allforms_rcptto((lcrzo_uint16)i, dstemail, relaydomain, 
					     relayipl, &rcptto);
    if (ret==LCRZO_ERR_OKDATAEND) { ret=LCRZO_ERR_OK; break; }
    else if (ret!=LCRZO_ERR_OK) break;
    /* check it */
    ret=lcrzoex_000264_session_mailfrom_rcptto(&sock, mailfrom, rcptto);
    if (ret==LCRZO_ERR_OK)
    { lcrzo_epr(lcrzo_string_appendm_text("y", &lcrzoex_000264_rcptto_ok));
      printf("OK : %s\n", rcptto);
    }
    else if (ret==LCRZOEX_ERR_MISC_BADRESPONSE)
    { lcrzo_epr(lcrzo_string_appendm_text("n", &lcrzoex_000264_rcptto_ok));
      printf("KO : %s\n", rcptto);
    }
    else
    { lcrzo_epr(lcrzo_string_free2(&rcptto));
      break;
    }
    lcrzo_epr(lcrzo_string_free2(&rcptto));
    /*printf("%s\n", lcrzoex_000264_rcptto_ok);*/
    /* eventually close and reopen session */
    i++;
    if ( !(i%LCRZOEX_000264_MAXERR) )
    { lcrzo_epr(lcrzoex_000264_session_close(&sock));
      lcrzo_epr(lcrzoex_000264_session_init(relayipl, relayport, &sock));
    }
  }
  lcrzo_epr(lcrzo_string_free2(&mailfrom));

  /* close the socket */
  lcrzo_epr(lcrzoex_000264_session_close(&sock));
  return(ret);
}

/*-------------------------------------------------------------*/
int lcrzoex_000264_email_verify(lcrzo_ipl relayipl,
				lcrzo_uint16 relayport,
				lcrzo_conststring relaydomain,
				lcrzo_conststring dstemail)
{ int ret, i, j, number;
  lcrzo_string mailfrom, rcptto, headerbody;
  lcrzo_sock sock;
  
  /* initialize the client */
  lcrzo_epr(lcrzoex_000264_session_init(relayipl, relayport, &sock));

  for ( number=0,i=0 ; i<(int)strlen(lcrzoex_000264_mailfrom_ok) ; i++ )
  { if ( lcrzoex_000264_mailfrom_ok[i] != 'y' ) continue;
    for ( j=0 ; j<(int)strlen(lcrzoex_000264_rcptto_ok) ; j++ )
    { if ( lcrzoex_000264_rcptto_ok[j] != 'y' ) continue;
      /* generate emails */
      lcrzo_epr(lcrzoex_000264_email_allforms_mailfrom((lcrzo_uint16)i, 
						       dstemail, relaydomain,
						       relayipl, &mailfrom));
      lcrzo_epr(lcrzoex_000264_email_allforms_rcptto((lcrzo_uint16)j,
						     dstemail, relaydomain, 
						     relayipl, &rcptto));
      /* check them */
      ret=lcrzoex_000264_session_mailfrom_rcptto(&sock, mailfrom, rcptto);
      if (ret==LCRZO_ERR_OK)
      { printf("OK : %s --> %s\n", mailfrom, rcptto);
        lcrzo_epr(lcrzo_string_initm_text("Subject: relaying test\r\n",
					  &headerbody));
        lcrzo_epr(lcrzo_string_appendm_text("\r\n", &headerbody));
        lcrzo_epr(lcrzo_string_appendm_text("Mail from : ", &headerbody));
        lcrzo_epr(lcrzo_string_appendm_text(mailfrom, &headerbody));
        lcrzo_epr(lcrzo_string_appendm_text("\r\n", &headerbody));
        lcrzo_epr(lcrzo_string_appendm_text("Rcpt to   : ", &headerbody));
        lcrzo_epr(lcrzo_string_appendm_text(rcptto, &headerbody));
        lcrzo_epr(lcrzo_string_appendm_text("\r\n", &headerbody));
        lcrzo_epr(shared_smtp_sendmsg(relayipl, relayport, mailfrom, rcptto, 
				      (lcrzo_data)headerbody,
				      strlen(headerbody)));
	lcrzo_epr(lcrzo_string_free2(&headerbody));
      }
      else if (ret==LCRZOEX_ERR_MISC_BADRESPONSE)
      { printf("KO : %s --> %s\n", mailfrom, rcptto);
      }
      else
      { lcrzo_epr(lcrzo_string_free2(&mailfrom));
        lcrzo_epr(lcrzo_string_free2(&rcptto));
        lcrzo_epr(lcrzoex_000264_session_close(&sock));
	return(ret);
      }
      lcrzo_epr(lcrzo_string_free2(&mailfrom));
      lcrzo_epr(lcrzo_string_free2(&rcptto));
      /* eventually close and reopen session */
      number++;
      if ( !(number%LCRZOEX_000264_MAXERR) )
      { lcrzo_epr(lcrzoex_000264_session_close(&sock));
        lcrzo_epr(lcrzoex_000264_session_init(relayipl, relayport, &sock));
      }
    }
  }

  /* close the socket */
  lcrzo_epr(lcrzoex_000264_session_close(&sock));
  return(LCRZO_ERR_OK);
}



