
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000267=
{ /*reference*/     "lcrzoex", 267,
  /*version*/       1, 0,
  /*title*/         "real tcp client [download all messages in a newsgroup]",
  /*french title*/  "client tcp reel [telecharge tous les messages d'un newsgroup]",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "server_name server_port newsgroup directory",
  /*french usage*/  "nom_serveur port_serveur newsgroup repertoire",
  /*usage example*/ "1.2.3.4 119 fr.test /tmp/download",
  /*fr. usage ex.*/ "1.2.3.4 119 fr.test /tmp/download",
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000267_main(int argc, char *argv[]);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000267_main(int argc, char *argv[])
{ lcrzo_ipl ipl;
  lcrzo_uint16 port;
  lcrzo_list list;
  lcrzo_int32 count, i;
  lcrzo_uint32 j;
  lcrzo_data data;
  lcrzo_int32 datasize;
  shared_nntp_list_newsgroups snln;
  lcrzo_bool newsgroupfound, direxist;
  lcrzo_string filename;
  lcrzo_int16 language;

  lcrzo_init();
  if ( argc<5 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000267, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }

  lcrzo_epr(shared_para_ipl_init_dst(argv[1], &ipl));
  lcrzo_epr(shared_para_port_init_dst(argv[2], 119, &port));

  /* get newsgroups on this server */
  lcrzo_epr(shared_nntp_list_init_newsgroups(ipl, port, &list));
  lcrzo_epr(lcrzo_list_count(&list, &count));
  newsgroupfound=LCRZO_FALSE;
  snln.numberfirst=snln.numberlast=0; /* for the compiler warning */
  for ( i=1 ; i<=count ; i++ )
  { lcrzo_epr(lcrzo_list_value_pos(&list, i, &snln));
    if (!strcmp(snln.name, argv[3]))
    { newsgroupfound=LCRZO_TRUE;
      break;
    }
  }
  lcrzo_epr(lcrzo_list_close(&list));

  /* check if found this newsgroup */
  if ( !newsgroupfound )
  { lcrzo_epr(lcrzo_global_get_language(&language));
    if ( language==LCRZO_GLOBAL_LANG_FR )
      printf("Le newsgroup %s n'a pas ete trouve sur ce serveur\n", argv[3]);
    else
      printf("The newsgroup %s wasn't found on this server\n", argv[3]);
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }

  /* check if given directory exist */
  lcrzo_epr(lcrzo_dir_exist(argv[4], &direxist));
  if ( !direxist )
  { lcrzo_epr(lcrzo_global_get_language(&language));
    if ( language==LCRZO_GLOBAL_LANG_FR )
      printf("Le repertoire %s n'existe pas\n", argv[4]);
    else
      printf("The directory %s doesn't exist\n", argv[4]);
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }

  /* get each message */
  for ( j=snln.numberfirst ; j<=snln.numberlast ; j++ )
  { lcrzo_epr(shared_nntp_recvmsg(ipl, port, argv[3], j, &data, &datasize));
    lcrzo_epr(lcrzo_uint32_print("", j, "%u", " : "));
    lcrzo_epr(lcrzo_int32_print("", datasize, "%d", "\n"));
    lcrzo_epr(lcrzo_string_initm_text(argv[4], &filename));
    lcrzo_epr(lcrzo_string_appendm_text("/", &filename));
    lcrzo_epr(lcrzo_string_appendm_text(argv[3], &filename));
    lcrzo_epr(lcrzo_string_appendm_text(".", &filename));
    lcrzo_epr(lcrzo_string_appendm_uint32(j, "%010u", &filename));
    lcrzo_epr(lcrzo_filename_write(filename, LCRZO_FALSE, LCRZO_FALSE, 
				   data, datasize));
    lcrzo_epr(lcrzo_string_free2(&filename));
    lcrzo_epr(lcrzo_data_free2(&data));
  }

  lcrzo_close();
  return(LCRZO_ERR_OK);
}


