
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000269=
{ /*reference*/     "lcrzoex", 269,
  /*version*/       1, 0,
  /*title*/         "acknowledge every TCP SYN",
  /*french title*/  "acquitte tous les SYN TCP",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "device iplist portlist [\"bpf_filter\"]",
  /*french usage*/  "device listeip listedeports [\"filtre_bpf\"]",
  /*usage example*/ "eth0 1.2.3.3-1.2.3.6 \"any,!21,!23\" \"net 1.2.3.0/24\"",
  /*fr. usage ex.*/ "eth0 1.2.3.3-1.2.3.6 \"any,!21,!23\" \"net 1.2.3.0/24\"",
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declaration */
int lcrzoex_000269_main(int argc, char *argv[]);
int lcrzoex_000269_synack(lcrzo_device device,
			  lcrzo_list *piplist,
			  lcrzo_list *pportlist,
			  lcrzo_data data, 
			  lcrzo_int32 datasize);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000269_main(int argc, char *argv[])
{ lcrzo_device device;
  lcrzo_uint32 mtu;
  lcrzo_sniff sniff;
  lcrzo_data sniffdata;
  lcrzo_int32 sniffdatasize;
  lcrzo_list iplist;
  lcrzo_list portlist;
  int ret;

  lcrzo_init();
  if ( argc<4 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000269, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }

  /* initialize device, lists and mtu */
  lcrzo_epr(shared_para_device_init_src(argv[1], device));
  lcrzo_epr(lcrzo_mtu_init_device(device, &mtu));
  lcrzo_epr(lcrzo_iplist_init_text(argv[2], &iplist));
  lcrzo_epr(lcrzo_portlist_init_text(argv[3], &portlist));

  /* ask user if he really wants to continue */
  lcrzo_epr(shared_misc_dangerousexample());

  /* init sniff */
  if (argc<=4)
  { lcrzo_epr(lcrzo_sniff_init(device, mtu, LCRZO_SNIFF_TYPE_IPREAS,
			       "", &sniff));
  }
  else
  { lcrzo_epr(lcrzo_sniff_init(device, mtu, LCRZO_SNIFF_TYPE_IPREAS,
			       argv[4], &sniff));
  }

  /* main loop */
  ret=LCRZO_ERR_OK;
  while (LCRZO_TRUE)
  { ret=lcrzo_sniff_nextm(&sniff, LCRZO_WAIT_INFINITE, 
			&sniffdata, &sniffdatasize);
    if (ret!=LCRZO_ERR_OK) break;
    /*lcrzo_packet_print_eth(sniffdata, sniffdatasize,
                           LCRZO_PRINTPROFILE_SYNTHSYNTH);*/
    /* now, we send a SYNACK for each SYN */
    ret=lcrzoex_000269_synack(device, &iplist, &portlist, 
			      sniffdata, sniffdatasize);
    lcrzo_epr(lcrzo_data_free2(&sniffdata));
    if (ret!=LCRZO_ERR_OK) break;
  }

  /* close everything */
  lcrzo_epr(lcrzo_list_close(&iplist));
  lcrzo_epr(lcrzo_list_close(&portlist));
  lcrzo_epr(lcrzo_sniff_close(&sniff));
  lcrzo_epr(lcrzo_err_print(ret));
  lcrzo_close();
  return (ret);
}

/*-------------------------------------------------------------*/
int lcrzoex_000269_synack(lcrzo_device device,
			  lcrzo_list *piplist,
			  lcrzo_list *pportlist,
			  lcrzo_data data, 
			  lcrzo_int32 datasize)
{ lcrzo_hdrleth  recv_hdreth,  send_hdreth;
  lcrzo_hdrlip   recv_hdrip,   send_hdrip;
  lcrzo_hdrltcp recv_hdrtcp, send_hdrtcp;
  lcrzo_spoof spoof;
  lcrzo_bool yes;
  int ret;

  /* try to decode tcp */
  ret=lcrzo_packet_decodem_ethipopttcpoptdata(data, datasize, &recv_hdreth,
					      &recv_hdrip, NULL, NULL,
					      &recv_hdrtcp, NULL, NULL, 
					      NULL, NULL);
  if ( ret!=LCRZO_ERR_OK )
    return(LCRZO_ERR_OK);
  /* only answer to syn */
  if ( !recv_hdrtcp.syn || recv_hdrtcp.rst || recv_hdrtcp.ack ||
       recv_hdrtcp.fin || recv_hdrtcp.urg || recv_hdrtcp.psh )
    return(LCRZO_ERR_OK);
  /* only if destination IP is in list */
  lcrzo_epr(lcrzo_iplist_isinlist(piplist, recv_hdrip.daddr, &yes));
  if (!yes) return(LCRZO_ERR_OK);
  /* only if destination port is in list */
  lcrzo_epr(lcrzo_portlist_isinlist(pportlist, recv_hdrtcp.dport, &yes));
  if (!yes) return(LCRZO_ERR_OK);

  /* create the new Ethernet header */
  lcrzo_epr(lcrzo_hdrleth_initdefault(&send_hdreth));
  memcpy(send_hdreth.src, recv_hdreth.dst, LCRZO_ETHA_MAXBYTES);
  memcpy(send_hdreth.dst, recv_hdreth.src, LCRZO_ETHA_MAXBYTES);

  /* create the new IP header */
  lcrzo_epr(lcrzo_hdrlip_initdefault(&send_hdrip));
  send_hdrip.saddr=recv_hdrip.daddr;
  send_hdrip.daddr=recv_hdrip.saddr;

  /* create the new TCP header */
  lcrzo_epr(lcrzo_hdrltcp_initdefault(&send_hdrtcp));
  send_hdrtcp.syn=1;
  send_hdrtcp.ack=1;
  send_hdrtcp.window=0;
  lcrzo_epr(lcrzo_uint32_init_rand(1, 0xEFFFFFFFu, &(send_hdrtcp.seqnum)));
  send_hdrtcp.acknum=recv_hdrtcp.seqnum+1;
  send_hdrtcp.sport=recv_hdrtcp.dport;
  send_hdrtcp.dport=recv_hdrtcp.sport;

  /* send packet */
  lcrzo_epr(lcrzo_spoof_init(&spoof));
  lcrzo_epr(lcrzo_spoof_set_printbeforesending(&spoof, LCRZO_TRUE));
  lcrzo_epr(lcrzo_spoof_set_printprofile(&spoof, 
					 LCRZO_PRINTPROFILE_SYNTHSYNTH));
  lcrzo_epr(lcrzo_spoof_ethipopttcpoptdata(&spoof, device, send_hdreth, 
					   send_hdrip, NULL, 0,
					   send_hdrtcp, NULL, 0,
					   NULL, 0));
  lcrzo_epr(lcrzo_spoof_close(&spoof));
  return (LCRZO_ERR_OK);
}
