
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000272=
{ /*reference*/     "lcrzoex", 272,
  /*version*/       1, 0,
  /*title*/         "interactive telnet client",
  /*french title*/  "client telnet interactif",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "server_name server_port username",
  /*french usage*/  "nom_serveur port_serveur nomdelogin",
  /*usage example*/ "1.2.3.4 23 user",
  /*fr. usage ex.*/ "1.2.3.4 23 util",
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000272_main(int argc, char *argv[]);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000272_main(int argc, char *argv[])
{ lcrzo_ipl serveripl;
  lcrzo_uint16 serverport;
  shared_telnet telnet;
  lcrzo_data data;
  lcrzo_string passwd, command;
  lcrzo_int32 datasize;
  lcrzo_file stdinfile;
  lcrzo_int16 language;

  lcrzo_init();
  if ( argc<4 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000272, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }
  lcrzo_epr(shared_para_ipl_init_dst(argv[1], &serveripl));
  lcrzo_epr(shared_para_port_init_dst(argv[2], 23, &serverport));

  /* init the session */
  lcrzo_epr(shared_telnet_init(serveripl, serverport, &telnet));

  /* send login/password */
  lcrzo_epr(lcrzo_string_initm_text_passwd("Passwd", "", &passwd));
  lcrzo_epr(shared_telnet_login(&telnet, argv[3], passwd));
  lcrzo_epr(lcrzo_string_free2(&passwd));

  lcrzo_epr(lcrzo_global_get_language(&language));
  if ( language==LCRZO_GLOBAL_LANG_FR )
  { puts("Entrez votre commande");
  }
  else
  { puts("Enter your command : ");
  }

  lcrzo_epr(lcrzo_file_init_fd(0, &stdinfile));
  while (LCRZO_TRUE)
  { lcrzo_epr(lcrzo_file_readm_line(&stdinfile, LCRZO_WAIT_INFINITE, 
				    &data, &datasize));
    lcrzo_epr(lcrzo_string_initm_data(data, datasize, &command));
    lcrzo_epr(lcrzo_data_free2(&data)); 
    if ( !strncmp(command, "exit", 4) || !strncmp(command, "logout", 6) )
    { lcrzo_epr(lcrzo_string_free2(&command)); 
      break;
    }
    lcrzo_epr(shared_telnet_exec(&telnet, command, 1000000, &data, &datasize));
    lcrzo_epr(lcrzo_string_free2(&command)); 
    lcrzo_epr(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_TEXT));
    lcrzo_epr(lcrzo_data_free2(&data));
  }
  lcrzo_epr(lcrzo_file_close(&stdinfile));

  /* logout */
  puts("");
  lcrzo_epr(shared_telnet_logout(&telnet));

  /* close the session */
  lcrzo_epr(shared_telnet_close(&telnet));

  lcrzo_close();
  return (LCRZO_ERR_OK);
}

