
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000274=
{ /*reference*/     "lcrzoex", 274,
  /*version*/       1, 0,
  /*title*/         "sniff packets and print them (easy device selection)",
  /*french title*/  "sniffe des paquets et les affiche (selection simple du device)",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "[\"bpf_filter\"]",
  /*french usage*/  "[\"filtre_bpf\"]",
  /*usage example*/ "\"host 1.2.3.4 and port 25\"",
  /*fr. usage ex.*/ "\"host 1.2.3.4 and port 25\"", 
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declaration */
int lcrzoex_000274_main(int argc, char *argv[]);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000274_main(int argc, char *argv[])
{ lcrzo_device device;
  lcrzo_uint32 mtu;
  lcrzo_sniff sniff;
  lcrzo_data sniffdata;
  lcrzo_int32 sniffdatasize;
  lcrzo_printprofile printprofile;
  lcrzo_int16 language;
  lcrzo_string snifffilter;
  int ret;
  
  lcrzo_init();

  lcrzo_epr(lcrzo_global_get_language(&language));

  /* init device and mtu */
  ret=lcrzo_device_value_pos(LCRZO_DEVICE_TYPE_BEST, 1, device);
  if (ret!=LCRZO_ERR_OK)
  { lcrzo_er(lcrzo_device_initdefault(device));
  }
  lcrzo_epr(lcrzo_device_init_stdin((language==LCRZO_GLOBAL_LANG_FR)?
				    "Choisissez le device":
				    "Choose the device", device, device));
  lcrzo_epr(lcrzo_mtu_init_device(device, &mtu));

  /* get printprofile */
  lcrzo_epr(lcrzo_printprofile_init_stdin((language==LCRZO_GLOBAL_LANG_FR)?
				"Choisissez le profil d'affichage":
				"Choose the print profile",
				LCRZO_PRINTPROFILE_ARRAYDUMP, &printprofile));

  /* init sniff */
  if (argc<=1)
  { lcrzo_epr(lcrzo_sniff_init(device, mtu, LCRZO_SNIFF_TYPE_NORMAL,
			       "", &sniff));
  }
  else
  { lcrzo_epr(shared_para_string_init_snifffilter(argv[1], &snifffilter));
    lcrzo_epr(lcrzo_sniff_init(device, mtu, LCRZO_SNIFF_TYPE_NORMAL, 
                               snifffilter, &sniff));
    lcrzo_epr(lcrzo_string_free2(&snifffilter));
  }

  /* main loop */
  ret=LCRZO_ERR_OK;
  while(LCRZO_TRUE)
  { ret=lcrzo_sniff_nextm(&sniff, LCRZO_WAIT_INFINITE, 
			  &sniffdata, &sniffdatasize);
    if (ret!=LCRZO_ERR_OK) break;
    lcrzo_epr(lcrzo_packet_print_eth(sniffdata, sniffdatasize, printprofile));
    lcrzo_epr(lcrzo_data_free2(&sniffdata));
    if (printprofile.hdr!=LCRZO_PRINTTYPE_SYNTH) printf("\n");
  }

  /* close everything */
  lcrzo_epr(lcrzo_sniff_close(&sniff));
  lcrzo_epr(lcrzo_err_print(ret));

  lcrzo_close();
  return(LCRZO_ERR_OK);
}

