
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000280=
{ /*reference*/     "lcrzoex", 280,
  /*version*/       1, 1,
  /*title*/         "passive ftp listing a directory",
  /*french title*/  "ftp passif listant un repertoire",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "cliname sername cliport serport clidataport directory user [passwd]",
  /*french usage*/  "nomcli nomserv portcli portserv portdonneescli repertoire utilisateur [password]",
  /*usage example*/ "1.2.3.4 1.2.3.5 6666 21 7777 /etc \"joe\" \"mypass\"",
  /*fr. usage ex.*/ "1.2.3.4 1.2.3.5 6666 21 7777 /etc \"laurent\" \"maison\"",
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000280_main(int argc, char *argv[]);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000280_main(int argc, char *argv[])
{ shared_sock_address ssa;
  lcrzo_sock sockcontrol, sockdata;
  lcrzo_list listfile, listdir, listlinkname, listlinkobj;
  lcrzo_int32 count, i;
  lcrzo_uint32 filesize;
  lcrzo_string pwd, str, str2;
  lcrzo_int16 language;

  lcrzo_init();
  if ( argc<8 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000280, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }
  lcrzo_epr(lcrzo_global_get_language(&language));

  /* initialize IP addresses */
  lcrzo_epr(shared_para_ipl_init_src(argv[1], &ssa.clientipl));
  lcrzo_epr(shared_para_ipl_init_dst(argv[2], &ssa.serveripl));

  /* initialize ports */
  lcrzo_epr(shared_para_port_init_src(argv[3], &ssa.clientport));
  lcrzo_epr(shared_para_port_init_dst(argv[4], 21, &ssa.serverport));
  lcrzo_epr(shared_para_port_init_ftpdatalocal(argv[5], &ssa.clientport2));

  /* initialize control connection */
  lcrzo_epr(lcrzo_sock_tcpcli_real2(ssa.clientipl, ssa.serveripl,
				    ssa.clientport, ssa.serverport, 
				    NULL, 0, &sockcontrol));

  /* first, we connect */
  if (argc>8)
  { lcrzo_epr(lcrzo_string_initm_text(argv[8], &pwd));
  }
  else
  { lcrzo_epr(lcrzo_string_initm_text_passwd("Passwd", "", &pwd));
  }
  lcrzo_epr(shared_ftp_login(&sockcontrol, argv[7], pwd));
  lcrzo_epr(lcrzo_string_free2(&pwd));

  /* then, we send a 'PASV' */
  lcrzo_epr(shared_ftp_cmd_pasv(&sockcontrol, &ssa.serveripl,
				&ssa.serverport2));
  /*lcrzo_ipl_print("Server IP : ", ssa.serveripl, "\n");
    printf("Server port : %d\n", ssa.serverport2);*/

  /* initialize data connection */
  lcrzo_epr(lcrzo_sock_tcpcli_real2(ssa.clientipl, ssa.serveripl, 
				    ssa.clientport2, ssa.serverport2,
				    NULL, 0,
				    &sockdata));

  /* we send a 'LIST' */
  lcrzo_epr(shared_ftp_data_list(&sockcontrol, LCRZO_TRUE, &sockdata, NULL,
				 argv[6], &listfile, &listdir,
				 &listlinkname, &listlinkobj));

  /* display files */
  lcrzo_epr(lcrzo_list_count(&listfile, &count));
  if ( language==LCRZO_GLOBAL_LANG_FR )
    puts("Fichiers");
  else
    puts("Files");
  for ( i=1 ; i<=count ; i++ )
  { lcrzo_epr(lcrzo_stringlist_valuem_pos(&listfile, i, &filesize, &str));
    printf("  %s ", str);
    lcrzo_epr(lcrzo_int32_print("(", filesize, "%d", ")\n"));
    lcrzo_epr(lcrzo_string_free2(&str));
  }
  lcrzo_epr(lcrzo_list_close(&listfile));

  /* display directories */
  lcrzo_epr(lcrzo_list_count(&listdir, &count));
  if ( language==LCRZO_GLOBAL_LANG_FR )
    puts("Repertoires");
  else
    puts("Directories");
  for ( i=1 ; i<=count ; i++ )
  { lcrzo_epr(lcrzo_stringlist_valuem_pos(&listdir, i, NULL, &str));
    printf("  %s\n", str);
    lcrzo_epr(lcrzo_string_free2(&str));
  }
  lcrzo_epr(lcrzo_list_close(&listdir));

  /* display links */
  lcrzo_epr(lcrzo_list_count(&listlinkname, &count));
  if ( language==LCRZO_GLOBAL_LANG_FR )
    puts("Liens");
  else
    puts("Links");
  for ( i=1 ; i<=count ; i++ )
  { lcrzo_epr(lcrzo_stringlist_valuem_pos(&listlinkname, i, NULL, &str));
    lcrzo_epr(lcrzo_stringlist_valuem_pos(&listlinkobj, i, &filesize, &str2));
    printf("  %s ---> %s ", str, str2);
    lcrzo_epr(lcrzo_int32_print("(", filesize, "%d", ")\n"));
    lcrzo_epr(lcrzo_string_free2(&str2));
    lcrzo_epr(lcrzo_string_free2(&str));
  }
  lcrzo_epr(lcrzo_list_close(&listlinkname));
  lcrzo_epr(lcrzo_list_close(&listlinkobj));

  /* then, we send a 'QUIT' */
  lcrzo_epr(shared_ftp_cmd_quit(&sockcontrol));

  /* close the socket */
  lcrzo_epr(lcrzo_sock_close(&sockcontrol));

  lcrzo_close();
  return (LCRZO_ERR_OK);
}
