
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000284=
{ /*reference*/     "lcrzoex", 284,
  /*version*/       1, 0,
  /*title*/         "passive ftp uploading a file",
  /*french title*/  "ftp passif uploadant(put) un fichier",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "cliname sername cliport serport clidataport localfile distantfile user [passwd]",
  /*french usage*/  "nomcli nomserv portcli portserv portdonneescli fichierlocal fichierdistant utilisateur [password]",
  /*usage example*/ "1.2.3.4 1.2.3.5 6666 21 7777 ./file /etc/passwd \"joe\" \"mypass\"",
  /*fr. usage ex.*/ "1.2.3.4 1.2.3.5 6666 21 7777 ./fichier /etc/passwd \"laurent\" \"maison\"",
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000284_main(int argc, char *argv[]);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000284_main(int argc, char *argv[])
{ shared_sock_address ssa;
  lcrzo_sock sockcontrol, sockdata;
  lcrzo_string pwd;
  lcrzo_int16 language;

  lcrzo_init();
  if ( argc<9 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000284, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }
  lcrzo_epr(lcrzo_global_get_language(&language));

  /* initialize IP addresses */
  lcrzo_epr(shared_para_ipl_init_src(argv[1], &ssa.clientipl));
  lcrzo_epr(shared_para_ipl_init_dst(argv[2], &ssa.serveripl));

  /* initialize ports */
  lcrzo_epr(shared_para_port_init_src(argv[3], &ssa.clientport));
  lcrzo_epr(shared_para_port_init_dst(argv[4], 21, &ssa.serverport));
  lcrzo_epr(shared_para_port_init_ftpdatalocal(argv[5], &ssa.clientport2));

  /* initialize control connection */
  lcrzo_epr(lcrzo_sock_tcpcli_real2(ssa.clientipl, ssa.serveripl,
				    ssa.clientport, ssa.serverport, 
				    NULL, 0, &sockcontrol));

  /* first, we connect */
  if (argc>9)
  { lcrzo_epr(lcrzo_string_initm_text(argv[9], &pwd));
  }
  else
  { lcrzo_epr(lcrzo_string_initm_text_passwd("Passwd", "", &pwd));
  }
  lcrzo_epr(shared_ftp_login(&sockcontrol, argv[8], pwd));
  lcrzo_epr(lcrzo_string_free2(&pwd));

  /* then, we send a 'PASV' */
  lcrzo_epr(shared_ftp_cmd_pasv(&sockcontrol, &ssa.serveripl,
				&ssa.serverport2));

  /* initialize data connection */
  lcrzo_epr(lcrzo_sock_tcpcli_real2(ssa.clientipl, ssa.serveripl, 
				    ssa.clientport2, ssa.serverport2,
				    NULL, 0,
				    &sockdata));

  /* we send a 'STOR', and put file */
  lcrzo_epr(shared_ftp_data_stor(&sockcontrol, LCRZO_TRUE,
				 &sockdata, NULL, argv[6], argv[7]));

  /* then, we send a 'QUIT' */
  lcrzo_epr(shared_ftp_cmd_quit(&sockcontrol));

  /* close the socket */
  lcrzo_epr(lcrzo_sock_close(&sockcontrol));

  lcrzo_close();
  return (LCRZO_ERR_OK);
}
