
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000285=
{ /*reference*/     "lcrzoex", 285,
  /*version*/       1, 1,
  /*title*/         "active ftp downloading a directory",
  /*french title*/  "ftp actif telechargeant un repertoire",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "server_name server_port distant_dir local_dir login [passwd]",
  /*french usage*/  "nom_serveur port_serveur rep_distant rep_local login [passwd]",
  /*usage example*/ "1.2.3.4 21 /etc/ /tmp/here bob mypass",
  /*fr. usage ex.*/ "1.2.3.4 21 /etc/ /tmp/ici bob monpwd",
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000285_main(int argc, char *argv[]);
int lcrzoex_000285_recursiveftpget(lcrzo_ipl serverip,
				   lcrzo_uint16 serverport,
				   lcrzo_conststring distantdir,
				   lcrzo_conststring localdir,
				   lcrzo_conststring login,
				   lcrzo_conststring passwd);
int lcrzoex_000285_recursiveget(lcrzo_sock *psock, 
				lcrzo_conststring distantdir,
				lcrzo_conststring localdir,
                                lcrzo_ipl clientipl,
                                lcrzo_uint16 *pclientport);
int lcrzoex_000285_getfile(lcrzo_sock *psock, 
			   lcrzo_conststring distantfile,
			   lcrzo_int32 distantfilesize,
			   lcrzo_conststring localfile,
			   lcrzo_ipl clientipl,
			   lcrzo_uint16 *pclientport);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000285_main(int argc, char *argv[])
{ lcrzo_ipl ipl;
  lcrzo_uint16 port;
  lcrzo_string pwd;
  lcrzo_int16 language;

  lcrzo_init();
  if ( argc<6 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000285, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }

  lcrzo_epr(shared_para_ipl_init_dst(argv[1], &ipl));
  lcrzo_epr(shared_para_port_init_dst(argv[2], 21, &port));

  if (argc>6)
  { lcrzo_epr(lcrzo_string_initm_text(argv[6], &pwd));
  }
  else
  { lcrzo_epr(lcrzo_string_initm_text_passwd("Passwd", "", &pwd));
  }
  lcrzo_epr(lcrzoex_000285_recursiveftpget(ipl, port,
					   argv[3], argv[4], argv[5], pwd));
  lcrzo_epr(lcrzo_string_free2(&pwd));

  lcrzo_epr(lcrzo_global_get_language(&language));
  if ( language==LCRZO_GLOBAL_LANG_FR )
    puts("Fin");
  else
    puts("End");
  lcrzo_close();
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000285_recursiveftpget(lcrzo_ipl serverip,
				   lcrzo_uint16 serverport,
				   lcrzo_conststring distantdir,
				   lcrzo_conststring localdir,
				   lcrzo_conststring login,
				   lcrzo_conststring passwd)
{ lcrzo_sock sock;
  lcrzo_ipl clientipl;
  lcrzo_uint16 clientport;

  /* initialize the client */
  lcrzo_epr(lcrzo_sock_tcpcli_real(serverip, serverport, &sock));
  /* obtain the local ip used */
  lcrzo_epr(lcrzo_sock_get_infos(&sock, NULL, NULL, &clientipl, NULL,
                                 NULL, NULL, NULL));

  /* choose a random starting client port */
  lcrzo_epr(lcrzo_uint16_init_rand(30000, 60000, &clientport));

  /* first, we connect */
  lcrzo_epr(shared_ftp_login(&sock, login, passwd));

  /* then, we send a 'TYPE I' */
  lcrzo_epr(shared_ftp_cmd_type_bin(&sock));

  /* treat every file */  
  lcrzo_epr(lcrzoex_000285_recursiveget(&sock, distantdir, localdir,
                                        clientipl, &clientport));
  
  /* then, we send a 'QUIT' */
  lcrzo_epr(shared_ftp_cmd_quit(&sock));

  /* close the socket */
  lcrzo_epr(lcrzo_sock_close(&sock));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000285_recursiveget(lcrzo_sock *psock, 
				lcrzo_conststring distantdir,
				lcrzo_conststring localdir,
                                lcrzo_ipl clientipl,
                                lcrzo_uint16 *pclientport)
{ shared_sock_ipctcpser ssits;
  lcrzo_ipc ipcdata;
  lcrzo_list listdir, listfile, listlinkname, listlinkobj;
  lcrzo_int32 count, i;
  lcrzo_uint32 filesize, linktype;
  lcrzo_string distantobj, localobj, localobj2, str, str2;
  lcrzo_int16 language;
  int ret;

  lcrzo_epr(lcrzo_global_get_language(&language));

  /* first, create local directory */
  /*ignore*/lcrzo_dir_create(localdir);

  /* obtain the list of files and directories */
  (*pclientport)++;
  lcrzo_epr(shared_sock_ipctcpser_real2_init(clientipl, *pclientport,
                                             NULL,0, &ssits, &ipcdata));
  lcrzo_epr(shared_ftp_cmd_port(psock, clientipl, *pclientport));
  ret=shared_ftp_data_list(psock, LCRZO_FALSE,
			   NULL, &ipcdata, distantdir, 
			   &listfile, &listdir, &listlinkname, &listlinkobj);
  lcrzo_epr(shared_sock_ipctcpser_close(&ssits));
  if (ret!=LCRZO_ERR_OK) return(LCRZO_ERR_OK); /* do not download. skip*/

  /* get files */
  lcrzo_epr(lcrzo_list_count(&listfile, &count));
  for ( i=1 ; i<=count ; i++ )
  { lcrzo_epr(lcrzo_stringlist_valuem_pos(&listfile, i, &filesize, &str));
    if ( language==LCRZO_GLOBAL_LANG_FR )
      printf("  Fich");
    else
      printf("  File");
    printf("  %s ", str);
    lcrzo_epr(lcrzo_int32_print("(", filesize, "%d", ")\n"));
    lcrzo_epr(lcrzo_string_initm_text(distantdir, &distantobj));
    lcrzo_epr(lcrzo_string_appendm_text("/", &distantobj));
    lcrzo_epr(lcrzo_string_appendm_text(str, &distantobj));
    lcrzo_epr(lcrzo_string_initm_text(localdir, &localobj));
    lcrzo_epr(lcrzo_string_appendm_text("/", &localobj));
    lcrzo_epr(lcrzo_string_appendm_text(str, &localobj));
    lcrzo_epr(lcrzo_string_free2(&str));
    printf(" -> %s\n", localobj);
    lcrzo_epr(lcrzoex_000285_getfile(psock, distantobj, filesize,
				     localobj, clientipl, pclientport));
    lcrzo_epr(lcrzo_string_free2(&distantobj));
    lcrzo_epr(lcrzo_string_free2(&localobj));
  }
  lcrzo_epr(lcrzo_list_close(&listfile));

  /* get directories */
  lcrzo_epr(lcrzo_list_count(&listdir, &count));
  for ( i=1 ; i<=count ; i++ )
  { lcrzo_epr(lcrzo_stringlist_valuem_pos(&listdir, i, &filesize, &str));
    lcrzo_epr(lcrzo_string_initm_text(distantdir, &distantobj));
    lcrzo_epr(lcrzo_string_appendm_text("/", &distantobj));
    lcrzo_epr(lcrzo_string_appendm_text(str, &distantobj));
    lcrzo_epr(lcrzo_string_initm_text(localdir, &localobj));
    lcrzo_epr(lcrzo_string_appendm_text("/", &localobj));
    lcrzo_epr(lcrzo_string_appendm_text(str, &localobj));
    lcrzo_epr(lcrzo_string_free2(&str));
    if ( language==LCRZO_GLOBAL_LANG_FR )
      printf("Rep   ");
    else
      printf("Dir   ");
    printf("  %s -> %s\n", distantobj, localobj);
    lcrzo_epr(lcrzoex_000285_recursiveget(psock, distantobj, localobj,
					  clientipl, pclientport));
    lcrzo_epr(lcrzo_string_free2(&distantobj));
    lcrzo_epr(lcrzo_string_free2(&localobj));
  }
  lcrzo_epr(lcrzo_list_close(&listdir));

  /* create links
     Note : we do not get file outside the source directory */
  lcrzo_epr(lcrzo_list_count(&listlinkname, &count));
  for ( i=1 ; i<=count ; i++ )
  { lcrzo_epr(lcrzo_stringlist_valuem_pos(&listlinkname, i, &linktype, &str));
    lcrzo_epr(lcrzo_stringlist_valuem_pos(&listlinkobj, i, &filesize, &str2));
    lcrzo_epr(lcrzo_string_initm_text(distantdir, &distantobj));
    lcrzo_epr(lcrzo_string_appendm_text("/", &distantobj));
    lcrzo_epr(lcrzo_string_appendm_text(str, &distantobj));
    lcrzo_epr(lcrzo_string_initm_text(localdir, &localobj));
    lcrzo_epr(lcrzo_string_appendm_text("/", &localobj));
    lcrzo_epr(lcrzo_string_appendm_text(str, &localobj));
    lcrzo_epr(lcrzo_string_initm_text(str2, &localobj2));
    lcrzo_epr(lcrzo_string_free2(&str));
    lcrzo_epr(lcrzo_string_free2(&str2));
    if ( language==LCRZO_GLOBAL_LANG_FR )
      printf("  Lien ");
    else
      printf("  Link ");
    printf("  %s -> %s--->%s\n", distantobj, localobj, localobj2);
    lcrzo_ep(shared_misc_symlink(localobj2, localobj));
    lcrzo_epr(lcrzo_string_free2(&distantobj));
    lcrzo_epr(lcrzo_string_free2(&localobj));
    lcrzo_epr(lcrzo_string_free2(&localobj2));
  }
  lcrzo_epr(lcrzo_list_close(&listlinkname));
  lcrzo_epr(lcrzo_list_close(&listlinkobj));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000285_getfile(lcrzo_sock *psock, 
			   lcrzo_conststring distantfile,
			   lcrzo_int32 distantfilesize,
			   lcrzo_conststring localfile,
			   lcrzo_ipl clientipl,
			   lcrzo_uint16 *pclientport)
{ lcrzo_int32 localfilesize;
  shared_sock_ipctcpser ssits;
  lcrzo_ipc ipcdata;
  lcrzo_int16 language;

  lcrzo_epr(lcrzo_global_get_language(&language));

  /* create the listening data port */
  (*pclientport)++;
  lcrzo_epr(shared_sock_ipctcpser_real2_init(clientipl, *pclientport,
                                             NULL,0, &ssits, &ipcdata));

  /* then, we send a 'PORT' */
  lcrzo_epr(shared_ftp_cmd_port(psock, clientipl, *pclientport));

  /* we send a 'RETR', and get file */
  lcrzo_epr(shared_ftp_data_retr(psock, LCRZO_FALSE,
				 NULL, &ipcdata, distantfile, localfile));
  lcrzo_epr(shared_sock_ipctcpser_close(&ssits));

  /* we send a 'SIZE file' */
  lcrzo_epr(lcrzo_filename_size(localfile, &localfilesize));
  /* verify size */
  if ( distantfilesize!=localfilesize )
  { if ( language==LCRZO_GLOBAL_LANG_FR )
    { printf("Les tailles sont differentes : ");
      lcrzo_epr(lcrzo_int32_print("", distantfilesize, "%d", " "));
      lcrzo_epr(lcrzo_int32_print("", localfilesize, "%d", ", je reessaie\n"));
    }
    else
    { printf("Sizes are differents : ");
      lcrzo_epr(lcrzo_int32_print("", distantfilesize, "%d", " "));
      lcrzo_epr(lcrzo_int32_print("", localfilesize, "%d", ", we retry\n"));
    }
    /* retry */
    lcrzo_epr(lcrzoex_000285_getfile(psock, distantfile, distantfilesize,
				     localfile, clientipl, pclientport));
  }

  return(LCRZO_ERR_OK);
}
