
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000288=
{ /*reference*/     "lcrzoex", 288,
  /*version*/       1, 0,
  /*title*/         "passive ftp uploading a directory",
  /*french title*/  "ftp passif uploadant(put) un repertoire",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "server_name server_port local_dir distant_dir login [passwd]",
  /*french usage*/  "nom_serveur port_serveur rep_local rep_distant login [passwd]",
  /*usage example*/ "1.2.3.4 21 /tmp/here . bob mypass",
  /*fr. usage ex.*/ "1.2.3.4 21 /tmp/ici . bob monpwd",
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000288_main(int argc, char *argv[]);
int lcrzoex_000288_recursiveftpput(lcrzo_ipl serverip,
				   lcrzo_uint16 serverport,
				   lcrzo_conststring localdir,
				   lcrzo_conststring distantdir,
				   lcrzo_conststring login,
				   lcrzo_conststring passwd);
int lcrzoex_000288_recursiveput(lcrzo_sock *psock, 
				lcrzo_conststring abslocaldir,
				lcrzo_conststring rellocaldir);
int lcrzoex_000288_putfile(lcrzo_sock *psock, 
			   lcrzo_conststring absfilename, 
			   lcrzo_int32 filesize,
			   lcrzo_conststring relfilename);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000288_main(int argc, char *argv[])
{ lcrzo_ipl ipl;
  lcrzo_uint16 port;
  lcrzo_string pwd;
  lcrzo_int16 language;

  lcrzo_init();
  if ( argc<6 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000288, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }

  lcrzo_epr(shared_para_ipl_init_dst(argv[1], &ipl));
  lcrzo_epr(shared_para_port_init_dst(argv[2], 21, &port));

  if (argc>6)
  { lcrzo_epr(lcrzo_string_initm_text(argv[6], &pwd));
  }
  else
  { lcrzo_epr(lcrzo_string_initm_text_passwd("Passwd", "", &pwd));
  }
  lcrzo_epr(lcrzoex_000288_recursiveftpput(ipl, port,
					   argv[3], argv[4], argv[5], pwd));
  lcrzo_epr(lcrzo_string_free2(&pwd));

  lcrzo_epr(lcrzo_global_get_language(&language));
  if ( language==LCRZO_GLOBAL_LANG_FR )
    puts("Fin");
  else
    puts("End");
  lcrzo_close();
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000288_recursiveftpput(lcrzo_ipl serverip,
				   lcrzo_uint16 serverport,
				   lcrzo_conststring localdir,
				   lcrzo_conststring distantdir,
				   lcrzo_conststring login,
				   lcrzo_conststring passwd)
{ lcrzo_sock sock;

  /* initialize the client */
  lcrzo_epr(lcrzo_sock_tcpcli_real(serverip, serverport, &sock));

  /* first, we connect */
  lcrzo_epr(shared_ftp_login(&sock, login, passwd));

  /* then, we send a 'TYPE I' */
  lcrzo_epr(shared_ftp_cmd_type_bin(&sock));

  /* treat every file */  
  lcrzo_epr(lcrzoex_000288_recursiveput(&sock, localdir, distantdir));

  /* then, we send a 'QUIT' */
  lcrzo_epr(shared_ftp_cmd_quit(&sock));

  /* close the socket */
  lcrzo_epr(lcrzo_sock_close(&sock));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000288_recursiveput(lcrzo_sock *psock, 
				lcrzo_conststring abslocaldir,
				lcrzo_conststring rellocaldir)
{ lcrzo_int32 dircount, i, filesize;
  lcrzo_string localfile, filename;
  lcrzo_int16 language, filetype;

  lcrzo_epr(lcrzo_global_get_language(&language));

  /* we send a 'MKD dir' */
  lcrzo_epr(shared_ftp_cmd_mkd(psock, rellocaldir));

  /* we send a 'CWD dir' */
  lcrzo_epr(shared_ftp_cmd_cwd(psock, rellocaldir));

  /* go through the directory */
  lcrzo_epr(lcrzo_dir_count(abslocaldir, &dircount));
  for ( i=dircount ; i>=1; i-- )
  { lcrzo_epr(lcrzo_dir_value_pos(abslocaldir, i, &filename));
    /* ignore . and .. */
    if ( !strcmp(filename, ".") || !strcmp(filename, "..") )
    { lcrzo_epr(lcrzo_string_free2(&filename));
      continue;
    }
    /* set the local file */
    lcrzo_epr(lcrzo_string_initm_text(abslocaldir, &localfile));
    if ( abslocaldir[strlen(abslocaldir)-1] != '/' )
    { lcrzo_epr(lcrzo_string_appendm_char('/', 1, &localfile));
    }
    lcrzo_epr(lcrzo_string_appendm_text(filename, &localfile));
    /* stat it */
    lcrzo_epr(lcrzo_path_type(localfile, &filetype));
    if ( filetype==LCRZO_PATH_TYPE_DIR )
    { if ( language==LCRZO_GLOBAL_LANG_FR )
        printf("%s est un repertoire\n", localfile);
      else
        printf("%s is a directory\n", localfile);
      lcrzo_epr(lcrzoex_000288_recursiveput(psock, localfile, 
					    filename));
      /* this is very DANGEROUS to uncomment this : it DELETEs the local dir*/
      /*lcrzo_epr(lcrzo_dir_remove(localfile));*/
    }
    else if ( filetype==LCRZO_PATH_TYPE_REG )
    { if ( language==LCRZO_GLOBAL_LANG_FR )
        printf("%s est un fichier\n", localfile);
      else
	printf("%s is regular\n", localfile);
      lcrzo_epr(lcrzo_filename_size(localfile, &filesize));
      lcrzo_epr(lcrzoex_000288_putfile(psock, localfile, filesize,
				       filename));
      /* this is very DANGEROUS to uncomment this : it DELETEs the local file*/
      /*lcrzo_epr(lcrzo_filename_remove(localfile));*/
    }
    lcrzo_epr(lcrzo_string_free2(&filename));
    lcrzo_epr(lcrzo_string_free2(&localfile));
  }

  /* we send a 'CWD ..' */
  lcrzo_epr(shared_ftp_cmd_cwd(psock, ".."));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000288_putfile(lcrzo_sock *psock, 
			   lcrzo_conststring absfilename,
			   lcrzo_int32 filesize,
			   lcrzo_conststring relfilename)
{ lcrzo_int32 servsize;
  lcrzo_ipl pasvipl;
  lcrzo_uint16 pasvport;
  lcrzo_sock datasock;
  lcrzo_int16 language;

  lcrzo_epr(lcrzo_global_get_language(&language));

  /* get the port to which connect */
  lcrzo_epr(shared_ftp_cmd_pasv(psock, &pasvipl, &pasvport));
  /*lcrzo_ipl_print("IP : ", pasvipl, ":");
    printf("%d\n", pasvport);*/

  /* initialize the data connexion */
  lcrzo_epr(lcrzo_sock_tcpcli_real(pasvipl, pasvport, &datasock));

  /* we send a 'STOR', and put file */
  lcrzo_epr(shared_ftp_data_stor(psock, LCRZO_TRUE,
				 &datasock, NULL, absfilename, relfilename));

  /* we send a 'SIZE file' */
  lcrzo_epr(shared_ftp_cmd_size(psock, relfilename, &servsize));
  /* verify size */
  if ( filesize!=servsize )
  { if ( language==LCRZO_GLOBAL_LANG_FR )
    { printf("Les tailles sont differentes : ");
      lcrzo_epr(lcrzo_int32_print("", filesize, "%d", " "));
      lcrzo_epr(lcrzo_int32_print("", servsize, "%d", ", je reessaie\n"));
    }
    else
    { printf("Sizes are differents : ");
      lcrzo_epr(lcrzo_int32_print("", filesize, "%d", " "));
      lcrzo_epr(lcrzo_int32_print("", servsize, "%d", ", we retry\n"));
    }
    /* retry */
    lcrzo_epr(lcrzoex_000288_putfile(psock, absfilename, filesize,
				     relfilename));
  }

  return(LCRZO_ERR_OK);
}
