
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000290=
{ /*reference*/     "lcrzoex", 290,
  /*version*/       1, 0,
  /*title*/         "ftp client displaying type of a file",
  /*french title*/  "client ftp affichant le type d'un fichier",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "sername serport file user [passwd]",
  /*french usage*/  "nomserv portserv fichier utilisateur [password]",
  /*usage example*/ "1.2.3.5 21 /etc \"joe\" \"mypass\"",
  /*fr. usage ex.*/ "1.2.3.5 21 /etc \"laurent\" \"maison\"",
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000290_main(int argc, char *argv[]);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000290_main(int argc, char *argv[])
{ lcrzo_sock sockcontrol;
  lcrzo_int32 filesize;
  lcrzo_ipl serveripl;
  lcrzo_uint16 serverport;
  lcrzo_string pwd;
  lcrzo_bool isfile, isdir, islink;
  lcrzo_int16 language;

  lcrzo_init();
  if ( argc<5 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000290, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }
  lcrzo_epr(lcrzo_global_get_language(&language));

  /* initialize IP addresses */
  lcrzo_epr(shared_para_ipl_init_dst(argv[1], &serveripl));

  /* initialize ports */
  lcrzo_epr(shared_para_port_init_dst(argv[2], 21, &serverport));

  /* initialize control connection */
  lcrzo_epr(lcrzo_sock_tcpcli_real(serveripl, serverport, &sockcontrol));

  /* first, we connect */
  if (argc>5)
  { lcrzo_epr(lcrzo_string_initm_text(argv[5], &pwd));
  }
  else
  { lcrzo_epr(lcrzo_string_initm_text_passwd("Passwd", "", &pwd));
  }
  lcrzo_epr(shared_ftp_login(&sockcontrol, argv[4], pwd));
  lcrzo_epr(lcrzo_string_free2(&pwd));

  /* obtain info */
  lcrzo_epr(shared_ftp_file_typesize(&sockcontrol, argv[3],
				     &isfile, &isdir, &islink,
				     &filesize));

  /* then, we send a 'QUIT' */
  lcrzo_epr(shared_ftp_cmd_quit(&sockcontrol));

  /* close the socket */
  lcrzo_epr(lcrzo_sock_close(&sockcontrol));

  /* display info */
  if ( language==LCRZO_GLOBAL_LANG_FR )
  { printf("Fichier %s :\n", argv[3]);
    printf(" est un fichier     : %d (0=non,1=oui)\n", isfile);
    printf(" est un repertoire  : %d\n", isdir);
    printf(" est un lien        : %d\n", islink);
    lcrzo_epr(lcrzo_int32_print(" taille du fichier  : ", filesize, 
		      "%d", " octets\n"));
  }
  else
  { printf("File %s :\n", argv[3]);
    printf(" is a file          : %d (0=no,1=yes)\n", isfile);
    printf(" is a directory     : %d\n", isdir);
    printf(" is a link          : %d\n", islink);
    lcrzo_epr(lcrzo_int32_print(" file has a size of : ", filesize, 
		      "%d", " bytes\n"));
  }

  lcrzo_close();
  return (LCRZO_ERR_OK);
}
