
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000292=
{ /*reference*/     "lcrzoex", 292,
  /*version*/       1, 1,
  /*title*/         "ftp client deleting a directory recursively",
  /*french title*/  "client ftp effacant recursivement un repertoire",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "server_name server_port dir login [passwd]",
  /*french usage*/  "nom_serveur port_serveur reper login [passwd]",
  /*usage example*/ "1.2.3.4 21 /pub/dir/ bob mypass",
  /*fr. usage ex.*/ "1.2.3.4 21 /pub/rep/ bob monpwd",
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000292_main(int argc, char *argv[]);
int lcrzoex_000292_recursiveftpdel(lcrzo_ipl serverip,
				   lcrzo_uint16 serverport,
				   lcrzo_conststring distantdir,
				   lcrzo_conststring login,
				   lcrzo_conststring passwd);
int lcrzoex_000292_recursivedel(lcrzo_sock *psock, 
				lcrzo_conststring distantdir);
int lcrzoex_000292_delfile(lcrzo_sock *psock, 
                           lcrzo_conststring distantfile);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000292_main(int argc, char *argv[])
{ lcrzo_ipl ipl;
  lcrzo_uint16 port;
  lcrzo_string pwd;
  lcrzo_int16 language;

  lcrzo_init();
  if ( argc<5 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000292, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }

  lcrzo_epr(shared_para_ipl_init_dst(argv[1], &ipl));
  lcrzo_epr(shared_para_port_init_dst(argv[2], 21, &port));

  if (argc>5)
  { lcrzo_epr(lcrzo_string_initm_text(argv[5], &pwd));
  }
  else
  { lcrzo_epr(lcrzo_string_initm_text_passwd("Passwd", "", &pwd));
  }
  lcrzo_epr(lcrzoex_000292_recursiveftpdel(ipl, port,
					   argv[3], argv[4], pwd));
  lcrzo_epr(lcrzo_string_free2(&pwd));

  lcrzo_epr(lcrzo_global_get_language(&language));
  if ( language==LCRZO_GLOBAL_LANG_FR )
    puts("Fin");
  else
    puts("End");
  lcrzo_close();
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000292_recursiveftpdel(lcrzo_ipl serverip,
				   lcrzo_uint16 serverport,
				   lcrzo_conststring distantdir,
				   lcrzo_conststring login,
				   lcrzo_conststring passwd)
{ lcrzo_sock sock;

  /* initialize the client */
  lcrzo_epr(lcrzo_sock_tcpcli_real(serverip, serverport, &sock));

  /* first, we connect */
  lcrzo_epr(shared_ftp_login(&sock, login, passwd));

  /* treat every file */  
  lcrzo_epr(lcrzoex_000292_recursivedel(&sock, distantdir));
  
  /* then, we send a 'QUIT' */
  lcrzo_epr(shared_ftp_cmd_quit(&sock));

  /* close the socket */
  lcrzo_epr(lcrzo_sock_close(&sock));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000292_recursivedel(lcrzo_sock *psock, 
				lcrzo_conststring distantdir)
{ lcrzo_sock sockdata;
  lcrzo_list listdir, listfile, listlinkname;
  lcrzo_ipl serveripl;
  lcrzo_uint16 serverport;
  lcrzo_int32 count, i;
  lcrzo_string distantobj, str;
  lcrzo_int16 language;
  int ret;

  lcrzo_epr(lcrzo_global_get_language(&language));

  /* obtain the list of files and directories */
  lcrzo_epr(shared_ftp_cmd_pasv(psock, &serveripl, &serverport));
  lcrzo_epr(lcrzo_sock_tcpcli_real(serveripl, serverport, &sockdata));
  ret=shared_ftp_data_list(psock, LCRZO_TRUE, &sockdata, NULL,
			   distantdir, &listfile, &listdir, 
			   &listlinkname, NULL);
  if (ret!=LCRZO_ERR_OK) return(LCRZO_ERR_OK); /* do not download. skip*/

  /* del files */
  lcrzo_epr(lcrzo_list_count(&listfile, &count));
  for ( i=1 ; i<=count ; i++ )
  { lcrzo_epr(lcrzo_stringlist_valuem_pos(&listfile, i, NULL, &str));
    if ( language==LCRZO_GLOBAL_LANG_FR )
      printf("  Fich");
    else
      printf("  File");
    printf("  %s\n", str);
    lcrzo_epr(lcrzo_string_initm_text(distantdir, &distantobj));
    lcrzo_epr(lcrzo_string_appendm_text("/", &distantobj));
    lcrzo_epr(lcrzo_string_appendm_text(str, &distantobj));
    lcrzo_epr(lcrzo_string_free2(&str));
    lcrzo_epr(lcrzoex_000292_delfile(psock, distantobj));
    lcrzo_epr(lcrzo_string_free2(&distantobj));
  }
  lcrzo_epr(lcrzo_list_close(&listfile));

  /* del directories */
  lcrzo_epr(lcrzo_list_count(&listdir, &count));
  for ( i=1 ; i<=count ; i++ )
  { lcrzo_epr(lcrzo_stringlist_valuem_pos(&listdir, i, NULL, &str));
    lcrzo_epr(lcrzo_string_initm_text(distantdir, &distantobj));
    lcrzo_epr(lcrzo_string_appendm_text("/", &distantobj));
    lcrzo_epr(lcrzo_string_appendm_text(str, &distantobj));
    lcrzo_epr(lcrzo_string_free2(&str));
    if ( language==LCRZO_GLOBAL_LANG_FR )
      printf("Rep   ");
    else
      printf("Dir   ");
    printf("  %s\n", distantobj);
    lcrzo_epr(lcrzoex_000292_recursivedel(psock, distantobj));
    lcrzo_epr(lcrzo_string_free2(&distantobj));
  }
  lcrzo_epr(lcrzo_list_close(&listdir));

  /* delete links */
  lcrzo_epr(lcrzo_list_count(&listlinkname, &count));
  for ( i=1 ; i<=count ; i++ )
  { lcrzo_epr(lcrzo_stringlist_valuem_pos(&listlinkname, i, NULL, &str));
    lcrzo_epr(lcrzo_string_initm_text(distantdir, &distantobj));
    lcrzo_epr(lcrzo_string_appendm_text("/", &distantobj));
    lcrzo_epr(lcrzo_string_appendm_text(str, &distantobj));
    lcrzo_epr(lcrzo_string_free2(&str));
    if ( language==LCRZO_GLOBAL_LANG_FR )
      printf("  Lien ");
    else
      printf("  Link ");
    printf("  %s\n", distantobj);
    lcrzo_epr(lcrzoex_000292_delfile(psock, distantobj));
    lcrzo_epr(lcrzo_string_free2(&distantobj));
  }
  lcrzo_epr(lcrzo_list_close(&listlinkname));

  lcrzo_epr(shared_ftp_cmd_dele(psock, distantdir));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000292_delfile(lcrzo_sock *psock, 
                           lcrzo_conststring distantfile)
{ lcrzo_epr(shared_ftp_cmd_dele(psock, distantfile));
  return(LCRZO_ERR_OK);
}
