
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"


/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000293=
{ /*reference*/     "lcrzoex", 293,
  /*version*/       1, 0,
  /*title*/         "nntp client checking if posting is allowed",
  /*french title*/  "client client nntp verifiant si l'on peut poster",
  /*description*/   "note : we only check banner. It can be necessary to have a password to post on some groups.",
  /*french desc.*/  "note : on ne verifie que la banniere. Il peut etre necessaire de posseder un mot de passe pour poster.",
  /*usage*/         "server_name server_port",
  /*french usage*/  "nom_serveur port_serveur",
  /*usage example*/ "1.2.3.4 119",
  /*fr. usage ex.*/ "1.2.3.4 119",
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000293_main(int argc, char *argv[]);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000293_main(int argc, char *argv[])
{ lcrzo_ipl ipserver;
  lcrzo_uint16 portserver;
  lcrzo_sock sock;
  lcrzo_data data, tmp;
  lcrzo_int32 datasize, tmpsize;
  lcrzo_time maxwaitingtime;
  lcrzo_bool recognizedcode;
  lcrzo_int16 language;
  int ret;

  lcrzo_init();
  if ( argc<2 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000293, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }
 
  lcrzo_epr(lcrzo_global_get_language(&language));

  lcrzo_epr(shared_para_ipl_init_dst(argv[1], &ipserver));
  lcrzo_epr(shared_para_port_init_dst(argv[2], 119, &portserver));

  ret=lcrzo_sock_tcpcli_real(ipserver, portserver, &sock);
  if (ret!=LCRZO_ERR_OK)
  { if (language==LCRZO_GLOBAL_LANG_FR)
      printf("%s:%d : connection refusee\n", argv[1], portserver);
    else
      printf("%s:%d : refused connection\n", argv[1], portserver);
    return(ret);
  }
  
  lcrzo_epr(lcrzo_time_init_sec(3, &maxwaitingtime));
  ret=lcrzo_sock_readm_line(&sock, &maxwaitingtime, &data, &datasize);
  if (ret==LCRZO_ERR_OKTEMPDATAEND)
  { printf("%s:%d : timeout\n", argv[1], portserver);
    return(ret);
  }
  else if (ret!=LCRZO_ERR_OK)
  { lcrzo_epr(lcrzo_err_print(ret));
    return(ret);
  }


  /* analyze the data */
  /*lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_DUMP);*/ 
  recognizedcode=LCRZO_FALSE;
  if ( datasize>=4 )
  { if ( data[0]=='2' && data[1]=='0' && data[3]==' ')
    { if ( data[2]=='0' )
      { if (language==LCRZO_GLOBAL_LANG_FR)
          printf("%s:%d : peut poster\n", argv[1], portserver);
        else
          printf("%s:%d : can post\n", argv[1], portserver);
        recognizedcode=LCRZO_TRUE;
      }
      else if ( data[2]=='1' )
      { if (language==LCRZO_GLOBAL_LANG_FR)
          printf("%s:%d : ne peut pas poster\n", argv[1], portserver);
        else
          printf("%s:%d : can not post\n", argv[1], portserver);
        recognizedcode=LCRZO_TRUE;
      }
    }
  }
  if (!recognizedcode)
  { if ( datasize>=4 )
    { lcrzo_epr(lcrzo_data_initm_data(data, 4, &tmp, &tmpsize));
    }
    else
    { lcrzo_epr(lcrzo_data_initm_text("????", &tmp, &tmpsize));
    }
    tmp[tmpsize]='\0';
    if (language==LCRZO_GLOBAL_LANG_FR)
      printf("%s:%d : erreur '%s' recue\n", argv[1], portserver, tmp);
    else
      printf("%s:%d : error '%s' received\n", argv[1], portserver, tmp);
    lcrzo_epr(lcrzo_data_free2(&tmp)); 
  }
  lcrzo_epr(lcrzo_data_free2(&data));
  
  /* send a quit */
  lcrzo_epr(lcrzo_data_initm_text("QUIT\r\n", &data, &datasize));
  lcrzo_epr(lcrzo_sock_write(&sock, data, datasize));
  lcrzo_epr(lcrzo_data_free2(&data));

  lcrzo_epr(lcrzo_sock_close(&sock));
  
  lcrzo_close();
  return(LCRZO_ERR_OK);
}
