
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000316=
{ /*reference*/     "lcrzoex", 316,
  /*version*/       1, 0,
  /*title*/         "tcp traceroute (true IP, sent at IP level)",
  /*french title*/  "traceroute tcp (vrai IP, envoye au niveau IP)",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "dstip dstport maxttl",
  /*french usage*/  "ipdst portdst ttlmax",
  /*usage example*/ "5.6.7.78 80 30",
  /*fr. usage ex.*/ "5.6.7.78 80 30",
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000316_main(int argc, char *argv[]);
int lcrzoex_000316_traceroute(lcrzo_device device,
			      lcrzo_ipl clientipl, 
			      lcrzo_ipl serveripl, 
			      lcrzo_uint16 serverport,
			      lcrzo_uint8 maxttl);
int lcrzoex_000316_child_spoof(lcrzo_ptr infos, 
			       lcrzo_int32 infossize, 
			       lcrzo_process *pparentprocess);
int lcrzoex_000316_child_sniff(lcrzo_ptr infos, 
			       lcrzo_int32 infossize, 
			       lcrzo_process *pparentprocess);
int lcrzoex_000316_printroutei(lcrzo_ipl clientipl, 
			       lcrzo_ipl serveripl, 
			       lcrzo_uint16 serverport,
			       lcrzo_uint16 startingclientport,
			       lcrzo_uint32 startingclientseqnum,
			       lcrzo_uint8 maxttl,
			       lcrzo_data sniffdata,
			       lcrzo_int32 sniffdatasize);
int lcrzoex_000316_printroutet(lcrzo_ipl clientipl, 
			       lcrzo_ipl serveripl, 
			       lcrzo_uint16 serverport,
			       lcrzo_uint16 startingclientport,
			       lcrzo_uint32 startingclientseqnum,
			       lcrzo_data sniffdata,
			       lcrzo_int32 sniffdatasize);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000316_main(int argc, char *argv[])
{ lcrzo_device device;
  lcrzo_ipl clientipl, serveripl;
  lcrzo_uint16 serverport;
  lcrzo_uint8 maxttl;

  lcrzo_init();
  if ( argc<4 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000316, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }
  lcrzo_epr(shared_para_ipl_init_dst(argv[1], &serveripl));
  lcrzo_epr(shared_para_port_init_dst(argv[2], 80, &serverport));
  lcrzo_epr(shared_para_uint8_init(argv[3], "Maximum TTL", 1, 255,
				   30, &maxttl)); 
  if (maxttl==0) maxttl=1;

  lcrzo_epr(lcrzo_conf_route_to_host_ip(serveripl, device, &clientipl, NULL));

  /* call the main function */
  lcrzo_epr(lcrzoex_000316_traceroute(device, 
				      clientipl, serveripl, serverport,
				      maxttl));

  lcrzo_close();
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000316_traceroute(lcrzo_device device,
			      lcrzo_ipl clientipl, 
			      lcrzo_ipl serveripl, 
			      lcrzo_uint16 serverport,
			      lcrzo_uint8 maxttl)
{ shared_sock_address ssa;
  lcrzo_process childspoof, childsniff;
  lcrzo_wait waitspoof, waitsniff;
  lcrzo_bool eventspoof, eventsniff;
  int ret;

  /* set values */
  lcrzo_epr(lcrzo_device_init(device, ssa.device));
  ssa.clientipl=clientipl;
  ssa.serveripl=serveripl;
  ssa.serverport=serverport;
  ssa.ttl=maxttl;

  /* determine starting values */
  lcrzo_epr(lcrzo_uint16_init_rand(5000, 30000, &ssa.clientport2));
  lcrzo_epr(lcrzo_uint32_init_rand(0, 0xEFFFFFFFU, &ssa.seqnum));

  /* this process sends a SYN */
  lcrzo_epr(lcrzo_process_other_init(&lcrzoex_000316_child_spoof, 
                                     (lcrzo_data)&ssa, sizeof(ssa), 
				     &childspoof));

  /* this process sniff and prints sequence numbers */
  ret=lcrzo_process_other_init(&lcrzoex_000316_child_sniff, 
			       (lcrzo_data)&ssa, sizeof(ssa),
			       &childsniff);
  if ( ret!=LCRZO_ERR_OK)
  { /* kill the first child */
    lcrzo_epr(shared_lcrzoex_debug_processotherexit_wanttowarn(LCRZO_FALSE));
    lcrzo_epr(lcrzo_process_other_exit(&childspoof));
    lcrzo_epr(shared_lcrzoex_debug_processotherexit_wanttowarn(LCRZO_TRUE));
    lcrzo_epr(lcrzo_process_other_wait_end(&childspoof, LCRZO_WAIT_INFINITE, 
					   NULL, NULL));
    /* exit */
    return(ret);
  }

  /* wait for the end of one process */
  lcrzo_epr(lcrzo_wait_init_process_end(&childspoof, &waitspoof));
  lcrzo_epr(lcrzo_wait_init_process_end(&childsniff, &waitsniff));
  lcrzo_epr(lcrzo_wait_wait2(&waitspoof, &waitsniff, LCRZO_WAIT_INFINITE,
			     &eventspoof, &eventsniff));
  lcrzo_epr(lcrzo_wait_close(&waitspoof));
  lcrzo_epr(lcrzo_wait_close(&waitsniff));

  /* kill childs */
  if (eventspoof)
  { lcrzo_epr(shared_lcrzoex_debug_processotherexit_wanttowarn(LCRZO_FALSE));
    lcrzo_epr(lcrzo_process_other_exit(&childsniff));
    lcrzo_epr(shared_lcrzoex_debug_processotherexit_wanttowarn(LCRZO_TRUE));
    lcrzo_epr(lcrzo_process_other_wait_end(&childsniff, LCRZO_WAIT_INFINITE, 
					   NULL, NULL));
    lcrzo_epr(lcrzo_process_other_wait_end(&childspoof, LCRZO_WAIT_INFINITE, 
					   NULL, &ret));
    lcrzo_epr(lcrzo_process_close(&childspoof));
    lcrzo_epr(lcrzo_process_close(&childsniff));
    return(ret);
  }
  if (eventsniff)
  { lcrzo_epr(shared_lcrzoex_debug_processotherexit_wanttowarn(LCRZO_FALSE));
    lcrzo_epr(lcrzo_process_other_exit(&childspoof));
    lcrzo_epr(shared_lcrzoex_debug_processotherexit_wanttowarn(LCRZO_TRUE));
    lcrzo_epr(lcrzo_process_other_wait_end(&childspoof, LCRZO_WAIT_INFINITE, 
					   NULL, NULL));
    lcrzo_epr(lcrzo_process_other_wait_end(&childsniff, LCRZO_WAIT_INFINITE, 
					   NULL, &ret));
    lcrzo_epr(lcrzo_process_close(&childspoof));
    lcrzo_epr(lcrzo_process_close(&childsniff));
    return(ret);
  }

  /* should not be reached */
  return (LCRZO_ERR_IEINTERNALERROR);
}

/*-------------------------------------------------------------*/
int lcrzoex_000316_child_spoof(lcrzo_ptr infos, 
			       lcrzo_int32 infossize, 
			       lcrzo_process *pparentprocess)
{ shared_sock_address *pssa=(shared_sock_address *)infos;
  lcrzo_hdrlip hdrlip;
  lcrzo_hdrltcp hdrltcp;
  lcrzo_spoof spoof;
  lcrzo_uint8 i;
  int ret;

  /* check for correct usage of this function */
  if (infossize!=sizeof(shared_sock_address))
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);

  /* close parent process handle because we do not need it */
  lcrzo_epr(lcrzo_process_close(pparentprocess));

  /* initialize IP header */
  lcrzo_epr(lcrzo_hdrlip_initdefault(&hdrlip));
  hdrlip.saddr=pssa->clientipl;
  hdrlip.daddr=pssa->serveripl;
  hdrlip.ttl=1;
  hdrlip.id=pssa->clientport2;

  /* initialize TCP header */
  lcrzo_epr(lcrzo_hdrltcp_initdefault(&hdrltcp));
  hdrltcp.dport=pssa->serverport;
  hdrltcp.syn=1;
  hdrltcp.sport=pssa->clientport2;
  hdrltcp.seqnum=pssa->seqnum;

  /* little sleep to be sure sniff process is ready (should be done with
     an IPC) */
  lcrzo_epr(lcrzo_time_sleep_msec(300));

  /* spoof */
  lcrzo_epr(lcrzo_spoof_init(&spoof));
  /*lcrzo_epr(lcrzo_spoof_set_printbeforesending(&spoof, LCRZO_TRUE));*/
  ret=LCRZO_ERR_OK;
  for ( i=1 ; i<=pssa->ttl ; i++ )
  { ret=lcrzo_spoof_ipopttcpoptdata(&spoof,
				    hdrlip, NULL, 0,
				    hdrltcp, NULL, 0,
				    NULL, 0);
    if (ret!=LCRZO_ERR_OK) break;
    hdrlip.id++;
    hdrlip.ttl++;
    hdrltcp.sport++;
    hdrltcp.seqnum+=10;
    lcrzo_epr(lcrzo_time_sleep_sec(1));
  }
  lcrzo_epr(lcrzo_spoof_close(&spoof));
  
  return(ret);
}

/*-------------------------------------------------------------*/
int lcrzoex_000316_child_sniff(lcrzo_ptr infos, 
			       lcrzo_int32 infossize, 
			       lcrzo_process *pparentprocess)
{ shared_sock_address *pssa=(shared_sock_address *)infos;
  lcrzo_sniff sniff;
  lcrzo_uint32 mtu;
  lcrzo_data sniffdata;
  lcrzo_int32 sniffdatasize;
  int ret;

  /* check for correct usage of this function */
  if (infossize!=sizeof(shared_sock_address))
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);

  /* close parent process handle because we do not need it */
  lcrzo_epr(lcrzo_process_close(pparentprocess));

  /* init sniff */
  lcrzo_epr(lcrzo_mtu_init_device(pssa->device, &mtu));
  lcrzo_epr(lcrzo_sniff_init(pssa->device, mtu, LCRZO_SNIFF_TYPE_NORMAL, 
			     "", &sniff));

  /* main loop */
  ret=LCRZO_ERR_OK;
  while (LCRZO_TRUE)
  { ret=lcrzo_sniff_nextm(&sniff, LCRZO_WAIT_INFINITE, 
			  &sniffdata, &sniffdatasize);
    if (ret!=LCRZO_ERR_OK) break;
    /* decode an icmp error, and print it */
    ret=lcrzoex_000316_printroutei(pssa->clientipl, pssa->serveripl, 
				   pssa->serverport,
				   pssa->clientport2, 
				   pssa->seqnum,
				   pssa->ttl, sniffdata, sniffdatasize);
    if (ret!=LCRZO_ERR_OK) { lcrzo_epr(lcrzo_data_free2(&sniffdata)); break; }
    /* decode a tcp synack or reset, and print it */
    ret=lcrzoex_000316_printroutet(pssa->clientipl, pssa->serveripl, 
				   pssa->serverport,
				   pssa->clientport2, 
				   pssa->seqnum,
				   sniffdata, sniffdatasize);
    lcrzo_epr(lcrzo_data_free2(&sniffdata));
    if (ret!=LCRZO_ERR_OK) break;
  }
  lcrzo_epr(lcrzo_sniff_close(&sniff));
  return(ret);
}

/*-------------------------------------------------------------*/
int lcrzoex_000316_printroutei(lcrzo_ipl clientipl, 
			       lcrzo_ipl serveripl, 
			       lcrzo_uint16 serverport,
			       lcrzo_uint16 startingclientport,
			       lcrzo_uint32 startingclientseqnum,
			       lcrzo_uint8 maxttl,
			       lcrzo_data sniffdata,
			       lcrzo_int32 sniffdatasize)
{ lcrzo_hdrlip hdrlip, badhdrip;
  lcrzo_data baddata;
  lcrzo_string string;
  lcrzo_uint16 baddatasize, badtcpsport, badtcpdport;
  lcrzo_uint8 type, code;
  lcrzo_uint32 badtcpseqnum;
  lcrzo_int16 language;
  int ret, indexidok, indexidko, indexsport, indexseqnum;

  /*lcrzo_epr(lcrzo_packet_print_eth(sniffdata, sniffdatasize,
				   LCRZO_PRINTPROFILE_ARRAYDUMP));*/

  /* try to decode icmp */
  ret=lcrzo_packet_decodem_ethipopticmperror(sniffdata, sniffdatasize,
					     NULL, &hdrlip, NULL,NULL,
					     &type, &code, NULL, NULL,
					     &baddata, &baddatasize);
  if ( ret!=LCRZO_ERR_OK )
    return(LCRZO_ERR_OK);

  /* check if the addresses are the one we are looking for */
  if ( hdrlip.daddr!=clientipl )
  { lcrzo_epr(lcrzo_data_free2(&baddata));
    return(LCRZO_ERR_OK);
  }

  /* icmp should contain tcp */
  ret=lcrzo_ip64bits_decodem_iptcp(baddata, baddatasize,
				   &badhdrip, &badtcpsport, &badtcpdport,
				   &badtcpseqnum);
  lcrzo_epr(lcrzo_data_free2(&baddata));
  if ( ret!=LCRZO_ERR_OK )
    return(LCRZO_ERR_OK);

  /* check various values in baddata */
  if ( badtcpdport!=serverport ||
       badhdrip.saddr!=clientipl ||
       badhdrip.daddr!=serveripl )
    return(LCRZO_ERR_OK);

  /* compute indexid from the id of the received packet */
  indexidok=badhdrip.id-startingclientport+1;
  /* this one is because there is a bug in the IP stack of BSD :
     id is reversed :( */
  indexidko= ( (badhdrip.id&0xFF)<<8 ) | ( (badhdrip.id&0xFF00)>>8 );
  indexidko=indexidko-startingclientport+1;
  /* compute indexsport from the sport of the received packet */
  indexsport=badtcpsport-startingclientport+1;
  /* compute indexseqnum from the seqnum of the received packet */
  indexseqnum=1+(badtcpseqnum-startingclientseqnum)/10;

  /* verify that indexes are equal */
  if ( (indexidok!=indexsport&&indexidko!=indexsport) ||
       indexsport!=indexseqnum )
    return(LCRZO_ERR_OK);

  /* print the result */
  printf("% 3d : ", indexsport);
  lcrzo_epr(lcrzo_ipl_print("", hdrlip.saddr, ""));
      
  /* print the type of error */
  if ( type==LCRZO_ICMPTYPE_TIMEEXCEED &&
       code == LCRZO_ICMPCODE_TIMEEXCEED_TTL )
  { /* do not display the error type and code */
    printf("\n"); 
  }
  else { 
    /* display the error type and code */
    lcrzo_epr(lcrzo_string_initm_icmptypecode(type, code, &string));
    printf(" : %s\n", string);
    lcrzo_epr(lcrzo_string_free2(&string));
  }

  /* if maxttl was too short */
  if ( indexsport == maxttl )
  { lcrzo_epr(lcrzo_global_get_language(&language));
    if ( language==LCRZO_GLOBAL_LANG_FR )
      printf("maxttl(%d) est trop petit pour atteindre la destination\n", 
	     maxttl);
    else
      printf("maxttl(%d) was too short to reach the destination\n", 
	     maxttl);
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000316_printroutet(lcrzo_ipl clientipl, 
			       lcrzo_ipl serveripl, 
			       lcrzo_uint16 serverport,
			       lcrzo_uint16 startingclientport,
			       lcrzo_uint32 startingclientseqnum,
			       lcrzo_data sniffdata,
			       lcrzo_int32 sniffdatasize)
{ lcrzo_hdrlip hdrlip;
  lcrzo_hdrltcp hdrltcp;
  lcrzo_int16 language;
  int ret, theindex1, theindex2;

  /*lcrzo_epr(lcrzo_packet_print_eth(sniffdata, sniffdatasize,
    LCRZO_PRINTPROFILE_ARRAYDUMP));*/

  /* try to decode it */
  ret=lcrzo_packet_decodem_ethipopttcpoptdata(sniffdata, sniffdatasize,
					      NULL, &hdrlip, NULL, NULL,
					      &hdrltcp, NULL, NULL,
					      NULL, NULL);
  if ( ret!=LCRZO_ERR_OK )
    return(LCRZO_ERR_OK);

  /* check if the addresses are the one we are looking for */
  if ( hdrlip.saddr!=serveripl || hdrlip.daddr!=clientipl )
    return(LCRZO_ERR_OK);

  /* check server port */
  if ( hdrltcp.sport!=serverport )
    return(LCRZO_ERR_OK);

  /* compute the theindex from the received packet */
  theindex1=hdrltcp.dport-startingclientport+1;
  theindex2=1+(hdrltcp.acknum-1-startingclientseqnum)/10;
  if ( theindex1!=theindex2 )
    return(LCRZO_ERR_OK);

  /* if we received synack or reset, destination is reached */
  lcrzo_epr(lcrzo_global_get_language(&language));
  if ( ( hdrltcp.syn && hdrltcp.ack ) || hdrltcp.rst )
  { printf("% 3d : ", theindex1);
    lcrzo_epr(lcrzo_ipl_print("", hdrlip.saddr, "\n"));
    if ( language==LCRZO_GLOBAL_LANG_FR )
      { printf("Destination atteinte.\n");
        if (hdrltcp.rst)
	  printf("Note : le port TCP %d est ferme (on a recu un reset).\n", 
		 serverport);
      }
    else
      { printf("Destination reached.\n");
        if (hdrltcp.rst)
	  printf("Note : the TCP port %d is closed (a reset was received).\n", 
		 serverport);
      }
    lcrzo_epr(lcrzo_process_current_exit(LCRZO_ERR_OK));
  }

  return(LCRZO_ERR_OK);
}
