
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000317=
{ /*reference*/     "lcrzoex", 317,
  /*version*/       1, 0,
  /*title*/         "udp traceroute (true IP, sent at IP level)",
  /*french title*/  "traceroute udp (vrai IP, envoye au niveau IP)",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "dstip dstport maxttl",
  /*french usage*/  "ipdst portdst ttlmax",
  /*usage example*/ "5.6.7.78 53 30",
  /*fr. usage ex.*/ "5.6.7.78 53 30",
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000317_main(int argc, char *argv[]);
int lcrzoex_000317_traceroute(lcrzo_device device,
			      lcrzo_ipl clientipl, 
			      lcrzo_ipl serveripl, 
			      lcrzo_uint16 serverport,
			      lcrzo_uint8 maxttl);
int lcrzoex_000317_child_spoof(lcrzo_ptr infos, 
			       lcrzo_int32 infossize, 
			       lcrzo_process *pparentprocess);
int lcrzoex_000317_child_sniff(lcrzo_ptr infos, 
			       lcrzo_int32 infossize, 
			       lcrzo_process *pparentprocess);
int lcrzoex_000317_printroutei(lcrzo_ipl clientipl, 
			       lcrzo_ipl serveripl, 
			       lcrzo_uint16 serverport,
			       lcrzo_uint16 startingclientport,
			       lcrzo_uint8 maxttl,
			       lcrzo_data sniffdata,
			       lcrzo_int32 sniffdatasize);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000317_main(int argc, char *argv[])
{ lcrzo_device device;
  lcrzo_ipl clientipl, serveripl;
  lcrzo_uint16 serverport;
  lcrzo_uint8 maxttl;

  lcrzo_init();
  if ( argc<4 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000317, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }
  lcrzo_epr(shared_para_ipl_init_dst(argv[1], &serveripl));
  lcrzo_epr(shared_para_port_init_dst(argv[2], 53, &serverport));
  lcrzo_epr(shared_para_uint8_init(argv[3], "Maximum TTL", 1, 255,
                                   30, &maxttl)); 
  if (maxttl==0) maxttl=1;

  lcrzo_epr(lcrzo_conf_route_to_host_ip(serveripl, device, &clientipl, NULL));

  /* call the main function */
  lcrzo_epr(lcrzoex_000317_traceroute(device, 
				      clientipl, serveripl, serverport,
				      maxttl));

  lcrzo_close();
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000317_traceroute(lcrzo_device device,
			      lcrzo_ipl clientipl, 
			      lcrzo_ipl serveripl, 
			      lcrzo_uint16 serverport,
			      lcrzo_uint8 maxttl)
{ shared_sock_address ssa;
  lcrzo_process childspoof, childsniff;
  lcrzo_wait waitspoof, waitsniff;
  lcrzo_bool eventspoof, eventsniff;
  int ret;

  /* set values */
  lcrzo_epr(lcrzo_device_init(device, ssa.device));
  ssa.clientipl=clientipl;
  ssa.serveripl=serveripl;
  ssa.serverport=serverport;
  ssa.ttl=maxttl;

  /* determine starting values */
  lcrzo_epr(lcrzo_uint16_init_rand(5000, 30000, &ssa.clientport2));

  /* this process sends */
  lcrzo_epr(lcrzo_process_other_init(&lcrzoex_000317_child_spoof, 
                                     (lcrzo_data)&ssa, sizeof(ssa), 
				     &childspoof));

  /* this process sniff and prints sequence numbers */
  ret=lcrzo_process_other_init(&lcrzoex_000317_child_sniff, 
			       (lcrzo_data)&ssa, sizeof(ssa),
			       &childsniff);
  if ( ret!=LCRZO_ERR_OK)
  { /* kill the first child */
    lcrzo_epr(shared_lcrzoex_debug_processotherexit_wanttowarn(LCRZO_FALSE));
    lcrzo_epr(lcrzo_process_other_exit(&childspoof));
    lcrzo_epr(shared_lcrzoex_debug_processotherexit_wanttowarn(LCRZO_TRUE));
    lcrzo_epr(lcrzo_process_other_wait_end(&childspoof, LCRZO_WAIT_INFINITE, 
					   NULL, NULL));
    /* exit */
    return(ret);
  }

  /* wait for the end of one process */
  lcrzo_epr(lcrzo_wait_init_process_end(&childspoof, &waitspoof));
  lcrzo_epr(lcrzo_wait_init_process_end(&childsniff, &waitsniff));
  lcrzo_epr(lcrzo_wait_wait2(&waitspoof, &waitsniff, LCRZO_WAIT_INFINITE,
			     &eventspoof, &eventsniff));
  lcrzo_epr(lcrzo_wait_close(&waitspoof));
  lcrzo_epr(lcrzo_wait_close(&waitsniff));

  /* kill childs */
  if (eventspoof)
  { lcrzo_epr(shared_lcrzoex_debug_processotherexit_wanttowarn(LCRZO_FALSE));
    lcrzo_epr(lcrzo_process_other_exit(&childsniff));
    lcrzo_epr(shared_lcrzoex_debug_processotherexit_wanttowarn(LCRZO_TRUE));
    lcrzo_epr(lcrzo_process_other_wait_end(&childsniff, LCRZO_WAIT_INFINITE, 
					   NULL, NULL));
    lcrzo_epr(lcrzo_process_other_wait_end(&childspoof, LCRZO_WAIT_INFINITE, 
					   NULL, &ret));
    lcrzo_epr(lcrzo_process_close(&childspoof));
    lcrzo_epr(lcrzo_process_close(&childsniff));
    return(ret);
  }
  if (eventsniff)
  { lcrzo_epr(shared_lcrzoex_debug_processotherexit_wanttowarn(LCRZO_FALSE));
    lcrzo_epr(lcrzo_process_other_exit(&childspoof));
    lcrzo_epr(shared_lcrzoex_debug_processotherexit_wanttowarn(LCRZO_TRUE));
    lcrzo_epr(lcrzo_process_other_wait_end(&childspoof, LCRZO_WAIT_INFINITE, 
					   NULL, NULL));
    lcrzo_epr(lcrzo_process_other_wait_end(&childsniff, LCRZO_WAIT_INFINITE, 
					   NULL, &ret));
    lcrzo_epr(lcrzo_process_close(&childspoof));
    lcrzo_epr(lcrzo_process_close(&childsniff));
    return(ret);
  }

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000317_child_spoof(lcrzo_ptr infos, 
			       lcrzo_int32 infossize, 
			       lcrzo_process *pparentprocess)
{ shared_sock_address *pssa=(shared_sock_address *)infos;
  lcrzo_hdrlip hdrlip;
  lcrzo_hdrludp hdrludp;
  lcrzo_spoof spoof;
  lcrzo_uint8 i;

  /* check for correct usage of this function */
  if (infossize!=sizeof(shared_sock_address))
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);

  /* close parent process handle because we do not need it */
  lcrzo_epr(lcrzo_process_close(pparentprocess));

  /* initialize IP header */
  lcrzo_epr(lcrzo_hdrlip_initdefault(&hdrlip));
  hdrlip.saddr=pssa->clientipl;
  hdrlip.daddr=pssa->serveripl;
  hdrlip.ttl=1;
  hdrlip.id=pssa->clientport2;

  /* initialize UDP header */
  lcrzo_epr(lcrzo_hdrludp_initdefault(&hdrludp));
  hdrludp.sport=pssa->clientport2;
  hdrludp.dport=pssa->serverport;

  /* little sleep to be sure sniff process is ready (should be done with
     an IPC) */
  lcrzo_epr(lcrzo_time_sleep_msec(300));

  /* spoof */
  lcrzo_epr(lcrzo_spoof_init(&spoof));
  /*lcrzo_epr(lcrzo_spoof_set_printbeforesending(&lcrzoex_000317_spoof,
                                               LCRZO_TRUE));*/
  for ( i=1 ; i<=pssa->ttl ; i++ )
  { lcrzo_epr(lcrzo_spoof_ipoptudpdata(&spoof,
				       hdrlip, NULL, 0,
				       hdrludp, NULL, 0));
    hdrlip.id++;
    hdrlip.ttl++;
    hdrludp.sport++;
    lcrzo_epr(lcrzo_time_sleep_sec(1));
  }
  lcrzo_epr(lcrzo_spoof_close(&spoof));
  
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000317_child_sniff(lcrzo_ptr infos, 
			       lcrzo_int32 infossize, 
			       lcrzo_process *pparentprocess)
{ shared_sock_address *pssa=(shared_sock_address *)infos;
  lcrzo_sniff sniff;
  lcrzo_uint32 mtu;
  lcrzo_data sniffdata;
  lcrzo_int32 sniffdatasize;
  int ret;

  /* check for correct usage of this function */
  if (infossize!=sizeof(shared_sock_address))
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);

  /* close parent process handle because we do not need it */
  lcrzo_epr(lcrzo_process_close(pparentprocess));

  /* init sniff */
  lcrzo_epr(lcrzo_mtu_init_device(pssa->device, &mtu));
  lcrzo_epr(lcrzo_sniff_init(pssa->device, mtu, LCRZO_SNIFF_TYPE_NORMAL, 
			     "", &sniff));

  /* main loop */
  ret=lcrzo_sniff_nextm(&sniff, LCRZO_WAIT_INFINITE, 
			&sniffdata, &sniffdatasize);
  while (ret==LCRZO_ERR_OK)
  { /* decode an icmp error, and print it */
    lcrzo_epr(lcrzoex_000317_printroutei(pssa->clientipl, pssa->serveripl,
					 pssa->serverport,
					 pssa->clientport2, pssa->ttl,
					 sniffdata, sniffdatasize));
    lcrzo_epr(lcrzo_data_free2(&sniffdata));
    /* sniff next packet */
    ret=lcrzo_sniff_nextm(&sniff, LCRZO_WAIT_INFINITE, 
			  &sniffdata, &sniffdatasize);
  }
  lcrzo_epr(lcrzo_sniff_close(&sniff));

  /*never reached*/
  return(ret);
}

/*-------------------------------------------------------------*/
int lcrzoex_000317_printroutei(lcrzo_ipl clientipl, 
			       lcrzo_ipl serveripl, 
			       lcrzo_uint16 serverport,
			       lcrzo_uint16 startingclientport,
			       lcrzo_uint8 maxttl,
			       lcrzo_data sniffdata,
			       lcrzo_int32 sniffdatasize)
{ static int ttlreached=0xFFFF; 
  lcrzo_hdrlip hdrlip, badhdrip;
  lcrzo_data baddata;
  lcrzo_string string;
  lcrzo_uint16 baddatasize, badudpsport, badudpdport;
  lcrzo_uint8 type, code;
  lcrzo_int16 language;
  int ret, indexidok, indexidko, indexsport;

  /*lcrzo_epr(lcrzo_packet_print_eth(sniffdata, sniffdatasize,
                                   LCRZO_PRINTPROFILE_ARRAYDUMP));*/

  /* try to decode icmp */
  ret=lcrzo_packet_decodem_ethipopticmperror(sniffdata, sniffdatasize,
					     NULL, &hdrlip, NULL,NULL,
					     &type, &code, NULL, NULL,
					     &baddata, &baddatasize);
  if ( ret!=LCRZO_ERR_OK )
    return(LCRZO_ERR_OK);

  /* check if the addresses are the one we are looking for */
  if ( hdrlip.daddr!=clientipl )
  { lcrzo_epr(lcrzo_data_free2(&baddata));
    return(LCRZO_ERR_OK);
  }

  /* icmp should contain udp */
  ret=lcrzo_ip64bits_decodem_ipudp(baddata, baddatasize,
				   &badhdrip, &badudpsport, &badudpdport,
				   NULL, NULL);
  lcrzo_epr(lcrzo_data_free2(&baddata));
  if ( ret!=LCRZO_ERR_OK )
    return(LCRZO_ERR_OK);

  /* check various values in baddata */
  if ( badudpdport!=serverport ||
       badhdrip.saddr!=clientipl ||
       badhdrip.daddr!=serveripl )
    return(LCRZO_ERR_OK);

  /* compute indexid from the id of the received packet */
  indexidok=badhdrip.id-startingclientport+1;
  /* this one is because there is a bug in the IP stack of BSD :
     id is reversed :( */
  indexidko= ( (badhdrip.id&0xFF)<<8 ) | ( (badhdrip.id&0xFF00)>>8 );
  indexidko=indexidko-startingclientport+1;
  /* compute indexsport from the sport of the received packet */
  indexsport=badudpsport-startingclientport+1;

  /* verify that indexes are equal */
  if ( indexidok!=indexsport && indexidko!=indexsport )
    return(LCRZO_ERR_OK);

  /* print the result */
  if ( indexsport<ttlreached )
  { printf("% 3d : ", indexsport);
    lcrzo_epr(lcrzo_ipl_print("", hdrlip.saddr, ""));
  
    /* print the type of error */
    if ( type==LCRZO_ICMPTYPE_TIMEEXCEED &&
	 code == LCRZO_ICMPCODE_TIMEEXCEED_TTL )
    { /* do not display the error type and code */
      printf("\n"); 
    }
    else if ( type==LCRZO_ICMPTYPE_DESTUNREACH &&
	      code == LCRZO_ICMPCODE_DESTUNREACH_PORT )
    { /* host reached */
      printf("\n"); 
      ttlreached=indexsport;
      lcrzo_epr(lcrzo_global_get_language(&language));
      if ( language==LCRZO_GLOBAL_LANG_FR )
      { printf("Destination atteinte.\n");
      }
      else
      { printf("Destination reached.\n");
      }
      lcrzo_epr(lcrzo_process_current_exit(LCRZO_ERR_OK));
    }
    else 
    { /* display the error type and code */
      lcrzo_epr(lcrzo_string_initm_icmptypecode(type, code, &string));
      printf(" : %s\n", string);
      lcrzo_epr(lcrzo_string_free2(&string));
    }

    /* if maxttl was too short */
    if ( indexsport == maxttl )
    { lcrzo_epr(lcrzo_global_get_language(&language));
      if ( language==LCRZO_GLOBAL_LANG_FR )
	printf("maxttl(%d) est trop petit pour atteindre la destination\n", 
	       maxttl);
      else
	printf("maxttl(%d) was too short to reach the destination\n", 
	       maxttl);
      return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
    }
  }

  return(LCRZO_ERR_OK);
}








