
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000318=
{ /*reference*/     "lcrzoex", 318,
  /*version*/       1, 0,
  /*title*/         "icmp traceroute (true IP, sent at IP level)",
  /*french title*/  "traceroute icmp (vrai IP, envoye au niveau IP)",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "dstip maxttl",
  /*french usage*/  "ipdst ttlmax",
  /*usage example*/ "5.6.7.78 30",
  /*fr. usage ex.*/ "5.6.7.78 30",
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000318_main(int argc, char *argv[]);
int lcrzoex_000318_traceroute(lcrzo_device device,
			      lcrzo_ipl clientipl, 
			      lcrzo_ipl serveripl, 
			      lcrzo_uint8 maxttl);
int lcrzoex_000318_child_spoof(lcrzo_ptr infos, 
			       lcrzo_int32 infossize, 
			       lcrzo_process *pparentprocess);
int lcrzoex_000318_child_sniff(lcrzo_ptr infos, 
			       lcrzo_int32 infossize, 
			       lcrzo_process *pparentprocess);
int lcrzoex_000318_printroutei(lcrzo_ipl clientipl, 
			       lcrzo_ipl serveripl, 
			       lcrzo_uint16 startingid,
			       lcrzo_uint8 maxttl,
			       lcrzo_data sniffdata,
			       lcrzo_int32 sniffdatasize);
int lcrzoex_000318_printrouter(lcrzo_ipl clientipl, 
			       lcrzo_ipl serveripl, 
			       lcrzo_uint16 startingid,
			       lcrzo_data sniffdata,
			       lcrzo_int32 sniffdatasize);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000318_main(int argc, char *argv[])
{ lcrzo_device device;
  lcrzo_ipl clientipl, serveripl;
  lcrzo_uint8 maxttl;

  lcrzo_init();
  if ( argc<3 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000318, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }
  lcrzo_epr(shared_para_ipl_init_dst(argv[1], &serveripl));
  lcrzo_epr(shared_para_uint8_init(argv[2], "Maximum TTL", 1, 255,
                                   30, &maxttl)); 
  if (maxttl==0) maxttl=1;

  lcrzo_epr(lcrzo_conf_route_to_host_ip(serveripl, device, &clientipl, NULL));

  /* call the main function */
  lcrzo_epr(lcrzoex_000318_traceroute(device, 
				      clientipl, serveripl,
				      maxttl));

  lcrzo_close();
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000318_traceroute(lcrzo_device device,
			      lcrzo_ipl clientipl, 
			      lcrzo_ipl serveripl, 
			      lcrzo_uint8 maxttl)
{ shared_sock_address ssa;
  lcrzo_process childspoof, childsniff;
  lcrzo_wait waitspoof, waitsniff;
  lcrzo_bool eventspoof, eventsniff;
  int ret;

  /* set values */
  lcrzo_epr(lcrzo_device_init(device, ssa.device));
  ssa.clientipl=clientipl;
  ssa.serveripl=serveripl;
  ssa.ttl=maxttl;

  /* determine starting values */
  lcrzo_epr(lcrzo_uint16_init_rand(5000, 30000, &ssa.clientport2));

  /* this process sends */
  lcrzo_epr(lcrzo_process_other_init(&lcrzoex_000318_child_spoof, 
                                     (lcrzo_data)&ssa, sizeof(ssa), 
				     &childspoof));

  /* this process sniff and prints sequence numbers */
  ret=lcrzo_process_other_init(&lcrzoex_000318_child_sniff, 
			       (lcrzo_data)&ssa, sizeof(ssa),
			       &childsniff);
  if ( ret!=LCRZO_ERR_OK)
  { /* kill the first child */
    lcrzo_epr(shared_lcrzoex_debug_processotherexit_wanttowarn(LCRZO_FALSE));
    lcrzo_epr(lcrzo_process_other_exit(&childspoof));
    lcrzo_epr(shared_lcrzoex_debug_processotherexit_wanttowarn(LCRZO_TRUE));
    lcrzo_epr(lcrzo_process_other_wait_end(&childspoof, LCRZO_WAIT_INFINITE, 
					   NULL, NULL));
    /* exit */
    return(ret);
  }

  /* wait for the end of one process */
  lcrzo_epr(lcrzo_wait_init_process_end(&childspoof, &waitspoof));
  lcrzo_epr(lcrzo_wait_init_process_end(&childsniff, &waitsniff));
  lcrzo_epr(lcrzo_wait_wait2(&waitspoof, &waitsniff, LCRZO_WAIT_INFINITE,
			     &eventspoof, &eventsniff));
  lcrzo_epr(lcrzo_wait_close(&waitspoof));
  lcrzo_epr(lcrzo_wait_close(&waitsniff));

  /* kill childs */
  if (eventspoof)
  { lcrzo_epr(shared_lcrzoex_debug_processotherexit_wanttowarn(LCRZO_FALSE));
    lcrzo_epr(lcrzo_process_other_exit(&childsniff));
    lcrzo_epr(shared_lcrzoex_debug_processotherexit_wanttowarn(LCRZO_TRUE));
    lcrzo_epr(lcrzo_process_other_wait_end(&childsniff, LCRZO_WAIT_INFINITE, 
					   NULL, NULL));
    lcrzo_epr(lcrzo_process_other_wait_end(&childspoof, LCRZO_WAIT_INFINITE, 
					   NULL, &ret));
    lcrzo_epr(lcrzo_process_close(&childspoof));
    lcrzo_epr(lcrzo_process_close(&childsniff));
    return(ret);
  }
  if (eventsniff)
  { lcrzo_epr(shared_lcrzoex_debug_processotherexit_wanttowarn(LCRZO_FALSE));
    lcrzo_epr(lcrzo_process_other_exit(&childspoof));
    lcrzo_epr(shared_lcrzoex_debug_processotherexit_wanttowarn(LCRZO_TRUE));
    lcrzo_epr(lcrzo_process_other_wait_end(&childspoof, LCRZO_WAIT_INFINITE, 
					   NULL, NULL));
    lcrzo_epr(lcrzo_process_other_wait_end(&childsniff, LCRZO_WAIT_INFINITE, 
					   NULL, &ret));
    lcrzo_epr(lcrzo_process_close(&childspoof));
    lcrzo_epr(lcrzo_process_close(&childsniff));
    return(ret);
  }

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000318_child_spoof(lcrzo_ptr infos, 
			       lcrzo_int32 infossize, 
			       lcrzo_process *pparentprocess)
{ shared_sock_address *pssa=(shared_sock_address *)infos;
  lcrzo_hdrlip hdrlip;
  lcrzo_spoof spoof;
  lcrzo_data data;
  lcrzo_int32 datasize;
  lcrzo_uint8 i;

  /* check for correct usage of this function */
  if (infossize!=sizeof(shared_sock_address))
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);

  /* close parent process handle because we do not need it */
  lcrzo_epr(lcrzo_process_close(pparentprocess));

  /* initialize IP header */
  lcrzo_epr(lcrzo_hdrlip_initdefault(&hdrlip));
  hdrlip.saddr=pssa->clientipl;
  hdrlip.daddr=pssa->serveripl;
  hdrlip.ttl=1;
  hdrlip.id=pssa->clientport2;

  /* little sleep to be sure sniff process is ready (should be done with
     an IPC) */
  lcrzo_epr(lcrzo_time_sleep_msec(300));

  /* spoof */
  lcrzo_epr(lcrzo_spoof_init(&spoof));
  /*lcrzo_epr(lcrzo_spoof_set_printbeforesending(&lcrzoex_000318_spoof,
                                               LCRZO_TRUE));*/
  for ( i=1 ; i<=pssa->ttl ; i++ )
  { lcrzo_epr(lcrzo_packet_initm_ipopticmpecho(hdrlip, NULL, 0,
					       LCRZO_ICMPTYPE_ECHOREQ,
					       hdrlip.id, hdrlip.id,
					       (lcrzo_constdata)"ICMP traceroute", 15,
					       &data, &datasize));
    lcrzo_epr(lcrzo_spoof_ip(&spoof, data, datasize));
    lcrzo_epr(lcrzo_data_free2(&data));
    hdrlip.id++;
    hdrlip.ttl++;
    lcrzo_epr(lcrzo_time_sleep_sec(1));
  }
  lcrzo_epr(lcrzo_spoof_close(&spoof));
  
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000318_child_sniff(lcrzo_ptr infos, 
			       lcrzo_int32 infossize, 
			       lcrzo_process *pparentprocess)
{ shared_sock_address *pssa=(shared_sock_address *)infos;
  lcrzo_sniff sniff;
  lcrzo_uint32 mtu;
  lcrzo_data sniffdata;
  lcrzo_int32 sniffdatasize;
  int ret;

  /* check for correct usage of this function */
  if (infossize!=sizeof(shared_sock_address))
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);

  /* close parent process handle because we do not need it */
  lcrzo_epr(lcrzo_process_close(pparentprocess));

  /* init sniff */
  lcrzo_epr(lcrzo_mtu_init_device(pssa->device, &mtu));
  lcrzo_epr(lcrzo_sniff_init(pssa->device, mtu, LCRZO_SNIFF_TYPE_NORMAL, "",
			     &sniff));

  /* main loop */
  ret=lcrzo_sniff_nextm(&sniff, LCRZO_WAIT_INFINITE, 
			&sniffdata, &sniffdatasize);
  while (ret==LCRZO_ERR_OK)
  { /* decode an icmp error, and print it */
    lcrzo_epr(lcrzoex_000318_printroutei(pssa->clientipl, pssa->serveripl, 
					 pssa->clientport2,
					 pssa->ttl, sniffdata, sniffdatasize));
    /* decode an icmp echo reply, and print it */
    lcrzo_epr(lcrzoex_000318_printrouter(pssa->clientipl, pssa->serveripl, 
					 pssa->clientport2,
					 sniffdata, sniffdatasize));
    lcrzo_epr(lcrzo_data_free2(&sniffdata));
    /* sniff next packet */
    ret=lcrzo_sniff_nextm(&sniff, LCRZO_WAIT_INFINITE, 
			  &sniffdata, &sniffdatasize);
  }
  lcrzo_epr(lcrzo_sniff_close(&sniff));

  return(ret);
}

/*-------------------------------------------------------------*/
int lcrzoex_000318_printroutei(lcrzo_ipl clientipl, 
			       lcrzo_ipl serveripl, 
			       lcrzo_uint16 startingid,
			       lcrzo_uint8 maxttl,
			       lcrzo_data sniffdata,
			       lcrzo_int32 sniffdatasize)
{ lcrzo_hdrlip hdrlip, badhdrip;
  lcrzo_data baddata;
  lcrzo_string string;
  lcrzo_uint16 baddatasize;
  lcrzo_uint8 type, code, badicmptype, badicmpcode, b4, b5, b6, b7;
  lcrzo_int16 language;
  int ret, indexidok, indexidko, indexechoid, indexechoseqnum;

  /*lcrzo_epr(lcrzo_packet_print_eth(sniffdata, sniffdatasize,
				   LCRZO_PRINTPROFILE_ARRAYDUMP));*/

  /* try to decode icmp */
  ret=lcrzo_packet_decodem_ethipopticmperror(sniffdata, sniffdatasize,
					     NULL, &hdrlip, NULL,NULL,
					     &type, &code, NULL, NULL,
					     &baddata, &baddatasize);
  if ( ret!=LCRZO_ERR_OK )
    return(LCRZO_ERR_OK);

  /* check if the addresses are the one we are looking for */
  if ( hdrlip.daddr!=clientipl )
  { lcrzo_epr(lcrzo_data_free2(&baddata));
    return(LCRZO_ERR_OK);
  }

  /* icmp should contain icmp */
  ret=lcrzo_ip64bits_decodem_ipicmp(baddata, baddatasize,
				    &badhdrip, &badicmptype, &badicmpcode,
				    NULL, &b4, &b5, &b6, &b7);
  lcrzo_epr(lcrzo_data_free2(&baddata));
  if ( ret!=LCRZO_ERR_OK )
    return(LCRZO_ERR_OK);

  /* check various values in baddata */
  if ( badicmptype!=LCRZO_ICMPTYPE_ECHOREQ ||
       badicmpcode!=0 ||
       badhdrip.saddr!=clientipl ||
       badhdrip.daddr!=serveripl )
    return(LCRZO_ERR_OK);

  /* compute indexid from the id of the received packet */
  indexidok=badhdrip.id-startingid+1;
  /* this one is because there is a bug in the IP stack of BSD :
     id is reversed :( */
  indexidko= ( (badhdrip.id&0xFF)<<8 ) | ( (badhdrip.id&0xFF00)>>8 );
  indexidko=indexidko-startingid+1;
  /* compute indexechoid from the id of the received packet */
  indexechoid=((b4<<8)|b5)-startingid+1;
  /* compute indexechoseqnum from the seqnum of the received packet */
  indexechoseqnum=((b6<<8)|b7)-startingid+1;

  /* verify that indexes are equal */
  if ( (indexidok!=indexechoid&&indexidko!=indexechoid) ||
       indexechoid!=indexechoseqnum )
    return(LCRZO_ERR_OK);

  /* print the result */
  printf("% 3d : ", indexechoid);
  lcrzo_epr(lcrzo_ipl_print("", hdrlip.saddr, ""));
      
  /* print the type of error */
  if ( type==LCRZO_ICMPTYPE_TIMEEXCEED &&
       code == LCRZO_ICMPCODE_TIMEEXCEED_TTL )
  { /* do not display the error type and code */
    printf("\n"); 
  }
  else 
  { /* display the error type and code */
    lcrzo_epr(lcrzo_string_initm_icmptypecode(type, code, &string));
    printf(" : %s\n", string);
    lcrzo_epr(lcrzo_string_free2(&string));
  }

  /* if maxttl was too short */
  if ( indexechoid == maxttl )
  { lcrzo_epr(lcrzo_global_get_language(&language));
    if ( language==LCRZO_GLOBAL_LANG_FR )
      printf("maxttl(%d) est trop petit pour atteindre la destination\n", 
	     maxttl);
    else
      printf("maxttl(%d) was too short to reach the destination\n", 
	     maxttl);
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000318_printrouter(lcrzo_ipl clientipl, 
			       lcrzo_ipl serveripl, 
			       lcrzo_uint16 startingid,
			       lcrzo_data sniffdata,
			       lcrzo_int32 sniffdatasize)
{ static lcrzo_bool destreached=LCRZO_FALSE;
  lcrzo_hdrlip hdrlip;
  lcrzo_uint8 type;
  lcrzo_uint16 id, seqnum;
  lcrzo_int16 language;
  int ret, theindex1, theindex2;

  /*lcrzo_epr(lcrzo_packet_print_eth(sniffdata, sniffdatasize,
    LCRZO_PRINTPROFILE_ARRAYDUMP));*/

  /* try to decode it */
  ret=lcrzo_packet_decodem_ethipopticmpecho(sniffdata, sniffdatasize,
					    NULL, &hdrlip, NULL, NULL,
					    &type, &id, &seqnum, NULL, NULL);
  if ( ret!=LCRZO_ERR_OK )
    return(LCRZO_ERR_OK);

  /* check if the addresses are the one we are looking for */
  if ( hdrlip.saddr!=serveripl || hdrlip.daddr!=clientipl )
    return(LCRZO_ERR_OK);

  /* check type */
  if ( type!=LCRZO_ICMPTYPE_ECHOREP )
    return(LCRZO_ERR_OK);

  /* compute the theindex from the received packet */
  theindex1=id-startingid+1;
  theindex2=seqnum-startingid+1;
  if ( theindex1!=theindex2 )
    return(LCRZO_ERR_OK);

  /* destination is reached */
  lcrzo_epr(lcrzo_global_get_language(&language));
  if ( !destreached )
  { destreached=LCRZO_TRUE;
    printf("% 3d : ", theindex1);
    lcrzo_epr(lcrzo_ipl_print("", hdrlip.saddr, "\n"));
    if ( language==LCRZO_GLOBAL_LANG_FR )
    { printf("Destination atteinte.\n");
    }
    else
    { printf("Destination reached.\n");
    }
    lcrzo_epr(lcrzo_process_current_exit(LCRZO_ERR_OK));
  }

  return(LCRZO_ERR_OK);
}
