
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000325=
{ /*reference*/     "lcrzoex", 325,
  /*version*/       1, 0,
  /*title*/         "dump a binary file in a record (permit to edit it)",
  /*french title*/  "dumpe un fichier binaire dans un record (permet de l'editer)",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "binaryfile recordfile",
  /*french usage*/  "fichierbinaire fichierrecord",
  /*usage example*/ "file textrecord",
  /*fr. usage ex.*/ "fichier recordtexte",
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000325_main(int argc, char *argv[]);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000325_main(int argc, char *argv[])
{ lcrzo_file inputfile;
  lcrzo_record outputrecord;
  lcrzo_string fin, fout;
  lcrzo_data data, pstart;
  lcrzo_int32 datasize, chuncksize;
  int ret;

  lcrzo_init();
  if ( argc<3 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000325, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }
 
  lcrzo_epr(shared_para_string_init_filein(argv[1], &fin));
  lcrzo_epr(shared_para_string_init_fileout(argv[2], &fout));

  /* open the file and the record */
  lcrzo_epr(lcrzo_file_open_read(fin, &inputfile));
  lcrzo_epr(lcrzo_record_open(&outputrecord, fout, 
			      LCRZO_RECORD_OPEN_WRITE));

  /* convert file content */
  ret=LCRZO_ERR_OK;
  while (ret==LCRZO_ERR_OK)
  { /* read from fin */
    ret=lcrzo_file_readm(&inputfile, LCRZO_WAIT_INFINITE,
			       &data, &datasize);
    if (ret==LCRZO_ERR_OK)
    { pstart=data;
      while(datasize)
      { if (datasize>0xFFF) chuncksize=0xFFF;
        else chuncksize=datasize;
        ret=lcrzo_record_write_data(&outputrecord, pstart, 
				    chuncksize, LCRZO_PRINTTYPE_CHMS);
        pstart+=chuncksize;
        datasize-=chuncksize;
      }
      lcrzo_epr(lcrzo_data_free2(&data));
    }
  }
  if (ret==LCRZO_ERR_OKDATAEND) ret=LCRZO_ERR_OK;
      
  /* close files */
  lcrzo_epr(lcrzo_file_close(&inputfile));
  lcrzo_epr(lcrzo_record_close(&outputrecord));
  lcrzo_epr(lcrzo_string_free2(&fin));
  lcrzo_epr(lcrzo_string_free2(&fout));

  lcrzo_close();
  return(ret);
}


