
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000328=
{ /*reference*/     "lcrzoex", 328,
  /*version*/       1, 0,
  /*title*/         "{ETH,IP,UDP} spoof of a DNS question",
  /*french title*/  "spoof {ETH,IP,UDP} d'une question DNS",
  /*description*/   "type can be : a, ns, soa, ptr, etc.",
  /*french desc.*/  "type peut etre : a, ns, soa, ptr, etc.",
  /*usage*/         "device src_eth dst_eth ip_source_address dnsserver source_port query type",
  /*french usage*/  "device eth_src eth_dst adresse_ip_source serveurdns port_source question type",
  /*usage example*/ "eth0 aa:bb:cc:dd:ee:ff 11:22:33:44:55:66 1.2.3.4 5.6.7.8 1234 www.microsoft.com a",
  /*fr. usage ex.*/ "eth0 aa:bb:cc:dd:ee:ff 11:22:33:44:55:66 1.2.3.4 5.6.7.8 1234 www.microsoft.com a", 
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000328_main(int argc, char *argv[]);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000328_main(int argc, char *argv[])
{ lcrzo_spoof spoof;
  lcrzo_device device;
  lcrzo_hdrleth hdrleth;
  lcrzo_hdrlip hdrlip;
  lcrzo_hdrludp hdrludp;
  lcrzo_data data;
  lcrzo_int32 datasize;

  lcrzo_init();
  if ( argc<9 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000328, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }

  /* initialize device */
  lcrzo_epr(shared_para_device_init_src(argv[1], device));

  /* initialize Ethernet header */
  lcrzo_epr(lcrzo_hdrleth_initdefault(&hdrleth));
  lcrzo_epr(shared_para_etha_init_src(argv[2], hdrleth.src));
  lcrzo_epr(shared_para_etha_init_dst(argv[3], hdrleth.dst));

  /* initialize IP header */
  lcrzo_epr(lcrzo_hdrlip_initdefault(&hdrlip));
  lcrzo_epr(shared_para_ipl_init_src(argv[4], &(hdrlip.saddr)));
  lcrzo_epr(shared_para_ipl_init_dst(argv[5], &(hdrlip.daddr)));

  /* initialize UDP header */
  lcrzo_epr(lcrzo_hdrludp_initdefault(&hdrludp));
  lcrzo_epr(shared_para_port_init_src(argv[6], &hdrludp.sport));
  hdrludp.dport=53;
 
  /* create the packet */
  lcrzo_epr(shared_dns_packet_quest_initm(argv[7], argv[8],
					  &data, &datasize));

  /* spoof */
  lcrzo_epr(lcrzo_spoof_init(&spoof));
  lcrzo_epr(lcrzo_spoof_ethipoptudpdata(&spoof, device, hdrleth, 
					hdrlip, NULL, 0,
					hdrludp, 
					data, (lcrzo_uint16)datasize));
  lcrzo_epr(lcrzo_spoof_close(&spoof));

  /* free */
  lcrzo_epr(lcrzo_data_free2(&data));
  lcrzo_close();
  return(LCRZO_ERR_OK);
}





