
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000341=
{ /*reference*/     "lcrzoex", 341,
  /*version*/       1, 0,
  /*title*/         "query a DNS server",
  /*french title*/  "interroge un serveur DNS",
  /*description*/   "type can be : a, ns, soa, ptr, etc.",
  /*french desc.*/  "type peut etre : a, ns, soa, ptr, etc.",
  /*usage*/         "dnsserver query [type]",
  /*french usage*/  "serveurdns question [type]",
  /*usage example*/ "1.2.3.4 www.microsoft.com a",
  /*fr. usage ex.*/ "1.2.3.4 www.microsoft.com a", 
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000341_main(int argc, char *argv[]);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000341_main(int argc, char *argv[])
{ lcrzo_data data;
  lcrzo_int32 datasize;
  lcrzo_ipl ipl;
  lcrzo_uint16 port;
  lcrzo_sock sock;
  lcrzo_time maxwaitingtime;
  lcrzo_string rrtype;
  int ret;

  lcrzo_init();
  if ( argc<3 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000341, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }

  /* connect */
  lcrzo_epr(shared_para_ipl_init_dst(argv[1], &ipl));
  port=53;
  lcrzo_epr(lcrzo_sock_udpcli_real(ipl, port, &sock));

  /* send the question */
  if ( argc<4 )
  { lcrzo_epr(lcrzo_string_initm_text("all", &rrtype));
  }
  else
  { lcrzo_epr(lcrzo_string_initm_text(argv[3], &rrtype));
  }
  lcrzo_epr(shared_dns_packet_quest_initm(argv[2], rrtype,
					  &data, &datasize));
  lcrzo_epr(lcrzo_string_free2(&rrtype));
  lcrzo_epr(lcrzo_sock_write(&sock, data, datasize));
  lcrzo_epr(shared_dns_print(data, datasize, LCRZO_PRINTTYPE_ARRAY));
  lcrzo_epr(lcrzo_data_free2(&data));

  /* wait the answer */
  lcrzo_epr(lcrzo_time_init_sec(5, &maxwaitingtime));
  ret=lcrzo_sock_readm(&sock, &maxwaitingtime, &data, &datasize);
  if (ret==LCRZO_ERR_OK)
  { ret=shared_dns_print(data, datasize, LCRZO_PRINTTYPE_ARRAY);
    if (ret!=LCRZO_ERR_OK)
    { printf("Could not decode the answer :\n");
      lcrzo_epr(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_DUMP));
    }
    lcrzo_epr(lcrzo_data_free2(&data));
  }
  else if (ret==LCRZO_ERR_OKTEMPDATAEND)
  { puts("Timeout");
  }
  else
  { lcrzo_epr(lcrzo_err_print(ret));
  }
  lcrzo_epr(lcrzo_sock_close(&sock));

  /* free */
  lcrzo_close();
  return(LCRZO_ERR_OK);
}





