
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000343=
{ /*reference*/     "lcrzoex", 343,
  /*version*/       1, 0,
  /*title*/         "zone transfer of a DNS server using TCP",
  /*french title*/  "transfert de zone d'un serveur DNS en employant TCP",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "dnsserver zone",
  /*french usage*/  "serveurdns zone",
  /*usage example*/ "1.2.3.4 server.com",
  /*fr. usage ex.*/ "1.2.3.4 server.com", 
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000343_main(int argc, char *argv[]);
int lcrzoex_000343_dns_print(lcrzo_data packet,
			     lcrzo_int32 packetsize, 
			     lcrzo_bool displayheader);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000343_main(int argc, char *argv[])
{ lcrzo_data data, twobytes;
  lcrzo_int32 datasize;
  lcrzo_ipl ipl;
  lcrzo_uint16 port;
  lcrzo_sock sock;
  lcrzo_time maxwaitingtime;
  int ret, i;

  lcrzo_init();
  if ( argc<3 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000343, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }

  /* connect */
  lcrzo_epr(shared_para_ipl_init_dst(argv[1], &ipl));
  port=53;
  lcrzo_epr(lcrzo_sock_tcpcli_real(ipl, port, &sock));

  /* send the question */
  lcrzo_epr(shared_dns_packet_quest_initm(argv[2], "axfr",
					  &data, &datasize));
  lcrzo_epr(lcrzo_data_alloc(2, &twobytes));
  twobytes[0]=(lcrzo_uint8)((datasize>>8)&0xFF);
  twobytes[1]=(lcrzo_uint8)(datasize&0xFF);
  lcrzo_epr(lcrzo_sock_write(&sock, twobytes, 2));
  lcrzo_epr(lcrzo_data_free2(&twobytes));
  lcrzo_epr(lcrzo_sock_write(&sock, data, datasize));
  lcrzo_epr(shared_dns_print(data, datasize, LCRZO_PRINTTYPE_ARRAY));
  lcrzo_epr(lcrzo_data_free2(&data));

  i=0;
  while (LCRZO_TRUE)
  { i++;

    /* wait the answer size */
    lcrzo_epr(lcrzo_time_init_msec(200, &maxwaitingtime));
    ret=lcrzo_sock_readm_fixed(&sock, &maxwaitingtime, 2, &twobytes);
    if (ret!=LCRZO_ERR_OK)
    { if (ret==LCRZO_ERR_OKTEMPDATAEND)
      { break;
      }
      else
      { lcrzo_epr(lcrzo_err_print(ret));
        return(ret);
      }
    }
    datasize= (twobytes[0]<<8) | twobytes[1];
    lcrzo_epr(lcrzo_data_free2(&twobytes));
    
    /* wait the answer */
    lcrzo_epr(lcrzo_time_init_sec(4, &maxwaitingtime));
    ret=lcrzo_sock_readm_fixed(&sock, &maxwaitingtime, datasize, &data);
    if (ret!=LCRZO_ERR_OK)
    { if (ret==LCRZO_ERR_OKTEMPDATAEND)
      { puts("Timeout");
      }
      else
      { lcrzo_epr(lcrzo_err_print(ret));
      }
      return(ret);
    }

    /* decode it */
    ret=lcrzoex_000343_dns_print(data, datasize, (lcrzo_bool)(i==1));
    if (ret!=LCRZO_ERR_OK)
    { printf("Could not decode the answer :\n");
      lcrzo_epr(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_DUMP));
    }
    lcrzo_epr(lcrzo_data_free2(&data));

  }
  printf(" |___________________________________");
  printf("____________________________________|\n");

  /* free */
  lcrzo_epr(lcrzo_sock_close(&sock));
  lcrzo_close();
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000343_dns_print(lcrzo_data packet,
			     lcrzo_int32 packetsize, 
			     lcrzo_bool displayheader)
{ shared_dns_header dnsheader;
  shared_dns_rr rr;
  lcrzo_list rrlist;
  lcrzo_int32 count, i;

  lcrzo_er(shared_dns_packet_decode(packet, packetsize,
				    &dnsheader, &rrlist));

  if (displayheader)
  { lcrzo_er(shared_dns_header_print(dnsheader, LCRZO_PRINTTYPE_ARRAY));
  }

  lcrzo_er(lcrzo_list_count(&rrlist, &count));
  for ( i=1 ; i<=count ; i++ )
  { lcrzo_er(shared_dns_rrlist_value_pos(&rrlist, i, &rr));
    lcrzo_er(shared_dns_rr_print(rr, LCRZO_PRINTTYPE_ARRAY));
    lcrzo_er(shared_dns_rr_close(&rr));
  }

  lcrzo_er(lcrzo_list_close(&rrlist));
  return(LCRZO_ERR_OK);
}


