
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000344=
{ /*reference*/     "lcrzoex", 344,
  /*version*/       1, 0,
  /*title*/         "obtain the version of a Bind DNS server",
  /*french title*/  "obtient la version d'un serveur DNS Bind",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "dnsserver",
  /*french usage*/  "serveurdns",
  /*usage example*/ "1.2.3.4",
  /*fr. usage ex.*/ "1.2.3.4", 
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000344_main(int argc, char *argv[]);
int lcrzoex_000344_dns_print(lcrzo_data packet,
                             lcrzo_int32 packetsize);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000344_main(int argc, char *argv[])
{ lcrzo_data data;
  lcrzo_int32 datasize;
  lcrzo_ipl ipl;
  lcrzo_uint16 port;
  lcrzo_sock sock;
  lcrzo_time maxwaitingtime;
  shared_dns_header dnsheader;
  shared_dns_rr rr;
  lcrzo_list rrlist;
  int ret;

  lcrzo_init();
  if ( argc<2 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000344, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }

  /* connect */
  lcrzo_epr(shared_para_ipl_init_dst(argv[1], &ipl));
  port=53;
  lcrzo_epr(lcrzo_sock_udpcli_real(ipl, port, &sock));

  /* prepare the question */
  /* init header */
  lcrzo_epr(shared_dns_header_initdefault(&dnsheader));
  dnsheader.isaresponse=LCRZO_FALSE;
  /* init rr */
  lcrzo_epr(shared_dns_rr_init_quest_unknown("version.bind",
					     SHARED_DNS_RR_TYPE_TXT,
					     SHARED_DNS_RR_CLASS_CH,
					     &rr));
  lcrzo_epr(shared_dns_rrlist_init(&rrlist));
  lcrzo_epr(shared_dns_rrlist_add_pos(&rrlist, +1, rr));
  lcrzo_epr(shared_dns_rr_close(&rr));
  /* init packet */
  lcrzo_epr(shared_dns_packet_initm(dnsheader, &rrlist, &data, &datasize));
  lcrzo_epr(lcrzo_list_close(&rrlist));

  /* send the question */
  lcrzo_epr(lcrzo_sock_write(&sock, data, datasize));
  /*shared_dns_print(data, datasize, LCRZO_PRINTTYPE_ARRAY);*/
  lcrzo_er(lcrzo_data_free2(&data));

  /* wait the answer */
  lcrzo_epr(lcrzo_time_init_sec(5, &maxwaitingtime));
  ret=lcrzo_sock_readm(&sock, &maxwaitingtime, &data, &datasize);
  if (ret==LCRZO_ERR_OK)
  { ret=lcrzoex_000344_dns_print(data, datasize);
    if (ret!=LCRZO_ERR_OK)
    { printf("Could not decode the answer :\n");
      lcrzo_epr(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_DUMP));
    }
    lcrzo_er(lcrzo_data_free2(&data));
  }
  else if (ret==LCRZO_ERR_OKTEMPDATAEND)
  { puts("Timeout");
  }
  else
  { lcrzo_epr(lcrzo_err_print(ret));
  }
  lcrzo_epr(lcrzo_sock_close(&sock));

  /* free */
  lcrzo_close();
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000344_dns_print(lcrzo_data packet,
                             lcrzo_int32 packetsize)
{ shared_dns_header dnsheader;
  shared_dns_rr rr;
  lcrzo_list rrlist;
  lcrzo_string str;
  lcrzo_int32 count, i;
  int ret;

  lcrzo_er(shared_dns_packet_decode(packet, packetsize,
                                    &dnsheader, &rrlist));

  lcrzo_er(lcrzo_list_count(&rrlist, &count));
  for ( i=1 ; i<=count ; i++ )
  { lcrzo_er(shared_dns_rrlist_value_pos(&rrlist, i, &rr));
    if ( rr.rrclass==SHARED_DNS_RR_CLASS_CH &&
	 rr.type==SHARED_DNS_RR_TYPE_TXT &&
	 rr.rdatasize )
    { /*lcrzo_er(shared_dns_rr_print(rr, LCRZO_PRINTTYPE_ARRAY));*/
      ret=shared_dns_rdata_decode_txt(rr.rdata, rr.rdatasize, &str);
      if (ret==LCRZO_ERR_OK)
      { printf("%s\n", str);
        lcrzo_er(lcrzo_string_free2(&str));
      }
    }
    lcrzo_er(shared_dns_rr_close(&rr));
  }

  lcrzo_er(lcrzo_list_close(&rrlist));
  return(LCRZO_ERR_OK);
}
