
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000348=
{ /*reference*/     "lcrzoex", 348,
  /*version*/       1, 0,
  /*title*/         "DNS server always answering FAILURE (virtual server)",
  /*french title*/  "serveur DNS repondant toujours FAILURE (serveur virtuel)",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "device serveth servname",
  /*french usage*/  "device ethserv nomserv",
  /*usage example*/ "eth0 aa:bb:cc:dd:ee:ff 1.2.3.4",
  /*fr. usage ex.*/ "eth0 aa:bb:cc:dd:ee:ff 1.2.3.4", 
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000348_main(int argc, char *argv[]);
int lcrzoex_000345_dns_answer_fail(lcrzo_sock *psock,
				   lcrzo_constdata data,
				   lcrzo_int32 datasize);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000348_main(int argc, char *argv[])
{ lcrzo_etha servereth;
  lcrzo_ipl serveripl;
  lcrzo_sock sock;
  lcrzo_device device;
  lcrzo_data data;
  lcrzo_int32 datasize;
  int ret;

  lcrzo_init();
  if ( argc<4 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000348, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }

  lcrzo_epr(shared_para_device_init_listen(argv[1], device));

  /* initialize Ethernet addresses */
  lcrzo_epr(shared_para_etha_init_listen(argv[2], servereth));

  /* initialize IP addresses */
  lcrzo_epr(shared_para_ipl_init_listen(argv[3], &serveripl));

  /* initialize the socket */
  lcrzo_epr(lcrzo_sock_udpmulser_virt(device,
                                      servereth, serveripl, 53,
                                      &sock));

  /* treat every packet */
  ret=LCRZO_ERR_OK;
  while (LCRZO_TRUE)
  { ret=lcrzo_sock_readm(&sock, LCRZO_WAIT_INFINITE, &data, &datasize);
    if (ret!=LCRZO_ERR_OK) break;
    ret=lcrzoex_000345_dns_answer_fail(&sock, data, datasize);
    lcrzo_er(lcrzo_data_free2(&data));
    if (ret!=LCRZO_ERR_OK) { lcrzo_epr(lcrzo_err_print(ret)); }
  }
  lcrzo_epr(lcrzo_err_print(ret));

  /* free */
  lcrzo_epr(lcrzo_sock_close(&sock));
  lcrzo_close();
  return(LCRZO_ERR_OK);
}



