
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000349=
{ /*reference*/     "lcrzoex", 349,
  /*version*/       1, 0,
  /*title*/         "DNS server always answering the same values",
  /*french title*/  "serveur DNS repondant toujours les memes valeurs",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "dnsserverip dnsservername hostname ipaddress",
  /*french usage*/  "ipserveurdns nomserveurdns nommachine ipmachine",
  /*usage example*/ "1.2.3.4 ns.s.com www.s.com 1.2.3.5",
  /*fr. usage ex.*/ "1.2.3.4 ns.s.com www.s.com 1.2.3.5", 
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000349_main(int argc, char *argv[]);
int lcrzoex_000349_dns_answer(lcrzo_sock *psock,
			      lcrzo_ipl dnsserverip,
			      lcrzo_consths dnsservername,
			      lcrzo_consths thehostname, 
			      lcrzo_ipl ipaddress,
			      lcrzo_constdata data,
			      lcrzo_int32 datasize);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000349_main(int argc, char *argv[])
{ lcrzo_data data;
  lcrzo_int32 datasize;
  lcrzo_sock sock;
  lcrzo_ipl dnsserverip, ipaddress;
  lcrzo_hs dnsservername, thehostname;
  int ret;

  lcrzo_init();
  if ( argc<5 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000349, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }

  lcrzo_epr(lcrzo_ipl_init_hs(argv[1], &dnsserverip));
  lcrzo_epr(lcrzo_hs_init(argv[2], dnsservername));
  lcrzo_epr(lcrzo_hs_init(argv[3], thehostname));
  lcrzo_epr(lcrzo_ipl_init_hs(argv[4], &ipaddress));

  /* listen */
  lcrzo_epr(lcrzo_sock_udpmulser_real(53, &sock));

  /* treat every packet */
  ret=LCRZO_ERR_OK;
  while (LCRZO_TRUE)
  { ret=lcrzo_sock_readm(&sock, LCRZO_WAIT_INFINITE, &data, &datasize);
    if (ret!=LCRZO_ERR_OK) break;
    ret=lcrzoex_000349_dns_answer(&sock, dnsserverip, dnsservername,
				  thehostname, ipaddress, 
				  data, datasize);
    lcrzo_er(lcrzo_data_free2(&data));
    if (ret!=LCRZO_ERR_OK) { lcrzo_epr(lcrzo_err_print(ret)); }
  }
  lcrzo_epr(lcrzo_err_print(ret));

  /* free */
  lcrzo_epr(lcrzo_sock_close(&sock));
  lcrzo_close();
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000349_dns_answer(lcrzo_sock *psock,
			      lcrzo_ipl dnsserverip,
			      lcrzo_consths dnsservername,
			      lcrzo_consths thehostname, 
			      lcrzo_ipl ipaddress,
			      lcrzo_constdata data,
			      lcrzo_int32 datasize)
{ shared_dns_header dnsheader;
  lcrzo_data response;
  lcrzo_list rrlist;
  lcrzo_int32 responsesize;
  shared_dns_rr rrquestion, rr;
  lcrzo_uint32 ttl;
  int ret;

  /* display received data */
  lcrzo_er(shared_dns_print(data, datasize, LCRZO_PRINTTYPE_ARRAY));

  /* preparate for the response */
  lcrzo_er(shared_dns_packet_decode(data, datasize,
                                    &dnsheader, &rrlist));

  /* only answer to questions */
  if (dnsheader.isaresponse)
  { lcrzo_er(lcrzo_list_close(&rrlist));
    return(LCRZO_ERR_OK);
  }
  if ( dnsheader.count_question!=1 && 
       dnsheader.count_answer!=0 && 
       dnsheader.count_authority!=0 && 
       dnsheader.count_additional!=0 )
  { lcrzo_er(lcrzo_list_close(&rrlist));
    return(LCRZO_ERR_OK);
  }

  /* get the first (and only) RR */
  lcrzo_er(shared_dns_rrlist_value_pos(&rrlist, +1, &rrquestion));

  /* ignore other classes */
  if ( rrquestion.rrclass!=SHARED_DNS_RR_CLASS_IN )
  { lcrzo_er(shared_dns_rr_close(&rrquestion));
    lcrzo_er(lcrzo_list_close(&rrlist));
    return(LCRZO_ERR_OK);
  }

  /* prepare new header */
  dnsheader.isaresponse=LCRZO_TRUE;
  dnsheader.rcode=SHARED_DNS_HEADER_RCODE_OK;

  /* now add reponses in the list */
  ttl=60;
  if ( rrquestion.type==SHARED_DNS_RR_TYPE_A )
  { lcrzo_er(shared_dns_rr_init_resp_a(rrquestion.name, ttl, ipaddress,
				       SHARED_DNS_RR_KIND_ANSWER, &rr));
    lcrzo_er(shared_dns_rrlist_add_pos(&rrlist, -1, rr));
    lcrzo_er(shared_dns_rr_close(&rr));
    lcrzo_er(shared_dns_rr_init_resp_ns(rrquestion.name, ttl, dnsservername,
					SHARED_DNS_RR_KIND_AUTHORITY, &rr));
    lcrzo_er(shared_dns_rrlist_add_pos(&rrlist, -1, rr));
    lcrzo_er(shared_dns_rr_close(&rr));
    lcrzo_er(shared_dns_rr_init_resp_a(dnsservername, ttl, dnsserverip,
				       SHARED_DNS_RR_KIND_ADDITIONAL, &rr));
    lcrzo_er(shared_dns_rrlist_add_pos(&rrlist, -1, rr));
    lcrzo_er(shared_dns_rr_close(&rr));
  }
  else if ( rrquestion.type==SHARED_DNS_RR_TYPE_NS )
  { lcrzo_er(shared_dns_rr_init_resp_ns(rrquestion.name, ttl, dnsservername,
					SHARED_DNS_RR_KIND_ANSWER, &rr));
    lcrzo_er(shared_dns_rrlist_add_pos(&rrlist, -1, rr));
    lcrzo_er(shared_dns_rr_close(&rr));
    lcrzo_er(shared_dns_rr_init_resp_a(dnsservername, ttl, dnsserverip,
				       SHARED_DNS_RR_KIND_ADDITIONAL, &rr));
    lcrzo_er(shared_dns_rrlist_add_pos(&rrlist, -1, rr));
    lcrzo_er(shared_dns_rr_close(&rr));
  }
  else if ( rrquestion.type==SHARED_DNS_RR_TYPE_PTR )
  { lcrzo_er(shared_dns_rr_init_resp_ptr(rrquestion.name, ttl, thehostname,
				       SHARED_DNS_RR_KIND_ANSWER, &rr));
    lcrzo_er(shared_dns_rrlist_add_pos(&rrlist, -1, rr));
    lcrzo_er(shared_dns_rr_close(&rr));
    lcrzo_er(shared_dns_rr_init_resp_ns(rrquestion.name, ttl, dnsservername,
					SHARED_DNS_RR_KIND_AUTHORITY, &rr));
    lcrzo_er(shared_dns_rrlist_add_pos(&rrlist, -1, rr));
    lcrzo_er(shared_dns_rr_close(&rr));
    lcrzo_er(shared_dns_rr_init_resp_a(dnsservername, ttl, dnsserverip,
				       SHARED_DNS_RR_KIND_ADDITIONAL, &rr));
    lcrzo_er(shared_dns_rrlist_add_pos(&rrlist, -1, rr));
    lcrzo_er(shared_dns_rr_close(&rr));
  }
  else
  { dnsheader.rcode=SHARED_DNS_HEADER_RCODE_FAILURE;
  }

  /* do not need it anymore */
  lcrzo_epr(shared_dns_rr_close(&rrquestion));
    
  /* construct the packet */
  ret=shared_dns_packet_initm(dnsheader, &rrlist, &response, &responsesize);
  lcrzo_er(lcrzo_list_close(&rrlist));
  if (ret!=LCRZO_ERR_OK) return(ret);
  lcrzo_er(shared_dns_print(response, responsesize, LCRZO_PRINTTYPE_ARRAY));

  /* response */
  lcrzo_er(lcrzo_sock_write(psock, response, responsesize));

  lcrzo_er(lcrzo_data_free2(&response));
  return(LCRZO_ERR_OK);  
}




