
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000352=
{ /*reference*/     "lcrzoex", 352,
  /*version*/       1, 0,
  /*title*/         "Ethernet bridge with 2 network cards",
  /*french title*/  "Pont Ethernet avec 2 cartes reseau",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "device1 device2 recordfile1 recordfile2",
  /*french usage*/  "device1 device2 fichierrecord1 fichierrecord2",
  /*usage example*/ "eth0 eth1 /tmp/saveeth0 /tmp/saveeth1",
  /*fr. usage ex.*/ "eth0 eth1 /tmp/sauveeth0 /tmp/sauveeth1", 
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000352_main(int argc, char *argv[]);
int lcrzoex_000352_start(lcrzo_device device[2],
	                 lcrzo_record records[2]);
int lcrzoex_000352_checkdevice(lcrzo_list ethlists[], 
			       lcrzo_int32 indexsniff,
			       lcrzo_int32 indexspoof,
			       lcrzo_constdata sniffdata, 
			       lcrzo_int32 sniffdatasize, 
			       lcrzo_bool *pok);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000352_main(int argc, char *argv[])
{ lcrzo_device devices[2];
  lcrzo_record records[2];

  lcrzo_init();
  if ( argc<5 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000352, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }
  lcrzo_epr(shared_para_device_init_src(argv[1], devices[0]));
  lcrzo_epr(shared_para_device_init_src(argv[2], devices[1]));

  /* open the records */
  lcrzo_epr(lcrzo_record_open(&records[0], argv[3], LCRZO_RECORD_OPEN_WRITE));
  lcrzo_epr(lcrzo_record_open(&records[1], argv[4], LCRZO_RECORD_OPEN_WRITE));

  /* call the main function */
  lcrzo_epr(lcrzoex_000352_start(devices, records));

  lcrzo_epr(lcrzo_record_close(&records[0]));
  lcrzo_epr(lcrzo_record_close(&records[1]));

  lcrzo_close();
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000352_start(lcrzo_device devices[2],
	                 lcrzo_record records[2])
{ lcrzo_spoof spoof[2];
  lcrzo_sniff sniff[2];
  lcrzo_wait waitsniff[2];
  lcrzo_list ethlists[2];
  lcrzo_uint32 mtu;
  lcrzo_data sniffdata;
  lcrzo_int32 sniffdatasize, indexsniff, indexspoof;
  lcrzo_bool event0, event1, ok;
  int ret;

  /* init sniff */
  lcrzo_epr(lcrzo_mtu_init_device(devices[0], &mtu));
  lcrzo_epr(lcrzo_sniff_init(devices[0], mtu, 
			     LCRZO_SNIFF_TYPE_NORMAL, "", &sniff[0]));
  lcrzo_epr(lcrzo_mtu_init_device(devices[1], &mtu));
  lcrzo_epr(lcrzo_sniff_init(devices[1], mtu, 
			     LCRZO_SNIFF_TYPE_NORMAL, "", &sniff[1]));
  /* init spoof */
  lcrzo_epr(lcrzo_spoof_init(&spoof[0]));
  lcrzo_epr(lcrzo_spoof_init(&spoof[1]));
  /*lcrzo_epr(lcrzo_spoof_set_printbeforesending(&spoof[0],
                                               LCRZO_TRUE));*/

  /* init wait */
  lcrzo_epr(lcrzo_wait_init_sniff_read(&sniff[0], &waitsniff[0]));
  lcrzo_epr(lcrzo_wait_init_sniff_read(&sniff[1], &waitsniff[1]));

  /* init ethlists */
  lcrzo_epr(lcrzo_ethlist_init(&(ethlists[0])));
  lcrzo_epr(lcrzo_ethlist_init(&(ethlists[1])));

  /* main loop */
  ret=LCRZO_ERR_OK;
  while(LCRZO_TRUE)
  { ret=lcrzo_wait_wait2(&waitsniff[0], &waitsniff[1], LCRZO_WAIT_INFINITE,
                         &event0, &event1);
    if (ret!=LCRZO_ERR_OK) break;
    /* set indexes */
    if (event0) { indexsniff=0; indexspoof=1; }
    else if (event1) { indexsniff=1; indexspoof=0; }
    else { ret=LCRZO_ERR_IEINTERNALERROR; break; }
    lcrzo_epr(lcrzo_int32_print("Sniff data on device ", indexsniff, "%d",
				"\n"));
    /* sniff */
    ret=lcrzo_sniff_nextm(&sniff[indexsniff], LCRZO_WAIT_INFINITE, 
			  &sniffdata, &sniffdatasize);
    if (ret!=LCRZO_ERR_OK) break;
    /* check if data is received on the good side */
    ret=lcrzoex_000352_checkdevice(ethlists, indexsniff, indexspoof,
				   sniffdata, sniffdatasize, &ok);
    if (ret!=LCRZO_ERR_OK) break;
    if (!ok)
    { lcrzo_epr(lcrzo_data_free2(&sniffdata));
      /* do not send data on this device */
      continue;
    }
    /* we intercept and modify data */
    /* currently nothing is done on sniffdata */
    lcrzo_epr(lcrzo_packet_print_eth(sniffdata, sniffdatasize,
				     LCRZO_PRINTPROFILE_ARRAYDUMP));
    lcrzo_epr(lcrzo_record_write_eth(&records[indexsniff], 
				     sniffdata, sniffdatasize,
                                     LCRZO_PRINTPROFILE_HEXANCHMS));
    /* spoof */   
    lcrzo_epr(lcrzo_spoof_eth(&spoof[indexspoof], devices[indexspoof],
			      sniffdata, sniffdatasize));
    lcrzo_epr(lcrzo_data_free2(&sniffdata));
  }

  /* close */
  lcrzo_epr(lcrzo_list_close(&ethlists[0]));
  lcrzo_epr(lcrzo_list_close(&ethlists[1]));
  lcrzo_epr(lcrzo_wait_close(&waitsniff[1]));
  lcrzo_epr(lcrzo_wait_close(&waitsniff[0]));
  lcrzo_epr(lcrzo_spoof_close(&spoof[0]));
  lcrzo_epr(lcrzo_spoof_close(&spoof[1]));
  lcrzo_epr(lcrzo_sniff_close(&sniff[0]));
  lcrzo_epr(lcrzo_sniff_close(&sniff[1]));

  return (ret);
}

/*-------------------------------------------------------------*/
int lcrzoex_000352_checkdevice(lcrzo_list ethlists[], 
			       lcrzo_int32 indexsniff,
			       lcrzo_int32 indexspoof,
			       lcrzo_constdata sniffdata, 
			       lcrzo_int32 sniffdatasize, 
			       lcrzo_bool *pok)
{ lcrzo_hdrleth hdreth;
  lcrzo_bool found;
  int ret;
  
  /* decode eth */
  ret=lcrzo_packet_decodem_ethdata(sniffdata, sniffdatasize, &hdreth,
				   NULL,NULL);
  if (ret!=LCRZO_ERR_OK) return (LCRZO_ERR_OK);

  /* search source ethernet address */
  lcrzo_epr(lcrzo_ethlist_isinlist(&ethlists[indexspoof], 
				   hdreth.src, &found));
  if (found)
  { /* not ok */
    if (pok!=NULL) *pok=LCRZO_FALSE;
    return (LCRZO_ERR_OK);
  }
  
  /* ok */
  if (pok!=NULL) *pok=LCRZO_TRUE;
  /* add in the local list */
  lcrzo_epr(lcrzo_ethlist_add_range(&ethlists[indexsniff], 
				    hdreth.src, hdreth.src));
  return (LCRZO_ERR_OK);
}




