
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000353=
{ /*reference*/     "lcrzoex", 353,
  /*version*/       1, 0,
  /*title*/         "sniff packets and search mixed data",
  /*french title*/  "sniffe des paquets et recherche une donnee mixte",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "\"mixedsearchstring\" [device [\"bpf_filter\"]]",
  /*french usage*/  "\"chainechercheemixte\" [device [\"filtre_bpf\"]]",
  /*usage example*/ "\"'PA' 07 'TO' 0D0A\" eth0 \"host 1.2.3.4 and port 25\"",
  /*fr. usage ex.*/ "\"'PA' 07 'TO' 0D0A\" eth0 \"host 1.2.3.4 and port 25\"", 
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declaration */
int lcrzoex_000353_main(int argc, char *argv[]);
int lcrzoex_000353_mixedgrep(lcrzo_constdata searcheddata,
			     lcrzo_int32 searcheddatasize,
			     lcrzo_constdevice device,
			     lcrzo_conststring filter,
			     lcrzo_int16 snifftype);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000353_main(int argc, char *argv[])
{ lcrzo_data searcheddata;
  lcrzo_int32 searcheddatasize;
  lcrzo_string filter;
  lcrzo_device device;

  lcrzo_init();
  if ( argc<2 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000353, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }

  lcrzo_epr(lcrzo_data_initm_mixed(argv[1], 
				   &searcheddata, &searcheddatasize));
  if (argc>=3)
  { lcrzo_epr(shared_para_device_init_src(argv[2], device));
  }
  else
  { lcrzo_epr(lcrzo_device_value_pos(LCRZO_DEVICE_TYPE_BEST, 1, device));
  }
  if (argc>=4)
  { lcrzo_epr(shared_para_string_init_snifffilter(argv[3], &filter));
  }
  else
  { lcrzo_epr(lcrzo_string_initm_text("", &filter));
  }

  lcrzo_epr(lcrzoex_000353_mixedgrep(searcheddata, searcheddatasize, 
				     device, filter, LCRZO_SNIFF_TYPE_NORMAL));

  lcrzo_epr(lcrzo_string_free2(&filter));
  lcrzo_epr(lcrzo_data_free2(&searcheddata));

  lcrzo_close();
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000353_mixedgrep(lcrzo_constdata searcheddata,
			     lcrzo_int32 searcheddatasize,
			     lcrzo_constdevice device,
			     lcrzo_conststring filter,
			     lcrzo_int16 snifftype)
{ lcrzo_uint32 mtu;
  lcrzo_sniff sniff;
  lcrzo_data sniffdata;
  lcrzo_int32 sniffdatasize;
  lcrzo_bool pointprinted;
  int ret;
  
  /* init mtu */
  lcrzo_epr(lcrzo_mtu_init_device(device, &mtu));

  /* init sniff */
  lcrzo_epr(lcrzo_sniff_init(device, mtu, snifftype, filter, &sniff));

  /* main loop */
  pointprinted=LCRZO_FALSE;
  do
  { ret=lcrzo_sniff_nextm(&sniff, LCRZO_WAIT_INFINITE, 
			  &sniffdata, &sniffdatasize);
    if (ret!=LCRZO_ERR_OK) break;
    ret=lcrzo_data_search_data(sniffdata, sniffdatasize,
                               LCRZO_FALSE, +1, -1,
			       searcheddata, searcheddatasize,
			       NULL, NULL, NULL, NULL);
    if (ret==LCRZO_ERR_OK)
    { if (pointprinted) printf("\n\n");
      lcrzo_epr(lcrzo_packet_print_eth(sniffdata, sniffdatasize, 
				       LCRZO_PRINTPROFILE_ARRAYDUMP));
      printf("\n");
      pointprinted=LCRZO_FALSE;
    }
    else if (ret==LCRZO_ERR_OKSEARCHNOTFOUND)
    { printf(".");
      lcrzo_epr(lcrzo_stream_flush(stdout));
      pointprinted=LCRZO_TRUE;
    }
    else
    { break;
    }
    lcrzo_epr(lcrzo_data_free2(&sniffdata));
  }
  while(LCRZO_TRUE);

  /* close everything */
  lcrzo_epr(lcrzo_sniff_close(&sniff));
  return (ret);
}
