
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000354=
{ /*reference*/     "lcrzoex", 354,
  /*version*/       1, 0,
  /*title*/         "sniff packets and search several mixed data",
  /*french title*/  "sniffe des paquets et recherche plusieurs donnees mixtes",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "device \"bpf_filter\" \"mixedsearchstring\" [\"mixedsearchstring\" [...]]",
  /*french usage*/  "device \"filtre_bpf\" \"chainechercheemixte\" [\"chainechercheemixte\" [...]]",
  /*usage example*/ "eth0 \"\" \"'USER'\" \"'PASS'\"",
  /*fr. usage ex.*/ "eth0 \"\" \"'USER'\" \"'PASS'\"", 
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declaration */
int lcrzoex_000354_main(int argc, char *argv[]);
int lcrzoex_000354_mixedgrepn(lcrzo_list *plist,
			      lcrzo_constdevice device,
			      lcrzo_conststring filter,
			      lcrzo_int16 snifftype);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000354_main(int argc, char *argv[])
{ lcrzo_data searcheddata;
  lcrzo_int32 searcheddatasize;
  lcrzo_list datalist;
  lcrzo_string filter;
  lcrzo_device device;
  int i;

  lcrzo_init();
  if ( argc<4 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000354, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }

  lcrzo_epr(shared_para_device_init_src(argv[1], device));
  lcrzo_epr(shared_para_string_init_snifffilter(argv[2], &filter));

  /* set searched data list */
  lcrzo_epr(lcrzo_datalist_init(&datalist));
  for ( i=3 ; i<argc ; i++ )
  { lcrzo_epr(lcrzo_data_initm_mixed(argv[i], 
				     &searcheddata, &searcheddatasize));
    lcrzo_epr(lcrzo_datalist_add_pos(&datalist, -1, 0, 
				     searcheddata, searcheddatasize));
    lcrzo_epr(lcrzo_data_free2(&searcheddata));
  }

  /* call the main loop */
  lcrzo_epr(lcrzoex_000354_mixedgrepn(&datalist, device, filter, 
				      LCRZO_SNIFF_TYPE_NORMAL));

  lcrzo_epr(lcrzo_string_free2(&filter));
  lcrzo_epr(lcrzo_list_close(&datalist));
  lcrzo_close();
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000354_mixedgrepn(lcrzo_list *plist,
			      lcrzo_constdevice device,
			      lcrzo_conststring filter,
			      lcrzo_int16 snifftype)
{ lcrzo_uint32 mtu;
  lcrzo_sniff sniff;
  lcrzo_data sniffdata, searcheddata;
  lcrzo_int32 sniffdatasize, searcheddatasize, i, count;
  lcrzo_bool pointprinted;
  int ret;
  
  /* init mtu */
  lcrzo_epr(lcrzo_mtu_init_device(device, &mtu));

  /* init sniff */
  lcrzo_epr(lcrzo_sniff_init(device, mtu, snifftype, filter, &sniff));

  lcrzo_epr(lcrzo_list_count(plist, &count));

  /* main loop */
  pointprinted=LCRZO_FALSE;
  do
  { ret=lcrzo_sniff_nextm(&sniff, LCRZO_WAIT_INFINITE, 
			  &sniffdata, &sniffdatasize);
    if (ret!=LCRZO_ERR_OK) break;
    /* loop through each item in the list */
    ret=LCRZO_ERR_OKSEARCHNOTFOUND;
    for ( i=1 ; i<=count && ret==LCRZO_ERR_OKSEARCHNOTFOUND ; i++ )
    { lcrzo_epr(lcrzo_datalist_valuem_pos(plist, i, NULL,
					  &searcheddata, &searcheddatasize));
      ret=lcrzo_data_search_data(sniffdata, sniffdatasize,
				 LCRZO_FALSE, +1, -1,
				 searcheddata, searcheddatasize,
				 NULL, NULL, NULL, NULL);
      lcrzo_epr(lcrzo_data_free2(&searcheddata));
    }
    /* display if found */
    if (ret==LCRZO_ERR_OK)
    { if (pointprinted) printf("\n\n");
      lcrzo_epr(lcrzo_packet_print_eth(sniffdata, sniffdatasize, 
				       LCRZO_PRINTPROFILE_ARRAYDUMP));
      printf("\n");
      pointprinted=LCRZO_FALSE;
    }
    else if (ret==LCRZO_ERR_OKSEARCHNOTFOUND)
    { printf(".");
      lcrzo_epr(lcrzo_stream_flush(stdout));
      pointprinted=LCRZO_TRUE;
    }
    else
    { break;
    }
    lcrzo_epr(lcrzo_data_free2(&sniffdata));
  }
  while(LCRZO_TRUE);

  /* close everything */
  lcrzo_epr(lcrzo_sniff_close(&sniff));
  return (ret);
}
