
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000356=
{ /*reference*/     "lcrzoex", 356,
  /*version*/       1, 0,
  /*title*/         "sniff packets and search several regexp data",
  /*french title*/  "sniffe des paquets et recherche plusieurs donnees mixtes",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "device \"bpf_filter\" \"searchedregexpr\" [\"searchedregexpr\" [...]]",
  /*french usage*/  "device \"filtre_bpf\" \"regexpcherchee\" [\"regexpcherchee\" [...]]",
  /*usage example*/ "eth0 \"\" \"x.*[0-9]z\" \"[A-Z][A-X]\"",
  /*fr. usage ex.*/ "eth0 \"\" \"x.*[0-9]z\" \"[A-Z][A-X]\"", 
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declaration */
int lcrzoex_000356_main(int argc, char *argv[]);
int lcrzoex_000356_regexpgrepn(lcrzo_list *plist,
			       lcrzo_constdevice device,
			       lcrzo_conststring filter,
			       lcrzo_int16 snifftype);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000356_main(int argc, char *argv[])
{ lcrzo_list stringlist;
  lcrzo_string filter;
  lcrzo_device device;
  int i;

  lcrzo_init();
  if ( argc<4 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000356, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }

  lcrzo_epr(shared_para_device_init_src(argv[1], device));
  lcrzo_epr(shared_para_string_init_snifffilter(argv[2], &filter));

  /* set searched list */
  lcrzo_epr(lcrzo_stringlist_init(&stringlist));
  for ( i=3 ; i<argc ; i++ )
  { lcrzo_epr(lcrzo_stringlist_add_pos(&stringlist, -1, 0, argv[i]));
  }

  /* call the main loop */
  lcrzo_epr(lcrzoex_000356_regexpgrepn(&stringlist, device, filter, 
				       LCRZO_SNIFF_TYPE_NORMAL));

  lcrzo_epr(lcrzo_string_free2(&filter));
  lcrzo_epr(lcrzo_list_close(&stringlist));
  lcrzo_close();
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000356_regexpgrepn(lcrzo_list *plist,
			       lcrzo_constdevice device,
			       lcrzo_conststring filter,
			       lcrzo_int16 snifftype)
{ lcrzo_string searchedregexp;
  lcrzo_uint32 mtu;
  lcrzo_sniff sniff;
  lcrzo_data sniffdata;
  lcrzo_int32 sniffdatasize, i, count;
  lcrzo_bool pointprinted;
  int ret;
  
  /* init mtu */
  lcrzo_epr(lcrzo_mtu_init_device(device, &mtu));

  /* init sniff */
  lcrzo_epr(lcrzo_sniff_init(device, mtu, snifftype, filter, &sniff));

  lcrzo_epr(lcrzo_list_count(plist, &count));

  /* main loop */
  pointprinted=LCRZO_FALSE;
  do
  { ret=lcrzo_sniff_nextm(&sniff, LCRZO_WAIT_INFINITE, 
			  &sniffdata, &sniffdatasize);
    if (ret!=LCRZO_ERR_OK) break;
    /* loop through each item in the list */
    ret=LCRZO_ERR_OKSEARCHNOTFOUND;
    for ( i=1 ; i<=count && ret==LCRZO_ERR_OKSEARCHNOTFOUND ; i++ )
    { lcrzo_epr(lcrzo_stringlist_valuem_pos(plist, i, NULL, &searchedregexp));
      ret=lcrzo_data_search_re(sniffdata, sniffdatasize,
			       LCRZO_FALSE, +1, -1,
			       searchedregexp,
			       NULL, NULL, NULL, NULL);
      lcrzo_epr(lcrzo_string_free2(&searchedregexp));
    }
    /* display if found */
    if (ret==LCRZO_ERR_OK)
    { if (pointprinted) printf("\n\n");
      lcrzo_epr(lcrzo_packet_print_eth(sniffdata, sniffdatasize, 
				       LCRZO_PRINTPROFILE_ARRAYDUMP));
      printf("\n");
      pointprinted=LCRZO_FALSE;
    }
    else if (ret==LCRZO_ERR_OKSEARCHNOTFOUND)
    { printf(".");
      lcrzo_epr(lcrzo_stream_flush(stdout));
      pointprinted=LCRZO_TRUE;
    }
    else
    { break;
    }
    lcrzo_epr(lcrzo_data_free2(&sniffdata));
  }
  while(LCRZO_TRUE);

  /* close everything */
  lcrzo_epr(lcrzo_sniff_close(&sniff));
  return (ret);
}
