
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000358=
{ /*reference*/     "lcrzoex", 358,
  /*version*/       1, 0,
  /*title*/         "web server",
  /*french title*/  "server web",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "server_name server_port path [login [password]]",
  /*french usage*/  "nom_serveur port_serveur path [login [password]]",
  /*usage example*/ "1.2.3.4 5566 /tmp/shared",
  /*fr. usage ex.*/ "1.2.3.4 5566 /tmp/partage",
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000358_main(int argc, char *argv[]);
int lcrzoex_000358_server(lcrzo_sock *psock, lcrzo_ptr pinfos);
int lcrzoex_000358_badcred(lcrzo_sock *psock);
int lcrzoex_000358_goodcred(lcrzo_sock *psock,
			    lcrzo_conststring method,
			    lcrzo_conststring url,
			    lcrzo_constdata body,
			    lcrzo_int32 bodysize,
			    lcrzo_conststring path);
int lcrzoex_000358_display_dir(lcrzo_sock *psock,
			       lcrzo_conststring path,
			       lcrzo_conststring url,
			       lcrzo_int16 language);
int lcrzoex_000358_display_file(lcrzo_sock *psock,
				lcrzo_conststring path,
				lcrzo_int16 language);

/*-------------------------------------------------------------*/
/* timeout for reading from socket, in seconds */
#define LCRZOEX_000358_TIMEOUT_HEADER 10
#define LCRZOEX_000358_TIMEOUT_BODY   30
typedef struct {
  lcrzo_int8 user[32];
  lcrzo_int8 pass[32];
  lcrzo_int8 path[1024];
} lcrzoex_000358_userpasspath;

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000358_main(int argc, char *argv[])
{ lcrzo_ipl serveripl;
  lcrzo_uint16 serverport;
  lcrzoex_000358_userpasspath upp;
  lcrzo_string tmp;
  lcrzo_int16 language, pathtype;
  int ret;

  lcrzo_init();
  if ( argc<4 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000358, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }
  lcrzo_epr(lcrzo_global_get_language(&language));

  /* the default resolver timeout is too short if it's long to reach
     webserver's DNS */
  lcrzo_epr(shared_http_set_resolver_timeout());

  /* initialize IP address */
  lcrzo_epr(shared_para_ipl_init_listen(argv[1], &serveripl));

  /* initialize port number */
  lcrzo_epr(shared_para_port_init_listen(argv[2], &serverport));

  /* check path */
  lcrzo_epr(lcrzo_path_clean(argv[3], &tmp));
  lcrzo_epr(lcrzo_string_init_text(tmp, sizeof(upp.path), upp.path));
  lcrzo_epr(lcrzo_string_free2(&tmp));
  ret=lcrzo_path_type(upp.path, &pathtype);
  if (ret!=LCRZO_ERR_OK)
  { printf("Directory %s does not exists\n", upp.path);
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }
  if (pathtype!=LCRZO_PATH_TYPE_DIR)
  { printf("File %s is not a directory\n", upp.path);
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }

  /* initialize login and passwd */
  strcpy(upp.user, "");
  strcpy(upp.pass, "");
  if ( argc>4 )
  { lcrzo_epr(lcrzo_string_init_text(argv[4], sizeof(upp.user), upp.user));
    if ( argc>5 )
    { lcrzo_epr(lcrzo_string_init_text(argv[5], sizeof(upp.pass), upp.pass));
    }
    else
    {lcrzo_epr(lcrzo_string_initm_text_passwd((language==LCRZO_GLOBAL_LANG_FR)?
            "Entrez le mot de passe que l'utilisateur aura a saisir":
            "Enter the password which will be needed by the user",
					      "lcrzoex",  &tmp));
      lcrzo_epr(lcrzo_string_init_text(tmp, sizeof(upp.pass), upp.pass));
      lcrzo_epr(lcrzo_string_free2(&tmp));
    }
  }

  lcrzo_epr(lcrzo_sock_tcpmulser_real2(serveripl, serverport, NULL, 0,
                                       &lcrzoex_000358_server, &upp));

  lcrzo_close();
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000358_server(lcrzo_sock *psock, lcrzo_ptr pinfos)
{ lcrzo_data body, data;
  lcrzo_int32 bodysize, contentlength, datasize;
  lcrzo_string method, url, sitelogin, sitepasswd, urlok;
  lcrzo_time maxwaitingtime;
  lcrzoex_000358_userpasspath *pupp=(lcrzoex_000358_userpasspath *)pinfos;
  lcrzo_int16 language;
  lcrzo_bool goodcred;
  int ret;

  /* Note : this server is not KeepAlive */

  /* read header */
  lcrzo_epr(lcrzo_time_init_sec(LCRZOEX_000358_TIMEOUT_HEADER, 
				&maxwaitingtime));
  lcrzo_epr(shared_http_request_readheader(psock, &maxwaitingtime,
					   &method, &url,
					   NULL, NULL, NULL, NULL, NULL,
					   &sitelogin, &sitepasswd,
					   &contentlength, NULL, NULL));

  if ( !strcmp(method, "POST") && contentlength )
  { /* As said in rfc 2068, there is a bug in some clients :
       RFC 2068 : 4.1 Message Types
       Note: certain buggy HTTP/1.0 client implementations generate an
       extra CRLF's after a POST request. To restate what is explicitly
       forbidden by the BNF, an HTTP/1.1 client must not preface or follow
       a request with an extra CRLF.
    */
    /* read body */
    lcrzo_epr(lcrzo_time_init_sec(LCRZOEX_000358_TIMEOUT_BODY, 
				  &maxwaitingtime));
    lcrzo_epr(shared_http_response_readbody(psock, &maxwaitingtime, 
					    contentlength,
					    &body, &bodysize));
  }
  else
  { body=NULL;
    bodysize=0;
  }

  /* check if credentials are ok (allow small incompatibilities) */
  goodcred=LCRZO_FALSE;
  if ( strlen(pupp->user)==0 )
  { /* do not check more */
    goodcred=LCRZO_TRUE;
  }
  else
  { if ( sitelogin!=NULL )
    { if ( !strcmp(sitelogin, pupp->user) )
      { if ( strlen(pupp->pass)==0 )
        { /* do not check more */
	  goodcred=LCRZO_TRUE;
	}
        else
	{ if ( sitepasswd!=NULL )
  	  { if ( !strcmp(sitepasswd, pupp->pass) )
	    { goodcred=LCRZO_TRUE;
	    }
	  }
	}
      }
    }
  }

  /* treat */
  if ( goodcred )
  { /* good credentials */
    ret=shared_http_string_initm_pt(url, &urlok);
    if (ret==LCRZO_ERR_OK) 
    { lcrzo_epr(lcrzoex_000358_goodcred(psock, method, urlok, 
					body, bodysize,
					pupp->path));
      lcrzo_epr(lcrzo_string_free2(&urlok));
    }
    else
    { lcrzo_epr(shared_http_initm_httpvererr("1.0", 
					     SHARED_HTTP_ERRORCODE_BADREQUEST,
					     "Bad Request", 
					     &data, &datasize));
      lcrzo_epr(shared_http_appendm_connection(NULL, datasize, 
					       &data,&datasize));
      lcrzo_epr(shared_http_appendm_contenttype(NULL, datasize,
						&data,&datasize));
      lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, &data, &datasize));
      lcrzo_epr(lcrzo_data_appendm_text("<HTML><BODY>\n", datasize,
					&data, &datasize));

      lcrzo_epr(lcrzo_global_get_language(&language));
      if ( language==LCRZO_GLOBAL_LANG_FR)
      { lcrzo_epr(lcrzo_data_appendm_text("L'url est malformee\n", 
					  datasize, &data, &datasize));
      }
      else
      { lcrzo_epr(lcrzo_data_appendm_text("Url is not well formed\n", 
					  datasize, &data, &datasize));
      }
      lcrzo_epr(lcrzo_data_appendm_text("</BODY></HTML>\n", datasize,
					&data, &datasize));
      
      lcrzo_epr(lcrzo_sock_write(psock, data, datasize));
      lcrzo_epr(lcrzo_data_free2(&data));
      return(LCRZO_ERR_OK);
    }
  }
  else
  { /* bad credentials */ 
    lcrzo_epr(lcrzoex_000358_badcred(psock));
  }
  
  /* free memory */
  if (body!=NULL) { lcrzo_epr(lcrzo_data_free2(&body)); }
  lcrzo_epr(lcrzo_string_free2(&method));
  lcrzo_epr(lcrzo_string_free2(&url));
  if (sitelogin!=NULL) { lcrzo_epr(lcrzo_string_free2(&sitelogin)); }
  if (sitepasswd!=NULL) { lcrzo_epr(lcrzo_string_free2(&sitepasswd)); }

  return(LCRZO_ERR_OK);
  pinfos=pinfos; /*just for the compiler warning*/
}

/*-------------------------------------------------------------*/
int lcrzoex_000358_badcred(lcrzo_sock *psock)
{ lcrzo_int16 language;
  lcrzo_data data;
  lcrzo_int32 datasize;

  /* get language for messages */
  lcrzo_epr(lcrzo_global_get_language(&language));

  lcrzo_epr(shared_http_initm_httpvererr("1.0",
					 SHARED_HTTP_ERRORCODE_UNAUTHORIZED, 
					 (language==LCRZO_GLOBAL_LANG_FR)?
					 "Vous devez vous authentifier":
					 "Authorization Required",
					 &data, &datasize));
  lcrzo_epr(shared_http_appendm_connection(NULL, datasize, &data, &datasize));
  lcrzo_epr(shared_http_appendm_contenttype(NULL, datasize, &data, &datasize));
  lcrzo_epr(shared_http_appendm_wwwauth((language==LCRZO_GLOBAL_LANG_FR)?
					"Acces web a lcrzoex 358":
					"Lcrzoex 358 web access",
					datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, &data, &datasize));

  lcrzo_epr(lcrzo_data_appendm_text("<HTML><BODY>\n", datasize,
				    &data, &datasize));
  if ( language==LCRZO_GLOBAL_LANG_FR)
  { lcrzo_epr(lcrzo_data_appendm_text(
              "Entrez le login/password de lcrzoex_000358.c\n",
				      datasize, &data, &datasize));
  }
  else
  { lcrzo_epr(lcrzo_data_appendm_text(
              "Enter login/password from lcrzoex_000358.c\n", 
				      datasize, &data, &datasize));
  }
  lcrzo_epr(lcrzo_data_appendm_text("</BODY></HTML>\n", datasize, 
				    &data, &datasize));

  lcrzo_epr(lcrzo_sock_write(psock, data, datasize));
  lcrzo_epr(lcrzo_data_free2(&data));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000358_goodcred(lcrzo_sock *psock,
			    lcrzo_conststring method,
			    lcrzo_conststring url,
			    lcrzo_constdata body,
			    lcrzo_int32 bodysize,
			    lcrzo_conststring path)
{ lcrzo_int16 language, pathtype;
  lcrzo_data data;
  lcrzo_int32 datasize;
  lcrzo_string cleanurl, truepath, tmp;
  int ret;

  /* get language for messages */
  lcrzo_epr(lcrzo_global_get_language(&language));

  /* verify the method */
  if ( strcmp(method, "GET") )
  { lcrzo_epr(shared_http_initm_httpvererr("1.0",
					 SHARED_HTTP_ERRORCODE_NOTIMPLEMENTED, 
					   "Method Not Implemented", 
					   &data,&datasize));
    lcrzo_epr(shared_http_appendm_connection(NULL, datasize, &data,&datasize));
    lcrzo_epr(shared_http_appendm_contenttype(NULL, datasize,&data,&datasize));
    lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, &data, &datasize));
    lcrzo_epr(lcrzo_data_appendm_text("<HTML><BODY>\n", datasize,
				      &data, &datasize));
    if ( language==LCRZO_GLOBAL_LANG_FR)
    { lcrzo_epr(lcrzo_data_appendm_text("Seule la methode GET \n", 
					datasize, &data, &datasize));
      lcrzo_epr(lcrzo_data_appendm_text("est actuellement supportee\n", 
					datasize, &data, &datasize));
      lcrzo_epr(lcrzo_data_appendm_text("dans lcrzoex_000358.c.\n", 
					datasize, &data, &datasize));
    }
    else
    { lcrzo_epr(lcrzo_data_appendm_text("Only GET method \n", 
					datasize, &data, &datasize));
      lcrzo_epr(lcrzo_data_appendm_text("is actually supported\n", 
					datasize, &data, &datasize));
      lcrzo_epr(lcrzo_data_appendm_text("in lcrzoex_000358.c.\n", 
					datasize, &data, &datasize));
    }
    if ( ! strcmp(method, "POST") )
    { lcrzo_epr(lcrzo_data_appendm_text("POST :\n", 
					datasize, &data, &datasize));
      lcrzo_epr(lcrzo_data_appendm_data(body, bodysize, 
					datasize, &data, &datasize));
      
    }
    lcrzo_epr(lcrzo_data_appendm_text("</BODY></HTML>\n", datasize,
				      &data, &datasize));
    lcrzo_epr(lcrzo_sock_write(psock, data, datasize));
    lcrzo_epr(lcrzo_data_free2(&data));
    return(LCRZO_ERR_OK);
  }

  /* verify the url */
  lcrzo_epr(lcrzo_string_initm_text("/", &tmp)); /*always start with '/'*/
  lcrzo_epr(lcrzo_string_appendm_text(url, &tmp));
  ret=lcrzo_path_clean(tmp, &cleanurl);
  lcrzo_epr(lcrzo_string_free2(&tmp));
  if (ret!=LCRZO_ERR_OK)
  { lcrzo_epr(shared_http_initm_httpvererr("1.0", 
					   SHARED_HTTP_ERRORCODE_BADREQUEST,
					   "Bad Request", 
					   &data, &datasize));
    lcrzo_epr(shared_http_appendm_connection(NULL, datasize, &data,&datasize));
    lcrzo_epr(shared_http_appendm_contenttype(NULL, datasize,&data,&datasize));
    lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, &data, &datasize));
    lcrzo_epr(lcrzo_data_appendm_text("<HTML><BODY>\n", datasize,
				      &data, &datasize));
    if ( language==LCRZO_GLOBAL_LANG_FR)
    { lcrzo_epr(lcrzo_data_appendm_text("L'url est malformee\n", 
					datasize, &data, &datasize));
    }
    else
    { lcrzo_epr(lcrzo_data_appendm_text("Url is not well formed\n", 
					datasize, &data, &datasize));
    }
    lcrzo_epr(lcrzo_data_appendm_text("</BODY></HTML>\n", datasize,
				      &data, &datasize));
   
    lcrzo_epr(lcrzo_sock_write(psock, data, datasize));
    lcrzo_epr(lcrzo_data_free2(&data));
    return(LCRZO_ERR_OK);
  }

  /* create the true url */
  lcrzo_er(lcrzo_string_initm_text(".", &tmp));
  lcrzo_er(lcrzo_string_appendm_text(cleanurl, &tmp));
  lcrzo_epr(lcrzo_string_free2(&cleanurl));
  lcrzo_epr(lcrzo_path_init_concat(path, LCRZO_TRUE, LCRZO_TRUE,
				   tmp, LCRZO_TRUE, &truepath));  
  lcrzo_epr(lcrzo_string_free2(&tmp));

  /* check if file exists */
  ret=lcrzo_path_type(truepath, &pathtype);
  if (ret!=LCRZO_ERR_OK)
  { lcrzo_epr(shared_http_initm_httpvererr("1.0",
					   SHARED_HTTP_ERRORCODE_NOTFOUND,
					   "File not found", 
					   &data, &datasize));
    lcrzo_epr(shared_http_appendm_connection(NULL, datasize, &data,&datasize));
    lcrzo_epr(shared_http_appendm_contenttype(NULL, datasize,&data,&datasize));
    lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, &data, &datasize));
    lcrzo_epr(lcrzo_data_appendm_text("<HTML><BODY>\n", datasize,
				      &data, &datasize));
    if ( language==LCRZO_GLOBAL_LANG_FR)
    { lcrzo_epr(lcrzo_data_appendm_text("Fichier non trouve\n", 
					datasize, &data, &datasize));
    }
    else
    { lcrzo_epr(lcrzo_data_appendm_text("File not found\n", 
					datasize, &data, &datasize));
    }
    lcrzo_epr(lcrzo_data_appendm_text("</BODY></HTML>\n", datasize,
				      &data, &datasize));
   
    lcrzo_epr(lcrzo_sock_write(psock, data, datasize));
    lcrzo_epr(lcrzo_data_free2(&data));
    lcrzo_epr(lcrzo_string_free2(&truepath));
    return(LCRZO_ERR_OK);
  }

  /* display the file */
  if (pathtype==LCRZO_PATH_TYPE_DIR)
  { if ( url[strlen(url)-1]=='/' )
    { lcrzo_epr(lcrzoex_000358_display_dir(psock, truepath, url, language));
    }
    else
    { lcrzo_epr(lcrzo_string_initm_text(url, &tmp));
      lcrzo_epr(lcrzo_string_appendm_text("/", &tmp));
      lcrzo_epr(shared_http_initm_httpvererr("1.0",
				     SHARED_HTTP_ERRORCODE_MOVEDPERMANENTLY,
					     "Moved Permanently", 
					     &data, &datasize));
      lcrzo_epr(shared_http_appendm_connection(NULL, datasize, 
					       &data,&datasize));
      lcrzo_epr(shared_http_appendm_contenttype(NULL, datasize,
						&data,&datasize));
      lcrzo_epr(shared_http_appendm_location(tmp, datasize,
					     &data, &datasize));
      lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, &data, &datasize));
      lcrzo_epr(lcrzo_data_appendm_text("<HTML><BODY>\n", datasize,
					&data, &datasize));
      if ( language==LCRZO_GLOBAL_LANG_FR)
      { lcrzo_epr(lcrzo_data_appendm_text("Le document est ici : ", 
					  datasize, &data, &datasize));
      }
      else
      { lcrzo_epr(lcrzo_data_appendm_text("The document has moved : ", 
					  datasize, &data, &datasize));
      }
      lcrzo_epr(lcrzo_data_appendm_text("<A HREF=\"", 
					datasize, &data, &datasize));
      lcrzo_epr(lcrzo_data_appendm_text(tmp, 
					datasize, &data, &datasize));
      lcrzo_epr(lcrzo_data_appendm_text("\">", 
					datasize, &data, &datasize));
      lcrzo_epr(lcrzo_data_appendm_text(tmp, 
					datasize, &data, &datasize));
      lcrzo_epr(lcrzo_data_appendm_text("</A><BR>\n", 
					datasize, &data, &datasize));
      lcrzo_epr(lcrzo_data_appendm_text("</BODY></HTML>\n", datasize,
					&data, &datasize));
      lcrzo_epr(lcrzo_string_free2(&tmp));

      lcrzo_epr(lcrzo_sock_write(psock, data, datasize));
      lcrzo_epr(lcrzo_data_free2(&data));
      lcrzo_epr(lcrzo_string_free2(&truepath));
      return(LCRZO_ERR_OK);
    }
  }
  else if (pathtype==LCRZO_PATH_TYPE_REG)
  { lcrzo_epr(lcrzoex_000358_display_file(psock, truepath, language));
  }
  else
  { lcrzo_epr(shared_http_initm_httpvererr("1.0",
					   SHARED_HTTP_ERRORCODE_NOTFOUND,
					   "File not found", 
					   &data, &datasize));
    lcrzo_epr(shared_http_appendm_connection(NULL, datasize, &data,&datasize));
    lcrzo_epr(shared_http_appendm_contenttype(NULL, datasize,&data,&datasize));
    lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, &data, &datasize));
    lcrzo_epr(lcrzo_data_appendm_text("<HTML><BODY>\n", datasize,
				      &data, &datasize));
    if ( language==LCRZO_GLOBAL_LANG_FR)
    { lcrzo_epr(lcrzo_data_appendm_text("Mauvais type de fichier : ", 
					datasize, &data, &datasize));
    }
    else
    { lcrzo_epr(lcrzo_data_appendm_text("Bad file type : ", 
					datasize, &data, &datasize));
    }
    lcrzo_epr(lcrzo_data_appendm_int16(pathtype, "%d",
				       datasize, &data, &datasize));
    lcrzo_epr(lcrzo_data_appendm_text("<BR>\n", 
				      datasize, &data, &datasize));
    lcrzo_epr(lcrzo_data_appendm_text("</BODY></HTML>\n", datasize,
				      &data, &datasize));
   
    lcrzo_epr(lcrzo_sock_write(psock, data, datasize));
    lcrzo_epr(lcrzo_data_free2(&data));
    lcrzo_epr(lcrzo_string_free2(&truepath));
    return(LCRZO_ERR_OK);
  }

  lcrzo_epr(lcrzo_string_free2(&truepath));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000358_display_dir(lcrzo_sock *psock,
			       lcrzo_conststring path,
			       lcrzo_conststring url,
			       lcrzo_int16 language)
{ lcrzo_data data;
  lcrzo_int32 datasize, count, i;
  lcrzo_string filename, tmp;
  lcrzo_int16 pathtype;
  int ret;

  lcrzo_epr(shared_http_initm_httpvererr("1.0", SHARED_HTTP_ERRORCODE_OK,
					 "OK", &data, &datasize));
  lcrzo_epr(shared_http_appendm_connection(NULL, datasize, &data,&datasize));
  lcrzo_epr(shared_http_appendm_contenttype(NULL, datasize,&data,&datasize));
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text("<HTML><BODY>\n", datasize,
				    &data, &datasize));
  if ( language==LCRZO_GLOBAL_LANG_FR)
  { lcrzo_epr(lcrzo_data_appendm_text("<H2>Index de ", 
				      datasize, &data, &datasize));
  }
  else
  { lcrzo_epr(lcrzo_data_appendm_text("<H2>Index of ", 
				      datasize, &data, &datasize));
  }
  lcrzo_epr(lcrzo_data_appendm_text(url, 
				    datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text("</H2><BR>\n", 
				    datasize, &data, &datasize));

  ret=lcrzo_dir_count(path, &count);
  if (ret!=LCRZO_ERR_OK)
  { if ( language==LCRZO_GLOBAL_LANG_FR)
    { lcrzo_epr(lcrzo_data_appendm_text("Impossible de lire le repertoire", 
					datasize, &data, &datasize));
    }
    else
    { lcrzo_epr(lcrzo_data_appendm_text("Could not read the directory", 
					datasize, &data, &datasize));
    }
    lcrzo_epr(lcrzo_sock_write(psock, data, datasize));
    lcrzo_epr(lcrzo_data_free2(&data));
    return(LCRZO_ERR_OK);
  }

  lcrzo_epr(lcrzo_data_appendm_text("<A HREF=\"..\">..</A><BR>\n",
				    datasize, &data, &datasize));

  for ( i=1 ; i<=count ; i++ )
  { ret=lcrzo_dir_value_pos(path, i, &filename);
    if (ret!=LCRZO_ERR_OK)
    { if ( language==LCRZO_GLOBAL_LANG_FR)
      { lcrzo_epr(lcrzo_data_appendm_text("Impossible de lire le repertoire", 
					  datasize, &data, &datasize));
      }
      else
      { lcrzo_epr(lcrzo_data_appendm_text("Could not read the directory", 
					  datasize, &data, &datasize));
      }
      break;
    }
    if ( filename[strlen(filename)-1]!='/' )
    { lcrzo_epr(lcrzo_string_initm_text(path, &tmp));
      lcrzo_epr(lcrzo_string_appendm_text("/", &tmp));
      lcrzo_epr(lcrzo_string_appendm_text(filename, &tmp));
      ret=lcrzo_path_type(tmp, &pathtype);
      lcrzo_epr(lcrzo_string_free2(&tmp));
      if ( ret==LCRZO_ERR_OK && pathtype==LCRZO_PATH_TYPE_DIR )
      { lcrzo_epr(lcrzo_string_appendm_text("/", &filename));
      }
    }
    lcrzo_epr(lcrzo_data_appendm_text("<A HREF=\"", 
				      datasize, &data, &datasize));
    lcrzo_epr(lcrzo_data_appendm_text(filename, 
				      datasize, &data, &datasize));
    lcrzo_epr(lcrzo_data_appendm_text("\">", 
				      datasize, &data, &datasize));
    lcrzo_epr(lcrzo_data_appendm_text(filename, 
				      datasize, &data, &datasize));
    lcrzo_epr(lcrzo_data_appendm_text("</A><BR>\n",
				      datasize, &data, &datasize));
    lcrzo_epr(lcrzo_string_free2(&filename));
  }
  lcrzo_epr(lcrzo_data_appendm_text("</BODY></HTML>\n", datasize,
				    &data, &datasize));
  lcrzo_epr(lcrzo_sock_write(psock, data, datasize));
  lcrzo_epr(lcrzo_data_free2(&data));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000358_display_file(lcrzo_sock *psock,
				lcrzo_conststring path,
				lcrzo_int16 language)
{ lcrzo_data data, tmp;
  lcrzo_int32 datasize, tmpsize;
  lcrzo_bool stdcontent;
  lcrzo_string ext;
  lcrzo_file file;
  char *pc;
  int ret;

  stdcontent=LCRZO_FALSE;
  pc=strrchr(path, '.');
  if (pc!=NULL)
  { lcrzo_epr(lcrzo_string_initm_text(pc+1, &ext));
    if ( !shared_misc_strcasecmp(ext, "txt") || 
	 !shared_misc_strcasecmp(ext, "htm") || 
	 !shared_misc_strcasecmp(ext, "html") )
    { stdcontent=LCRZO_TRUE;
    }
    lcrzo_epr(lcrzo_string_free2(&ext));
  }

  ret=lcrzo_file_open_read(path, &file);
  if (ret!=LCRZO_ERR_OK)
  { lcrzo_epr(shared_http_initm_httpvererr("1.0", 
					   SHARED_HTTP_ERRORCODE_FORBIDDEN,
					   "Forbidden", &data, &datasize));
    lcrzo_epr(shared_http_appendm_connection(NULL, datasize, &data,&datasize));
    lcrzo_epr(shared_http_appendm_contenttype(NULL, datasize,&data,&datasize));
    lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, &data, &datasize));
    lcrzo_epr(lcrzo_data_appendm_text("<HTML><BODY>\n", datasize,
				      &data, &datasize));
    if ( language==LCRZO_GLOBAL_LANG_FR)
    { lcrzo_epr(lcrzo_data_appendm_text("Impossible d'acceder au fichier ", 
					datasize, &data, &datasize));
    }
    else
    { lcrzo_epr(lcrzo_data_appendm_text("You don't have permission to access ",
					datasize, &data, &datasize));
    }
    lcrzo_epr(lcrzo_data_appendm_text(path,
				      datasize, &data, &datasize));
    lcrzo_epr(lcrzo_data_appendm_text("</BODY></HTML>\n", datasize,
				      &data, &datasize));

    lcrzo_epr(lcrzo_sock_write(psock, data, datasize));
    lcrzo_epr(lcrzo_data_free2(&data));
    return(LCRZO_ERR_OK);
  }

  lcrzo_epr(shared_http_initm_httpvererr("1.0", SHARED_HTTP_ERRORCODE_OK,
					 "OK", &data, &datasize));
  lcrzo_epr(shared_http_appendm_connection(NULL, datasize, &data,&datasize));
  if (stdcontent)
  { lcrzo_epr(shared_http_appendm_contenttype(NULL, datasize,&data,&datasize));
  }
  else
  { lcrzo_epr(shared_http_appendm_contenttype("application/octet-stream",
					      datasize, &data, &datasize));
  }
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, &data, &datasize));

  while(LCRZO_TRUE)
  { ret=lcrzo_file_readm(&file, LCRZO_WAIT_INFINITE, &tmp, &tmpsize);
    if (ret!=LCRZO_ERR_OK) break;
    lcrzo_epr(lcrzo_data_appendm_data(tmp, tmpsize, datasize,
				      &data, &datasize));
    lcrzo_epr(lcrzo_data_free2(&tmp));
  }

  lcrzo_epr(lcrzo_file_close(&file));
  lcrzo_epr(lcrzo_sock_write(psock, data, datasize));
  lcrzo_epr(lcrzo_data_free2(&data));
  return(LCRZO_ERR_OK);
}




