
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000364=
{ /*reference*/     "lcrzoex", 364,
  /*version*/       1, 0,
  /*title*/         "display TCP data flow",
  /*french title*/  "affiche le contenu de flux TCP",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "[device [\"bpf_filter\"]]",
  /*french usage*/  "[device [\"filtre_bpf\"]]",
  /*usage example*/ "eth0 \"tcp and host 1.2.3.4 and port 25\"",
  /*fr. usage ex.*/ "eth0 \"tcp and host 1.2.3.4 and port 25\"",
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000364_main(int argc, char *argv[]);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000364_main(int argc, char *argv[])
{ lcrzo_device device;
  lcrzo_uint32 mtu;
  lcrzo_sniff sniff;
  lcrzo_data sniffdata, data;
  lcrzo_int32 sniffdatasize;
  lcrzo_uint16 datasize;
  int ret;
  
  lcrzo_init();

  /* init device and mtu */
  lcrzo_epr(lcrzo_device_value_pos(LCRZO_DEVICE_TYPE_BEST, 1, device));
  if (argc>=2)
  { lcrzo_epr(shared_para_device_init_src(argv[1], device));
  }
  else
  { lcrzo_epr(lcrzo_device_value_pos(LCRZO_DEVICE_TYPE_BEST, 1, device));
  }
  lcrzo_epr(lcrzo_mtu_init_device(device, &mtu));

  /* init sniff */
  if (argc<=2)
  { lcrzo_epr(lcrzo_sniff_init(device, mtu, LCRZO_SNIFF_TYPE_TCPREOR, 
			       "tcp", &sniff));
  }
  else
  { lcrzo_string snifffilter;
    lcrzo_epr(shared_para_string_init_snifffilter(argv[2], &snifffilter));
    lcrzo_epr(lcrzo_sniff_init(device, mtu, LCRZO_SNIFF_TYPE_TCPREOR, 
			       snifffilter, &sniff));
    lcrzo_er(lcrzo_string_free2(&snifffilter));
  }

  /* main loop */
  ret=LCRZO_ERR_OK; /* just for the compiler warning */
  while(LCRZO_TRUE)
  { ret=lcrzo_sniff_nextm(&sniff, LCRZO_WAIT_INFINITE, 
			  &sniffdata, &sniffdatasize);
    if (ret!=LCRZO_ERR_OK) break;
    ret=lcrzo_packet_decodem_ethipopttcpoptdata(sniffdata, sniffdatasize,
						NULL, NULL, NULL, NULL,
						NULL, NULL, NULL,
						&data, &datasize);
    if (ret==LCRZO_ERR_OK)
    { lcrzo_epr(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_TEXT));
      lcrzo_er(lcrzo_data_free2(&data));
    }
    lcrzo_er(lcrzo_data_free2(&sniffdata));
  }

  /* close everything */
  lcrzo_epr(lcrzo_sniff_close(&sniff));
  if (ret!=LCRZO_ERR_OK) { lcrzo_epr(lcrzo_err_print(ret)); }

  lcrzo_close();
  return (ret);
}



