
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000365=
{ /*reference*/     "lcrzoex", 365,
  /*version*/       1, 0,
  /*title*/         "save TCP data flow in separate files",
  /*french title*/  "sauve le contenu de flux TCP dans des fichiers separes",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "directorycontainingfile [device [\"bpf_filter\"]]",
  /*french usage*/  "repertoirecontenantlesficheirs [device [\"filtre_bpf\"]]",
  /*usage example*/ "/tmp/dir/ eth0 \"tcp and host 1.2.3.4 and port 25\"",
  /*fr. usage ex.*/ "/tmp/dir eth0 \"tcp and host 1.2.3.4 and port 25\"",
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000365_main(int argc, char *argv[]);
int lcrzoex_000365_initm_filename(lcrzo_conststring directory,
				  lcrzo_bool reord,
				  lcrzo_ipl ipl1,
				  lcrzo_ipl ipl2,
				  lcrzo_uint16 port1,
				  lcrzo_uint16 port2,
				  lcrzo_string *pfilename);
int lcrzoex_000365_save(lcrzo_conststring directory,
			lcrzo_constdata sniffdata,
			lcrzo_int32 sniffdatasize);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000365_main(int argc, char *argv[])
{ lcrzo_device device;
  lcrzo_uint32 mtu;
  lcrzo_sniff sniff;
  lcrzo_data sniffdata;
  lcrzo_int32 sniffdatasize;
  lcrzo_string directory;
  int ret;
  
  lcrzo_init();
  if ( argc<2 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000365, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }
  lcrzo_epr(lcrzo_string_initm_text(argv[1], &directory));

  /* init device and mtu */
  if (argc>=3)
  { lcrzo_epr(shared_para_device_init_src(argv[2], device));
  }
  else
  { lcrzo_epr(lcrzo_device_value_pos(LCRZO_DEVICE_TYPE_BEST, 1, device));
  }
  lcrzo_epr(lcrzo_mtu_init_device(device, &mtu));

  /* init sniff */
  if (argc<=3)
  { lcrzo_epr(lcrzo_sniff_init(device, mtu, LCRZO_SNIFF_TYPE_TCPREOR, 
			       "tcp", &sniff));
  }
  else
  { lcrzo_string snifffilter;
    lcrzo_epr(shared_para_string_init_snifffilter(argv[3], &snifffilter));
    lcrzo_epr(lcrzo_sniff_init(device, mtu, LCRZO_SNIFF_TYPE_TCPREOR, 
			       snifffilter, &sniff));
    lcrzo_er(lcrzo_string_free2(&snifffilter));
  }

  /* main loop */
  ret=LCRZO_ERR_OK; /* just for the compiler warning */
  while(LCRZO_TRUE)
  { ret=lcrzo_sniff_nextm(&sniff, LCRZO_WAIT_INFINITE, 
			  &sniffdata, &sniffdatasize);
    if (ret!=LCRZO_ERR_OK) break;
    lcrzo_epr(lcrzoex_000365_save(directory, sniffdata, sniffdatasize));
    lcrzo_er(lcrzo_data_free2(&sniffdata));
  }

  /* close everything */
  lcrzo_epr(lcrzo_sniff_close(&sniff));
  if (ret!=LCRZO_ERR_OK) { lcrzo_epr(lcrzo_err_print(ret)); }
  lcrzo_er(lcrzo_string_free2(&directory)); 

  lcrzo_close();
  return (ret);
}

/*-------------------------------------------------------------*/
int lcrzoex_000365_initm_filename(lcrzo_conststring directory,
				  lcrzo_bool reord,
				  lcrzo_ipl ipl1,
				  lcrzo_ipl ipl2,
				  lcrzo_uint16 port1,
				  lcrzo_uint16 port2,
				  lcrzo_string *pfilename)
{ lcrzo_ips ips;
  lcrzo_ipl ipltmp;
  lcrzo_uint16 porttmp;

  if (reord)
  { if ( ipl2>ipl1 )
    { ipltmp=ipl2;
      ipl2=ipl1;
      ipl1=ipltmp;
    }
    if ( port2>port1 )
    { porttmp=port2;
      port2=port1;
      port1=porttmp;
    }
  }

  lcrzo_er(lcrzo_string_initm_text(directory, pfilename));
  lcrzo_er(lcrzo_string_appendm_text("/", pfilename));
  if (reord)
  { lcrzo_er(lcrzo_string_appendm_text("All_", pfilename));
  }
  lcrzo_er(lcrzo_ips_init_ipl(ipl1, ips));
  lcrzo_er(lcrzo_string_appendm_text(ips, pfilename));
  lcrzo_er(lcrzo_string_appendm_text("-", pfilename));
  lcrzo_er(lcrzo_ips_init_ipl(ipl2, ips));
  lcrzo_er(lcrzo_string_appendm_text(ips, pfilename));
  lcrzo_er(lcrzo_string_appendm_text("-", pfilename));
  lcrzo_er(lcrzo_string_appendm_uint16(port1, "%u", pfilename));
  lcrzo_er(lcrzo_string_appendm_text("-", pfilename));
  lcrzo_er(lcrzo_string_appendm_uint16(port2, "%u", pfilename));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000365_save(lcrzo_conststring directory,
			lcrzo_constdata sniffdata,
			lcrzo_int32 sniffdatasize)
{ lcrzo_data data;
  lcrzo_uint16 datasize;
  lcrzo_hdrlip hdrlip;
  lcrzo_hdrltcp hdrltcp;
  lcrzo_string filename;
  int ret;
  
  ret=lcrzo_packet_decodem_ethipopttcpoptdata(sniffdata, sniffdatasize,
					      NULL, &hdrlip, NULL, NULL,
					      &hdrltcp, NULL, NULL,
					      &data, &datasize);
  if (ret!=LCRZO_ERR_OK)
  { /* do not save it */
    printf(".");
    lcrzo_epr(lcrzo_stream_flush(stdout));
    return(LCRZO_ERR_OK);
  }
  printf("+");
  lcrzo_epr(lcrzo_stream_flush(stdout));

  /* put data in corresponding file */
  lcrzo_er(lcrzoex_000365_initm_filename(directory, LCRZO_FALSE,
					 hdrlip.saddr, hdrlip.daddr, 
					 hdrltcp.sport, hdrltcp.dport, 
					 &filename));
  lcrzo_er(lcrzo_filename_append(filename, LCRZO_FALSE, data, datasize));
  lcrzo_epr(lcrzo_string_free2(&filename));

  /* put data in corresponding file */
  lcrzo_er(lcrzoex_000365_initm_filename(directory, LCRZO_TRUE,
					 hdrlip.saddr, hdrlip.daddr, 
					 hdrltcp.sport, hdrltcp.dport, 
					 &filename));
  lcrzo_er(lcrzo_filename_append(filename, LCRZO_FALSE, data, datasize));
  lcrzo_epr(lcrzo_string_free2(&filename));

  lcrzo_er(lcrzo_data_free2(&data));
  return(LCRZO_ERR_OK);
}


