
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000370=
{ /*reference*/     "lcrzoex", 370,
  /*version*/       1, 0,
  /*title*/         "sniff and display web urls",
  /*french title*/  "sniffe et affiche les urls web",
  /*description*/   "note : does not work on fragmented data",
  /*french desc.*/  "note : ne marche pas avec des donnees fragmentees",
  /*usage*/         "[device [\"bpf_filter\"]]",
  /*french usage*/  "[device [\"filtre_bpf\"]]",
  /*usage example*/ "eth0 \"tcp and host 1.2.3.4 and (port 80 or port 8080)\"",
  /*fr. usage ex.*/ "eth0 \"tcp and host 1.2.3.4 and (port 80 or port 8080)\"",
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000370_main(int argc, char *argv[]);
int lcrzoex_000370_findurl(lcrzo_constdata data,
			   lcrzo_int32 datasize);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000370_main(int argc, char *argv[])
{ lcrzo_device device;
  lcrzo_uint32 mtu;
  lcrzo_sniff sniff;
  lcrzo_data sniffdata, data;
  lcrzo_int32 sniffdatasize;
  lcrzo_uint16 datasize;
  int ret;
  
  lcrzo_init();

  /* init device and mtu */
  if (argc>=2)
  { lcrzo_epr(shared_para_device_init_src(argv[1], device));
  }
  else
  { lcrzo_epr(lcrzo_device_value_pos(LCRZO_DEVICE_TYPE_BEST, 1, device));
  }
  lcrzo_epr(lcrzo_mtu_init_device(device, &mtu));

  /* init sniff */
  if (argc<=2)
  { lcrzo_epr(lcrzo_sniff_init(device, mtu, LCRZO_SNIFF_TYPE_TCPREOR, 
			       "tcp and port 80", &sniff));
  }
  else
  { lcrzo_string snifffilter;
    lcrzo_epr(shared_para_string_init_snifffilter(argv[2], &snifffilter));
    lcrzo_epr(lcrzo_sniff_init(device, mtu, LCRZO_SNIFF_TYPE_TCPREOR, 
			       snifffilter, &sniff));
    lcrzo_er(lcrzo_string_free2(&snifffilter));
  }

  /* main loop */
  ret=LCRZO_ERR_OK; /* just for the compiler warning */
  while(LCRZO_TRUE)
  { ret=lcrzo_sniff_nextm(&sniff, LCRZO_WAIT_INFINITE, 
			  &sniffdata, &sniffdatasize);
    if (ret!=LCRZO_ERR_OK) break;
    ret=lcrzo_packet_decodem_ethipopttcpoptdata(sniffdata, sniffdatasize,
						NULL, NULL, NULL, NULL,
						NULL, NULL, NULL,
						&data, &datasize);
    if (ret==LCRZO_ERR_OK)
    { lcrzo_er(lcrzoex_000370_findurl(data, datasize));
      lcrzo_er(lcrzo_data_free2(&data));
    }
    lcrzo_er(lcrzo_data_free2(&sniffdata));
  }

  /* close everything */
  lcrzo_epr(lcrzo_sniff_close(&sniff));
  if (ret!=LCRZO_ERR_OK) { lcrzo_epr(lcrzo_err_print(ret)); }

  lcrzo_close();
  return (ret);
}

/*-------------------------------------------------------------*/
int lcrzoex_000370_findurl(lcrzo_constdata data,
			   lcrzo_int32 datasize)
{ lcrzo_string line, url;
  lcrzo_int32 i, iend;
  lcrzo_uint8 c;
  lcrzo_regexp re;
  int ret;

  /*lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_TEXT);*/

  /* get the first line */
  iend=-1;
  for ( i=0 ; i<datasize ; i++ )
  { c=data[i];
    if ( c=='\0' ) return(LCRZO_ERR_OK);
    if ( c==0x0D || c==0x0A )
    { iend=i;
      break;
    }
  }
  if (iend==-1) return(LCRZO_ERR_OK);
  lcrzo_er(lcrzo_string_initm_data(data, iend, &line));

  /* search it */
  ret=lcrzo_string_search_regexp(line, LCRZO_FALSE, +1, -1,
				 "^[A-Za-z]+ (.*) HTTP/...$", re);
  if (ret!=LCRZO_ERR_OK)
  { lcrzo_er(lcrzo_string_free2(&line));
    return(LCRZO_ERR_OK);
  }

  ret=lcrzo_string_initm_range(line, re[1].startindex_positive,
			       re[1].endindex_positive, &url);
  lcrzo_er(lcrzo_string_free2(&line));
  if (ret!=LCRZO_ERR_OK)
  { return(LCRZO_ERR_OK);
  }

  /* display url */
  printf("%s\n", url);
  lcrzo_er(lcrzo_string_free2(&url));

  return(LCRZO_ERR_OK);
}
