
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000372=
{ /*reference*/     "lcrzoex", 372,
  /*version*/       1, 0,
  /*title*/         "split a file in several parts",
  /*french title*/  "separe un fichier en plusieurs morceaux",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "file directory_where_to_put_slices slices_size",
  /*french usage*/  "fichier repertoire_ou_mettre_les_morceaux taille_morceaux",
  /*usage example*/ "myfile /tmp/dir/ 1400000",
  /*fr. usage ex.*/ "monfichier /tmp/dir/ 1400000",
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declaration */
int lcrzoex_000372_main(int argc, char *argv[]);
int lcrzoex_000372_write_uint32(lcrzo_file *pfile,
				lcrzo_uint32 number);
int lcrzoex_000372_opennewfile(lcrzo_conststring outputdirectoryname,
			       lcrzo_uint32 slicenumber,
			       lcrzo_file *pfile);
int lcrzoex_000372_slice(lcrzo_conststring inputfilename,
			 lcrzo_conststring outputdirectoryname,
			 lcrzo_uint32 slicesize);
#define LCRZOEX_000372_HEADERSIZE 16

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000372_main(int argc, char *argv[])
{ lcrzo_uint32 slicesize;
  lcrzo_string inputfilename, outputdirectoryname;
  int ret;

  lcrzo_init();
  if ( argc<4 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000372, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }

  /* initialize values */
  lcrzo_epr(shared_para_string_init_filein(argv[1], &inputfilename));
  lcrzo_epr(shared_para_string_init_dirlocal(argv[2], &outputdirectoryname));
  lcrzo_epr(lcrzo_uint32_init_text(argv[3], &slicesize));

  /* split files */
  ret=lcrzoex_000372_slice(inputfilename, outputdirectoryname, slicesize);

  /* close everything */
  lcrzo_epr(lcrzo_string_free2(&outputdirectoryname));
  lcrzo_epr(lcrzo_string_free2(&inputfilename));

  lcrzo_close();
  return (ret);
}

/*-------------------------------------------------------------*/
int lcrzoex_000372_opennewfile(lcrzo_conststring outputdirectoryname,
			       lcrzo_uint32 slicenumber,
			       lcrzo_file *pfile)
{ lcrzo_string outputfilename;

  lcrzo_er(lcrzo_string_initm_text(outputdirectoryname, &outputfilename));
  lcrzo_er(lcrzo_string_appendm_text("/", &outputfilename));
  lcrzo_er(lcrzo_string_appendm_uint32(slicenumber, "%010u", &outputfilename));
  lcrzo_er(lcrzo_string_appendm_text(".dat", &outputfilename));
  lcrzo_er(lcrzo_file_open_write(outputfilename, LCRZO_FALSE, LCRZO_TRUE,
				 pfile));
  lcrzo_er(lcrzo_string_free2(&outputfilename));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000372_write_uint32(lcrzo_file *pfile,
				lcrzo_uint32 number)
{ lcrzo_data data;

  lcrzo_er(lcrzo_data_alloc(4, &data));
  data[0]=(lcrzo_uint8)(number>>24);
  data[1]=(lcrzo_uint8)((number>>16)&0xff);
  data[2]=(lcrzo_uint8)((number>>8)&0xff);
  data[3]=(lcrzo_uint8)(number&0xff);
  lcrzo_er(lcrzo_file_write(pfile, data, 4));
  lcrzo_er(lcrzo_data_free2(&data));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000372_slice(lcrzo_conststring inputfilename,
			 lcrzo_conststring outputdirectoryname,
			 lcrzo_uint32 slicesize)
{ lcrzo_int16 type, language;
  lcrzo_int32 filesize, datasize, leftdatasize;
  lcrzo_int32 writtenbytesinfile, firstpartsize;
  lcrzo_uint32 numberofslices, slicenumber;
  lcrzo_data data, leftdata;
  lcrzo_file inputfile, outputfile;
  lcrzo_bool needtoopenfile;
  int ret;

  lcrzo_epr(lcrzo_global_get_language(&language));
 
  /* check if input file exists */
  ret=lcrzo_path_type(inputfilename, &type);
  if ( ret==LCRZO_ERR_OKFILENOTFOUND )
  { if ( language==LCRZO_GLOBAL_LANG_FR )
      printf("Fichier %s non trouve.\n", inputfilename);
    else
      printf("File %s not found.\n", inputfilename);
    return(ret);
  }
  else if ( ret==LCRZO_ERR_OK )
  { if (type!=LCRZO_PATH_TYPE_REG)
    { if ( language==LCRZO_GLOBAL_LANG_FR )
        printf("Fichier %s n'est pas regulier.\n", inputfilename);
      else
        printf("File %s is not regular.\n", inputfilename);
      return(ret);
    }
  }
  else
  { return(ret);
  }

  /* check output directory */
  ret=lcrzo_path_type(outputdirectoryname, &type);
  if ( ret==LCRZO_ERR_OKFILENOTFOUND )
  { if ( language==LCRZO_GLOBAL_LANG_FR )
      printf("Repertoire %s non trouve. Il sera cree.\n", outputdirectoryname);
    else
      printf("Directory %s not found. It will be created.\n", 
	     outputdirectoryname);
  }
  else if ( ret==LCRZO_ERR_OK )
  { if (type!=LCRZO_PATH_TYPE_DIR)
    { if ( language==LCRZO_GLOBAL_LANG_FR )
        printf("Fichier %s n'est pas un repertoire.\n", outputdirectoryname);
      else
        printf("File %s is not a directory.\n", outputdirectoryname);
      return(ret);
    }
  }
  else
  { return(ret);
  }

  /* check slicesize */
  if (slicesize<LCRZOEX_000372_HEADERSIZE)
  { lcrzo_er(lcrzo_uint32_print("Slicesize(", slicesize, "%u", ") "));
    if ( language==LCRZO_GLOBAL_LANG_FR )
      printf("est trop petit");
    else
      printf("is too small");
    printf(" (minimum=%d).\n", LCRZOEX_000372_HEADERSIZE);
    return(ret);
  }

  /* obtain file size */
  lcrzo_er(lcrzo_filename_size(inputfilename, &filesize));

  /* compute number of slices */
  numberofslices=(filesize+LCRZOEX_000372_HEADERSIZE+slicesize-1)/slicesize;

  /* open input file */
  lcrzo_er(lcrzo_file_open_read(inputfilename, &inputfile));

  /* open the first file */
  slicenumber=1;
  lcrzo_er(lcrzoex_000372_opennewfile(outputdirectoryname, slicenumber,
				      &outputfile));

  /* in the firsts bytes, put numberofslices */
  writtenbytesinfile=0;
  lcrzo_er(lcrzoex_000372_write_uint32(&outputfile, 0x0D0A0A0Au));
  writtenbytesinfile+=4;
  lcrzo_er(lcrzoex_000372_write_uint32(&outputfile, numberofslices));
  writtenbytesinfile+=4;
  lcrzo_er(lcrzoex_000372_write_uint32(&outputfile, slicesize));
  writtenbytesinfile+=4;
  lcrzo_er(lcrzoex_000372_write_uint32(&outputfile, filesize));
  writtenbytesinfile+=4;
  if ( writtenbytesinfile!=LCRZOEX_000372_HEADERSIZE )
  { return(LCRZO_ERR_IEINTERNALERROR);
  }

  needtoopenfile=LCRZO_FALSE;
  while(LCRZO_TRUE)
  { /* read more data */
    ret=lcrzo_file_readm(&inputfile, LCRZO_WAIT_INFINITE, &data, &datasize);
    if (ret!=LCRZO_ERR_OK) break;
    if (datasize==0)
    { lcrzo_er(lcrzo_data_free2(&data));
      continue;
    }
    leftdata=data;
    leftdatasize=datasize;
    while(leftdatasize)
    { /* eventually open the file */
      if (needtoopenfile)
      { slicenumber++;
        lcrzo_er(lcrzoex_000372_opennewfile(outputdirectoryname, slicenumber,
					    &outputfile));
	writtenbytesinfile=0;
	needtoopenfile=LCRZO_FALSE;
      }
      /* write it */
      if ( (lcrzo_uint32)(writtenbytesinfile+leftdatasize)<slicesize )
      { /* data feet in the file */
	lcrzo_er(lcrzo_file_write(&outputfile, leftdata, leftdatasize));
	writtenbytesinfile+=leftdatasize;
	leftdata+=leftdatasize;
	leftdatasize=0;
      }
      else if ( (lcrzo_uint32)(writtenbytesinfile+leftdatasize)==slicesize )
      { /* data just feet in the file */
        lcrzo_er(lcrzo_file_write(&outputfile, leftdata, leftdatasize));
        lcrzo_er(lcrzo_file_close(&outputfile));
        needtoopenfile=LCRZO_TRUE;
	leftdata+=leftdatasize;
	leftdatasize=0;
      }
      else
      { /* data does not feet in the file */
        firstpartsize=slicesize-writtenbytesinfile;
	lcrzo_er(lcrzo_file_write(&outputfile, leftdata, firstpartsize));
	lcrzo_er(lcrzo_file_close(&outputfile));
        needtoopenfile=LCRZO_TRUE;
	leftdata+=firstpartsize;
	leftdatasize-=firstpartsize;
      }
    }
    /* free read data */
    lcrzo_er(lcrzo_data_free2(&data));
  }
  if (ret==LCRZO_ERR_OKDATAEND) ret=LCRZO_ERR_OK;

  /* close files */
  if (!needtoopenfile)
  { lcrzo_er(lcrzo_file_close(&outputfile));
  }
  lcrzo_er(lcrzo_file_close(&inputfile));

  return(ret);
}
