
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000373=
{ /*reference*/     "lcrzoex", 373,
  /*version*/       1, 0,
  /*title*/         "reassemble a file from several parts",
  /*french title*/  "reassemble un fichier depuis plusieurs morceaux",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "directory_containing_slices output_file",
  /*french usage*/  "repertoire_contenant_les_morceaux fichier_de_sortie",
  /*usage example*/ "/tmp/dir/ /tmp/output",
  /*fr. usage ex.*/ "/tmp/dir/ /tmp/fichsortie",
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declaration */
int lcrzoex_000373_main(int argc, char *argv[]);
int lcrzoex_000373_read_uint32(lcrzo_file *pfile,
			       lcrzo_uint32 *pnumber);
int lcrzoex_000373_filename(lcrzo_conststring inputdirectoryname,
			    lcrzo_uint32 slicenumber,
			    lcrzo_string *pfilename);
int lcrzoex_000373_openfile(lcrzo_conststring inputdirectoryname,
			    lcrzo_uint32 slicenumber,
			    lcrzo_file *pfile);
int lcrzoex_000373_slice(lcrzo_conststring inputdirectoryname,
			 lcrzo_conststring outputfilename);
#define LCRZOEX_000373_HEADERSIZE 16

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000373_main(int argc, char *argv[])
{ lcrzo_string outputfilename, inputdirectoryname;
  int ret;

  lcrzo_init();
  if ( argc<3 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000373, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }

  /* initialize values */
  lcrzo_epr(shared_para_string_init_dirlocal(argv[1], &inputdirectoryname));
  lcrzo_epr(shared_para_string_init_filein(argv[2], &outputfilename));

  /* split files */
  ret=lcrzoex_000373_slice(inputdirectoryname, outputfilename);

  /* close everything */
  lcrzo_epr(lcrzo_string_free2(&outputfilename));
  lcrzo_epr(lcrzo_string_free2(&inputdirectoryname));

  lcrzo_close();
  return (ret);
}

/*-------------------------------------------------------------*/
int lcrzoex_000373_read_uint32(lcrzo_file *pfile,
			       lcrzo_uint32 *pnumber)
{ lcrzo_data data;
  lcrzo_uint32 number;

  lcrzo_er(lcrzo_file_readm_fixed(pfile, LCRZO_WAIT_INFINITE, 4, &data));
  number=(lcrzo_uint32) ( (((lcrzo_uint8)data[0])<<24) |
			  (((lcrzo_uint8)data[1])<<16) |
			  (((lcrzo_uint8)data[2])<<8) |
			  ((lcrzo_uint8)data[3]) );
  lcrzo_er(lcrzo_data_free2(&data));

  if (pnumber!=NULL) *pnumber=number;
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000373_filename(lcrzo_conststring inputdirectoryname,
			    lcrzo_uint32 slicenumber,
			    lcrzo_string *pfilename)
{ lcrzo_string inputfilename;

  lcrzo_er(lcrzo_string_initm_text(inputdirectoryname, &inputfilename));
  lcrzo_er(lcrzo_string_appendm_text("/", &inputfilename));
  lcrzo_er(lcrzo_string_appendm_uint32(slicenumber, "%010u", &inputfilename));
  lcrzo_er(lcrzo_string_appendm_text(".dat", &inputfilename));
  lcrzo_er(lcrzo_string_initm_text(inputfilename, pfilename));
  lcrzo_er(lcrzo_string_free2(&inputfilename));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000373_openfile(lcrzo_conststring inputdirectoryname,
			    lcrzo_uint32 slicenumber,
			    lcrzo_file *pfile)
{ lcrzo_string inputfilename;
  lcrzo_er(lcrzoex_000373_filename(inputdirectoryname, slicenumber, 
				   &inputfilename));
  lcrzo_er(lcrzo_file_open_read(inputfilename, pfile));
  lcrzo_er(lcrzo_string_free2(&inputfilename));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000373_slice(lcrzo_conststring inputdirectoryname,
			 lcrzo_conststring outputfilename)
{ lcrzo_int16 type, language;
  lcrzo_uint32 numberofslices, i, pattern, outputfilesize, slicesize;
  lcrzo_int32 datasize, inputfilesize, wantedfilesize;
  lcrzo_data data;
  lcrzo_file inputfile, outputfile;
  lcrzo_bool inputfileexists;
  lcrzo_string inputfilename;
  int ret;

  lcrzo_epr(lcrzo_global_get_language(&language));

  /* check input directory */
  ret=lcrzo_path_type(inputdirectoryname, &type);
  if ( ret==LCRZO_ERR_OKFILENOTFOUND )
  { if ( language==LCRZO_GLOBAL_LANG_FR )
      printf("Repertoire %s non trouve.\n", inputdirectoryname);
    else
      printf("Directory %s not found.\n", inputdirectoryname);
      return(ret);
  }
  else if ( ret==LCRZO_ERR_OK )
  { if (type!=LCRZO_PATH_TYPE_DIR)
    { if ( language==LCRZO_GLOBAL_LANG_FR )
        printf("Fichier %s n'est pas un repertoire.\n", inputdirectoryname);
      else
        printf("File %s is not a directory.\n", inputdirectoryname);
      return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
    }
  }
  else
  { return(ret);
  }
  lcrzo_er(lcrzoex_000373_filename(inputdirectoryname, 1, &inputfilename));
  ret=lcrzo_filename_exist(inputfilename, &inputfileexists);
  if ( ret!=LCRZO_ERR_OK || !inputfileexists )
  { if ( language==LCRZO_GLOBAL_LANG_FR )
      printf("Fichier %s inexistant.\n", inputfilename);
    else
      printf("File %s absent.\n", inputfilename);
    lcrzo_er(lcrzo_string_free2(&inputfilename));
    return(ret);
  }
  lcrzo_er(lcrzo_string_free2(&inputfilename));

  /* check output file */
  ret=lcrzo_path_type(outputfilename, &type);
  if ( ret==LCRZO_ERR_OKFILENOTFOUND )
  { /*ok*/
  }
  else if ( ret==LCRZO_ERR_OK )
  { if ( language==LCRZO_GLOBAL_LANG_FR )
      printf("Fichier %s existe deja.\n", outputfilename);
    else
      printf("File %s already exists.\n", outputfilename);
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }
  else
  { return(ret);
  }

  /* read header */
  lcrzo_er(lcrzoex_000373_filename(inputdirectoryname, 1, &inputfilename));
  lcrzo_er(lcrzo_filename_size(inputfilename, &inputfilesize));
  lcrzo_er(lcrzo_string_free2(&inputfilename));
  if (inputfilesize<LCRZOEX_000373_HEADERSIZE)
  { lcrzo_er(lcrzo_uint32_print("First_file_size(", inputfilesize,
				"%u", ") "));
    if ( language==LCRZO_GLOBAL_LANG_FR )
      printf("est trop petit");
    else
      printf("is too small");
    printf(" (minimum=%d).\n", LCRZOEX_000373_HEADERSIZE);
    return(ret);
  }
  lcrzo_er(lcrzoex_000373_openfile(inputdirectoryname, 1, &inputfile));
  lcrzo_er(lcrzoex_000373_read_uint32(&inputfile, &pattern));
  lcrzo_er(lcrzoex_000373_read_uint32(&inputfile, &numberofslices));
  lcrzo_er(lcrzoex_000373_read_uint32(&inputfile, &slicesize));
  lcrzo_er(lcrzoex_000373_read_uint32(&inputfile, &outputfilesize));
  /*printf("%08lX %lu %lu %lu\n", pattern, numberofslices,
    slicesize, outputfilesize);*/
  lcrzo_er(lcrzo_file_close(&inputfile));

  /* check all those values */
  if (pattern!=0x0D0A0A0Au)
  { if ( language==LCRZO_GLOBAL_LANG_FR )
      printf("Fichier 1 corrompu (modif initial 0x0D0A0A0A a change).\n");
    else
      printf("Corrupted file 1 (initial pattern 0x0D0A0A0A changed).\n");
    return(ret);
  }
  if ( numberofslices!=
       (outputfilesize+LCRZOEX_000372_HEADERSIZE+slicesize-1)/slicesize )
  { if ( language==LCRZO_GLOBAL_LANG_FR )
      printf("Fichier 1 corrompu (mauvais calcul de numberofslices).\n");
    else
      printf("Corrupted file 1 (bad reck on of numberofslices).\n");
    return(ret);
  }
  for ( i=1 ; i<=numberofslices ; i++ )
  { lcrzo_er(lcrzoex_000373_filename(inputdirectoryname, i, &inputfilename));
    ret=lcrzo_filename_exist(inputfilename, &inputfileexists);
    if ( ret!=LCRZO_ERR_OK || !inputfileexists )
    { if ( language==LCRZO_GLOBAL_LANG_FR )
        printf("Fichier %s inexistant.\n", inputfilename);
      else
        printf("File %s absent.\n", inputfilename);
      lcrzo_er(lcrzo_string_free2(&inputfilename));
      return(ret);
    }
    lcrzo_er(lcrzo_filename_size(inputfilename, &inputfilesize));
    if (i!=numberofslices)
      wantedfilesize=slicesize;
    else
      wantedfilesize=outputfilesize+LCRZOEX_000372_HEADERSIZE
	-(numberofslices-1)*slicesize;
    if ( inputfilesize!=wantedfilesize )
    { if ( language==LCRZO_GLOBAL_LANG_FR )
        printf("Taille du fichier %s incorrecte ", inputfilename);
      else
        printf("File size for %s incorrect ", inputfilename);
      lcrzo_er(lcrzo_string_free2(&inputfilename));
      lcrzo_er(lcrzo_uint32_print("(", inputfilesize, "%u", "!="));
      lcrzo_er(lcrzo_uint32_print("", wantedfilesize, "%u", ")\n"));
      return(ret);
    }
    lcrzo_er(lcrzo_string_free2(&inputfilename));
  }

  /* create output file */
  lcrzo_er(lcrzo_file_open_write(outputfilename, LCRZO_FALSE, LCRZO_TRUE,
				 &outputfile));
  ret=LCRZO_ERR_OK;
  for ( i=1 ; i<=numberofslices && ret==LCRZO_ERR_OK ; i++ )
  { lcrzo_er(lcrzoex_000373_openfile(inputdirectoryname, i, &inputfile));
    if (i==1)
    { /* skip header */
      lcrzo_er(lcrzo_file_readm_fixed(&inputfile, LCRZO_WAIT_INFINITE,
				      LCRZOEX_000373_HEADERSIZE, NULL));
    }
    ret=LCRZO_ERR_OK;
    while(ret==LCRZO_ERR_OK)
    { ret=lcrzo_file_readm(&inputfile, LCRZO_WAIT_INFINITE, &data, &datasize);
      if (ret!=LCRZO_ERR_OK) continue;
      lcrzo_er(lcrzo_file_write(&outputfile, data, datasize));
      lcrzo_er(lcrzo_data_free2(&data));
    }
    if (ret==LCRZO_ERR_OKDATAEND) ret=LCRZO_ERR_OK;
    lcrzo_er(lcrzo_file_close(&inputfile));
  }

  /* close file */
  lcrzo_er(lcrzo_file_close(&outputfile));

  return(ret);
}
