
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000382=
{ /*reference*/     "lcrzoex", 382,
  /*version*/       1, 0,
  /*title*/         "display the error code of a web page",
  /*french title*/  "affiche le code d'erreur d'une page web",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "http://sitelogin:sitepass@site:port/file [http://proxylogin:proxypass@proxy:port/]",
  /*french usage*/  "http://sitelogin:sitepass@site:port/fichier [http://proxylogin:proxypass@proxy:port/]",
  /*usage example*/ "http://server:81/index.html http://bob:pass@proxy:8080/",
  /*fr. usage ex.*/ "http://serveur:81/index.html http://laurent:passe@proxy:8080/",
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000382_main(int argc, char *argv[]);
int lcrzoex_000382_head(lcrzo_conststring siteurl, 
			lcrzo_conststring proxyurl);
int lcrzoex_000382_head2(lcrzo_conststring server, 
			 lcrzo_uint16 port,
			 lcrzo_conststring url,
			 lcrzo_conststring proxylogin,
			 lcrzo_conststring proxypasswd,
			 lcrzo_conststring sitelogin,
			 lcrzo_conststring sitepasswd);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000382_main(int argc, char *argv[])
{ lcrzo_string urlsite, urlproxy;
  int ret;

  lcrzo_init();
  if ( argc<2 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000382, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }

  /* the default resolver timeout is too short if it's long to reach
     webserver's DNS */
  lcrzo_epr(shared_http_set_resolver_timeout());

  /* set parameters */
  lcrzo_epr(lcrzo_string_initm_text(argv[1], &urlsite));
  if ( argc<3 )
  { lcrzo_epr(lcrzo_string_initm_text("", &urlproxy));
  }
  else
  { lcrzo_epr(lcrzo_string_initm_text(argv[2], &urlproxy));
  }

  ret=lcrzoex_000382_head(urlsite, urlproxy);
  if (ret!=LCRZO_ERR_OK) { lcrzo_epr(lcrzo_err_print(ret)); }

  lcrzo_epr(lcrzo_string_free2(&urlsite));
  lcrzo_epr(lcrzo_string_free2(&urlproxy));

  lcrzo_close();
  return(ret);
}

/*-------------------------------------------------------------*/
int lcrzoex_000382_head(lcrzo_conststring siteurl, 
			lcrzo_conststring proxyurl)
{ lcrzo_string url;
  lcrzo_uint16 siteport, proxyport;
  lcrzo_string siteproto, sitelogin, sitepasswd, siteserver, sitefile;
  lcrzo_string proxyproto, proxylogin, proxypasswd, proxyserver;
  lcrzo_string sitevarval;
  lcrzo_bool useproxy;
  int ret; 

  /* extract parts of the url */
  lcrzo_er(shared_http_url_decode(siteurl, &siteproto, &sitelogin,
				  &sitepasswd, &siteserver, &siteport,
				  &sitefile, &sitevarval, NULL)); 
  ret=shared_http_url_decode(proxyurl, &proxyproto, &proxylogin,
			     &proxypasswd, &proxyserver, &proxyport,
			     NULL, NULL, NULL); 
  useproxy=LCRZO_TRUE;
  if (ret!=LCRZO_ERR_OK)
  { useproxy=LCRZO_FALSE;
  }

  if ( useproxy )
  { /* use proxy */
    lcrzo_er(shared_http_url_init(siteproto, NULL, NULL, siteserver, siteport,
				  sitefile, sitevarval, NULL, &url));
    ret=lcrzoex_000382_head2(proxyserver, proxyport,
			     url,
			     proxylogin, proxypasswd,
			     sitelogin, sitepasswd);
    lcrzo_er(lcrzo_string_free2(&url));
  }
  else
  { if ( !strcmp(siteproto, "http") )
    { /* connect to the server */
      lcrzo_er(shared_http_dp_init(sitefile, sitevarval, NULL, &url));
      ret=lcrzoex_000382_head2(siteserver, siteport,
			       url,
			       NULL, NULL,
			       sitelogin, sitepasswd);
      lcrzo_er(lcrzo_string_free2(&url));
    }
    else
    { ret=LCRZO_ERR_IEUNIMPLEMENTED;
    }
  }

  /* free variables */
  lcrzo_er(lcrzo_string_free2(&siteproto));
  if (sitelogin!=NULL) { lcrzo_er(lcrzo_string_free2(&sitelogin)); }
  if (sitepasswd!=NULL) { lcrzo_er(lcrzo_string_free2(&sitepasswd)); }
  lcrzo_er(lcrzo_string_free2(&siteserver));
  lcrzo_er(lcrzo_string_free2(&sitefile));
  lcrzo_er(lcrzo_string_free2(&sitevarval));
  if (useproxy)
  { lcrzo_er(lcrzo_string_free2(&proxyproto));
    if (proxylogin!=NULL) { lcrzo_er(lcrzo_string_free2(&proxylogin)); }
    if (proxypasswd!=NULL) { lcrzo_er(lcrzo_string_free2(&proxypasswd)); }
    lcrzo_er(lcrzo_string_free2(&proxyserver));
  }
  return(ret);
}

/*-------------------------------------------------------------*/
int lcrzoex_000382_head2(lcrzo_conststring server, 
			 lcrzo_uint16 port,
			 lcrzo_conststring url,
			 lcrzo_conststring proxylogin,
			 lcrzo_conststring proxypasswd,
			 lcrzo_conststring sitelogin,
			 lcrzo_conststring sitepasswd)
{ lcrzo_ipl ipl;
  lcrzo_sock sock;
  lcrzo_string errormsg, servertype, contenttype, location;
  lcrzo_data header;
  lcrzo_int32 headersize, contentlength;
  lcrzo_uint16 errorcode;

  /* open the connection */
  lcrzo_er(lcrzo_ipl_init_hs(server, &ipl));
  lcrzo_er(lcrzo_sock_tcpcli_real(ipl, port, &sock));
  
  /* send a request */
  lcrzo_er(shared_http_request_get(&sock, url, server, port,
				   proxylogin, proxypasswd, 
				   sitelogin, sitepasswd));

  /* analyze the answer */
  lcrzo_er(shared_http_response_readheader(&sock,
					   &errorcode,
					   &errormsg,
					   &servertype,
					   &contenttype,
					   &contentlength,
					   &location,
					   &header, &headersize));

  lcrzo_er(lcrzo_uint16_print("", errorcode, "%u", "\n"));

  lcrzo_er(lcrzo_string_free2(&errormsg));
  lcrzo_er(lcrzo_string_free2(&servertype));
  lcrzo_er(lcrzo_string_free2(&contenttype));
  lcrzo_er(lcrzo_string_free2(&location));
  lcrzo_er(lcrzo_data_free2(&header));

  lcrzo_er(lcrzo_sock_close(&sock));

  return(LCRZO_ERR_OK);
}






