
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000383=
{ /*reference*/     "lcrzoex", 383,
  /*version*/       1, 0,
  /*title*/         "sniff packets and execute a command for each one",
  /*french title*/  "sniffe des paquets et execute une commande pour chacun",
  /*description*/   "Note : packet data is send on stdin of program",
  /*french desc.*/  "Note : les donnees du paquet sont envoyees sur le stdin du programme",
  /*usage*/         "command [device [\"bpf_filter\"]]",
  /*french usage*/  "commande [device [\"filtre_bpf\"]]",
  /*usage example*/ "\"touch /tmp/file\" eth0 \"host 1.2.3.4 and port 25\"",
  /*fr. usage ex.*/ "\"touch /tmp/fich\" eth0 \"host 1.2.3.4 and port 25\"", 
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declaration */
int lcrzoex_000383_main(int argc, char *argv[]);
int lcrzoex_000383_exec(lcrzo_constdata data,
			lcrzo_int32 datasize,
			lcrzo_conststring command);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000383_main(int argc, char *argv[])
{ lcrzo_device device;
  lcrzo_uint32 mtu;
  lcrzo_sniff sniff;
  lcrzo_data sniffdata;
  lcrzo_int32 sniffdatasize;
  int ret;

  lcrzo_init();
  if ( argc<2 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000383, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }

  /* init device and mtu */
  if (argc>=3)
  { lcrzo_epr(shared_para_device_init_src(argv[2], device));
  }
  else
  { lcrzo_epr(lcrzo_device_value_pos(LCRZO_DEVICE_TYPE_BEST, 1, device));
  }
  lcrzo_epr(lcrzo_mtu_init_device(device, &mtu));

  /* init sniff */
  if (argc<=3)
  { lcrzo_epr(lcrzo_sniff_init(device, mtu, LCRZO_SNIFF_TYPE_NORMAL, 
			       "", &sniff));
  }
  else
  { lcrzo_string snifffilter;
    lcrzo_epr(shared_para_string_init_snifffilter(argv[3], &snifffilter));
    lcrzo_epr(lcrzo_sniff_init(device, mtu, LCRZO_SNIFF_TYPE_NORMAL,
			       snifffilter, &sniff));
    lcrzo_er(lcrzo_string_free2(&snifffilter));
  }

  /* main loop */
  ret=LCRZO_ERR_OK;
  while(LCRZO_TRUE)
  { ret=lcrzo_sniff_nextm(&sniff, LCRZO_WAIT_INFINITE, 
			  &sniffdata, &sniffdatasize);
    if (ret!=LCRZO_ERR_OK) break;
    ret=lcrzoex_000383_exec(sniffdata, sniffdatasize, argv[1]);
    if (ret!=LCRZO_ERR_OK) break;
    lcrzo_epr(lcrzo_data_free2(&sniffdata));
  }

  /* close everything */
  lcrzo_epr(lcrzo_sniff_close(&sniff));

  lcrzo_close();
  return(ret);
}

/*-------------------------------------------------------------*/
int lcrzoex_000383_exec(lcrzo_constdata sniffdata,
			lcrzo_int32 sniffdatasize,
			lcrzo_conststring command)
{ lcrzo_data data;
  lcrzo_int32 datasize;
  lcrzo_ipc ipc;
  shared_misc_ipcexec smi;
  int ret;

  /* execute the given command */
  lcrzo_er(shared_misc_ipcexec_init(command, &smi, &ipc));

  /* write all data */
  lcrzo_er(lcrzo_ipc_write(&ipc, sniffdata, sniffdatasize));
  lcrzo_er(lcrzo_ipc_close_write(&ipc));

  /* read all data */
  while ( LCRZO_TRUE ) 
  { ret=lcrzo_ipc_readm(&ipc, LCRZO_WAIT_INFINITE, &data, &datasize);
    if ( ret!=LCRZO_ERR_OK )
    { if (ret==LCRZO_ERR_OKDATAEND) break;
      return(ret);
    }
    lcrzo_er(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_TEXT));
    lcrzo_er(lcrzo_data_free2(&data));
  }
  lcrzo_er(lcrzo_ipc_close(&ipc));
  lcrzo_er(shared_misc_ipcexec_close(&smi));

  return(LCRZO_ERR_OK);
}
