
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"
#include "shared/shared_toolbox.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000389=
{ /*reference*/     "lcrzoex", 389,
  /*version*/       1, 0,
  /*title*/         "run a command specified in a file",
  /*french title*/  "execute une commande specifiee dans un fichier",
  /*description*/   "commandfile contains for example \"2 www.s.com\"",
  /*french desc.*/  "fichierdecommande contient par exemple \"2 www.s.com\"",
  /*usage*/         "commandfile",
  /*french usage*/  "fichierdecommande",
  /*usage example*/ "/tmp/file",
  /*fr. usage ex.*/ "/tmp/fichier",
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000389_main(int argc, char *argv[]);
int lcrzoex_000389_readfile(lcrzo_conststring filename,
			    lcrzo_list *plist);
int lcrzoex_000389_readfile_line(lcrzo_conststring line,
				 lcrzo_list *plist);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000389_main(int argc, char *argv[])
{ lcrzo_uint32 toolnumber;
  lcrzo_list list;
  lcrzo_int32 count, i;
  lcrzo_string str;
  lcrzo_int16 language;
  int ac, ret;
  char **av;

  lcrzo_init();
  if ( argc<2 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000389, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }
  lcrzo_epr(lcrzo_global_get_language(&language));

  /* read the command */
  lcrzo_epr(lcrzo_stringlist_init(&list));
  lcrzo_epr(lcrzoex_000389_readfile(argv[1], &list));

  /* convert it to an array list argc/argv */
  lcrzo_epr(lcrzo_list_count(&list, &count));
  if (count==0)
  { if (language==LCRZO_GLOBAL_LANG_FR) 
      printf("Le fichier %s ne contient pas de commande", argv[1]);
    else
      printf("File %s contains no command", argv[1]);
    lcrzo_epr(lcrzo_list_close(&list));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }

  /* set tool number */
  lcrzo_epr(lcrzo_stringlist_valuem_pos(&list, 1, NULL, &str));
  ret=lcrzo_uint32_init_text(str, &toolnumber);
  if (ret!=LCRZO_ERR_OK)
  { if (language==LCRZO_GLOBAL_LANG_FR) 
      printf("Le premier parametre (%s) n'est pas un nombre\n", str);
    else
      printf("The first parameter (%s) is not a number\n", str);
    lcrzo_epr(lcrzo_list_close(&list));
    lcrzo_epr(lcrzo_string_free2(&str));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }
  lcrzo_epr(lcrzo_string_free2(&str));
  lcrzo_epr(lcrzo_list_remove_pos(&list, +1));

  /* initialize the argv */
  ac=count;
  lcrzo_epr(lcrzo_data_alloc((ac+1)*sizeof(void*), (lcrzo_data*)&av));
  lcrzo_epr(lcrzo_string_initm_text("lcrzoex ", &str));
  lcrzo_epr(lcrzo_string_appendm_uint32(toolnumber, "%u", &str));
  av[0]=str;
  av[ac]=NULL;

  /* set other parameters */
  for ( i=1 ; i<=count-1 ; i++ )
  { lcrzo_epr(lcrzo_stringlist_valuem_pos(&list, i, NULL, &str));
    av[i]=str;
  }
  lcrzo_epr(lcrzo_list_close(&list));

  /* call the wanted tool */
  ret=shared_toolbox__x_main(ac, av, toolnumber);
  /* because the tool might close lcrzo */
  lcrzo_init();

  /* free allocated resources */
  for ( i=0 ; i<ac ; i++ )
  { lcrzo_epr(lcrzo_string_free2(&(av[i])));
  }
  lcrzo_epr(lcrzo_data_free2((lcrzo_data*)&av));
  lcrzo_close();
  return(ret);
}

/*-------------------------------------------------------------*/
int lcrzoex_000389_readfile(lcrzo_conststring filename,
			    lcrzo_list *plist)
{ lcrzo_file file;
  lcrzo_data data, pdata;
  lcrzo_string line;
  lcrzo_int32 datasize, pdatasize;
  int ret;

  /* open the file */
  lcrzo_epr(lcrzo_file_open_read(filename, &file));

  /* analyze file content */
  ret=LCRZO_ERR_OK;
  while (LCRZO_TRUE)
  { /* read from file */
    ret=lcrzo_file_readm_line(&file, LCRZO_WAIT_INFINITE, &data, &datasize);
    if (ret!=LCRZO_ERR_OK) break;
    /* skip spaces (no size check because it ends with '\n') */
    pdata=data;
    pdatasize=datasize;
    while ( *pdata==' ' || *pdata=='\t' ) { pdata++; pdatasize--; }
    /* on end of line, skip */
    if ( *pdata=='\n' )
    { lcrzo_epr(lcrzo_data_free2(&data));
      continue;
    }
    /* on comment, skip */
    if ( *pdata=='#' )
    { lcrzo_epr(lcrzo_data_free2(&data));
      continue;
    }
    /* now, analyze each part in the line */
    lcrzo_epr(lcrzo_string_initm_data(pdata, pdatasize, &line));
    ret=lcrzoex_000389_readfile_line(line, plist);
    lcrzo_epr(lcrzo_string_free2(&line));
    lcrzo_epr(lcrzo_data_free2(&data));
    if (ret!=LCRZO_ERR_OK) break;
   }
  if (ret==LCRZO_ERR_OKDATAEND) ret=LCRZO_ERR_OK;
  lcrzo_epr(lcrzo_file_close(&file));
  return(ret);
}

/*-------------------------------------------------------------*/
int lcrzoex_000389_readfile_line(lcrzo_conststring line,
				 lcrzo_list *plist)
{ lcrzo_string str;
  lcrzo_conststring pcfirstchar, pclastchar, pcchar;
  lcrzo_int16 language;
  char c;
  int i;

  lcrzo_epr(lcrzo_global_get_language(&language));
  pclastchar=line;

  while (LCRZO_TRUE)
  {
    /* first, skip spaces */
    pcfirstchar=pclastchar;
    while ( *pcfirstchar==' ' || *pcfirstchar=='\t' ||
	    *pcfirstchar=='\r' || *pcfirstchar=='\n' ) pcfirstchar++;
    
    /* if we are at end, stop here */
    if (*pcfirstchar=='\0') return(LCRZO_ERR_OK);
    
    /* now check if there is " */
    if (*pcfirstchar!='"')
    { /* stop on first space */
      pclastchar=pcfirstchar;
      while ( *pclastchar!=' ' && *pclastchar!='\t' && *pclastchar!='\0' && 
	      *pclastchar!='\r' && *pclastchar!='\n' ) pclastchar++;
      /* add the item in the list */
      lcrzo_er(shared_misc_string_initm_2ptrs(pcfirstchar, pclastchar-1,&str));
      lcrzo_epr(lcrzo_stringlist_add_pos(plist, -1, 0, str));
      lcrzo_epr(lcrzo_string_free2(&str));
    }
    else
    { /* stop on first " */
      lcrzo_er(lcrzo_string_alloc(strlen(line), &str));
      i=0;
      pcchar=pcfirstchar+1;
      while (LCRZO_TRUE)
      {
	if ( *pcchar=='\0' )
	{ /* premature end : error */
	  str[i]='\0';
	  if (language==LCRZO_GLOBAL_LANG_FR) 
	    printf("Erreur : il n'y a pas de deuxieme '\"' pour \"%s\"\n",
		   line);
	  else
	    printf("Error : there is no second '\"' for \"%s\"\n", line);
	  lcrzo_epr(lcrzo_string_free2(&str));
	  return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
	}
	else if ( *pcchar=='\\' )
	{ /* skip this char */
	  pcchar++;
	  c=*pcchar;
	  if ( c=='\0' )
	  { /* premature end : error */
	    str[i]='\0';
	    if (language==LCRZO_GLOBAL_LANG_FR) 
	      printf("Erreur : il n'y a rien apres le '\\' de \"%s\"\n", line);
	    else
	      printf("Error : there is nothing after the '\\' for \"%s\"\n",
		     line);
	    lcrzo_epr(lcrzo_string_free2(&str));
	    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
	  }
	  /* simply put the second char, with eventually its meaning */
	  else if ( c=='0' ) str[i++]='\0';
	  else if ( c=='a' ) str[i++]=7; /*'\a', otherwise warning -wtradi...*/
	  else if ( c=='b' ) str[i++]='\b';
	  else if ( c=='t' ) str[i++]='\t';
	  else if ( c=='n' ) str[i++]='\n';
	  else if ( c=='v' ) str[i++]='\v';
	  else if ( c=='f' ) str[i++]='\f';
	  else if ( c=='r' ) str[i++]='\r';
	  else str[i++]=c;
	}
	else if ( *pcchar=='"' )
	{ /* end */
	  pclastchar=pcchar+1;
	  str[i]='\0';
	  lcrzo_epr(lcrzo_stringlist_add_pos(plist, -1, 0, str));
	  lcrzo_epr(lcrzo_string_free2(&str));
	  break;
	}
	else
	{ /* store the char */
	  str[i++]=*pcchar;
	}
	pcchar++;
      }
    }
  }

  return(LCRZO_ERR_OK);
}


