
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000391=
{ /*reference*/     "lcrzoex", 391,
  /*version*/       1, 0,
  /*title*/         "Data record : add Eth header, spoof",
  /*french title*/  "Data record : ajoute entete Eth, spoofe",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "file device srceth dsteth type [startpacket [endpacket]]",
  /*french usage*/  "nom_fichier device ethsrc ethdst type [packet_debut [packet_fin]]",
  /*usage example*/ "/tmp/file eth0 a:a:a:a:a:a b:b:b:b:b:b 123 1 5",
  /*fr. usage ex.*/ "/tmp/fichier eth0 a:a:a:a:a:a b:b:b:b:b:b 1 123 1 5", 
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declaration */
int lcrzoex_000391_main(int argc, char *argv[]);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000391_main(int argc, char *argv[])
{ lcrzo_int32 start, end;
  lcrzo_record record;
  shared_record_exact sre;
  lcrzo_spoof spoof;
  lcrzo_device device;
  lcrzo_hdrleth addhdreth;

  lcrzo_init();
  if ( argc<6 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000391, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }
  if (argc>=7)
  { lcrzo_epr(lcrzo_int32_init_text(argv[6], &start));
  }
  else 
  { start=+1;
  }
  if (argc>=8)
  { lcrzo_epr(lcrzo_int32_init_text(argv[7], &end));
  }
  else
  { end=-1;
  }

  /* init device */
  lcrzo_epr(shared_para_device_init_src(argv[2], device));

  /* init new addresses */
  lcrzo_epr(lcrzo_hdrleth_initdefault(&addhdreth));
  lcrzo_epr(shared_para_etha_init_src(argv[3], addhdreth.src));
  lcrzo_epr(shared_para_etha_init_dst(argv[4], addhdreth.dst));
  lcrzo_epr(shared_para_uint16_init(argv[5], "Ethernet type",
				    0, 0xFFFFu, LCRZO_HDRLETH_TYPE_IP,
				    &(addhdreth.type)));

  /* init spoof */
  lcrzo_epr(lcrzo_spoof_init(&spoof));
  lcrzo_epr(lcrzo_spoof_set_printbeforesending(&spoof, LCRZO_TRUE));

  /* initialize the sre */
  lcrzo_epr(shared_record_exact_initdefault(&sre));
  sre.pspoof=&spoof;
  sre.pspoofdev=&device;
  sre.paddhdreth=&addhdreth;
  
  /* loop through the record */
  lcrzo_epr(lcrzo_record_open(&record, argv[1], LCRZO_RECORD_OPEN_READ));
  lcrzo_epr(lcrzo_record_loop_range(&record, start, end,
				    &shared_record_exact_data_spoof, &sre));
  lcrzo_epr(lcrzo_record_close(&record));

  /* close spoof */
  lcrzo_epr(lcrzo_spoof_close(&spoof));

  lcrzo_close();
  return(LCRZO_ERR_OK);
}


