
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000396=
{ /*reference*/     "lcrzoex", 396,
  /*version*/       1, 0,
  /*title*/         "Data record : add IPTCP header, spoof",
  /*french title*/  "TCP record : ajoute entete IPTCP, spoofe",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "file srcip dstip sport dport [startpacket [endpacket]]",
  /*french usage*/  "nom_fichier ipsrc ipdst sport dport [packet_debut [packet_fin]]",
  /*usage example*/ "/tmp/file 1.2.3.4 5.6.7.8 1234 567 1 5",
  /*fr. usage ex.*/ "/tmp/fichier 1.2.3.4 5.6.7.8 1234 567 1 5", 
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declaration */
int lcrzoex_000396_main(int argc, char *argv[]);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000396_main(int argc, char *argv[])
{ lcrzo_int32 start, end;
  lcrzo_record record;
  shared_record_exact sre;
  lcrzo_spoof spoof;
  lcrzo_device device;
  lcrzo_hdrlip addhdrip;
  lcrzo_hdrltcp addhdrtcp;

  lcrzo_init();
  if ( argc<6 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000396, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }
  if (argc>=7)
  { lcrzo_epr(lcrzo_int32_init_text(argv[6], &start));
  }
  else 
  { start=+1;
  }
  if (argc>=8)
  { lcrzo_epr(lcrzo_int32_init_text(argv[7], &end));
  }
  else
  { end=-1;
  }

  /* init new addresses */
  lcrzo_epr(lcrzo_hdrlip_initdefault(&addhdrip));
  lcrzo_epr(shared_para_ipl_init_src(argv[2], &(addhdrip.saddr)));
  lcrzo_epr(shared_para_ipl_init_dst(argv[3], &(addhdrip.daddr)));
  addhdrip.protocol=LCRZO_HDRLIP_PROTOCOL_TCP;
  lcrzo_epr(lcrzo_hdrltcp_initdefault(&addhdrtcp));
  lcrzo_epr(shared_para_port_init_src(argv[4], &(addhdrtcp.sport)));
  lcrzo_epr(shared_para_port_init_dst(argv[5], 25, &(addhdrtcp.dport)));

  /* init spoof */
  lcrzo_epr(lcrzo_spoof_init(&spoof));
  lcrzo_epr(lcrzo_spoof_set_printbeforesending(&spoof, LCRZO_TRUE));

  /* initialize the sre */
  lcrzo_epr(shared_record_exact_initdefault(&sre));
  sre.pspoof=&spoof;
  sre.pspoofdev=&device;
  sre.paddhdrip=&addhdrip;
  sre.paddhdrtcp=&addhdrtcp;
  
  /* loop through the record */
  lcrzo_epr(lcrzo_record_open(&record, argv[1], LCRZO_RECORD_OPEN_READ));
  lcrzo_epr(lcrzo_record_loop_range(&record, start, end,
				    &shared_record_exact_data_spoof, &sre));
  lcrzo_epr(lcrzo_record_close(&record));

  /* close spoof */
  lcrzo_epr(lcrzo_spoof_close(&spoof));

  lcrzo_close();
  return(LCRZO_ERR_OK);
}
