
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000399=
{ /*reference*/     "lcrzoex", 399,
  /*version*/       1, 0,
  /*title*/         "Data record : add EthIPICMP header, spoof",
  /*french title*/  "ICMP record : ajoute entete EthIPICMP, spoofe",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "file device srceth dsteth srcip dstip type code [startpacket [endpacket]]",
  /*french usage*/  "nom_fichier device ethsrc ethdst ipsrc ipdst type code [packet_debut [packet_fin]]",
  /*usage example*/ "/tmp/file eth0 a:a:a:a:a:a b:b:b:b:b:b 1.2.3.4 5.6.7.8 1 2 1 5",
  /*fr. usage ex.*/ "/tmp/fichier eth0 a:a:a:a:a:a b:b:b:b:b:b 1.2.3.4 5.6.7.8 1 2 1 5", 
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declaration */
int lcrzoex_000399_main(int argc, char *argv[]);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000399_main(int argc, char *argv[])
{ lcrzo_int32 start, end;
  lcrzo_record record;
  shared_record_exact sre;
  lcrzo_spoof spoof;
  lcrzo_device device;
  lcrzo_hdrleth addhdreth;
  lcrzo_hdrlip addhdrip;
  lcrzo_hdrlicmp addhdricmp;

  lcrzo_init();
  if ( argc<9 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000399, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }
  if (argc>=10)
  { lcrzo_epr(lcrzo_int32_init_text(argv[9], &start));
  }
  else 
  { start=+1;
  }
  if (argc>=11)
  { lcrzo_epr(lcrzo_int32_init_text(argv[10], &end));
  }
  else
  { end=-1;
  }

  /* init device */
  lcrzo_epr(shared_para_device_init_src(argv[2], device));

  /* init new addresses */
  lcrzo_epr(lcrzo_hdrleth_initdefault(&addhdreth));
  lcrzo_epr(shared_para_etha_init_src(argv[3], addhdreth.src));
  lcrzo_epr(shared_para_etha_init_dst(argv[4], addhdreth.dst));
  addhdreth.type=LCRZO_HDRLETH_TYPE_IP;
  lcrzo_epr(lcrzo_hdrlip_initdefault(&addhdrip));
  lcrzo_epr(shared_para_ipl_init_src(argv[5], &(addhdrip.saddr)));
  lcrzo_epr(shared_para_ipl_init_dst(argv[6], &(addhdrip.daddr)));
  addhdrip.protocol=LCRZO_HDRLIP_PROTOCOL_ICMP;
  lcrzo_epr(lcrzo_hdrlicmp_initdefault(&addhdricmp));
  lcrzo_epr(shared_para_uint8_init_icmptype(argv[7], &(addhdricmp.type)));
  lcrzo_epr(shared_para_uint8_init_icmpcode(argv[8], &(addhdricmp.code)));

  /* init spoof */
  lcrzo_epr(lcrzo_spoof_init(&spoof));
  lcrzo_epr(lcrzo_spoof_set_printbeforesending(&spoof, LCRZO_TRUE));

  /* initialize the sre */
  lcrzo_epr(shared_record_exact_initdefault(&sre));
  sre.pspoof=&spoof;
  sre.pspoofdev=&device;
  sre.paddhdreth=&addhdreth;
  sre.paddhdrip=&addhdrip;
  sre.paddhdricmp=&addhdricmp;
  
  /* loop through the record */
  lcrzo_epr(lcrzo_record_open(&record, argv[1], LCRZO_RECORD_OPEN_READ));
  lcrzo_epr(lcrzo_record_loop_range(&record, start, end,
				    &shared_record_exact_data_spoof, &sre));
  lcrzo_epr(lcrzo_record_close(&record));

  /* close spoof */
  lcrzo_epr(lcrzo_spoof_close(&spoof));

  lcrzo_close();
  return(LCRZO_ERR_OK);
}
