
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000400=
{ /*reference*/     "lcrzoex", 400,
  /*version*/       1, 0,
  /*title*/         "Eth record : update fields, spoof",
  /*french title*/  "Eth record : met a jour les champs, spoofe",
  /*description*/   "Updated fields are : ip.totlen, ip.check, udp.len, udp.check, tcp.check, icmp.check",
  /*french desc.*/  "Les champs mis a jour sont : ip.totlen, ip.check, udp.len, udp.check, tcp.check, icmp.check",
  /*usage*/         "file [device [startpacket [endpacket]]]",
  /*french usage*/  "nom_fichier [device [packet_debut [packet_fin]]]",
  /*usage example*/ "/tmp/file eth0 1 5",
  /*fr. usage ex.*/ "/tmp/fichier eth0 1 5", 
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declaration */
int lcrzoex_000400_main(int argc, char *argv[]);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000400_main(int argc, char *argv[])
{ lcrzo_int32 start, end;
  lcrzo_record record;
  shared_record_update sru;
  lcrzo_spoof spoof;
  lcrzo_device device;

  lcrzo_init();
  if ( argc<2 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000400, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }
  if (argc>=4)
  { lcrzo_epr(lcrzo_int32_init_text(argv[3], &start));
  }
  else 
  { start=+1;
  }
  if (argc>=5)
  { lcrzo_epr(lcrzo_int32_init_text(argv[4], &end));
  }
  else
  { end=-1;
  }

  /* init device */
  if (argc>=3)
  { lcrzo_epr(shared_para_device_init_src(argv[2], device));
  }
  else
  { lcrzo_epr(lcrzo_device_value_pos(LCRZO_DEVICE_TYPE_BEST, 1, device));
  }

  /* init spoof */
  lcrzo_epr(lcrzo_spoof_init(&spoof));
  lcrzo_epr(lcrzo_spoof_set_printbeforesending(&spoof, LCRZO_TRUE));

  /* initialize the sru */
  lcrzo_epr(shared_record_update_initdefault(&sru));
  sru.pspoof=&spoof;
  sru.pspoofdev=&device;
  
  /* loop through the record */
  lcrzo_epr(lcrzo_record_open(&record, argv[1], LCRZO_RECORD_OPEN_READ));
  lcrzo_epr(lcrzo_record_loop_range(&record, start, end,
				    &shared_record_update_eth_spoof, &sru));
  lcrzo_epr(lcrzo_record_close(&record));

  /* close spoof */
  lcrzo_epr(lcrzo_spoof_close(&spoof));

  lcrzo_close();
  return(LCRZO_ERR_OK);
}
