
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000404=
{ /*reference*/     "lcrzoex", 404,
  /*version*/       1, 0,
  /*title*/         "UDP record : update fields, add EthIP header, spoof",
  /*french title*/  "UDP record : met a jour les champs, ajoute entete EthIP, spoofe",
  /*description*/   "Updated fields are : udp.len, udp.check",
  /*french desc.*/  "Les champs mis a jour sont : udp.len, udp.check",
  /*usage*/         "file device srceth dsteth srcip dstip [startpacket [endpacket]]",
  /*french usage*/  "nom_fichier device ethsrc ethdst ipsrc ipdst [packet_debut [packet_fin]]",
  /*usage example*/ "/tmp/file eth0 a:a:a:a:a:a b:b:b:b:b:b 1.2.3.4 5.6.7.8 1 5",
  /*fr. usage ex.*/ "/tmp/fichier eth0 a:a:a:a:a:a b:b:b:b:b:b 1.2.3.4 5.6.7.8 1 1 5", 
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declaration */
int lcrzoex_000404_main(int argc, char *argv[]);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000404_main(int argc, char *argv[])
{ lcrzo_int32 start, end;
  lcrzo_record record;
  shared_record_update sru;
  lcrzo_spoof spoof;
  lcrzo_device device;
  lcrzo_hdrleth addhdreth;
  lcrzo_hdrlip addhdrip;

  lcrzo_init();
  if ( argc<7 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000404, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }
  if (argc>=8)
  { lcrzo_epr(lcrzo_int32_init_text(argv[7], &start));
  }
  else 
  { start=+1;
  }
  if (argc>=9)
  { lcrzo_epr(lcrzo_int32_init_text(argv[8], &end));
  }
  else
  { end=-1;
  }

  /* init device */
  lcrzo_epr(shared_para_device_init_src(argv[2], device));

  /* init new addresses */
  lcrzo_epr(lcrzo_hdrleth_initdefault(&addhdreth));
  lcrzo_epr(shared_para_etha_init_src(argv[3], addhdreth.src));
  lcrzo_epr(shared_para_etha_init_dst(argv[4], addhdreth.dst));
  addhdreth.type=LCRZO_HDRLETH_TYPE_IP;
  lcrzo_epr(lcrzo_hdrlip_initdefault(&addhdrip));
  lcrzo_epr(shared_para_ipl_init_src(argv[5], &(addhdrip.saddr)));
  lcrzo_epr(shared_para_ipl_init_dst(argv[6], &(addhdrip.daddr)));
  addhdrip.protocol=LCRZO_HDRLIP_PROTOCOL_UDP;

  /* init spoof */
  lcrzo_epr(lcrzo_spoof_init(&spoof));
  lcrzo_epr(lcrzo_spoof_set_printbeforesending(&spoof, LCRZO_TRUE));

  /* initialize the sru */
  lcrzo_epr(shared_record_update_initdefault(&sru));
  sru.pspoof=&spoof;
  sru.pspoofdev=&device;
  sru.paddhdreth=&addhdreth;
  sru.paddhdrip=&addhdrip;
  
  /* loop through the record */
  lcrzo_epr(lcrzo_record_open(&record, argv[1], LCRZO_RECORD_OPEN_READ));
  lcrzo_epr(lcrzo_record_loop_range(&record, start, end,
				    &shared_record_update_udp_spoof, &sru));
  lcrzo_epr(lcrzo_record_close(&record));

  /* close spoof */
  lcrzo_epr(lcrzo_spoof_close(&spoof));

  lcrzo_close();
  return(LCRZO_ERR_OK);
}


