
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000405=
{ /*reference*/     "lcrzoex", 405,
  /*version*/       1, 0,
  /*title*/         "TCP record : add IP header, update fields, spoof",
  /*french title*/  "TCP record : ajoute entete IP, met a jour les champs, spoofe",
  /*description*/   "Updated fields are : tcp.check",
  /*french desc.*/  "Les champs mis a jour sont : tcp.check",
  /*usage*/         "file srcip dstip [startpacket [endpacket]]",
  /*french usage*/  "nom_fichier ipsrc ipdst [packet_debut [packet_fin]]",
  /*usage example*/ "/tmp/file 1.2.3.4 5.6.7.8 1 5",
  /*fr. usage ex.*/ "/tmp/fichier 1.2.3.4 5.6.7.8 1 5", 
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declaration */
int lcrzoex_000405_main(int argc, char *argv[]);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000405_main(int argc, char *argv[])
{ lcrzo_int32 start, end;
  lcrzo_record record;
  shared_record_update sru;
  lcrzo_spoof spoof;
  lcrzo_hdrlip addhdrip;

  lcrzo_init();
  if ( argc<4 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000405, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }
  if (argc>=5)
  { lcrzo_epr(lcrzo_int32_init_text(argv[4], &start));
  }
  else 
  { start=+1;
  }
  if (argc>=6)
  { lcrzo_epr(lcrzo_int32_init_text(argv[5], &end));
  }
  else
  { end=-1;
  }

  /* init new addresses */
  lcrzo_epr(lcrzo_hdrlip_initdefault(&addhdrip));
  lcrzo_epr(shared_para_ipl_init_src(argv[2], &(addhdrip.saddr)));
  lcrzo_epr(shared_para_ipl_init_dst(argv[3], &(addhdrip.daddr)));
  addhdrip.protocol=LCRZO_HDRLIP_PROTOCOL_TCP;

  /* init spoof */
  lcrzo_epr(lcrzo_spoof_init(&spoof));
  lcrzo_epr(lcrzo_spoof_set_printbeforesending(&spoof, LCRZO_TRUE));

  /* initialize the sru */
  lcrzo_epr(shared_record_update_initdefault(&sru));
  sru.pspoof=&spoof;
  sru.paddhdrip=&addhdrip;
  
  /* loop through the record */
  lcrzo_epr(lcrzo_record_open(&record, argv[1], LCRZO_RECORD_OPEN_READ));
  lcrzo_epr(lcrzo_record_loop_range(&record, start, end,
				    &shared_record_update_tcp_spoof, &sru));
  lcrzo_epr(lcrzo_record_close(&record));

  /* close spoof */
  lcrzo_epr(lcrzo_spoof_close(&spoof));

  lcrzo_close();
  return(LCRZO_ERR_OK);
}


