
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
shared_lcrzoex_infos lcrzoex_000409=
{ /*reference*/     "lcrzoex", 409,
  /*version*/       1, 0,
  /*title*/         "passive ftp downloading a directory by walking through",
  /*french title*/  "ftp passif telechargeant un repertoire en le parcourant",
  /*description*/   "This tool has to be used (instead of 286) if a directory name contains a space",
  /*french desc.*/  "Cet outil doit etre employe (au lieu du 286) si un nom de repertoire contient un espace",
  /*usage*/         "server_name server_port distant_dir local_dir login [passwd]",
  /*french usage*/  "nom_serveur port_serveur rep_distant rep_local login [passwd]",
  /*usage example*/ "1.2.3.4 21 /etc/ /tmp/here bob mypass",
  /*fr. usage ex.*/ "1.2.3.4 21 /etc/ /tmp/ici bob monpwd",
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000409_main(int argc, char *argv[]);
int lcrzoex_000409_recursiveftpget(lcrzo_ipl serverip,
				   lcrzo_uint16 serverport,
				   lcrzo_conststring distantdir,
				   lcrzo_conststring localdir,
				   lcrzo_conststring login,
				   lcrzo_conststring passwd);
int lcrzoex_000409_recursiveget(lcrzo_sock *psock, 
				lcrzo_conststring distantdir,
				lcrzo_conststring localdir);
int lcrzoex_000409_getfile(lcrzo_sock *psock, 
			   lcrzo_conststring distantfile,
			   lcrzo_int32 distantfilesize,
			   lcrzo_conststring localfile);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000409_main(int argc, char *argv[])
{ lcrzo_ipl ipl;
  lcrzo_uint16 port;
  lcrzo_string pwd;
  lcrzo_int16 language;

  lcrzo_init();
  if ( argc<6 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000409, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }

  lcrzo_epr(shared_para_ipl_init_dst(argv[1], &ipl));
  lcrzo_epr(shared_para_port_init_dst(argv[2], 21, &port));

  if (argc>6)
  { lcrzo_epr(lcrzo_string_initm_text(argv[6], &pwd));
  }
  else
  { lcrzo_epr(lcrzo_string_initm_text_passwd("Passwd", "", &pwd));
  }
  lcrzo_epr(lcrzoex_000409_recursiveftpget(ipl, port,
					   argv[3], argv[4], argv[5], pwd));
  lcrzo_epr(lcrzo_string_free2(&pwd));

  lcrzo_epr(lcrzo_global_get_language(&language));
  if ( language==LCRZO_GLOBAL_LANG_FR )
    puts("Fin");
  else
    puts("End");
  lcrzo_close();
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000409_recursiveftpget(lcrzo_ipl serverip,
				   lcrzo_uint16 serverport,
				   lcrzo_conststring distantdir,
				   lcrzo_conststring localdir,
				   lcrzo_conststring login,
				   lcrzo_conststring passwd)
{ lcrzo_sock sock;

  /* initialize the client */
  lcrzo_epr(lcrzo_sock_tcpcli_real(serverip, serverport, &sock));

  /* first, we connect */
  lcrzo_epr(shared_ftp_login(&sock, login, passwd));

  /* then, we send a 'TYPE I' */
  lcrzo_epr(shared_ftp_cmd_type_bin(&sock));

  /* we go to the root */
  lcrzo_epr(shared_ftp_cmd_cwd(&sock, distantdir));

  /* treat every file */  
  lcrzo_epr(lcrzoex_000409_recursiveget(&sock, distantdir, localdir));
  
  /* then, we send a 'QUIT' */
  lcrzo_epr(shared_ftp_cmd_quit(&sock));

  /* close the socket */
  lcrzo_epr(lcrzo_sock_close(&sock));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000409_recursiveget(lcrzo_sock *psock, 
				lcrzo_conststring distantdir,
				lcrzo_conststring localdir)
{ lcrzo_sock sockdata;
  lcrzo_list listdir, listfile, listlinkname, listlinkobj;
  lcrzo_ipl serveripl;
  lcrzo_uint16 serverport;
  lcrzo_int32 count, i;
  lcrzo_uint32 filesize;
  lcrzo_string distantobj, localobj, localobj2, str, str2;
  lcrzo_int16 language;
  int ret;

  lcrzo_epr(lcrzo_global_get_language(&language));

  /* first, create local directory */
  /*ignore*/lcrzo_dir_create(localdir);

  /* obtain the list of files and directories */
  lcrzo_epr(shared_ftp_cmd_pasv(psock, &serveripl, &serverport));
  lcrzo_epr(lcrzo_sock_tcpcli_real(serveripl, serverport, &sockdata));
  ret=shared_ftp_data_list(psock, LCRZO_TRUE, &sockdata, NULL,
			   ".", &listfile, &listdir, 
			   &listlinkname, &listlinkobj);
  if (ret!=LCRZO_ERR_OK) return(LCRZO_ERR_OK); /* do not download. skip*/

  /* get files */
  lcrzo_epr(lcrzo_list_count(&listfile, &count));
  for ( i=1 ; i<=count ; i++ )
  { lcrzo_epr(lcrzo_stringlist_valuem_pos(&listfile, i, &filesize, &str));
    if ( language==LCRZO_GLOBAL_LANG_FR )
      printf("  Fich");
    else
      printf("  File");
    printf("  %s ", str);
    lcrzo_epr(lcrzo_int32_print("(", filesize, "%d", ")\n"));
    lcrzo_epr(lcrzo_string_initm_text(distantdir, &distantobj));
    lcrzo_epr(lcrzo_string_appendm_text("/", &distantobj));
    lcrzo_epr(lcrzo_string_appendm_text(str, &distantobj));
    lcrzo_epr(lcrzo_string_initm_text(localdir, &localobj));
    lcrzo_epr(lcrzo_string_appendm_text("/", &localobj));
    lcrzo_epr(lcrzo_string_appendm_text(str, &localobj));
    printf(" -> %s\n", localobj);
    lcrzo_epr(lcrzoex_000409_getfile(psock, str, filesize, localobj));
    lcrzo_epr(lcrzo_string_free2(&str));
    lcrzo_epr(lcrzo_string_free2(&distantobj));
    lcrzo_epr(lcrzo_string_free2(&localobj));
  }
  lcrzo_epr(lcrzo_list_close(&listfile));

  /* get directories */
  lcrzo_epr(lcrzo_list_count(&listdir, &count));
  for ( i=1 ; i<=count ; i++ )
  { lcrzo_epr(lcrzo_stringlist_valuem_pos(&listdir, i, &filesize, &str));
    lcrzo_epr(lcrzo_string_initm_text(distantdir, &distantobj));
    lcrzo_epr(lcrzo_string_appendm_text("/", &distantobj));
    lcrzo_epr(lcrzo_string_appendm_text(str, &distantobj));
    lcrzo_epr(lcrzo_string_initm_text(localdir, &localobj));
    lcrzo_epr(lcrzo_string_appendm_text("/", &localobj));
    lcrzo_epr(lcrzo_string_appendm_text(str, &localobj));
    if ( language==LCRZO_GLOBAL_LANG_FR )
      printf("Rep   ");
    else
      printf("Dir   ");
    printf("  %s -> %s\n", distantobj, localobj);
    lcrzo_epr(shared_ftp_cmd_cwd(psock, str));
    lcrzo_epr(lcrzoex_000409_recursiveget(psock, distantobj, localobj));
    lcrzo_epr(shared_ftp_cmd_cwd(psock, ".."));
    lcrzo_epr(lcrzo_string_free2(&str));
    lcrzo_epr(lcrzo_string_free2(&distantobj));
    lcrzo_epr(lcrzo_string_free2(&localobj));
  }
  lcrzo_epr(lcrzo_list_close(&listdir));

  /* create links
     Note : we do not get file outside the source directory */
  lcrzo_epr(lcrzo_list_count(&listlinkname, &count));
  for ( i=1 ; i<=count ; i++ )
  { lcrzo_epr(lcrzo_stringlist_valuem_pos(&listlinkname, i, NULL, &str));
    lcrzo_epr(lcrzo_stringlist_valuem_pos(&listlinkobj, i, &filesize, &str2));
    lcrzo_epr(lcrzo_string_initm_text(distantdir, &distantobj));
    lcrzo_epr(lcrzo_string_appendm_text("/", &distantobj));
    lcrzo_epr(lcrzo_string_appendm_text(str, &distantobj));
    lcrzo_epr(lcrzo_string_initm_text(localdir, &localobj));
    lcrzo_epr(lcrzo_string_appendm_text("/", &localobj));
    lcrzo_epr(lcrzo_string_appendm_text(str, &localobj));
    lcrzo_epr(lcrzo_string_initm_text(str2, &localobj2));
    lcrzo_epr(lcrzo_string_free2(&str));
    lcrzo_epr(lcrzo_string_free2(&str2));
    if ( language==LCRZO_GLOBAL_LANG_FR )
      printf("  Lien ");
    else
      printf("  Link ");
    printf("  %s -> %s--->%s\n", distantobj, localobj, localobj2);
    lcrzo_ep(shared_misc_symlink(localobj2, localobj));
    lcrzo_epr(lcrzo_string_free2(&distantobj));
    lcrzo_epr(lcrzo_string_free2(&localobj));
    lcrzo_epr(lcrzo_string_free2(&localobj2));
  }
  lcrzo_epr(lcrzo_list_close(&listlinkname));
  lcrzo_epr(lcrzo_list_close(&listlinkobj));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000409_getfile(lcrzo_sock *psock, 
			   lcrzo_conststring distantfile,
			   lcrzo_int32 distantfilesize,
			   lcrzo_conststring localfile)
{ lcrzo_int32 localfilesize;
  lcrzo_ipl pasvipl;
  lcrzo_uint16 pasvport;
  lcrzo_sock datasock;
  lcrzo_int16 language;

  lcrzo_epr(lcrzo_global_get_language(&language));

  /* get the port to which connect */
  lcrzo_epr(shared_ftp_cmd_pasv(psock, &pasvipl, &pasvport));
  /*lcrzo_ipl_print("IP : ", pasvipl, ":");
    printf("%d\n", pasvport);*/

  /* initialize the data connexion */
  lcrzo_epr(lcrzo_sock_tcpcli_real(pasvipl, pasvport, &datasock));

  /* we send a 'RETR', and get file */
  lcrzo_epr(shared_ftp_data_retr(psock, LCRZO_TRUE,
				 &datasock, NULL, distantfile, localfile));

  /* we send a 'SIZE file' */
  lcrzo_epr(lcrzo_filename_size(localfile, &localfilesize));
  /* verify size */
  if ( distantfilesize!=localfilesize )
  { if ( language==LCRZO_GLOBAL_LANG_FR )
    { printf("Les tailles sont differentes : ");
      lcrzo_epr(lcrzo_int32_print("", distantfilesize, "%d", " "));
      lcrzo_epr(lcrzo_int32_print("", localfilesize, "%d", ", je reessaie\n"));
    }
    else
    { printf("Sizes are differents : ");
      lcrzo_epr(lcrzo_int32_print("", distantfilesize, "%d", " "));
      lcrzo_epr(lcrzo_int32_print("", localfilesize, "%d", ", we retry\n"));
    }
    /* retry */
    lcrzo_epr(lcrzoex_000409_getfile(psock, distantfile, distantfilesize,
				     localfile));
  }

  return(LCRZO_ERR_OK);
}
