/*
		                 LCRZOEX
                             Network toolbox
             Copyright(c) 1999,2000,2001,2002 Laurent Constantin
                                  -----

  Main server    : http://www.laurentconstantin.com/
  Backup servers : http://go.to/laurentconstantin/
                   http://laurentconstantin.est-la.com/
                   http://laurentconstantin.free.fr/
                   http://membres.lycos.fr/lauconstantin/
  [my current email address is on the web servers]

                                  -----
  This file is part of Lcrzoex.

  Lcrzoex is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Lcrzoex is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details (http://www.gnu.org/).

------------------------------------------------------------------------
*/

#include <lcrzo.h>
#include "../lcrzoex.h"

/*-------------------------------------------------------------*/
typedef struct {
  lcrzo_string pvar;
  lcrzo_string pval;
} shared_conffile_list;

/*-------------------------------------------------------------*/
int shared_conffile_list_free(lcrzo_ptr pelem);
int shared_conffile_list_free(lcrzo_ptr pelem)
{ shared_conffile_list *plist;
  plist=(shared_conffile_list *)pelem;
  lcrzo_er(lcrzo_string_free2(&(plist->pvar)));
  lcrzo_er(lcrzo_string_free2(&(plist->pval)));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_conffile_list_init(lcrzo_list *plist)
{ return(lcrzo_list_init(plist, sizeof(shared_conffile_list),
                         &shared_conffile_list_free));
}


/*-------------------------------------------------------------*/
int shared_conffile_list_add_pos(lcrzo_list *plist,
				 lcrzo_int32 pos,
				 lcrzo_conststring var,
				 lcrzo_conststring val)
{ shared_conffile_list listitem;

  /*parameters verification*/
  if (plist==NULL) return(LCRZO_ERR_PANULLPTR);
  if (var==NULL) return(LCRZO_ERR_PANULLPTR);

  /*create the string*/
  lcrzo_er(lcrzo_string_initm_text(var, &(listitem.pvar)));
  if (val==NULL)
  { lcrzo_er(lcrzo_string_initm_text("", &(listitem.pval)));
  }
  else
  { lcrzo_er(lcrzo_string_initm_text(val, &(listitem.pval)));
  }

  /*add the string*/
  lcrzo_er(lcrzo_list_add_pos(plist, pos, &listitem));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_conffile_list_valuem_pos(lcrzo_list *plist,
				    lcrzo_int32 pos,
				    lcrzo_string *pvar,
				    lcrzo_string *pval)
{ shared_conffile_list listitem;

  /*parameters verification*/
  if (plist==NULL) return(LCRZO_ERR_PANULLPTR);

  /*get the value*/
  lcrzo_er(lcrzo_list_value_pos(plist, pos, &listitem));

  lcrzo_er(lcrzo_string_initm_text(listitem.pvar, pvar));
  lcrzo_er(lcrzo_string_initm_text(listitem.pval, pval));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_priv_conffile_err(lcrzo_conststring filename,
			     lcrzo_int32 linenumber,
			     lcrzo_conststring frstr,
			     lcrzo_conststring enstr);
int shared_priv_conffile_err(lcrzo_conststring filename,
			     lcrzo_int32 linenumber,
			     lcrzo_conststring frstr,
			     lcrzo_conststring enstr)
{ lcrzo_int16 language;

  lcrzo_er(lcrzo_global_get_language(&language));
  if ( language==LCRZO_GLOBAL_LANG_FR )
  { lcrzo_er(lcrzo_int32_print("Erreur a la ligne ", linenumber, "%d",
			       " du "));
    printf("fichier %s : ", filename);
    printf("%s.\n", frstr);
  }
  else
  { lcrzo_er(lcrzo_int32_print("Error at line ", linenumber, "%d", 
			       " of file "));
    printf("%s : ", filename);
    printf("%s.\n", enstr);
  }
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_priv_conffile_list_init_file(lcrzo_list *plist,
					lcrzo_conststring filename,
					lcrzo_int32 linenumber,
					lcrzo_conststring line);
int shared_priv_conffile_list_init_file(lcrzo_list *plist,
					lcrzo_conststring filename,
					lcrzo_int32 linenumber,
					lcrzo_conststring line)
{ lcrzo_conststring pc;
  lcrzo_conststring pcvarstart, pcvarend;
  lcrzo_conststring pcvalstart, pcvalend;
  lcrzo_string var, val;
  lcrzo_bool enclosed, previousisbackslash;
  char c;

  /* skip spaces */
  pc=line;
  while ( *pc==' ' || *pc=='\t' ) pc++;
  /* nothing to add */
  if ( *pc=='\0' || *pc=='#' )
  { return(LCRZO_ERR_OK);
  }

  /* now search the var */
  pcvarstart=pc;
  while ( *pc!='\0' && *pc!=' ' && *pc!='\t' && *pc!='#' && *pc!='=' ) pc++;
  pcvarend=pc-1;
  if (pcvarend==pcvarstart-1)
  { lcrzo_er(shared_priv_conffile_err(filename, linenumber,
			     "Il n'y a pas de variable avant le '='",
			     "There is no variable before '='"));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }
  lcrzo_er(shared_misc_string_initm_2ptrs(pcvarstart, pcvarend, &var));
  /* skip spaces */
  if ( *pc==' ' || *pc=='\t' )
  { while ( *pc==' ' || *pc=='\t' )
    { pc++;
    }
  }
  if ( *pc=='\0' || *pc=='#' )
  { /* there is no val */
    lcrzo_er(shared_conffile_list_add_pos(plist, -1, var, ""));
    lcrzo_er(lcrzo_string_free2(&var));
    return(LCRZO_ERR_OK);
  }
  if ( *pc!='=' )
  { lcrzo_er(shared_priv_conffile_err(filename, linenumber,
			     "Il n'y a pas de '=' apres le nom de la variable",
			     "There is no '=' after the variable name"));
    lcrzo_er(lcrzo_string_free2(&var));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }
  pc++; /* skip '=' */

  /* skip spaces */
  while ( *pc==' ' || *pc=='\t' ) pc++;
  if ( *pc=='\0' || *pc=='#' )
  { /* there is no val */
    lcrzo_er(shared_priv_conffile_err(filename, linenumber,
			     "Il n'y a pas de valeur apres '='",
			     "There is no value after '='"));
    lcrzo_er(lcrzo_string_free2(&var));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }

  /* check type of value */
  if (*pc=='"') enclosed=LCRZO_TRUE;
  else enclosed=LCRZO_FALSE;

  /* set value */
  if (!enclosed)
  { /* simple case */
    pcvalstart=pc;
    while ( *pc!='\0' && *pc!='#' ) pc++;
    pcvalend=pc-1;
    /* ignore endings space values */
    while ( *pcvalend==' ' || *pcvalend=='\t' ) pcvalend--;
    /* set value */
    lcrzo_er(shared_misc_string_initm_2ptrs(pcvalstart, pcvalend, &val));
  }
  else
  { /* we have to deal with " and \ */
    pc++;
    lcrzo_er(lcrzo_string_initm_text("", &val));
    while (LCRZO_TRUE)
    { c=*pc;
      previousisbackslash=LCRZO_FALSE;
      if ( c=='\\' )
      { pc++;
        c=*pc;
        previousisbackslash=LCRZO_TRUE;
      }
      if ( c=='\0' )
      { lcrzo_er(shared_priv_conffile_err(filename, linenumber,
				 "Il manque le second '\"'",
				 "The second '\"' is missing"));
        lcrzo_er(lcrzo_string_free2(&var));
        lcrzo_er(lcrzo_string_free2(&val));
	return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
      }
      else if ( c=='"' && !previousisbackslash )
      { /* we've found the end */
	break;
      }
      else
      { /* add the char */
	lcrzo_er(lcrzo_string_appendm_char(c, 1, &val));
      }
      pc++;
    }
    pc++;
    /* check if we are really at end */
    while ( *pc==' ' || *pc=='\t' ) pc++;
    if ( *pc!='\0' && *pc!='#' )
    { lcrzo_er(shared_priv_conffile_err(filename, linenumber,
			       "Il y a des donnees apres le second '\"'",
			       "There is data after the second '\"'"));
      lcrzo_er(lcrzo_string_free2(&var));
      lcrzo_er(lcrzo_string_free2(&val));
      return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
    }
  }

  lcrzo_er(shared_conffile_list_add_pos(plist, -1, var, val));
  lcrzo_er(lcrzo_string_free2(&var));
  lcrzo_er(lcrzo_string_free2(&val));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_conffile_list_init_file(lcrzo_conststring filename,
				   lcrzo_list *plist)
{ lcrzo_file file;
  lcrzo_data data;
  lcrzo_int32 datasize, linenumber;
  int ret;

  /*parameters verification*/
  if (filename==NULL) return(LCRZO_ERR_PANULLPTR);
  if (plist==NULL) return(LCRZO_ERR_PANULLPTR);

  /* open the configuration file */
  lcrzo_er(lcrzo_file_open_read(filename, &file));

  /* init the list */
  lcrzo_er(shared_conffile_list_init(plist));

  /* read every line */
  ret=LCRZO_ERR_OK;
  linenumber=0;
  while (LCRZO_TRUE)
  { ret=lcrzo_file_readm_line(&file, LCRZO_WAIT_INFINITE, &data, &datasize);
    linenumber++;
    if (ret!=LCRZO_ERR_OK) break;
    data[datasize]='\0';
    ret=shared_priv_conffile_list_init_file(plist, filename, linenumber,
					    (lcrzo_conststring)data);
    lcrzo_er(lcrzo_data_free2(&data));
    if (ret!=LCRZO_ERR_OK) break;
  }
  if ( ret==LCRZO_ERR_OKDATAEND ) ret=LCRZO_ERR_OK;

  lcrzo_er(lcrzo_file_close(&file));
  return(ret);
}

/*-------------------------------------------------------------*/
int shared_conffile_list_init_file_text(void);
int shared_conffile_list_init_file_text(void)
{ lcrzo_list list;
  lcrzo_int32 count, i;
  lcrzo_string var, val;

  lcrzo_epr(shared_conffile_list_init_file("z", &list));
  lcrzo_epr(lcrzo_list_count(&list, &count));
  for ( i=1 ; i<=count ; i++ )
  { lcrzo_epr(shared_conffile_list_valuem_pos(&list, i, &var, &val));
    printf("AA%sYY%sZZ\n", var, val);
    lcrzo_er(lcrzo_string_free2(&var));
    lcrzo_er(lcrzo_string_free2(&val));
  }
  lcrzo_er(lcrzo_list_close(&list));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_conffile_create_init(shared_conffile *pconffile,
				lcrzo_conststring filename)
{ lcrzo_file file;

  /* parameters verification */
  if (pconffile==NULL) return(LCRZO_ERR_PANULLPTR);
  if (filename==NULL) return(LCRZO_ERR_PANULLPTR);

  /* open the configuration file */
  lcrzo_er(lcrzo_file_open_write(filename, LCRZO_FALSE, LCRZO_FALSE, 
				 &file));
  lcrzo_er(lcrzo_stream_init_file(&file, &(pconffile->pfile)));
  lcrzo_er(lcrzo_file_close(&file));
  
  lcrzo_er(shared_conffile_create_comment(pconffile,
					  "Le format de ce fichier est :",
					  "This format of this file is :"));
  lcrzo_er(shared_conffile_create_comment(pconffile,
					  " # commentaire",
					  " # comment"));
  lcrzo_er(shared_conffile_create_comment(pconffile,
					  " nomdevariable=valeur",
					  " variablename=value"));
  lcrzo_er(shared_conffile_create_comment(pconffile,
					  " nomdevariable=\"valeur\"",
					  " variablename=\"value\""));
  lcrzo_er(shared_conffile_create_comment(pconffile,
					  " nomdevariable=\" \\\\ \\\"\"",
					  " variablename=\" \\\\ \\\"\""));

  lcrzo_er(shared_conffile_create_emptyline(pconffile));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_conffile_create_comment(shared_conffile *pconffile,
				   lcrzo_conststring frcmt,
				   lcrzo_conststring encmt)
{ lcrzo_int16 language;
 
  /* parameters verification */
  if (pconffile==NULL) return(LCRZO_ERR_PANULLPTR);

  /* write */
  lcrzo_er(lcrzo_global_get_language(&language));
  fprintf((FILE*)pconffile->pfile, "# ");
  if ( language==LCRZO_GLOBAL_LANG_FR )
  { if (frcmt!=NULL) fprintf((FILE*)pconffile->pfile, frcmt);
  }
  else
  { if (encmt!=NULL) fprintf((FILE*)pconffile->pfile, encmt);
  }
  fprintf((FILE*)pconffile->pfile, "\r\n");

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_conffile_create_emptyline(shared_conffile *pconffile)
{
  /* parameters verification */
  if (pconffile==NULL) return(LCRZO_ERR_PANULLPTR);

  /* write */
  fprintf((FILE*)pconffile->pfile, "\r\n");

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_conffile_create_varval(shared_conffile *pconffile,
				  lcrzo_conststring var,
				  lcrzo_conststring val)
{ char c;
  int i, strlenval;

  /* parameters verification */
  if (pconffile==NULL) return(LCRZO_ERR_PANULLPTR);
  if (var==NULL) return(LCRZO_ERR_PANULLPTR);

  /* write */
  fprintf((FILE*)pconffile->pfile, "%s=\"", var);
  if (val!=NULL)
  { strlenval=strlen(val);
    for ( i=0 ; i<strlenval ; i++ )
    { c=val[i];
      if ( c=='\\' || c=='\"' )
      { fputc('\\', (FILE*)pconffile->pfile);
      }
      fputc(c, (FILE*)pconffile->pfile);
    }
  }
 fprintf((FILE*)pconffile->pfile, "\"\r\n");

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_conffile_create_close(shared_conffile *pconffile)
{ fclose((FILE*)pconffile->pfile);
  return(LCRZO_ERR_OK);
}











