/*
		                 LCRZOEX
                             Network toolbox
             Copyright(c) 1999,2000,2001,2002 Laurent Constantin
                                  -----

  Main server    : http://www.laurentconstantin.com/
  Backup servers : http://go.to/laurentconstantin/
                   http://laurentconstantin.est-la.com/
                   http://laurentconstantin.free.fr/
                   http://membres.lycos.fr/lauconstantin/
  [my current email address is on the web servers]

                                  -----
  This file is part of Lcrzoex.

  Lcrzoex is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Lcrzoex is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details (http://www.gnu.org/).

------------------------------------------------------------------------
*/

#include <lcrzo.h>
#include "../lcrzoex.h"

/*-------------------------------------------------------------*/
#define SHARED_DNS_DECODE_STRICT 0

/*-------------------------------------------------------------*/
int shared_priv_dns_printtpe_supported(lcrzo_printtype *pprinttype);
int shared_priv_dns_printtpe_supported(lcrzo_printtype *pprinttype)
{
  /* to simplify, only support those printypes */
  switch( *pprinttype & 0x00FF )
  { case LCRZO_PRINTTYPE_NOTHING_ :
      return(LCRZO_ERR_OK);
    case LCRZO_PRINTTYPE_SYNTH_ :
      *pprinttype=LCRZO_PRINTTYPE_SYNTH_;
      break;
    case LCRZO_PRINTTYPE_ARRAY_ :
    default:
      *pprinttype=LCRZO_PRINTTYPE_ARRAY_;
  }
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_dns_priv_rrlist_free(lcrzo_ptr pelem);
int shared_dns_priv_rrlist_free(lcrzo_ptr pelem)
{ shared_dns_rr *prr;
  prr=(shared_dns_rr *)pelem;
  if (prr->name!=NULL)
  { lcrzo_er(lcrzo_string_free2(&(prr->name)));
  }
  if (prr->rdata!=NULL)
  { lcrzo_er(lcrzo_data_free2(&(prr->rdata)));
  }
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
#define shared_priv_uint16_0(x) (lcrzo_uint8)(((x)>>8)&0xFF) 
#define shared_priv_uint16_1(x) (lcrzo_uint8)((x)&0xFF) 
#define shared_priv_uint32_0(x) (lcrzo_uint8)(((x)>>24)&0xFF) 
#define shared_priv_uint32_1(x) (lcrzo_uint8)(((x)>>16)&0xFF) 
#define shared_priv_uint32_2(x) (lcrzo_uint8)(((x)>>8)&0xFF) 
#define shared_priv_uint32_3(x) (lcrzo_uint8)((x)&0xFF) 
#define shared_priv_uint16(x,i) (lcrzo_uint16)((x[i]<<8)|x[i+1]) 
#define shared_priv_uint32(x,i) (lcrzo_uint32)((x[i]<<24)|(x[i+1]<<16)|(x[i+2]<<8)|x[i+3])

/*-------------------------------------------------------------*/
int shared_dns_priv_data_initm_host(lcrzo_conststring host,
				    lcrzo_data *pdata,
				    lcrzo_int32 *pdatasize);
int shared_dns_priv_data_initm_host(lcrzo_conststring host,
				    lcrzo_data *pdata,
				    lcrzo_int32 *pdatasize)
{ int i, j, jsize, jsizepos, strlenhost;
  char c;

  /* on worst case data is twice bigger as host ("a"->3, "aa"->4, 
     "a.b"->5, "a.b."->5, ".a.b"->5) */  
  strlenhost=strlen(host);
  *pdatasize=(strlenhost+1)*2;
  lcrzo_er(lcrzo_data_alloc(*pdatasize, pdata));

  /* convert aa.bbb.c to 02'ab'03'bbb'01'c'00 */
  j=0;
  jsize=0;
  jsizepos=0;
  j++;
  for ( i=0 ; i<strlenhost ; i++ )
  { c=host[i];
    if ( c=='.' || c=='@' )
    { if (jsize) /* otherwise "^." or ".." */
      { (*pdata)[jsizepos]=(lcrzo_uint8)jsize;
        jsizepos=j;
        jsize=0;
        j++;
      }
    }
    else
    { (*pdata)[j]=c;
      jsize++;
      j++;
    }
    if (jsize>64)
    { lcrzo_er(lcrzo_data_free2(pdata));
      return(LCRZO_ERR_PATOOBIG);
    }
  }
  if (jsize==0)
  { /* ends with a dot */
    (*pdata)[jsizepos]=0;
    *pdatasize=jsizepos+1;
  }
  else
  { (*pdata)[jsizepos]=(lcrzo_uint8)jsize;
    (*pdata)[j]=0;
    *pdatasize=j+1;
  }

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_dns_priv_ipa_init_ips(lcrzo_constips ipsin, 
				 lcrzo_ipa ipa,
				 lcrzo_int8 *pnumberset);
int shared_dns_priv_ipa_init_ips(lcrzo_constips ipsin, 
				 lcrzo_ipa ipa,
				 lcrzo_int8 *pnumberset)
{ lcrzo_ips ips;
  int i, j, n, cn;
  char c;

  if (pnumberset!=NULL) *pnumberset=0;
  lcrzo_er(lcrzo_string_init_coretext(ipsin, LCRZO_IPS_MAXBYTES, ips));
  i=0;
  j=0;
  n=-1;
  while ( LCRZO_TRUE )
  { c=ips[i];
    /* it's a number */
    if ( c>='0' && c<='9' )
    { cn=c-'0';
      if ( n==-1 )
      { /* first digit */
        n=cn;
      }
      else
      { n = n*10 + cn;
        if ( n>255 ) return(LCRZO_ERR_FMIP);
      }
    }
    /* it's a separator */
    else if ( c=='.' || c=='\0' )
    { /* check we have a valid number */
      if ( n==-1 ) return(LCRZO_ERR_FMIP);
      if ( ipa!=NULL ) ipa[j]=(lcrzo_uint8)n;
      /* prepare for next */
      j++;
      n=-1;
      /* check for stop */
      if ( j==LCRZO_IPA_MAXBYTES && c!='\0' ) return(LCRZO_ERR_FMIP);
      if ( c=='\0' ) break;
    }
    /* it's a forbidden char */
    else
    { return(LCRZO_ERR_FMIP); 
    }
    /* take next char */
    i++;
  }
  /* set entries */
  if (pnumberset!=NULL) *pnumberset=(lcrzo_uint8)j;
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_dns_priv_data_initm_header(shared_dns_header dnsheader,
				      lcrzo_data *pdata,
				      lcrzo_int32 *pdatasize);
int shared_dns_priv_data_initm_header(shared_dns_header dnsheader,
				      lcrzo_data *pdata,
				      lcrzo_int32 *pdatasize)
{ lcrzo_uint16 u;

  /* compute bit fields */
  if (dnsheader.opcode>0xF) return(LCRZO_ERR_PATOOBIG);
  if (dnsheader.rcode>0xF) return(LCRZO_ERR_PATOOBIG);
  u=0;
  if (dnsheader.isaresponse) u |= 0x8000;
  u |= (dnsheader.opcode<<11);
  if (dnsheader.authanswer) u |= 0x0400;
  if (dnsheader.wastruncated) u |= 0x0200;
  if (dnsheader.recurdesired) u |= 0x0100;
  if (dnsheader.recuravail) u |= 0x0080;
  u |= (dnsheader.rcode);

  /* first, alloc and set values of enddata */
  *pdatasize=2+2+8;
  lcrzo_er(lcrzo_data_alloc(*pdatasize, pdata));
  (*pdata)[0]=shared_priv_uint16_0(dnsheader.id);
  (*pdata)[1]=shared_priv_uint16_1(dnsheader.id);
  (*pdata)[2]=shared_priv_uint16_0(u);
  (*pdata)[3]=shared_priv_uint16_1(u);
  (*pdata)[4]=shared_priv_uint16_0(dnsheader.count_question);
  (*pdata)[5]=shared_priv_uint16_1(dnsheader.count_question);
  (*pdata)[6]=shared_priv_uint16_0(dnsheader.count_answer);
  (*pdata)[7]=shared_priv_uint16_1(dnsheader.count_answer);
  (*pdata)[8]=shared_priv_uint16_0(dnsheader.count_authority);
  (*pdata)[9]=shared_priv_uint16_1(dnsheader.count_authority);
  (*pdata)[10]=shared_priv_uint16_0(dnsheader.count_additional);
  (*pdata)[11]=shared_priv_uint16_1(dnsheader.count_additional);

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_dns_priv_data_initm_rr_quest(shared_dns_rr rr,
					lcrzo_data *pdata,
					lcrzo_int32 *pdatasize);
int shared_dns_priv_data_initm_rr_quest(shared_dns_rr rr,
					lcrzo_data *pdata,
					lcrzo_int32 *pdatasize)
{ lcrzo_data enddata;
  lcrzo_int32 enddatasize;

  if (rr.name==NULL) return(LCRZO_ERR_PANULLPTR);

  /* first, alloc and set values of enddata */
  enddatasize=2+2;
  lcrzo_er(lcrzo_data_alloc(enddatasize, &enddata));
  enddata[0]=shared_priv_uint16_0(rr.type);
  enddata[1]=shared_priv_uint16_1(rr.type);
  enddata[2]=shared_priv_uint16_0(rr.rrclass);
  enddata[3]=shared_priv_uint16_1(rr.rrclass);

  lcrzo_er(shared_dns_priv_data_initm_host(rr.name, pdata, pdatasize));
  /* add end */
  lcrzo_er(lcrzo_data_appendm_data(enddata, enddatasize, 
				   *pdatasize, pdata, pdatasize));
  lcrzo_er(lcrzo_data_free2(&enddata));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_dns_priv_data_initm_rr_resp(shared_dns_rr rr,
				       lcrzo_data *pdata,
				       lcrzo_int32 *pdatasize);
int shared_dns_priv_data_initm_rr_resp(shared_dns_rr rr,
				       lcrzo_data *pdata,
				       lcrzo_int32 *pdatasize)
{ lcrzo_data enddata;
  lcrzo_int32 enddatasize;

  if (rr.name==NULL) return(LCRZO_ERR_PANULLPTR);
  if (rr.rdata==NULL) return(LCRZO_ERR_PANULLPTR);

  /* first, alloc and set values of enddata */
  enddatasize=2+2+4+2+rr.rdatasize;
  lcrzo_er(lcrzo_data_alloc(enddatasize, &enddata));
  enddata[0]=shared_priv_uint16_0(rr.type);
  enddata[1]=shared_priv_uint16_1(rr.type);
  enddata[2]=shared_priv_uint16_0(rr.rrclass);
  enddata[3]=shared_priv_uint16_1(rr.rrclass);
  enddata[4]=shared_priv_uint32_0(rr.ttl);
  enddata[5]=shared_priv_uint32_1(rr.ttl);
  enddata[6]=shared_priv_uint32_2(rr.ttl);
  enddata[7]=shared_priv_uint32_3(rr.ttl);
  enddata[8]=shared_priv_uint16_0(rr.rdatasize);
  enddata[9]=shared_priv_uint16_1(rr.rdatasize);
  memcpy(enddata+10, rr.rdata, rr.rdatasize);

  /* convert name and set it */
  lcrzo_er(shared_dns_priv_data_initm_host(rr.name, pdata, pdatasize));
  /* add end */
  lcrzo_er(lcrzo_data_appendm_data(enddata, enddatasize, 
				   *pdatasize, pdata, pdatasize));
  lcrzo_er(lcrzo_data_free2(&enddata));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
#define SHARED_PRIV_DNS_DECODE_COMPRESS_START 1
int shared_priv_dns_packet_decode_compress2(lcrzo_constdata packet,
					    lcrzo_int32 packetsize,
					    lcrzo_bool canuncompress,
					    lcrzo_int32 *preatchedpacketsize,
					    lcrzo_int32 level,
					    lcrzo_string *pstr);
int shared_priv_dns_packet_decode_compress2(lcrzo_constdata packet,
					    lcrzo_int32 packetsize,
					    lcrzo_bool canuncompress,
					    lcrzo_int32 *preatchedpacketsize,
					    lcrzo_int32 level,
					    lcrzo_string *pstr)
{ lcrzo_uint8 size, sizetype;
  lcrzo_int32 pos, compresspos, i;
  lcrzo_bool iscompressed, addapoint, lastpartreached;
  lcrzo_constdata p1;
  lcrzo_string tmp, p2;
  lcrzo_int8 c;
  int ret, ret2;
 
  pos=*preatchedpacketsize;
  ret=LCRZO_ERR_OK;
  lcrzo_er(lcrzo_string_initm_text("", pstr));
  addapoint=LCRZO_FALSE;
  compresspos=LCRZO_FALSE;
  size=0; /* for the compiler warning */
  size=0; /* for the compiler warning */
  lastpartreached=LCRZO_FALSE;
  while(!lastpartreached)
  { /* determine if it is compressed, or the size */
    if (pos>=packetsize) return(LCRZO_ERR_OKNOTDECODED);
    sizetype=(lcrzo_uint8)(packet[pos]>>6);
    if ( sizetype==0 )
    { iscompressed=LCRZO_FALSE;
      size=packet[pos];
      pos++;
    }
    else if ( sizetype==3 )  
    { if (!canuncompress) return(LCRZO_ERR_OKNOTDECODED);
      iscompressed=LCRZO_TRUE;
      if (pos+1>=packetsize) return(LCRZO_ERR_OKNOTDECODED);
      compresspos= ((packet[pos]&0x3F)<<8) | packet[pos+1];
      pos+=2;
    }
    else
    { return(LCRZO_ERR_OKNOTDECODED);
    }
    /* use size */
    if (!iscompressed)
    { if (size==0)
      { /* it's the end */
        break;
      }
      else
      { if (pos+size>=packetsize) return(LCRZO_ERR_OKNOTDECODED);
        lcrzo_er(lcrzo_string_alloc(size, &tmp));
	/* equivalent of memcpy(tmp, packet+pos, size); */
	p1=packet+pos;
	p2=tmp;
        for ( i=0 ; i<size ; i++ )
	{ c=*p1;
  	  if ( c>='A' && c<='Z' ) c=(lcrzo_int8)(c-'A'+'a');
          *p2=c;
          p1++;
          p2++;
	}
	tmp[size]='\0';
        pos+=size;
      }      
    }
    /* use compression */
    else
    { if ( compresspos>=pos ) return(LCRZO_ERR_OKNOTDECODED);
      ret=shared_priv_dns_packet_decode_compress2(packet, packetsize,
						 LCRZO_TRUE,
						 &compresspos, 
						 level+1, &tmp);
      /* compression is always at end */
      lastpartreached=LCRZO_TRUE;
    }
    /* now append tmp */
    if (addapoint)
    { lcrzo_er(lcrzo_string_appendm_text(".", pstr));
    }
    lcrzo_er(lcrzo_string_appendm_text(tmp, pstr));
    lcrzo_er(lcrzo_string_free2(&tmp));
    addapoint=LCRZO_TRUE;
  }

  /* set output */
  *preatchedpacketsize=pos;
  if (level==SHARED_PRIV_DNS_DECODE_COMPRESS_START)
  { /* add a '.' only if it's not an IP address */
    ret2=lcrzo_ipl_init_ips(*pstr, NULL);
    if (ret2!=LCRZO_ERR_OK)
    { lcrzo_er(lcrzo_string_appendm_text(".", pstr));
    }
  }
  if (ret!=LCRZO_ERR_OK)
  { lcrzo_er(lcrzo_string_free2(pstr));
  }
  return(ret);
}

/*-------------------------------------------------------------*/
int shared_priv_dns_packet_decode_compress(lcrzo_constdata packet,
					   lcrzo_int32 packetsize,
					   lcrzo_bool canuncompress,
					   lcrzo_int32 *preatchedpacketsize,
					   lcrzo_string *pstr);
int shared_priv_dns_packet_decode_compress(lcrzo_constdata packet,
					   lcrzo_int32 packetsize,
					   lcrzo_bool canuncompress,
					   lcrzo_int32 *preatchedpacketsize,
					   lcrzo_string *pstr)
{ return(shared_priv_dns_packet_decode_compress2(packet, packetsize,
						 canuncompress,
						 preatchedpacketsize,
					 SHARED_PRIV_DNS_DECODE_COMPRESS_START,
						 pstr));
}

/*-------------------------------------------------------------*/
int shared_priv_dns_packet_decode_unrecomp(lcrzo_constdata packet,
					   lcrzo_int32 packetsize,
					   lcrzo_int32 *preatchedpacketsize,
					   lcrzo_data *pdata,
					   lcrzo_int32 *pdatasize);
int shared_priv_dns_packet_decode_unrecomp(lcrzo_constdata packet,
					   lcrzo_int32 packetsize,
					   lcrzo_int32 *preatchedpacketsize,
					   lcrzo_data *pdata,
					   lcrzo_int32 *pdatasize)
{ lcrzo_string str;
  int ret;

  lcrzo_er(shared_priv_dns_packet_decode_compress(packet, packetsize,
						  LCRZO_TRUE,
						  preatchedpacketsize,
						  &str));
  ret=shared_dns_priv_data_initm_host(str, pdata, pdatasize);
  lcrzo_er(lcrzo_string_free2(&str));
  return(ret);
}

/*-------------------------------------------------------------*/
int shared_priv_dns_packet_decode_rr_h(lcrzo_constdata packet,
				       lcrzo_int32 packetsize,
				       lcrzo_int32 *preatchedpacketsize,
				       shared_dns_rr *prr);
int shared_priv_dns_packet_decode_rr_h(lcrzo_constdata packet,
				       lcrzo_int32 packetsize,
				       lcrzo_int32 *preatchedpacketsize,
				       shared_dns_rr *prr)
{ 
  lcrzo_er(shared_dns_rr_initdefault(prr));

  lcrzo_er(shared_priv_dns_packet_decode_compress(packet, packetsize,
						  LCRZO_TRUE,
						  preatchedpacketsize,
						  &(prr->name)));

  if (*preatchedpacketsize+2>packetsize) return(LCRZO_ERR_OKNOTDECODED);
  prr->type=shared_priv_uint16(packet, *preatchedpacketsize);
  (*preatchedpacketsize)+=2;

  if (*preatchedpacketsize+2>packetsize) return(LCRZO_ERR_OKNOTDECODED);
  prr->rrclass=shared_priv_uint16(packet, *preatchedpacketsize);
  (*preatchedpacketsize)+=2;

#if SHARED_DNS_DECODE_STRICT>=1
  /* check type */
  switch (prr->type) {
  case SHARED_DNS_RR_TYPE_A :
  case SHARED_DNS_RR_TYPE_NS :
  case SHARED_DNS_RR_TYPE_CNAME :
  case SHARED_DNS_RR_TYPE_SOA :
  case SHARED_DNS_RR_TYPE_PTR :
  case SHARED_DNS_RR_TYPE_HINFO :
  case SHARED_DNS_RR_TYPE_MX :
  case SHARED_DNS_RR_TYPE_TXT :
  case SHARED_DNS_RR_TYPE_AAAA :
  case SHARED_DNS_RR_TYPE_AXFR :
  case SHARED_DNS_RR_TYPE_ALL :
  case 18 : /* AFS Data Base location - RFC 1183 - cern.ch uses it */
    break;
  case 41 : /* OPT - RFC2671 */
    /* do not check class in this case */
    return(LCRZO_ERR_OK);
    break;
  default:
    printf("Unknown type : %d\n", prr->type);
    return(LCRZO_ERR_OKNOTDECODED);
  }
  /* check class */
  switch (prr->rrclass) {
  case SHARED_DNS_RR_CLASS_IN :
  case SHARED_DNS_RR_CLASS_CS :
  case SHARED_DNS_RR_CLASS_CH :
  case SHARED_DNS_RR_CLASS_HS :
  case SHARED_DNS_RR_CLASS_ALL :
  case 254 : /* None - RFC2136 */
    break;
  default:
    printf("Unknown class : %d\n", prr->rrclass);
    return(LCRZO_ERR_OKNOTDECODED);
  }
#endif
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_priv_dns_packet_decode_rdata_ns(lcrzo_constdata packet,
					   lcrzo_int32 packetsize,
					   lcrzo_int32 reatchedpacketsize,
					   lcrzo_int32 rdatasize,
					   lcrzo_data *prdata,
					   lcrzo_int32 *prdatasize);
int shared_priv_dns_packet_decode_rdata_ns(lcrzo_constdata packet,
					   lcrzo_int32 packetsize,
					   lcrzo_int32 reatchedpacketsize,
					   lcrzo_int32 rdatasize,
					   lcrzo_data *prdata,
					   lcrzo_int32 *prdatasize)
{ lcrzo_int32 posbefore;

  posbefore=reatchedpacketsize;
  lcrzo_er(shared_priv_dns_packet_decode_unrecomp(packet, packetsize,
						  &reatchedpacketsize,
						  prdata, prdatasize));
  if (reatchedpacketsize-posbefore!=rdatasize) 
    return(LCRZO_ERR_OKNOTDECODED);

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_priv_dns_packet_decode_rdata_soa(lcrzo_constdata packet,
					    lcrzo_int32 packetsize,
					    lcrzo_int32 reatchedpacketsize,
					    lcrzo_int32 rdatasize,
					    lcrzo_data *prdata,
					    lcrzo_int32 *prdatasize);
int shared_priv_dns_packet_decode_rdata_soa(lcrzo_constdata packet,
					    lcrzo_int32 packetsize,
					    lcrzo_int32 reatchedpacketsize,
					    lcrzo_int32 rdatasize,
					    lcrzo_data *prdata,
					    lcrzo_int32 *prdatasize)
{ lcrzo_data data, tmpdata;
  lcrzo_int32 datasize, tmpdatasize, posbefore;

  posbefore=reatchedpacketsize;
  lcrzo_er(shared_priv_dns_packet_decode_unrecomp(packet, packetsize,
						  &reatchedpacketsize,
						  &tmpdata, &tmpdatasize));
  lcrzo_er(lcrzo_data_initm_data(tmpdata, tmpdatasize, &data, &datasize));
  lcrzo_er(lcrzo_data_free2(&tmpdata));
  lcrzo_er(shared_priv_dns_packet_decode_unrecomp(packet, packetsize,
						  &reatchedpacketsize,
						  &tmpdata, &tmpdatasize));
  lcrzo_er(lcrzo_data_appendm_data(tmpdata, tmpdatasize, datasize,
				   &data, &datasize));
  lcrzo_er(lcrzo_data_free2(&tmpdata));

  if (reatchedpacketsize-posbefore!=rdatasize-20) 
    return(LCRZO_ERR_OKNOTDECODED);

  lcrzo_er(lcrzo_data_appendm_data(packet+reatchedpacketsize, 20, datasize,
				   &data, &datasize));
  
  if (prdata!=NULL)
    *prdata=data;
  else
    { lcrzo_er(lcrzo_data_free2(&data)); }
  if (prdatasize!=NULL) *prdatasize=datasize;
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_priv_dns_packet_decode_rdata_mx(lcrzo_constdata packet,
					   lcrzo_int32 packetsize,
					   lcrzo_int32 reatchedpacketsize,
					   lcrzo_int32 rdatasize,
					   lcrzo_data *prdata,
					   lcrzo_int32 *prdatasize);
int shared_priv_dns_packet_decode_rdata_mx(lcrzo_constdata packet,
					   lcrzo_int32 packetsize,
					   lcrzo_int32 reatchedpacketsize,
					   lcrzo_int32 rdatasize,
					   lcrzo_data *prdata,
					   lcrzo_int32 *prdatasize)
{ lcrzo_data data, tmpdata;
  lcrzo_int32 datasize, tmpdatasize, posbefore;

  posbefore=reatchedpacketsize;

  if (rdatasize<2) return(LCRZO_ERR_OKNOTDECODED);
  lcrzo_er(lcrzo_data_initm_data(packet+reatchedpacketsize, 2,
				 &data, &datasize));
  reatchedpacketsize+=2;

  lcrzo_er(shared_priv_dns_packet_decode_unrecomp(packet, packetsize,
						  &reatchedpacketsize,
						  &tmpdata, &tmpdatasize));
  lcrzo_er(lcrzo_data_appendm_data(tmpdata, tmpdatasize, datasize,
				   &data, &datasize));
  lcrzo_er(lcrzo_data_free2(&tmpdata));

  if (reatchedpacketsize-posbefore!=rdatasize) 
    return(LCRZO_ERR_OKNOTDECODED);
  
  if (prdata!=NULL)
    *prdata=data;
  else
    { lcrzo_er(lcrzo_data_free2(&data)); }
  if (prdatasize!=NULL) *prdatasize=datasize;
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_priv_dns_packet_decode_rr_b(lcrzo_constdata packet,
				       lcrzo_int32 packetsize,
				       lcrzo_int32 *preatchedpacketsize,
				       shared_dns_rr *prr);
int shared_priv_dns_packet_decode_rr_b(lcrzo_constdata packet,
				       lcrzo_int32 packetsize,
				       lcrzo_int32 *preatchedpacketsize,
				       shared_dns_rr *prr)
{ lcrzo_constdata rdata;
  lcrzo_int32 rdatasize;
  int ret;
 
  if (*preatchedpacketsize+4>packetsize) return(LCRZO_ERR_OKNOTDECODED);
  prr->ttl=shared_priv_uint32(packet, *preatchedpacketsize);
  (*preatchedpacketsize)+=4;

  if (*preatchedpacketsize+2>packetsize) return(LCRZO_ERR_OKNOTDECODED);
  rdatasize=shared_priv_uint16(packet, *preatchedpacketsize);
  (*preatchedpacketsize)+=2;

  if (*preatchedpacketsize+rdatasize>packetsize)
    return(LCRZO_ERR_OKNOTDECODED);
  rdata=packet + *preatchedpacketsize;
  switch(prr->type) {
  case SHARED_DNS_RR_TYPE_A :
  case SHARED_DNS_RR_TYPE_HINFO :
  case SHARED_DNS_RR_TYPE_TXT :
  case SHARED_DNS_RR_TYPE_AAAA :
    ret=lcrzo_data_initm_data(rdata, rdatasize, 
			      &(prr->rdata), &(prr->rdatasize));    
    break;
  case SHARED_DNS_RR_TYPE_NS :
  case SHARED_DNS_RR_TYPE_CNAME :
  case SHARED_DNS_RR_TYPE_PTR :
    ret=shared_priv_dns_packet_decode_rdata_ns(packet, packetsize,
					       *preatchedpacketsize,
					       rdatasize, 
					       &(prr->rdata),
					       &(prr->rdatasize));    
    break;
  case SHARED_DNS_RR_TYPE_SOA :
    ret=shared_priv_dns_packet_decode_rdata_soa(packet, packetsize,
						*preatchedpacketsize,
						rdatasize, 
						&(prr->rdata),
						&(prr->rdatasize));    
    break;
  case SHARED_DNS_RR_TYPE_MX :
    ret=shared_priv_dns_packet_decode_rdata_mx(packet, packetsize,
					       *preatchedpacketsize,
					       rdatasize, 
					       &(prr->rdata),
					       &(prr->rdatasize));    
    break;
  default : 
#if SHARED_DNS_DECODE_STRICT==2
    printf("Unknown type : %d\n", prr->type);
    ret=LCRZO_ERR_OKNOTDECODED;
#else
    ret=lcrzo_data_initm_data(rdata, rdatasize,
			      &(prr->rdata), &(prr->rdatasize));
#endif
  }
  if (ret!=LCRZO_ERR_OK)
  {
#if SHARED_DNS_DECODE_STRICT>=1
    lcrzo_er(lcrzo_data_initm_data(rdata, rdatasize,
				   &(prr->rdata), &(prr->rdatasize)));
#else
    ret=lcrzo_data_initm_data(rdata, rdatasize,
			      &(prr->rdata), &(prr->rdatasize));
#endif
  }
  (*preatchedpacketsize)+=rdatasize;

  return(ret);
}

/*-------------------------------------------------------------*/
/*-------------------------------------------------------------*/
/*-------------------------------------------------------------*/
/*-------------------------------------------------------------*/

/*-------------------------------------------------------------*/
int shared_dns_header_initdefault(shared_dns_header *pdnsheader)
{
  if (pdnsheader!=NULL)
  { lcrzo_er(lcrzo_uint16_init_rand(0, 0xFFFFu, &(pdnsheader->id)));
    pdnsheader->isaresponse=LCRZO_FALSE;
    pdnsheader->opcode=SHARED_DNS_HEADER_OPCODE_QUERY;
    pdnsheader->authanswer=LCRZO_FALSE;
    pdnsheader->wastruncated=LCRZO_FALSE;
    pdnsheader->recurdesired=LCRZO_FALSE;
    pdnsheader->recuravail=LCRZO_FALSE;
    pdnsheader->rcode=SHARED_DNS_HEADER_RCODE_OK;
    pdnsheader->count_question=0;
    pdnsheader->count_answer=0;
    pdnsheader->count_authority=0;
    pdnsheader->count_additional=0;
  }
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_dns_rr_initdefault(shared_dns_rr *prr)
{
  if (prr!=NULL)
  { prr->name=NULL;
    prr->type=0;
    prr->rrclass=0;
    prr->ttl=0;
    prr->rdata=NULL;
    prr->rdatasize=0;
    prr->kind=0;
  }
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_dns_rr_close(shared_dns_rr *prr)
{
  if (prr!=NULL)
  { if (prr->name!=NULL) 
    { lcrzo_er(lcrzo_string_free2(&(prr->name)));
      prr->name=NULL;
    }
    prr->type=0;
    prr->rrclass=0;
    prr->ttl=0;
    if (prr->rdata!=NULL) 
    { lcrzo_er(lcrzo_data_free2(&(prr->rdata)));
      prr->rdata=NULL;
    }
    prr->rdatasize=0;
    prr->kind=0;
  }
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_dns_rr_init_quest_unknown(lcrzo_conststring rrname,
				     lcrzo_uint16 type,
				     lcrzo_uint16 rrclass,
				     shared_dns_rr *prr)
{ 
  if (prr!=NULL)
  { lcrzo_er(lcrzo_string_initm_text(rrname, &(prr->name)));
    prr->type=type;
    prr->rrclass=rrclass;
    prr->rdata=NULL;
    prr->rdatasize=0;
    prr->kind=SHARED_DNS_RR_KIND_QUESTION;
  }
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_dns_rr_init_quest_a(lcrzo_conststring rrname,
			       shared_dns_rr *prr)
{ 
  lcrzo_er(shared_dns_rr_init_quest_unknown(rrname,
					    SHARED_DNS_RR_TYPE_A,
					    SHARED_DNS_RR_CLASS_IN,
					    prr));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_dns_rr_init_quest_ns(lcrzo_conststring rrname,
				shared_dns_rr *prr)
{ 
  lcrzo_er(shared_dns_rr_init_quest_unknown(rrname,
					    SHARED_DNS_RR_TYPE_NS,
					    SHARED_DNS_RR_CLASS_IN,
					    prr));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_dns_rr_init_quest_cname(lcrzo_conststring rrname,
				   shared_dns_rr *prr)
{ 
  lcrzo_er(shared_dns_rr_init_quest_unknown(rrname,
					    SHARED_DNS_RR_TYPE_CNAME,
					    SHARED_DNS_RR_CLASS_IN,
					    prr));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_dns_rr_init_quest_soa(lcrzo_conststring rrname,
				 shared_dns_rr *prr)
{ 
  lcrzo_er(shared_dns_rr_init_quest_unknown(rrname,
					    SHARED_DNS_RR_TYPE_SOA,
					    SHARED_DNS_RR_CLASS_IN,
					    prr));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_dns_rr_init_quest_ptr(lcrzo_conststring rrname,
				 shared_dns_rr *prr)
{ 
  lcrzo_er(shared_dns_rr_init_quest_unknown(rrname,
					    SHARED_DNS_RR_TYPE_PTR,
					    SHARED_DNS_RR_CLASS_IN,
					    prr));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_dns_rr_init_quest_hinfo(lcrzo_conststring rrname,
				   shared_dns_rr *prr)
{ 
  lcrzo_er(shared_dns_rr_init_quest_unknown(rrname,
					    SHARED_DNS_RR_TYPE_HINFO,
					    SHARED_DNS_RR_CLASS_IN,
					    prr));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_dns_rr_init_quest_mx(lcrzo_conststring rrname,
				shared_dns_rr *prr)
{ 
  lcrzo_er(shared_dns_rr_init_quest_unknown(rrname,
					    SHARED_DNS_RR_TYPE_MX,
					    SHARED_DNS_RR_CLASS_IN,
					    prr));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_dns_rr_init_quest_txt(lcrzo_conststring rrname,
				 shared_dns_rr *prr)
{ 
  lcrzo_er(shared_dns_rr_init_quest_unknown(rrname,
					    SHARED_DNS_RR_TYPE_TXT,
					    SHARED_DNS_RR_CLASS_IN,
					    prr));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_dns_rr_init_quest_axfr(lcrzo_conststring rrname,
				  shared_dns_rr *prr)
{ 
  lcrzo_er(shared_dns_rr_init_quest_unknown(rrname,
					    SHARED_DNS_RR_TYPE_AXFR,
					    SHARED_DNS_RR_CLASS_IN,
					    prr));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_dns_rr_init_quest_all(lcrzo_conststring rrname,
				 shared_dns_rr *prr)
{ 
  lcrzo_er(shared_dns_rr_init_quest_unknown(rrname,
					    SHARED_DNS_RR_TYPE_ALL,
					    SHARED_DNS_RR_CLASS_IN,
					    prr));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_dns_rr_init_quest_known(lcrzo_conststring rrname,
				   lcrzo_uint16 type,
				   shared_dns_rr *prr)
{ 
  switch(type) {
    case SHARED_DNS_RR_TYPE_A :
      lcrzo_epr(shared_dns_rr_init_quest_a(rrname, prr));
      break;
    case SHARED_DNS_RR_TYPE_NS :
      lcrzo_epr(shared_dns_rr_init_quest_ns(rrname, prr));
      break;
    case SHARED_DNS_RR_TYPE_CNAME :
      lcrzo_epr(shared_dns_rr_init_quest_cname(rrname, prr));
      break;
    case SHARED_DNS_RR_TYPE_SOA :
      lcrzo_epr(shared_dns_rr_init_quest_soa(rrname, prr));
      break;
    case SHARED_DNS_RR_TYPE_PTR :
      lcrzo_epr(shared_dns_rr_init_quest_ptr(rrname, prr));
      break;
    case SHARED_DNS_RR_TYPE_HINFO :
      lcrzo_epr(shared_dns_rr_init_quest_hinfo(rrname, prr));
      break;
    case SHARED_DNS_RR_TYPE_MX :
      lcrzo_epr(shared_dns_rr_init_quest_mx(rrname, prr));
      break;
    case SHARED_DNS_RR_TYPE_TXT :
      lcrzo_epr(shared_dns_rr_init_quest_txt(rrname, prr));
      break;
    case SHARED_DNS_RR_TYPE_AXFR :
      lcrzo_epr(shared_dns_rr_init_quest_axfr(rrname, prr));
      break;
    case SHARED_DNS_RR_TYPE_ALL :
      lcrzo_epr(shared_dns_rr_init_quest_all(rrname, prr));
      break;
    default:
      return(LCRZOEX_ERR_DNS_BADRRTYPE);
      break;
  }
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_dns_rr_init_resp_unknown(lcrzo_conststring rrname,
				    lcrzo_uint16 type,
				    lcrzo_uint16 rrclass,
				    lcrzo_uint32 ttl,
				    lcrzo_constdata rdata,
				    lcrzo_int32 rdatasize,
				    lcrzo_uint8 kind,
				    shared_dns_rr *prr)
{ 
  if (prr!=NULL)
  { lcrzo_er(lcrzo_string_initm_text(rrname, &(prr->name)));
    prr->type=type;
    prr->rrclass=rrclass;
    prr->ttl=ttl;
    lcrzo_er(lcrzo_data_initm_data(rdata, rdatasize,
				   &(prr->rdata), &(prr->rdatasize)));
    prr->kind=kind;
  }
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_dns_rr_init_resp_a(lcrzo_conststring rrname,
			      lcrzo_uint32 ttl,
			      lcrzo_ipl address,
			      lcrzo_uint8 kind,
			      shared_dns_rr *prr)
{ lcrzo_ipa ipa;

  lcrzo_er(lcrzo_ipa_init_ipl(address, ipa));
  lcrzo_er(shared_dns_rr_init_resp_unknown(rrname,
					   SHARED_DNS_RR_TYPE_A,
					   SHARED_DNS_RR_CLASS_IN,
					   ttl,
					   ipa, LCRZO_IPA_MAXBYTES,
					   kind,
					   prr));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_dns_rr_init_resp_ns(lcrzo_conststring rrname,
			       lcrzo_uint32 ttl,
			       lcrzo_conststring nsdname,
			       lcrzo_uint8 kind,
			       shared_dns_rr *prr)
{ lcrzo_data data;
  lcrzo_int32 datasize;

  lcrzo_er(shared_dns_priv_data_initm_host(nsdname, &data, &datasize));
  lcrzo_er(shared_dns_rr_init_resp_unknown(rrname,
					   SHARED_DNS_RR_TYPE_NS,
					   SHARED_DNS_RR_CLASS_IN,
					   ttl,
					   data, datasize,
					   kind,
					   prr));
  lcrzo_er(lcrzo_data_free2(&data));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_dns_rr_init_resp_cname(lcrzo_conststring rrname,
				  lcrzo_uint32 ttl,
				  lcrzo_conststring cname,
				  lcrzo_uint8 kind,
				  shared_dns_rr *prr)
{ lcrzo_data data;
  lcrzo_int32 datasize;

  lcrzo_er(shared_dns_priv_data_initm_host(cname, &data, &datasize));
  lcrzo_er(shared_dns_rr_init_resp_unknown(rrname,
					   SHARED_DNS_RR_TYPE_CNAME,
					   SHARED_DNS_RR_CLASS_IN,
					   ttl,
					   data, datasize,
					   kind,
					   prr));
  lcrzo_er(lcrzo_data_free2(&data));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_dns_rr_init_resp_soa(lcrzo_conststring rrname,
				lcrzo_uint32 ttl,
				lcrzo_conststring domainname,
				lcrzo_conststring email,
				lcrzo_uint32 serial,
				lcrzo_uint32 refresh,
				lcrzo_uint32 retry,
				lcrzo_uint32 expire,
				lcrzo_uint32 minimum,
				lcrzo_uint8 kind,
				shared_dns_rr *prr)
{ lcrzo_data tmpdata, data;
  lcrzo_int32 tmpdatasize, datasize;

  lcrzo_er(shared_dns_priv_data_initm_host(domainname, 
					   &data, &datasize));
  lcrzo_er(shared_dns_priv_data_initm_host(email, 
					   &tmpdata, &tmpdatasize));
  lcrzo_er(lcrzo_data_appendm_data(tmpdata, tmpdatasize, datasize,
				   &data, &datasize));
  lcrzo_er(lcrzo_data_free2(&tmpdata));
  tmpdatasize=4+4+4+4+4;
  lcrzo_er(lcrzo_data_alloc(tmpdatasize, &tmpdata));
  tmpdata[0]=shared_priv_uint32_0(serial);
  tmpdata[1]=shared_priv_uint32_1(serial);
  tmpdata[2]=shared_priv_uint32_2(serial);
  tmpdata[3]=shared_priv_uint32_3(serial);
  tmpdata[4]=shared_priv_uint32_0(refresh);
  tmpdata[5]=shared_priv_uint32_1(refresh);
  tmpdata[6]=shared_priv_uint32_2(refresh);
  tmpdata[7]=shared_priv_uint32_3(refresh);
  tmpdata[8]=shared_priv_uint32_0(retry);
  tmpdata[9]=shared_priv_uint32_1(retry);
  tmpdata[10]=shared_priv_uint32_2(retry);
  tmpdata[11]=shared_priv_uint32_3(retry);
  tmpdata[12]=shared_priv_uint32_0(expire);
  tmpdata[13]=shared_priv_uint32_1(expire);
  tmpdata[14]=shared_priv_uint32_2(expire);
  tmpdata[15]=shared_priv_uint32_3(expire);
  tmpdata[16]=shared_priv_uint32_0(minimum);
  tmpdata[17]=shared_priv_uint32_1(minimum);
  tmpdata[18]=shared_priv_uint32_2(minimum);
  tmpdata[19]=shared_priv_uint32_3(minimum);
  lcrzo_er(lcrzo_data_appendm_data(tmpdata, tmpdatasize, datasize,
				   &data, &datasize));
  lcrzo_er(lcrzo_data_free2(&tmpdata));

  lcrzo_er(shared_dns_rr_init_resp_unknown(rrname,
					   SHARED_DNS_RR_TYPE_SOA,
					   SHARED_DNS_RR_CLASS_IN,
					   ttl,
					   data, datasize,
					   kind,
					   prr));
  lcrzo_er(lcrzo_data_free2(&data));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_dns_rr_init_resp_ptr(lcrzo_conststring rrname,
				lcrzo_uint32 ttl,
				lcrzo_conststring ptrdname,
				lcrzo_uint8 kind,
				shared_dns_rr *prr)
{ lcrzo_data data;
  lcrzo_int32 datasize;

  lcrzo_er(shared_dns_priv_data_initm_host(ptrdname, &data, &datasize));
  lcrzo_er(shared_dns_rr_init_resp_unknown(rrname,
					   SHARED_DNS_RR_TYPE_PTR,
					   SHARED_DNS_RR_CLASS_IN,
					   ttl,
					   data, datasize,
					   kind,
					   prr));
  lcrzo_er(lcrzo_data_free2(&data));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_dns_rr_init_resp_hinfo(lcrzo_conststring rrname,
				  lcrzo_uint32 ttl,
				  lcrzo_conststring cpu,
				  lcrzo_conststring os,
				  lcrzo_uint8 kind,
				  shared_dns_rr *prr)
{ lcrzo_data data;
  lcrzo_int32 datasize;
  int strlencpu, strlenos;

  strlencpu=strlen(cpu);
  if (strlencpu>254) return(LCRZO_ERR_PATOOBIG);
  strlenos=strlen(os);
  if (strlenos>254) return(LCRZO_ERR_PATOOBIG);
  datasize=strlencpu+strlenos+2;
  lcrzo_er(lcrzo_data_alloc(datasize, &data));
  data[0]=(lcrzo_uint8)strlencpu;
  memcpy(data+1, cpu, strlencpu);
  data[strlencpu+1]=(lcrzo_uint8)strlenos;
  memcpy(data+1+strlencpu+1, os, strlenos);

  lcrzo_er(shared_dns_rr_init_resp_unknown(rrname,
					   SHARED_DNS_RR_TYPE_HINFO,
					   SHARED_DNS_RR_CLASS_IN,
					   ttl,
					   data, datasize,
					   kind,
					   prr));
  lcrzo_er(lcrzo_data_free2(&data));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_dns_rr_init_resp_mx(lcrzo_conststring rrname,
			       lcrzo_uint32 ttl,
			       lcrzo_uint16 preference,
			       lcrzo_conststring exchange,
			       lcrzo_uint8 kind,
			       shared_dns_rr *prr)
{ lcrzo_data data, tmpdata;
  lcrzo_int32 datasize, tmpdatasize;

  datasize=2;
  lcrzo_er(lcrzo_data_alloc(datasize, &data));
  data[0]=shared_priv_uint16_0(preference);
  data[1]=shared_priv_uint16_1(preference);

  lcrzo_er(shared_dns_priv_data_initm_host(exchange, &tmpdata, &tmpdatasize));
  lcrzo_er(lcrzo_data_appendm_data(tmpdata, tmpdatasize, 
				   datasize, &data, &datasize));
  lcrzo_er(lcrzo_data_free2(&tmpdata));

  lcrzo_er(shared_dns_rr_init_resp_unknown(rrname,
					   SHARED_DNS_RR_TYPE_MX,
					   SHARED_DNS_RR_CLASS_IN,
					   ttl,
					   data, datasize,
					   kind,
					   prr));
  lcrzo_er(lcrzo_data_free2(&data));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_dns_rr_init_resp_txt(lcrzo_conststring rrname,
				lcrzo_uint32 ttl,
				lcrzo_conststring text,
				lcrzo_uint8 kind,
				shared_dns_rr *prr)
{ lcrzo_data data;
  lcrzo_int32 datasize;
  int strlentext;

  strlentext=strlen(text);
  if (strlentext>254) return(LCRZO_ERR_PATOOBIG);
  datasize=strlentext+1;
  lcrzo_er(lcrzo_data_alloc(datasize, &data));
  data[0]=(lcrzo_uint8)strlentext;
  memcpy(data+1, text, strlentext);

  lcrzo_er(shared_dns_rr_init_resp_unknown(rrname,
					   SHARED_DNS_RR_TYPE_TXT,
					   SHARED_DNS_RR_CLASS_IN,
					   ttl,
					   data, datasize,
					   kind,
					   prr));
  lcrzo_er(lcrzo_data_free2(&data));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_dns_rrlist_init(lcrzo_list *prrlist)
{ return(lcrzo_list_init(prrlist, sizeof(shared_dns_rr),
                         &shared_dns_priv_rrlist_free));
}

/*-------------------------------------------------------------*/
int shared_dns_rrlist_add_pos(lcrzo_list *prrlist,
			      lcrzo_int32 pos,
			      shared_dns_rr rr)
{ shared_dns_rr rr2;

  rr2=rr;
  if ( rr.name!=NULL )
  { lcrzo_er(lcrzo_string_initm_text(rr.name, &(rr2.name)));
  }
  if ( rr.rdata!=NULL )
  { lcrzo_er(lcrzo_data_initm_data(rr.rdata, rr.rdatasize,
				 &(rr2.rdata), NULL));
  }
  lcrzo_er(lcrzo_list_add_pos(prrlist, pos, &rr2));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_dns_rrlist_value_pos(lcrzo_list *prrlist,
				lcrzo_int32 pos,
				shared_dns_rr *prr)
{ shared_dns_rr rr;

  lcrzo_er(lcrzo_list_value_pos(prrlist, pos, &rr));
  if (prr!=NULL)
  { *prr=rr;
    if ( rr.name!=NULL )
    { lcrzo_er(lcrzo_string_initm_text(rr.name, &(prr->name)));
    }
    if ( rr.rdata!=NULL )
    { lcrzo_er(lcrzo_data_initm_data(rr.rdata, rr.rdatasize,
				     &(prr->rdata), NULL));
    }
  }
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_dns_rdata_decode_a(lcrzo_constdata rdata,
			      lcrzo_int32 rdatasize,
			      lcrzo_ipl *paddress)
{ 
  if (rdatasize!=4) return(LCRZO_ERR_OKNOTDECODED);
  lcrzo_er(lcrzo_ipl_init_ipa(rdata, paddress));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_dns_rdata_decode_ns(lcrzo_constdata rdata,
			       lcrzo_int32 rdatasize,
			       lcrzo_string *pnsdname)
{ lcrzo_int32 pos;

  pos=0;
  lcrzo_er(shared_priv_dns_packet_decode_compress(rdata, rdatasize,
						  LCRZO_FALSE, &pos,
						  pnsdname));
  if (pos!=rdatasize)
  { if (pnsdname!=NULL) { lcrzo_er(lcrzo_string_free2(pnsdname)); }
    return(LCRZO_ERR_OKNOTDECODED);
  }
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_dns_rdata_decode_cname(lcrzo_constdata rdata,
				  lcrzo_int32 rdatasize,
				  lcrzo_string *pcname)
{ lcrzo_int32 pos;

  pos=0;
  lcrzo_er(shared_priv_dns_packet_decode_compress(rdata, rdatasize,
						  LCRZO_FALSE, &pos,
						  pcname));
  if (pos!=rdatasize)
  { if (pcname!=NULL) { lcrzo_er(lcrzo_string_free2(pcname)); }
    return(LCRZO_ERR_OKNOTDECODED);
  }
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_dns_rdata_decode_soa(lcrzo_constdata rdata,
				lcrzo_int32 rdatasize,
				lcrzo_string *pdomainname,
				lcrzo_string *pemail,
				lcrzo_uint32 *pserial,
				lcrzo_uint32 *prefresh,
				lcrzo_uint32 *pretry,
				lcrzo_uint32 *pexpire,
				lcrzo_uint32 *pminimum)
{ lcrzo_int32 pos;

  pos=0;
  lcrzo_er(shared_priv_dns_packet_decode_compress(rdata, rdatasize,
						  LCRZO_FALSE, &pos,
						  pdomainname));
  lcrzo_er(shared_priv_dns_packet_decode_compress(rdata, rdatasize,
						  LCRZO_FALSE, &pos,
						  pemail));
  if (pserial!=NULL) *pserial=shared_priv_uint32(rdata, pos);
  pos+=4;
  if (prefresh!=NULL) *prefresh=shared_priv_uint32(rdata, pos);
  pos+=4;
  if (pretry!=NULL) *pretry=shared_priv_uint32(rdata, pos);
  pos+=4;
  if (pexpire!=NULL) *pexpire=shared_priv_uint32(rdata, pos);
  pos+=4;
  if (pminimum!=NULL) *pminimum=shared_priv_uint32(rdata, pos);
  pos+=4;

  if (pos!=rdatasize)
  { if (pdomainname!=NULL) { lcrzo_er(lcrzo_string_free2(pdomainname)); }
    if (pemail!=NULL) { lcrzo_er(lcrzo_string_free2(pemail)); }
    return(LCRZO_ERR_OKNOTDECODED);
  }
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_dns_rdata_decode_ptr(lcrzo_constdata rdata,
				lcrzo_int32 rdatasize,
				lcrzo_string *pptrdname)
{ lcrzo_int32 pos;

  pos=0;
  lcrzo_er(shared_priv_dns_packet_decode_compress(rdata, rdatasize,
						  LCRZO_FALSE, &pos,
						  pptrdname));
  if (pos!=rdatasize)
  { if (pptrdname!=NULL) { lcrzo_er(lcrzo_string_free2(pptrdname)); }
    return(LCRZO_ERR_OKNOTDECODED);
  }
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_dns_rdata_decode_hinfo(lcrzo_constdata rdata,
				  lcrzo_int32 rdatasize,
				  lcrzo_string *pcpu,
				  lcrzo_string *pos)
{ lcrzo_uint8 cpusize, ossize;

  if (rdatasize<2) return(LCRZO_ERR_OKNOTDECODED);
  cpusize=rdata[0];
  if (rdatasize<1+cpusize+1) return(LCRZO_ERR_OKNOTDECODED);
  ossize=rdata[1+cpusize];
  if (rdatasize!=1+cpusize+1+ossize) return(LCRZO_ERR_OKNOTDECODED);

  lcrzo_er(lcrzo_string_alloc(cpusize, pcpu));
  memcpy(*pcpu, rdata+1, cpusize);
  (*pcpu)[cpusize]='\0';
  lcrzo_er(lcrzo_string_alloc(ossize, pos));
  memcpy(*pos, rdata+1+cpusize+1, ossize);
  (*pos)[ossize]='\0';

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_dns_rdata_decode_mx(lcrzo_constdata rdata,
			       lcrzo_int32 rdatasize,
			       lcrzo_uint16 *ppreference,
			       lcrzo_string *pexchange)
{ lcrzo_int32 pos;

  if (rdatasize<2) return(LCRZO_ERR_OKNOTDECODED);
  pos=0;
  if (ppreference!=NULL) *ppreference=shared_priv_uint16(rdata, pos);
  pos+=2;

  lcrzo_er(shared_priv_dns_packet_decode_compress(rdata, rdatasize,
						  LCRZO_FALSE, &pos,
						  pexchange));

  if (pos!=rdatasize)
  { if (pexchange!=NULL) { lcrzo_er(lcrzo_string_free2(pexchange)); }
    return(LCRZO_ERR_OKNOTDECODED);
  }
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_dns_rdata_decode_txt(lcrzo_constdata rdata,
				lcrzo_int32 rdatasize,
				lcrzo_string *ptext)
{ lcrzo_uint8 textsize;

  if (rdatasize<2) return(LCRZO_ERR_OKNOTDECODED);
  textsize=rdata[0];
  if (rdatasize!=1+textsize) return(LCRZO_ERR_OKNOTDECODED);

  lcrzo_er(lcrzo_string_alloc(textsize, ptext));
  memcpy(*ptext, rdata+1, textsize);
  (*ptext)[textsize]='\0';

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_dns_rdata_decode_aaaa(lcrzo_constdata rdata,
				 lcrzo_int32 rdatasize,
				 lcrzo_string *pipv6str)
{ lcrzo_int32 i;
  lcrzo_uint8 first, second;

  if (rdatasize!=16) return(LCRZO_ERR_OKNOTDECODED);
  lcrzo_er(lcrzo_string_initm_text("", pipv6str));
  for ( i=0 ; i<8 ; i++ )
  { first=rdata[2*i];
    second=rdata[2*i+1];
    if (first!=0)    
    { lcrzo_er(lcrzo_string_appendm_uint8(first, "%X", pipv6str));
      lcrzo_er(lcrzo_string_appendm_uint8(second, "%02X", pipv6str));
    }
    else if (second!=0)
    { lcrzo_er(lcrzo_string_appendm_uint8(second, "%X", pipv6str));
    }
    if (i!=7)
    { lcrzo_er(lcrzo_string_appendm_text(":", pipv6str));
    }
  }
    
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_dns_packet_initm(shared_dns_header dnsheader,
			    lcrzo_list *prrlist,
			    lcrzo_data *ppacket,
			    lcrzo_int32 *ppacketsize)
{ lcrzo_int32 i, count;
  lcrzo_uint16 rrcount, kind;
  shared_dns_rr rr;
  lcrzo_data tmpdata, rrdata;
  lcrzo_int32 tmpdatasize, rrdatasize;

  /* initialize an empty rr data */
  rrdatasize=0;
  lcrzo_er(lcrzo_data_alloc(rrdatasize, &rrdata));

  /* update rr data */
  lcrzo_er(lcrzo_list_count(prrlist, &count));
  for ( kind=SHARED_DNS_RR_KIND_QUESTION ;
	kind<=SHARED_DNS_RR_KIND_ADDITIONAL ; kind++ )
  { rrcount=0;
    /* create data representing the rr */
    for ( i=1 ; i<=count ; i++ )
    { lcrzo_er(shared_dns_rrlist_value_pos(prrlist, i, &rr));
      if (rr.kind==kind)
      { if (kind==SHARED_DNS_RR_KIND_QUESTION)
        { lcrzo_er(shared_dns_priv_data_initm_rr_quest(rr, 
						       &tmpdata, 
						       &tmpdatasize));
	}
        else
        { lcrzo_er(shared_dns_priv_data_initm_rr_resp(rr, &tmpdata, 
						      &tmpdatasize));
	}
        lcrzo_er(lcrzo_data_appendm_data(tmpdata, tmpdatasize,
					 rrdatasize, &rrdata, &rrdatasize));
	lcrzo_er(lcrzo_data_free2(&tmpdata));
	rrcount++;
      }
      lcrzo_er(shared_dns_rr_close(&rr));
    }
    /* update the count of rr */
    switch(kind) {
    case SHARED_DNS_RR_KIND_QUESTION : 
      dnsheader.count_question=rrcount;
      break;
    case SHARED_DNS_RR_KIND_ANSWER : 
      dnsheader.count_answer=rrcount;
      break;
    case SHARED_DNS_RR_KIND_AUTHORITY : 
      dnsheader.count_authority=rrcount;
      break;
    case SHARED_DNS_RR_KIND_ADDITIONAL : 
      dnsheader.count_additional=rrcount;
      break;
    }
  }

  /* now create the header data */
  lcrzo_er(shared_dns_priv_data_initm_header(dnsheader,
					     &tmpdata, &tmpdatasize));
  lcrzo_er(lcrzo_data_appendm_data(rrdata, rrdatasize,
				   tmpdatasize, &tmpdata, &tmpdatasize));
  lcrzo_er(lcrzo_data_free2(&rrdata));

  /* set output */
  if (ppacket!=NULL)
  { *ppacket=tmpdata;
  }
  else
  { lcrzo_er(lcrzo_data_free2(&tmpdata));
  }
  if (ppacketsize!=NULL) *ppacketsize=tmpdatasize;

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_dns_packet_decode(lcrzo_constdata packet,
			     lcrzo_int32 packetsize,
			     shared_dns_header *pdnsheader,
			     lcrzo_list *prrlist)
{ lcrzo_uint16 u;
  lcrzo_int32 reatchedpacketsize;
  lcrzo_uint16 count_question, count_answer, count_authority, count_additional;
  lcrzo_uint16 i;
  lcrzo_uint8 kind;
  lcrzo_uint32 count_total;
  shared_dns_rr rr;
  int ret;

  if ( packetsize < 12 ) return(LCRZO_ERR_OKNOTDECODED);

  /* decode the header */
  if (pdnsheader!=NULL)
  { pdnsheader->id=shared_priv_uint16(packet, 0);
    u=shared_priv_uint16(packet, 2);
    pdnsheader->count_question=shared_priv_uint16(packet, 4);
    pdnsheader->count_answer=shared_priv_uint16(packet, 6);
    pdnsheader->count_authority=shared_priv_uint16(packet, 8);
    pdnsheader->count_additional=shared_priv_uint16(packet, 10);
    pdnsheader->isaresponse=(lcrzo_bool)((u&0x8000)?LCRZO_TRUE:LCRZO_FALSE);
    pdnsheader->opcode=(lcrzo_uint8)((u>>11)&0xF);
    pdnsheader->authanswer=(lcrzo_bool)((u&0x0400)?LCRZO_TRUE:LCRZO_FALSE);
    pdnsheader->wastruncated=(lcrzo_bool)((u&0x0200)?LCRZO_TRUE:LCRZO_FALSE);
    pdnsheader->recurdesired=(lcrzo_bool)((u&0x0100)?LCRZO_TRUE:LCRZO_FALSE);
    pdnsheader->recuravail=(lcrzo_bool)((u&0x0080)?LCRZO_TRUE:LCRZO_FALSE);
    pdnsheader->rcode=(lcrzo_uint8)(u&0xF);
#if SHARED_DNS_DECODE_STRICT>=1
    /* to check unrecognized bits */
    u=(u>>4)&7;
    if ( /*(u&1) || ((u>>1)&1) || */ ((u>>2)&1) )
    { printf("Reserved bit set in header : %d\n", u);
      return(LCRZO_ERR_OKNOTDECODED);
    }
    /* check opcode */
    switch (pdnsheader->opcode) {
    case SHARED_DNS_HEADER_OPCODE_QUERY :
    case SHARED_DNS_HEADER_OPCODE_IQUERY :
    case SHARED_DNS_HEADER_OPCODE_STATUS :
    case SHARED_DNS_HEADER_OPCODE_NOTIFY :
    case SHARED_DNS_HEADER_OPCODE_UPDATE :
      break;
    default:
      printf("Unknown opcode : %d\n", pdnsheader->opcode);
      return(LCRZO_ERR_OKNOTDECODED);
    }
    /* check rcode */
    switch (pdnsheader->rcode) {
    case SHARED_DNS_HEADER_RCODE_OK :
    case SHARED_DNS_HEADER_RCODE_FMTERROR :
    case SHARED_DNS_HEADER_RCODE_FAILURE :
    case SHARED_DNS_HEADER_RCODE_NAMEERROR :
    case SHARED_DNS_HEADER_RCODE_UNIMPLEMENTED :
    case SHARED_DNS_HEADER_RCODE_REFUSED :
      break;
    default:
      printf("Unknown rcode : %d\n", pdnsheader->rcode);
      return(LCRZO_ERR_OKNOTDECODED);
    }
#endif
  }

  /* if user does not want the list, do not decode it */
  if (prrlist==NULL) return(LCRZO_ERR_OK);

  /* initialize values */
  count_question=shared_priv_uint16(packet, 4);
  count_answer=shared_priv_uint16(packet, 6);
  count_authority=shared_priv_uint16(packet, 8);
  count_additional=shared_priv_uint16(packet, 10);
  count_total=count_question+count_answer+count_authority+count_additional;
  reatchedpacketsize=12;

  /* initialize the list */
  lcrzo_er(shared_dns_rrlist_init(prrlist));

  /* loop through each rr */
  ret=LCRZO_ERR_OK;
  for ( i=0 ; i<count_total && ret==LCRZO_ERR_OK ; i++ )
  { /* determine the kind */
    if ( i<count_question )
      kind=SHARED_DNS_RR_KIND_QUESTION;
    else if ( i<count_question+count_answer )
      kind=SHARED_DNS_RR_KIND_ANSWER;
    else if ( i<count_question+count_answer+count_authority )
      kind=SHARED_DNS_RR_KIND_AUTHORITY;
    else if ( i<count_question+count_answer+count_authority+count_additional )
      kind=SHARED_DNS_RR_KIND_ADDITIONAL;
    else
      return(LCRZO_ERR_IEINTERNALERROR);
    /* decode common fields */
    ret=shared_priv_dns_packet_decode_rr_h(packet, packetsize,
					   &reatchedpacketsize,
					   &rr);
    if (ret!=LCRZO_ERR_OK) break;
    /* for questions, there is no body */
    if (kind!=SHARED_DNS_RR_KIND_QUESTION)
    { /* decode the body, depending on the type */
      ret=shared_priv_dns_packet_decode_rr_b(packet, packetsize,
					     &reatchedpacketsize,
					     &rr);
      if (ret!=LCRZO_ERR_OK) break;
    }
    /* add in the list */
    rr.kind=kind;
    ret=shared_dns_rrlist_add_pos(prrlist, -1, rr);
    if (ret!=LCRZO_ERR_OK) break;
    ret=shared_dns_rr_close(&rr);
    if (ret!=LCRZO_ERR_OK) break;
  }

  if (ret==LCRZO_ERR_OK && reatchedpacketsize!=packetsize)
  { ret=LCRZO_ERR_OKNOTDECODED;
  }
  if (ret!=LCRZO_ERR_OK)
  { lcrzo_er(lcrzo_list_close(prrlist));
  }
  return(ret);
}

/*-------------------------------------------------------------*/
int shared_dns_rr_type_init_text(lcrzo_conststring stringtype,
				 lcrzo_uint16 *ptype)
{ lcrzo_uint16 type;

  if (!shared_misc_strcasecmp(stringtype, "a"))
    type=SHARED_DNS_RR_TYPE_A;
  else if (!shared_misc_strcasecmp(stringtype, "ns"))
    type=SHARED_DNS_RR_TYPE_NS;
  else if (!shared_misc_strcasecmp(stringtype, "cname"))
    type=SHARED_DNS_RR_TYPE_CNAME;
  else if (!shared_misc_strcasecmp(stringtype, "soa"))
    type=SHARED_DNS_RR_TYPE_SOA;
  else if (!shared_misc_strcasecmp(stringtype, "ptr"))
    type=SHARED_DNS_RR_TYPE_PTR;
  else if (!shared_misc_strcasecmp(stringtype, "hinfo"))
    type=SHARED_DNS_RR_TYPE_HINFO;
  else if (!shared_misc_strcasecmp(stringtype, "mx"))
    type=SHARED_DNS_RR_TYPE_MX;
  else if (!shared_misc_strcasecmp(stringtype, "txt"))
    type=SHARED_DNS_RR_TYPE_TXT;
  else if (!shared_misc_strcasecmp(stringtype, "aaaa"))
    type=SHARED_DNS_RR_TYPE_AAAA;
  else if (!shared_misc_strcasecmp(stringtype, "axfr"))
    type=SHARED_DNS_RR_TYPE_AXFR;
  else if (!shared_misc_strcasecmp(stringtype, "all"))
    type=SHARED_DNS_RR_TYPE_ALL;
  else
    type=0;

  if (ptype!=NULL) *ptype=type;
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_dns_packet_quest_initm(lcrzo_conststring rrname,
				  lcrzo_conststring stringtype,
				  lcrzo_data *ppacket,
				  lcrzo_int32 *ppacketsize)
{ shared_dns_header dnsheader;
  shared_dns_rr dnsrr;
  lcrzo_list rrlist;
  lcrzo_hs hs;
  lcrzo_uint16 type;

  /* init header */
  lcrzo_er(shared_dns_header_initdefault(&dnsheader));
  dnsheader.isaresponse=LCRZO_FALSE;

  /* init rr */
  lcrzo_er(shared_dns_rr_type_init_text(stringtype, &type));
  lcrzo_er(shared_dns_inaddrarpa_initm_hs(rrname, hs));
  lcrzo_er(shared_dns_rr_init_quest_known(hs, type, &dnsrr));
  lcrzo_er(shared_dns_rrlist_init(&rrlist));
  lcrzo_er(shared_dns_rrlist_add_pos(&rrlist, +1, dnsrr));
  lcrzo_er(shared_dns_rr_close(&dnsrr));

  /* init packet */
  lcrzo_er(shared_dns_packet_initm(dnsheader, &rrlist,
				   ppacket, ppacketsize));

  lcrzo_er(lcrzo_list_close(&rrlist));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_dns_packet_resp_a_initm(lcrzo_uint16 id,
				   lcrzo_uint32 ttl,
				   lcrzo_conststring hostname,
				   lcrzo_ipl hostnameipl,
				   lcrzo_conststring authns,
				   lcrzo_ipl authnsipl,
				   lcrzo_data *ppacket,
				   lcrzo_int32 *ppacketsize)
{ shared_dns_header dnsheader;
  shared_dns_rr dnsrr;
  lcrzo_list rrlist;

  /* init header */
  lcrzo_er(shared_dns_header_initdefault(&dnsheader));
  dnsheader.id=id;
  dnsheader.isaresponse=LCRZO_TRUE;
  dnsheader.rcode=SHARED_DNS_HEADER_RCODE_OK;
  dnsheader.count_question=1;
  dnsheader.count_answer=1;
  dnsheader.count_authority=1;
  dnsheader.count_additional=1;

  /* init rr */
  lcrzo_er(shared_dns_rrlist_init(&rrlist));
  lcrzo_er(shared_dns_rr_init_quest_a(hostname, &dnsrr));
  lcrzo_er(shared_dns_rrlist_add_pos(&rrlist, -1, dnsrr));
  lcrzo_er(shared_dns_rr_close(&dnsrr));
  lcrzo_er(shared_dns_rr_init_resp_a(hostname, ttl, hostnameipl,
				     SHARED_DNS_RR_KIND_ANSWER, &dnsrr));
  lcrzo_er(shared_dns_rrlist_add_pos(&rrlist, -1, dnsrr));
  lcrzo_er(shared_dns_rr_close(&dnsrr));
  lcrzo_er(shared_dns_rr_init_resp_ns(hostname, ttl, authns,
				     SHARED_DNS_RR_KIND_AUTHORITY, &dnsrr));
  lcrzo_er(shared_dns_rrlist_add_pos(&rrlist, -1, dnsrr));
  lcrzo_er(shared_dns_rr_close(&dnsrr));
  lcrzo_er(shared_dns_rr_init_resp_a(authns, ttl, authnsipl,
				     SHARED_DNS_RR_KIND_ADDITIONAL, &dnsrr));
  lcrzo_er(shared_dns_rrlist_add_pos(&rrlist, -1, dnsrr));
  lcrzo_er(shared_dns_rr_close(&dnsrr));

  /* init packet */
  lcrzo_er(shared_dns_packet_initm(dnsheader, &rrlist,
				   ppacket, ppacketsize));

  lcrzo_er(lcrzo_list_close(&rrlist));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_dns_packet_resp_ns_initm(lcrzo_uint16 id,
				    lcrzo_uint32 ttl,
				    lcrzo_conststring domain,
				    lcrzo_conststring authns,
				    lcrzo_ipl authnsipl,
				    lcrzo_data *ppacket,
				    lcrzo_int32 *ppacketsize)
{ shared_dns_header dnsheader;
  shared_dns_rr dnsrr;
  lcrzo_list rrlist;

  /* init header */
  lcrzo_er(shared_dns_header_initdefault(&dnsheader));
  dnsheader.id=id;
  dnsheader.isaresponse=LCRZO_TRUE;
  dnsheader.rcode=SHARED_DNS_HEADER_RCODE_OK;
  dnsheader.count_question=1;
  dnsheader.count_answer=1;
  dnsheader.count_additional=1;

  /* init rr */
  lcrzo_er(shared_dns_rrlist_init(&rrlist));
  lcrzo_er(shared_dns_rr_init_quest_ns(domain, &dnsrr));
  lcrzo_er(shared_dns_rrlist_add_pos(&rrlist, -1, dnsrr));
  lcrzo_er(shared_dns_rr_close(&dnsrr));
  lcrzo_er(shared_dns_rr_init_resp_ns(domain, ttl, authns,
				      SHARED_DNS_RR_KIND_ANSWER, &dnsrr));
  lcrzo_er(shared_dns_rrlist_add_pos(&rrlist, -1, dnsrr));
  lcrzo_er(shared_dns_rr_close(&dnsrr));
  lcrzo_er(shared_dns_rr_init_resp_a(authns, ttl, authnsipl,
				     SHARED_DNS_RR_KIND_ADDITIONAL, &dnsrr));
  lcrzo_er(shared_dns_rrlist_add_pos(&rrlist, -1, dnsrr));
  lcrzo_er(shared_dns_rr_close(&dnsrr));

  /* init packet */
  lcrzo_er(shared_dns_packet_initm(dnsheader, &rrlist,
				   ppacket, ppacketsize));

  lcrzo_er(lcrzo_list_close(&rrlist));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_dns_packet_resp_ptr_initm(lcrzo_uint16 id,
				     lcrzo_uint32 ttl,
				     lcrzo_conststring ips,
				     lcrzo_conststring hostname,
				     lcrzo_conststring authns,
				     lcrzo_ipl authnsipl,
				     lcrzo_data *ppacket,
				     lcrzo_int32 *ppacketsize)
{ shared_dns_header dnsheader;
  shared_dns_rr dnsrr;
  lcrzo_list rrlist;

  /* init header */
  lcrzo_er(shared_dns_header_initdefault(&dnsheader));
  dnsheader.id=id;
  dnsheader.isaresponse=LCRZO_TRUE;
  dnsheader.rcode=SHARED_DNS_HEADER_RCODE_OK;
  dnsheader.count_question=1;
  dnsheader.count_answer=1;
  dnsheader.count_authority=1;
  dnsheader.count_additional=1;

  /* init rr */
  lcrzo_er(shared_dns_rrlist_init(&rrlist));
  lcrzo_er(shared_dns_rr_init_quest_ptr(ips, &dnsrr));
  lcrzo_er(shared_dns_rrlist_add_pos(&rrlist, -1, dnsrr));
  lcrzo_er(shared_dns_rr_close(&dnsrr));
  lcrzo_er(shared_dns_rr_init_resp_ptr(ips, ttl, hostname,
				       SHARED_DNS_RR_KIND_ANSWER, &dnsrr));
  lcrzo_er(shared_dns_rrlist_add_pos(&rrlist, -1, dnsrr));
  lcrzo_er(shared_dns_rr_close(&dnsrr));
  lcrzo_er(shared_dns_rr_init_resp_ns(ips, ttl, authns,
				      SHARED_DNS_RR_KIND_AUTHORITY, &dnsrr));
  lcrzo_er(shared_dns_rrlist_add_pos(&rrlist, -1, dnsrr));
  lcrzo_er(shared_dns_rr_close(&dnsrr));
  lcrzo_er(shared_dns_rr_init_resp_a(authns, ttl, authnsipl,
				     SHARED_DNS_RR_KIND_ADDITIONAL, &dnsrr));
  lcrzo_er(shared_dns_rrlist_add_pos(&rrlist, -1, dnsrr));
  lcrzo_er(shared_dns_rr_close(&dnsrr));

  /* init packet */
  lcrzo_er(shared_dns_packet_initm(dnsheader, &rrlist,
				   ppacket, ppacketsize));

  lcrzo_er(lcrzo_list_close(&rrlist));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_dns_packet_resp_hinfo_initm(lcrzo_uint16 id,
				       lcrzo_uint32 ttl,
				       lcrzo_conststring hostname,
				       lcrzo_conststring cpu,
				       lcrzo_conststring os,
				       lcrzo_conststring authns,
				       lcrzo_ipl authnsipl,
				       lcrzo_data *ppacket,
				       lcrzo_int32 *ppacketsize)
{ shared_dns_header dnsheader;
  shared_dns_rr dnsrr;
  lcrzo_list rrlist;

  /* init header */
  lcrzo_er(shared_dns_header_initdefault(&dnsheader));
  dnsheader.id=id;
  dnsheader.isaresponse=LCRZO_TRUE;
  dnsheader.rcode=SHARED_DNS_HEADER_RCODE_OK;
  dnsheader.count_question=1;
  dnsheader.count_answer=1;
  dnsheader.count_authority=1;
  dnsheader.count_additional=1;

  /* init rr */
  lcrzo_er(shared_dns_rrlist_init(&rrlist));
  lcrzo_er(shared_dns_rr_init_quest_hinfo(hostname, &dnsrr));
  lcrzo_er(shared_dns_rrlist_add_pos(&rrlist, -1, dnsrr));
  lcrzo_er(shared_dns_rr_close(&dnsrr));
  lcrzo_er(shared_dns_rr_init_resp_hinfo(hostname, ttl, cpu, os,
					 SHARED_DNS_RR_KIND_ANSWER, &dnsrr));
  lcrzo_er(shared_dns_rrlist_add_pos(&rrlist, -1, dnsrr));
  lcrzo_er(shared_dns_rr_close(&dnsrr));
  lcrzo_er(shared_dns_rr_init_resp_ns(hostname, ttl, authns,
				      SHARED_DNS_RR_KIND_AUTHORITY, &dnsrr));
  lcrzo_er(shared_dns_rrlist_add_pos(&rrlist, -1, dnsrr));
  lcrzo_er(shared_dns_rr_close(&dnsrr));
  lcrzo_er(shared_dns_rr_init_resp_a(authns, ttl, authnsipl,
				     SHARED_DNS_RR_KIND_ADDITIONAL, &dnsrr));
  lcrzo_er(shared_dns_rrlist_add_pos(&rrlist, -1, dnsrr));
  lcrzo_er(shared_dns_rr_close(&dnsrr));

  /* init packet */
  lcrzo_er(shared_dns_packet_initm(dnsheader, &rrlist,
				   ppacket, ppacketsize));

  lcrzo_er(lcrzo_list_close(&rrlist));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_dns_packet_resp_mx_initm(lcrzo_uint16 id,
				    lcrzo_uint32 ttl,
				    lcrzo_conststring domain,
				    lcrzo_uint16 preference,
				    lcrzo_conststring exchange,
				    lcrzo_conststring authns,
				    lcrzo_ipl authnsipl,
				    lcrzo_data *ppacket,
				    lcrzo_int32 *ppacketsize)
{ shared_dns_header dnsheader;
  shared_dns_rr dnsrr;
  lcrzo_list rrlist;

  /* init header */
  lcrzo_er(shared_dns_header_initdefault(&dnsheader));
  dnsheader.id=id;
  dnsheader.isaresponse=LCRZO_TRUE;
  dnsheader.rcode=SHARED_DNS_HEADER_RCODE_OK;
  dnsheader.count_question=1;
  dnsheader.count_answer=1;
  dnsheader.count_authority=1;
  dnsheader.count_additional=1;

  /* init rr */
  lcrzo_er(shared_dns_rrlist_init(&rrlist));
  lcrzo_er(shared_dns_rr_init_quest_mx(domain, &dnsrr));
  lcrzo_er(shared_dns_rrlist_add_pos(&rrlist, -1, dnsrr));
  lcrzo_er(shared_dns_rr_close(&dnsrr));
  lcrzo_er(shared_dns_rr_init_resp_mx(domain, ttl, preference, exchange,
				      SHARED_DNS_RR_KIND_ANSWER, &dnsrr));
  lcrzo_er(shared_dns_rrlist_add_pos(&rrlist, -1, dnsrr));
  lcrzo_er(shared_dns_rr_close(&dnsrr));
  lcrzo_er(shared_dns_rr_init_resp_ns(domain, ttl, authns,
				      SHARED_DNS_RR_KIND_AUTHORITY, &dnsrr));
  lcrzo_er(shared_dns_rrlist_add_pos(&rrlist, -1, dnsrr));
  lcrzo_er(shared_dns_rr_close(&dnsrr));
  lcrzo_er(shared_dns_rr_init_resp_a(authns, ttl, authnsipl,
				     SHARED_DNS_RR_KIND_ADDITIONAL, &dnsrr));
  lcrzo_er(shared_dns_rrlist_add_pos(&rrlist, -1, dnsrr));
  lcrzo_er(shared_dns_rr_close(&dnsrr));

  /* init packet */
  lcrzo_er(shared_dns_packet_initm(dnsheader, &rrlist,
				   ppacket, ppacketsize));

  lcrzo_er(lcrzo_list_close(&rrlist));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_dns_packet_resp_txt_initm(lcrzo_uint16 id,
				     lcrzo_uint32 ttl,
				     lcrzo_conststring hostname,
				     lcrzo_conststring text,
				     lcrzo_conststring authns,
				     lcrzo_ipl authnsipl,
				     lcrzo_data *ppacket,
				     lcrzo_int32 *ppacketsize)
{ shared_dns_header dnsheader;
  shared_dns_rr dnsrr;
  lcrzo_list rrlist;

  /* init header */
  lcrzo_er(shared_dns_header_initdefault(&dnsheader));
  dnsheader.id=id;
  dnsheader.isaresponse=LCRZO_TRUE;
  dnsheader.rcode=SHARED_DNS_HEADER_RCODE_OK;
  dnsheader.count_question=1;
  dnsheader.count_answer=1;
  dnsheader.count_authority=1;
  dnsheader.count_additional=1;

  /* init rr */
  lcrzo_er(shared_dns_rrlist_init(&rrlist));
  lcrzo_er(shared_dns_rr_init_quest_txt(hostname, &dnsrr));
  lcrzo_er(shared_dns_rrlist_add_pos(&rrlist, -1, dnsrr));
  lcrzo_er(shared_dns_rr_close(&dnsrr));
  lcrzo_er(shared_dns_rr_init_resp_txt(hostname, ttl, text,
				       SHARED_DNS_RR_KIND_ANSWER, &dnsrr));
  lcrzo_er(shared_dns_rrlist_add_pos(&rrlist, -1, dnsrr));
  lcrzo_er(shared_dns_rr_close(&dnsrr));
  lcrzo_er(shared_dns_rr_init_resp_ns(hostname, ttl, authns,
				      SHARED_DNS_RR_KIND_AUTHORITY, &dnsrr));
  lcrzo_er(shared_dns_rrlist_add_pos(&rrlist, -1, dnsrr));
  lcrzo_er(shared_dns_rr_close(&dnsrr));
  lcrzo_er(shared_dns_rr_init_resp_a(authns, ttl, authnsipl,
				     SHARED_DNS_RR_KIND_ADDITIONAL, &dnsrr));
  lcrzo_er(shared_dns_rrlist_add_pos(&rrlist, -1, dnsrr));
  lcrzo_er(shared_dns_rr_close(&dnsrr));

  /* init packet */
  lcrzo_er(shared_dns_packet_initm(dnsheader, &rrlist,
				   ppacket, ppacketsize));

  lcrzo_er(lcrzo_list_close(&rrlist));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_dns_header_fprint(LCRZOFILE *pf,
			     shared_dns_header dnsheader, 
			     lcrzo_printtype printtype)
{
  lcrzo_er(shared_priv_dns_printtpe_supported(&printtype));

  /* display synthetic */
  if (printtype==LCRZO_PRINTTYPE_SYNTH_)
  { fprintf((FILE*)pf, "DNS");
    if (dnsheader.isaresponse)
      fprintf((FILE*)pf, "A");
    else
      fprintf((FILE*)pf, "Q");
    lcrzo_er(lcrzo_uint16_fprint(pf, "", dnsheader.id, "%u", ""));
    return(LCRZO_ERR_OK);;
  }

  /* now, the printtype is array */
  if (dnsheader.isaresponse)
    fprintf((FILE*)pf, " DNS_answer__________________________");
  else
    fprintf((FILE*)pf, " DNS_question________________________");
  fprintf((FILE*)pf, "____________________________________.\n");

  lcrzo_er(lcrzo_uint16_fprint(pf, " | id=", dnsheader.id, "%-5u",
			       "            "));
  fprintf((FILE*)pf, "rcode=");
  switch (dnsheader.rcode) {
  case SHARED_DNS_HEADER_RCODE_OK :
    fprintf((FILE*)pf, "OK           "); break;
  case SHARED_DNS_HEADER_RCODE_FMTERROR :
    fprintf((FILE*)pf, "FMTERROR     "); break;
  case SHARED_DNS_HEADER_RCODE_FAILURE :
    fprintf((FILE*)pf, "FAILURE      "); break;
  case SHARED_DNS_HEADER_RCODE_NAMEERROR :
    fprintf((FILE*)pf, "NAMEERROR    "); break;
  case SHARED_DNS_HEADER_RCODE_UNIMPLEMENTED :
    fprintf((FILE*)pf, "UNIMPLEMENTED"); break;
  case SHARED_DNS_HEADER_RCODE_REFUSED :
    fprintf((FILE*)pf, "REFUSED      "); break;
  default:
    lcrzo_er(lcrzo_uint8_fprint(pf, "", dnsheader.rcode, "%-2u",
				"           "));
  }
  lcrzo_er(lcrzo_uint8_fprint(pf, "opcode=", dnsheader.opcode, "%-2u", 
			      "         "));
  fprintf((FILE*)pf, "             |\n");
  lcrzo_er(lcrzo_bool_fprint(pf, " | authanswer=", dnsheader.authanswer, 
			     "        "));
  lcrzo_er(lcrzo_bool_fprint(pf, "wastruncated=", dnsheader.wastruncated,
			     "     "));
  lcrzo_er(lcrzo_bool_fprint(pf, "recurdesired=", dnsheader.recurdesired,
			     "  "));
  lcrzo_er(lcrzo_bool_fprint(pf, "recuravail=", dnsheader.recuravail,
			     "   |\n"));
  lcrzo_er(lcrzo_uint16_fprint(pf, " | countquestion=",
			       dnsheader.count_question, 
			       "%-5u"," "));
  lcrzo_er(lcrzo_uint16_fprint(pf, "countanswer=", dnsheader.count_answer,
			       "%-5u","  "));
  lcrzo_er(lcrzo_uint16_fprint(pf, "countauth=", dnsheader.count_authority,
			       "%-5u"," "));
  lcrzo_er(lcrzo_uint16_fprint(pf, "countaddi=", dnsheader.count_additional, 
			       "%-5u", "|\n"));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_priv_dns_rr_fprint(LCRZOFILE *pf,
			      shared_dns_rr dnsrr, 
			      lcrzo_bool printsynth);
int shared_priv_dns_rr_fprint(LCRZOFILE *pf,
			      shared_dns_rr dnsrr, 
			      lcrzo_bool printsynth)
{ int ret;

  if (dnsrr.name!=NULL) fprintf((FILE*)pf, "%s", dnsrr.name);

  if (!printsynth)
  { 
    switch(dnsrr.rrclass) {
    case SHARED_DNS_RR_CLASS_IN : fprintf((FILE*)pf, " IN"); break;
    case SHARED_DNS_RR_CLASS_CS : fprintf((FILE*)pf, " CS"); break;
    case SHARED_DNS_RR_CLASS_CH : fprintf((FILE*)pf, " CH"); break;
    case SHARED_DNS_RR_CLASS_HS : fprintf((FILE*)pf, " HS"); break;
    default : fprintf((FILE*)pf, " class%d", dnsrr.rrclass); break;
    }
  }

  switch(dnsrr.type) {
  case SHARED_DNS_RR_TYPE_A : fprintf((FILE*)pf, " A"); break;
  case SHARED_DNS_RR_TYPE_NS : fprintf((FILE*)pf, " NS"); break;
  case SHARED_DNS_RR_TYPE_CNAME : fprintf((FILE*)pf, " CNAME"); break;
  case SHARED_DNS_RR_TYPE_SOA : fprintf((FILE*)pf, " SOA"); break;
  case SHARED_DNS_RR_TYPE_PTR : fprintf((FILE*)pf, " PTR"); break;
  case SHARED_DNS_RR_TYPE_HINFO : fprintf((FILE*)pf, " HINFO"); break;
  case SHARED_DNS_RR_TYPE_MX : fprintf((FILE*)pf, " MX"); break;
  case SHARED_DNS_RR_TYPE_TXT : fprintf((FILE*)pf, " TXT"); break;
  case SHARED_DNS_RR_TYPE_AAAA : fprintf((FILE*)pf, " AAAA"); break;
  case SHARED_DNS_RR_TYPE_AXFR : fprintf((FILE*)pf, " AXFR"); break;
  case SHARED_DNS_RR_TYPE_ALL : fprintf((FILE*)pf, " ALL"); break;
  default : fprintf((FILE*)pf, " type%d", dnsrr.type); break;
  }

  if (dnsrr.kind==SHARED_DNS_RR_KIND_QUESTION)
  { /* ends here */
    return(LCRZO_ERR_OK);
  }

  if (!printsynth)
  { lcrzo_er(lcrzo_uint32_fprint(pf, " ", dnsrr.ttl, "%u", "s"));
  }

  if ( dnsrr.rdata!=NULL )
  { lcrzo_ipl ipl;
    lcrzo_string str1, str2;
    lcrzo_uint32 u1, u2, u3, u4, u5;
    lcrzo_uint16 v1;

    fprintf((FILE*)pf, " ");
    switch(dnsrr.type) {
    case SHARED_DNS_RR_TYPE_A :
      ret=shared_dns_rdata_decode_a(dnsrr.rdata, dnsrr.rdatasize, &ipl);
      if ( ret==LCRZO_ERR_OK )
      { lcrzo_er(lcrzo_ipl_fprint(pf, "", ipl, ""));
      }
      break;
    case SHARED_DNS_RR_TYPE_NS :
      ret=shared_dns_rdata_decode_ns(dnsrr.rdata, dnsrr.rdatasize, &str1);
      if ( ret==LCRZO_ERR_OK )
      { fprintf((FILE*)pf, "%s", str1);
        lcrzo_er(lcrzo_string_free2(&str1));
      }
      break;
    case SHARED_DNS_RR_TYPE_CNAME :
      ret=shared_dns_rdata_decode_cname(dnsrr.rdata, dnsrr.rdatasize, &str1);
      if ( ret==LCRZO_ERR_OK )
      { fprintf((FILE*)pf, "%s", str1);
        lcrzo_er(lcrzo_string_free2(&str1));
      }
      break;
    case SHARED_DNS_RR_TYPE_SOA :
      ret=shared_dns_rdata_decode_soa(dnsrr.rdata, dnsrr.rdatasize, 
				      &str1, &str2,
				      &u1, &u2, &u3, &u4, &u5);
      if ( ret==LCRZO_ERR_OK )
      { fprintf((FILE*)pf, "%s %s ", str1, str2);
        lcrzo_er(lcrzo_uint32_fprint(pf, "(", u1, "%u", " "));
	lcrzo_er(lcrzo_uint32_fprint(pf, "", u2, "%u", " "));
	lcrzo_er(lcrzo_uint32_fprint(pf, "", u3, "%u", " "));
	lcrzo_er(lcrzo_uint32_fprint(pf, "", u4, "%u", " "));
	lcrzo_er(lcrzo_uint32_fprint(pf, "", u5, "%u", ")"));
	lcrzo_er(lcrzo_string_free2(&str1));
        lcrzo_er(lcrzo_string_free2(&str2));
      }
      break;
    case SHARED_DNS_RR_TYPE_PTR :
      ret=shared_dns_rdata_decode_ptr(dnsrr.rdata, dnsrr.rdatasize, &str1);
      if ( ret==LCRZO_ERR_OK )
      { fprintf((FILE*)pf, "%s", str1);
        lcrzo_er(lcrzo_string_free2(&str1));
      }
      break;
    case SHARED_DNS_RR_TYPE_HINFO :
      ret=shared_dns_rdata_decode_hinfo(dnsrr.rdata, dnsrr.rdatasize, 
					&str1, &str2);
      if ( ret==LCRZO_ERR_OK )
      { fprintf((FILE*)pf, "%s %s", str1, str2);
        lcrzo_er(lcrzo_string_free2(&str1));
        lcrzo_er(lcrzo_string_free2(&str2));
      }
      break;
    case SHARED_DNS_RR_TYPE_MX :
      ret=shared_dns_rdata_decode_mx(dnsrr.rdata, dnsrr.rdatasize, 
				     &v1, &str1);
      if ( ret==LCRZO_ERR_OK )
      { lcrzo_er(lcrzo_uint16_fprint(pf, "", v1, "%u", " "));
	fprintf((FILE*)pf, "%s", str1);
	lcrzo_er(lcrzo_string_free2(&str1));
      }
      break;
    case SHARED_DNS_RR_TYPE_TXT :
      ret=shared_dns_rdata_decode_txt(dnsrr.rdata, dnsrr.rdatasize, &str1);
      if ( ret==LCRZO_ERR_OK )
      { fprintf((FILE*)pf, "%s", str1);
        lcrzo_er(lcrzo_string_free2(&str1));
      }
      break;
    case SHARED_DNS_RR_TYPE_AAAA :
      ret=shared_dns_rdata_decode_aaaa(dnsrr.rdata, dnsrr.rdatasize, &str1);
      if ( ret==LCRZO_ERR_OK )
      { fprintf((FILE*)pf, "%s", str1);
	lcrzo_er(lcrzo_string_free2(&str1));
      }
      break;
    default :
      ret=LCRZO_ERR_IEUNIMPLEMENTED;
    }
    if (ret!=LCRZO_ERR_OK)
    { lcrzo_er(lcrzo_data_fprint(pf, dnsrr.rdata, dnsrr.rdatasize, 
				 LCRZO_PRINTTYPE_HEXA));
    }
  }

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_dns_rr_fprint(LCRZOFILE *pf,
			 shared_dns_rr dnsrr, 
			 lcrzo_printtype printtype)
{ 
  lcrzo_er(shared_priv_dns_printtpe_supported(&printtype));

  if (printtype==LCRZO_PRINTTYPE_SYNTH_)
  { /* display synthetic */
    fprintf((FILE*)pf, " ");
    lcrzo_er(shared_priv_dns_rr_fprint(pf, dnsrr, LCRZO_TRUE));
  }
  else
  { /* now, the printtype is array */
    fprintf((FILE*)pf, " | ");
    lcrzo_er(shared_priv_dns_rr_fprint(pf, dnsrr, LCRZO_FALSE));
    fprintf((FILE*)pf, "\n");
  }
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_priv_dns_rrlist_fprint(LCRZOFILE *pf,
				  shared_dns_header dnsheader, 
				  lcrzo_list *prrlist, 
				  lcrzo_printtype printtype);
int shared_priv_dns_rrlist_fprint(LCRZOFILE *pf,
				  shared_dns_header dnsheader, 
				  lcrzo_list *prrlist, 
				  lcrzo_printtype printtype)
{ lcrzo_int32 count, i;
  shared_dns_rr rr;


  lcrzo_er(shared_priv_dns_printtpe_supported(&printtype));
  lcrzo_er(lcrzo_list_count(prrlist, &count));

  /* display synthetic */
  if (printtype==LCRZO_PRINTTYPE_SYNTH_)
  { if (dnsheader.rcode==SHARED_DNS_HEADER_RCODE_OK)
    { for ( i=1 ; i<=count ; i++ )
      { lcrzo_er(shared_dns_rrlist_value_pos(prrlist, i, &rr));
        if ( !dnsheader.isaresponse && rr.kind==SHARED_DNS_RR_KIND_QUESTION )
	{ lcrzo_er(shared_dns_rr_fprint(pf, rr, printtype));
	}
	else if ( dnsheader.isaresponse && rr.kind==SHARED_DNS_RR_KIND_ANSWER )
        { lcrzo_er(shared_dns_rr_fprint(pf, rr, printtype));
	}
	lcrzo_er(shared_dns_rr_close(&rr));
      }
    }
    else
    { fprintf((FILE*)pf, " Error");
    }
  }
  /* display an array */
  else
  { for ( i=1 ; i<=count ; i++ )
    { lcrzo_er(shared_dns_rrlist_value_pos(prrlist, i, &rr));
      lcrzo_er(shared_dns_rr_fprint(pf, rr, printtype));
      lcrzo_er(shared_dns_rr_close(&rr));
    }
    fprintf((FILE*)pf, " |___________________________________");
    fprintf((FILE*)pf, "____________________________________|\n");
  }
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_dns_fprint(LCRZOFILE *pf,
		      lcrzo_constdata packet,
		      lcrzo_int32 packetsize, 
		      lcrzo_printtype printtype)
{ lcrzo_list rrlist;
  shared_dns_header dnsheader;

  lcrzo_er(shared_dns_packet_decode(packet, packetsize,
				    &dnsheader, &rrlist));

  lcrzo_er(shared_dns_header_fprint(pf, dnsheader, printtype));
  lcrzo_er(shared_priv_dns_rrlist_fprint(pf, dnsheader, &rrlist, printtype));
  lcrzo_er(lcrzo_list_close(&rrlist));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_dns_packet_eth_fprint(LCRZOFILE *pf,
				 lcrzo_constdata ethpacket,
				 lcrzo_int32 ethpacketsize, 
				 lcrzo_printprofile printprofile)
{ lcrzo_data data;
  lcrzo_uint16 datasize;
  lcrzo_hdrleth hdreth;
  lcrzo_hdrlip hdrip;
  lcrzo_hdrludp hdrudp;
  lcrzo_ipopt ipopt;
  lcrzo_uint8 ipoptsize;
  int ret;

  lcrzo_er(lcrzo_packet_decodem_ethipoptudpdata(ethpacket, ethpacketsize,
						&hdreth, 
						&hdrip, ipopt, &ipoptsize,
						&hdrudp, 
						&data, &datasize));
  if ( hdrudp.sport!=53 && hdrudp.dport!=53 )
    return(LCRZO_ERR_OKNOTDECODED);

  /* print eth ip and udp */
  ret=lcrzo_hdrleth_fprint(pf, hdreth, printprofile.hdreth);
  if (ret!=LCRZO_ERR_OK)
  { lcrzo_er(lcrzo_data_free2(&data));
    return(ret);
  }
  lcrzo_er(shared_priv_dns_printtpe_supported(&printprofile.hdreth));
  if (printprofile.hdreth==LCRZO_PRINTTYPE_SYNTH_) fprintf((FILE*)pf, " ");
  ret=lcrzo_hdrlip_fprint(pf, hdrip, printprofile.hdrip);
  if (ret!=LCRZO_ERR_OK)
  { lcrzo_er(lcrzo_data_free2(&data));
    return(ret);
  }
  if (ipoptsize)
  { lcrzo_er(shared_priv_dns_printtpe_supported(&printprofile.hdrip));
    if (printprofile.hdrip==LCRZO_PRINTTYPE_SYNTH_) fprintf((FILE*)pf, " ");
    ret=lcrzo_data_fprint(pf, ipopt, ipoptsize, printprofile.hdrip);
    if (ret!=LCRZO_ERR_OK)
    { lcrzo_er(lcrzo_data_free2(&data));
      return(ret);
    }
  }
  lcrzo_er(shared_priv_dns_printtpe_supported(&printprofile.hdrip));
  if (printprofile.hdrip==LCRZO_PRINTTYPE_SYNTH_) fprintf((FILE*)pf, " ");
  ret=lcrzo_hdrludp_fprint(pf, hdrudp, printprofile.hdrudp);
  if (ret!=LCRZO_ERR_OK)
  { lcrzo_er(lcrzo_data_free2(&data));
    return(ret);
  }

  /* print DNS */
  lcrzo_er(shared_priv_dns_printtpe_supported(&printprofile.hdrudp));
  if (printprofile.hdrudp==LCRZO_PRINTTYPE_SYNTH_) fprintf((FILE*)pf, " ");
  ret=shared_dns_fprint(pf, data, datasize, printprofile.dataudp);
  lcrzo_er(shared_priv_dns_printtpe_supported(&printprofile.dataudp));
  if (printprofile.dataudp==LCRZO_PRINTTYPE_SYNTH_) fprintf((FILE*)pf, "\n");
  lcrzo_er(lcrzo_data_free2(&data));
  return(ret);
}

/*-------------------------------------------------------------*/
int shared_dns_packet_ip_fprint(LCRZOFILE *pf,
				lcrzo_constdata ippacket,
				lcrzo_int32 ippacketsize, 
				lcrzo_printprofile printprofile)
{ lcrzo_data data;
  lcrzo_uint16 datasize;
  lcrzo_hdrlip hdrip;
  lcrzo_hdrludp hdrudp;
  lcrzo_ipopt ipopt;
  lcrzo_uint8 ipoptsize;
  int ret;

  lcrzo_er(lcrzo_packet_decodem_ipoptudpdata(ippacket, ippacketsize,
					     &hdrip, ipopt, &ipoptsize,
					     &hdrudp, 
					     &data, &datasize));
  if ( hdrudp.sport!=53 && hdrudp.dport!=53 )
    return(LCRZO_ERR_OKNOTDECODED);

  /* print ip and udp */
  ret=lcrzo_hdrlip_fprint(pf, hdrip, printprofile.hdrip);
  if (ret!=LCRZO_ERR_OK)
  { lcrzo_er(lcrzo_data_free2(&data));
    return(ret);
  }
  if (ipoptsize)
  { lcrzo_er(shared_priv_dns_printtpe_supported(&printprofile.hdrip));
    if (printprofile.hdrip==LCRZO_PRINTTYPE_SYNTH_) fprintf((FILE*)pf, " ");
    ret=lcrzo_data_fprint(pf, ipopt, ipoptsize, printprofile.hdrip);
    if (ret!=LCRZO_ERR_OK)
    { lcrzo_er(lcrzo_data_free2(&data));
      return(ret);
    }
  }
  lcrzo_er(shared_priv_dns_printtpe_supported(&printprofile.hdrip));
  if (printprofile.hdrip==LCRZO_PRINTTYPE_SYNTH_) fprintf((FILE*)pf, " ");
  ret=lcrzo_hdrludp_fprint(pf, hdrudp, printprofile.hdrudp);
  if (ret!=LCRZO_ERR_OK)
  { lcrzo_er(lcrzo_data_free2(&data));
    return(ret);
  }

  /* print DNS */
  lcrzo_er(shared_priv_dns_printtpe_supported(&printprofile.hdrudp));
  if (printprofile.hdrudp==LCRZO_PRINTTYPE_SYNTH_) fprintf((FILE*)pf, " ");
  ret=shared_dns_fprint(pf, data, datasize, printprofile.dataudp);
  lcrzo_er(shared_priv_dns_printtpe_supported(&printprofile.dataudp));
  if (printprofile.dataudp==LCRZO_PRINTTYPE_SYNTH_) fprintf((FILE*)pf, "\n");
  lcrzo_er(lcrzo_data_free2(&data));
  return(ret);
}

/*-------------------------------------------------------------*/
int shared_dns_inaddrarpa_initm_hs(lcrzo_consths hs,
				   lcrzo_hs inaddrarpahs)
{ lcrzo_ipa ipa;
  lcrzo_string str;
  lcrzo_int8 numberset;
  int ret;

  ret=shared_dns_priv_ipa_init_ips(hs, ipa, &numberset);
  if ( ret==LCRZO_ERR_OK && numberset!=0 )
  { lcrzo_er(lcrzo_string_initm_text("", &str));
    if (numberset==4)
    { lcrzo_er(lcrzo_string_appendm_uint8(ipa[3], "%u", &str));
      lcrzo_er(lcrzo_string_appendm_text(".", &str));
    }
    if (numberset>=3)
    { lcrzo_er(lcrzo_string_appendm_uint8(ipa[2], "%u", &str));
      lcrzo_er(lcrzo_string_appendm_text(".", &str));
    }
    if (numberset>=2)
    { lcrzo_er(lcrzo_string_appendm_uint8(ipa[1], "%u", &str));
      lcrzo_er(lcrzo_string_appendm_text(".", &str));
    }
    if (numberset>=1)
    { lcrzo_er(lcrzo_string_appendm_uint8(ipa[0], "%u", &str));
      lcrzo_er(lcrzo_string_appendm_text(".in-addr.arpa", &str));
    }
    ret=lcrzo_hs_init(str, inaddrarpahs);
    lcrzo_er(lcrzo_string_free2(&str));
    return(ret);
  }
  else
  { lcrzo_er(lcrzo_hs_init(hs, inaddrarpahs));
  }

  return(LCRZO_ERR_OK);
}

