#ifndef LCRZOEX_DNS_INCLUDED
#define LCRZOEX_DNS_INCLUDED 1

/*-------------------------------------------------------------*/
typedef struct {
  lcrzo_uint16 id;
  lcrzo_bool isaresponse;
  lcrzo_uint8 opcode;
  lcrzo_bool authanswer;
  lcrzo_bool wastruncated;
  lcrzo_bool recurdesired;
  lcrzo_bool recuravail;
  lcrzo_uint8 rcode;
  lcrzo_uint16 count_question;
  lcrzo_uint16 count_answer;
  lcrzo_uint16 count_authority;
  lcrzo_uint16 count_additional;
} shared_dns_header;

/*-------------------------------------------------------------*/
#define SHARED_DNS_HEADER_OPCODE_QUERY     0
#define SHARED_DNS_HEADER_OPCODE_IQUERY    1
#define SHARED_DNS_HEADER_OPCODE_STATUS    2
#define SHARED_DNS_HEADER_OPCODE_NOTIFY    4
#define SHARED_DNS_HEADER_OPCODE_UPDATE    5

/*-------------------------------------------------------------*/
#define SHARED_DNS_HEADER_RCODE_OK            0
#define SHARED_DNS_HEADER_RCODE_FMTERROR      1
#define SHARED_DNS_HEADER_RCODE_FAILURE       2
#define SHARED_DNS_HEADER_RCODE_NAMEERROR     3
#define SHARED_DNS_HEADER_RCODE_UNIMPLEMENTED 4
#define SHARED_DNS_HEADER_RCODE_REFUSED       5

/*-------------------------------------------------------------*/
typedef struct {
  lcrzo_string name;
  lcrzo_uint16 type;
  lcrzo_uint16 rrclass; /* cannot use "class" because of C++ */
  lcrzo_uint32 ttl;
  lcrzo_data rdata;
  lcrzo_int32 rdatasize;
  lcrzo_uint8 kind;
} shared_dns_rr;

/*-------------------------------------------------------------*/
#define SHARED_DNS_RR_TYPE_A       1
#define SHARED_DNS_RR_TYPE_NS      2
#define SHARED_DNS_RR_TYPE_CNAME   5 
#define SHARED_DNS_RR_TYPE_SOA     6
#define SHARED_DNS_RR_TYPE_PTR    12
#define SHARED_DNS_RR_TYPE_HINFO  13
#define SHARED_DNS_RR_TYPE_MX     15
#define SHARED_DNS_RR_TYPE_TXT    16
#define SHARED_DNS_RR_TYPE_AAAA   28
#define SHARED_DNS_RR_TYPE_AXFR  252
#define SHARED_DNS_RR_TYPE_ALL   255

/*-------------------------------------------------------------*/
#define SHARED_DNS_RR_CLASS_IN     1
#define SHARED_DNS_RR_CLASS_CS     2
#define SHARED_DNS_RR_CLASS_CH     3
#define SHARED_DNS_RR_CLASS_HS     4
#define SHARED_DNS_RR_CLASS_ALL  255

/*-------------------------------------------------------------*/
#define SHARED_DNS_RR_KIND_QUESTION   1
#define SHARED_DNS_RR_KIND_ANSWER     2
#define SHARED_DNS_RR_KIND_AUTHORITY  3
#define SHARED_DNS_RR_KIND_ADDITIONAL 4

/*-------------------------------------------------------------*/
int shared_dns_header_initdefault(shared_dns_header *pdnsheader);

/*-------------------------------------------------------------*/
int shared_dns_rr_initdefault(shared_dns_rr *prr);
int shared_dns_rr_close(shared_dns_rr *prr);

/*-------------------------------------------------------------*/
int shared_dns_rr_init_quest_unknown(lcrzo_conststring rrname,
				     lcrzo_uint16 type,
				     lcrzo_uint16 rrclass,
				     shared_dns_rr *prr);
int shared_dns_rr_init_quest_a(lcrzo_conststring rrname,
			       shared_dns_rr *prr);
int shared_dns_rr_init_quest_ns(lcrzo_conststring rrname,
				shared_dns_rr *prr);
int shared_dns_rr_init_quest_cname(lcrzo_conststring rrname,
				   shared_dns_rr *prr);
int shared_dns_rr_init_quest_soa(lcrzo_conststring rrname,
				 shared_dns_rr *prr);
int shared_dns_rr_init_quest_ptr(lcrzo_conststring rrname,
				 shared_dns_rr *prr);
int shared_dns_rr_init_quest_hinfo(lcrzo_conststring rrname,
				   shared_dns_rr *prr);
int shared_dns_rr_init_quest_mx(lcrzo_conststring rrname,
				shared_dns_rr *prr);
int shared_dns_rr_init_quest_txt(lcrzo_conststring rrname,
				 shared_dns_rr *prr);
int shared_dns_rr_init_quest_axfr(lcrzo_conststring rrname,
				  shared_dns_rr *prr);
int shared_dns_rr_init_quest_all(lcrzo_conststring rrname,
				 shared_dns_rr *prr);
int shared_dns_rr_init_quest_known(lcrzo_conststring rrname,
				   lcrzo_uint16 type,
				   shared_dns_rr *prr);

/*-------------------------------------------------------------*/
int shared_dns_rr_init_resp_unknown(lcrzo_conststring rrname,
				    lcrzo_uint16 type,
				    lcrzo_uint16 rrclass,
				    lcrzo_uint32 ttl,
				    lcrzo_constdata rdata,
				    lcrzo_int32 rdatasize,
				    lcrzo_uint8 kind,
				    shared_dns_rr *prr);
int shared_dns_rr_init_resp_a(lcrzo_conststring rrname,
			      lcrzo_uint32 ttl,
			      lcrzo_ipl address,
			      lcrzo_uint8 kind,
			      shared_dns_rr *prr);
int shared_dns_rr_init_resp_ns(lcrzo_conststring rrname,
			       lcrzo_uint32 ttl,
			       lcrzo_conststring nsdname,
			       lcrzo_uint8 kind,
			       shared_dns_rr *prr);
int shared_dns_rr_init_resp_cname(lcrzo_conststring rrname,
				  lcrzo_uint32 ttl,
				  lcrzo_conststring cname,
				  lcrzo_uint8 kind,
				  shared_dns_rr *prr);
int shared_dns_rr_init_resp_soa(lcrzo_conststring rrname,
				lcrzo_uint32 ttl,
				lcrzo_conststring domainname,
				lcrzo_conststring email,
				lcrzo_uint32 serial,
				lcrzo_uint32 refresh,
				lcrzo_uint32 retry,
				lcrzo_uint32 expire,
				lcrzo_uint32 minimum,
				lcrzo_uint8 kind,
				shared_dns_rr *prr);
int shared_dns_rr_init_resp_ptr(lcrzo_conststring rrname,
				lcrzo_uint32 ttl,
				lcrzo_conststring ptrdname,
				lcrzo_uint8 kind,
				shared_dns_rr *prr);
int shared_dns_rr_init_resp_hinfo(lcrzo_conststring rrname,
				  lcrzo_uint32 ttl,
				  lcrzo_conststring cpu,
				  lcrzo_conststring os,
				  lcrzo_uint8 kind,
				  shared_dns_rr *prr);
int shared_dns_rr_init_resp_mx(lcrzo_conststring rrname,
			       lcrzo_uint32 ttl,
			       lcrzo_uint16 preference,
			       lcrzo_conststring exchange,
			       lcrzo_uint8 kind,
			       shared_dns_rr *prr);
int shared_dns_rr_init_resp_txt(lcrzo_conststring rrname,
				lcrzo_uint32 ttl,
				lcrzo_conststring text,
				lcrzo_uint8 kind,
				shared_dns_rr *prr);
int shared_dns_rr_init_resp_axfr(lcrzo_conststring rrname,
				 lcrzo_uint32 ttl,
				 lcrzo_uint8 kind,
				 shared_dns_rr *prr);
int shared_dns_rr_init_resp_all(lcrzo_conststring rrname,
				lcrzo_uint32 ttl,
				lcrzo_uint8 kind,
				shared_dns_rr *prr);

/*-------------------------------------------------------------*/
int shared_dns_rrlist_init(lcrzo_list *prrlist);
int shared_dns_rrlist_add_pos(lcrzo_list *prrlist,
			      lcrzo_int32 pos,
			      shared_dns_rr rr);
int shared_dns_rrlist_value_pos(lcrzo_list *prrlist,
				lcrzo_int32 pos,
				shared_dns_rr *prr);

/*-------------------------------------------------------------*/
int shared_dns_rdata_decode_a(lcrzo_constdata rdata,
			      lcrzo_int32 rdatasize,
			      lcrzo_ipl *paddress);
int shared_dns_rdata_decode_ns(lcrzo_constdata rdata,
			       lcrzo_int32 rdatasize,
			       lcrzo_string *pnsdname);
int shared_dns_rdata_decode_cname(lcrzo_constdata rdata,
				  lcrzo_int32 rdatasize,
				  lcrzo_string *pcname);
int shared_dns_rdata_decode_soa(lcrzo_constdata rdata,
				lcrzo_int32 rdatasize,
				lcrzo_string *pdomainname,
				lcrzo_string *pemail,
				lcrzo_uint32 *pserial,
				lcrzo_uint32 *prefresh,
				lcrzo_uint32 *pretry,
				lcrzo_uint32 *pexpire,
				lcrzo_uint32 *pminimum);
int shared_dns_rdata_decode_ptr(lcrzo_constdata rdata,
				lcrzo_int32 rdatasize,
				lcrzo_string *pptrdname);
int shared_dns_rdata_decode_hinfo(lcrzo_constdata rdata,
				  lcrzo_int32 rdatasize,
				  lcrzo_string *pcpu,
				  lcrzo_string *pos);
int shared_dns_rdata_decode_mx(lcrzo_constdata rdata,
			       lcrzo_int32 rdatasize,
			       lcrzo_uint16 *ppreference,
			       lcrzo_string *pexchange);
int shared_dns_rdata_decode_txt(lcrzo_constdata rdata,
				lcrzo_int32 rdatasize,
				lcrzo_string *ptext);
int shared_dns_rdata_decode_aaaa(lcrzo_constdata rdata,
				 lcrzo_int32 rdatasize,
				 lcrzo_string *pipv6str);

/*-------------------------------------------------------------*/
int shared_dns_packet_initm(shared_dns_header dnsheader,
			    lcrzo_list *prrlist,
			    lcrzo_data *ppacket,
			    lcrzo_int32 *ppacketsize);
int shared_dns_packet_decode(lcrzo_constdata packet,
			     lcrzo_int32 packetsize,
			     shared_dns_header *pdnsheader,
			     lcrzo_list *prrlist);

/*-------------------------------------------------------------*/
int shared_dns_rr_type_init_text(lcrzo_conststring stringtype,
				 lcrzo_uint16 *ptype);

/*-------------------------------------------------------------*/
int shared_dns_packet_quest_initm(lcrzo_conststring rrname,
				  lcrzo_conststring stringtype,
				  lcrzo_data *ppacket,
				  lcrzo_int32 *ppacketsize);

/*-------------------------------------------------------------*/
int shared_dns_packet_resp_a_initm(lcrzo_uint16 id,
				   lcrzo_uint32 ttl,
				   lcrzo_conststring hostname,
				   lcrzo_ipl hostnameipl,
				   lcrzo_conststring authns,
				   lcrzo_ipl authnsipl,
				   lcrzo_data *ppacket,
				   lcrzo_int32 *ppacketsize);
int shared_dns_packet_resp_ns_initm(lcrzo_uint16 id,
				    lcrzo_uint32 ttl,
				    lcrzo_conststring domain,
				    lcrzo_conststring authns,
				    lcrzo_ipl authnsipl,
				    lcrzo_data *ppacket,
				    lcrzo_int32 *ppacketsize);
int shared_dns_packet_resp_ptr_initm(lcrzo_uint16 id,
				     lcrzo_uint32 ttl,
				     lcrzo_conststring ips,
				     lcrzo_conststring hostname,
				     lcrzo_conststring authns,
				     lcrzo_ipl authnsipl,
				     lcrzo_data *ppacket,
				     lcrzo_int32 *ppacketsize);
int shared_dns_packet_resp_hinfo_initm(lcrzo_uint16 id,
				       lcrzo_uint32 ttl,
				       lcrzo_conststring hostname,
				       lcrzo_conststring cpu,
				       lcrzo_conststring os,
				       lcrzo_conststring authns,
				       lcrzo_ipl authnsipl,
				       lcrzo_data *ppacket,
				       lcrzo_int32 *ppacketsize);
int shared_dns_packet_resp_mx_initm(lcrzo_uint16 id,
				    lcrzo_uint32 ttl,
				    lcrzo_conststring domain,
				    lcrzo_uint16 preference,
				    lcrzo_conststring exchange,
				    lcrzo_conststring authns,
				    lcrzo_ipl authnsipl,
				    lcrzo_data *ppacket,
				    lcrzo_int32 *ppacketsize);
int shared_dns_packet_resp_txt_initm(lcrzo_uint16 id,
				     lcrzo_uint32 ttl,
				     lcrzo_conststring hostname,
				     lcrzo_conststring text,
				     lcrzo_conststring authns,
				     lcrzo_ipl authnsipl,
				     lcrzo_data *ppacket,
				     lcrzo_int32 *ppacketsize);

/*-------------------------------------------------------------*/
int shared_dns_header_fprint(LCRZOFILE *pf,
			     shared_dns_header dnsheader, 
			     lcrzo_printtype printtype);
int shared_dns_rr_fprint(LCRZOFILE *pf,
			 shared_dns_rr dnsrr, 
			 lcrzo_printtype printtype);
int shared_dns_fprint(LCRZOFILE *pf,
		      lcrzo_constdata packet,
		      lcrzo_int32 packetsize, 
		      lcrzo_printtype printtype);
#define shared_dns_header_print(a,b) shared_dns_header_fprint(stdout,a,b)
#define shared_dns_rr_print(a,b) shared_dns_rr_fprint(stdout,a,b)
#define shared_dns_print(a,b,c) shared_dns_fprint(stdout,a,b,c)

/*-------------------------------------------------------------*/
int shared_dns_packet_eth_fprint(LCRZOFILE *pf,
				 lcrzo_constdata ethpacket,
				 lcrzo_int32 ethpacketsize, 
				 lcrzo_printprofile printprofile);
int shared_dns_packet_ip_fprint(LCRZOFILE *pf,
				lcrzo_constdata ippacket,
				lcrzo_int32 ippacketsize, 
				lcrzo_printprofile printprofile);
#define shared_dns_packet_eth_print(a,b,c) shared_dns_packet_eth_fprint(stdout,a,b,c)
#define shared_dns_packet_ip_print(a,b,c) shared_dns_packet_ip_fprint(stdout,a,b,c)

/*-------------------------------------------------------------*/
int shared_dns_inaddrarpa_initm_hs(lcrzo_consths hs,
				   lcrzo_hs inaddrarpahs);

#endif
