/*
		                 LCRZOEX
                             Network toolbox
             Copyright(c) 1999,2000,2001,2002 Laurent Constantin
                                  -----

  Main server    : http://www.laurentconstantin.com/
  Backup servers : http://go.to/laurentconstantin/
                   http://laurentconstantin.est-la.com/
                   http://laurentconstantin.free.fr/
                   http://membres.lycos.fr/lauconstantin/
  [my current email address is on the web servers]

                                  -----
  This file is part of Lcrzoex.

  Lcrzoex is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Lcrzoex is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details (http://www.gnu.org/).

------------------------------------------------------------------------
*/

#include <lcrzo.h>
#include "../lcrzoex.h"

/*-------------------------------------------------------------*/
int shared_priv_skipfield(lcrzo_conststring str,
			  lcrzo_int32 *ppos,
			  lcrzo_string *pstr);
int shared_priv_skipfield(lcrzo_conststring str,
			  lcrzo_int32 *ppos,
			  lcrzo_string *pstr)
{ int start, end;
  const char *pc;

  pc=str+*ppos;

  while ( *pc==' ' && *pc!='\0' ) pc++;
  if (*pc=='\0') return(LCRZO_ERR_OKDATAEND);
  start=pc-str;

  while ( *pc!=' ' && *pc!='\0' ) pc++;
  end=pc-str;

  lcrzo_er(lcrzo_string_initm_range(str, start+1, end+1, pstr));
  *ppos=end;
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_priv_skipfield_last(lcrzo_conststring str,
			       lcrzo_int32 pos,
			       lcrzo_string *pstr);
int shared_priv_skipfield_last(lcrzo_conststring str,
			       lcrzo_int32 pos,
			       lcrzo_string *pstr)
{ int start;
  const char *pc;

  pc=str+pos;

  while ( *pc==' ' && *pc!='\0' ) pc++;
  if (*pc=='\0') return(LCRZO_ERR_OKDATAEND);
  start=pc-str;

  lcrzo_er(lcrzo_string_initm_range(str, start+1, strlen(str)+1, pstr));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_priv_ftp_decode_unixfile(lcrzo_conststring line,
				    lcrzo_list *plistfile);
int shared_priv_ftp_decode_unixfile(lcrzo_conststring line,
				    lcrzo_list *plistfile)
{ lcrzo_int32 p;
  lcrzo_uint32 size;
  lcrzo_string str;

  p=0;
  lcrzo_er(shared_priv_skipfield(line, &p, NULL));
  lcrzo_er(shared_priv_skipfield(line, &p, NULL));
  lcrzo_er(shared_priv_skipfield(line, &p, NULL));
  lcrzo_er(shared_priv_skipfield(line, &p, NULL));
  lcrzo_er(shared_priv_skipfield(line, &p, &str));
  lcrzo_er(lcrzo_uint32_init_text(str, &size));
  lcrzo_er(lcrzo_string_free2(&str));
  lcrzo_er(shared_priv_skipfield(line, &p, NULL));
  lcrzo_er(shared_priv_skipfield(line, &p, NULL));
  lcrzo_er(shared_priv_skipfield(line, &p, NULL));
  lcrzo_er(shared_priv_skipfield_last(line, p, &str));
  lcrzo_er(lcrzo_stringlist_add_pos(plistfile, -1, size, str));
  lcrzo_er(lcrzo_string_free2(&str));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_priv_ftp_decode_unixlink(lcrzo_conststring line,
				    lcrzo_list *plistlink);
int shared_priv_ftp_decode_unixlink(lcrzo_conststring line,
				    lcrzo_list *plistlink)
{ lcrzo_int32 p;
  lcrzo_string str;

  p=0;
  lcrzo_er(shared_priv_skipfield(line, &p, NULL));
  lcrzo_er(shared_priv_skipfield(line, &p, NULL));
  lcrzo_er(shared_priv_skipfield(line, &p, NULL));
  lcrzo_er(shared_priv_skipfield(line, &p, NULL));
  lcrzo_er(shared_priv_skipfield(line, &p, NULL));
  lcrzo_er(shared_priv_skipfield(line, &p, NULL));
  lcrzo_er(shared_priv_skipfield(line, &p, NULL));
  lcrzo_er(shared_priv_skipfield(line, &p, NULL));
  lcrzo_er(shared_priv_skipfield_last(line, p, &str));
  lcrzo_er(lcrzo_stringlist_add_pos(plistlink, -1, 0, str));
  lcrzo_er(lcrzo_string_free2(&str));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_priv_ftp_decode_unixdir(lcrzo_conststring line,
				   lcrzo_list *plistdir);
int shared_priv_ftp_decode_unixdir(lcrzo_conststring line,
				   lcrzo_list *plistdir)
{ lcrzo_int32 p;
  lcrzo_string str;

  p=0;
  lcrzo_er(shared_priv_skipfield(line, &p, NULL));
  lcrzo_er(shared_priv_skipfield(line, &p, NULL));
  lcrzo_er(shared_priv_skipfield(line, &p, NULL));
  lcrzo_er(shared_priv_skipfield(line, &p, NULL));
  lcrzo_er(shared_priv_skipfield(line, &p, NULL));
  lcrzo_er(shared_priv_skipfield(line, &p, NULL));
  lcrzo_er(shared_priv_skipfield(line, &p, NULL));
  lcrzo_er(shared_priv_skipfield(line, &p, NULL));
  lcrzo_er(shared_priv_skipfield_last(line, p, &str));
  if ( strcmp(str, ".") && strcmp(str, "..") )
  { lcrzo_er(lcrzo_stringlist_add_pos(plistdir, -1, 0, str));
  }
  lcrzo_er(lcrzo_string_free2(&str));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_priv_ftp_decode_windirfile(lcrzo_conststring line,
				      lcrzo_list *plistfile,
				      lcrzo_list *plistdir);
int shared_priv_ftp_decode_windirfile(lcrzo_conststring line,
				      lcrzo_list *plistfile,
				      lcrzo_list *plistdir)
{ lcrzo_int32 p;
  lcrzo_string str;
  lcrzo_uint32 size;
  lcrzo_bool isdir;

  size=0;
  p=0;
  lcrzo_er(shared_priv_skipfield(line, &p, NULL));
  lcrzo_er(shared_priv_skipfield(line, &p, NULL));
  lcrzo_er(shared_priv_skipfield(line, &p, &str));
  if (!strcmp(str, "<DIR>"))
  { isdir=LCRZO_TRUE;
  }
  else
  { isdir=LCRZO_FALSE;
    lcrzo_er(lcrzo_uint32_init_text(str, &size));
  }
  lcrzo_er(lcrzo_string_free2(&str));
  lcrzo_er(shared_priv_skipfield_last(line, p, &str));
  if (isdir)
  { if (plistdir!=NULL)
    { lcrzo_er(lcrzo_stringlist_add_pos(plistdir, -1, 0, str));
    }
  }
  else
  { if (plistfile!=NULL) 
    { lcrzo_er(lcrzo_stringlist_add_pos(plistfile, -1, size, str));
    }
  }
  lcrzo_er(lcrzo_string_free2(&str));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_priv_ftp_decode_dirfiletlink(lcrzo_bool usesock,
				       lcrzo_sock *psock,
				       lcrzo_ipc *pipc,
				       lcrzo_list *plistfile,
				       lcrzo_list *plistdir,
				       lcrzo_list *plisttlink,
				       lcrzo_bool *preaddataisassociatedtodir);
int shared_priv_ftp_decode_dirfiletlink(lcrzo_bool usesock,
				       lcrzo_sock *psock,
				       lcrzo_ipc *pipc,
				       lcrzo_list *plistfile,
				       lcrzo_list *plistdir,
				       lcrzo_list *plisttlink,
				       lcrzo_bool *preaddataisassociatedtodir)
{ lcrzo_data data;
  lcrzo_int32 datasize;
  int ret, i;

  /* init lists */
  if (plistfile!=NULL)
  { lcrzo_epr(lcrzo_stringlist_init(plistfile));
  }
  if (plistdir!=NULL)
  { lcrzo_epr(lcrzo_stringlist_init(plistdir));
  }
  if (plisttlink!=NULL)
  { lcrzo_epr(lcrzo_stringlist_init(plisttlink));
  }
  if (preaddataisassociatedtodir!=NULL)
  { *preaddataisassociatedtodir=LCRZO_FALSE;
  }
  i=0;

  /* display received data */
  while(LCRZO_TRUE)
  { if (usesock)
    { ret=lcrzo_sock_readm_line(psock, LCRZO_WAIT_INFINITE,
				&data, &datasize);
    }
    else
    { ret=lcrzo_ipc_readm_line(pipc, LCRZO_WAIT_INFINITE,
			       &data, &datasize);
    }
    if ( ret!=LCRZO_ERR_OK ) break;
    data[datasize]='\0';

    if ( data[0]=='-' )
    {
      if (plistfile!=NULL)
      { lcrzo_er(shared_priv_ftp_decode_unixfile((lcrzo_string)data, plistfile));
      }
      i++;
    }
    else if ( data[0]=='d' )
    {
      if (plistdir!=NULL) 
      { lcrzo_er(shared_priv_ftp_decode_unixdir((lcrzo_string)data, plistdir));
      }
      i++;
    }
    else if ( data[0]=='l' )
    { 
      if (plisttlink!=NULL) 
      { lcrzo_er(shared_priv_ftp_decode_unixlink((lcrzo_string)data, plisttlink));
      }
      i++;
    }
    else if ( datasize>5 &&
	      data[0]>='0' && data[0]<='9' &&
	      data[1]>='0' && data[1]<='9' &&
	      data[2]=='-' && 
	      data[3]>='0' && data[3]<='9' &&
	      data[4]>='0' && data[4]<='9' )
    { lcrzo_er(shared_priv_ftp_decode_windirfile((lcrzo_string)data, plistfile, plistdir));
      i++;
    }
    
    if ( i>1 && preaddataisassociatedtodir!=NULL )
    { *preaddataisassociatedtodir=LCRZO_TRUE;
    } 

    if ( !strncmp((lcrzo_string)data, "213 ", 4) )
    { /* special end for stat */
      lcrzo_er(lcrzo_data_free2(&data));
      return(LCRZO_ERR_OK);
    }
    lcrzo_er(lcrzo_data_free2(&data));
  }

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_priv_ftp_decode_tlink(lcrzo_sock *psockcontrol,
				lcrzo_conststring dir,
				lcrzo_list *plisttlink,
				lcrzo_list *plistlinkname,
				lcrzo_list *plistlinkobj);
int shared_priv_ftp_decode_tlink(lcrzo_sock *psockcontrol,
				lcrzo_conststring dir,
				lcrzo_list *plisttlink,
				lcrzo_list *plistlinkname,
				lcrzo_list *plistlinkobj)
{ lcrzo_int32 count, i, slashnum, filesize;
  lcrzo_string str, tmpfilen, f1, f2, f2_, pwd;
  char *pc;
  int ret;

  /* init lists */
  if (plistlinkname!=NULL)
  { lcrzo_epr(lcrzo_stringlist_init(plistlinkname));
  }
  if (plistlinkobj!=NULL)
  { lcrzo_epr(lcrzo_stringlist_init(plistlinkobj));
  }

  lcrzo_epr(shared_ftp_cmd_type_bin(psockcontrol));
  lcrzo_epr(lcrzo_list_count(plisttlink, &count));
  for ( i=1 ; i<=count ; i++ )
  { lcrzo_epr(lcrzo_stringlist_valuem_pos(plisttlink, i, NULL, 
					&str));
    /* data contains "f1 -> f2" (suppose no filename contains "->" ; to do
       so, we should compare with NLST) */
    pc=strstr(str, " -> ");
    if ( pc==NULL )
    { /* invalid, ignore */
      lcrzo_er(lcrzo_string_free2(&str));
      continue;
    }
    *pc='\0';

    lcrzo_epr(lcrzo_string_initm_text(str, &f1));
    lcrzo_epr(lcrzo_string_initm_text(pc+4, &f2));
    lcrzo_er(lcrzo_string_free2(&str));
    /* if f2 start with '/', it is absolute */
    if ( f2[0]=='/' )
    { ret=shared_ftp_cmd_pwd(psockcontrol, &pwd);
      if (ret==LCRZO_ERR_OK)
      { if (pwd[strlen(pwd)-1]!='/')
	{ lcrzo_er(lcrzo_string_appendm_char('/', 1, &pwd));
	}
        /* now, count the number of '/' */
        pc=pwd;
        slashnum=0;
        while (pc!=NULL) 
        { pc=strchr(pc, '/');
	  if (pc==NULL) break;
          pc++;
          slashnum++;
	}
        /* create the "../../.." string */
        lcrzo_epr(lcrzo_string_initm_text("", &f2_));
        while(slashnum--)
	{ lcrzo_epr(lcrzo_string_appendm_text("../", &f2_));
	}
        lcrzo_epr(lcrzo_string_appendm_text(f2+1, &f2_));
        lcrzo_er(lcrzo_string_free2(&f2));
        lcrzo_epr(lcrzo_string_initm_text(f2_, &f2));
        lcrzo_er(lcrzo_string_free2(&f2_));
	lcrzo_er(lcrzo_string_free2(&pwd));
      }
    }

    /* set the item */
    if (plistlinkname!=NULL)
    { lcrzo_er(lcrzo_stringlist_add_pos(plistlinkname, -1, 0, f1));
    }
    /* try to obtain the size of the file */
    if (plistlinkobj!=NULL)
    { lcrzo_er(lcrzo_string_initm_text(dir, &tmpfilen));
      lcrzo_er(lcrzo_string_appendm_text("/", &tmpfilen));
      lcrzo_er(lcrzo_string_appendm_text(f2, &tmpfilen));
      ret=shared_ftp_cmd_size(psockcontrol, tmpfilen, &filesize);
      if ( ret==LCRZO_ERR_OK )
      { lcrzo_er(lcrzo_stringlist_add_pos(plistlinkobj, -1, filesize, f2));
      }
      else
      { lcrzo_er(lcrzo_stringlist_add_pos(plistlinkobj, -1, 0, f2));
      }
      lcrzo_er(lcrzo_string_free2(&tmpfilen));
    }
    lcrzo_er(lcrzo_string_free2(&f1));
    lcrzo_er(lcrzo_string_free2(&f2));
  }

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_priv_ftp_cmd_stat(lcrzo_sock *psockcontrol,
			     lcrzo_conststring fileordir,
			     lcrzo_list *plistfile,
			     lcrzo_list *plistdir,
			     lcrzo_list *plistlinkname,
			     lcrzo_list *plistlinkobj,
			     lcrzo_bool *preaddataisassociatedtodir);
int shared_priv_ftp_cmd_stat(lcrzo_sock *psockcontrol,
			     lcrzo_conststring fileordir,
			     lcrzo_list *plistfile,
			     lcrzo_list *plistdir,
			     lcrzo_list *plistlinkname,
			     lcrzo_list *plistlinkobj,
			     lcrzo_bool *preaddataisassociatedtodir)
{ lcrzo_list listtlink;
  lcrzo_data data;
  lcrzo_int32 datasize;
  lcrzo_bool readdataisassociatedtodir;
  lcrzo_string dir;

  lcrzo_epr(lcrzo_data_initm_text("STAT ", &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text(fileordir, datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, &data, &datasize));
  lcrzo_epr(lcrzo_sock_write(psockcontrol, data, datasize));
  lcrzo_er(lcrzo_data_free2(&data));
  lcrzo_epr(lcrzo_sock_readm_line(psockcontrol, LCRZO_WAIT_INFINITE,
				  &data, &datasize));
  if (strncmp((lcrzo_string)data, "213-", 4))
  { lcrzo_er(lcrzo_data_free2(&data));
    return(LCRZOEX_ERR_FTP_BADRESPONSE);
  }
  lcrzo_er(lcrzo_data_free2(&data));
    
  /* analyze data to obtain list of files */
  lcrzo_epr(shared_priv_ftp_decode_dirfiletlink(LCRZO_TRUE, psockcontrol, 
					       NULL, plistfile, plistdir,
					       &listtlink, 
					       &readdataisassociatedtodir));
 
  if (readdataisassociatedtodir)
  { lcrzo_epr(lcrzo_string_initm_text(fileordir, &dir));
  }
  else
  { /* we need the parent dir */
    lcrzo_epr(lcrzo_path_decode_parentdir(fileordir, LCRZO_FALSE, &dir));
  }

  /* now, find types and sizes of links */
  if ( plistlinkname!=NULL || plistlinkobj!=NULL )
  { lcrzo_epr(shared_priv_ftp_decode_tlink(psockcontrol, dir, &listtlink,
					  plistlinkname, plistlinkobj));
  }
  lcrzo_epr(lcrzo_list_close(&listtlink));
  lcrzo_er(lcrzo_string_free2(&dir));

  if (preaddataisassociatedtodir!=NULL)
  { *preaddataisassociatedtodir=readdataisassociatedtodir;
  }
  return(LCRZO_ERR_OK);
}


/*-------------------------------------------------------------*/
/*-------------------------------------------------------------*/
/*-------------------------------------------------------------*/
int shared_ftp_cmd_user(lcrzo_sock *psock,
			lcrzo_conststring user)
{ lcrzo_data data;
  lcrzo_int32 datasize;

  /* we send a 'USER' */
  lcrzo_epr(lcrzo_data_initm_text("USER ", &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text(user, datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, &data, &datasize));
  lcrzo_epr(lcrzo_sock_write(psock, data, datasize));
  lcrzo_er(lcrzo_data_free2(&data));
  lcrzo_epr(shared_misc_verifyresponse1(psock, 331));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_ftp_cmd_pass(lcrzo_sock *psock,
			lcrzo_conststring pass)
{ lcrzo_data data;
  lcrzo_int32 datasize;

  /* we send a 'PASS' */
  lcrzo_epr(lcrzo_data_initm_text("PASS ", &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text(pass, datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, &data, &datasize));
  lcrzo_epr(lcrzo_sock_write(psock, data, datasize));
  lcrzo_er(lcrzo_data_free2(&data));
  lcrzo_epr(shared_misc_verifyresponse1(psock, 230));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_ftp_cmd_quit(lcrzo_sock *psock)
{ lcrzo_data data;
  lcrzo_int32 datasize;

  /* we send a 'QUIT' */
  lcrzo_epr(lcrzo_data_initm_text("QUIT\r\n", &data, &datasize));
  lcrzo_epr(lcrzo_sock_write(psock, data, datasize));
  lcrzo_er(lcrzo_data_free2(&data));
  lcrzo_epr(shared_misc_verifyresponse1(psock, 221));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_ftp_cmd_port(lcrzo_sock *psock,
			lcrzo_ipl clientipl, 
			lcrzo_uint16 clientdataport)
{ lcrzo_data data;
  lcrzo_int32 datasize;
  lcrzo_ipa clientipa; 

  /* we send a 'PORT' */
  lcrzo_epr(lcrzo_ipa_init_ipl(clientipl, clientipa));
  lcrzo_epr(lcrzo_data_initm_text("PORT ", &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_uint8(clientipa[0], "%u", datasize,
				   &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_char(',', 1, datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_uint8(clientipa[1], "%u", datasize,
				   &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_char(',', 1, datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_uint8(clientipa[2], "%u", datasize,
				   &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_char(',', 1, datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_uint8(clientipa[3], "%u", datasize,
				   &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_char(',', 1, datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_uint8((lcrzo_uint8)(clientdataport>>8), "%u",
				     datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_char(',', 1, datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_uint8((lcrzo_uint8)(clientdataport&0xFF), "%u",
				     datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, &data, &datasize));
  lcrzo_epr(lcrzo_sock_write(psock, data, datasize));
  lcrzo_er(lcrzo_data_free2(&data));
  lcrzo_epr(shared_misc_verifyresponse1(psock, 200));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_ftp_cmd_pasv(lcrzo_sock *psock,
			lcrzo_ipl *pserveripl, 
			lcrzo_uint16 *pserverdataport)
{ lcrzo_data data;
  lcrzo_string string;
  lcrzo_int32 datasize;
  lcrzo_regexp re;
  lcrzo_uint8 pasvvalues[6];
  int i, ret;
  lcrzo_int16 language;

  lcrzo_epr(lcrzo_global_get_language(&language));

  /* we send a 'PASV' */
  lcrzo_epr(lcrzo_data_initm_text("PASV\r\n", &data, &datasize));
  lcrzo_epr(lcrzo_sock_write(psock, data, datasize));
  lcrzo_er(lcrzo_data_free2(&data));
  lcrzo_epr(lcrzo_sock_readm(psock, LCRZO_WAIT_INFINITE, &data, &datasize));
  /*lcrzo_epr(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_DUMP));*/

  ret=lcrzo_data_search_regexp(data, datasize, LCRZO_FALSE, +1, -1,
			       "227 .*\\(([0-9]+),([0-9]+),([0-9]+),([0-9]+),([0-9]+),([0-9]+)\\)", re);
  if ( ret!=LCRZO_ERR_OK )
  { if ( language==LCRZO_GLOBAL_LANG_FR )
      printf("Je voulais 227, mais j'ai recu :\n"); 
    else
      printf("I wanted 227, but I received :\n"); 
    lcrzo_epr(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_DUMP));
    lcrzo_er(lcrzo_data_free2(&data));
    return(LCRZOEX_ERR_FTP_BADRESPONSE);
  }

  /* we set each value */
  data[datasize]='\0';
  for ( i=0 ; i<6 ; i++ )
  { lcrzo_epr(lcrzo_string_initm_range((lcrzo_string)data,
				       re[i+1].startindex_positive,
				       re[i+1].endindex_positive, &string));
    lcrzo_epr(lcrzo_uint8_init_text(string, &(pasvvalues[i])));
    lcrzo_er(lcrzo_string_free2(&string));
  }
  lcrzo_er(lcrzo_data_free2(&data));

  /* initialize return value */
  lcrzo_epr(lcrzo_ipl_init_ipa(pasvvalues, pserveripl));
  if (pserverdataport!=NULL)
  { *pserverdataport=(lcrzo_uint16)((pasvvalues[4]<<8)|pasvvalues[5]);
  }

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_ftp_cmd_retr(lcrzo_sock *psock,
			lcrzo_conststring file)
{ lcrzo_data data;
  lcrzo_int32 datasize;

  /* we send a 'RETR' */
  lcrzo_epr(lcrzo_data_initm_text("RETR ", &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text(file, datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, &data, &datasize));
  lcrzo_epr(lcrzo_sock_write(psock, data, datasize));
  lcrzo_er(lcrzo_data_free2(&data));
  lcrzo_epr(shared_misc_verifyresponse2(psock, 125, 150));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_ftp_cmd_stor(lcrzo_sock *psock,
			lcrzo_conststring file)
{ lcrzo_data data;
  lcrzo_int32 datasize;

  /* we send a 'STOR' */
  lcrzo_epr(lcrzo_data_initm_text("STOR ", &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text(file, datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, &data, &datasize));
  lcrzo_epr(lcrzo_sock_write(psock, data, datasize));
  lcrzo_er(lcrzo_data_free2(&data));
  lcrzo_epr(shared_misc_verifyresponse2(psock, 125, 150));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_ftp_cmd_dele(lcrzo_sock *psock,
			lcrzo_conststring file)
{ lcrzo_data data;
  lcrzo_int32 datasize;

  /* we send a 'DELE' */
  lcrzo_epr(lcrzo_data_initm_text("DELE ", &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text(file, datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, &data, &datasize));
  lcrzo_epr(lcrzo_sock_write(psock, data, datasize));
  lcrzo_er(lcrzo_data_free2(&data));
  lcrzo_epr(shared_misc_verifyresponse1(psock, 250));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_ftp_cmd_type(lcrzo_sock *psock,
			lcrzo_conststring type)
{ lcrzo_data data;
  lcrzo_int32 datasize;

  /* we send a 'TYPE' */
  lcrzo_epr(lcrzo_data_initm_text("TYPE ", &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text(type, datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, &data, &datasize));
  lcrzo_epr(lcrzo_sock_write(psock, data, datasize));
  lcrzo_er(lcrzo_data_free2(&data));
  lcrzo_epr(shared_misc_verifyresponse1(psock, 200));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_ftp_cmd_stat(lcrzo_sock *psockcontrol,
			lcrzo_conststring fileordir,
			lcrzo_list *plistfile,
			lcrzo_list *plistdir,
      lcrzo_list *plistlinkname,
			lcrzo_list *plistlinkobj)
{ return(shared_priv_ftp_cmd_stat(psockcontrol, fileordir, plistfile, plistdir,
				  plistlinkname, plistlinkobj, NULL));
}

/*-------------------------------------------------------------*/
int shared_ftp_cmd_mkd(lcrzo_sock *psock,
		       lcrzo_conststring dir)
{ lcrzo_data data;
  lcrzo_int32 datasize;

  /* we send a 'MKD' */
  lcrzo_epr(lcrzo_data_initm_text("MKD ", &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text(dir, datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, &data, &datasize));
  lcrzo_epr(lcrzo_sock_write(psock, data, datasize));
  lcrzo_er(lcrzo_data_free2(&data));
  lcrzo_epr(shared_misc_verifyresponse3(psock, 257, 550, 521));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_ftp_cmd_cwd(lcrzo_sock *psock,
		       lcrzo_conststring dir)
{ lcrzo_data data;
  lcrzo_int32 datasize;

  /* we send a 'CWD dir' */
  lcrzo_epr(lcrzo_data_initm_text("CWD ", &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text(dir, datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, &data, &datasize));
  lcrzo_epr(lcrzo_sock_write(psock, data, datasize));
  lcrzo_er(lcrzo_data_free2(&data));
  lcrzo_epr(shared_misc_verifyresponse1(psock, 250));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_ftp_cmd_pwd(lcrzo_sock *psock,
		       lcrzo_string *pcurrentdir)
{ lcrzo_data data;
  lcrzo_int32 datasize;
  lcrzo_regexp re;
  lcrzo_int16 language;
  int ret;

  lcrzo_epr(lcrzo_global_get_language(&language));

  /* we send a 'PWD' */
  lcrzo_epr(lcrzo_data_initm_text("PWD\r\n", &data, &datasize));
  lcrzo_epr(lcrzo_sock_write(psock, data, datasize));
  lcrzo_er(lcrzo_data_free2(&data));

  lcrzo_epr(lcrzo_sock_readm(psock, LCRZO_WAIT_INFINITE, &data, &datasize));
  ret=lcrzo_data_search_regexp(data, datasize, LCRZO_FALSE, +1, -1,
			       "257 \"(.*)\"", re);
  if ( ret!=LCRZO_ERR_OK )
  { if ( language==LCRZO_GLOBAL_LANG_FR )
      printf("Je voulais 257, mais j'ai recu :\n"); 
    else
      printf("I wanted 257, but I received :\n"); 
    lcrzo_epr(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_DUMP));
    lcrzo_er(lcrzo_data_free2(&data));
    return(LCRZOEX_ERR_FTP_BADRESPONSE);
  }

  data[datasize]='\0';
  lcrzo_epr(lcrzo_string_initm_range((lcrzo_string)data, 
				     re[1].startindex_positive,
				     re[1].endindex_positive, pcurrentdir));
  lcrzo_er(lcrzo_data_free2(&data));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_ftp_cmd_list(lcrzo_sock *psock,
			lcrzo_conststring dir)
{ lcrzo_data data;
  lcrzo_int32 datasize;

  /* we send a 'LIST dir' */
  lcrzo_epr(lcrzo_data_initm_text("LIST ", &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text(dir, datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, &data, &datasize));
  lcrzo_epr(lcrzo_sock_write(psock, data, datasize));
  lcrzo_er(lcrzo_data_free2(&data));
  lcrzo_epr(shared_misc_verifyresponse2(psock, 125, 150));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
/* the SIZE command is unfortunately an extension, so we also try with STAT */
int shared_priv_ftp_cmd_size(lcrzo_sock *psock,
			     lcrzo_conststring file,
			     lcrzo_int32 *pfilesize);
int shared_priv_ftp_cmd_size(lcrzo_sock *psock,
			     lcrzo_conststring file,
			     lcrzo_int32 *pfilesize)
{ lcrzo_data data;
  lcrzo_int32 datasize;
  lcrzo_string sizeserver;
  lcrzo_regexp re;
  lcrzo_int16 language;
  int ret;

  lcrzo_epr(lcrzo_global_get_language(&language));

  /* we send a 'SIZE file' */
  lcrzo_epr(lcrzo_data_initm_text("SIZE ", &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text(file, datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, &data, &datasize));
  lcrzo_epr(lcrzo_sock_write(psock, data, datasize));
  lcrzo_er(lcrzo_data_free2(&data));

  lcrzo_epr(lcrzo_sock_readm(psock, LCRZO_WAIT_INFINITE, &data, &datasize));
  ret=lcrzo_data_search_regexp(data, datasize, LCRZO_FALSE, +1, -1,
			       "213 ([0-9]*)", re);
  if ( ret!=LCRZO_ERR_OK )
  { /*if ( language==LCRZO_GLOBAL_LANG_FR )
      printf("Je voulais 213, mais j'ai recu :\n"); 
    else
      printf("I wanted 213, but I received :\n"); 
      lcrzo_epr(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_DUMP));*/
    lcrzo_er(lcrzo_data_free2(&data));
    return(LCRZOEX_ERR_FTP_BADRESPONSE);
  }

  data[datasize]='\0';
  lcrzo_epr(lcrzo_string_initm_range((lcrzo_string)data, 
				     re[1].startindex_positive,
				     re[1].endindex_positive, &sizeserver));
  lcrzo_er(lcrzo_data_free2(&data));
  lcrzo_epr(lcrzo_int32_init_text(sizeserver, pfilesize));
  lcrzo_er(lcrzo_string_free2(&sizeserver));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_ftp_cmd_size(lcrzo_sock *psock,
			lcrzo_conststring file,
			lcrzo_int32 *pfilesize)
{ lcrzo_bool isfile;
  int ret;

  ret=shared_priv_ftp_cmd_size(psock, file, pfilesize);
  if (ret!=LCRZO_ERR_OK)
  { /* try with STAT */
    lcrzo_er(shared_ftp_file_typesize(psock, file,
				      &isfile, NULL, NULL,
				      pfilesize));
    if ( ! isfile ) return(LCRZOEX_ERR_FTP_BADRESPONSE);
  }
  return(LCRZO_ERR_OK);
}


/*-------------------------------------------------------------*/
/*-------------------------------------------------------------*/
int shared_ftp_cmd_readbanner(lcrzo_sock *psock)
{ return(shared_misc_verifyresponse1(psock, 220));
}

/*-------------------------------------------------------------*/
int shared_ftp_cmd_datasock_waitend(lcrzo_sock *psock)
{ 
  /* wait the transfert complete */
  lcrzo_epr(shared_misc_verifyresponse1(psock, 226));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_ftp_cmd_type_ascii(lcrzo_sock *psock)
{ return(shared_ftp_cmd_type(psock, "A"));
}

/*-------------------------------------------------------------*/
int shared_ftp_cmd_type_bin(lcrzo_sock *psock)
{ return(shared_ftp_cmd_type(psock, "I"));
}


/*-------------------------------------------------------------*/
/*-------------------------------------------------------------*/
int shared_ftp_login(lcrzo_sock *psock,
		     lcrzo_conststring user,
		     lcrzo_conststring pass)
{ 
  lcrzo_epr(shared_ftp_cmd_readbanner(psock));
  lcrzo_epr(shared_ftp_cmd_user(psock, user));
  lcrzo_epr(shared_ftp_cmd_pass(psock, pass));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_ftp_login_anonymous(lcrzo_sock *psock)
{ 
  lcrzo_epr(shared_ftp_login(psock, "anonymous",
			     "tool_from_lcrzoex@laurentconstantin.com"));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_ftp_data_list(lcrzo_sock *psockcontrol,
			 lcrzo_bool usesockdata,
			 lcrzo_sock *psockdata,
			 lcrzo_ipc *pipcdata,
			 lcrzo_conststring fileordir,
			 lcrzo_list *plistfile,
			 lcrzo_list *plistdir,
			 lcrzo_list *plistlinkname,
 			 lcrzo_list *plistlinkobj)
{ lcrzo_list listtlink;
  lcrzo_bool readdataisassociatedtodir;
  lcrzo_string dir;

  lcrzo_epr(shared_ftp_cmd_list(psockcontrol, fileordir));

  /* analyze data to obtain list of files */
  lcrzo_epr(shared_priv_ftp_decode_dirfiletlink(usesockdata, psockdata, 
					       pipcdata, plistfile, plistdir,
					       &listtlink,
					       &readdataisassociatedtodir));
  if (usesockdata)
  { lcrzo_epr(lcrzo_sock_close(psockdata));
  }
  else
  { lcrzo_epr(lcrzo_ipc_close(pipcdata));
  }
  lcrzo_epr(shared_ftp_cmd_datasock_waitend(psockcontrol));
  
  if (readdataisassociatedtodir)
  { lcrzo_epr(lcrzo_string_initm_text(fileordir, &dir));
  }
  else
  { /* we need the parent dir */
    lcrzo_epr(lcrzo_path_decode_parentdir(fileordir, LCRZO_FALSE, &dir));
  }

  /* now, find types and sizes of links */
  if ( plistlinkname!=NULL || plistlinkobj!=NULL )
  { lcrzo_epr(shared_priv_ftp_decode_tlink(psockcontrol, dir, &listtlink,
					  plistlinkname, plistlinkobj));
  }
  lcrzo_epr(lcrzo_list_close(&listtlink));
  lcrzo_er(lcrzo_string_free2(&dir));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_ftp_data_retr(lcrzo_sock *psockcontrol,
			 lcrzo_bool usesockdata,
			 lcrzo_sock *psockdata,
			 lcrzo_ipc *pipcdata,
			 lcrzo_conststring distantfilename,
			 lcrzo_conststring localfilename)
{ lcrzo_data data;
  lcrzo_int32 datasize;
  lcrzo_file file;
  int ret;

  lcrzo_epr(shared_ftp_cmd_retr(psockcontrol, distantfilename));
  lcrzo_epr(lcrzo_file_open_write(localfilename, LCRZO_FALSE, LCRZO_FALSE,
				  &file));

  /* receive data */
  while(LCRZO_TRUE)
  { if (usesockdata)
    { ret=lcrzo_sock_readm(psockdata, LCRZO_WAIT_INFINITE,
			   &data, &datasize);
    }
    else
    { ret=lcrzo_ipc_readm(pipcdata, LCRZO_WAIT_INFINITE,
			  &data, &datasize);
    }
    if ( ret!=LCRZO_ERR_OK ) break;
    ret=lcrzo_file_write(&file, data, datasize);
    lcrzo_er(lcrzo_data_free2(&data));
    if ( ret!=LCRZO_ERR_OK ) break;
  }
  if (usesockdata)
  { lcrzo_epr(lcrzo_sock_close(psockdata));
  }
  else
  { lcrzo_epr(lcrzo_ipc_close(pipcdata));
  }
  lcrzo_epr(lcrzo_file_close(&file));
  lcrzo_epr(shared_ftp_cmd_datasock_waitend(psockcontrol));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_ftp_data_stor(lcrzo_sock *psockcontrol,
			 lcrzo_bool usesockdata,
			 lcrzo_sock *psockdata,
			 lcrzo_ipc *pipcdata,
			 lcrzo_conststring localfilename,
			 lcrzo_conststring distantfilename)
{ lcrzo_data data;
  lcrzo_int32 datasize;
  lcrzo_file file;
  int ret;

  lcrzo_epr(shared_ftp_cmd_stor(psockcontrol, distantfilename));
  lcrzo_epr(lcrzo_file_open_read(localfilename, &file));

  /* receive data */
  while(LCRZO_TRUE)
  { ret=lcrzo_file_readm(&file, LCRZO_WAIT_INFINITE, &data, &datasize);
    if ( ret!=LCRZO_ERR_OK ) break;
    if (usesockdata)
    { ret=lcrzo_sock_write(psockdata, data, datasize);
    }
    else
    { ret=lcrzo_ipc_write(pipcdata, data, datasize);
    }
    lcrzo_er(lcrzo_data_free2(&data));
    if ( ret!=LCRZO_ERR_OK ) break;
  }
  if (usesockdata)
  { lcrzo_epr(lcrzo_sock_close(psockdata));
  }
  else
  { lcrzo_epr(lcrzo_ipc_close(pipcdata));
  }
  lcrzo_epr(shared_ftp_cmd_datasock_waitend(psockcontrol));
  lcrzo_epr(lcrzo_file_close(&file));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_ftp_file_typesize(lcrzo_sock *psockcontrol,
			     lcrzo_conststring file,
			     lcrzo_bool *pisfile,
			     lcrzo_bool *pisdir,
			     lcrzo_bool *pislink,
			     lcrzo_int32 *pfilesize)
{ lcrzo_list listfile, listdir, listlinkname;
  lcrzo_int32 countlistfile, countlistdir, countlistlinkname;
  lcrzo_int32 total;
  lcrzo_uint32 linktype;
  lcrzo_string str;
  lcrzo_bool associatedtodir;

  /* obtain file infos */
  lcrzo_epr(shared_priv_ftp_cmd_stat(psockcontrol, file, &listfile, &listdir,
				     &listlinkname, NULL, &associatedtodir));

  /* count items */
  lcrzo_epr(lcrzo_list_count(&listfile, &countlistfile));
  lcrzo_epr(lcrzo_list_count(&listdir, &countlistdir));
  lcrzo_epr(lcrzo_list_count(&listlinkname, &countlistlinkname));
  total=countlistfile+countlistdir+countlistlinkname;

  if (pfilesize!=NULL) *pfilesize=0;
  if (pisfile!=NULL) *pisfile=LCRZO_FALSE;
  if (pisdir!=NULL) *pisdir=LCRZO_FALSE;
  if (pislink!=NULL) *pislink=LCRZO_FALSE;

  /* file not found */
  if ( total==0 )
  { lcrzo_epr(lcrzo_list_close(&listfile));
    lcrzo_epr(lcrzo_list_close(&listdir));
    lcrzo_epr(lcrzo_list_close(&listlinkname));
    if (associatedtodir)
    { /* case of empty directory */
      if (pisdir!=NULL) *pisdir=LCRZO_TRUE;
      return(LCRZO_ERR_OK);
    }
    return(LCRZO_ERR_OKFILENOTFOUND);
  }

  /* file is a directory */
  if ( total>1 )
  { lcrzo_epr(lcrzo_list_close(&listfile));
    lcrzo_epr(lcrzo_list_close(&listdir));
    lcrzo_epr(lcrzo_list_close(&listlinkname));
    if (pisdir!=NULL) *pisdir=LCRZO_TRUE;
    return(LCRZO_ERR_OK);
  }
  /* now, total==1 */

  /* analyze to determine exactly */
  if (countlistfile)
  { /* can be a file or a directory containing one file */
    lcrzo_epr(lcrzo_stringlist_valuem_pos(&listfile, 1, 
					  (lcrzo_uint32*)pfilesize, &str));
    if (!strcmp(str, file))
    { if (pisfile!=NULL) *pisfile=LCRZO_TRUE;
    }
    else
    { if (pisdir!=NULL) *pisdir=LCRZO_TRUE;
    }
    lcrzo_er(lcrzo_string_free2(&str));
    lcrzo_epr(lcrzo_list_close(&listfile));
    lcrzo_epr(lcrzo_list_close(&listdir));
    lcrzo_epr(lcrzo_list_close(&listlinkname));
    return(LCRZO_ERR_OK);
  }
  lcrzo_epr(lcrzo_list_close(&listfile));

  if (countlistdir)
  { /* must be a directory */
    if (pisdir!=NULL) *pisdir=LCRZO_TRUE;
    lcrzo_epr(lcrzo_list_close(&listdir));
    lcrzo_epr(lcrzo_list_close(&listlinkname));
    return(LCRZO_ERR_OK);
  }
  lcrzo_epr(lcrzo_list_close(&listdir));

  if (countlistlinkname)
  { /* can be a link to a file or a directory containing one link */
    lcrzo_epr(lcrzo_stringlist_valuem_pos(&listlinkname, 1, 
					  &linktype, &str));
    if (!strcmp(str, file))
    { if (pislink!=NULL) *pislink=LCRZO_TRUE;
    }
    else
    { if (pisdir!=NULL) *pisdir=LCRZO_TRUE;
    }
    lcrzo_er(lcrzo_string_free2(&str));
    lcrzo_epr(lcrzo_list_close(&listlinkname));
    return(LCRZO_ERR_OK);
  }
  lcrzo_epr(lcrzo_list_close(&listlinkname));

  /* should not be reached */
  return(LCRZO_ERR_IEINTERNALERROR);
}
