/*
		                 LCRZOEX
                             Network toolbox
             Copyright(c) 1999,2000,2001,2002 Laurent Constantin
                                  -----

  Main server    : http://www.laurentconstantin.com/
  Backup servers : http://go.to/laurentconstantin/
                   http://laurentconstantin.est-la.com/
                   http://laurentconstantin.free.fr/
                   http://membres.lycos.fr/lauconstantin/
  [my current email address is on the web servers]

                                  -----
  This file is part of Lcrzoex.

  Lcrzoex is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Lcrzoex is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details (http://www.gnu.org/).

------------------------------------------------------------------------
*/

#include <lcrzo.h>
#include "../lcrzoex.h"

/*-------------------------------------------------------------*/
int shared_priv_lcrzoex_infos_fprint(LCRZOFILE *pf,
				     shared_lcrzoex_infos infos,
				     lcrzo_conststring progname,
				     lcrzo_bool checkadmin);
int shared_priv_lcrzoex_infos_fprint(LCRZOFILE *pf,
				     shared_lcrzoex_infos infos,
				     lcrzo_conststring progname,
				     lcrzo_bool checkadmin)
{ lcrzo_int16 language;

  lcrzo_er(lcrzo_global_get_language(&language));

  if (checkadmin)
  { /* check if user has admin privileges */
    lcrzo_epr(shared_misc_checkadmin(language));
  }

  fprintf((FILE*)pf, "Reference    : %s_", infos.refname);
  lcrzo_epr(lcrzo_uint32_fprint(pf, "", infos.refvalue, "%06u", ", "));
  if (language==LCRZO_GLOBAL_LANG_FR)
  { fprintf((FILE*)pf, "version (de cet outil)");
  }
  else
  { fprintf((FILE*)pf, "version (of this tool)");
  }
  fprintf((FILE*)pf, " %d.%02d\n", infos.majorversion, infos.minorversion);
  if (language==LCRZO_GLOBAL_LANG_FR)
  { fprintf((FILE*)pf, "Titre        : %s\n", infos.title_fr);
    if ( strlen(infos.description_fr) )
      fprintf((FILE*)pf, "Description  : %s\n", infos.description_fr);
    if ( strlen(infos.author) )
      fprintf((FILE*)pf, "Auteur       : %s\n", infos.author);
    fprintf((FILE*)pf, "Usage        : %s %s\n", progname, infos.usage_fr);
    fprintf((FILE*)pf, "Exemple      : %s %s\n", progname, infos.usageex_fr);
  }
  else
  { fprintf((FILE*)pf, "Title        : %s\n", infos.title_en);
    if ( strlen(infos.description_en) )
      fprintf((FILE*)pf, "Description  : %s\n", infos.description_en);
    if ( strlen(infos.author) )
      fprintf((FILE*)pf, "Author       : %s\n", infos.author);
    fprintf((FILE*)pf, "Usage        : %s %s\n", progname, infos.usage_en);
    fprintf((FILE*)pf, "Example      : %s %s\n", progname, infos.usageex_en);
  }

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_lcrzoex_infos_fprint(LCRZOFILE *pf,
				shared_lcrzoex_infos infos,
				lcrzo_conststring progname)
{ return(shared_priv_lcrzoex_infos_fprint(pf, infos, progname, LCRZO_TRUE));
}

/*-------------------------------------------------------------*/
int shared_lcrzoex_infos_noadmcheck_fprint(LCRZOFILE *pf,
					   shared_lcrzoex_infos infos,
					   lcrzo_conststring progname)
{ return(shared_priv_lcrzoex_infos_fprint(pf, infos, progname, LCRZO_FALSE));
}

/*-------------------------------------------------------------*/
#if LCRZO_DEBUG_PROCESSOTHEREXIT_WARN==1
int lcrzo_debug_processotherexit_wanttowarn(lcrzo_bool yes);
int shared_lcrzoex_debug_processotherexit_wanttowarn(lcrzo_bool yes)
{ return(lcrzo_debug_processotherexit_wanttowarn(yes));
}
#endif

/*-------------------------------------------------------------*/
int shared_lcrzoex_err_print(int ret)
{ lcrzo_int16 language;

  lcrzo_er(lcrzo_global_get_language(&language));
  switch(ret) {
  case LCRZOEX_ERR_MISC_BADTOOLUSAGE :
    if (language==LCRZO_GLOBAL_LANG_FR)
      printf("mauvaise utilisation de l'outil\n");
    else
      printf("bad tool usage\n");
    break;
  case LCRZOEX_ERR_MISC_BADRESPONSE :
    if (language==LCRZO_GLOBAL_LANG_FR)
      printf("mauvaise reponse\n");
    else
      printf("bad response\n");
    break;
  case LCRZOEX_ERR_MISC_HASHBADSIZE :
    if (language==LCRZO_GLOBAL_LANG_FR)
      printf("mauvaise taille du hache\n");
    else
      printf("bad hash size\n");
    break;
  case LCRZOEX_ERR_MISC_BADTOOLNUMBER :
    if (language==LCRZO_GLOBAL_LANG_FR)
      printf("mauvais numero d'outil\n");
    else
      printf("bad tool number\n");
    break;
  case LCRZOEX_ERR_MISC_ERRTOOBIG :
    if (language==LCRZO_GLOBAL_LANG_FR)
      printf("code d'erreur tronque sous Unix\n");
    else
      printf("error code truncated under Unix\n");
    break;
  case LCRZOEX_ERR_HTTP_MISSCHARAFTERPERCENT :
    if (language==LCRZO_GLOBAL_LANG_FR)
      printf("HTTP : un caractere manque apres le %%\n");
    else
      printf("HTTP : a character is missing after %%\n");
    break;
  case LCRZOEX_ERR_HTTP_BADCHARAFTERPERCENT :
    if (language==LCRZO_GLOBAL_LANG_FR)
      printf("HTTP : un caractere apres %% est invalide\n");
    else
      printf("HTTP : a character after %% is invalid\n");
    break;
  case LCRZOEX_ERR_HTTP_INVALIDREQUEST :
    if (language==LCRZO_GLOBAL_LANG_FR)
      printf("HTTP : requete invalide\n");
    else
      printf("HTTP : invalid request\n");
    break;
  case LCRZOEX_ERR_HTTP_LOGNULLPWDNOTNULL :
    if (language==LCRZO_GLOBAL_LANG_FR)
      printf("HTTP : login==NULL et passwd!=NULL\n");
    else
      printf("HTTP : login==NULL and passwd!=NULL\n");
    break;
  case LCRZOEX_ERR_TELNET_BADLOGINPASS :
    if (language==LCRZO_GLOBAL_LANG_FR)
      printf("telnet : mauvais login/password\n");
    else
      printf("telnet : bad login/password\n");
    break;
  case LCRZOEX_ERR_FU_EXECL :
    if (language==LCRZO_GLOBAL_LANG_FR)
      printf("erreur lors de l'appel execl()\n");
    else
      printf("error in execl()\n");
    break;
  case LCRZOEX_ERR_FU_DUP2 :
    if (language==LCRZO_GLOBAL_LANG_FR)
      printf("erreur lors de l'appel dup2()\n");
    else
      printf("error in dup2()\n");
    break;
  case LCRZOEX_ERR_FU_CREATEPROCESS :
    if (language==LCRZO_GLOBAL_LANG_FR)
      printf("erreur lors de l'appel CreateProcess()\n");
    else
      printf("error in CreateProcess()\n");
    break;
  case LCRZOEX_ERR_FU_SETSTDHANDLE :
    if (language==LCRZO_GLOBAL_LANG_FR)
      printf("erreur lors de l'appel SetStdHandle()\n");
    else
      printf("error in SetStdHandle()\n");
    break;
  case LCRZOEX_ERR_FU_SYMLINK :
    if (language==LCRZO_GLOBAL_LANG_FR)
      printf("erreur lors de l'appel symlink()\n");
    else
      printf("error in symlink()\n");
    break;
  case LCRZOEX_ERR_FTP_BADRESPONSE :
    if (language==LCRZO_GLOBAL_LANG_FR)
      printf("FTP : mauvaise reponse\n");
    else
      printf("FTP : bas answer\n");
    break;
  case LCRZOEX_ERR_PARA_UNKNOWNTYPE :
    if (language==LCRZO_GLOBAL_LANG_FR)
      printf("Seuls les types d,dn,e,er,el,h,i,ir,il,p,pr,pl,n,s,sp sont reconnus\n");
    else
      printf("Only types d,dn,e,er,el,h,i,ir,il,p,pr,pl,n,s,sp are recognized\n");
    break;
  case LCRZOEX_ERR_PARA_BADOUTPUTTYPE :
    if (language==LCRZO_GLOBAL_LANG_FR)
      printf("Le type de sortie ne correspond pas au type voulu\n");
    else
      printf("Output type do not correspond to wanted type\n");
    break;
  case LCRZOEX_ERR_PARA_CANTCONVERT :
    if (language==LCRZO_GLOBAL_LANG_FR)
      printf("Le type entre ne peut pas etre convertit en type de sortie\n");
    else
      printf("Input type cannot be converted to output type\n");
    break;
  case LCRZOEX_ERR_DNS_BADRRTYPE :
    if (language==LCRZO_GLOBAL_LANG_FR)
      printf("DNS : Le type de RR n'est pas reconnu (A, PTR, etc.)\n");
    else
      printf("DNS : RR type isn't known (A, NS, PTR, etc.)\n");
    break;
  default :
    return(LCRZO_ERR_LOUNKNOWNERROR);
  }
  return(LCRZO_ERR_OK);
}


