/*
		                 LCRZOEX
                             Network toolbox
             Copyright(c) 1999,2000,2001,2002 Laurent Constantin
                                  -----

  Main server    : http://www.laurentconstantin.com/
  Backup servers : http://go.to/laurentconstantin/
                   http://laurentconstantin.est-la.com/
                   http://laurentconstantin.free.fr/
                   http://membres.lycos.fr/lauconstantin/
  [my current email address is on the web servers]

                                  -----
  This file is part of Lcrzoex.

  Lcrzoex is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Lcrzoex is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details (http://www.gnu.org/).

------------------------------------------------------------------------
*/

#include <lcrzo.h>
#include "../lcrzoex.h"

/*-------------------------------------------------------------*/
/*-------------------------------------------------------------*/
/* This code is adapted from a code of Colin Plumb written in 1993.
   No copyright was claimed.
 */

/*-------------------------------------------------------------*/
#if LCRZODEF_ENDIANLITTLE==1
#define byteReverse(buf, len)
#else
void byteReverse(lcrzo_data buf, lcrzo_uint32 nblongs);
void byteReverse(lcrzo_data buf, lcrzo_uint32 nblongs)
{ lcrzo_uint8 tmp;

  while (nblongs)
  { tmp=buf[0];
    buf[0]=buf[3];
    buf[3]=tmp;
    tmp=buf[1];
    buf[1]=buf[2];
    buf[2]=tmp;
    buf += 4;
    nblongs--;   
  }
}
#endif

/*-------------------------------------------------------------*/
/* The four core functions - F1 is optimized somewhat */
/* #define F1(x, y, z) (x & y | ~x & z) */
#define F1(x, y, z) (z ^ (x & (y ^ z)))
#define F2(x, y, z) F1(z, x, y)
#define F3(x, y, z) (x ^ y ^ z)
#define F4(x, y, z) (y ^ (x | ~z))

/*-------------------------------------------------------------*/
/* This is the central step in the MD5 algorithm. */
#define MD5STEP(f, w, x, y, z, data, s) \
	( w += f(x, y, z) + data,  w = w<<s | w>>(32-s),  w += x )

/*-------------------------------------------------------------*/
int shared_priv_md5_transform(lcrzo_uint32 buf[4], lcrzo_uint32 const in[16]);
int shared_priv_md5_transform2(lcrzo_uint32 buf[4], lcrzo_uint8 const in[64]);
int shared_priv_md5_transform2(lcrzo_uint32 buf[4], lcrzo_uint8 const in[64])
{ lcrzo_uint32 in2[16];
  int i;

  for ( i=0 ; i<16 ; i++ )
  { in2[i] =
#if LCRZODEF_ENDIANLITTLE==1
      in[4*i] | (in[4*i+1]<<8) | (in[4*i+2]<<16) | (in[4*i+3]<<24) ;
#else
      (in[4*i]<<24) | (in[4*i+1]<<16) | (in[4*i+2]<<8) | in[4*i+3] ;
#endif
  }

  lcrzo_er(shared_priv_md5_transform(buf, in2));
  return(LCRZO_ERR_OK);
}
int shared_priv_md5_transform(lcrzo_uint32 buf[4], lcrzo_uint32 const in[16])
{ register lcrzo_uint32 a, b, c, d;

  a = buf[0];
  b = buf[1];
  c = buf[2];
  d = buf[3];
  
  MD5STEP(F1, a, b, c, d, in[0] + 0xd76aa478u, 7);
  MD5STEP(F1, d, a, b, c, in[1] + 0xe8c7b756u, 12);
  MD5STEP(F1, c, d, a, b, in[2] + 0x242070db, 17);
  MD5STEP(F1, b, c, d, a, in[3] + 0xc1bdceeeu, 22);
  MD5STEP(F1, a, b, c, d, in[4] + 0xf57c0fafu, 7);
  MD5STEP(F1, d, a, b, c, in[5] + 0x4787c62a, 12);
  MD5STEP(F1, c, d, a, b, in[6] + 0xa8304613u, 17);
  MD5STEP(F1, b, c, d, a, in[7] + 0xfd469501u, 22);
  MD5STEP(F1, a, b, c, d, in[8] + 0x698098d8, 7);
  MD5STEP(F1, d, a, b, c, in[9] + 0x8b44f7afu, 12);
  MD5STEP(F1, c, d, a, b, in[10] + 0xffff5bb1u, 17);
  MD5STEP(F1, b, c, d, a, in[11] + 0x895cd7beu, 22);
  MD5STEP(F1, a, b, c, d, in[12] + 0x6b901122, 7);
  MD5STEP(F1, d, a, b, c, in[13] + 0xfd987193u, 12);
  MD5STEP(F1, c, d, a, b, in[14] + 0xa679438eu, 17);
  MD5STEP(F1, b, c, d, a, in[15] + 0x49b40821, 22);
  
  MD5STEP(F2, a, b, c, d, in[1] + 0xf61e2562u, 5);
  MD5STEP(F2, d, a, b, c, in[6] + 0xc040b340u, 9);
  MD5STEP(F2, c, d, a, b, in[11] + 0x265e5a51, 14);
  MD5STEP(F2, b, c, d, a, in[0] + 0xe9b6c7aau, 20);
  MD5STEP(F2, a, b, c, d, in[5] + 0xd62f105du, 5);
  MD5STEP(F2, d, a, b, c, in[10] + 0x02441453, 9);
  MD5STEP(F2, c, d, a, b, in[15] + 0xd8a1e681u, 14);
  MD5STEP(F2, b, c, d, a, in[4] + 0xe7d3fbc8u, 20);
  MD5STEP(F2, a, b, c, d, in[9] + 0x21e1cde6, 5);
  MD5STEP(F2, d, a, b, c, in[14] + 0xc33707d6u, 9);
  MD5STEP(F2, c, d, a, b, in[3] + 0xf4d50d87u, 14);
  MD5STEP(F2, b, c, d, a, in[8] + 0x455a14ed, 20);
  MD5STEP(F2, a, b, c, d, in[13] + 0xa9e3e905u, 5);
  MD5STEP(F2, d, a, b, c, in[2] + 0xfcefa3f8u, 9);
  MD5STEP(F2, c, d, a, b, in[7] + 0x676f02d9, 14);
  MD5STEP(F2, b, c, d, a, in[12] + 0x8d2a4c8au, 20);
  
  MD5STEP(F3, a, b, c, d, in[5] + 0xfffa3942u, 4);
  MD5STEP(F3, d, a, b, c, in[8] + 0x8771f681u, 11);
  MD5STEP(F3, c, d, a, b, in[11] + 0x6d9d6122, 16);
  MD5STEP(F3, b, c, d, a, in[14] + 0xfde5380cu, 23);
  MD5STEP(F3, a, b, c, d, in[1] + 0xa4beea44u, 4);
  MD5STEP(F3, d, a, b, c, in[4] + 0x4bdecfa9, 11);
  MD5STEP(F3, c, d, a, b, in[7] + 0xf6bb4b60u, 16);
  MD5STEP(F3, b, c, d, a, in[10] + 0xbebfbc70u, 23);
  MD5STEP(F3, a, b, c, d, in[13] + 0x289b7ec6, 4);
  MD5STEP(F3, d, a, b, c, in[0] + 0xeaa127fau, 11);
  MD5STEP(F3, c, d, a, b, in[3] + 0xd4ef3085u, 16);
  MD5STEP(F3, b, c, d, a, in[6] + 0x04881d05, 23);
  MD5STEP(F3, a, b, c, d, in[9] + 0xd9d4d039u, 4);
  MD5STEP(F3, d, a, b, c, in[12] + 0xe6db99e5u, 11);
  MD5STEP(F3, c, d, a, b, in[15] + 0x1fa27cf8, 16);
  MD5STEP(F3, b, c, d, a, in[2] + 0xc4ac5665u, 23);
  
  MD5STEP(F4, a, b, c, d, in[0] + 0xf4292244u, 6);
  MD5STEP(F4, d, a, b, c, in[7] + 0x432aff97, 10);
  MD5STEP(F4, c, d, a, b, in[14] + 0xab9423a7u, 15);
  MD5STEP(F4, b, c, d, a, in[5] + 0xfc93a039u, 21);
  MD5STEP(F4, a, b, c, d, in[12] + 0x655b59c3, 6);
  MD5STEP(F4, d, a, b, c, in[3] + 0x8f0ccc92u, 10);
  MD5STEP(F4, c, d, a, b, in[10] + 0xffeff47du, 15);
  MD5STEP(F4, b, c, d, a, in[1] + 0x85845dd1u, 21);
  MD5STEP(F4, a, b, c, d, in[8] + 0x6fa87e4f, 6);
  MD5STEP(F4, d, a, b, c, in[15] + 0xfe2ce6e0u, 10);
  MD5STEP(F4, c, d, a, b, in[6] + 0xa3014314u, 15);
  MD5STEP(F4, b, c, d, a, in[13] + 0x4e0811a1, 21);
  MD5STEP(F4, a, b, c, d, in[4] + 0xf7537e82u, 6);
  MD5STEP(F4, d, a, b, c, in[11] + 0xbd3af235u, 10);
  MD5STEP(F4, c, d, a, b, in[2] + 0x2ad7d2bb, 15);
  MD5STEP(F4, b, c, d, a, in[9] + 0xeb86d391u, 21);
  
  buf[0] += a;
  buf[1] += b;
  buf[2] += c;
  buf[3] += d;

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
/*-------------------------------------------------------------*/
int shared_md5_init(shared_md5_context *pcontext)
{
  pcontext->buf[0] = 0x67452301;
  pcontext->buf[1] = 0xefcdab89u;
  pcontext->buf[2] = 0x98badcfeu;
  pcontext->buf[3] = 0x10325476;
  
  pcontext->bits[0] = 0;
  pcontext->bits[1] = 0;
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_md5_update(shared_md5_context *pcontext,
		      lcrzo_constdata data,
		      lcrzo_int32 datasize)
{ lcrzo_uint32 t;

  /* Update bitcount */
  t = pcontext->bits[0];
  pcontext->bits[0] = t + ((lcrzo_uint32) datasize << 3);
  if ( pcontext->bits[0] < t ) pcontext->bits[1]++; /* Carry from low to high*/
  pcontext->bits[1] += datasize >> 29;

  t = (t >> 3) & 0x3f;	/* Bytes already in shsInfo->data */

  /* Handle any leading odd-sized chunks */
  if (t) {
    unsigned char *p = (unsigned char *) pcontext->in + t;
    
    t = 64 - t;
    if ((lcrzo_uint32)datasize < t) {
      memcpy(p, data, datasize);
      return(LCRZO_ERR_OK);
    }
    memcpy(p, data, t);
    byteReverse(pcontext->in, 16);
    shared_priv_md5_transform2(pcontext->buf, pcontext->in);
    data += t;
    datasize -= t;
  }

  /* Process data in 64-byte chunks */
  while (datasize >= 64) {
    memcpy(pcontext->in, data, 64);
    byteReverse(pcontext->in, 16);
    shared_priv_md5_transform2(pcontext->buf, pcontext->in);
    data += 64;
    datasize -= 64;
  }
  
  /* Handle any remaining bytes of data. */
  memcpy(pcontext->in, data, datasize);
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_md5_update_text(shared_md5_context *pcontext,
			   lcrzo_conststring str)
{ return(shared_md5_update(pcontext, (lcrzo_constdata)str, strlen(str)));
}

/*-------------------------------------------------------------*/
int shared_md5_final(shared_md5_context *pcontext,
		     lcrzo_string *phexamd5)
{ unsigned char digest[16];
  unsigned count;
  unsigned char *p;
  lcrzo_uint32 tmp;
  
  /* Compute number of bytes mod 64 */
  count = (pcontext->bits[0] >> 3) & 0x3F;
  
  /* Set the first char of padding to 0x80.  This is safe since there is
     always at least one byte free */
  p = pcontext->in + count;
  *p++ = 0x80;
  
  /* Bytes of padding needed to make 64 bytes */
  count = 64 - 1 - count;
  
  /* Pad out to 56 mod 64 */
  if (count < 8) {
    /* Two lots of padding:  Pad the first block to 64 bytes */
    memset(p, 0, count);
    byteReverse(pcontext->in, 16);
    shared_priv_md5_transform2(pcontext->buf, pcontext->in);
    
    /* Now fill the next block with 56 bytes */
    memset(pcontext->in, 0, 56);
  } else {
    /* Pad block to 56 bytes */
    memset(p, 0, count - 8);
  }
  byteReverse(pcontext->in, 14);
  
  /* Append length in bits and transform */
  /*  ((lcrzo_uint32 *) pcontext->in)[14] = pcontext->bits[0];
      ((lcrzo_uint32 *) pcontext->in)[15] = pcontext->bits[1];*/
  tmp=pcontext->bits[0];
#if LCRZODEF_ENDIANLITTLE==1
  pcontext->in[56]=(lcrzo_uint8)(tmp&0xFF);
  pcontext->in[57]=(lcrzo_uint8)((tmp>>8)&0xFF);
  pcontext->in[58]=(lcrzo_uint8)((tmp>>16)&0xFF);
  pcontext->in[59]=(lcrzo_uint8)((tmp>>24)&0xFF);
#else
  pcontext->in[59]=(lcrzo_uint8)(tmp&0xFF);
  pcontext->in[58]=(lcrzo_uint8)((tmp>>8)&0xFF);
  pcontext->in[57]=(lcrzo_uint8)((tmp>>16)&0xFF);
  pcontext->in[56]=(lcrzo_uint8)((tmp>>24)&0xFF);
#endif
  tmp=pcontext->bits[1];
#if LCRZODEF_ENDIANLITTLE==1
  pcontext->in[60]=(lcrzo_uint8)(tmp&0xFF);
  pcontext->in[61]=(lcrzo_uint8)((tmp>>8)&0xFF);
  pcontext->in[62]=(lcrzo_uint8)((tmp>>16)&0xFF);
  pcontext->in[63]=(lcrzo_uint8)((tmp>>24)&0xFF);
#else
  pcontext->in[63]=(lcrzo_uint8)(tmp&0xFF);
  pcontext->in[62]=(lcrzo_uint8)((tmp>>8)&0xFF);
  pcontext->in[61]=(lcrzo_uint8)((tmp>>16)&0xFF);
  pcontext->in[60]=(lcrzo_uint8)((tmp>>24)&0xFF);
#endif
  
  shared_priv_md5_transform2(pcontext->buf, pcontext->in);
  byteReverse((lcrzo_data)pcontext->buf, 4);
  memcpy(digest, pcontext->buf, 16);
  memset(pcontext, 0, sizeof(pcontext));	/* In case it's sensitive */

  /* convert the hash to hexa */
  lcrzo_er(lcrzo_hexa_initm_data(digest, 16, phexamd5));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_md5_compute_data(lcrzo_constdata data,
			    lcrzo_int32 datasize,
			    lcrzo_string *phexamd5)
{ shared_md5_context ctx;

  lcrzo_er(shared_md5_init(&ctx));
  lcrzo_er(shared_md5_update(&ctx, data, datasize));
  lcrzo_er(shared_md5_final(&ctx, phexamd5));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_md5_compute_string(lcrzo_conststring str,
			      lcrzo_string *phexamd5)
{ shared_md5_context ctx;

  lcrzo_er(shared_md5_init(&ctx));
  lcrzo_er(shared_md5_update(&ctx, (lcrzo_constdata)str, strlen(str)));
  lcrzo_er(shared_md5_final(&ctx, phexamd5));

  return(LCRZO_ERR_OK);
}
