/*
		                 LCRZOEX
                             Network toolbox
             Copyright(c) 1999,2000,2001,2002 Laurent Constantin
                                  -----

  Main server    : http://www.laurentconstantin.com/
  Backup servers : http://go.to/laurentconstantin/
                   http://laurentconstantin.est-la.com/
                   http://laurentconstantin.free.fr/
                   http://membres.lycos.fr/lauconstantin/
  [my current email address is on the web servers]

                                  -----
  This file is part of Lcrzoex.

  Lcrzoex is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Lcrzoex is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details (http://www.gnu.org/).

------------------------------------------------------------------------
*/

#include <lcrzo.h>
#include "../lcrzoex.h"

/*-------------------------------------------------------------*/
#if defined LCRZODEF_SYSTEM_Unix
 #include <unistd.h>
 #include <sys/types.h>
#elif defined LCRZODEF_SYSTEM_Windows
 #include <windows.h>
 #include <io.h>
 #include <fcntl.h>
#else
 #error "Unknown value for LCRZODEF_SYSTEM"
#endif

/*-------------------------------------------------------------*/
int shared_misc_compilparam_print(void)
{ lcrzo_int16 language;

  /* get the current language */
  lcrzo_epr(lcrzo_global_get_language(&language));
  if ( language==LCRZO_GLOBAL_LANG_FR )
  { printf("Lcrzoex version %s.\n", LCRZOEXDEF_VERSION);
    printf("Lcrzoex est un logiciel libre, couvert par GNU ");
    printf("General Public License.\n");
    printf("Lcrzoex a ete compile avec les parametres suivants :\n");
  }
  else
  { printf("Lcrzoex version %s.\n", LCRZOEXDEF_VERSION);
    printf("Lcrzoex is free software, covered by the GNU General Public License.\n");
    printf("Lcrzoex was compiled with the following parameters :\n");
  }

  printf("  LCRZOEXDEF_VERSION = %s\n", LCRZOEXDEF_VERSION);
  printf("  LCRZOEXDEF_VERSIONMAJOR = %d\n", LCRZOEXDEF_VERSIONMAJOR);
  printf("  LCRZOEXDEF_VERSIONMINOR = %02d\n", LCRZOEXDEF_VERSIONMINOR);
  printf("  LCRZODEF_VERSION = %s\n", LCRZODEF_VERSION);
  printf("  LCRZODEF_VERSIONMAJOR = %d\n", LCRZODEF_VERSIONMAJOR);
  printf("  LCRZODEF_VERSIONMINOR = %02d\n", LCRZODEF_VERSIONMINOR);
  printf("  LCRZODEF_LANG = %d\n", LCRZODEF_LANG);
  printf("  LCRZODEF_SYS = %s\n", LCRZODEF_SYS);
  printf("  LCRZODEF_SYSTEM = %s\n", LCRZODEF_SYSTEM);
  printf("  LCRZODEF_SYSVER = %s\n", LCRZODEF_SYSVER);
  printf("  LCRZODEF_ENDIANLITTLE = %d\n", LCRZODEF_ENDIANLITTLE);
  printf("  LCRZODEF_ENDIANBIG = %d\n", LCRZODEF_ENDIANBIG);

  puts("");
  lcrzo_epr(lcrzo_compilparam_print());
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_misc_userstop1_printmsg(void)
{ lcrzo_int16 language;

  lcrzo_epr(lcrzo_global_get_language(&language));
  if (language==LCRZO_GLOBAL_LANG_FR)
    puts("Pressez q ou Q pour sortir.");
  else
    puts("Press q or Q to exit.");
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_misc_userstop1_test(lcrzo_file *pstdinfile,
			       lcrzo_bool redisplaymsgifbadchar,
			       lcrzo_bool *pwanttostop,
			       lcrzo_uint8 *preadchar)
{ lcrzo_data data;
  lcrzo_bool wanttostop;
  lcrzo_uint8 readchar;

  /* read the character */
  lcrzo_epr(lcrzo_file_readm_fixed(pstdinfile, LCRZO_WAIT_INFINITE, 1, &data));
  readchar=data[0];
  lcrzo_er(lcrzo_data_free2(&data));

  /* test */
  if (readchar=='q' || readchar=='Q')
  { wanttostop=LCRZO_TRUE;
    if (redisplaymsgifbadchar) puts(""); /* new line */
  }
  else
  { wanttostop=LCRZO_FALSE;
    if (redisplaymsgifbadchar)
    { printf("  ");
      lcrzo_epr(shared_misc_userstop1_printmsg());
    }
  }
  if (pwanttostop!=NULL) *pwanttostop=wanttostop;
  if (preadchar!=NULL) *preadchar=readchar;
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_misc_userstop2_printmsg(void)
{ lcrzo_int16 language;

  lcrzo_epr(lcrzo_global_get_language(&language));
  if (language==LCRZO_GLOBAL_LANG_FR)
    puts("Pressez q ou Q, suivi d'un saut de ligne, pour sortir.");
  else
    puts("Press q or Q, followed by a new line, to exit.");
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_misc_userstop2_test(lcrzo_data data,
			       lcrzo_int32 datasize,
			       lcrzo_bool *pwanttostop)
{ lcrzo_bool wanttostop;
  lcrzo_uint8 c;
  lcrzo_int16 language;

  lcrzo_epr(lcrzo_global_get_language(&language));
  wanttostop=LCRZO_FALSE;
  if (datasize==1)
  { if (data[0]=='q' || data[0]=='Q')
    { if (language==LCRZO_GLOBAL_LANG_FR)
      { lcrzo_epr(lcrzo_stdin_char("Voulez-vous quitter ?", 
				   (lcrzo_constdata)"oOnN", 'o', &c));
        if (c=='o' || c=='O') wanttostop=LCRZO_TRUE;
      }
      else
      { lcrzo_epr(lcrzo_stdin_char("Do you want to quit ?", 
				   (lcrzo_constdata)"yYnN", 'y', &c));
        if (c=='y' || c=='Y') wanttostop=LCRZO_TRUE;
      }
    }
  }

  if (pwanttostop!=NULL) *pwanttostop=wanttostop;
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_misc_userstop3_printmsgtest(lcrzo_bool *pwanttostop)
{ lcrzo_bool wanttostop;
  lcrzo_uint8 c;
  lcrzo_int16 language;

  wanttostop=LCRZO_FALSE;
  lcrzo_epr(lcrzo_global_get_language(&language));
  if (language==LCRZO_GLOBAL_LANG_FR)
  { lcrzo_epr(lcrzo_stdin_char("Voulez-vous terminer ?", 
			       (lcrzo_constdata)"oOnN", 'n', &c));
    if (c=='o' || c=='O') wanttostop=LCRZO_TRUE;
  }
  else
  { lcrzo_epr(lcrzo_stdin_char("Do you want to finish ?", 
			       (lcrzo_constdata)"yYnN", 'n', &c));
    if (c=='y' || c=='Y') wanttostop=LCRZO_TRUE;
  }

  if (pwanttostop!=NULL) *pwanttostop=wanttostop;
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_misc_dangerousexample(void)
{ lcrzo_uint8 c;
  lcrzo_int16 language;

  lcrzo_epr(lcrzo_global_get_language(&language));

  if ( language==LCRZO_GLOBAL_LANG_FR )
  { puts("Ceci va perturber le fonctionnement de tout le reseau.");
    lcrzo_epr(lcrzo_stdin_char("Voulez-vous reellement continuer ?",
			       (lcrzo_constdata)"oOnN", 'n', &c));
  }
  else
  { puts("This example will disturb your network.");
    lcrzo_epr(lcrzo_stdin_char("Do you really want to continue ?",
			       (lcrzo_constdata)"yYnN", 'n', &c));
  }
  if (c=='n' || c=='N') exit(LCRZO_ERR_OK);
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_misc_verifyresponse(lcrzo_sock *psock, 
			       lcrzo_uint16 error1, 
			       lcrzo_uint16 error2, 
			       lcrzo_uint16 error3, 
			       lcrzo_uint16 error4,
			       lcrzo_bool displayerrors,
			       lcrzo_uint16 *preaderror,
			       lcrzo_data *preaddata,
			       lcrzo_int32 *preaddatasize)
{ lcrzo_data data, readdata;
  lcrzo_int32 datasize, readdatasize;
  lcrzo_uint16 readerror;
  lcrzo_string errorlist;
  lcrzo_bool notenough;
  lcrzo_int16 language;
  int ret;

  /* read response from the socket */
  notenough=LCRZO_TRUE;
  readerror=0; /*just for the compiler*/
  lcrzo_epr(lcrzo_data_initm_text("", &readdata, &readdatasize));
  while (notenough)
  { /* read a line */
    lcrzo_epr(lcrzo_sock_readm_line(psock, LCRZO_WAIT_INFINITE, 
				    &data, &datasize));
    /* append data in readdata */
    if ( datasize>=4 )
    { /* if data is "    YYYY" we save only YYYY, an continue */
      if ( data[0]==' ' && data[1]==' ' && data[2]==' ' && data[3]==' ' )
      { lcrzo_epr(lcrzo_data_appendm_data(data+4, datasize-4, readdatasize, 
					  &readdata, &readdatasize));
      }   
      /* if data is "xxx.YYYY" we save only YYYY */
      else if ( isdigit(data[0]) && isdigit(data[1]) && isdigit(data[2]) )
      { lcrzo_epr(lcrzo_data_appendm_data(data+4, datasize-4, readdatasize, 
					  &readdata, &readdatasize));
        /* if data is "xxx YYYY" there is no more line to read */
        if ( data[3]==' ' ) 
	{ notenough=LCRZO_FALSE;
	  data[3]='\0';
    lcrzo_epr(lcrzo_uint16_init_text((lcrzo_string)data, &readerror));
	}
      }
      /* else, we append the entire line */
      else
      { lcrzo_epr(lcrzo_data_appendm_data(data, datasize, readdatasize, 
					  &readdata, &readdatasize));
      }
      /* separate each line by a newline */
      lcrzo_epr(lcrzo_data_appendm_text("\n", readdatasize, 
					&readdata, &readdatasize));
    }
    lcrzo_er(lcrzo_data_free2(&data));
  }

  /* verify error */
  if ( readerror!=error1 && readerror!=error2 &&
       readerror!=error3 && readerror!=error4 )
  { if (displayerrors)
    { /* prepare error string */
      lcrzo_epr(lcrzo_string_initm_uint16(error1, "%u", &errorlist));
      if ( error2 != error1 )
      { lcrzo_epr(lcrzo_string_appendm_char('-', 1, &errorlist));
        lcrzo_epr(lcrzo_string_appendm_uint16(error2, "%u", &errorlist));
        if ( error3 != error2 )
        { lcrzo_epr(lcrzo_string_appendm_char('-', 1, &errorlist));
   	  lcrzo_epr(lcrzo_string_appendm_uint16(error3, "%u", &errorlist));
	  if ( error4 != error3 )
	  { lcrzo_epr(lcrzo_string_appendm_char('-', 1, &errorlist));
	    lcrzo_epr(lcrzo_string_appendm_uint16(error4, "%u", &errorlist));
	  }
	}
      }
      /* print message */
      lcrzo_epr(lcrzo_global_get_language(&language));
      if ( language==LCRZO_GLOBAL_LANG_FR )
	printf("Je voulais %s, mais j'ai recu %d\n", errorlist, readerror); 
      else
	printf("I wanted %s, but I received %d\n", errorlist, readerror); 
      lcrzo_epr(lcrzo_data_print(readdata, readdatasize,LCRZO_PRINTTYPE_DUMP));
      lcrzo_er(lcrzo_string_free2(&errorlist));
    }
    ret=LCRZOEX_ERR_MISC_BADRESPONSE;
  }
  else
    ret=LCRZO_ERR_OK;

  /* set output */
  if (preaderror!=NULL) *preaderror=readerror;
  if (preaddatasize!=NULL) *preaddatasize=readdatasize;
  if (preaddata!=NULL)
  { *preaddata=readdata;
  }
  else
  { lcrzo_er(lcrzo_data_free2(&readdata));
  }

  return(ret);
}

/*-------------------------------------------------------------*/
int shared_misc_verifyresponse1(lcrzo_sock *psock, 
				lcrzo_uint16 error)
{ return(shared_misc_verifyresponse(psock, error, error, error, error,
				    LCRZO_TRUE, NULL, NULL, NULL));
}

/*-------------------------------------------------------------*/
int shared_misc_verifyresponse2(lcrzo_sock *psock, 
				lcrzo_uint16 error1, 
				lcrzo_uint16 error2)
{ return(shared_misc_verifyresponse(psock, error1, error2, error2, error2,
				    LCRZO_TRUE, NULL, NULL, NULL));
}

/*-------------------------------------------------------------*/
int shared_misc_verifyresponse3(lcrzo_sock *psock, 
				lcrzo_uint16 error1, 
				lcrzo_uint16 error2, 
				lcrzo_uint16 error3)
{ return(shared_misc_verifyresponse(psock, error1, error2, error3, error3,
				    LCRZO_TRUE, NULL, NULL, NULL));
}

/*-------------------------------------------------------------*/
int shared_misc_verifyresponse4(lcrzo_sock *psock, 
				lcrzo_uint16 error1, 
				lcrzo_uint16 error2, 
				lcrzo_uint16 error3, 
				lcrzo_uint16 error4)
{ return(shared_misc_verifyresponse(psock, error1, error2, error3, error4,
				    LCRZO_TRUE, NULL, NULL, NULL));
}

/*-------------------------------------------------------------*/
int shared_misc_printnumberbase(char *string, int base)
{ lcrzo_uint32 i;
  lcrzo_int16 language;
  lcrzo_string str;
  char *p;

  lcrzo_epr(lcrzo_global_get_language(&language));
  lcrzo_epr(shared_para_string_init(string, "nb ", "10", &str));
  i=strtoul(str, &p, base);
  if (*p!='\0')
  { if ( language==LCRZO_GLOBAL_LANG_FR )
      printf("Le nombre %s n'est pas en base %d\n", str, base);
    else
      printf("The number %s is not in base %d\n", str, base);
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }
  if ( errno==ERANGE ) 
  { if ( language==LCRZO_GLOBAL_LANG_FR )
      printf("Le nombre %s en base %d est trop long\n", str, base);
    else
      printf("The number %s in base %d is too long\n", str, base);
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }
  lcrzo_er(lcrzo_string_free2(&str));

  lcrzo_epr(shared_misc_printnumber(i));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_priv_misc_printnumber_bin(lcrzo_int16 language,
				     lcrzo_conststring type,
				     lcrzo_uint32 number,
				     lcrzo_bool addaminus,
				     lcrzo_int32 bitsize);
int shared_priv_misc_printnumber_bin(lcrzo_int16 language,
				     lcrzo_conststring type,
				     lcrzo_uint32 number,
				     lcrzo_bool addaminus,
				     lcrzo_int32 bitsize)
{ lcrzo_uint32 n;
  int j;

  if ( language==LCRZO_GLOBAL_LANG_FR )
    printf("Binaire %s        : ", type);
  else
    printf("Binary %s         : ", type);
  if (addaminus) printf("-");

  for ( j=bitsize-1 ; j>=0 ; j-- )
  { n=(number>>j)&1;
    printf("%c", (n==0)?'0':'1');
    if ( !(j%8) ) printf(" ");
  }
  printf("\n");
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_priv_misc_printnumber_hex(lcrzo_int16 language,
				     lcrzo_conststring type,
				     lcrzo_uint32 number,
				     lcrzo_bool addaminus,
				     lcrzo_int32 bitsize);
int shared_priv_misc_printnumber_hex(lcrzo_int16 language,
				     lcrzo_conststring type,
				     lcrzo_uint32 number,
				     lcrzo_bool addaminus,
				     lcrzo_int32 bitsize)
{ 
  printf("Hexa %s : ", type);
  if (addaminus) printf("-");

  if (bitsize==8)
  { lcrzo_er(lcrzo_uint8_print("", (lcrzo_uint8)number, "%02X", "\n"));
  }
  else if (bitsize==16)
  { lcrzo_er(lcrzo_uint16_print("", (lcrzo_uint16)number, "%04X", "\n"));
  }
  else if (bitsize==32)
  { lcrzo_er(lcrzo_uint32_print("", number, "%08X", "\n"));
  }
  else
    return(LCRZO_ERR_IEINTERNALERROR);

  return(LCRZO_ERR_OK);
  language=language;
}

/*-------------------------------------------------------------*/
int shared_misc_printnumber(lcrzo_uint32 number)
{ lcrzo_uint32 u32, u32x;
  lcrzo_uint16 u16, u16x;
  lcrzo_uint8  u8, u8x;
  lcrzo_int32 i32;
  lcrzo_int16 i16;
  lcrzo_int8  i8;
  lcrzo_bool ok16, ok8, u32xsig, u16xsig, u8xsig;
  lcrzo_ipa ipa;
  lcrzo_int16 language;

  lcrzo_epr(lcrzo_global_get_language(&language));

  /* set default values */
  ok16=LCRZO_FALSE;
  ok8=LCRZO_FALSE;
  u32xsig=LCRZO_FALSE;
  u16xsig=LCRZO_FALSE;
  u8xsig=LCRZO_FALSE;
  u16=0;
  i16=0;
  u16x=0;
  u8=0;
  i8=0;
  u8x=0;

  /* set values */
  u32=number;
  i32=(lcrzo_int32)u32;
  u32x=u32;
  if (u32>0x7FFFFFFF) 
  { u32x=(~u32)+1; 
    u32xsig=LCRZO_TRUE;
    if (u32x<=0x8000)
    { ok16=LCRZO_TRUE;
      u16xsig=LCRZO_TRUE; 
      u16x=(lcrzo_uint16)(u32x&0xFFFF);
      i16=(lcrzo_int16)(-(lcrzo_int16)u16x);
      u16=(lcrzo_uint16)((~u16x)+1);
    }
    if (u32x<=0x80)
    { ok8=LCRZO_TRUE;
      u8xsig=LCRZO_TRUE; 
      u8x=(lcrzo_uint8)(u32x&0xFF);
      i8=(lcrzo_int8)(-(lcrzo_int8)u8x);
      u8=(lcrzo_uint8)((~u8x)+1);
    }
  }
  else
  { if (u32<=0xFFFF)
    { ok16=LCRZO_TRUE;
      u16=(lcrzo_uint16)(u32&0xFFFF);
      i16=(lcrzo_int16)u16;
      u16x=u16;
      if (u16>0x7FFF) 
      { u16x=(lcrzo_uint16)((~u16)+1); 
        u16xsig=LCRZO_TRUE; 
      }
    }
    if (u32<=0xFF)
    { ok8=LCRZO_TRUE;
      u8=(lcrzo_uint8)(u32&0xFF);
      i8=(lcrzo_int8)u8;
      u8x=u8;
      if (u8>0x7F) 
      { u8x=(lcrzo_uint8)((~u8)+1); 
        u8xsig=LCRZO_TRUE;
      }
    }
  }

  /* binary */
  lcrzo_er(shared_priv_misc_printnumber_bin(language, "uint32", u32,
					    LCRZO_FALSE, 32));
  lcrzo_er(shared_priv_misc_printnumber_bin(language, "int32 ", u32x, u32xsig,
					    32));
  if (ok16)
  { lcrzo_er(shared_priv_misc_printnumber_bin(language, "uint16", u16,
					      LCRZO_FALSE, 16));
    lcrzo_er(shared_priv_misc_printnumber_bin(language, "int16 ", u16x,
					      u16xsig, 16));
  }
  if (ok8)
  { lcrzo_er(shared_priv_misc_printnumber_bin(language, "uint8 ", u8,
					      LCRZO_FALSE, 8));
    lcrzo_er(shared_priv_misc_printnumber_bin(language, "int8  ", u8x,
					      u8xsig, 8));
  }

  /* octal */
  lcrzo_er(lcrzo_uint32_print("Octal                 : ", u32, "%o", "\n"));

  /* decimal */
  lcrzo_er(lcrzo_uint32_print("Decimal uint32        : ", u32, "%u", "\n"));
  lcrzo_er(lcrzo_int32_print( "Decimal int32         : ", i32, "%d", "\n"));
  lcrzo_er(lcrzo_uint32_print("Decimal htonl(uint32) : ", lcrzo_htonl(u32),
			      "%u", "\n"));
  if (ok16)
  { lcrzo_er(lcrzo_uint16_print("Decimal uint16        : ", u16, "%u", "\n"));
    lcrzo_er(lcrzo_int16_print("Decimal int16         : ", i16, "%d", "\n"));
    lcrzo_er(lcrzo_uint16_print("Decimal htons(uint16) : ", lcrzo_htons(u16),
				"%u", "\n"));
  }
  if (ok8)
  { lcrzo_er(lcrzo_uint8_print( "Decimal uint8         : ", u8, "%u", "\n"));
    lcrzo_er(lcrzo_int8_print("Decimal int8          : ", i8, "%d", "\n"));
  }

  /* hexadecimal */
  lcrzo_er(shared_priv_misc_printnumber_hex(language, "uint32          ",
				   u32, LCRZO_FALSE, 32));
  lcrzo_er(shared_priv_misc_printnumber_hex(language, "int32           ", u32x,
				   u32xsig, 32));
  lcrzo_er(shared_priv_misc_printnumber_hex(language, "htonl(uint32)   ", 
				   lcrzo_htonl(u32), LCRZO_FALSE, 32));
  if (ok16)
  { lcrzo_er(shared_priv_misc_printnumber_hex(language, "uint16          ",
				     u16, LCRZO_FALSE, 16));
    lcrzo_er(shared_priv_misc_printnumber_hex(language, "int16           ", 
				     u16x, u16xsig, 16));
    lcrzo_er(shared_priv_misc_printnumber_hex(language, "htons(uint16)   ", 
				     lcrzo_htons(u16), LCRZO_FALSE, 16));
  }
  if (ok8)
  { lcrzo_er(shared_priv_misc_printnumber_hex(language, "uint8           ", 
				     u8, LCRZO_FALSE, 8));
    lcrzo_er(shared_priv_misc_printnumber_hex(language, "int8            ", 
				     u8x, u8xsig, 8));
  }

  /* character */
  if ( ok8 )
  { if ( language==LCRZO_GLOBAL_LANG_FR )
      printf("Caractere             : ");
    else
      printf("Character             : ");
    printf("%c\n", i8);
  }

  /* IP address */
  lcrzo_epr(lcrzo_ipa_init_ipl(number, ipa));
  if ( language==LCRZO_GLOBAL_LANG_FR )
    printf("Adresse IP            : ");
  else
    printf("IP address            : ");
  printf("%d.%d.%d.%d\n", 
	 (unsigned int)ipa[0], (unsigned int)ipa[1],
	 (unsigned int)ipa[2], (unsigned int)ipa[3]);
  if ( language==LCRZO_GLOBAL_LANG_FR )
    printf("Adresse IP htonl      : ");
  else
    printf("IP address htonl      : ");
  printf("%d.%d.%d.%d\n",
	 (unsigned int)ipa[3], (unsigned int)ipa[2],
	 (unsigned int)ipa[1], (unsigned int)ipa[0]);

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_misc_printdata(lcrzo_data data, lcrzo_int32 datasize)
{ lcrzo_int16 language;
  lcrzo_string hexamd5;

  lcrzo_epr(lcrzo_global_get_language(&language));

  if ( language==LCRZO_GLOBAL_LANG_FR )
    printf("***********Texte affichable :\n");
  else
    printf("***********Printable text :\n");
  lcrzo_epr(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_PTEXTN));

  printf("***********Dump :\n");
  lcrzo_epr(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_DUMP));

  printf("***********Hexadecimal :\n");
  lcrzo_epr(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_HEXAN));

  if ( language==LCRZO_GLOBAL_LANG_FR )
    printf("***********Mixte :\n");
  else
    printf("***********Mixed :\n");
  lcrzo_epr(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_MIXEDN));

  printf("***********Base 64 :\n");
  lcrzo_epr(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_BASE64N));

  printf("***********MD5 :\n");
  lcrzo_epr(shared_md5_compute_data(data, datasize, &hexamd5));
  printf("%s\n", hexamd5);
  lcrzo_er(lcrzo_string_free2(&hexamd5));

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
/* This function was created before having shared_md5. I'm not anymore
   sure about the security of the hash. I found it on the web 5 years
   ago (so, I had a copy on my computer 3 years before starting lcrzo).
   However, before using it I did not take time to check it. It looks like
   a strange implementation of SHA. Now, the more I look at it, the
   more I find it suspect. So you should not use it anymore.
*/
int shared_misc_randpasshash_old(lcrzo_conststring serverrand, 
				 lcrzo_conststring clientrand,
				 lcrzo_conststring password,
				 lcrzo_string *phash)
{ lcrzo_uint8 pass[17];
  lcrzo_uint32 arr[12];
  lcrzo_uint32 A, B, C, D, E;
  int i;

  if (strlen(serverrand)!=16) return(LCRZOEX_ERR_MISC_HASHBADSIZE);
  if (strlen(clientrand)!=16) return(LCRZOEX_ERR_MISC_HASHBADSIZE);

  /* generate a 16 bytes long password */
  for ( i=0 ; i<16 ; i++ )  
  { if ( i>=(int)strlen(password) ) pass[i]=' ';
    else pass[i]=password[i];
  }
  pass[16]='\0';

  /* generate the array of 3*16=3*4*4=12*4 bytes */
  #define SHARED_MISC_ARR(x) (((x)[0]<<24)|((x)[1]<<16)|((x)[2]<<8)|(x)[3])
  for ( i=0 ; i<4 ; i++ )  
  { arr[i]=SHARED_MISC_ARR(serverrand+(i<<2));
    arr[4+i]=SHARED_MISC_ARR(pass+(i<<2));
    arr[8+i]=SHARED_MISC_ARR(clientrand+(i<<2));
  }

  /* compute the hash */
  #define SHARED_MISC_F1(x,y,z)  ( (z) ^ ( (x) & ( (y) ^ (z) ) ) )
  #define SHARED_MISC_F2(x,y,z)  ( (x) ^ (y) ^ (z) )
  #define SHARED_MISC_F3(x,y,z)  ( ( (x) & (y) ) | ( (z) & ( (x) | (y) ) ) )
  #define SHARED_MISC_F4(x,y,z)  ( (x) ^ (y) ^ (z) )
  #define SHARED_MISC_ROTL(n,X)  ( ( ( X ) << n ) | ( ( X ) >> ( 32 - n ) ) )
  #define SHARED_MISC_FF(a, b, c, d, e, f, k, data) ( e += SHARED_MISC_ROTL( 5, a ) + f( b, c, d ) + k + data, b = SHARED_MISC_ROTL( 30, b ) )
  A=0x4B2A71C8u; B=0x81CA93B6u; C=0xBD61ADC9u; D=0x492D16CAu; E=0xBD72C9A1u;
  SHARED_MISC_FF( A, B, C, D, E, SHARED_MISC_F1, 0x381C3A72u, arr[0]);
  SHARED_MISC_FF( E, A, B, C, D, SHARED_MISC_F2, 0xC51A85BFu, arr[1] );
  SHARED_MISC_FF( E, A, B, C, D, SHARED_MISC_F1, 0xC714A7B3u, arr[2] );
  SHARED_MISC_FF( D, E, A, B, C, SHARED_MISC_F4, 0x2934C1AEu, arr[3] );
  SHARED_MISC_FF( C, D, E, A, B, SHARED_MISC_F3, 0x31CA83A5u, arr[4] );
  SHARED_MISC_FF( B, C, D, E, A, SHARED_MISC_F2, 0xFE341A94u, arr[5] );
  SHARED_MISC_FF( A, B, C, D, E, SHARED_MISC_F4, 0xFA183B6Au, arr[6] );
  SHARED_MISC_FF( D, C, D, A, E, SHARED_MISC_F1, 0x381C3A72u, arr[7]);
  SHARED_MISC_FF( C, C, A, B, D, SHARED_MISC_F3, 0xDE529753u, arr[8] );
  SHARED_MISC_FF( A, E, C, B, D, SHARED_MISC_F2, 0x69BCDF12u, arr[9] );
  SHARED_MISC_FF( B, D, E, A, C, SHARED_MISC_F1, 0x9CED1864u, arr[10] );
  SHARED_MISC_FF( C, A, B, D, E, SHARED_MISC_F3, 0xBCCD19ADu, arr[11] );
  lcrzo_er(lcrzo_string_initm_uint32(A, "%08X", phash));
  lcrzo_er(lcrzo_string_appendm_uint32(B, "%08X", phash));
  lcrzo_er(lcrzo_string_appendm_uint32(C, "%08X", phash));
  lcrzo_er(lcrzo_string_appendm_uint32(D, "%08X", phash));
  lcrzo_er(lcrzo_string_appendm_uint32(E, "%08X", phash));

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_misc_randpasshash_md5(lcrzo_conststring serverrand, 
				 lcrzo_conststring clientrand,
				 lcrzo_conststring password,
				 lcrzo_string *phash)
{ shared_md5_context ctx;

  lcrzo_er(shared_md5_init(&ctx));
  lcrzo_er(shared_md5_update_text(&ctx, serverrand));
  lcrzo_er(shared_md5_update_text(&ctx, clientrand));
  lcrzo_er(shared_md5_update_text(&ctx, password));
  lcrzo_er(shared_md5_final(&ctx, phash));

  return (LCRZO_ERR_OK);
}


/*-------------------------------------------------------------*/
int shared_misc_strcasecmp(lcrzo_conststring s1, lcrzo_conststring s2)
{ const char *p1 = (const char *) s1;
  const char *p2 = (const char *) s2;
  lcrzo_uint8 c1, c2;

  if (p1 == p2)
    return 0;

  do
  { c1 = (lcrzo_uint8)tolower(*p1++);
    c2 = (lcrzo_uint8)tolower(*p2++);
    if (c1 == '\0') break;
  }
  while (c1 == c2);

  return (c1 - c2);
}

/*-------------------------------------------------------------*/
int shared_misc_strncasecmp(lcrzo_conststring s1, 
			    lcrzo_conststring s2,
			    lcrzo_int32 n)
{ const char *p1 = (const char *) s1;
  const char *p2 = (const char *) s2;
  lcrzo_uint8 c1, c2;
  int i;

  if (p1 == p2)
    return 0;

  i=0;
  do
  { c1 = (lcrzo_uint8)tolower(*p1++);
    c2 = (lcrzo_uint8)tolower(*p2++);
    if (c1 == '\0') break;
    i++;
    if ( i==n ) break;
  }
  while (c1 == c2);

  return (c1 - c2);
}

/*-------------------------------------------------------------*/
int shared_string_initm_text_case(lcrzo_conststring text,
				  lcrzo_bool converttoupper,
				  lcrzo_string *pstringout)
{ lcrzo_string text2;
  char c;
  int i, strlentext;

  lcrzo_er(lcrzo_string_initm_text(text, &text2));
  strlentext=strlen(text);
  for ( i=0 ; i<strlentext ; i++ )
  { c=text[i];
    if (converttoupper)
    { if ( c>='a' && c<='z' ) c=(char)('A'+c-'a');
    }
    else
    { if ( c>='A' && c<='Z' ) c=(char)('a'+c-'A');
    }
    text2[i]=c;
  }
  
  if (pstringout==NULL)
  { lcrzo_er(lcrzo_string_free2(&text2));
  }
  else
    *pstringout=text2;
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_string_search_text_case(lcrzo_conststring stringin,
				   lcrzo_bool searchcomplement,
				   lcrzo_int32 stringinstartpos,
				   lcrzo_int32 stringinendpos,
				   lcrzo_conststring searchedtext,
				   lcrzo_int32 *pstartfoundposition_negative,
				   lcrzo_int32 *pstartfoundposition_positive,
				   lcrzo_int32 *pendfoundposition_negative,
				   lcrzo_int32 *pendfoundposition_positive)
{ lcrzo_string stringin2, searchedtext2;
  int ret;

  lcrzo_er(shared_string_initm_text_case(stringin, LCRZO_FALSE, &stringin2));
  lcrzo_er(shared_string_initm_text_case(searchedtext, LCRZO_FALSE, 
					 &searchedtext2));

  ret=lcrzo_string_search_text(stringin2, searchcomplement,
			       stringinstartpos, stringinendpos,
			       searchedtext2,
			       pstartfoundposition_negative,
			       pstartfoundposition_positive,
			       pendfoundposition_negative,
			       pendfoundposition_positive);

  lcrzo_er(lcrzo_string_free2(&stringin2));
  lcrzo_er(lcrzo_string_free2(&searchedtext2));

  return(ret);
}

/*-------------------------------------------------------------*/
typedef struct
{ lcrzo_int8 command[512];
  lcrzo_ipc ipc;
  lcrzo_ipc ipcchildend;
  int readpipe[2];
  int writepipe[2];
} shared_priv_misc_ipcexec;

/*-------------------------------------------------------------*/
int shared_misc_ipcexec_child(lcrzo_ptr infos, 
			      lcrzo_int32 infossize, 
			      lcrzo_process *pparentprocess);
#if defined LCRZODEF_SYSTEM_Unix
int shared_misc_ipcexec_child2(lcrzo_ptr infos, 
			       lcrzo_int32 infossize, 
			       lcrzo_process *pparentprocess);
int shared_misc_ipcexec_child2(lcrzo_ptr infos, 
			       lcrzo_int32 infossize, 
			       lcrzo_process *pparentprocess)
{ shared_priv_misc_ipcexec *pspmi=(shared_priv_misc_ipcexec *)infos;
  int ret;

  /* check for correct usage of this function */
  if (infossize!=sizeof(shared_priv_misc_ipcexec))
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);

  /* close parent process handle because we do not need it */
  lcrzo_epr(lcrzo_process_close(pparentprocess));

  /* close unneeded file descriptors */
  ret=close(pspmi->writepipe[0]);
  if (ret==-1) return(LCRZO_ERR_FUCLOSE);
  ret=close(pspmi->readpipe[0]);
  if (ret==-1) return(LCRZO_ERR_FUCLOSE);

  /* associate file descriptors items */
  ret=dup2(pspmi->readpipe[1], 0);
  if (ret==-1) return(LCRZOEX_ERR_FU_DUP2);
  ret=close(pspmi->readpipe[1]);
  if (ret==-1) return(LCRZO_ERR_FUCLOSE);
  ret=dup2(pspmi->writepipe[1], 1);
  if (ret==-1) return(LCRZOEX_ERR_FU_DUP2);
  ret=close(pspmi->writepipe[1]);
  if (ret==-1) return(LCRZO_ERR_FUCLOSE);
  ret=close(2);
  if (ret==-1) return(LCRZO_ERR_FUCLOSE);
 
  /* execute the command (do not purge environment variables
     for security reason, because user might really want to
     use environment variable). It's the same with file descriptors */
  ret=execl("/bin/sh", "sh", "-c", pspmi->command, NULL);
  if (ret==-1) return(LCRZOEX_ERR_FU_EXECL);

  /* should not be reached */
  return (LCRZO_ERR_IEINTERNALERROR);
}
int shared_misc_ipcexec_child(lcrzo_ptr infos, 
			      lcrzo_int32 infossize, 
			      lcrzo_process *pparentprocess)
{ shared_priv_misc_ipcexec *pspmi=(shared_priv_misc_ipcexec *)infos;
  shared_priv_misc_ipcexec spmi;
  lcrzo_process child;
  lcrzo_fd2 fd2readpipe, fd2writepipe;
  lcrzo_wait waitipc, waitipcend, waitreadpipe, waitchildend;
  lcrzo_bool eventipc, eventipcend, eventreadpipe, eventchildend;
  lcrzo_bool canwaitonipc, canwaitonreadpipe;
  lcrzo_data data;
  lcrzo_int32 datasize;
  int pipefd[2], ret, childret;

  /* check for correct usage of this function */
  if (infossize!=sizeof(shared_priv_misc_ipcexec))
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);

  /* close parent process handle because we do not need it */
  lcrzo_epr(lcrzo_process_close(pparentprocess));

  /* set IPC */
  lcrzo_epr(lcrzo_ipc_currentprocessissideb(&pspmi->ipc));
  lcrzo_epr(lcrzo_ipc_currentprocessissideb(&pspmi->ipcchildend));
  lcrzo_epr(lcrzo_ipc_close_write(&pspmi->ipcchildend));

  /* copy command */
  lcrzo_epr(lcrzo_string_init_text(pspmi->command, sizeof(spmi.command),
				   spmi.command));

  /* we use pipe to communicate with the second child */
  ret=pipe(pipefd);
  if (ret==-1) return(LCRZO_ERR_FUPIPE);
  spmi.readpipe[0]=pipefd[0];
  spmi.writepipe[1]=pipefd[1];
  ret=pipe(pipefd);
  if (ret==-1) return(LCRZO_ERR_FUPIPE);
  spmi.readpipe[1]=pipefd[0];
  spmi.writepipe[0]=pipefd[1];

  /* start the second child process */
  lcrzo_epr(lcrzo_process_other_init(&shared_misc_ipcexec_child2, 
                                     (lcrzo_data)&spmi, sizeof(spmi), 
                                     &child));

  /* initialize wait */
  lcrzo_epr(lcrzo_wait_init_ipc_read(&pspmi->ipc, &waitipc));
  lcrzo_epr(lcrzo_wait_init_ipc_read(&pspmi->ipcchildend, &waitipcend));
  lcrzo_epr(lcrzo_wait_init_process_end(&child, &waitchildend));
  lcrzo_epr(lcrzo_fd2_init_fd(spmi.writepipe[0], &fd2writepipe));
  lcrzo_epr(lcrzo_fd2_init_fd(spmi.readpipe[0], &fd2readpipe));
  lcrzo_epr(lcrzo_wait_init_fd2_read(&fd2readpipe, &waitreadpipe));

  /* close unneeded file descriptors */
  ret=close(spmi.writepipe[1]);
  if (ret==-1) return(LCRZO_ERR_FUCLOSE);
  ret=close(spmi.readpipe[1]);
  if (ret==-1) return(LCRZO_ERR_FUCLOSE);

  /* copy input to output, and output to input */
  canwaitonipc=LCRZO_TRUE;
  canwaitonreadpipe=LCRZO_TRUE;
  while(LCRZO_TRUE)
  { if ( canwaitonipc && canwaitonreadpipe )
    { ret=lcrzo_wait_wait4(&waitipc, &waitipcend, &waitchildend, &waitreadpipe,
			   LCRZO_WAIT_INFINITE,
			   &eventipc, &eventipcend, &eventchildend,
			   &eventreadpipe);
#define SHARED_IPCEXEC_DEBUG 0
#if SHARED_IPCEXEC_DEBUG==1
      printf("A: %d %d %d %d\n", eventipc, eventreadpipe, eventipcend,
	     eventchildend);
#endif
    }
    else if ( canwaitonipc )
    { ret=lcrzo_wait_wait3(&waitipc, &waitipcend, &waitchildend,
			   LCRZO_WAIT_INFINITE,
			   &eventipc, &eventipcend, &eventchildend);
#if SHARED_IPCEXEC_DEBUG==1
      printf("B: %d %d %d\n", eventipc, eventipcend, eventchildend);
#endif
    }
    else if ( canwaitonreadpipe )
    { ret=lcrzo_wait_wait3(&waitipcend, &waitchildend, &waitreadpipe,
			   LCRZO_WAIT_INFINITE,
			   &eventipcend, &eventchildend, &eventreadpipe);
#if SHARED_IPCEXEC_DEBUG==1
      printf("C: %d %d %d\n", eventreadpipe, eventipcend, eventchildend);
#endif
    }
    else 
    { 
#if SHARED_IPCEXEC_DEBUG==1
      printf("D\n");
#endif
      break;
    }
    if (ret!=LCRZO_ERR_OK) break;

    if ( eventipc )
    { ret=lcrzo_ipc_readm(&pspmi->ipc, LCRZO_WAIT_INFINITE, &data, &datasize);
      if (ret==LCRZO_ERR_OK)
      { ret=lcrzo_fd2_write(&fd2writepipe, data, datasize);
        /* if the command doesn't allow stdin data, do not exit on error.
	   For example "cat > /tmp/f" is ok, but "ls /tmp" refuses stdin
	   data */
        if (ret==LCRZO_ERR_FUWRITE) ret=LCRZO_ERR_OK;
        lcrzo_er(lcrzo_data_free2(&data));
      }
      else if ( ret==LCRZO_ERR_OKDATAEND )
      { canwaitonipc=LCRZO_FALSE;
        lcrzo_epr(lcrzo_fd2_close(&fd2writepipe));
	ret=close(spmi.writepipe[0]);
	if (ret==-1) return(LCRZO_ERR_FUCLOSE);
	lcrzo_epr(lcrzo_wait_close(&waitipc));
	ret=LCRZO_ERR_OK;
	eventipc=LCRZO_FALSE;
      }
      else
      { break;
      }
    }

    if ( eventreadpipe )
    { ret=lcrzo_fd2_readm(&fd2readpipe, LCRZO_WAIT_INFINITE,&data,&datasize);
      if (ret==LCRZO_ERR_OK)
      { ret=lcrzo_ipc_write(&pspmi->ipc, data, datasize);
        lcrzo_er(lcrzo_data_free2(&data));
      }
      else if ( ret==LCRZO_ERR_OKDATAEND )
      { canwaitonreadpipe=LCRZO_FALSE;
        lcrzo_epr(lcrzo_wait_close(&waitreadpipe));
	lcrzo_epr(lcrzo_fd2_close(&fd2readpipe));
	lcrzo_epr(lcrzo_ipc_close_write(&pspmi->ipc));
	ret=LCRZO_ERR_OK;
	eventreadpipe=LCRZO_FALSE;
      }
      else
      { break;
      }
    }
    
    if ( eventipcend )
    { /* called on ipcexec_close, so force an immediate exit */
      lcrzo_er(shared_lcrzoex_debug_processotherexit_wanttowarn(LCRZO_FALSE));
      ret=lcrzo_process_other_exit(&child);
      lcrzo_er(shared_lcrzoex_debug_processotherexit_wanttowarn(LCRZO_TRUE));
      if (ret!=LCRZO_ERR_OK) break;
    }

    if ( eventchildend && !eventreadpipe )
    { canwaitonreadpipe=LCRZO_FALSE;
      lcrzo_epr(lcrzo_wait_close(&waitreadpipe));
      lcrzo_epr(lcrzo_fd2_close(&fd2readpipe));
      lcrzo_epr(lcrzo_ipc_close_write(&pspmi->ipc));
    }
  }
  if ( ret!=LCRZO_ERR_OK )
  { lcrzo_er(lcrzo_err_print(ret));
  }

  /* close resources */
  if (canwaitonipc)
  { lcrzo_epr(lcrzo_fd2_close(&fd2writepipe));
    ret=close(spmi.writepipe[0]);
    if (ret==-1) return(LCRZO_ERR_FUCLOSE);
    lcrzo_epr(lcrzo_wait_close(&waitipc));
  }
  if (canwaitonreadpipe)
  { lcrzo_epr(lcrzo_wait_close(&waitreadpipe));
    lcrzo_epr(lcrzo_fd2_close(&fd2readpipe));
    lcrzo_epr(lcrzo_ipc_close_write(&pspmi->ipc));
  }

  lcrzo_epr(lcrzo_wait_close(&waitipcend));
  lcrzo_epr(lcrzo_wait_close(&waitchildend));
  lcrzo_epr(lcrzo_ipc_close(&pspmi->ipc));
  ret=close(spmi.readpipe[0]);
  if (ret==-1) return(LCRZO_ERR_FUCLOSE);

  /* wait the child */
  lcrzo_er(shared_lcrzoex_debug_processotherexit_wanttowarn(LCRZO_FALSE));
  lcrzo_epr(lcrzo_process_other_exit(&child));
  lcrzo_er(shared_lcrzoex_debug_processotherexit_wanttowarn(LCRZO_TRUE));
  lcrzo_epr(lcrzo_process_other_wait_end(&child, LCRZO_WAIT_INFINITE, 
					 NULL, &childret));
  lcrzo_epr(lcrzo_process_close(&child));
  /* exec returns 127 on error */
  if (childret==127) childret=LCRZOEX_ERR_FU_EXECL;
  if ( childret!=LCRZO_ERR_OK && childret!=LCRZO_ERR_OKPROCESSABRUPTTERM )
  { if (childret!=LCRZOEX_ERR_FU_EXECL)
    { lcrzo_er(lcrzo_err_print(childret));
    }
    if (ret==LCRZO_ERR_OK) return(childret);
  }
  return(ret);
}
#elif defined LCRZODEF_SYSTEM_Windows
int shared_misc_ipcexec_child2(lcrzo_string command,
							   HANDLE *pstdinwrite, 
							   HANDLE *pstdoutread);
int shared_misc_ipcexec_child2(lcrzo_string command,
							   HANDLE *pstdinwrite, 
							   HANDLE *pstdoutread)
{ SECURITY_ATTRIBUTES sa; 
  PROCESS_INFORMATION pi; 
  STARTUPINFO si; 
  HANDLE stdinread, stdoutwrite;
  HANDLE stdinstd, stdoutstd, stderrstd; 
  BOOL err; 
  lcrzo_string cmd;
  lcrzo_uint32 systemversion;
 
  /* create needed pipes */
  sa.nLength = sizeof(SECURITY_ATTRIBUTES); 
  sa.bInheritHandle = TRUE; 
  sa.lpSecurityDescriptor = NULL;
  err=CreatePipe(pstdoutread, &stdoutwrite, &sa, 0);
  if ( !err ) return(LCRZO_ERR_FUCREATEPIPE);
  err=CreatePipe(&stdinread, pstdinwrite, &sa, 0);
  if ( !err ) return(LCRZO_ERR_FUCREATEPIPE);

  /* associate stdin and stdout */
  stdinstd=GetStdHandle(STD_INPUT_HANDLE); 
  err=SetStdHandle(STD_INPUT_HANDLE, stdinread);
  if ( !err ) return(LCRZOEX_ERR_FU_SETSTDHANDLE);
  stdoutstd=GetStdHandle(STD_OUTPUT_HANDLE); 
  err=SetStdHandle(STD_OUTPUT_HANDLE, stdoutwrite);
  if ( !err ) return(LCRZOEX_ERR_FU_SETSTDHANDLE);
  stderrstd=GetStdHandle(STD_ERROR_HANDLE); 
  err=SetStdHandle(STD_ERROR_HANDLE, stdoutwrite);
  if ( !err ) return(LCRZOEX_ERR_FU_SETSTDHANDLE);

  /* create the child process */
  ZeroMemory( &pi, sizeof(PROCESS_INFORMATION) );
  ZeroMemory( &si, sizeof(STARTUPINFO) );
  si.cb = sizeof(STARTUPINFO);

  /* select which command.exe to use */
  lcrzo_er(lcrzo_windows_getsystemversion(&systemversion, NULL, NULL, NULL));
  switch((int)systemversion)
  { case LCRZO_WINDOWS_SYSTEMVERSION_nt4:
    case LCRZO_WINDOWS_SYSTEMVERSION_2k:
    case LCRZO_WINDOWS_SYSTEMVERSION_xp:
    case LCRZO_WINDOWS_SYSTEMVERSION_net:
      lcrzo_er(lcrzo_string_initm_text("cmd.exe /c ", &cmd));
      break;  
    case LCRZO_WINDOWS_SYSTEMVERSION_31:
    case LCRZO_WINDOWS_SYSTEMVERSION_95:
    case LCRZO_WINDOWS_SYSTEMVERSION_98:
    case LCRZO_WINDOWS_SYSTEMVERSION_me:
	default:
	  lcrzo_er(lcrzo_string_initm_text("command.com /c ", &cmd));
      break;  
  }

  /* create it */
  lcrzo_er(lcrzo_string_appendm_text(command, &cmd));
  err=CreateProcess(NULL, cmd, NULL, NULL, TRUE, 0, NULL, NULL, &si, &pi);
  if ( !err ) return(LCRZOEX_ERR_FU_CREATEPROCESS);
  lcrzo_er(lcrzo_string_free2(&cmd));

  /* restore stdin and stdout handles */
  err=SetStdHandle(STD_INPUT_HANDLE, stdinstd);
  if ( !err ) return(LCRZOEX_ERR_FU_SETSTDHANDLE);
  err=SetStdHandle(STD_OUTPUT_HANDLE, stdoutstd);
  if ( !err ) return(LCRZOEX_ERR_FU_SETSTDHANDLE);
  err=SetStdHandle(STD_ERROR_HANDLE, stderrstd);
  if ( !err ) return(LCRZOEX_ERR_FU_SETSTDHANDLE);

  /* close unneeded handles */
  err=CloseHandle(stdoutwrite); 
  if ( !err ) return(LCRZO_ERR_FUCLOSEHANDLE);
  err=CloseHandle(stdinread); 
  if ( !err ) return(LCRZO_ERR_FUCLOSEHANDLE);

  return(LCRZO_ERR_OK); 
}
int shared_misc_ipcexec_child(lcrzo_ptr infos, 
			      lcrzo_int32 infossize, 
			      lcrzo_process *pparentprocess)
{ shared_priv_misc_ipcexec *pspmi=(shared_priv_misc_ipcexec *)infos;
  HANDLE hstdoutread, hstdinwrite;
  lcrzo_winhandle whstdoutread, whstdinwrite;
  lcrzo_wait waitipc, waitipcend, waitreadpipe;
  lcrzo_bool eventipc, eventipcend, eventreadpipe;
  lcrzo_bool canwaitonipc, canwaitonreadpipe;
  lcrzo_data data;
  lcrzo_int32 datasize;
  int ret;
  BOOL err;

  /* check for correct usage of this function */
  if (infossize!=sizeof(shared_priv_misc_ipcexec))
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);

  /* close parent process handle because we do not need it */
  lcrzo_epr(lcrzo_process_close(pparentprocess));

  /* set IPC */
  lcrzo_epr(lcrzo_ipc_currentprocessissideb(&pspmi->ipc));
  lcrzo_epr(lcrzo_ipc_currentprocessissideb(&pspmi->ipcchildend));
  lcrzo_epr(lcrzo_ipc_close_write(&pspmi->ipcchildend));

  /* call the command and associate stdin/stdout */
  lcrzo_epr(shared_misc_ipcexec_child2(pspmi->command, &hstdinwrite,
				       &hstdoutread));
  lcrzo_epr(lcrzo_winhandle_init_handle(hstdinwrite, &whstdinwrite));
  lcrzo_epr(lcrzo_winhandle_init_handle(hstdoutread, &whstdoutread));

  /* initialize wait */
  lcrzo_epr(lcrzo_wait_init_ipc_read(&pspmi->ipc, &waitipc));
  lcrzo_epr(lcrzo_wait_init_ipc_read(&pspmi->ipcchildend, &waitipcend));
  lcrzo_epr(lcrzo_wait_init_winhandle_read(&whstdoutread, &waitreadpipe));

  /* copy input to output, and output to input */
  ret=LCRZO_ERR_OK; /* for the compiler warning */
  canwaitonipc=LCRZO_TRUE;
  canwaitonreadpipe=LCRZO_TRUE;
  eventreadpipe=LCRZO_FALSE; /* for the compiler warning */
  eventipc=LCRZO_FALSE; /* for the compiler warning */
  while(LCRZO_TRUE)
  {
#define SHARED_IPCEXEC_DEBUG 0
#if SHARED_IPCEXEC_DEBUG==1
      printf("before wait\n");
#endif
    if ( canwaitonipc && canwaitonreadpipe )
    { ret=lcrzo_wait_wait3(&waitipc, &waitipcend, &waitreadpipe,
			   LCRZO_WAIT_INFINITE,
			   &eventipc, &eventipcend, &eventreadpipe);
#if SHARED_IPCEXEC_DEBUG==1
      printf("A: %d %d %d\n", eventipc, eventipcend, eventreadpipe);
#endif
    }
    else if ( canwaitonipc )
    { ret=lcrzo_wait_wait2(&waitipc, &waitipcend,
			   LCRZO_WAIT_INFINITE,
			   &eventipc, &eventipcend);
#if SHARED_IPCEXEC_DEBUG==1
      printf("B: %d %d\n", eventipc, eventipcend);
#endif
    }
    else if ( canwaitonreadpipe )
    { ret=lcrzo_wait_wait2(&waitipcend, &waitreadpipe,
			   LCRZO_WAIT_INFINITE,
			   &eventipcend, &eventreadpipe);
#if SHARED_IPCEXEC_DEBUG==1
      printf("C: %d %d\n", eventipcend, eventreadpipe);
#endif
    }
    else 
    { 
#if SHARED_IPCEXEC_DEBUG==1
      printf("D\n");
#endif
      break;
    }
    if (ret!=LCRZO_ERR_OK) break;

    if ( eventipc )
    { ret=lcrzo_ipc_readm(&pspmi->ipc, LCRZO_WAIT_INFINITE, &data, &datasize);
      if (ret==LCRZO_ERR_OK)
      { ret=lcrzo_winhandle_write(&whstdinwrite, data, datasize);
        /* if the command doesn't allow stdin data, do not exit on error.
	   For example "cat > /tmp/f" is ok, but "ls /tmp" refuses stdin
	   data */
        if (ret==LCRZO_ERR_FUWRITE) ret=LCRZO_ERR_OK;
        lcrzo_er(lcrzo_data_free2(&data));
      }
      else if ( ret==LCRZO_ERR_OKDATAEND )
      { canwaitonipc=LCRZO_FALSE;
        lcrzo_epr(lcrzo_winhandle_close(&whstdinwrite));
        err=CloseHandle(hstdinwrite); 
	      if ( !err ) return(LCRZO_ERR_FUCLOSEHANDLE);
        lcrzo_epr(lcrzo_wait_close(&waitipc));
        lcrzo_epr(lcrzo_ipc_close_read(&pspmi->ipc));
	      ret=LCRZO_ERR_OK;
	      eventipc=LCRZO_FALSE;
      }
      else
      { break;
      }
    }

    if ( eventreadpipe )
    { ret=lcrzo_winhandle_readm(&whstdoutread, LCRZO_WAIT_INFINITE,&data,&datasize);
      if (ret==LCRZO_ERR_OK)
      { ret=lcrzo_ipc_write(&pspmi->ipc, data, datasize);
        lcrzo_er(lcrzo_data_free2(&data));
      }
      else if ( ret==LCRZO_ERR_OKDATAEND )
      { canwaitonreadpipe=LCRZO_FALSE;
        lcrzo_epr(lcrzo_wait_close(&waitreadpipe));
	      lcrzo_epr(lcrzo_winhandle_close(&whstdoutread));
	      err=CloseHandle(hstdoutread); 
	      if ( !err ) return(LCRZO_ERR_FUCLOSEHANDLE);
        lcrzo_epr(lcrzo_ipc_close_write(&pspmi->ipc));
	      ret=LCRZO_ERR_OK;
	      eventreadpipe=LCRZO_FALSE;
      }
      else
      { break;
      }
    }

    if ( eventipcend )
    { /* called on ipcexec_close, so force an immediate exit */
      /* we do not call TerminateProcess because it's too dangerous */
      break;
    }
  }

  if ( ret!=LCRZO_ERR_OK )
  { lcrzo_er(lcrzo_err_print(ret));
  }

  /* close resources */
  if (canwaitonipc)
  { lcrzo_epr(lcrzo_winhandle_close(&whstdinwrite));
    err=CloseHandle(hstdinwrite); 
    if ( !err ) return(LCRZO_ERR_FUCLOSEHANDLE);
    lcrzo_epr(lcrzo_wait_close(&waitipc));
  }
  if (canwaitonreadpipe)
  { lcrzo_epr(lcrzo_wait_close(&waitreadpipe));
    lcrzo_epr(lcrzo_winhandle_close(&whstdoutread));
    err=CloseHandle(hstdoutread); 
    if ( !err ) return(LCRZO_ERR_FUCLOSEHANDLE);
  }

  lcrzo_epr(lcrzo_wait_close(&waitipcend));
  lcrzo_epr(lcrzo_ipc_close(&pspmi->ipc));
  lcrzo_epr(lcrzo_ipc_close(&pspmi->ipcchildend));
  return(ret);
}
#else
 #error "Unknown value for LCRZODEF_SYSTEM"
#endif
/*-------------------------------------------------------------*/
int shared_misc_ipcexec_init(lcrzo_conststring command,
			     shared_misc_ipcexec *psmi,
			     lcrzo_ipc *pipc)
{ shared_priv_misc_ipcexec spmi;

  /* copy command */
  lcrzo_epr(lcrzo_string_init_text(command, sizeof(spmi.command),
				   spmi.command));

  /* open communication IPC */
  lcrzo_epr(lcrzo_ipc_init(pipc));
  spmi.ipc=*pipc;
  lcrzo_epr(lcrzo_ipc_init(&psmi->ipcchildend));
  spmi.ipcchildend=psmi->ipcchildend;

  /* start child process */
  lcrzo_epr(lcrzo_process_other_init(&shared_misc_ipcexec_child, 
                                     (lcrzo_data)&spmi, sizeof(spmi), 
                                     &psmi->child));
  lcrzo_epr(lcrzo_ipc_currentprocessissidea(pipc));
  lcrzo_epr(lcrzo_ipc_currentprocessissidea(&psmi->ipcchildend));
  lcrzo_epr(lcrzo_ipc_close_read(&psmi->ipcchildend));

  return(LCRZO_ERR_OK);
}
int shared_misc_ipcexec_close(shared_misc_ipcexec *psmi)
{ int ret;

  ret=lcrzo_ipc_write_uint32(&psmi->ipcchildend, LCRZO_TRUE);
  if ( ret!=LCRZO_ERR_OK && ret!=LCRZO_ERR_LONOTOPENFORWRITING ) return(ret);
  lcrzo_epr(lcrzo_process_other_wait_end(&psmi->child, LCRZO_WAIT_INFINITE, 
					 NULL, &ret));
  lcrzo_epr(lcrzo_process_close(&psmi->child));
  lcrzo_epr(lcrzo_ipc_close(&psmi->ipcchildend));
  if (ret==LCRZO_ERR_OKPROCESSABRUPTTERM) ret=LCRZO_ERR_OK;
  return(ret);
}
int testshared_misc_ipcexec(void);
int testshared_misc_ipcexec(void)
{ lcrzo_data data;
  lcrzo_int32 datasize;
  lcrzo_bool writenotpossible, readnotpossible, userwantstoexit;
  lcrzo_file stdinfile;
  lcrzo_wait waitipc, waitstdin;
  lcrzo_bool eventipc, eventstdin;
  lcrzo_ipc ipc;
  shared_misc_ipcexec smi;
  lcrzo_int16 language;
  int ret;

  lcrzo_epr(lcrzo_global_get_language(&language));

  /* init ipcexec */
#if 1
  lcrzo_epr(shared_misc_ipcexec_init("dir c:\\", &smi, &ipc));
#elif 0
  lcrzo_epr(shared_misc_ipcexec_init("/usr/bin/xargs -ifile echo file", &smi, &ipc));
#elif 0
  lcrzo_epr(shared_misc_ipcexec_init("/bin/ls", &smi, &ipc));
#elif 0
  lcrzo_epr(shared_misc_ipcexec_init("cat /etc/passwd", &smi, &ipc));
#elif 1
  lcrzo_epr(shared_misc_ipcexec_init("catuu", &smi, &ipc));
#endif

  /* wait events */
  lcrzo_epr(shared_misc_userstop2_printmsg());
  lcrzo_epr(lcrzo_wait_init_ipc_read(&ipc, &waitipc));
  lcrzo_epr(lcrzo_file_init_fd(0, &stdinfile));
  lcrzo_epr(lcrzo_wait_init_file_read_line(&stdinfile, &waitstdin));

  /* main loop */
  readnotpossible=LCRZO_FALSE;
  writenotpossible=LCRZO_FALSE;
  userwantstoexit=LCRZO_FALSE;
  ret=LCRZO_ERR_OK;
  while(LCRZO_TRUE)
  { ret=lcrzo_wait_wait2(&waitipc, &waitstdin, LCRZO_WAIT_INFINITE,
                         &eventipc, &eventstdin);
    /*printf("eventipc=%d eventstdin=%d\n", eventipc, eventstdin);*/
    if (ret!=LCRZO_ERR_OK) break;
    /* an event from the ipc */
    if (eventipc)
    { /* look if data was received */
      ret=lcrzo_ipc_readm(&ipc, LCRZO_WAIT_INFINITE, &data, &datasize);
      if (ret!=LCRZO_ERR_OK)
      { readnotpossible=LCRZO_TRUE;
        break;
      }
      if ( datasize!=0 )
      { lcrzo_er(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_TEXT));
      }
      lcrzo_er(lcrzo_data_free2(&data));
    }
    /* an event from stdin */
    if (eventstdin)
    { ret=lcrzo_file_readm_line(&stdinfile, LCRZO_WAIT_INFINITE, 
				&data, &datasize);
      if (ret!=LCRZO_ERR_OK)
      { writenotpossible=LCRZO_TRUE;
        break;
      }
      ret=shared_misc_userstop2_test(data, datasize, &userwantstoexit);
      if ( ret!=LCRZO_ERR_OK || userwantstoexit )
      { lcrzo_er(lcrzo_data_free2(&data));
        break;
      }
      /* write data */
      ret=lcrzo_ipc_write(&ipc, data, datasize);
      if (ret!=LCRZO_ERR_OK)
      { writenotpossible=LCRZO_TRUE;
	lcrzo_er(lcrzo_data_free2(&data));
	break;
      }
      lcrzo_er(lcrzo_data_free2(&data));
      ret=lcrzo_ipc_write(&ipc, (lcrzo_constdata)"\r\n", 2);
      if (ret!=LCRZO_ERR_OK)
      { writenotpossible=LCRZO_TRUE;
	break;
      }
    }
  }

  /* print errors */
  if (readnotpossible)
  { if ( language==LCRZO_GLOBAL_LANG_FR )
      puts("\nLa lecture n'est plus possible.");
    else
      puts("\nWe cannot read anymore.");
  }
  else if (writenotpossible)
  { if ( language==LCRZO_GLOBAL_LANG_FR )
      puts("\nL'ecriture n'est plus possible.");
    else 
      puts("\nWe cannot write anymore.");
  }
  else
    { if (ret!=LCRZO_ERR_OK) { lcrzo_er(lcrzo_err_print(ret)); }
  }

  /* close */
  lcrzo_epr(lcrzo_wait_close(&waitipc));
  lcrzo_epr(lcrzo_wait_close(&waitstdin));
  lcrzo_epr(lcrzo_file_close(&stdinfile));
  lcrzo_epr(lcrzo_ipc_close(&ipc));
  lcrzo_epr(shared_misc_ipcexec_close(&smi));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_priv_misc_isadmin(lcrzo_bool *pyes);
int shared_priv_misc_isadmin(lcrzo_bool *pyes)
#if defined LCRZODEF_SYSTEM_Unix
{ lcrzo_bool isadmin; 

  isadmin=LCRZO_FALSE;
  if ( !getuid() || !geteuid() )
  { isadmin=LCRZO_TRUE;
  }
 
  if (pyes!=NULL) *pyes=isadmin;
  return(LCRZO_ERR_OK);
}
#elif defined LCRZODEF_SYSTEM_Windows
/* Taken from Q118626 */
{ HANDLE hToken;
  DWORD  dwStatus;
  DWORD  dwAccessMask;
  DWORD  dwAccessDesired;
  DWORD  dwACLSize;
  DWORD  dwStructureSize = sizeof(PRIVILEGE_SET);
  PACL   pACL            = NULL;
  PSID   psidAdmin       = NULL;
  BOOL   bReturn         = FALSE;
  PRIVILEGE_SET   ps;
  GENERIC_MAPPING GenericMapping;
  PSECURITY_DESCRIPTOR     psdAdmin           = NULL;
  SID_IDENTIFIER_AUTHORITY SystemSidAuthority = SECURITY_NT_AUTHORITY;
  lcrzo_uint32 systemversion;

  lcrzo_er(lcrzo_windows_getsystemversion(&systemversion, NULL, NULL, NULL));
  switch((int)systemversion)
  { case LCRZO_WINDOWS_SYSTEMVERSION_unknown:
    case LCRZO_WINDOWS_SYSTEMVERSION_31:
    case LCRZO_WINDOWS_SYSTEMVERSION_95:
    case LCRZO_WINDOWS_SYSTEMVERSION_98:
    case LCRZO_WINDOWS_SYSTEMVERSION_me:
      if (pyes!=NULL) *pyes=LCRZO_TRUE;
      return(LCRZO_ERR_OK);
  }

#define SHARED_PRIV_MISC_ISADMIN_ACCESS_READ  1
#define SHARED_PRIV_MISC_ISADMIN_ACCESS_WRITE 2
   
  __try {

    /* AccessCheck() requires an impersonation token. */
    ImpersonateSelf(SecurityImpersonation);
    
    if (!OpenThreadToken(GetCurrentThread(), TOKEN_QUERY, FALSE, 
			 &hToken)) {
      
      if (GetLastError() != ERROR_NO_TOKEN)
	__leave;
      
      /* If the thread does not have an access token, we'll 
	 examine the access token associated with the process.*/
      if (!OpenProcessToken(GetCurrentProcess(), TOKEN_QUERY, 
			    &hToken))
	__leave;
    }
    
    if (!AllocateAndInitializeSid(&SystemSidAuthority, 2, 
				  SECURITY_BUILTIN_DOMAIN_RID,
				  DOMAIN_ALIAS_RID_ADMINS,
				  0, 0, 0, 0, 0, 0, &psidAdmin))
      __leave;
    
    psdAdmin = LocalAlloc(LPTR, SECURITY_DESCRIPTOR_MIN_LENGTH);
    if (psdAdmin == NULL)
      __leave;
    
    if (!InitializeSecurityDescriptor(psdAdmin,
				      SECURITY_DESCRIPTOR_REVISION))
      __leave;
    
    /* Compute size needed for the ACL. */
    dwACLSize = sizeof(ACL) + sizeof(ACCESS_ALLOWED_ACE) +
      GetLengthSid(psidAdmin) - sizeof(DWORD);

    /* Allocate memory for ACL. */
    pACL = (PACL)LocalAlloc(LPTR, dwACLSize);
    if (pACL == NULL)
      __leave;
    
    /* Initialize the new ACL. */
    if (!InitializeAcl(pACL, dwACLSize, ACL_REVISION2))
      __leave;
    
    dwAccessMask= SHARED_PRIV_MISC_ISADMIN_ACCESS_READ | 
      SHARED_PRIV_MISC_ISADMIN_ACCESS_WRITE;
      
    /* Add the access-allowed ACE to the DACL. */
    if (!AddAccessAllowedAce(pACL, ACL_REVISION2,
			     dwAccessMask, psidAdmin))
      __leave;
    
    /* Set our DACL to the SD. */
    if (!SetSecurityDescriptorDacl(psdAdmin, TRUE, pACL, FALSE))
      __leave;

    /* AccessCheck is sensitive about what is in the SD; set
       the group and owner. */
    SetSecurityDescriptorGroup(psdAdmin, psidAdmin, FALSE);
    SetSecurityDescriptorOwner(psdAdmin, psidAdmin, FALSE);
    
    if (!IsValidSecurityDescriptor(psdAdmin))
      __leave;
    
    dwAccessDesired = SHARED_PRIV_MISC_ISADMIN_ACCESS_READ;
    
    /* Initialize GenericMapping structure even though we
       won't be using generic rights. */
    GenericMapping.GenericRead    = SHARED_PRIV_MISC_ISADMIN_ACCESS_READ;
    GenericMapping.GenericWrite   = SHARED_PRIV_MISC_ISADMIN_ACCESS_WRITE;
    GenericMapping.GenericExecute = 0;
    GenericMapping.GenericAll     = SHARED_PRIV_MISC_ISADMIN_ACCESS_READ |
      SHARED_PRIV_MISC_ISADMIN_ACCESS_WRITE;
    
    if (!AccessCheck(psdAdmin, hToken, dwAccessDesired, 
		     &GenericMapping, &ps, &dwStructureSize, &dwStatus, 
		     &bReturn)) {
      __leave;
    }

    RevertToSelf();
    
  } __finally {
    
    /* Cleanup */
    if (pACL) LocalFree(pACL);
    if (psdAdmin) LocalFree(psdAdmin);  
    if (psidAdmin) FreeSid(psidAdmin);
  }
  
  if (pyes!=NULL) *pyes=(lcrzo_bool)bReturn;
  return(LCRZO_ERR_OK);
}
#else
 #error "Unknown value for LCRZODEF_SYSTEM"
#endif

/*-------------------------------------------------------------*/
int shared_misc_checkadmin(int language)
{ lcrzo_bool isadmin; 

  lcrzo_epr(shared_priv_misc_isadmin(&isadmin));

  if (!isadmin)
  { printf("    / \\    /!\\ /!\\ /!\\ /!\\ /!\\ /!\\ /!\\ /!\\ /!\\");
    printf(" /!\\ /!\\ /!\\ /!\\ /!\\ /!\\\n");
    printf("   / | \\   ");
    if ( language==LCRZO_GLOBAL_LANG_FR )
      printf("Vous n'avez pas les droits administrateur.\n"); 
    else
      printf("Warning, you do not have administrator privileges.\n");
    printf("  /  o  \\  "); 
    if ( language==LCRZO_GLOBAL_LANG_FR )
      printf("Certains outils ne fonctionneront pas : sniff, spoof, etc.\n");
    else
      printf("Some functionalities might not work : sniff, spoof, etc.\n");
    printf(" /_______\\ /!\\ /!\\ /!\\ /!\\ /!\\ /!\\ /!\\ /!\\ /!\\");
    printf(" /!\\ /!\\ /!\\ /!\\ /!\\ /!\\\n");
  }

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
#ifdef __cplusplus
 extern "C" {
#endif
 int symlink(lcrzo_conststring from, lcrzo_conststring to);
#ifdef __cplusplus
}
#endif
int shared_misc_symlink(lcrzo_conststring oldpath, 
			lcrzo_conststring newpath)
#if defined LCRZODEF_SYSTEM_Unix
{ int ret;

  ret=symlink(oldpath, newpath);
  if (ret==-1) return(LCRZOEX_ERR_FU_SYMLINK);

  return(LCRZO_ERR_OK);
}
#elif defined LCRZODEF_SYSTEM_Windows
{ return(LCRZO_ERR_IEUNIMPLEMENTED);
  oldpath=oldpath;
  newpath=newpath;
}
#else
 #error "Unknown value for LCRZODEF_SYSTEM"
#endif

/*-------------------------------------------------------------*/
int shared_misc_string_initm_2ptrs(lcrzo_conststring pcfirstchar,
				   lcrzo_conststring pclastchar,
				   lcrzo_string *pstr)
{ lcrzo_int32 size;

  if ( pclastchar<pcfirstchar-1 ) 
    return(LCRZO_ERR_POINFHIGHERSUP);

  if ( pclastchar==pcfirstchar-1)
  { return(lcrzo_string_initm_text("", pstr));
  }

  if ( pstr!=NULL )
  { size=pclastchar-pcfirstchar+1;
    lcrzo_er(lcrzo_string_alloc(size, pstr));
    memcpy(*pstr, pcfirstchar, size);
    (*pstr)[size]='\0';
  }

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_misc_beep(void)
{
#if defined LCRZODEF_SYSTEM_Unix
  printf("%c", 7);
#elif defined LCRZODEF_SYSTEM_Windows
  Beep(4000, 200);
#else
 #error "Unknown value for LCRZODEF_SYSTEM"
#endif
  return(LCRZO_ERR_OK);
}

