#ifndef LCRZOEX_PARA_INCLUDED
#define LCRZOEX_PARA_INCLUDED 1

/*-------------------------------------------------------------*/
/* Those functions are equivalent to lcrzo_device_init, 
   lcrzo_etha_init_eths, lcrzo_ipl_init_hs, etc. but are more
   powerful.
   For example, if a tool needs an Ethernet address, we have
   to use :
     ./tool aa:bb:cc:dd:ee:ff
   With this module, we can also say :
     ./tool e[aa:bb:cc:dd:ee:ff]e : does nothing more, except
                                    saying the input parameter is
                                    Ethernet(e) and has to be converted
                                    to Ethernet(e)
     ./tool i[1.2.3.4]e : the input is the IP(i) address 1.2.3.4, and
                          it has to be converted to Ethernet(e)
                          This is equivalent to say "Ethernet address
                          of the IP address 1.2.3.4"
     ./tool h[server]e : the input is the hostname(h) server, and
                         it has to be converted to Ethernet(e)
                         This is equivalent to say "Ethernet address
                         of the hostname server"
     ./tool dn[eth0]e : the input is the device(d) eth0, and
                        it has to be converted to Ethernet(e)
                        This is equivalent to say "Ethernet address
                        of the device eth0"
     ./tool e[]e : the user will be asked to select an Ethernet address
     ./tool d[]e : the user will be asked to select a device name, and
                   the Ethernet address of this device will be used
     etc.
   To sum up, the syntax is :
       from[fromparameter]to
     From and to can be :
       d  : device
       dn : device number
       e  : Ethernet address
       er : Ethernet range
       el : Ethernet list
       h  : hostname
       i  : IP address
       ir : IP range
       il : IP list
       p  : port number
       pr : port range
       pl : port list
       n  : number
       s  : string
       sp : password string

 */

/*-------------------------------------------------------------*/
int shared_para_device_init(lcrzo_conststring userstr,
			    lcrzo_conststring message,
			    lcrzo_conststring defaultstr,
			    lcrzo_device device);
int shared_para_etha_init(lcrzo_conststring userstr,
			  lcrzo_conststring message,
			  lcrzo_conststring defaultstr,
			  lcrzo_etha etha);
int shared_para_ethrange_init(lcrzo_conststring userstr,
			      lcrzo_conststring message,
			      lcrzo_conststring defaultstr,
			      lcrzo_ethrange ethrange);
int shared_para_ethlist_init(lcrzo_conststring userstr,
			     lcrzo_conststring message,
			     lcrzo_conststring defaultstr,
			     lcrzo_string *pethliststr);
int shared_para_hs_init(lcrzo_conststring userstr,
			lcrzo_conststring message,
			lcrzo_conststring defaultstr,
			lcrzo_hs hsout);
int shared_para_ipl_init(lcrzo_conststring userstr,
			 lcrzo_conststring message,
			 lcrzo_conststring defaultstr,
			 lcrzo_ipl *pipl);
int shared_para_iprange_init(lcrzo_conststring userstr,
			     lcrzo_conststring message,
			     lcrzo_conststring defaultstr,
			     lcrzo_iprange iprange);
int shared_para_iplist_init(lcrzo_conststring userstr,
			    lcrzo_conststring message,
			    lcrzo_conststring defaultstr,
			    lcrzo_string *pipliststr);
int shared_para_port_init(lcrzo_conststring userstr,
			  lcrzo_conststring message,
			  lcrzo_uint16 minport,
			  lcrzo_uint16 maxport,
			  lcrzo_uint16 defaultport,
			  lcrzo_uint16 *pport);
int shared_para_portrange_init(lcrzo_conststring userstr,
			       lcrzo_conststring message,
			       lcrzo_conststring defaultstr,
			       lcrzo_portrange portrange);
int shared_para_portlist_init(lcrzo_conststring userstr,
			      lcrzo_conststring message,
			      lcrzo_conststring defaultstr,
			      lcrzo_string *pportliststr);
int shared_para_uint32_init(lcrzo_conststring userstr,
			    lcrzo_conststring message,
			    lcrzo_uint32 minvalue,
			    lcrzo_uint32 maxvalue,
			    lcrzo_uint32 defaultnumber,
			    lcrzo_uint32 *pnumber);
int shared_para_string_init(lcrzo_conststring userstr,
			    lcrzo_conststring message,
			    lcrzo_conststring defaultstr,
			    lcrzo_string *pstr);

/*-------------------------------------------------------------*/
int shared_para_int32_init(lcrzo_conststring userstr,
			   lcrzo_conststring message,
			   lcrzo_int32 minvalue,
			   lcrzo_int32 maxvalue,
			   lcrzo_int32 defaultnumber,
			   lcrzo_int32 *pnumber);
int shared_para_uint16_init(lcrzo_conststring userstr,
			    lcrzo_conststring message,
			    lcrzo_uint16 minvalue,
			    lcrzo_uint16 maxvalue,
			    lcrzo_uint16 defaultnumber,
			    lcrzo_uint16 *pnumber);
int shared_para_int16_init(lcrzo_conststring userstr,
			   lcrzo_conststring message,
			   lcrzo_int16 minvalue,
			   lcrzo_int16 maxvalue,
			   lcrzo_int16 defaultnumber,
			   lcrzo_int16 *pnumber);
int shared_para_uint8_init(lcrzo_conststring userstr,
			   lcrzo_conststring message,
			   lcrzo_uint8 minvalue,
			   lcrzo_uint8 maxvalue,
			   lcrzo_uint8 defaultnumber,
			   lcrzo_uint8 *pnumber);
int shared_para_int8_init(lcrzo_conststring userstr,
			  lcrzo_conststring message,
			  lcrzo_int8 minvalue,
			  lcrzo_int8 maxvalue,
			  lcrzo_int8 defaultnumber,
			  lcrzo_int8 *pnumber);
int shared_para_bool_init(lcrzo_conststring userstr,
			  lcrzo_conststring message,
			  lcrzo_bool defaultnumber,
			  lcrzo_bool *pnumber);

/*-------------------------------------------------------------*/
int shared_para_device_init_src(lcrzo_conststring userstr,
				lcrzo_device device);
int shared_para_device_init_listen(lcrzo_conststring userstr,
				   lcrzo_device device);
int shared_para_etha_init_src(lcrzo_conststring userstr,
			      lcrzo_etha etha);
int shared_para_etha_init_dst(lcrzo_conststring userstr,
			      lcrzo_etha etha);
int shared_para_etha_init_listen(lcrzo_conststring userstr,
				 lcrzo_etha etha);
int shared_para_hs_init_src(lcrzo_conststring userstr,
			    lcrzo_hs hs);
int shared_para_hs_init_dst(lcrzo_conststring userstr,
			    lcrzo_hs hs);
int shared_para_hs_init_listen(lcrzo_conststring userstr,
			       lcrzo_hs hs);
int shared_para_ipl_init_src(lcrzo_conststring userstr,
			     lcrzo_ipl *pipl);
int shared_para_ipl_init_dst(lcrzo_conststring userstr,
			     lcrzo_ipl *pipl);
int shared_para_ipl_init_listen(lcrzo_conststring userstr,
				lcrzo_ipl *pipl);
int shared_para_port_init_src(lcrzo_conststring userstr,
			      lcrzo_uint16 *pport);
int shared_para_port_init_dst(lcrzo_conststring userstr,
			      lcrzo_uint16 defaultport,
			      lcrzo_uint16 *pport);
int shared_para_port_init_listen(lcrzo_conststring userstr,
				 lcrzo_uint16 *pport);
int shared_para_uint8_init_ipprotocol(lcrzo_conststring userstr,
				      lcrzo_uint8 *pproto);
int shared_para_uint16_init_ipfragoctosize(lcrzo_conststring userstr,
					   lcrzo_uint16 *pfragoctosize);
int shared_para_bool_init_tcpsyn(lcrzo_conststring userstr,
				 lcrzo_bool *psyn);
int shared_para_bool_init_tcpack(lcrzo_conststring userstr,
				 lcrzo_bool *pack);
int shared_para_bool_init_tcprst(lcrzo_conststring userstr,
				 lcrzo_bool *prst);
int shared_para_bool_init_tcpfin(lcrzo_conststring userstr,
				 lcrzo_bool *pfin);
int shared_para_uint32_init_tcpseqnum(lcrzo_conststring userstr,
				      lcrzo_uint32 *pnumber);
int shared_para_uint32_init_tcpacknum(lcrzo_conststring userstr,
				      lcrzo_uint32 *pnumber);
int shared_para_uint8_init_icmptype(lcrzo_conststring userstr,
				    lcrzo_uint8 *ptype);
int shared_para_uint8_init_icmpcode(lcrzo_conststring userstr,
				    lcrzo_uint8 *pcode);
int shared_para_string_init_filelocal(lcrzo_conststring userstr,
				      lcrzo_string *pstr);
int shared_para_string_init_filedistant(lcrzo_conststring userstr,
					lcrzo_string *pstr);
int shared_para_string_init_dirlocal(lcrzo_conststring userstr,
				     lcrzo_string *pstr);
int shared_para_string_init_dirdistant(lcrzo_conststring userstr,
				       lcrzo_string *pstr);
int shared_para_string_init_snifffilter(lcrzo_conststring userstr,
					lcrzo_string *pstr);
int shared_para_port_init_ftpdatalocal(lcrzo_conststring userstr,
				       lcrzo_uint16 *pport);
int shared_para_string_init_filein(lcrzo_conststring userstr,
				   lcrzo_string *pstr);
int shared_para_string_init_fileout(lcrzo_conststring userstr,
				    lcrzo_string *pstr);

#endif
