/*
		                 LCRZOEX
                             Network toolbox
             Copyright(c) 1999,2000,2001,2002 Laurent Constantin
                                  -----

  Main server    : http://www.laurentconstantin.com/
  Backup servers : http://go.to/laurentconstantin/
                   http://laurentconstantin.est-la.com/
                   http://laurentconstantin.free.fr/
                   http://membres.lycos.fr/lauconstantin/
  [my current email address is on the web servers]

                                  -----
  This file is part of Lcrzoex.

  Lcrzoex is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Lcrzoex is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details (http://www.gnu.org/).

------------------------------------------------------------------------
*/

#include <lcrzo.h>
#include "../lcrzoex.h"

/*-------------------------------------------------------------*/
int shared_router_etha_init_ipl(lcrzo_ipl ipl, lcrzo_etha etha)
{ int ret;

  ret=lcrzo_etha_init_ipl(ipl, etha);
  if (ret!=LCRZO_ERR_OK)
  { lcrzo_er(lcrzo_ipl_print("Could not resolve Ethernet address for ", ipl,
			     " - ignore this packet.\n"));
    return(ret);
  }
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_router_forcenewchecksum(lcrzo_hdrlip hdrip,
				   lcrzo_constipopt ipopt,
				   lcrzo_uint8 ipoptsize,
				   lcrzo_constdata ipdata,
				   lcrzo_uint16 ipdatasize,
				   lcrzo_data *pethdata,
				   lcrzo_int32 *pethdatasize)
{ int ret;

  /* IP checksum */
  hdrip.check=LCRZO_HDRLIP_CHECK_DEFVAL;

  /* when we masquerade, we also have to recompute TCP checksum */
  if (hdrip.protocol==LCRZO_HDRLIP_PROTOCOL_TCP)
  { lcrzo_hdrltcp hdrtcp;
    lcrzo_tcpopt tcpopt;
    lcrzo_uint8 tcpoptsize;
    lcrzo_data tcpdata;
    lcrzo_uint16 tcpdatasize;
    ret=lcrzo_packet_decodem_tcpoptdata(ipdata, ipdatasize,
					&hdrtcp, tcpopt, &tcpoptsize,
					&tcpdata, &tcpdatasize);
    if (ret==LCRZO_ERR_OK)
    { hdrtcp.check=LCRZO_HDRLTCP_CHECK_DEFVAL; /* force compute*/
      ret=lcrzo_packet_initm_ipopttcpoptdata(hdrip, ipopt, ipoptsize,
					     hdrtcp, tcpopt, tcpoptsize,
					     tcpdata, tcpdatasize,
					     pethdata, pethdatasize);
      lcrzo_er(lcrzo_data_free2(&tcpdata));
      if (ret==LCRZO_ERR_OK)
      { /* Ethernet packet is constructed */
        return(LCRZO_ERR_OK);
      }
    }
  }

  /* when we masquerade, we also have to recompute UDP checksum */
  if (hdrip.protocol==LCRZO_HDRLIP_PROTOCOL_UDP)
  { lcrzo_hdrludp hdrudp;
    lcrzo_data udpdata;
    lcrzo_uint16 udpdatasize;
    ret=lcrzo_packet_decodem_udpdata(ipdata, ipdatasize,
				     &hdrudp, &udpdata, &udpdatasize);
    if (ret==LCRZO_ERR_OK)
    { hdrudp.check=LCRZO_HDRLUDP_CHECK_DEFVAL; /* force compute*/
      ret=lcrzo_packet_initm_ipoptudpdata(hdrip, ipopt, ipoptsize,
					  hdrudp, udpdata, udpdatasize,
					  pethdata, pethdatasize);
      lcrzo_er(lcrzo_data_free2(&udpdata));
      if (ret==LCRZO_ERR_OK)
      { /* Ethernet packet is constructed */
        return(LCRZO_ERR_OK);
      }
    }
  }

  /* if we are here, packet is not TCP nor UDP, or an error occured.
     we simply recompute IP checksum */
  lcrzo_epr(lcrzo_packet_initm_ipoptdata(hdrip, ipopt, ipoptsize,
					 ipdata, ipdatasize,
					 pethdata, pethdatasize));
  return(LCRZO_ERR_OK);
}


/*-------------------------------------------------------------*/
int shared_router_masq(lcrzo_constdata sniffdata,
		       lcrzo_int32 sniffdatasize,
		       lcrzo_spoof *pspoofclient,
		       lcrzo_spoof *pspoofgateway,
		       lcrzo_constdevice device,
		       lcrzo_ipl ipclient,
		       lcrzo_constetha ethins,
		       lcrzo_ipl ipins,
		       lcrzo_constetha ethmasq,
		       lcrzo_ipl ipmasq,
		       lcrzo_ipl ipgw,
		       lcrzo_ipl iplaninf,
		       lcrzo_ipl iplansup)
{
  /* answer to ARP request */
  lcrzo_epr(lcrzo_virtual_answer_alive(sniffdata, sniffdatasize, 
				       device, ethins, ipins));
  lcrzo_epr(lcrzo_virtual_answer_arp(sniffdata, sniffdatasize, 
				     device, ethmasq, ipmasq));
    
  /* packets destinated to ethins, but not for ipins, are 
     masqueraded as ipmask/ethmask and forwarded to the gateway
     or to the LAN */
  lcrzo_epr(shared_router_masqoutput(sniffdata, sniffdatasize,
				     pspoofgateway,
				     ipclient, ethins, ipins,
				     device, ethmasq, ipmasq, 
				     ipgw, iplaninf, iplansup));

  /* packets destinated to ethmasq and ipmasq, are 
     forwarded to the client */
  lcrzo_epr(shared_router_masqinput(sniffdata, sniffdatasize,
				    pspoofclient,
				    ethmasq, ipmasq,
				    device, ethins, ipclient));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_router_masqoutput(lcrzo_constdata sniffdata,
			     lcrzo_int32 sniffdatasize,
			     lcrzo_spoof *pspoof,
			     lcrzo_ipl ipclient,
			     lcrzo_constetha ethins,
			     lcrzo_ipl ipins,
			     lcrzo_constdevice device,
			     lcrzo_constetha ethmasq,
			     lcrzo_ipl ipmasq,
			     lcrzo_ipl ipgw,
			     lcrzo_ipl iplaninf,
			     lcrzo_ipl iplansup)
{ lcrzo_data ipdata, ethdata;
  lcrzo_uint16 ipdatasize;
  lcrzo_int32 ethdatasize;
  lcrzo_ipopt ipopt;
  lcrzo_uint8 ipoptsize;
  lcrzo_etha ethdst;
  lcrzo_hdrleth hdreth;
  lcrzo_hdrlip hdrip;
  lcrzo_bool equal;
  int ret;

  /* try to decode EthIP */
  ret=lcrzo_packet_decodem_ethipoptdata(sniffdata, sniffdatasize,
					&hdreth, &hdrip, ipopt, &ipoptsize,
					&ipdata, &ipdatasize);
  if (ret!=LCRZO_ERR_OK)
    return(LCRZO_ERR_OK);

  /* only masquerade packets from ipclient, destinated to ethins, but not
     to ipins */
  lcrzo_er(lcrzo_etha_equal2(hdreth.dst, ethins, &equal));
  if ( ! ( hdrip.saddr==ipclient && equal && hdrip.daddr!=ipins ) )
  { lcrzo_er(lcrzo_data_free2(&ipdata));
    return(LCRZO_ERR_OK);
  }

  /* check if we use the gateway, or if destination is on the LAN */
  if ( ( iplaninf!=0 || iplansup!=0 ) &&
       hdrip.daddr>=iplaninf && hdrip.daddr<=iplansup )
  { ret=shared_router_etha_init_ipl(hdrip.daddr, ethdst);
    if (ret!=LCRZO_ERR_OK)
    { lcrzo_er(lcrzo_data_free2(&ipdata));
      return(LCRZO_ERR_OK);
    }
  }
  else
  { ret=shared_router_etha_init_ipl(ipgw, ethdst);
    if (ret!=LCRZO_ERR_OK)
    { lcrzo_er(lcrzo_data_free2(&ipdata));
      return(LCRZO_ERR_OK);
    }
  }

  /* update header values */
  memcpy(hdreth.src, ethmasq, LCRZO_ETHA_MAXBYTES);
  memcpy(hdreth.dst, ethdst, LCRZO_ETHA_MAXBYTES);
  hdrip.saddr=ipmasq;

  /* force checksum computing, and create new packets */
  ret=shared_router_forcenewchecksum(hdrip, ipopt, ipoptsize, 
				     ipdata, ipdatasize,
				     &ethdata, &ethdatasize);
  lcrzo_er(lcrzo_data_free2(&ipdata));
  if (ret!=LCRZO_ERR_OK) return(ret);

  /* spoof */
  ret=lcrzo_spoof_ethdata(pspoof, device, hdreth,
			  ethdata, (lcrzo_uint16)ethdatasize);

  lcrzo_er(lcrzo_data_free2(&ethdata));
  return(ret);
}

/*-------------------------------------------------------------*/
int shared_router_masqinput(lcrzo_constdata sniffdata,
			    lcrzo_int32 sniffdatasize,
			    lcrzo_spoof *pspoof,
			    lcrzo_constetha ethmasq,
			    lcrzo_ipl ipmasq,
			    lcrzo_constdevice device,
			    lcrzo_constetha ethins,
			    lcrzo_ipl ipclient)
{ lcrzo_data ipdata, ethdata;
  lcrzo_uint16 ipdatasize;
  lcrzo_int32 ethdatasize;
  lcrzo_ipopt ipopt;
  lcrzo_uint8 ipoptsize;
  lcrzo_hdrleth hdreth;
  lcrzo_hdrlip hdrip;
  lcrzo_bool ethequal;
  int ret;

  /* try to decode EthIP */
  ret=lcrzo_packet_decodem_ethipoptdata(sniffdata, sniffdatasize,
					&hdreth, &hdrip, ipopt, &ipoptsize,
					&ipdata, &ipdatasize);
  if (ret!=LCRZO_ERR_OK)
    return(LCRZO_ERR_OK);

  /* packets destinated to ethmasq and ipmasq, are 
     forwarded to the client */
  lcrzo_er(lcrzo_etha_equal2(hdreth.dst, ethmasq, &ethequal));
  if ( ! ( ethequal && hdrip.daddr==ipmasq ) )
  { lcrzo_er(lcrzo_data_free2(&ipdata));
    return(LCRZO_ERR_OK);
  }

  /* update header values */
  memcpy(hdreth.src, ethins, LCRZO_ETHA_MAXBYTES);
  ret=shared_router_etha_init_ipl(ipclient, hdreth.dst);
  if (ret!=LCRZO_ERR_OK)
  { lcrzo_er(lcrzo_data_free2(&ipdata));
    return(LCRZO_ERR_OK);
  }
  hdrip.daddr=ipclient;

  /* force checksum computing, and create new packets */
  ret=shared_router_forcenewchecksum(hdrip, ipopt, ipoptsize, 
                                     ipdata, ipdatasize,
                                     &ethdata, &ethdatasize);
  lcrzo_er(lcrzo_data_free2(&ipdata));
  if (ret!=LCRZO_ERR_OK) return(ret);

  /* spoof */
  ret=lcrzo_spoof_ethdata(pspoof, device, hdreth,
			  ethdata, (lcrzo_uint16)ethdatasize);
  lcrzo_er(lcrzo_data_free2(&ethdata));
  return(ret);
}


/*-------------------------------------------------------------*/
int shared_router_forward(lcrzo_constdata sniffdata,
			  lcrzo_int32 sniffdatasize,
			  lcrzo_spoof *pspoofclient,
			  lcrzo_spoof *pspoofgateway,
			  lcrzo_constdevice device,
			  lcrzo_constetha ethins,
			  lcrzo_ipl ipins,
			  lcrzo_constetha ethout,
			  lcrzo_ipl ipout,
			  lcrzo_ipl ipgw,
			  lcrzo_ipl iplaninf,
			  lcrzo_ipl iplansup)
{
  /* answer to ARP request */
  lcrzo_epr(lcrzo_virtual_answer_alive(sniffdata, sniffdatasize, 
				       device, ethins, ipins));
  lcrzo_epr(lcrzo_virtual_answer_alive(sniffdata, sniffdatasize, 
				       device, ethout, ipout));
    
  /* packets destinated to ethins, but not for ipins, are 
     forwarded to the gateway or to the LAN */
  lcrzo_epr(shared_router_forwardoutput(sniffdata, sniffdatasize,
					pspoofgateway,
					ethins, ipins,
					device, ethout,
					ipgw, iplaninf, iplansup));

  /* packets destinated to ethout, but not for ipout, are 
     forwarded to the client */
  lcrzo_epr(shared_router_forwardinput(sniffdata, sniffdatasize,
				       pspoofclient,
				       ethout, ipout,
				       device, ethins));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_router_forwardoutput(lcrzo_constdata sniffdata,
				lcrzo_int32 sniffdatasize,
				lcrzo_spoof *pspoof,
				lcrzo_constetha ethins,
				lcrzo_ipl ipins,
				lcrzo_constdevice device,
				lcrzo_constetha ethout,
				lcrzo_ipl ipgw,
				lcrzo_ipl iplaninf,
				lcrzo_ipl iplansup)
{ lcrzo_data ipdata;
  lcrzo_uint16 ipdatasize;
  lcrzo_ipopt ipopt;
  lcrzo_uint8 ipoptsize;
  lcrzo_etha ethdst;
  lcrzo_hdrleth hdreth;
  lcrzo_hdrlip hdrip;
  lcrzo_bool equal;
  int ret;

  /* try to decode EthIP */
  ret=lcrzo_packet_decodem_ethipoptdata(sniffdata, sniffdatasize,
					&hdreth, &hdrip, ipopt, &ipoptsize,
					&ipdata, &ipdatasize);
  if (ret!=LCRZO_ERR_OK)
    return(LCRZO_ERR_OK);

  /* packets destinated to ethins, but not for ipins, are 
     forwarded to the gateway or to the LAN */
  lcrzo_er(lcrzo_etha_equal2(hdreth.dst, ethins, &equal));
  if ( ! ( equal && hdrip.daddr!=ipins ) )
  { lcrzo_er(lcrzo_data_free2(&ipdata));
    return(LCRZO_ERR_OK);
  }

  /* check if we use the gateway, or if destination is on the LAN */
  if ( ( iplaninf!=0 || iplansup!=0 ) &&
       hdrip.daddr>=iplaninf && hdrip.daddr<=iplansup )
  { ret=shared_router_etha_init_ipl(hdrip.daddr, ethdst);
    if (ret!=LCRZO_ERR_OK)
    { lcrzo_er(lcrzo_data_free2(&ipdata));
      return(LCRZO_ERR_OK);
    }
  }
  else
  { ret=shared_router_etha_init_ipl(ipgw, ethdst);
    if (ret!=LCRZO_ERR_OK)
    { lcrzo_er(lcrzo_data_free2(&ipdata));
      return(LCRZO_ERR_OK);
    }
  }

  /* update header values */
  memcpy(hdreth.src, ethout, LCRZO_ETHA_MAXBYTES);
  memcpy(hdreth.dst, ethdst, LCRZO_ETHA_MAXBYTES);

  /* force checksum computing */
  hdrip.check=LCRZO_HDRLIP_CHECK_DEFVAL;

  /* spoof */
  ret=lcrzo_spoof_ethipoptdata(pspoof, device, hdreth, 
			       hdrip, ipopt, ipoptsize,
			       ipdata, ipdatasize);
  lcrzo_er(lcrzo_data_free2(&ipdata));

  return(ret);
}

/*-------------------------------------------------------------*/
int shared_router_forwardinput(lcrzo_constdata sniffdata,
			       lcrzo_int32 sniffdatasize,
			       lcrzo_spoof *pspoof,
			       lcrzo_constetha ethout,
			       lcrzo_ipl ipout,
			       lcrzo_constdevice device,
			       lcrzo_constetha ethins)
{ lcrzo_data ipdata;
  lcrzo_uint16 ipdatasize;
  lcrzo_ipopt ipopt;
  lcrzo_uint8 ipoptsize;
  lcrzo_hdrleth hdreth;
  lcrzo_hdrlip hdrip;
  lcrzo_bool equal;
  int ret;

  /* try to decode EthIP */
  ret=lcrzo_packet_decodem_ethipoptdata(sniffdata, sniffdatasize,
					&hdreth, &hdrip, ipopt, &ipoptsize,
					&ipdata, &ipdatasize);
  if (ret!=LCRZO_ERR_OK)
    return(LCRZO_ERR_OK);

  /* packets destinated to ethout, but not for ipout, are 
     forwarded to the client */
  lcrzo_er(lcrzo_etha_equal2(hdreth.dst, ethout, &equal));
  if ( ! ( equal && hdrip.daddr!=ipout ) )
  { lcrzo_er(lcrzo_data_free2(&ipdata));
    return(LCRZO_ERR_OK);
  }

  /* update header values */
  memcpy(hdreth.src, ethins, LCRZO_ETHA_MAXBYTES);
  ret=shared_router_etha_init_ipl(hdrip.daddr, hdreth.dst);
  if (ret!=LCRZO_ERR_OK)
  { lcrzo_er(lcrzo_data_free2(&ipdata));
    return(LCRZO_ERR_OK);
  }

  /* force checksum computing */
  hdrip.check=LCRZO_HDRLIP_CHECK_DEFVAL;

  /* spoof */
  ret=lcrzo_spoof_ethipoptdata(pspoof, device, hdreth, 
			       hdrip, ipopt, ipoptsize,
			       ipdata, ipdatasize);
  lcrzo_er(lcrzo_data_free2(&ipdata));
	
  return(LCRZO_ERR_OK);
}

