#ifndef LCRZOEX_ROUTER_INCLUDED
#define LCRZOEX_ROUTER_INCLUDED 1

/*-------------------------------------------------------------*/
/* General functions */
int shared_router_etha_init_ipl(lcrzo_ipl ipl, lcrzo_etha etha);
int shared_router_forcenewchecksum(lcrzo_hdrlip hdrip,
				   lcrzo_constipopt ipopt,
				   lcrzo_uint8 ipoptsize,
				   lcrzo_constdata ipdata,
				   lcrzo_uint16 ipdatasize,
				   lcrzo_data *pethdata,
				   lcrzo_int32 *pethdatasize);

/*-------------------------------------------------------------*/
/*                 MASQUERADING ROUTER
 Principe :
   Clients using the virtual router are masqueraded as ipmasq :
   ________          ___________                    __________
   |      |    ethins|  lcrzo  |ethmasq             |  true  |
   |client|----------| virtual |--------------------|gateway |----- web
   |______|     ipins|_router__|ipmasq      iprouter|________|
   As with every masqueraded network, some protocols (FTP, H.323, etc.)
   cannot work because IP addresses are used inside the flow. You can 
   write a proxy if you want these to work.

 Example :
   ________                                         __________
   |      |192.168.1.1                 192.168.1.254|  true  |
   |client|--                                   ,---|gateway |----- web
   |______| |                                   |   |________|
            |            ___________            |
            | a:a:a:a:a:a|  lcrzo  |b:b:b:b:b:b |
            -------------| virtual |------------'
              192.168.1.3|_router__|192.168.1.4

 On this scheme, the virtual router is presented with 2 boards ; really,
 there is only one network board :
   ________                                        __________
   |      |192.168.1.1                192.168.1.254|  true  |
   |client|----------------------------------------|gateway |----- web
   |______|                  |                     |________|
                 a:a:a:a:a:a | b:b:b:b:b:b
                 192.168.1.3 | 192.168.1.4
                             | eth0
                        _____|_____               
                        |  lcrzo  |
                        | virtual |        
                        |_router__|
 
 The addresses 192.168.1.3, 192.168.1.4, a:a:a:a:a:a and b:b:b:b:b:b
 must not be used by other computers on the network.

 Case 1 : this router is only used to reach computers on the web. We use :
  lcrzoex num eth0 192.168.1.1 a:a:a:a:a:a 192.168.1.3 \
                   b:b:b:b:b:b 192.168.1.4 \
                   192.168.1.254
  The address of the default gateway is 192.168.1.254.
  The client 192.168.1.1, instead of having 192.168.1.254 as gateway, must
  be configured with 192.168.1.3 as gateway :
   dest        gw          mask            iface
   192.168.1.0 0.0.0.0     255.255.255.0   eth0
   0.0.0.0     192.168.1.3 0.0.0.0

 Case 2 : this router is used to reach computers on the web and computers
  on the LAN. We use :
  lcrzoex num eth0 192.168.1.1 a:a:a:a:a:a 192.168.1.3 \
                   b:b:b:b:b:b 192.168.1.4 \
                   192.168.1.254 192.168.1/24
  The address of local computers is 192.168.1/24. The address of the
  default gateway is 192.168.1.254.
  The client 192.168.1.1, instead of having 192.168.1.254 as gateway, must
  be configured with 192.168.1.3 as gateway (even to reach the local LAN) :
   dest        gw          mask            iface
   192.168.1.3 0.0.0.0     255.255.255.255 eth0
   0.0.0.0     192.168.1.3 0.0.0.0
 */
int shared_router_masq(lcrzo_constdata sniffdata,
		       lcrzo_int32 sniffdatasize,
		       lcrzo_spoof *pspoofclient,
		       lcrzo_spoof *pspoofgateway,
		       lcrzo_constdevice device,
		       lcrzo_ipl ipclient,
		       lcrzo_constetha ethins,
		       lcrzo_ipl ipins,
		       lcrzo_constetha ethmasq,
		       lcrzo_ipl ipmasq,
		       lcrzo_ipl ipgw,
		       lcrzo_ipl iplaninf,
		       lcrzo_ipl iplansup);
int shared_router_masqoutput(lcrzo_constdata sniffdata,
			     lcrzo_int32 sniffdatasize,
			     lcrzo_spoof *pspoof,
			     lcrzo_ipl ipclient,
			     lcrzo_constetha ethins,
			     lcrzo_ipl ipins,
			     lcrzo_constdevice device,
			     lcrzo_constetha ethmasq,
			     lcrzo_ipl ipmasq,
			     lcrzo_ipl ipgw,
			     lcrzo_ipl iplaninf,
			     lcrzo_ipl iplansup);
int shared_router_masqinput(lcrzo_constdata sniffdata,
			    lcrzo_int32 sniffdatasize,
			    lcrzo_spoof *pspoof,
			    lcrzo_constetha ethmasq,
			    lcrzo_ipl ipmasq,
			    lcrzo_constdevice device,
			    lcrzo_constetha ethins,
			    lcrzo_ipl ipclient);


/*-------------------------------------------------------------*/
/*                 FORWARDING ROUTER
 Principe :
   ________          ___________                    __________
   |      |    ethins|  lcrzo  |ethmasq             |  true  |
   |client|----------| virtual |--------------------|gateway |----- web
   |______|     ipins|_router__|ipmasq      iprouter|________|

 Example :
   ________                                         __________
   |      |192.168.1.x                 192.168.2.254|  true  |
   |client|--                                   ,---|gateway |----- web
   |______| |                                   |   |________|
            |            ___________            |
            | a:a:a:a:a:a|  lcrzo  |b:b:b:b:b:b |
            -------------| virtual |------------'
              192.168.1.3|_router__|192.168.2.4

 On this scheme, the virtual router is presented with 2 boards ; really,
 there is only one network board :
   ________                                        __________
   |      |192.168.1.x                192.168.2.254|  true  |
   |client|----------------------------------------|gateway |----- web
   |______|                  |                     |________|
                 a:a:a:a:a:a | b:b:b:b:b:b
                 192.168.1.3 | 192.168.2.4
                             | eth0
                        _____|_____               
                        |  lcrzo  |
                        | virtual |        
                        |_router__|
 
 The addresses 192.168.1.3, 192.168.2.4, a:a:a:a:a:a and b:b:b:b:b:b
 must not be used by other computers on the network.
 The computers 192.168.1.x must be configured with 192.168.1.3 as gateway.
 The computers 192.168.2.x (including 192.168.2.254) must be configured
 with 192.168.2.4 as gateway (this second configuration may be impossible
 with your network topology : in this case use routing with masquerading).
 */
int shared_router_forward(lcrzo_constdata sniffdata,
			  lcrzo_int32 sniffdatasize,
			  lcrzo_spoof *pspoofclient,
			  lcrzo_spoof *pspoofgateway,
			  lcrzo_constdevice device,
			  lcrzo_constetha ethins,
			  lcrzo_ipl ipins,
			  lcrzo_constetha ethout,
			  lcrzo_ipl ipout,
			  lcrzo_ipl ipgw,
			  lcrzo_ipl iplaninf,
			  lcrzo_ipl iplansup);
int shared_router_forwardoutput(lcrzo_constdata sniffdata,
				lcrzo_int32 sniffdatasize,
				lcrzo_spoof *pspoof,
				lcrzo_constetha ethins,
				lcrzo_ipl ipins,
				lcrzo_constdevice device,
				lcrzo_constetha ethout,
				lcrzo_ipl ipgw,
				lcrzo_ipl iplaninf,
				lcrzo_ipl iplansup);
int shared_router_forwardinput(lcrzo_constdata sniffdata,
			       lcrzo_int32 sniffdatasize,
			       lcrzo_spoof *pspoof,
			       lcrzo_constetha ethout,
			       lcrzo_ipl ipout,
			       lcrzo_constdevice device,
			       lcrzo_constetha ethins);

#endif




