/*
		                 LCRZOEX
                             Network toolbox
             Copyright(c) 1999,2000,2001,2002 Laurent Constantin
                                  -----

  Main server    : http://www.laurentconstantin.com/
  Backup servers : http://go.to/laurentconstantin/
                   http://laurentconstantin.est-la.com/
                   http://laurentconstantin.free.fr/
                   http://membres.lycos.fr/lauconstantin/
  [my current email address is on the web servers]

                                  -----
  This file is part of Lcrzoex.

  Lcrzoex is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Lcrzoex is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details (http://www.gnu.org/).

------------------------------------------------------------------------
*/

#include <lcrzo.h>
#include "../lcrzoex.h"

/*-------------------------------------------------------------*/
int shared_smtp_appendm_line_msgid(lcrzo_string *poutput)
{ lcrzo_uint32 r;

  lcrzo_epr(lcrzo_string_appendm_text("Message-ID: <", poutput));
  lcrzo_epr(lcrzo_uint32_init_rand(0, 0xFFFFFFFFu, &r));
  lcrzo_epr(lcrzo_string_appendm_uint32(r, "%08X", poutput));
  lcrzo_epr(lcrzo_string_appendm_char('.', 1, poutput));
  lcrzo_epr(lcrzo_uint32_init_rand(0, 0xFFFFFFFFu, &r));
  lcrzo_epr(lcrzo_string_appendm_uint32(r, "%08X", poutput));
  lcrzo_epr(lcrzo_string_appendm_text("@domain.com>\r\n", poutput));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_priv_smtp_date_zone_isvalid(lcrzo_conststring thezone,
				       lcrzo_bool *pisvalid);
int shared_priv_smtp_date_zone_isvalid(lcrzo_conststring thezone,
				       lcrzo_bool *pisvalid)
{ char c0, c1, c2, c3, c4;
  int strlenthezone;

  /* zone is [+-][0-9]{4} or [A-Z]{2,4} */
  strlenthezone=strlen(thezone);
  if ( strlenthezone<2 || strlenthezone>5 )
  { if (pisvalid!=NULL) *pisvalid=LCRZO_FALSE;
    return(LCRZO_ERR_OK);
  }

  /* we will check those chars */
  c0=thezone[0];
  c1=thezone[1];
  if (strlenthezone>2) c2=thezone[2]; else c2='\0';
  if (strlenthezone>3) c3=thezone[3]; else c3='\0';
  if (strlenthezone>4) c4=thezone[4]; else c4='\0';

  /* check [+-][0-9]{4} */
  if ( c0=='+' || c0=='-' )
  { if ( c1>='0' && c1<='9' )
    { if ( c2>='0' && c2<='9' )
      { if ( c3>='0' && c3<='9' )
        { if ( c4>='0' && c4<='9' )
	  { if (pisvalid!=NULL) *pisvalid=LCRZO_TRUE;
            return(LCRZO_ERR_OK);
	  }
	}
      }
    }
  }

  /* check [A-Z]{2,4} */
  if ( c0>='A' && c0<='Z' )
  { if ( c1>='A' && c1<='Z' )
    { if ( ( c2>='A' && c2<='Z' ) || c2=='\0' )
      { if ( ( c3>='A' && c3<='Z' ) || c3=='\0' )
        { if ( c4=='\0' )
	  { if (pisvalid!=NULL) *pisvalid=LCRZO_TRUE;
            return(LCRZO_ERR_OK);
	  }
	}
      }
    }
  }

  if (pisvalid!=NULL) *pisvalid=LCRZO_FALSE;
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_smtp_appendm_line_date(lcrzo_string *poutput)
{ char thedate[80];
  char thezone[80];
  char tmpfmt[5];
  time_t thetime;
  lcrzo_bool isvalid;
  int setlen;

  /* set time */
  lcrzo_epr(lcrzo_string_appendm_text("Date: ", poutput));
  thetime=time(NULL);
  setlen=strftime(thedate, 79, "%a, %d %b %Y %H:%M:%S", localtime(&thetime));
  if (setlen==0)
  { /* set a fake date */
    lcrzo_epr(lcrzo_string_appendm_text("Sun, 23 Jan 2000 01:01:01 GMT\r\n",
					poutput));
    return(LCRZO_ERR_OK);
  }
  lcrzo_epr(lcrzo_string_appendm_text(thedate, poutput));

  /* set zone */
  lcrzo_epr(lcrzo_string_appendm_text(" ", poutput));
  strcpy(tmpfmt, "%z");
  setlen=strftime(thezone, 79, tmpfmt, localtime(&thetime));
  if ( setlen!=0 )
  { /* we have to check because Windows set "Paris, Madrid", which is bad */
    lcrzo_er(shared_priv_smtp_date_zone_isvalid(thezone, &isvalid));
    if (isvalid)
    { lcrzo_epr(lcrzo_string_appendm_text(thezone, poutput));
      lcrzo_epr(lcrzo_string_appendm_text("\r\n", poutput));
      return(LCRZO_ERR_OK);
    }
  }
  setlen=strftime(thezone, 79, "%Z", localtime(&thetime));
  if ( setlen!=0 )
  { /* we have to check because Windows set "Paris, Madrid", which is bad */
    lcrzo_er(shared_priv_smtp_date_zone_isvalid(thezone, &isvalid));
    if (isvalid)
    { lcrzo_epr(lcrzo_string_appendm_text(thezone, poutput));
      lcrzo_epr(lcrzo_string_appendm_text("\r\n", poutput));
      return(LCRZO_ERR_OK);
    }
  }
  /* use this fake zone */
  lcrzo_epr(lcrzo_string_appendm_text("GMT", poutput));
  lcrzo_epr(lcrzo_string_appendm_text("\r\n", poutput));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_smtp_appendm_line_from(lcrzo_conststring username,
				  lcrzo_conststring email,
				  lcrzo_string *poutput)
{
  lcrzo_epr(lcrzo_string_appendm_text("From: \"", poutput));
  lcrzo_epr(lcrzo_string_appendm_text(username, poutput));
  lcrzo_epr(lcrzo_string_appendm_text("\" <", poutput));
  lcrzo_epr(lcrzo_string_appendm_text(email, poutput));
  lcrzo_epr(lcrzo_string_appendm_text(">\r\n", poutput));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_smtp_appendm_line_replyto(lcrzo_conststring username,
				     lcrzo_conststring email,
				     lcrzo_string *poutput)
{ lcrzo_epr(lcrzo_string_appendm_text("Reply-To: \"", poutput));
  lcrzo_epr(lcrzo_string_appendm_text(username, poutput));
  lcrzo_epr(lcrzo_string_appendm_text("\" <", poutput));
  lcrzo_epr(lcrzo_string_appendm_text(email, poutput));
  lcrzo_epr(lcrzo_string_appendm_text(">\r\n", poutput));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_smtp_appendm_line_to(lcrzo_conststring username,
				lcrzo_conststring email,
				lcrzo_string *poutput)
{ lcrzo_epr(lcrzo_string_appendm_text("To: \"", poutput));
  lcrzo_epr(lcrzo_string_appendm_text(username, poutput));
  lcrzo_epr(lcrzo_string_appendm_text("\" <", poutput));
  lcrzo_epr(lcrzo_string_appendm_text(email, poutput));
  lcrzo_epr(lcrzo_string_appendm_text(">\r\n", poutput));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_smtp_appendm_line_organization(lcrzo_string *poutput)
{ lcrzo_epr(lcrzo_string_appendm_text("Organization: ", poutput));
  lcrzo_epr(lcrzo_string_appendm_text("LCRZO LCRZOEX\r\n", poutput));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_smtp_appendm_line_xmailer(lcrzo_string *poutput)
{ lcrzo_epr(lcrzo_string_appendm_text("X-Mailer: ", poutput));
  lcrzo_epr(lcrzo_string_appendm_text("Lcrzoex Mailer\r\n", poutput));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_smtp_appendm_line_xacclang(lcrzo_string *poutput)
{ lcrzo_epr(lcrzo_string_appendm_text("X-Accept-Language: ", poutput));
  lcrzo_epr(lcrzo_string_appendm_text("en\r\n", poutput));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_smtp_appendm_line_mimeversion(lcrzo_string *poutput)
{ lcrzo_epr(lcrzo_string_appendm_text("MIME-Version: 1.0\r\n", poutput));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_smtp_appendm_line_conttype_mpmixed(lcrzo_conststring boundary,
					      lcrzo_string *poutput)
{ lcrzo_epr(lcrzo_string_appendm_text("Content-Type: ", poutput));
  lcrzo_epr(lcrzo_string_appendm_text("multipart/mixed;\r\n", poutput));
  lcrzo_epr(lcrzo_string_appendm_text(" boundary=\"", poutput));
  lcrzo_epr(lcrzo_string_appendm_text(boundary, poutput));
  lcrzo_epr(lcrzo_string_appendm_text("\"\r\n", poutput));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_smtp_appendm_line_conttype_textplainusascii(lcrzo_conststring name,
						       lcrzo_string *poutput)
{ lcrzo_epr(lcrzo_string_appendm_text("Content-Type: ", poutput));
  lcrzo_epr(lcrzo_string_appendm_text("text/plain; charset=us-ascii",poutput));
  if (name!=NULL)
  { lcrzo_epr(lcrzo_string_appendm_text(";\r\n",  poutput));
    lcrzo_epr(lcrzo_string_appendm_text(" name=\"", poutput));
    lcrzo_epr(lcrzo_string_appendm_text(name, poutput));
    lcrzo_epr(lcrzo_string_appendm_text("\"", poutput));
  }
  lcrzo_epr(lcrzo_string_appendm_text("\r\n", poutput));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_smtp_appendm_line_conttype_textplainiso8859(lcrzo_conststring name,
						       lcrzo_string *poutput)
{ lcrzo_epr(lcrzo_string_appendm_text("Content-Type: ", poutput));
  lcrzo_epr(lcrzo_string_appendm_text("text/plain; charset=iso-8859-1", 
				      poutput));
  if (name!=NULL)
  { lcrzo_epr(lcrzo_string_appendm_text(";\r\n",  poutput));
    lcrzo_epr(lcrzo_string_appendm_text(" name=\"", poutput));
    lcrzo_epr(lcrzo_string_appendm_text(name, poutput));
    lcrzo_epr(lcrzo_string_appendm_text("\"", poutput));
  }
  lcrzo_epr(lcrzo_string_appendm_text("\r\n", poutput));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_smtp_appendm_line_conttype_appoctstream(lcrzo_conststring name,
						   lcrzo_string *poutput)
{ lcrzo_epr(lcrzo_string_appendm_text("Content-Type: ", poutput));
  lcrzo_epr(lcrzo_string_appendm_text("application/octet-stream", 
				      poutput));
  if (name!=NULL)
  { lcrzo_epr(lcrzo_string_appendm_text(";\r\n",  poutput));
    lcrzo_epr(lcrzo_string_appendm_text(" name=\"", poutput));
    lcrzo_epr(lcrzo_string_appendm_text(name, poutput));
    lcrzo_epr(lcrzo_string_appendm_text("\"", poutput));
  }
  lcrzo_epr(lcrzo_string_appendm_text("\r\n", poutput));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_smtp_appendm_line_cte_7bit(lcrzo_string *poutput)
{ lcrzo_epr(lcrzo_string_appendm_text("Content-Transfer-Encoding: ", poutput));
  lcrzo_epr(lcrzo_string_appendm_text("7bit\r\n", poutput));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_smtp_appendm_line_cte_8bit(lcrzo_string *poutput)
{ lcrzo_epr(lcrzo_string_appendm_text("Content-Transfer-Encoding: ", poutput));
  lcrzo_epr(lcrzo_string_appendm_text("8bit\r\n", poutput));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_smtp_appendm_line_cte_base64(lcrzo_string *poutput)
{ lcrzo_epr(lcrzo_string_appendm_text("Content-Transfer-Encoding: ", poutput));
  lcrzo_epr(lcrzo_string_appendm_text("base64\r\n", poutput));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_smtp_appendm_line_contdisp_inline(lcrzo_conststring filename,
					     lcrzo_string *poutput)
{ lcrzo_epr(lcrzo_string_appendm_text("Content-Disposition: ", poutput));
  lcrzo_epr(lcrzo_string_appendm_text("inline", 
				      poutput));
  if (filename!=NULL)
  { lcrzo_epr(lcrzo_string_appendm_text(";\r\n",  poutput));
    lcrzo_epr(lcrzo_string_appendm_text(" filename=\"", poutput));
    lcrzo_epr(lcrzo_string_appendm_text(filename, poutput));
    lcrzo_epr(lcrzo_string_appendm_text("\"", poutput));
  }
  lcrzo_epr(lcrzo_string_appendm_text("\r\n", poutput));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_smtp_appendm_line_contdisp_attach(lcrzo_conststring filename,
					     lcrzo_string *poutput)
{ lcrzo_epr(lcrzo_string_appendm_text("Content-Disposition: ", poutput));
  lcrzo_epr(lcrzo_string_appendm_text("attachment", poutput));
  if (filename!=NULL)
  { lcrzo_epr(lcrzo_string_appendm_text(";\r\n",  poutput));
    lcrzo_epr(lcrzo_string_appendm_text(" filename=\"", poutput));
    lcrzo_epr(lcrzo_string_appendm_text(filename, poutput));
    lcrzo_epr(lcrzo_string_appendm_text("\"", poutput));
  }
  lcrzo_epr(lcrzo_string_appendm_text("\r\n", poutput));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_smtp_appendm_line_empty(lcrzo_string *poutput)
{ lcrzo_epr(lcrzo_string_appendm_text("\r\n", poutput));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_smtp_appendm_line_subject(lcrzo_conststring subject, 
				     lcrzo_string *poutput)
{ int i;
  lcrzo_uint8 c;
  lcrzo_bool specialcharfound;
  lcrzo_string thesubject;

  specialcharfound=LCRZO_FALSE;
  lcrzo_epr(lcrzo_string_initm_text("Subject: =?iso-8859-1?Q?", &thesubject));
  for ( i=0 ; i<(int)strlen(subject) ; i++ )
  { c=subject[i];
    if ( c<32 || c>=127 )
    { specialcharfound=LCRZO_TRUE;
      lcrzo_epr(lcrzo_string_appendm_text("=", &thesubject));
      lcrzo_epr(lcrzo_string_appendm_uint8(c, "%02X", &thesubject));
    }
    else
    { lcrzo_epr(lcrzo_string_appendm_char(c, 1, &thesubject));
    }
  }
  lcrzo_epr(lcrzo_string_appendm_text("?=\r\n", &thesubject));

  if (!specialcharfound)
  { lcrzo_er(lcrzo_string_free2(&thesubject));
    lcrzo_epr(lcrzo_string_appendm_text("Subject: ", poutput));
    lcrzo_epr(lcrzo_string_appendm_text(subject, poutput));
    lcrzo_epr(lcrzo_string_appendm_text("\r\n", poutput));
  }
  else
  { lcrzo_epr(lcrzo_string_appendm_text(thesubject, poutput));
    lcrzo_er(lcrzo_string_free2(&thesubject));  
  }

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_smtp_appendm_line_boundary(lcrzo_conststring boundary,
				      lcrzo_bool islast,
				      lcrzo_string *poutput)
{ lcrzo_epr(lcrzo_string_appendm_text("--", poutput));
  lcrzo_epr(lcrzo_string_appendm_text(boundary, poutput));
  if (islast)
  { lcrzo_epr(lcrzo_string_appendm_text("--", poutput));
  }
  lcrzo_epr(lcrzo_string_appendm_text("\r\n", poutput));
  return(LCRZO_ERR_OK);
}


/*-------------------------------------------------------------*/
int shared_smtp_generateboundary(lcrzo_string *pboundary)
{ lcrzo_uint32 r;

  lcrzo_epr(lcrzo_string_initm_text("--++", pboundary));
  lcrzo_epr(lcrzo_uint32_init_rand(0, 0xFFFFFFFFu, &r));
  lcrzo_epr(lcrzo_string_appendm_uint32(r, "%08X", pboundary));
  lcrzo_epr(lcrzo_uint32_init_rand(0, 0xFFFFFFFFu, &r));
  lcrzo_epr(lcrzo_string_appendm_uint32(r, "%08X", pboundary));
  lcrzo_epr(lcrzo_uint32_init_rand(0, 0xFFFFFFFFu, &r));
  lcrzo_epr(lcrzo_string_appendm_uint32(r, "%08X", pboundary));

  return(LCRZO_ERR_OK);
}


/*-------------------------------------------------------------*/
int shared_smtp_initm_headerbody_usascii(lcrzo_conststring from,
					 lcrzo_conststring replyto,
					 lcrzo_conststring to,
					 lcrzo_conststring subject, 
					 lcrzo_constdata bodydata,
					 lcrzo_int32 bodydatasize,
					 lcrzo_data *pdataout,
					 lcrzo_int32 *pdataoutsize)
{ lcrzo_int32 datasize;
  lcrzo_string header;

  /* generate the header */
  lcrzo_epr(lcrzo_string_initm_text("", &header));
  lcrzo_epr(shared_smtp_appendm_line_msgid(&header));
  lcrzo_epr(shared_smtp_appendm_line_date(&header));
  lcrzo_epr(shared_smtp_appendm_line_from(from, from, &header));
  if ( strcmp(from, replyto) )
  { lcrzo_epr(shared_smtp_appendm_line_replyto(replyto, replyto, &header));
  }
  lcrzo_epr(shared_smtp_appendm_line_organization(&header));
  lcrzo_epr(shared_smtp_appendm_line_xmailer(&header));
  lcrzo_epr(shared_smtp_appendm_line_xacclang(&header));
  lcrzo_epr(shared_smtp_appendm_line_to(to, to, &header));
  lcrzo_epr(shared_smtp_appendm_line_subject(subject, &header));
  lcrzo_epr(shared_smtp_appendm_line_empty(&header));

  /* append the data */
  lcrzo_epr(lcrzo_data_initm_text(header, pdataout, &datasize));
  lcrzo_epr(lcrzo_string_free2(&header));
  lcrzo_epr(lcrzo_data_appendm_data(bodydata, bodydatasize,
				    datasize, pdataout, &datasize));

  /* end */
  if (pdataoutsize!=NULL) *pdataoutsize=datasize;
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_smtp_initm_headerbody_iso8859(lcrzo_conststring from,
					 lcrzo_conststring replyto,
					 lcrzo_conststring to,
					 lcrzo_conststring subject, 
					 lcrzo_constdata bodydata,
					 lcrzo_int32 bodydatasize,
					 lcrzo_data *pdataout,
					 lcrzo_int32 *pdataoutsize)
{ lcrzo_int32 datasize;
  lcrzo_string header;

  /* generate the header */
  lcrzo_epr(lcrzo_string_initm_text("", &header));
  lcrzo_epr(shared_smtp_appendm_line_msgid(&header));
  lcrzo_epr(shared_smtp_appendm_line_date(&header));
  lcrzo_epr(shared_smtp_appendm_line_from(from, from, &header));
  if ( strcmp(from, replyto) )
  { lcrzo_epr(shared_smtp_appendm_line_replyto(replyto, replyto, &header));
  }
  lcrzo_epr(shared_smtp_appendm_line_organization(&header));
  lcrzo_epr(shared_smtp_appendm_line_xmailer(&header));
  lcrzo_epr(shared_smtp_appendm_line_xacclang(&header));
  lcrzo_epr(shared_smtp_appendm_line_mimeversion(&header));
  lcrzo_epr(shared_smtp_appendm_line_to(to, to, &header));
  lcrzo_epr(shared_smtp_appendm_line_subject(subject, &header));
  lcrzo_epr(shared_smtp_appendm_line_conttype_textplainiso8859(NULL, &header));
  lcrzo_epr(shared_smtp_appendm_line_cte_8bit(&header));
  lcrzo_epr(shared_smtp_appendm_line_empty(&header));

  /* append the data */
  lcrzo_epr(lcrzo_data_initm_text(header, pdataout, &datasize));
  lcrzo_epr(lcrzo_string_free2(&header));
  lcrzo_epr(lcrzo_data_appendm_data(bodydata, bodydatasize,
				    datasize, pdataout, &datasize));

  /* end */
  if (pdataoutsize!=NULL) *pdataoutsize=datasize;
  return(LCRZO_ERR_OK);
}


/*-------------------------------------------------------------*/
int shared_smtp_initm_multipart_header(lcrzo_conststring from,
				       lcrzo_conststring replyto,
				       lcrzo_conststring to,
				       lcrzo_conststring subject, 
				       lcrzo_string *pboundary,
				       lcrzo_data *pdataout,
				       lcrzo_int32 *pdataoutsize)
{ lcrzo_string header;

  lcrzo_epr(shared_smtp_generateboundary(pboundary));

  /* generate the header */
  lcrzo_epr(lcrzo_string_initm_text("", &header));
  lcrzo_epr(shared_smtp_appendm_line_msgid(&header));
  lcrzo_epr(shared_smtp_appendm_line_date(&header));
  lcrzo_epr(shared_smtp_appendm_line_from(from, from, &header));
  if ( strcmp(from, replyto) )
  { lcrzo_epr(shared_smtp_appendm_line_replyto(replyto, replyto, &header));
  }
  lcrzo_epr(shared_smtp_appendm_line_organization(&header));
  lcrzo_epr(shared_smtp_appendm_line_xmailer(&header));
  lcrzo_epr(shared_smtp_appendm_line_xacclang(&header));
  lcrzo_epr(shared_smtp_appendm_line_mimeversion(&header));
  lcrzo_epr(shared_smtp_appendm_line_to(to, to, &header));
  lcrzo_epr(shared_smtp_appendm_line_subject(subject, &header));
  lcrzo_epr(shared_smtp_appendm_line_conttype_mpmixed(*pboundary, &header));
  lcrzo_epr(shared_smtp_appendm_line_empty(&header));

  /* append the data */
  lcrzo_epr(lcrzo_string_appendm_text(
        "This is a multi-part message in MIME format.\r\n", &header));

  /* generate output data */
  lcrzo_epr(lcrzo_data_initm_text(header, pdataout, pdataoutsize));
  lcrzo_epr(lcrzo_string_free2(&header));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_smtp_appendm_multipart_usascii(lcrzo_conststring boundary,
					  lcrzo_conststring headername,
					  lcrzo_conststring headerfilename,
					  lcrzo_constdata data,
					  lcrzo_int32 datasize,
					  lcrzo_int32 dataoutcurrentsize,
					  lcrzo_data *pdataout,
					  lcrzo_int32 *pdataoutsize)
{ lcrzo_int32 dataoutsize;
  lcrzo_string header;

  /* generate the header */
  lcrzo_epr(lcrzo_string_initm_text("", &header));
  lcrzo_epr(shared_smtp_appendm_line_boundary(boundary, LCRZO_FALSE, &header));
  lcrzo_epr(shared_smtp_appendm_line_conttype_textplainusascii(headername, 
							       &header));
  lcrzo_epr(shared_smtp_appendm_line_cte_7bit(&header));
  lcrzo_epr(shared_smtp_appendm_line_contdisp_inline(headerfilename, 
						     &header));
  lcrzo_epr(shared_smtp_appendm_line_empty(&header));

  /* append the data */
  lcrzo_epr(lcrzo_data_appendm_text(header, dataoutcurrentsize, 
				    pdataout, &dataoutsize));
  lcrzo_epr(lcrzo_string_free2(&header));
  lcrzo_epr(lcrzo_data_appendm_data(data, datasize,
				    dataoutsize, pdataout, &dataoutsize));

  /* end */
  if (pdataoutsize!=NULL) *pdataoutsize=dataoutsize;
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_smtp_appendm_multipart_iso8859(lcrzo_conststring boundary,
					  lcrzo_conststring headername,
					  lcrzo_conststring headerfilename,
					  lcrzo_constdata data,
					  lcrzo_int32 datasize,
					  lcrzo_int32 dataoutcurrentsize,
					  lcrzo_data *pdataout,
					  lcrzo_int32 *pdataoutsize)
{ lcrzo_int32 dataoutsize;
  lcrzo_string header;

  /* generate the header */
  lcrzo_epr(lcrzo_string_initm_text("", &header));
  lcrzo_epr(shared_smtp_appendm_line_boundary(boundary, LCRZO_FALSE, &header));
  lcrzo_epr(shared_smtp_appendm_line_conttype_textplainiso8859(headername, 
							       &header));
  lcrzo_epr(shared_smtp_appendm_line_cte_8bit(&header));
  lcrzo_epr(shared_smtp_appendm_line_contdisp_inline(headerfilename, 
						     &header));
  lcrzo_epr(shared_smtp_appendm_line_empty(&header));

  /* append the data */
  lcrzo_epr(lcrzo_data_appendm_text(header, dataoutcurrentsize, 
				    pdataout, &dataoutsize));
  lcrzo_epr(lcrzo_string_free2(&header));
  lcrzo_epr(lcrzo_data_appendm_data(data, datasize,
				    dataoutsize, pdataout, &dataoutsize));

  /* end */
  if (pdataoutsize!=NULL) *pdataoutsize=dataoutsize;
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_smtp_appendm_multipart_base64(lcrzo_conststring boundary,
					 lcrzo_conststring headername,
					 lcrzo_conststring headerfilename,
					 lcrzo_constdata data,
					 lcrzo_int32 datasize,
					 lcrzo_int32 dataoutcurrentsize,
					 lcrzo_data *pdataout,
					 lcrzo_int32 *pdataoutsize)
{ lcrzo_int32 dataoutsize, i, chunknumber, bytesleft;
  lcrzo_string header, base64;

  /* generate the header */
  lcrzo_epr(lcrzo_string_initm_text("", &header));
  lcrzo_epr(shared_smtp_appendm_line_boundary(boundary, LCRZO_FALSE, &header));
  lcrzo_epr(shared_smtp_appendm_line_conttype_appoctstream(headername, 
							   &header));
  lcrzo_epr(shared_smtp_appendm_line_cte_base64(&header));
  lcrzo_epr(shared_smtp_appendm_line_contdisp_attach(headerfilename, &header));
  lcrzo_epr(shared_smtp_appendm_line_empty(&header));
  lcrzo_epr(lcrzo_data_appendm_text(header, dataoutcurrentsize, 
				    pdataout, &dataoutsize));
  lcrzo_epr(lcrzo_string_free2(&header));

  /* append the data */
  chunknumber=datasize/54;
  for ( i=0 ; i<chunknumber ; i++ )
  { lcrzo_epr(lcrzo_base64_initm_data(data+i*54, 54, &base64));
    lcrzo_epr(lcrzo_data_appendm_text(base64, 
				      dataoutsize, pdataout, &dataoutsize));
    lcrzo_epr(lcrzo_string_free2(&base64));
    lcrzo_epr(lcrzo_data_appendm_text("\r\n", 
				      dataoutsize, pdataout, &dataoutsize));
  }
  bytesleft=datasize-54*chunknumber;
  if (bytesleft)
  { lcrzo_epr(lcrzo_base64_initm_data(data+chunknumber*54, bytesleft,&base64));
    lcrzo_epr(lcrzo_data_appendm_text(base64, 
				      dataoutsize, pdataout, &dataoutsize));
    lcrzo_epr(lcrzo_string_free2(&base64));
    lcrzo_epr(lcrzo_data_appendm_text("\r\n", 
				      dataoutsize, pdataout, &dataoutsize));
  }
 
  /* end */
  if (pdataoutsize!=NULL) *pdataoutsize=dataoutsize;
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_smtp_appendm_multipart_end(lcrzo_conststring boundary,
				      lcrzo_int32 dataoutcurrentsize,
				      lcrzo_data *pdataout,
				      lcrzo_int32 *pdataoutsize)
{ lcrzo_int32 dataoutsize;
  lcrzo_string header;

  /* generate the header */
  lcrzo_epr(lcrzo_string_initm_text("", &header));
  lcrzo_epr(shared_smtp_appendm_line_boundary(boundary, LCRZO_TRUE, &header));

  /* append the data */
  lcrzo_epr(lcrzo_data_appendm_text(header, dataoutcurrentsize, 
				    pdataout, &dataoutsize));
  lcrzo_epr(lcrzo_string_free2(&header));

  /* end */
  if (pdataoutsize!=NULL) *pdataoutsize=dataoutsize;
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_smtp_sendmsg(lcrzo_ipl serverip, 
			lcrzo_uint16 serverport,
			lcrzo_conststring mailfrom, 
			lcrzo_conststring rcptto,
			lcrzo_constdata headerbody,
			lcrzo_int32 headerbodysize)
{ lcrzo_sock sock;
  lcrzo_data data;
  lcrzo_int32 datasize, foundpositive;
  lcrzo_string domainname;
  int ret;

  /* initialize the client domain name */
  ret=lcrzo_string_search_char(mailfrom, LCRZO_FALSE,
			       +1, -1, '@',
			       NULL, &foundpositive);
  if ( ret==LCRZO_ERR_OK )
  { lcrzo_epr(lcrzo_string_initm_range(mailfrom, foundpositive+1, -1,
				       &domainname));
  }
  else
  { lcrzo_epr(lcrzo_string_initm_text("microsoft.com", &domainname));
  }

  /* initialize the client */
  lcrzo_epr(lcrzo_sock_tcpcli_real(serverip, serverport, &sock));

  /* first, we read the banner */
  lcrzo_epr(shared_misc_verifyresponse1(&sock, 220));

  /* then, we send a 'HELO' */
  lcrzo_epr(lcrzo_data_initm_text("HELO ", &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text(domainname, datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, &data, &datasize));
  lcrzo_epr(lcrzo_sock_write(&sock, data, datasize));
  lcrzo_er(lcrzo_string_free2(&domainname));
  lcrzo_er(lcrzo_data_free2(&data));
  lcrzo_epr(shared_misc_verifyresponse1(&sock, 250));

  /* then, we send a 'MAIL FROM' */
  lcrzo_epr(lcrzo_data_initm_text("Mail From: <", &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text(mailfrom, datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text(">\r\n", datasize, &data, &datasize));
  lcrzo_epr(lcrzo_sock_write(&sock, data, datasize));
  lcrzo_er(lcrzo_data_free2(&data));
  lcrzo_epr(shared_misc_verifyresponse1(&sock, 250));

  /* then, we send a 'RCPT TO' */
  lcrzo_epr(lcrzo_data_initm_text("Rcpt to: <", &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text(rcptto, datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text(">\r\n", datasize, &data, &datasize));
  lcrzo_epr(lcrzo_sock_write(&sock, data, datasize));
  lcrzo_er(lcrzo_data_free2(&data));
  lcrzo_epr(shared_misc_verifyresponse1(&sock, 250));

  /* then, we send a 'DATA' */
  lcrzo_epr(lcrzo_data_initm_text("DATA\r\n",
				  &data, &datasize));
  lcrzo_epr(lcrzo_sock_write(&sock, data, datasize));
  lcrzo_er(lcrzo_data_free2(&data));
  lcrzo_epr(shared_misc_verifyresponse1(&sock, 354));

  /* then, we send data */
  lcrzo_epr(lcrzo_sock_write(&sock, headerbody, headerbodysize));
  lcrzo_epr(lcrzo_data_initm_text(".\r\n",
				  &data, &datasize));
  lcrzo_epr(lcrzo_sock_write(&sock, data, datasize));
  lcrzo_er(lcrzo_data_free2(&data));
  lcrzo_epr(shared_misc_verifyresponse1(&sock, 250));

  /* then, we send a 'QUIT' */
  lcrzo_epr(lcrzo_data_initm_text("QUIT\r\n",
				  &data, &datasize));
  lcrzo_epr(lcrzo_sock_write(&sock, data, datasize));
  lcrzo_er(lcrzo_data_free2(&data));
  lcrzo_epr(shared_misc_verifyresponse1(&sock, 221));

  /* close the socket */
  lcrzo_epr(lcrzo_sock_close(&sock));
  return(LCRZO_ERR_OK);
}


/*-------------------------------------------------------------*/
int shared_smtp_initm_subjectdata_file(lcrzo_conststring subject, 
				       lcrzo_conststring bodyfilename,
				       lcrzo_string *psubject, 
				       lcrzo_data *pdata,
				       lcrzo_int32 *pdatasize)
{ lcrzo_int32 linesize, datasize;
  lcrzo_data line;
  lcrzo_int16 language;
  lcrzo_file file;
  int ret;

  /* open the file */
  lcrzo_epr(lcrzo_global_get_language(&language));
  lcrzo_epr(lcrzo_file_open_read(bodyfilename, &file));

  /* eventually get the subject on the first line */
  if ( strlen(subject) )
  { lcrzo_epr(lcrzo_string_initm_text(subject, psubject));
  }
  else
  { /* the subject is on the first line of the file */
    lcrzo_epr(lcrzo_file_readm_line(&file, LCRZO_WAIT_INFINITE, 
				    &line, &linesize));
    line[linesize]='\0';
    lcrzo_epr(lcrzo_string_initm_text((lcrzo_string)line, psubject));
    lcrzo_er(lcrzo_data_free2(&line));
  }

  /* read the data in the file */
  lcrzo_epr(lcrzo_data_initm_text("", pdata, &datasize));
  while(LCRZO_TRUE)
  { ret=lcrzo_file_readm_line(&file, LCRZO_WAIT_INFINITE, 
			      &line, &linesize);
    if (ret==LCRZO_ERR_OKDATAEND)
    { break;
    }
    else if (ret==LCRZO_ERR_OK)
    { if ( linesize>0 && line[0]=='.' )
      { lcrzo_epr(lcrzo_data_appendm_char('.', 1, datasize, pdata, &datasize));
      }
      lcrzo_epr(lcrzo_data_appendm_data(line, linesize,
					datasize, pdata, &datasize));
      lcrzo_er(lcrzo_data_free2(&line));
      lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, pdata, &datasize));
    }
    else
    { lcrzo_epr(lcrzo_file_close(&file));
      return(ret);
    }
  }
  lcrzo_epr(lcrzo_file_close(&file));

  /* end */
  if (pdatasize!=NULL) *pdatasize=datasize;
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_smtp_initm_headerbody_file(lcrzo_conststring from,
				      lcrzo_conststring replyto,
				      lcrzo_conststring to,
				      lcrzo_conststring subject, 
				      lcrzo_conststring bodyfilename,
				      lcrzo_data *pdata,
				      lcrzo_int32 *pdatasize)
{ lcrzo_data bodydata;
  lcrzo_int32 bodydatasize;
  lcrzo_string thesubject;

  lcrzo_epr(shared_smtp_initm_subjectdata_file(subject, bodyfilename,
					       &thesubject,
					       &bodydata, &bodydatasize));

  lcrzo_epr(shared_smtp_initm_headerbody_iso8859(from, replyto, to, 
						 thesubject, 
						 bodydata, bodydatasize,
						 pdata, pdatasize));
  lcrzo_er(lcrzo_data_free2(&bodydata));
  lcrzo_er(lcrzo_string_free2(&thesubject));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_smtp_initm_multipart_2files(lcrzo_conststring from,
				       lcrzo_conststring replyto,
				       lcrzo_conststring to,
				       lcrzo_conststring subject, 
				       lcrzo_conststring bodyfilename,
				       lcrzo_conststring attachementfilename,
				       lcrzo_data *pdata,
				       lcrzo_int32 *pdatasize)
{ lcrzo_data bodydata;
  lcrzo_int32 bodydatasize, datasize;
  lcrzo_string thesubject, boundary, tmp;

  lcrzo_epr(shared_smtp_initm_subjectdata_file(subject, bodyfilename,
					       &thesubject,
					       &bodydata, &bodydatasize));

  /* header */
  lcrzo_epr(shared_smtp_initm_multipart_header(from, replyto, to, thesubject, 
					       &boundary, pdata, &datasize));
  lcrzo_er(lcrzo_string_free2(&thesubject));

  /* first file */
  lcrzo_epr(lcrzo_path_decode_lastitem(bodyfilename, LCRZO_FALSE, &tmp));
  lcrzo_epr(shared_smtp_appendm_multipart_iso8859(boundary, tmp, tmp,
						  bodydata, bodydatasize,
						  datasize, pdata, &datasize));
  lcrzo_er(lcrzo_string_free2(&tmp));
  lcrzo_er(lcrzo_data_free2(&bodydata));

  /* second file */
  lcrzo_epr(lcrzo_filename_read(attachementfilename, 
				&bodydata, &bodydatasize));
  lcrzo_epr(lcrzo_path_decode_lastitem(attachementfilename, LCRZO_FALSE,&tmp));
  lcrzo_epr(shared_smtp_appendm_multipart_base64(boundary, tmp, tmp,
						 bodydata, bodydatasize,
						 datasize, pdata, &datasize));
  lcrzo_er(lcrzo_string_free2(&tmp));
  lcrzo_er(lcrzo_data_free2(&bodydata));

  /* end */
  lcrzo_epr(shared_smtp_appendm_multipart_end(boundary, 
					      datasize, pdata, &datasize));
  lcrzo_er(lcrzo_string_free2(&boundary));

  if (pdatasize!=NULL) *pdatasize=datasize;
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_smtp_sendmsg_hb_file(lcrzo_ipl serverip, 
				lcrzo_uint16 serverport,
				lcrzo_conststring mailfrom, 
				lcrzo_conststring rcptto,
				lcrzo_conststring from, 
				lcrzo_conststring replyto, 
				lcrzo_conststring to,
				lcrzo_conststring subject, 
				lcrzo_conststring bodyfilename)
{ lcrzo_data headerbody;
  lcrzo_int32 headerbodysize;
  int ret;

  lcrzo_epr(shared_smtp_initm_headerbody_file(from, replyto, to, subject,
					      bodyfilename, &headerbody,
					      &headerbodysize));

  ret=shared_smtp_sendmsg(serverip, serverport, mailfrom, rcptto,
			  headerbody, headerbodysize);
  lcrzo_er(lcrzo_data_free2(&headerbody));
  return(ret);
}

/*-------------------------------------------------------------*/
int shared_smtp_sendmsg_mp_2files(lcrzo_ipl serverip, 
				  lcrzo_uint16 serverport,
				  lcrzo_conststring mailfrom, 
				  lcrzo_conststring rcptto,
				  lcrzo_conststring from, 
				  lcrzo_conststring replyto, 
				  lcrzo_conststring to,
				  lcrzo_conststring subject, 
				  lcrzo_conststring bodyfilename,
				  lcrzo_conststring attachementfilename)
{ lcrzo_data headerbody;
  lcrzo_int32 headerbodysize;
  int ret;

  lcrzo_epr(shared_smtp_initm_multipart_2files(from, replyto, to, subject,
					       bodyfilename, 
					       attachementfilename,
					       &headerbody,
					       &headerbodysize));

  ret=shared_smtp_sendmsg(serverip, serverport, mailfrom, rcptto,
			  headerbody, headerbodysize);
  lcrzo_er(lcrzo_data_free2(&headerbody));
  return(ret);
}

