/*
		                 LCRZOEX
                             Network toolbox
             Copyright(c) 1999,2000,2001,2002 Laurent Constantin
                                  -----

  Main server    : http://www.laurentconstantin.com/
  Backup servers : http://go.to/laurentconstantin/
                   http://laurentconstantin.est-la.com/
                   http://laurentconstantin.free.fr/
                   http://membres.lycos.fr/lauconstantin/
  [my current email address is on the web servers]

                                  -----
  This file is part of Lcrzoex.

  Lcrzoex is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Lcrzoex is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details (http://www.gnu.org/).

------------------------------------------------------------------------
*/

#include <lcrzo.h>
#include "../lcrzoex.h"

/*-------------------------------------------------------------*/
int shared_sniff_dump(int argc, char *argv[], lcrzo_int16 snifftype)
{ lcrzo_device device;
  lcrzo_uint32 mtu;
  lcrzo_sniff sniff;
  lcrzo_data sniffdata;
  lcrzo_int32 sniffdatasize;
  lcrzo_printprofile printprofile;
  lcrzo_int16 language;
  int ret;
  
  /* init device and mtu */
  if (argc>=2)
  { lcrzo_epr(shared_para_device_init_src(argv[1], device));
  }
  else
  { lcrzo_epr(lcrzo_device_value_pos(LCRZO_DEVICE_TYPE_BEST, 1, device));
  }
  lcrzo_epr(lcrzo_mtu_init_device(device, &mtu));
  lcrzo_epr(lcrzo_global_get_language(&language));
  if (language==LCRZO_GLOBAL_LANG_FR)
    printf("Ecoute sur %s\n", device);
  else
    printf("Listening on %s\n", device);

  /* get printprofile */
  lcrzo_epr(lcrzo_printprofile_init_stdin((language==LCRZO_GLOBAL_LANG_FR)?
				"Choisissez le profil d'affichage":
				"Choose the print profile",
				LCRZO_PRINTPROFILE_ARRAYDUMP, &printprofile));

  /* init sniff */
  if (argc<=2)
  { lcrzo_epr(lcrzo_sniff_init(device, mtu, snifftype, "", &sniff));
  }
  else
  { lcrzo_string snifffilter;
    lcrzo_epr(shared_para_string_init_snifffilter(argv[2], &snifffilter));
    lcrzo_epr(lcrzo_sniff_init(device, mtu, snifftype, snifffilter, &sniff));
    lcrzo_er(lcrzo_string_free2(&snifffilter));
  }

  /* main loop */
  do
  { ret=lcrzo_sniff_nextm(&sniff, LCRZO_WAIT_INFINITE, 
			  &sniffdata, &sniffdatasize);
    if (ret!=LCRZO_ERR_OK) break;
    lcrzo_epr(lcrzo_packet_print_eth(sniffdata, sniffdatasize, printprofile));
    lcrzo_er(lcrzo_data_free2(&sniffdata));
    if (printprofile.hdr!=LCRZO_PRINTTYPE_SYNTH) printf("\n");
  }
  while(LCRZO_TRUE);

  /* close everything */
  lcrzo_epr(lcrzo_sniff_close(&sniff));
  if (ret!=LCRZO_ERR_OK) { lcrzo_epr(lcrzo_err_print(ret)); }
  return (ret);
}

/*-------------------------------------------------------------*/
int shared_sniff_dumppause(int argc, char *argv[], lcrzo_int16 snifftype)
{ lcrzo_device device;
  lcrzo_uint32 mtu;
  lcrzo_sniff sniff;
  lcrzo_file file;
  lcrzo_wait waitsniff, waitread;
  lcrzo_bool eventsniff, eventread, displaydata;
  lcrzo_data data;
  lcrzo_int32 datasize;
  lcrzo_printprofile printprofile;
  lcrzo_int16 language;
  lcrzo_uint8 readchar;
  int ret;
  
  /* init device and mtu */
  if (argc>=2)
  { lcrzo_epr(shared_para_device_init_src(argv[1], device));
  }
  else
  { lcrzo_epr(lcrzo_device_value_pos(LCRZO_DEVICE_TYPE_BEST, 1, device));
  }
  lcrzo_epr(lcrzo_mtu_init_device(device, &mtu));

  /* get print profile */
  lcrzo_epr(lcrzo_global_get_language(&language));
  lcrzo_epr(lcrzo_printprofile_init_stdin((language==LCRZO_GLOBAL_LANG_FR)?
			   "Choisissez le profil d'affichage":
			   "Choose the print profile",
			   LCRZO_PRINTPROFILE_ARRAYDUMP, &printprofile));

  /* init sniff */
  if (argc<=2)
  { lcrzo_epr(lcrzo_sniff_init(device, mtu, snifftype, "", &sniff));
  }
  else
  { lcrzo_string snifffilter;
    lcrzo_epr(shared_para_string_init_snifffilter(argv[2], &snifffilter));
    lcrzo_epr(lcrzo_sniff_init(device, mtu, snifftype, snifffilter, &sniff));
    lcrzo_er(lcrzo_string_free2(&snifffilter));
  }

  /* how to exit */
  if (language==LCRZO_GLOBAL_LANG_FR)
    puts("Pressez q ou Q pour sortir. Pressez p ou P pour faire une pause.");
  else
    puts("Press q or Q to exit. Press p or P to pause.");
  lcrzo_epr(lcrzo_wait_init_sniff_read(&sniff, &waitsniff));
  lcrzo_epr(lcrzo_file_init_stream(stdin, &file));
  lcrzo_epr(lcrzo_wait_init_file_read(&file, &waitread));

  /* main loop */
  displaydata=LCRZO_TRUE;
  do
  { ret=lcrzo_wait_wait2(&waitread, &waitsniff, LCRZO_WAIT_INFINITE,
			 &eventread, &eventsniff);
    if (ret!=LCRZO_ERR_OK) break;
    /* a key was pressed */
    if (eventread)
    { /* read the character */
      lcrzo_epr(lcrzo_file_readm_fixed(&file, LCRZO_WAIT_INFINITE, 1, 
				       &data));
      readchar=data[0];
      lcrzo_er(lcrzo_data_free2(&data));
      /* test */
      if (readchar=='q' || readchar=='Q')
      { break;
      }
      else if (readchar=='p' || readchar=='P')
      { if (displaydata)
        { displaydata=LCRZO_FALSE;
          if (language==LCRZO_GLOBAL_LANG_FR)
	    puts("  Pressez p ou P pour arreter la pause.");
	  else
	    puts("  Press p or P to restart.");
	}
        else
        { displaydata=LCRZO_TRUE;
	}
      }
      else
      { if (language==LCRZO_GLOBAL_LANG_FR)
	  puts("  Pressez q ou Q pour sortir. Pressez p ou P pour une pause.");
	else
	  puts("  Press q or Q to exit. Press p or P to pause.");
      }
    }
    /* a packet was sniffed */
    if (eventsniff)
    { ret=lcrzo_sniff_nextm(&sniff, LCRZO_WAIT_INFINITE, &data, &datasize);
      if (ret!=LCRZO_ERR_OK) break;
      if (displaydata)
      { ret=shared_dns_packet_eth_print(data, datasize, printprofile);
	if (ret!=LCRZO_ERR_OK)
	{ lcrzo_epr(lcrzo_packet_print_eth(data, datasize, printprofile));
	}
        if (printprofile.hdr!=LCRZO_PRINTTYPE_SYNTH) printf("\n");
      }
      lcrzo_er(lcrzo_data_free2(&data));
    }
  }
  while(LCRZO_TRUE);

  /* close everything */
  lcrzo_epr(lcrzo_wait_close(&waitread));
  lcrzo_epr(lcrzo_file_close(&file));
  lcrzo_epr(lcrzo_wait_close(&waitsniff));
  lcrzo_epr(lcrzo_sniff_close(&sniff));
  if (ret!=LCRZO_ERR_OK) { lcrzo_epr(lcrzo_err_print(ret)); }
  return (ret);
}

/*-------------------------------------------------------------*/
int shared_sniff_dumpsaveeth(int argc, char *argv[], lcrzo_int16 snifftype)
{ lcrzo_device device;
  lcrzo_uint32 mtu;
  lcrzo_sniff sniff;
  lcrzo_record record;
  lcrzo_file file;
  lcrzo_wait waitsniff, waitread;
  lcrzo_bool eventsniff, eventread, wanttostop;
  lcrzo_data data;
  lcrzo_int32 datasize;
  lcrzo_printprofile printprofile1, printprofile2;
  lcrzo_int16 language;
  int ret;
  
  /* open the record */
  lcrzo_epr(lcrzo_record_open(&record, argv[1], LCRZO_RECORD_OPEN_WRITE));

  /* init device and mtu */
  if (argc>=3)
  { lcrzo_epr(shared_para_device_init_src(argv[2], device));
  }
  else
  { lcrzo_epr(lcrzo_device_value_pos(LCRZO_DEVICE_TYPE_BEST, 1, device));
  }
  lcrzo_epr(lcrzo_mtu_init_device(device, &mtu));

  /* get print profile */
  lcrzo_epr(lcrzo_global_get_language(&language));
  lcrzo_epr(lcrzo_printprofile_init_stdin((language==LCRZO_GLOBAL_LANG_FR)?
			   "Choisissez le profil d'affichage pour l'ecran":
			   "Choose the screen print profile",
			   LCRZO_PRINTPROFILE_ARRAYDUMP, &printprofile1));
  lcrzo_epr(lcrzo_printprofile_init_stdin_record((language==LCRZO_GLOBAL_LANG_FR)?
			   "Choisissez le profil d'affichage pour le record":
			   "Choose the record print profile",
			   LCRZO_PRINTPROFILE_HEXANCHMS, &printprofile2));

  /* init sniff */
  if (argc<=3)
  { lcrzo_epr(lcrzo_sniff_init(device, mtu, snifftype, "", &sniff));
  }
  else
  { lcrzo_string snifffilter;
    lcrzo_epr(shared_para_string_init_snifffilter(argv[3], &snifffilter));
    lcrzo_epr(lcrzo_sniff_init(device, mtu, snifftype, snifffilter, &sniff));
    lcrzo_er(lcrzo_string_free2(&snifffilter));
  }

  /* how to exit */
  lcrzo_epr(shared_misc_userstop1_printmsg());
  lcrzo_epr(lcrzo_wait_init_sniff_read(&sniff, &waitsniff));
  lcrzo_epr(lcrzo_file_init_stream(stdin, &file));
  lcrzo_epr(lcrzo_wait_init_file_read(&file, &waitread));

  /* main loop */
  do
  { ret=lcrzo_wait_wait2(&waitread, &waitsniff, LCRZO_WAIT_INFINITE,
			 &eventread, &eventsniff);
    if (ret!=LCRZO_ERR_OK) break;
    /* a key was pressed */
    if (eventread)
    { ret=shared_misc_userstop1_test(&file, LCRZO_TRUE, &wanttostop, NULL);
      if (ret!=LCRZO_ERR_OK) break;
      if (wanttostop) break;
    }
    /* a packet was sniffed */
    if (eventsniff)
    { ret=lcrzo_sniff_nextm(&sniff, LCRZO_WAIT_INFINITE, &data, &datasize);
      if (ret!=LCRZO_ERR_OK) break;
      ret=shared_dns_packet_eth_print(data, datasize, printprofile1);
      if (ret!=LCRZO_ERR_OK)
      { lcrzo_epr(lcrzo_packet_print_eth(data, datasize, printprofile1));
      }
      if (printprofile1.hdr!=LCRZO_PRINTTYPE_SYNTH) printf("\n");
      ret=lcrzo_record_write_eth(&record, data, datasize, printprofile2);
      lcrzo_er(lcrzo_data_free2(&data));
      if (ret!=LCRZO_ERR_OK) break;
    }
  }
  while(LCRZO_TRUE);

  /* close everything */
  lcrzo_epr(lcrzo_wait_close(&waitread));
  lcrzo_epr(lcrzo_file_close(&file));
  lcrzo_epr(lcrzo_wait_close(&waitsniff));
  lcrzo_epr(lcrzo_record_close(&record));
  lcrzo_epr(lcrzo_sniff_close(&sniff));
  if (ret!=LCRZO_ERR_OK) lcrzo_err_print(ret);
  return (ret);
}

/*-------------------------------------------------------------*/
int shared_sniff_dumpsaveip(int argc, char *argv[], lcrzo_int16 snifftype)
{ lcrzo_device device;
  lcrzo_uint32 mtu;
  lcrzo_sniff sniff;
  lcrzo_record record;
  lcrzo_file file;
  lcrzo_wait waitsniff, waitread;
  lcrzo_bool eventsniff, eventread, wanttostop;
  lcrzo_data data, ipdata;
  lcrzo_int32 datasize;
  lcrzo_uint16 ipdatasize;
  lcrzo_ipopt ipopt;
  lcrzo_uint8 ipoptsize;
  lcrzo_printprofile printprofile1, printprofile2;
  lcrzo_hdrlip hdrlip;
  lcrzo_int16 language;
  int ret;
  
  /* open the record */
  lcrzo_epr(lcrzo_record_open(&record, argv[1], LCRZO_RECORD_OPEN_WRITE));

  /* init device and mtu */
  if (argc>=3)
  { lcrzo_epr(shared_para_device_init_src(argv[2], device));
  }
  else
  { lcrzo_epr(lcrzo_device_value_pos(LCRZO_DEVICE_TYPE_BEST, 1, device));
  }
  lcrzo_epr(lcrzo_mtu_init_device(device, &mtu));

  /* get print profile */
  lcrzo_epr(lcrzo_global_get_language(&language));
  lcrzo_epr(lcrzo_printprofile_init_stdin((language==LCRZO_GLOBAL_LANG_FR)?
			   "Choisissez le profil d'affichage pour l'ecran":
			   "Choose the screen print profile",
			   LCRZO_PRINTPROFILE_ARRAYDUMP, &printprofile1));
  lcrzo_epr(lcrzo_printprofile_init_stdin_record((language==LCRZO_GLOBAL_LANG_FR)?
			   "Choisissez le profil d'affichage pour le record":
			   "Choose the record print profile",
			   LCRZO_PRINTPROFILE_HEXANCHMS, &printprofile2));

  /* init sniff */
  if (argc<=3)
  { lcrzo_epr(lcrzo_sniff_init(device, mtu, snifftype, "", &sniff));
  }
  else
  { lcrzo_string snifffilter;
    lcrzo_epr(shared_para_string_init_snifffilter(argv[3], &snifffilter));
    lcrzo_epr(lcrzo_sniff_init(device, mtu, snifftype, snifffilter, &sniff));
    lcrzo_er(lcrzo_string_free2(&snifffilter));
  }

  /* how to exit */
  lcrzo_epr(shared_misc_userstop1_printmsg());
  lcrzo_epr(lcrzo_wait_init_sniff_read(&sniff, &waitsniff));
  lcrzo_epr(lcrzo_file_init_stream(stdin, &file));
  lcrzo_epr(lcrzo_wait_init_file_read(&file, &waitread));

  /* main loop */
  do
  { ret=lcrzo_wait_wait2(&waitread, &waitsniff, LCRZO_WAIT_INFINITE,
			 &eventread, &eventsniff);
    if (ret!=LCRZO_ERR_OK) break;
    /* a key was pressed */
    if (eventread)
    { ret=shared_misc_userstop1_test(&file, LCRZO_TRUE, &wanttostop, NULL);
      if (ret!=LCRZO_ERR_OK) break;
      if (wanttostop) break;
    }
    /* a packet was sniffed */
    if (eventsniff)
    { ret=lcrzo_sniff_nextm(&sniff, LCRZO_WAIT_INFINITE, &data, &datasize);
      if (ret!=LCRZO_ERR_OK) break;
      /* only print if it's IP */
      ret=lcrzo_packet_decodem_ethipoptdata(data, datasize,
					    NULL, &hdrlip,
					    ipopt, &ipoptsize,
					    &ipdata, &ipdatasize);
      lcrzo_er(lcrzo_data_free2(&data));
      if ( ret==LCRZO_ERR_OK )
      { /* reconstruct the packet to print */
	ret=lcrzo_packet_initm_ipoptdata(hdrlip, ipopt, ipoptsize,
					 ipdata, ipdatasize,
					 &data, &datasize);
	lcrzo_er(lcrzo_data_free2(&ipdata));
	if (ret!=LCRZO_ERR_OK) break;
	ret=shared_dns_packet_ip_print(data, datasize, printprofile1);
        if (ret!=LCRZO_ERR_OK)
        { lcrzo_epr(lcrzo_packet_print_ip(data, datasize, printprofile1));
	}
	if (printprofile1.hdr!=LCRZO_PRINTTYPE_SYNTH) printf("\n");
	ret=lcrzo_record_write_ip(&record, data, datasize, printprofile2);
	lcrzo_er(lcrzo_data_free2(&data));
	if (ret!=LCRZO_ERR_OK) break;
      }
    }
  }
  while(LCRZO_TRUE);

  /* close everything */
  lcrzo_epr(lcrzo_wait_close(&waitread));
  lcrzo_epr(lcrzo_file_close(&file));
  lcrzo_epr(lcrzo_wait_close(&waitsniff));
  lcrzo_epr(lcrzo_record_close(&record));
  lcrzo_epr(lcrzo_sniff_close(&sniff));
  if (ret!=LCRZO_ERR_OK) lcrzo_err_print(ret);
  return (ret);
}

/*-------------------------------------------------------------*/
int shared_sniff_check(int argc, char *argv[], lcrzo_int16 snifftype)
{ lcrzo_device device;
  lcrzo_uint32 mtu;
  lcrzo_sniff sniff;
  lcrzo_data sniffdata;
  lcrzo_int32 sniffdatasize;
  int ret;
  
  /* init device and mtu */
  if (argc>=2)
  { lcrzo_epr(shared_para_device_init_src(argv[1], device));
  }
  else
  { lcrzo_epr(lcrzo_device_value_pos(LCRZO_DEVICE_TYPE_BEST, 1, device));
  }
  lcrzo_epr(lcrzo_mtu_init_device(device, &mtu));

  /* init sniff */
  if (argc<=2)
  { lcrzo_epr(lcrzo_sniff_init(device, mtu, snifftype, "", &sniff));
  }
  else
  { lcrzo_string snifffilter;
    lcrzo_epr(shared_para_string_init_snifffilter(argv[2], &snifffilter));
    lcrzo_epr(lcrzo_sniff_init(device, mtu, snifftype, snifffilter, &sniff));
    lcrzo_er(lcrzo_string_free2(&snifffilter));
  }

  /* main loop */
  do
  { ret=lcrzo_sniff_nextm(&sniff, LCRZO_WAIT_INFINITE, 
			  &sniffdata, &sniffdatasize);
    if (ret!=LCRZO_ERR_OK) break;
    ret=shared_sniff_check_verif(sniffdata, sniffdatasize);
    lcrzo_er(lcrzo_data_free2(&sniffdata));
    if (ret!=LCRZO_ERR_OK) break;
  }
  while(LCRZO_TRUE);

  /* close everything */
  lcrzo_epr(lcrzo_sniff_close(&sniff));
  if (ret!=LCRZO_ERR_OK) { lcrzo_epr(lcrzo_err_print(ret)); }
  return (ret);
}

/*-------------------------------------------------------------*/
int shared_sniff_check_verif(lcrzo_constdata packet,
			     lcrzo_int32 packetsize)
{ int ret;
  lcrzo_hdrleth hdrleth;
  lcrzo_hdrlip hdrlip;
  lcrzo_hdrludp hdrludp;
  lcrzo_hdrltcp hdrltcp;
  lcrzo_hdrlicmp hdrlicmp;
  lcrzo_data data, temp;
  lcrzo_int32 tempsize;
  lcrzo_ipopt ipopt;
  lcrzo_tcpopt tcpopt;
  lcrzo_uint8 ipoptsize, tcpoptsize;
  lcrzo_uint16 check1, check2, datasize;

  /* if it's IP+UDP */
  ret=lcrzo_packet_decodem_ethipoptudpdata(packet, packetsize,
					   &hdrleth, 
					   &hdrlip, ipopt, &ipoptsize,
					   &hdrludp, &data, &datasize);
  if (ret==LCRZO_ERR_OK)
  { printf("IP %04X\tUDP %04X\t\t", hdrlip.check, hdrludp.check);
    /* we save network values to compare them */
    check1=hdrlip.check;
    check2=hdrludp.check;
    /* the function has to compute checksums */
    hdrlip.check=LCRZO_HDRLIP_CHECK_DEFVAL;
    hdrludp.check=LCRZO_HDRLUDP_CHECK_DEFVAL; 
    lcrzo_epr(lcrzo_packet_initm_ipoptudpdata(hdrlip, ipopt, ipoptsize,
				    hdrludp, data, datasize,
				    &temp, &tempsize));
    lcrzo_er(lcrzo_data_free2(&data));
    lcrzo_epr(lcrzo_packet_decodem_ipoptudpdata(temp, tempsize,
				      &hdrlip, NULL, NULL,
				      &hdrludp, NULL, NULL));
    lcrzo_er(lcrzo_data_free2(&temp));
    /* compare */
    if ( check1==hdrlip.check &&
	 ((check2==hdrludp.check)||(hdrlip.offsetfrag|hdrlip.morefrag)) )
    { printf("ok\n");
    }
    else
    { printf("IP %04X\tUDP %04X\n", hdrlip.check, hdrludp.check);
      lcrzo_epr(lcrzo_packet_print_eth(packet, packetsize,
				       LCRZO_PRINTPROFILE_ARRAYDUMP));
    }


    return (LCRZO_ERR_OK);
  }

  /* if it's IP+TCP */
  ret=lcrzo_packet_decodem_ethipopttcpoptdata(packet, packetsize,
					      &hdrleth, 
					      &hdrlip, ipopt, &ipoptsize,
					      &hdrltcp, 
					      tcpopt, &tcpoptsize,
					      &data, &datasize);
  if (ret==LCRZO_ERR_OK)
  { printf("IP %04X\tTCP %04X\t\t", hdrlip.check, hdrltcp.check);
    check1=hdrlip.check;
    check2=hdrltcp.check;
    hdrlip.check=LCRZO_HDRLIP_CHECK_DEFVAL;
    hdrltcp.check=LCRZO_HDRLTCP_CHECK_DEFVAL; 
    lcrzo_epr(lcrzo_packet_initm_ipopttcpoptdata(hdrlip, ipopt, ipoptsize,
				       hdrltcp, tcpopt, tcpoptsize,
				       data, datasize, 
				       &temp, &tempsize));
    lcrzo_er(lcrzo_data_free2(&data));
    lcrzo_epr(lcrzo_packet_decodem_ipopttcpoptdata(temp, tempsize,
					 &hdrlip, NULL, NULL,
					 &hdrltcp, NULL, NULL, 
					 NULL, NULL));
    lcrzo_er(lcrzo_data_free2(&temp));
    if ( check1==hdrlip.check && 
	 ((check2==hdrltcp.check)||(hdrlip.offsetfrag|hdrlip.morefrag)) )
    { printf("ok\n");
    }
    else
    { printf("IP %04X\tTCP %04X\n", hdrlip.check, hdrltcp.check);
      lcrzo_epr(lcrzo_packet_print_eth(packet, packetsize,
				       LCRZO_PRINTPROFILE_ARRAYDUMP));
    }
    return (LCRZO_ERR_OK);
  }

  /* if it's IP+ICMP */
  ret=lcrzo_packet_decodem_ethipopticmpdata(packet, packetsize,
					    &hdrleth, 
					    &hdrlip, ipopt, &ipoptsize,
					    &hdrlicmp, 
					    &data, &datasize);
  if (ret==LCRZO_ERR_OK)
  { printf("IP %04X\tICMP %04X\t\t", hdrlip.check, hdrlicmp.check);
    check1=hdrlip.check;
    check2=hdrlicmp.check;
    hdrlip.check=LCRZO_HDRLIP_CHECK_DEFVAL;
    hdrlicmp.check=LCRZO_HDRLICMP_CHECK_DEFVAL; 
    lcrzo_epr(lcrzo_packet_initm_ipopticmpdata(hdrlip, ipopt, ipoptsize,
				     hdrlicmp, data, datasize,
				     &temp, &tempsize));
    lcrzo_er(lcrzo_data_free2(&data));
    lcrzo_epr(lcrzo_packet_decodem_ipopticmpdata(temp, tempsize,
				       &hdrlip, NULL, NULL,
				       &hdrlicmp,
				       NULL, NULL));
    lcrzo_er(lcrzo_data_free2(&temp));
    if ( check1==hdrlip.check &&
	 ((check2==hdrlicmp.check)||(hdrlip.offsetfrag|hdrlip.morefrag)) )
    { printf("ok\n");
    }
    else
    { printf("IP %04X\tICMP %04X\n", hdrlip.check, hdrlicmp.check);
      lcrzo_epr(lcrzo_packet_print_eth(packet, packetsize,
				       LCRZO_PRINTPROFILE_ARRAYDUMP));
    }
    return (LCRZO_ERR_OK);
  }

  /* if it's IP only */
  ret=lcrzo_packet_decodem_ethipoptdata(packet, packetsize,
					&hdrleth, 
					&hdrlip, ipopt, &ipoptsize,
					&data, &datasize);
  if (ret==LCRZO_ERR_OK)
  { printf("IP %04X\t\t\t", hdrlip.check);
    check1=hdrlip.check;
    hdrlip.check=LCRZO_HDRLIP_CHECK_DEFVAL;
    lcrzo_epr(lcrzo_packet_initm_ipoptdata(hdrlip, ipopt, ipoptsize,
				 data, datasize,
				 &temp, &tempsize));
    lcrzo_er(lcrzo_data_free2(&data));
    lcrzo_epr(lcrzo_packet_decodem_ipoptdata(temp, tempsize,
				   &hdrlip, NULL, NULL,
				   NULL, NULL));
    lcrzo_er(lcrzo_data_free2(&temp));
    if ( check1==hdrlip.check )
    { printf("ok\n");
    }
    else
    { printf("IP %04X\n", hdrlip.check);
      lcrzo_epr(lcrzo_packet_print_eth(packet, packetsize, 
				       LCRZO_PRINTPROFILE_ARRAYDUMP));
    }
    return (LCRZO_ERR_OK);
  }

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_sniff_count(int argc, char *argv[], lcrzo_int16 snifftype)
{ lcrzo_device device;
  lcrzo_uint32 mtu;
  lcrzo_sniff sniff;
  lcrzo_int16 language;
  int ret, i;
  
  /* init device and mtu */
  if (argc>=2)
  { lcrzo_epr(shared_para_device_init_src(argv[1], device));
  }
  else
  { lcrzo_epr(lcrzo_device_value_pos(LCRZO_DEVICE_TYPE_BEST, 1, device));
  }
  lcrzo_epr(lcrzo_mtu_init_device(device, &mtu));

  /* init sniff */
  if (argc<=2)
  { lcrzo_epr(lcrzo_sniff_init(device, mtu, snifftype, "", &sniff));
  }
  else
  { lcrzo_string snifffilter;
    lcrzo_epr(shared_para_string_init_snifffilter(argv[2], &snifffilter));
    lcrzo_epr(lcrzo_sniff_init(device, mtu, snifftype, snifffilter, &sniff));
    lcrzo_er(lcrzo_string_free2(&snifffilter));
  }

  /* main loop */
  lcrzo_epr(lcrzo_global_get_language(&language));
  i=0;
  do
  { ret=lcrzo_sniff_nextm(&sniff, LCRZO_WAIT_INFINITE, NULL, NULL);
    if (ret!=LCRZO_ERR_OK) break;
    i++;
    if (!(i%100))
    { printf((language==LCRZO_GLOBAL_LANG_FR)?"Paquet %d\n":"Packet %d\n", i);
    }
  }
  while(LCRZO_TRUE);

  /* close everything */
  lcrzo_epr(lcrzo_sniff_close(&sniff));
  if (ret!=LCRZO_ERR_OK) lcrzo_err_print(ret);
  return (ret);
}

/*-------------------------------------------------------------*/
int shared_sniff_printnewethip(lcrzo_list *plist,
			       lcrzo_constetha etha,
			       lcrzo_constipa ipa)
{ shared_sniff_ethip ethip;
  int ret;

  /* put data in structure */
  memcpy(ethip.eth, etha, LCRZO_ETHA_MAXBYTES);
  memcpy(ethip.ip, ipa, LCRZO_IPA_MAXBYTES);

  /* search structure in list */
  ret=lcrzo_list_searchitem_all(plist, &ethip, NULL);
  if ( ret!=LCRZO_ERR_OK )
  { /* element wasn't found : print it and add it */ 
    lcrzo_epr(lcrzo_etha_print("", etha, "\t"));
    lcrzo_epr(lcrzo_ipa_print("", ipa, "\n"));
    lcrzo_epr(lcrzo_list_add_first(plist, &ethip));
  }

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_sniff_decodearp_ethip(lcrzo_list *plist, 
				 lcrzo_constdata ethdata,
				 lcrzo_int32 ethdatasize)
{ lcrzo_hdrleth hdreth;
  lcrzo_hdrlarp hdrarp;
  int ret;

  /*lcrzo_epr(lcrzo_packet_print_eth(ethdata, ethdatasize, 
              LCRZO_PRINTPROFILE_ARRAYDUMP));*/
  
  /* try to decode arp */
  ret=lcrzo_packet_decode_etharp(ethdata, ethdatasize, &hdreth, &hdrarp);
  if ( ret!=LCRZO_ERR_OK )
    return(LCRZO_ERR_OK);
  if ( hdreth.type != LCRZO_HDRLETH_TYPE_ARP &&
       hdreth.type != LCRZO_HDRLETH_TYPE_RARP )
    return(LCRZO_ERR_OK);

  /* analyze an ARP request */
  if ( hdrarp.op==1 )
  { lcrzo_epr(shared_sniff_printnewethip(plist, hdrarp.hw_src, 
					 hdrarp.prot_src));
  }

  /* analyze an ARP reply */
  if ( hdrarp.op==2 )
  { lcrzo_epr(shared_sniff_printnewethip(plist, hdrarp.hw_src,
					 hdrarp.prot_src));
    lcrzo_epr(shared_sniff_printnewethip(plist, hdrarp.hw_dst,
					 hdrarp.prot_dst));
  }

  /* analyze a RARP reply */
  if ( hdrarp.op==4 )
  { lcrzo_epr(shared_sniff_printnewethip(plist, hdrarp.hw_src,
					 hdrarp.prot_src));
    lcrzo_epr(shared_sniff_printnewethip(plist, hdrarp.hw_dst,
					 hdrarp.prot_dst));
  }
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_sniff_decodeip_ethip(lcrzo_list *plist, 
				lcrzo_constdata ethdata,
				lcrzo_int32 ethdatasize)
{ lcrzo_hdrleth hdreth;
  lcrzo_hdrlip hdrip;
  lcrzo_ipa ipa;
  int ret;

  /*lcrzo_epr(lcrzo_packet_print_eth(ethdata, ethdatasize, 
              LCRZO_PRINTPROFILE_ARRAYDUMP));*/
  
  /* try to decode ip */
  ret=lcrzo_packet_decodem_ethipoptdata(ethdata, ethdatasize, 
					&hdreth, &hdrip, NULL, NULL,
					NULL, NULL);
  if ( ret!=LCRZO_ERR_OK )
    return(LCRZO_ERR_OK);

  /* eventually print new addresses */
  lcrzo_epr(lcrzo_ipa_init_ipl(hdrip.saddr, ipa));
  lcrzo_epr(shared_sniff_printnewethip(plist, hdreth.src, ipa));
  lcrzo_epr(lcrzo_ipa_init_ipl(hdrip.daddr, ipa));
  lcrzo_epr(shared_sniff_printnewethip(plist, hdreth.dst, ipa));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_sniff_printnewipport(lcrzo_list *plist,
				lcrzo_uint8 type,
				lcrzo_constipa ipa,
				lcrzo_uint16 port)
{ shared_sniff_ipport ipport;
  int ret;

  /* put data in structure */
  ipport.type=type;
  memcpy(ipport.ip, ipa, LCRZO_IPA_MAXBYTES);
  ipport.port=port;

  /* search structure in list */
  ret=lcrzo_list_searchitem_all(plist, &ipport, NULL);
  if ( ret!=LCRZO_ERR_OK )
  { /* element wasn't found : print it and add it */
    if ( type==1 )
      printf("UDP\t");
    else if ( type==2 )
      printf("TCP\t");
    else
      printf("???\t");
    lcrzo_epr(lcrzo_ipa_print("", ipa, "\t"));
    printf("%d\n", port);
    lcrzo_epr(lcrzo_list_add_first(plist, &ipport));
  }

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_sniff_decodeudp_ipport(lcrzo_list *plist, 
				  lcrzo_constdata ethdata,
				  lcrzo_int32 ethdatasize)
{ lcrzo_hdrlip hdrip;
  lcrzo_hdrludp hdrudp;
  lcrzo_ipa ipa;
  int ret;

  /*lcrzo_epr(lcrzo_packet_print_eth(ethdata, ethdatasize, 
              LCRZO_PRINTPROFILE_ARRAYDUMP));*/
  
  /* try to decode it */
  ret=lcrzo_packet_decodem_ethipoptudpdata(ethdata, ethdatasize, 
					   NULL, &hdrip, NULL, NULL,
					   &hdrudp, NULL, NULL);
  if ( ret!=LCRZO_ERR_OK )
    return(LCRZO_ERR_OK);

  /* eventually print new addresses */
  lcrzo_epr(lcrzo_ipa_init_ipl(hdrip.daddr, ipa));
  lcrzo_epr(shared_sniff_printnewipport(plist, 2, ipa, hdrudp.dport));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_sniff_decodetcp_ipport(lcrzo_list *plist, 
				  lcrzo_constdata ethdata,
				  lcrzo_int32 ethdatasize)
{ lcrzo_hdrlip hdrip;
  lcrzo_hdrltcp hdrtcp;
  lcrzo_ipa ipa;
  int ret;

  /*lcrzo_epr(lcrzo_packet_print_eth(ethdata, ethdatasize, 
              LCRZO_PRINTPROFILE_ARRAYDUMP));*/
  
  /* try to decode it */
  ret=lcrzo_packet_decodem_ethipopttcpoptdata(ethdata, ethdatasize, 
					      NULL, &hdrip, NULL, NULL,
					      &hdrtcp, NULL, NULL,
					      NULL, NULL);
  if ( ret!=LCRZO_ERR_OK )
    return(LCRZO_ERR_OK);

  /* only count SYNACK */
  if ( ! ( hdrtcp.syn && hdrtcp.ack && !hdrtcp.fin && !hdrtcp.rst ) )
    return(LCRZO_ERR_OK);

  /* eventually print new addresses */
  lcrzo_epr(lcrzo_ipa_init_ipl(hdrip.saddr, ipa));
  lcrzo_epr(shared_sniff_printnewipport(plist, 2, ipa, hdrtcp.sport));

  return(LCRZO_ERR_OK);
}


