/*
		                 LCRZOEX
                             Network toolbox
             Copyright(c) 1999,2000,2001,2002 Laurent Constantin
                                  -----

  Main server    : http://www.laurentconstantin.com/
  Backup servers : http://go.to/laurentconstantin/
                   http://laurentconstantin.est-la.com/
                   http://laurentconstantin.free.fr/
                   http://membres.lycos.fr/lauconstantin/
  [my current email address is on the web servers]

                                  -----
  This file is part of Lcrzoex.

  Lcrzoex is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Lcrzoex is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details (http://www.gnu.org/).

------------------------------------------------------------------------
*/

#include <lcrzo.h>
#include "../lcrzoex.h"


/*-------------------------------------------------------------*/
int shared_sock_banner(lcrzo_sock *psock)
{ lcrzo_data data;
  lcrzo_int32 datasize;
 
  /* first, we read the banner */
  lcrzo_epr(lcrzo_sock_readm(psock, LCRZO_WAIT_INFINITE, &data, &datasize));
  lcrzo_epr(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_DUMP));
  lcrzo_er(lcrzo_data_free2(&data));

  /* then, we send a 'quit' */
  lcrzo_epr(lcrzo_data_initm_text("QUIT\n", &data, &datasize));
  lcrzo_epr(lcrzo_sock_write(psock, data, datasize));
  lcrzo_er(lcrzo_data_free2(&data));

  /* close the socket */
  lcrzo_epr(lcrzo_sock_close(psock));
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_sock_stdin(lcrzo_sock *psock,
		      lcrzo_bool isanudpserver)
{ lcrzo_data data;
  lcrzo_int32 datasize;
  lcrzo_bool writenotpossible, readnotpossible, firsttime, userwantstoexit;
  lcrzo_int16 language;
  int ret;

  lcrzo_epr(lcrzo_global_get_language(&language));

  /* wait for the client */
  if (isanudpserver)
  { if ( language==LCRZO_GLOBAL_LANG_FR )
      puts("\nNous attendons que le client ecrive.");
    else
      puts("\nWe wait for the client to write.");
    lcrzo_epr(lcrzo_sock_readm(psock, LCRZO_WAIT_INFINITE, &data, &datasize));
    if ( datasize!=0 )
    { lcrzo_epr(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_DUMP));
    }
    lcrzo_er(lcrzo_data_free2(&data));
  }

  /* main loop */
  readnotpossible=LCRZO_FALSE;
  writenotpossible=LCRZO_FALSE;
  firsttime=LCRZO_TRUE;
  ret=LCRZO_ERR_OK;
  while(LCRZO_TRUE)
  { /* look if data was received */
    while (LCRZO_TRUE)
    { ret=lcrzo_sock_readm(psock, LCRZO_WAIT_NODELAY, &data, &datasize);
      if (ret!=LCRZO_ERR_OK) break;
      if ( datasize!=0 )
      { lcrzo_epr(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_DUMP));
      }
      lcrzo_er(lcrzo_data_free2(&data));
    }
    if (ret!=LCRZO_ERR_OK && ret!=LCRZO_ERR_OKTEMPDATAEND)
    { readnotpossible=LCRZO_TRUE;
      break;
    }
    /* ask if user wants to quit */
    if (!firsttime)
    { ret=shared_misc_userstop3_printmsgtest(&userwantstoexit);
      if (ret!=LCRZO_ERR_OK) break;
      if (userwantstoexit) break;
    }
    firsttime=LCRZO_FALSE;
    /* ask user if he wants to send data */
    ret=lcrzo_data_initm_mixed_stdin((language==LCRZO_GLOBAL_LANG_FR)?
				     "Vos donnees mixtes":
				     "Your mixed data",
				     "'hello' 0D0A", 
				     &data, &datasize);
    if (ret!=LCRZO_ERR_OK) break;
    /* write data */
    if ( datasize!=0 )
    { ret=lcrzo_sock_write(psock, data, datasize);
      if (ret!=LCRZO_ERR_OK)
      { writenotpossible=LCRZO_TRUE;
        lcrzo_er(lcrzo_data_free2(&data));
        break;
      }
    }
    lcrzo_er(lcrzo_data_free2(&data));
  }

  /* print errors */
  if (readnotpossible)
  { if ( language==LCRZO_GLOBAL_LANG_FR )
      puts("\nLa lecture sur la socket n'est plus possible.");
    else
      puts("\nWe cannot read from the socket anymore.");
  }
  else if (writenotpossible)
  { if ( language==LCRZO_GLOBAL_LANG_FR )
      puts("\nL'ecriture sur la socket n'est plus possible.");
    else 
      puts("\nWe cannot write to the socket anymore.");
  }
  else
  { if (ret!=LCRZO_ERR_OK) { lcrzo_epr(lcrzo_err_print(ret)); }
  }

  /* close */
  lcrzo_epr(lcrzo_sock_close(psock));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_sock_pipe(lcrzo_sock *psock,
		     lcrzo_bool isanudpserver,
		     lcrzo_bool proposecleantermination)
{ lcrzo_data data;
  lcrzo_int32 datasize;
  lcrzo_bool writenotpossible, readnotpossible, userwantstoexit;
  lcrzo_file stdinfile;
  lcrzo_wait waitsock, waitstdin;
  lcrzo_bool eventsock, eventstdin;
  lcrzo_int16 language;
  int ret;

  lcrzo_epr(lcrzo_global_get_language(&language));

  /* wait for the client */
  if (isanudpserver)
  { if (proposecleantermination)
    { if ( language==LCRZO_GLOBAL_LANG_FR )
        puts("\nNous attendons que le client ecrive.");
      else
        puts("\nWe wait for the client to write.");
    }
    lcrzo_epr(lcrzo_sock_readm(psock, LCRZO_WAIT_INFINITE, &data, &datasize));
    if ( datasize!=0 )
    { lcrzo_epr(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_TEXT));
    }
    lcrzo_er(lcrzo_data_free2(&data));
  }

  /* wait events */
  if (proposecleantermination)
  { lcrzo_epr(shared_misc_userstop2_printmsg());
  }
  lcrzo_epr(lcrzo_wait_init_sock_read(psock, &waitsock));
  lcrzo_epr(lcrzo_file_init_fd(0, &stdinfile));
  lcrzo_epr(lcrzo_wait_init_file_read_line(&stdinfile, &waitstdin));

  /* main loop */
  readnotpossible=LCRZO_FALSE;
  writenotpossible=LCRZO_FALSE;
  userwantstoexit=LCRZO_FALSE;
  ret=LCRZO_ERR_OK;
  while(LCRZO_TRUE)
  { ret=lcrzo_wait_wait2(&waitsock, &waitstdin, LCRZO_WAIT_INFINITE,
                         &eventsock, &eventstdin);
    if (ret!=LCRZO_ERR_OK) break;

    /* an event from the socket */
    if (eventsock)
    { /* look if data was received */
      ret=lcrzo_sock_readm(psock, LCRZO_WAIT_INFINITE, &data, &datasize);
      if (ret!=LCRZO_ERR_OK)
      { readnotpossible=LCRZO_TRUE;
        break;
      }
      if ( datasize!=0 )
      { lcrzo_epr(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_TEXT));
      }
      lcrzo_er(lcrzo_data_free2(&data));
    }
    /* an event from stdin */
    if (eventstdin)
    { ret=lcrzo_file_readm_line(&stdinfile, LCRZO_WAIT_INFINITE, 
				&data, &datasize);
      if (ret!=LCRZO_ERR_OK)
      { writenotpossible=LCRZO_TRUE;
        break;
      }
      if (proposecleantermination)
      { ret=shared_misc_userstop2_test(data, datasize, &userwantstoexit);
        if ( ret!=LCRZO_ERR_OK || userwantstoexit )
        { lcrzo_er(lcrzo_data_free2(&data));
          break;
        }
      }
      /* write data */
      ret=lcrzo_data_appendm_text("\r\n", datasize, &data, &datasize);
      if (ret!=LCRZO_ERR_OK)
      { lcrzo_er(lcrzo_data_free2(&data));
	break;
      }
      ret=lcrzo_sock_write(psock, data, datasize);
      lcrzo_er(lcrzo_data_free2(&data));
      if (ret!=LCRZO_ERR_OK)
      { writenotpossible=LCRZO_TRUE;
	break;
      }
    }
  }

  /* print errors */
  if (readnotpossible)
  { if (proposecleantermination)
    { if ( language==LCRZO_GLOBAL_LANG_FR )
        puts("\nLa lecture sur la socket n'est plus possible.");
      else
        puts("\nWe cannot read from the socket anymore.");
    }
  }
  else if (writenotpossible)
  { if (proposecleantermination)
    { if ( language==LCRZO_GLOBAL_LANG_FR )
        puts("\nL'ecriture sur la socket n'est plus possible.");
      else 
        puts("\nWe cannot write to the socket anymore.");
    }
  }
  else
  { if (ret!=LCRZO_ERR_OK) lcrzo_err_print(ret);
  }

  /* close */
  lcrzo_epr(lcrzo_wait_close(&waitsock));
  lcrzo_epr(lcrzo_wait_close(&waitstdin));
  lcrzo_epr(lcrzo_file_close(&stdinfile));
  lcrzo_epr(lcrzo_sock_close(psock));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_sock_echo(lcrzo_sock *psock)
{ lcrzo_data data;
  lcrzo_int32 datasize;
  lcrzo_bool writenotpossible, readnotpossible;
  lcrzo_int16 language;
  int ret;

  lcrzo_epr(lcrzo_global_get_language(&language));

  /* main loop */
  readnotpossible=LCRZO_FALSE;
  writenotpossible=LCRZO_FALSE;
  while(LCRZO_TRUE)
  { ret=lcrzo_sock_readm(psock, LCRZO_WAIT_INFINITE, &data, &datasize);
    if ( ret!=LCRZO_ERR_OK )
    { readnotpossible=LCRZO_TRUE;
      break;
    }
    if ( datasize!=0 )
    { if ( language==LCRZO_GLOBAL_LANG_FR )
	puts("On vient de lire :");
      else
        puts("We read :");
      ret=lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_DUMP);
      if ( ret!=LCRZO_ERR_OK ) break;
      ret=lcrzo_sock_write(psock, data, datasize);
      if (ret!=LCRZO_ERR_OK)
      { writenotpossible=LCRZO_TRUE;
        lcrzo_er(lcrzo_data_free2(&data));
        break;
      }
    }
    lcrzo_er(lcrzo_data_free2(&data));
  }

  /* print errors */
  if (readnotpossible)
  { if ( language==LCRZO_GLOBAL_LANG_FR )
      puts("\nLa lecture sur la socket n'est plus possible.");
    else
      puts("\nWe cannot read from the socket anymore.");
  }
  if (writenotpossible)
  { if ( language==LCRZO_GLOBAL_LANG_FR )
      puts("\nL'ecriture sur la socket n'est plus possible.");
    else 
      puts("\nWe cannot write to the socket anymore.");
  }

  /* close */
  lcrzo_er(lcrzo_sock_close(psock));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_sock_display(lcrzo_sock *psock)
{ lcrzo_data data;
  lcrzo_int32 datasize;
  lcrzo_etha clienteth;
  lcrzo_ipl clientip;
  lcrzo_uint16 clientport;
  lcrzo_int16 language;
  int ret;

  lcrzo_epr(lcrzo_global_get_language(&language));

  while(LCRZO_TRUE)
  { ret=lcrzo_sock_readm(psock, LCRZO_WAIT_INFINITE, &data, &datasize);
    if ( ret!=LCRZO_ERR_OK )
    { if ( language==LCRZO_GLOBAL_LANG_FR )
        puts("\nLa lecture sur la socket n'est plus possible.");
      else
	puts("\nWe cannot read from the socket anymore.");
      break;
    }
    if ( datasize!=0 )
    { ret=lcrzo_sock_get_infos(psock, NULL, NULL, NULL, NULL,
			       clienteth, &clientip, &clientport);
      if ( ret==LCRZO_ERR_OK )
      { if ( language==LCRZO_GLOBAL_LANG_FR )
	  printf("On vient de lire");
        else
          printf("We read");
        ret=lcrzo_etha_print(" (client ", clienteth, "-");
	if ( ret!=LCRZO_ERR_OK ) { lcrzo_er(lcrzo_data_free2(&data)); break; }
	ret=lcrzo_ipl_print("", clientip, "-");
	if ( ret!=LCRZO_ERR_OK ) { lcrzo_er(lcrzo_data_free2(&data)); break; }
	printf("%d) :\n", clientport);
      }
      ret=lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_DUMP);
      if ( ret!=LCRZO_ERR_OK ) { lcrzo_er(lcrzo_data_free2(&data)); break; }
    }
    lcrzo_er(lcrzo_data_free2(&data));
  }

  /* close */
  lcrzo_er(lcrzo_sock_close(psock));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_socktcpmulser_echo(lcrzo_sock *psock, lcrzo_ptr pinfos)
{ lcrzo_data data;
  lcrzo_int32 datasize;
  lcrzo_etha clienteth;
  lcrzo_ipl clientip;
  lcrzo_uint16 clientport;
  lcrzo_int16 language;
  int ret;

  lcrzo_epr(lcrzo_global_get_language(&language));

  while(LCRZO_TRUE)
  { ret=lcrzo_sock_readm(psock, LCRZO_WAIT_INFINITE, &data, &datasize);
    if ( ret!=LCRZO_ERR_OK )
    { if ( language==LCRZO_GLOBAL_LANG_FR )
        puts("\nLa lecture sur la socket n'est plus possible.");
      else
	puts("\nWe cannot read from the socket anymore.");
      return(LCRZO_ERR_OK);
    }
    if ( datasize!=0 )
    { /* print information about the client */
      lcrzo_epr(lcrzo_sock_get_infos(psock, NULL,
				     NULL, NULL, NULL,
				     clienteth, &clientip, &clientport));
       if ( language==LCRZO_GLOBAL_LANG_FR )
	printf("On a lu");
      else
        printf("We read");
      lcrzo_epr(lcrzo_etha_print(" (client ", clienteth, "-"));
      lcrzo_epr(lcrzo_ipl_print("", clientip, "-"));
      printf("%d) :\n", clientport);
      /* print the read data */
      lcrzo_epr(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_DUMP));
      /* write back a copy to the socket */
      lcrzo_epr(lcrzo_sock_write(psock, 
				 data, datasize));
      /* if we read 'QUIT', we exit */
      if ( lcrzo_data_search_text(data, datasize, LCRZO_FALSE,
				  +1, -1, "QUIT",
				  NULL, NULL, NULL, NULL)
	   == LCRZO_ERR_OK )
      { if ( language==LCRZO_GLOBAL_LANG_FR )
	  puts("Le client quitte");
        else
	  puts("The client sent a QUIT");
        return(LCRZO_ERR_OK);
      }
    }
    lcrzo_er(lcrzo_data_free2(&data));
  }

  /* do not need to close */
  return(LCRZO_ERR_OK);
  pinfos=pinfos; /*just for the compiler warning*/
}


/*-------------------------------------------------------------*/
int shared_sock_ipctcpser_real_init(lcrzo_ipl ipl,
				    shared_sock_ipctcpser *psits,
				    lcrzo_ipc *pipc)
{ return(shared_sock_ipctcpser_real2_init(ipl, 0, NULL, 0, psits, pipc));
}

/*-------------------------------------------------------------*/
typedef struct
{ lcrzo_ipl ipl;
  lcrzo_uint16 port;
  lcrzo_ipopt ipopt;
  lcrzo_uint8 ipoptsize;
  lcrzo_ipc ipc;
} shared_priv_sock_ipctcpser;

/*-------------------------------------------------------------*/
int shared_sock_ipctcpser_child(lcrzo_ptr infos, 
				lcrzo_int32 infossize, 
				lcrzo_process *pparentprocess);
int shared_sock_ipctcpser_child(lcrzo_ptr infos, 
				lcrzo_int32 infossize, 
				lcrzo_process *pparentprocess)
{ shared_priv_sock_ipctcpser *pspsits=(shared_priv_sock_ipctcpser *)infos;
  lcrzo_data data;
  lcrzo_int32 datasize;
  lcrzo_sock sock;
  lcrzo_wait waitipc, waitsock, waitipcend, waitsockend;
  lcrzo_bool eventipc, eventsock, eventipcend, eventsockend;
  int ret;

  /* check for correct usage of this function */
  if (infossize!=sizeof(shared_priv_sock_ipctcpser))
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);

  /* close parent process handle because we do not need it */
  lcrzo_epr(lcrzo_process_close(pparentprocess));

  /* set IPC */
  lcrzo_epr(lcrzo_ipc_currentprocessissideb(&pspsits->ipc));

  /* synchro */
  lcrzo_er(lcrzo_data_initm_text("synchro\n", &data, &datasize));
  lcrzo_epr(lcrzo_ipc_write(&pspsits->ipc, data, datasize));
  lcrzo_er(lcrzo_data_free2(&data));

  /* start the server */
  lcrzo_epr(lcrzo_sock_tcpser_real2(pspsits->ipl, pspsits->port,
				    pspsits->ipopt, pspsits->ipoptsize,
				    &sock));

  /* initialize wait */
  lcrzo_epr(lcrzo_wait_init_ipc_read(&pspsits->ipc, &waitipc));
  lcrzo_epr(lcrzo_wait_init_sock_read(&sock, &waitsock));
  lcrzo_epr(lcrzo_wait_init_ipc_end(&pspsits->ipc, &waitipcend));
  lcrzo_epr(lcrzo_wait_init_sock_end(&sock, &waitsockend));

  /* copy input to output, and output to input */
  ret=LCRZO_ERR_OK;
  while(LCRZO_TRUE)
  { ret=lcrzo_wait_wait4(&waitipc, &waitsock, &waitipcend, &waitsockend,
                         LCRZO_WAIT_INFINITE,
                         &eventipc, &eventsock, &eventipcend, &eventsockend);
    if (ret!=LCRZO_ERR_OK) break;
    /*printf("%d %d  %d %d\n", eventsock, eventipc, 
      eventsockend, eventipcend);*/
    if ( ( eventipcend || eventsockend ) && !eventipc && !eventsock ) break;
    if ( eventipc )
    { ret=lcrzo_ipc_readm(&pspsits->ipc, LCRZO_WAIT_INFINITE, &data,&datasize);
      if (ret!=LCRZO_ERR_OK) break;
      ret=lcrzo_sock_write(&sock, data, datasize);
      lcrzo_er(lcrzo_data_free2(&data));
      if (ret!=LCRZO_ERR_OK) break;
    }
    if ( eventsock )
    { ret=lcrzo_sock_readm(&sock, LCRZO_WAIT_INFINITE,&data,&datasize);
      if (ret!=LCRZO_ERR_OK) break;
      ret=lcrzo_ipc_write(&pspsits->ipc, data, datasize);
      lcrzo_er(lcrzo_data_free2(&data));
      if (ret!=LCRZO_ERR_OK) break;
    }
  }
  if ( ret==LCRZO_ERR_OKDATAEND ) ret=LCRZO_ERR_OK;
  if ( ret!=LCRZO_ERR_OK )
  { lcrzo_epr(lcrzo_err_print(ret));
  }

  /* close wait */
  lcrzo_epr(lcrzo_wait_close(&waitipc));
  lcrzo_epr(lcrzo_wait_close(&waitsock));
  lcrzo_epr(lcrzo_wait_close(&waitipcend));
  lcrzo_epr(lcrzo_wait_close(&waitsockend));
  lcrzo_epr(lcrzo_sock_close(&sock));
  lcrzo_epr(lcrzo_ipc_close(&pspsits->ipc));

  return(ret);
}

/*-------------------------------------------------------------*/
int shared_sock_ipctcpser_real2_init(lcrzo_ipl ipl,
				     lcrzo_uint16 port,
				     lcrzo_constipopt ipopt,
				     lcrzo_uint8 ipoptsize,
				     shared_sock_ipctcpser *psits,
				     lcrzo_ipc *pipc)
{ shared_priv_sock_ipctcpser spsits;
  
  /* set parameters */
  spsits.ipl=ipl;
  spsits.port=port;
  memcpy(spsits.ipopt, ipopt, ipoptsize);
  spsits.ipoptsize=ipoptsize;

  /* open communication IPC */
  lcrzo_epr(lcrzo_ipc_init(pipc));
  spsits.ipc=*pipc;

  /* start child process */
  lcrzo_epr(lcrzo_process_other_init(&shared_sock_ipctcpser_child, 
                                     &spsits, sizeof(spsits), 
                                     &psits->child));
  lcrzo_epr(lcrzo_ipc_currentprocessissidea(pipc));

  /* synchro */
  lcrzo_epr(lcrzo_ipc_readm(pipc, LCRZO_WAIT_INFINITE, NULL, NULL));
  /* also wait a little to ensure tcp server is ready */
  lcrzo_epr(lcrzo_time_sleep_msec(10));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_sock_ipctcpser_close(shared_sock_ipctcpser *psits)
{ int ret;

  lcrzo_epr(shared_lcrzoex_debug_processotherexit_wanttowarn(LCRZO_FALSE));
  lcrzo_epr(lcrzo_process_other_exit(&psits->child));
  lcrzo_epr(shared_lcrzoex_debug_processotherexit_wanttowarn(LCRZO_TRUE));
  lcrzo_epr(lcrzo_process_other_wait_end(&psits->child, LCRZO_WAIT_INFINITE, 
                                         NULL, &ret));
  lcrzo_epr(lcrzo_process_close(&psits->child));
  if (ret==LCRZO_ERR_OKPROCESSABRUPTTERM) ret=LCRZO_ERR_OK;
  return(ret);
}

