/*
		                 LCRZOEX
                             Network toolbox
             Copyright(c) 1999,2000,2001,2002 Laurent Constantin
                                  -----

  Main server    : http://www.laurentconstantin.com/
  Backup servers : http://go.to/laurentconstantin/
                   http://laurentconstantin.est-la.com/
                   http://laurentconstantin.free.fr/
                   http://membres.lycos.fr/lauconstantin/
  [my current email address is on the web servers]

                                  -----
  This file is part of Lcrzoex.

  Lcrzoex is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Lcrzoex is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details (http://www.gnu.org/).

------------------------------------------------------------------------
*/

#include <lcrzo.h>
#include "../lcrzoex.h"

/*-------------------------------------------------------------*/
typedef struct
{ lcrzo_ipl serveripl;
  lcrzo_uint16 serverport;
  shared_telnet telnet;
} shared_priv_telnet;

/*-------------------------------------------------------------*/
int shared_priv_telnet_writeuser(shared_telnet *ptelnet,
				 lcrzo_sock *psock);
int shared_priv_telnet_writeuser(shared_telnet *ptelnet,
				 lcrzo_sock *psock)
{ lcrzo_data data, sentdata;
  lcrzo_int32 datasize, sentdatasize;
  int ret, i;

  /* read */
  ret=lcrzo_ipc_readm(&(ptelnet->ipcdata), LCRZO_WAIT_NODELAY, 
		      &data, &datasize);
  if ( ret==LCRZO_ERR_OKTEMPDATAEND ) return(LCRZO_ERR_OK);
  else if (ret!=LCRZO_ERR_OK) return(ret);

  /* convert 0xFF to 0xFF0xFF */
  lcrzo_epr(lcrzo_data_alloc(datasize, &sentdata));
  for ( i=0,sentdatasize=0 ; i<datasize ; i++,sentdatasize++ )
  { if ( data[i]!=0xFF )
    { sentdata[sentdatasize]=data[i];
    }
    else
    { sentdata[sentdatasize++]=0xFF;
      lcrzo_epr(lcrzo_data_realloc(datasize+(sentdatasize-i), &sentdata));
      sentdata[sentdatasize]=0xFF;
    }
  }
  lcrzo_er(lcrzo_data_free2(&data));

  /* send data */
  lcrzo_epr(lcrzo_sock_write(psock, sentdata, sentdatasize));
  lcrzo_er(lcrzo_data_free2(&sentdata));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_priv_telnet_readmoreserverdata(lcrzo_sock *psock, 
					  lcrzo_data *pdata, 
					  lcrzo_int32 *pdatasize, 
					  lcrzo_int32 *pi,
					  lcrzo_int32 neededsize);
int shared_priv_telnet_readmoreserverdata(lcrzo_sock *psock, 
					  lcrzo_data *pdata, 
					  lcrzo_int32 *pdatasize, 
					  lcrzo_int32 *pi,
					  lcrzo_int32 neededsize)
{ lcrzo_data readdata;
  lcrzo_int32 readdatasize;

  while (LCRZO_TRUE)
  { if ( *pdatasize - *pi > neededsize ) return(LCRZO_ERR_OK);
    lcrzo_epr(lcrzo_sock_readm(psock, LCRZO_WAIT_INFINITE, 
			       &readdata, &readdatasize));
    lcrzo_epr(lcrzo_data_appendm_data(readdata, readdatasize,
				      *pdatasize, pdata, pdatasize));
    lcrzo_er(lcrzo_data_free2(&readdata));
  }
  return(LCRZO_ERR_IEINTERNALERROR); 
}

/*-------------------------------------------------------------*/
int shared_priv_telnet_option_answer(shared_telnet *ptelnet,
				     lcrzo_sock *psock, 
				     lcrzo_data *pdata, 
				     lcrzo_int32 *pdatasize, 
				     lcrzo_int32 *pi);
int shared_priv_telnet_option_answer(shared_telnet *ptelnet,
				     lcrzo_sock *psock, 
				     lcrzo_data *pdata, 
				     lcrzo_int32 *pdatasize, 
				     lcrzo_int32 *pi)
{ lcrzo_data sentdata;
  lcrzo_int32 sentdatasize;

  /* this function should be called like this */
  if ( *pdatasize - *pi < 1 ) return(LCRZO_ERR_IEINTERNALERROR);
  if ( (*pdata)[*pi]!=0xFF ) return(LCRZO_ERR_IEINTERNALERROR);

  /* eventually read more data from server */
  lcrzo_epr(shared_priv_telnet_readmoreserverdata(psock, pdata, pdatasize,
						  pi, 1));

  (*pi)++;
  switch((*pdata)[*pi]) {
  case 0xF1: /* NOP*/
    break;
  case 0xFD: /* DO */
  case 0xFE: /* DON'T */
    /* eventually read more data from server */
    lcrzo_epr(shared_priv_telnet_readmoreserverdata(psock, pdata, pdatasize,
						    pi, 1));
    /* answer WON'T */
    sentdatasize=3;
    lcrzo_epr(lcrzo_data_alloc(sentdatasize, &sentdata));
    (*pi)++;
    sentdata[0]=0xFF; sentdata[1]=0xFC; sentdata[2]=(*pdata)[*pi];
    lcrzo_epr(lcrzo_sock_write(psock, sentdata, sentdatasize));
    lcrzo_er(lcrzo_data_free2(&sentdata));
    break;
  case 0xFB: /* WILL */
  case 0xFC: /* WON'T */
    /* eventually read more data from server */
    lcrzo_epr(shared_priv_telnet_readmoreserverdata(psock, pdata, pdatasize,
						    pi, 1));
    /* answer DON'T */
    sentdatasize=3;
    lcrzo_epr(lcrzo_data_alloc(sentdatasize, &sentdata));
    (*pi)++;
    sentdata[0]=0xFF; sentdata[1]=0xFE; sentdata[2]=(*pdata)[*pi];
    lcrzo_epr(lcrzo_sock_write(psock, sentdata, sentdatasize));
    lcrzo_er(lcrzo_data_free2(&sentdata));
    break;
  case 0xFF: /* char 0xFF */
    sentdatasize=1;
    lcrzo_epr(lcrzo_data_alloc(sentdatasize, &sentdata));
    sentdata[0]=0xFF;
    lcrzo_epr(lcrzo_ipc_write(&(ptelnet->ipcdata), sentdata, sentdatasize));
    lcrzo_er(lcrzo_data_free2(&sentdata));
    break;
  default:
    printf("Warning : could not decode 0xFF%02X\n", (*pdata)[*pi]);
    return(123456);
    break;
  }

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_priv_telnet_readserver(shared_telnet *ptelnet,
				  lcrzo_sock *psock);
int shared_priv_telnet_readserver(shared_telnet *ptelnet,
				  lcrzo_sock *psock)
{ lcrzo_data data, sentdata;
  lcrzo_int32 datasize, sentdatasize, i;
  lcrzo_bool sentdatainit;
  int ret;

  /* read server data*/
  ret=lcrzo_sock_readm(psock, LCRZO_WAIT_NODELAY, &data, &datasize);
  if ( ret==LCRZO_ERR_OKTEMPDATAEND ) return(LCRZO_ERR_OK);
  else if (ret!=LCRZO_ERR_OK) return(ret);

  /* analyze its data, answer negatively to requests, and sent data to user */
  sentdatainit=LCRZO_FALSE;
  sentdata=NULL;
  sentdatasize=0;
  for ( i=0 ; i<datasize ; i++ )
  { /* data for user */
    if ( data[i]!=0xFF )
    { if (!sentdatainit)
      { lcrzo_epr(lcrzo_data_alloc(datasize, &sentdata));
        sentdatasize=0;
        sentdatainit=LCRZO_TRUE;
      }
      sentdata[sentdatasize++]=data[i];
    }
    /* control data */
    else
    { /* first send data for user */
      if (sentdatainit)
      { lcrzo_epr(lcrzo_ipc_write(&(ptelnet->ipcdata), sentdata,sentdatasize));
        lcrzo_er(lcrzo_data_free2(&sentdata));
	sentdatainit=LCRZO_FALSE;
      }
      /* decode this option */
      lcrzo_epr(shared_priv_telnet_option_answer(ptelnet, psock, 
						 &data, &datasize, &i));
    }
    /* we have sufficiently data */
    if (sentdatainit && sentdatasize>1000)
    { lcrzo_epr(lcrzo_ipc_write(&(ptelnet->ipcdata), sentdata, sentdatasize));
      lcrzo_er(lcrzo_data_free2(&sentdata));
      sentdatainit=LCRZO_FALSE;
    }
  }
  lcrzo_er(lcrzo_data_free2(&data));

  /* eventually send data for user */
  if (sentdatainit)
  { lcrzo_epr(lcrzo_ipc_write(&(ptelnet->ipcdata), sentdata, sentdatasize));
    lcrzo_er(lcrzo_data_free2(&sentdata));
  }

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_priv_telnet_child(lcrzo_ptr infos, 
			     lcrzo_int32 infossize, 
			     lcrzo_process *pparentprocess);
int shared_priv_telnet_child(lcrzo_ptr infos, 
			     lcrzo_int32 infossize, 
			     lcrzo_process *pparentprocess)
{ shared_priv_telnet *pspt=(shared_priv_telnet *)infos;
  lcrzo_sock sock;
  lcrzo_wait waitsock, waitipc;
  lcrzo_bool canreadserver, canwriteuser, waitforboth;
  int ret;

  /* check for correct usage of this function */
  if (infossize!=sizeof(shared_priv_telnet))
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);

  /* close parent process handle because we do not need it */
  lcrzo_epr(lcrzo_process_close(pparentprocess));

  /* init the tcp client */
  lcrzo_epr(lcrzo_sock_tcpcli_real(pspt->serveripl, pspt->serverport, &sock));

  /* close unneeded IPCs */
  lcrzo_epr(lcrzo_ipc_currentprocessissideb(&(pspt->telnet.ipcdata)));

  /* end of negociation */
  lcrzo_epr(lcrzo_ipc_write_uint32(&(pspt->telnet.ipcdata), LCRZO_ERR_OK));

  /* get file descriptors for select */
  lcrzo_epr(lcrzo_wait_init_sock_read(&sock, &waitsock));
  lcrzo_epr(lcrzo_wait_init_ipc_read(&pspt->telnet.ipcdata, &waitipc));
  waitforboth=LCRZO_TRUE;
  canreadserver=LCRZO_FALSE;
  while(LCRZO_TRUE)
  { if (waitforboth)
    { lcrzo_epr(lcrzo_wait_wait2(&waitsock, &waitipc, LCRZO_WAIT_INFINITE,
				 &canreadserver, &canwriteuser));
    }
    else
    { lcrzo_epr(lcrzo_wait_wait1(&waitipc, LCRZO_WAIT_INFINITE, &canwriteuser));
    }
    /*printf("%d_%d    %d_%d\n", fdreadserver, canreadserver, 
      fdwriteuser, canwriteuser);*/
    if ( canreadserver )
    { /* server has data */
      ret=shared_priv_telnet_readserver(&pspt->telnet, &sock);
      if ( ret==LCRZO_ERR_OKDATAEND )
      { /* server has closed connection : do not ask it anymore */
        waitforboth=LCRZO_FALSE;
      }
    }
    if ( canwriteuser )
    { /* user wants to write */
      lcrzo_epr(shared_priv_telnet_writeuser(&pspt->telnet, &sock));
    }
  }
  lcrzo_epr(lcrzo_wait_close(&waitsock));
  lcrzo_epr(lcrzo_wait_close(&waitipc));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_priv_telnet_ipcreadtime(lcrzo_ipc *pipc,
				   lcrzo_uint32 readtimeout,
				   lcrzo_data *preaddata,
				   lcrzo_int32 *preaddatasize);
int shared_priv_telnet_ipcreadtime(lcrzo_ipc *pipc,
				   lcrzo_uint32 readtimeout,
				   lcrzo_data *preaddata,
				   lcrzo_int32 *preaddatasize)
{ lcrzo_time timeout;
  lcrzo_data data, readdata;
  lcrzo_int32 datasize, readdatasize;
  int ret;

  lcrzo_epr(lcrzo_time_init_usec(readtimeout, &timeout));
  lcrzo_epr(lcrzo_data_initm_text("", &readdata, &readdatasize));

  /* read data */
  while (LCRZO_TRUE)
  { ret=lcrzo_ipc_readm(pipc, &timeout, &data, &datasize);
    if (ret!=LCRZO_ERR_OK) break;
    lcrzo_epr(lcrzo_data_appendm_data(data, datasize, readdatasize, 
				      &readdata, &readdatasize));
    lcrzo_er(lcrzo_data_free2(&data));
  }

  /* set output data */
  if (preaddatasize!=NULL) *preaddatasize=readdatasize;
  if (preaddata==NULL)
  { lcrzo_er(lcrzo_data_free2(&readdata));
  }
  else 
  { *preaddata=readdata;
  }
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
/*-------------------------------------------------------------*/
/*-------------------------------------------------------------*/
int shared_telnet_init(lcrzo_ipl serveripl, 
		       lcrzo_uint16 serverport,
		       shared_telnet *ptelnet)
{ lcrzo_uint32 ret;
  shared_priv_telnet spt;

  /* open two IPCs */
  lcrzo_epr(lcrzo_ipc_init(&(ptelnet->ipcdata)));

  /* set parameters */
  spt.serveripl=serveripl;
  spt.serverport=serverport;
  spt.telnet=*ptelnet;

  /* create the child */
  lcrzo_epr(lcrzo_process_other_init(&shared_priv_telnet_child, 
                                     (lcrzo_data)&spt, sizeof(spt), 
                                     &ptelnet->child));
  /* close unneeded IPCs */
  lcrzo_epr(lcrzo_ipc_currentprocessissidea(&(ptelnet->ipcdata)));

  /* wait the end of negociation */
  lcrzo_epr(lcrzo_ipc_read_uint32(&(ptelnet->ipcdata), LCRZO_WAIT_INFINITE, 
				  &ret));

  return((int)ret);
}

/*-------------------------------------------------------------*/
int shared_telnet_close(shared_telnet *ptelnet)
{
  /* kill child */
  lcrzo_epr(shared_lcrzoex_debug_processotherexit_wanttowarn(LCRZO_FALSE));
  lcrzo_epr(lcrzo_process_other_exit(&ptelnet->child));
  lcrzo_epr(shared_lcrzoex_debug_processotherexit_wanttowarn(LCRZO_TRUE));
  lcrzo_epr(lcrzo_process_other_wait_end(&ptelnet->child, LCRZO_WAIT_INFINITE, 
					 NULL, NULL));
  lcrzo_epr(lcrzo_process_close(&ptelnet->child));
  
  /* close IPCs */
  lcrzo_epr(lcrzo_ipc_close(&(ptelnet->ipcdata)));
    
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_telnet_login(shared_telnet *ptelnet,
			lcrzo_conststring login,
			lcrzo_conststring password)
{ lcrzo_data data, banner;
  lcrzo_int32 datasize, bannersize;
  lcrzo_regexp re;
  lcrzo_bool regcase;
  int ret;

  /* read till "login: "*/
  lcrzo_epr(lcrzo_global_get_data_regexp_casesensitive(&regcase));
  lcrzo_epr(lcrzo_global_set_data_regexp_casesensitive(LCRZO_FALSE));
  lcrzo_epr(lcrzo_data_initm_text("", &banner, &bannersize));
  while (LCRZO_TRUE)
  { lcrzo_epr(lcrzo_ipc_readm(&(ptelnet->ipcdata), LCRZO_WAIT_INFINITE, 
			      &data, &datasize));
    lcrzo_epr(lcrzo_data_appendm_data(data, datasize, bannersize,
				      &banner, &bannersize));
    lcrzo_epr(lcrzo_data_free2(&data));
    ret=lcrzo_data_search_regexp(banner, bannersize, LCRZO_FALSE, +1, -1,
                                 "login[ \t]*:[ \t]*$", re);
    if ( ret==LCRZO_ERR_OK ) break;
  }
  lcrzo_epr(lcrzo_data_free2(&banner));

  /* send login */
  lcrzo_epr(lcrzo_data_initm_text(login, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, &data, &datasize));
  lcrzo_epr(lcrzo_ipc_write(&(ptelnet->ipcdata), data, datasize));
  lcrzo_epr(lcrzo_data_free2(&data));

  /* read till "word: "*/
  lcrzo_epr(lcrzo_data_initm_text("", &banner, &bannersize));
  while (LCRZO_TRUE)
  { lcrzo_epr(lcrzo_ipc_readm(&(ptelnet->ipcdata), LCRZO_WAIT_INFINITE, 
			      &data, &datasize));
    lcrzo_epr(lcrzo_data_appendm_data(data, datasize, bannersize,
				      &banner, &bannersize));
    lcrzo_epr(lcrzo_data_free2(&data));
    ret=lcrzo_data_search_regexp(banner, bannersize, LCRZO_FALSE, +1, -1,
                                 "word[ \t]*:[ \t]*$", re);
    if ( ret==LCRZO_ERR_OK ) break;
  }
  lcrzo_epr(lcrzo_data_free2(&banner));

  /* send password */
  lcrzo_epr(lcrzo_data_initm_text(password, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, &data, &datasize));
  lcrzo_epr(lcrzo_ipc_write(&(ptelnet->ipcdata), data, datasize));
  lcrzo_epr(lcrzo_data_free2(&data));
  /* read \r\n sent by server */
  lcrzo_epr(lcrzo_ipc_readm_line(&(ptelnet->ipcdata), LCRZO_WAIT_INFINITE,
				 NULL, NULL));
    
  /* read answer */
  lcrzo_epr(lcrzo_ipc_readm_line(&(ptelnet->ipcdata), LCRZO_WAIT_INFINITE, 
				 &banner, &bannersize));
  ret=lcrzo_data_search_regexp(banner, bannersize, LCRZO_FALSE, +1, -1,
			       "^Login incorrect", re);
  lcrzo_epr(lcrzo_data_free2(&banner));
  lcrzo_epr(lcrzo_global_set_data_regexp_casesensitive(regcase));

  if ( ret==LCRZO_ERR_OK )
  { /* automatically exit telnet session */
    lcrzo_epr(shared_telnet_close(ptelnet));
    return(LCRZOEX_ERR_TELNET_BADLOGINPASS);
  }
 
  /* read all banners */
  lcrzo_epr(shared_priv_telnet_ipcreadtime(&(ptelnet->ipcdata), 1300000,
					   NULL, NULL));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_telnet_exec(shared_telnet *ptelnet,
		       lcrzo_conststring command,
		       lcrzo_uint32 readtimeout,
		       lcrzo_data *preaddata,
		       lcrzo_int32 *preaddatasize)
{ lcrzo_data data;
  lcrzo_int32 datasize;

  /* send command */
  lcrzo_epr(lcrzo_data_initm_text(command, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, &data, &datasize));
  lcrzo_epr(lcrzo_ipc_write(&(ptelnet->ipcdata), data, datasize));
  lcrzo_epr(lcrzo_data_free2(&data));

  /* read result */
  lcrzo_epr(shared_priv_telnet_ipcreadtime(&(ptelnet->ipcdata), readtimeout,
					   preaddata, preaddatasize));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_telnet_logout(shared_telnet *ptelnet)
{ lcrzo_data data;
  lcrzo_int32 datasize;
  int ret;

  /* send command */
  lcrzo_epr(lcrzo_data_initm_text("exit\r\n", &data, &datasize));
  lcrzo_epr(lcrzo_ipc_write(&(ptelnet->ipcdata), data, datasize));
  lcrzo_epr(lcrzo_data_free2(&data));

  /* read result */
  ret=shared_priv_telnet_ipcreadtime(&(ptelnet->ipcdata), 100000, NULL, NULL);
  if ( ret==LCRZO_ERR_OKDATAEND || ret==LCRZO_ERR_OK ) return(LCRZO_ERR_OK);
  return(ret);
}
