/*
		                 LCRZOEX
                             Network toolbox
             Copyright(c) 1999,2000,2001,2002 Laurent Constantin
                                  -----

  Main server    : http://www.laurentconstantin.com/
  Backup servers : http://go.to/laurentconstantin/
                   http://laurentconstantin.est-la.com/
                   http://laurentconstantin.free.fr/
                   http://membres.lycos.fr/lauconstantin/
  [my current email address is on the web servers]

                                  -----
  This file is part of Lcrzoex.

  Lcrzoex is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Lcrzoex is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details (http://www.gnu.org/).

------------------------------------------------------------------------
*/

#include <lcrzo.h>
#include "../lcrzoex.h"

/*-------------------------------------------------------------*/
/* we need those functions, because with HTTP, we want to keep 
   the last /, in order to simplify computations */
int shared_priv_http_path_clean(lcrzo_conststring pathname,
				lcrzo_string *pclean);
int shared_priv_http_path_clean(lcrzo_conststring pathname,
				lcrzo_string *pclean)
{ lcrzo_string cl;

  lcrzo_er(lcrzo_path_clean(pathname, &cl));
  if ( pathname[strlen(pathname)-1]=='/' && cl[strlen(cl)-1]!='/')
  { lcrzo_er(lcrzo_string_appendm_text("/", &cl));
  }
  lcrzo_er(lcrzo_string_initm_text(cl, pclean));
  lcrzo_er(lcrzo_string_free2(&cl));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_priv_http_path_dir(lcrzo_conststring fileordir,
			     lcrzo_string *pdirectoryoffileordir);
int shared_priv_http_path_dir(lcrzo_conststring fileordir,
			     lcrzo_string *pdirectoryoffileordir)
{ lcrzo_string path, tmp;

  lcrzo_er(shared_priv_http_path_clean(fileordir, &path));

  if ( fileordir[strlen(fileordir)-1]=='/' )
  { if (pdirectoryoffileordir!=NULL) *pdirectoryoffileordir=path;
    else { lcrzo_er(lcrzo_string_free2(&path)); }
  }
  else
  { lcrzo_er(lcrzo_path_decode_parentdir(path, LCRZO_TRUE, &tmp));
    lcrzo_er(lcrzo_string_free2(&path));
    if ( tmp[strlen(tmp)-1]!='/')
    { lcrzo_er(lcrzo_string_appendm_text("/", &tmp));
    }
    if (pdirectoryoffileordir!=NULL) *pdirectoryoffileordir=tmp;
    else { lcrzo_er(lcrzo_string_free2(&tmp)); }
  }

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_priv_http_path_relative(lcrzo_conststring fileordirdest,
				   lcrzo_conststring fileordircurrent,
				   lcrzo_string *prelativepath);
int shared_priv_http_path_relative(lcrzo_conststring fileordirdest,
				   lcrzo_conststring fileordircurrent,
				   lcrzo_string *prelativepath)
{ int ret;
  lcrzo_bool b, isdir;
  lcrzo_string relativepath;

  isdir=LCRZO_FALSE;
  if (fileordircurrent[strlen(fileordircurrent)-1]=='/') isdir=LCRZO_TRUE;

  ret=lcrzo_path_init_relb(fileordircurrent, LCRZO_FALSE, isdir,
			   fileordirdest, LCRZO_FALSE, &relativepath);
  if (ret!=LCRZO_ERR_OK)
  { ret=lcrzo_path_isabsolute(fileordirdest, LCRZO_FALSE, &b);
    if (ret==LCRZO_ERR_OK && b)
    { lcrzo_er(lcrzo_string_initm_text(fileordirdest, prelativepath));
      return(LCRZO_ERR_OK);
    }
    return(LCRZO_ERR_SPPATHCANTINIT);
  }

  if ( fileordirdest[strlen(fileordirdest)-1]=='/' &&
       relativepath[strlen(relativepath)-1]!='/' )
  { lcrzo_er(lcrzo_string_appendm_text("/", &relativepath));
  }
  if (prelativepath!=NULL) *prelativepath=relativepath;
  else { lcrzo_er(lcrzo_string_free2(&relativepath)); }

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_priv_http_path_relative_test(void);
int shared_priv_http_path_relative_test(void)
{ lcrzo_string str;

  lcrzo_epr(shared_priv_http_path_relative("/server/dir/p.html",
				    "/server/dir/index.html",
				    &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "./p.html")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));

  lcrzo_epr(shared_priv_http_path_relative("/server/dir/p.html",
				    "/server/dir/tt/index.html",
				    &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "../p.html")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));

  lcrzo_epr(shared_priv_http_path_relative("/server/dir/uu/p.html",
				    "/server/dir/tt/index.html",
				    &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "../uu/p.html")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));

  lcrzo_epr(shared_priv_http_path_relative("/server/dir/uu/p.html",
				    "/server/dir/index.html",
				    &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "./uu/p.html")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));

  lcrzo_epr(shared_priv_http_path_relative("./server/dir/uu/p.html",
				    "./server/dir/index.html",
				    &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "./uu/p.html")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));
  lcrzo_eipr(shared_priv_http_path_relative("server/dir/uu/p.html",
				    "/server/dir/index.html",
				    &str),
	     LCRZO_ERR_SPPATHCANTINIT);

  lcrzo_epr(shared_priv_http_path_relative("/server/dir/uu/p.html",
				    "server/dir/index.html",
				    &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "/server/dir/uu/p.html")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));

  lcrzo_epr(shared_priv_http_path_relative("./z/index.html",
				    "./z/",
				    &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "./index.html")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_set_resolver_timeout(void)
{ return(lcrzo_global_set_resolver_timeout(12));
}

/*-------------------------------------------------------------*/
int shared_http_initm_methurlhttpver(lcrzo_conststring method,
				     lcrzo_conststring url,
				     lcrzo_conststring version,
				     lcrzo_data *pdata,
				     lcrzo_int32 *pdatasize)
{ lcrzo_int32 datasize;

  /* create "METHOD url HTTP/version" */
  lcrzo_epr(lcrzo_data_initm_text(method, pdata, &datasize));
  lcrzo_epr(lcrzo_data_appendm_char(' ', 1, datasize, pdata, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text(url, datasize, pdata, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text(" HTTP/", datasize, pdata, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text(version, datasize, pdata, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, pdata, &datasize));

  /* set the output values */
  if (pdatasize!=NULL) *pdatasize=datasize;

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_appendm_hostport(lcrzo_conststring hostname,
				 lcrzo_uint16 port,
				 lcrzo_int32 datacurrentsize,
				 lcrzo_data *pdata,
				 lcrzo_int32 *pdatasize)
{ lcrzo_int32 datasize;

  /* append "Host: hostname:port" */
  lcrzo_epr(lcrzo_data_appendm_text("Host: ", datacurrentsize, 
				    pdata, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text(hostname, datasize, pdata, &datasize));
  if ( port != 80 )
  { lcrzo_epr(lcrzo_data_appendm_char(':', 1, datasize, pdata, &datasize));
    lcrzo_epr(lcrzo_data_appendm_uint16(port, "%u", datasize, 
					pdata, &datasize));
  }
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, pdata, &datasize));

  /* set the output values */
  if (pdatasize!=NULL) *pdatasize=datasize;

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_appendm_ipport(lcrzo_ipl ip,
			       lcrzo_uint16 port,
			       lcrzo_int32 datacurrentsize,
			       lcrzo_data *pdata,
			       lcrzo_int32 *pdatasize)
{ lcrzo_ips ips;

  lcrzo_epr(lcrzo_ips_init_ipl(ip, ips));
  lcrzo_epr(shared_http_appendm_hostport(ips, port, datacurrentsize, 
					 pdata, pdatasize));

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_appendm_authbasiclogpass(lcrzo_conststring login,
					 lcrzo_conststring passwd,
					 lcrzo_int32 datacurrentsize,
					 lcrzo_data *pdata,
					 lcrzo_int32 *pdatasize)
{ lcrzo_data data;
  lcrzo_int32 datasize;
  lcrzo_string logpass;

  /* error case */
  if ( login==NULL && passwd!=NULL ) 
    return(LCRZOEX_ERR_HTTP_LOGNULLPWDNOTNULL);

  /* special case of no line to add */
  if ( login==NULL && passwd==NULL )
  { if (pdatasize!=NULL) *pdatasize=datacurrentsize;
    return(LCRZO_ERR_OK);
  }

  /* first, we generate the base64 of login:passwd */
  lcrzo_epr(lcrzo_data_initm_text(login, &data, &datasize));
  if (passwd!=NULL)
  { lcrzo_epr(lcrzo_data_appendm_text(":", datasize, &data, &datasize));
    lcrzo_epr(lcrzo_data_appendm_text(passwd, datasize, &data, &datasize));
  }
  lcrzo_epr(lcrzo_base64_initm_data(data, datasize, &logpass));
  lcrzo_er(lcrzo_data_free2(&data));

  /* append "Authorization: Basic base64_of_login:passwd" */
  lcrzo_epr(lcrzo_data_appendm_text("Authorization: Basic ", datacurrentsize,
                                    pdata, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text(logpass, datasize, pdata, &datasize));
  lcrzo_er(lcrzo_string_free2(&logpass));
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, pdata, &datasize));

  /* set the output values */
  if (pdatasize!=NULL) *pdatasize=datasize;

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_appendm_proxyauthbasiclogpass(lcrzo_conststring login,
					      lcrzo_conststring passwd,
					      lcrzo_int32 datacurrentsize,
					      lcrzo_data *pdata,
					      lcrzo_int32 *pdatasize)
{ lcrzo_data data;
  lcrzo_int32 datasize;
  lcrzo_string logpass;
 
  /* error case */
  if ( login==NULL && passwd!=NULL )
    return(LCRZOEX_ERR_HTTP_LOGNULLPWDNOTNULL);

  /* special case of no line to add */
  if ( login==NULL && passwd==NULL )
  { if (pdatasize!=NULL) *pdatasize=datacurrentsize;
    return(LCRZO_ERR_OK);
  }

  /* first, we generate the base64 of login:passwd */
  lcrzo_epr(lcrzo_data_initm_text(login, &data, &datasize));
  if (passwd!=NULL)
  { lcrzo_epr(lcrzo_data_appendm_text(":", datasize, &data, &datasize));
    lcrzo_epr(lcrzo_data_appendm_text(passwd, datasize, &data, &datasize));
  }
  lcrzo_epr(lcrzo_base64_initm_data(data, datasize, &logpass));
  lcrzo_er(lcrzo_data_free2(&data));

  /* append "Proxy-Authorization: Basic base64_of_login:passwd" */
  lcrzo_epr(lcrzo_data_appendm_text("Proxy-Authorization: Basic ",
				    datacurrentsize,
                                    pdata, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text(logpass, datasize, pdata, &datasize));
  lcrzo_er(lcrzo_string_free2(&logpass));
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, pdata, &datasize));

  /* set the output values */
  if (pdatasize!=NULL) *pdatasize=datasize;

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_initm_httpvererr(lcrzo_conststring version,
				 lcrzo_uint16 errorcode,
				 lcrzo_conststring errorstring,
				 lcrzo_data *pdata,
				 lcrzo_int32 *pdatasize)
{ lcrzo_int32 datasize;

  /* create "HTTP/version 000 message" */
  lcrzo_epr(lcrzo_data_initm_text("HTTP/", pdata, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text(version, datasize, pdata, &datasize));
  lcrzo_epr(lcrzo_data_appendm_char(' ', 1, datasize, pdata, &datasize));
  lcrzo_epr(lcrzo_data_appendm_uint16(errorcode, "%u", datasize, 
				      pdata, &datasize));
  lcrzo_epr(lcrzo_data_appendm_char(' ', 1, datasize, pdata, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text(errorstring, datasize, pdata, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, pdata, &datasize));

  /* set the output values */
  if (pdatasize!=NULL) *pdatasize=datasize;

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_appendm_connection(lcrzo_conststring type,
				   lcrzo_int32 datacurrentsize,
				   lcrzo_data *pdata,
				   lcrzo_int32 *pdatasize)
{ lcrzo_int32 datasize;

  lcrzo_epr(lcrzo_data_appendm_text("Connection: ", datacurrentsize,
				    pdata, &datasize));
  if (type==NULL)
  { lcrzo_epr(lcrzo_data_appendm_text("close", datasize, pdata, &datasize));
  }
  else
  { lcrzo_epr(lcrzo_data_appendm_text(type, datasize, pdata, &datasize));
  }
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, pdata, &datasize));

  /* set the output values */
  if (pdatasize!=NULL) *pdatasize=datasize;

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_appendm_contenttype(lcrzo_conststring type,
				    lcrzo_int32 datacurrentsize,
				    lcrzo_data *pdata,
				    lcrzo_int32 *pdatasize)
{ lcrzo_int32 datasize;

  lcrzo_epr(lcrzo_data_appendm_text("Content-Type: ", datacurrentsize,
				    pdata, &datasize));
  if (type==NULL)
  { lcrzo_epr(lcrzo_data_appendm_text("text/html", datasize, pdata,&datasize));
  }
  else
  { lcrzo_epr(lcrzo_data_appendm_text(type, datasize, pdata, &datasize));
  }
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, pdata, &datasize));

  /* set the output values */
  if (pdatasize!=NULL) *pdatasize=datasize;

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_appendm_wwwauth(lcrzo_conststring realm,
				lcrzo_int32 datacurrentsize,
				lcrzo_data *pdata,
				lcrzo_int32 *pdatasize)
{ lcrzo_int32 datasize;

  lcrzo_epr(lcrzo_data_appendm_text("WWW-Authenticate: Basic realm=\"", 
				    datacurrentsize,
				    pdata, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text(realm, datasize, pdata, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text("\"\r\n", datasize, pdata, &datasize));

  /* set the output values */
  if (pdatasize!=NULL) *pdatasize=datasize;

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_appendm_location(lcrzo_conststring loc,
				 lcrzo_int32 datacurrentsize,
				 lcrzo_data *pdata,
				 lcrzo_int32 *pdatasize)
{ lcrzo_int32 datasize;

  lcrzo_epr(lcrzo_data_appendm_text("Location: ", 
				    datacurrentsize, pdata, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text(loc, datasize, pdata, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, pdata, &datasize));

  /* set the output values */
  if (pdatasize!=NULL) *pdatasize=datasize;

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_appendm_useragent(lcrzo_conststring useragent,
				  lcrzo_int32 datacurrentsize,
				  lcrzo_data *pdata,
				  lcrzo_int32 *pdatasize)
{ lcrzo_int32 datasize;

  /* append "Host: hostname:port" */
  lcrzo_epr(lcrzo_data_appendm_text("User-Agent: ", datacurrentsize, 
				    pdata, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text(useragent, datasize, pdata, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, pdata, &datasize));

  /* set the output values */
  if (pdatasize!=NULL) *pdatasize=datasize;

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_request_get2(lcrzo_sock *psock,
			     lcrzo_conststring url,
			     lcrzo_conststring hostname,
			     lcrzo_uint16 port,
			     lcrzo_conststring proxylogin,
			     lcrzo_conststring proxypasswd,
			     lcrzo_conststring sitelogin,
			     lcrzo_conststring sitepasswd,
			     lcrzo_conststring useragent)
{ lcrzo_data data;
  lcrzo_int32 datasize;
  
  /* construct the date to send*/
  lcrzo_epr(shared_http_initm_methurlhttpver("GET", url, "1.0",
					     &data, &datasize));
  lcrzo_epr(shared_http_appendm_hostport(hostname, port, datasize,
					 &data, &datasize));
  lcrzo_epr(shared_http_appendm_proxyauthbasiclogpass(proxylogin, proxypasswd,
						      datasize,
						      &data, &datasize));
  lcrzo_epr(shared_http_appendm_authbasiclogpass(sitelogin, sitepasswd, 
						 datasize, &data, &datasize));
  if ( useragent!=NULL )
  { lcrzo_epr(shared_http_appendm_useragent(useragent, datasize,
					    &data, &datasize));
  }
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, &data, &datasize));

  /* write data to the socket */
  /*lcrzo_epr(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_DUMP));*/
  lcrzo_epr(lcrzo_sock_write(psock, data, datasize));
  lcrzo_er(lcrzo_data_free2(&data));

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_request_get(lcrzo_sock *psock,
			    lcrzo_conststring url,
			    lcrzo_conststring hostname,
			    lcrzo_uint16 port,
			    lcrzo_conststring proxylogin,
			    lcrzo_conststring proxypasswd,
			    lcrzo_conststring sitelogin,
			    lcrzo_conststring sitepasswd)
{ return(shared_http_request_get2(psock, url, hostname, port,
				  proxylogin, proxypasswd,
				  sitelogin, sitepasswd, NULL));
}

/*-------------------------------------------------------------*/
int shared_http_request_post(lcrzo_sock *psock,
			     lcrzo_conststring url,
			     lcrzo_conststring hostname,
			     lcrzo_uint16 port,
			     lcrzo_conststring proxylogin,
			     lcrzo_conststring proxypasswd,
			     lcrzo_conststring sitelogin,
			     lcrzo_conststring sitepasswd,
			     lcrzo_conststring parameters)
{ lcrzo_data data;
  lcrzo_int32 datasize;
  
  /* construct the date to send*/
  lcrzo_epr(shared_http_initm_methurlhttpver("POST", url, "1.0",
					     &data, &datasize));
  lcrzo_epr(shared_http_appendm_hostport(hostname, port, datasize,
					 &data, &datasize));
  lcrzo_epr(shared_http_appendm_proxyauthbasiclogpass(proxylogin, proxypasswd,
						      datasize,
						      &data, &datasize));
  lcrzo_epr(shared_http_appendm_authbasiclogpass(sitelogin, sitepasswd, 
						 datasize, &data,&datasize));
  lcrzo_epr(lcrzo_data_appendm_text(
            "Content-Type: application/x-www-form-urlencoded\r\n",
				    datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text("Content-Length: ",
				    datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_int32(strlen(parameters), "%d", 
				     datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, &data, &datasize));
  /* now the parameters without an ending line */
  lcrzo_epr(lcrzo_data_appendm_text(parameters, datasize, &data, &datasize));

  /* write data to the socket */
  /*lcrzo_epr(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_DUMP));*/
  lcrzo_epr(lcrzo_sock_write(psock, data, datasize));
  lcrzo_er(lcrzo_data_free2(&data));

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_request_head(lcrzo_sock *psock,
			     lcrzo_conststring url,
			     lcrzo_conststring hostname,
			     lcrzo_uint16 port,
			     lcrzo_conststring proxylogin,
			     lcrzo_conststring proxypasswd,
			     lcrzo_conststring sitelogin,
			     lcrzo_conststring sitepasswd)
{ lcrzo_data data;
  lcrzo_int32 datasize;
  
  /* construct the date to send*/
  lcrzo_epr(shared_http_initm_methurlhttpver("HEAD", url, "1.0",
					     &data, &datasize));
  lcrzo_epr(shared_http_appendm_hostport(hostname, port, datasize,
					 &data, &datasize));
  lcrzo_epr(shared_http_appendm_proxyauthbasiclogpass(proxylogin, proxypasswd,
						      datasize,
						      &data, &datasize));
  lcrzo_epr(shared_http_appendm_authbasiclogpass(sitelogin, sitepasswd, 
						 datasize, &data,&datasize));
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, &data, &datasize));

  /* write data to the socket */
  /*lcrzo_epr(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_DUMP));*/
  lcrzo_epr(lcrzo_sock_write(psock, data, datasize));
  lcrzo_er(lcrzo_data_free2(&data));

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_request_put(lcrzo_sock *psock,
			    lcrzo_conststring url,
			    lcrzo_conststring hostname,
			    lcrzo_uint16 port,
			    lcrzo_conststring proxylogin,
			    lcrzo_conststring proxypasswd,
			    lcrzo_conststring sitelogin,
			    lcrzo_conststring sitepasswd,
			    lcrzo_data filedata,
			    lcrzo_int32 filedatasize)
{ lcrzo_data data;
  lcrzo_int32 datasize;
  
  /* construct the date to send*/
  lcrzo_epr(shared_http_initm_methurlhttpver("PUT", url, "1.0",
					     &data, &datasize));
  lcrzo_epr(shared_http_appendm_hostport(hostname, port, datasize,
					 &data, &datasize));
  lcrzo_epr(shared_http_appendm_proxyauthbasiclogpass(proxylogin, proxypasswd,
						      datasize,
						      &data, &datasize));
  lcrzo_epr(shared_http_appendm_authbasiclogpass(sitelogin, sitepasswd, 
						 datasize, &data,&datasize));
  lcrzo_epr(lcrzo_data_appendm_text("Content-Length: ",
				    datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_int32(filedatasize, "%d", 
				     datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, &data, &datasize));

  /* write data to the socket */
  /*lcrzo_epr(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_DUMP));*/
  lcrzo_epr(lcrzo_sock_write(psock, data, datasize));
  lcrzo_er(lcrzo_data_free2(&data));

  /* write the file to the socket */
  /*lcrzo_epr(lcrzo_data_print(filedata, filedatasize,LCRZO_PRINTTYPE_DUMP));*/
  lcrzo_epr(lcrzo_sock_write(psock, filedata, filedatasize));

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_request_delete(lcrzo_sock *psock,
			       lcrzo_conststring url,
			       lcrzo_conststring hostname,
			       lcrzo_uint16 port,
			       lcrzo_conststring proxylogin,
			       lcrzo_conststring proxypasswd,
			       lcrzo_conststring sitelogin,
			       lcrzo_conststring sitepasswd)
{ lcrzo_data data;
  lcrzo_int32 datasize;
  
  /* construct the date to send*/
  lcrzo_epr(shared_http_initm_methurlhttpver("DELETE", url, "1.0",
					     &data, &datasize));
  lcrzo_epr(shared_http_appendm_hostport(hostname, port, datasize,
					 &data, &datasize));
  lcrzo_epr(shared_http_appendm_proxyauthbasiclogpass(proxylogin, proxypasswd,
						      datasize,
						      &data, &datasize));
  lcrzo_epr(shared_http_appendm_authbasiclogpass(sitelogin, sitepasswd, 
						 datasize, &data,&datasize));
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, &data, &datasize));

  /* write data to the socket */
  /*lcrzo_epr(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_DUMP));*/
  lcrzo_epr(lcrzo_sock_write(psock, data, datasize));
  lcrzo_er(lcrzo_data_free2(&data));

  return (LCRZO_ERR_OK);
}


/*-------------------------------------------------------------*/
int shared_http_request_readheader(lcrzo_sock *psock,
				   lcrzo_time *pmaxwaitingtime,
				   lcrzo_string *pmethod,
				   lcrzo_string *purl,
				   lcrzo_string *pversion,
				   lcrzo_string *phost,
				   lcrzo_string *puseragent,
				   lcrzo_string *pproxylogin,
				   lcrzo_string *pproxypasswd,
				   lcrzo_string *psitelogin,
				   lcrzo_string *psitepasswd,
				   lcrzo_int32 *pcontentlength,
				   lcrzo_data *pheader,
				   lcrzo_int32 *pheadersize)
{ lcrzo_data data;
  lcrzo_int32 datasize, headersize;
  lcrzo_regexp re;
  lcrzo_bool hostfound, useragentfound, proxylogpassfound, sitelogpassfound;
  lcrzo_bool contentlengthfound;
  lcrzo_string authorization, authlogpass, integerstring;
  int ret;

  /* on the first line, there is "METHOD url HTTP/version" */
  lcrzo_epr(lcrzo_sock_readm_line(psock, pmaxwaitingtime, 
				  &data, &datasize));
  /*lcrzo_epr(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_TEXTN));*/
  ret=lcrzo_data_search_regexp(data, datasize, LCRZO_FALSE, +1, -1,
			       "^([A-Za-z]+) (.*) HTTP/(...)$", re);
  if (ret!=LCRZO_ERR_OK)
  { lcrzo_er(lcrzo_data_free2(&data));
    return(LCRZOEX_ERR_HTTP_INVALIDREQUEST);
  }

  data[datasize]='\0';
  lcrzo_epr(lcrzo_string_initm_range((lcrzo_string)data,
				     re[1].startindex_positive,
				     re[1].endindex_positive, 
				     pmethod));
  lcrzo_epr(lcrzo_string_initm_range((lcrzo_string)data,
				     re[2].startindex_positive,
				     re[2].endindex_positive, 
				     purl));
  lcrzo_epr(lcrzo_string_initm_range((lcrzo_string)data,
				     re[3].startindex_positive,
				     re[3].endindex_positive, 
				     pversion));
  lcrzo_epr(lcrzo_data_initm_data(data, datasize, pheader, &headersize));
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", headersize, pheader, &headersize));
  lcrzo_er(lcrzo_data_free2(&data));

  /* to default values in case they are not set in the header */
  hostfound=LCRZO_FALSE;
  useragentfound=LCRZO_FALSE;
  proxylogpassfound=LCRZO_FALSE;
  sitelogpassfound=LCRZO_FALSE;
  contentlengthfound=LCRZO_FALSE;

  /* we read the header till the empty line */
  while (datasize)
  { /* read the line */
    lcrzo_epr(lcrzo_sock_readm_line(psock, pmaxwaitingtime, 
				    &data, &datasize));
    /* add it to the returned header */     
    lcrzo_epr(lcrzo_data_appendm_data(data, datasize, headersize, 
				      pheader, &headersize));
    lcrzo_epr(lcrzo_data_appendm_text("\r\n", headersize,pheader,&headersize));
    /*lcrzo_epr(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_TEXTN));*/
    if ( !hostfound )
    { /* search Host */
      ret=lcrzo_data_search_regexp(data, datasize, LCRZO_FALSE, +1, -1,
				   "^Host: (.+)$", re);
      data[datasize]='\0';
      if ( ret==LCRZO_ERR_OK )
      { lcrzo_epr(lcrzo_string_initm_range((lcrzo_string)data,
					   re[1].startindex_positive,
					   re[1].endindex_positive, 
					   phost));
        hostfound=LCRZO_TRUE;
      }
    }
    if ( !useragentfound )
    { /* search User-Agent */
      lcrzo_er(lcrzo_global_set_data_regexp_casesensitive(LCRZO_FALSE));
      ret=lcrzo_data_search_regexp(data, datasize, LCRZO_FALSE, +1, -1,
				   "^User-Agent: (.+)$", re);
      lcrzo_er(lcrzo_global_set_data_regexp_casesensitive(LCRZO_TRUE));
      data[datasize]='\0';
      if ( ret==LCRZO_ERR_OK )
      { lcrzo_epr(lcrzo_string_initm_range((lcrzo_string)data,
					   re[1].startindex_positive,
					   re[1].endindex_positive, 
					   puseragent));
        useragentfound=LCRZO_TRUE;
      }
    }
    if ( !contentlengthfound )
    { /* search Content-Length */
      lcrzo_er(lcrzo_global_set_data_regexp_casesensitive(LCRZO_FALSE));
      ret=lcrzo_data_search_regexp(data, datasize, LCRZO_FALSE, +1, -1,
				   "^Content-Length: ([0-9]+)$", re);
      lcrzo_er(lcrzo_global_set_data_regexp_casesensitive(LCRZO_TRUE));
      data[datasize]='\0';
      if ( ret==LCRZO_ERR_OK )
      { lcrzo_epr(lcrzo_string_initm_range((lcrzo_string)data,
					   re[1].startindex_positive,
					   re[1].endindex_positive, 
					   &integerstring));
        if ( pcontentlength!=NULL ) *pcontentlength=atoi(integerstring);
	lcrzo_er(lcrzo_string_free2(&integerstring));
	contentlengthfound=LCRZO_TRUE;
      }
    }
    if ( !proxylogpassfound )
    { /* search Proxy-Authorization */
      lcrzo_er(lcrzo_global_set_data_regexp_casesensitive(LCRZO_FALSE));
      ret=lcrzo_data_search_regexp(data, datasize, LCRZO_FALSE, +1, -1,
				   "^Proxy-Authorization: Basic (.+)$", re);
      lcrzo_er(lcrzo_global_set_data_regexp_casesensitive(LCRZO_TRUE));
      data[datasize]='\0';
      if ( ret==LCRZO_ERR_OK )
      { lcrzo_epr(lcrzo_string_initm_range((lcrzo_string)data,
					   re[1].startindex_positive,
					   re[1].endindex_positive, 
					   &authorization));
        lcrzo_epr(lcrzo_string_initm_base64(authorization, &authlogpass));
	lcrzo_er(lcrzo_string_free2(&authorization));
        ret=lcrzo_string_search_regexp(authlogpass, LCRZO_FALSE, +1, -1,
				       "^(.*):(.*)$", re);
        if ( ret==LCRZO_ERR_OK )
        { lcrzo_epr(lcrzo_string_initm_range(authlogpass, 
					     re[1].startindex_positive,
					     re[1].endindex_positive, 
					     pproxylogin));
          lcrzo_epr(lcrzo_string_initm_range(authlogpass, 
					     re[2].startindex_positive,
					     re[2].endindex_positive, 
					     pproxypasswd));
	}
	else
	{ /* there is only the login */
	  lcrzo_epr(lcrzo_string_initm_text(authlogpass, pproxylogin));
	  if (pproxypasswd!=NULL) *pproxypasswd=NULL;
	}
	proxylogpassfound=LCRZO_TRUE;
	lcrzo_er(lcrzo_string_free2(&authlogpass));
      }
    }
   if ( !sitelogpassfound )
    { /* search Authorization */
      ret=lcrzo_data_search_regexp(data, datasize, LCRZO_FALSE, +1, -1,
				   "^Authorization: Basic (.+)$", re);
      data[datasize]='\0';
      if ( ret==LCRZO_ERR_OK )
      { lcrzo_epr(lcrzo_string_initm_range((lcrzo_string)data, 
					   re[1].startindex_positive,
					   re[1].endindex_positive, 
					   &authorization));
        lcrzo_epr(lcrzo_string_initm_base64(authorization, &authlogpass));
	lcrzo_er(lcrzo_string_free2(&authorization));
        ret=lcrzo_string_search_regexp(authlogpass, LCRZO_FALSE, +1, -1,
				       "^(.*):(.*)$", re);
        if ( ret==LCRZO_ERR_OK )
        { lcrzo_epr(lcrzo_string_initm_range(authlogpass, 
					     re[1].startindex_positive,
					     re[1].endindex_positive, 
					     psitelogin));
          lcrzo_epr(lcrzo_string_initm_range(authlogpass, 
					     re[2].startindex_positive,
					     re[2].endindex_positive, 
					     psitepasswd));
	}
	else
	{ /* there is only the login */
	  lcrzo_epr(lcrzo_string_initm_text(authlogpass, psitelogin));
	  if (psitepasswd!=NULL) *psitepasswd=NULL;
	}
	sitelogpassfound=LCRZO_TRUE;
	lcrzo_er(lcrzo_string_free2(&authlogpass));
      }
    }

    /* we do not need data */
    lcrzo_er(lcrzo_data_free2(&data));
  }

  /* set ouput values */
  if ( !hostfound && phost!=NULL )
  { lcrzo_epr(lcrzo_string_initm_text("", phost));
  }
  if ( !useragentfound && puseragent!=NULL )
  { lcrzo_epr(lcrzo_string_initm_text("", puseragent));
  }
  if ( !proxylogpassfound )
  { if (pproxylogin!=NULL) *pproxylogin=NULL;
    if (pproxypasswd!=NULL) *pproxypasswd=NULL;
  }
  if ( !sitelogpassfound )
  { if (psitelogin!=NULL) *psitelogin=NULL;
    if (psitepasswd!=NULL) *psitepasswd=NULL;
  }
  if ( !contentlengthfound && pcontentlength!=NULL )
  { *pcontentlength=0; 
  }
  if ( pheadersize!=NULL ) *pheadersize=headersize;

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_response_readheader(lcrzo_sock *psock,
				    lcrzo_uint16 *perrornumber,
				    lcrzo_string *perrormsg,
				    lcrzo_string *pservertype,
				    lcrzo_string *pcontenttype,
				    lcrzo_int32 *pcontentlength,
				    lcrzo_string *plocation,
				    lcrzo_data *pheader,
				    lcrzo_int32 *pheadersize)
{ lcrzo_data data;
  lcrzo_int32 datasize, headersize;
  lcrzo_string integerstring;
  lcrzo_regexp re;
  lcrzo_bool servertypefound, contenttypefound, contentlengthfound;
  lcrzo_bool locationfound;
  int ret;

  /* on the first line, there is the error code */
  lcrzo_epr(lcrzo_sock_readm_line(psock, LCRZO_WAIT_INFINITE, 
				  &data, &datasize));
  /*lcrzo_epr(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_TEXTN));*/
  ret=lcrzo_data_search_regexp(data, datasize, LCRZO_FALSE, +1, -1,
                                 "HTTP/... ([0-9]+) (.+)", re);
  data[datasize]='\0';
  if ( ret==LCRZO_ERR_OK )
  { lcrzo_epr(lcrzo_string_initm_range((lcrzo_string)data,
				       re[1].startindex_positive,
				       re[1].endindex_positive, 
				       &integerstring));
    lcrzo_epr(lcrzo_uint16_init_text(integerstring, perrornumber));
    lcrzo_er(lcrzo_string_free2(&integerstring));
    lcrzo_epr(lcrzo_string_initm_range((lcrzo_string)data, 
				       re[2].startindex_positive,
				       re[2].endindex_positive, 
				       perrormsg));
  }
  else
  { /* try this way (404 on dmoz.org for example) */
    ret=lcrzo_data_search_regexp(data, datasize, LCRZO_FALSE, +1, -1,
                                 "HTTP/... ([0-9]+)", re);
    if ( ret==LCRZO_ERR_OK )
    { lcrzo_epr(lcrzo_string_initm_range((lcrzo_string)data,
					 re[1].startindex_positive,
					 re[1].endindex_positive, 
					 &integerstring));
      lcrzo_epr(lcrzo_uint16_init_text(integerstring, perrornumber));
      lcrzo_er(lcrzo_string_free2(&integerstring));
    }
    else
    { if (perrornumber!=NULL)
      { *perrornumber=SHARED_HTTP_ERRORCODE_INTERNALSERVERERROR;
      }
    }
    lcrzo_epr(lcrzo_string_initm_text("", perrormsg));
  }

  lcrzo_epr(lcrzo_data_initm_data(data, datasize, pheader, &headersize));
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", headersize, pheader, &headersize));
  lcrzo_er(lcrzo_data_free2(&data));

  /* to default values in case they are not set in the header */
  servertypefound=LCRZO_FALSE;
  contenttypefound=LCRZO_FALSE;
  contentlengthfound=LCRZO_FALSE;
  locationfound=LCRZO_FALSE;

  /* we read the header till the empty line */
  while (datasize)
  { /* read the line */
    lcrzo_epr(lcrzo_sock_readm_line(psock, LCRZO_WAIT_INFINITE, 
				    &data, &datasize));
    /* add it to the returned header */     
    lcrzo_epr(lcrzo_data_appendm_data(data, datasize, headersize, 
				      pheader, &headersize));
    lcrzo_epr(lcrzo_data_appendm_text("\r\n", headersize,pheader,&headersize));
    /*lcrzo_epr(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_TEXTN));*/
    if ( !servertypefound )
    { /* search Server */
      ret=lcrzo_data_search_regexp(data, datasize, LCRZO_FALSE, +1, -1,
				   "^Server: (.+)$", re);
      data[datasize]='\0';
      if ( ret==LCRZO_ERR_OK )
      { lcrzo_epr(lcrzo_string_initm_range((lcrzo_string)data, 
					   re[1].startindex_positive,
					   re[1].endindex_positive, 
					   pservertype));
        servertypefound=LCRZO_TRUE;
      }
    }
    if ( !contenttypefound )
    { /* search Content-Type */
      lcrzo_er(lcrzo_global_set_data_regexp_casesensitive(LCRZO_FALSE));
      ret=lcrzo_data_search_regexp(data, datasize, LCRZO_FALSE, +1, -1,
				   "^Content-Type: (.+)$", re);
      lcrzo_er(lcrzo_global_set_data_regexp_casesensitive(LCRZO_TRUE));
      data[datasize]='\0';
      if ( ret==LCRZO_ERR_OK )
      { lcrzo_epr(lcrzo_string_initm_range((lcrzo_string)data, 
					   re[1].startindex_positive,
					   re[1].endindex_positive, 
					   pcontenttype));
        contenttypefound=LCRZO_TRUE;
      }
    }
    if ( !contentlengthfound )
    { /* search Content-Length */
      lcrzo_er(lcrzo_global_set_data_regexp_casesensitive(LCRZO_FALSE));
      ret=lcrzo_data_search_regexp(data, datasize, LCRZO_FALSE, +1, -1,
				   "^Content-Length: ([0-9]+)$", re);
      lcrzo_er(lcrzo_global_set_data_regexp_casesensitive(LCRZO_TRUE));
      data[datasize]='\0';
      if ( ret==LCRZO_ERR_OK )
      { lcrzo_epr(lcrzo_string_initm_range((lcrzo_string)data,
					   re[1].startindex_positive,
					   re[1].endindex_positive, 
					   &integerstring));
        if ( pcontentlength!=NULL ) *pcontentlength=atoi(integerstring);
	lcrzo_er(lcrzo_string_free2(&integerstring));
	contentlengthfound=LCRZO_TRUE;
      }
    }
    if ( !locationfound )
    { /* search Location */
      lcrzo_er(lcrzo_global_set_data_regexp_casesensitive(LCRZO_FALSE));
      ret=lcrzo_data_search_regexp(data, datasize, LCRZO_FALSE, +1, -1,
				   "^Location: (.+)$", re);
      lcrzo_er(lcrzo_global_set_data_regexp_casesensitive(LCRZO_TRUE));
      data[datasize]='\0';
      if ( ret==LCRZO_ERR_OK )
      { lcrzo_epr(lcrzo_string_initm_range((lcrzo_string)data, 
					   re[1].startindex_positive,
					   re[1].endindex_positive, 
					   plocation));
        locationfound=LCRZO_TRUE;
      }
    }
    /* we do not need data */
    lcrzo_er(lcrzo_data_free2(&data));
  }

  /* set ouput values */
  if ( !servertypefound && pservertype!=NULL )
  { lcrzo_epr(lcrzo_string_initm_text("", pservertype));
  }
  if ( !contenttypefound && pcontenttype!=NULL )
  { lcrzo_epr(lcrzo_string_initm_text("", pcontenttype));
  }
  if ( !contentlengthfound && pcontentlength!=NULL )
  { *pcontentlength=-1; /* this value indicates that the header 
                           didn't contain the Content-Length line.
			   In this case (rfc2068), we have to read
			   till the server closes the connection :(
			*/
  }
  if ( !locationfound && plocation!=NULL )
  { lcrzo_epr(lcrzo_string_initm_text("", plocation));
  }
  if ( pheadersize!=NULL ) *pheadersize=headersize;

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_response_readbody(lcrzo_sock *psock,
				  lcrzo_time *pmaxwaitingtime,
				  lcrzo_int32 contentlength,
				  lcrzo_data *pbody,
				  lcrzo_int32 *pbodysize)
{ lcrzo_data data;
  lcrzo_int32 datasize, bodysize;
  lcrzo_bool continuetoread;
  int ret;

  if ( contentlength != -1 )
  { lcrzo_er(lcrzo_sock_readm_fixed(psock, pmaxwaitingtime, 
				    contentlength, pbody));
    if ( pbodysize != NULL ) *pbodysize=contentlength;
  }
  else
  { /* -1 indicates that the header didn't contain the Content-Length line.
       In this case (rfc2068), we have to read till the server closes
       the connection :(
    */
    lcrzo_epr(lcrzo_data_initm_text("", pbody, &bodysize));
    continuetoread=LCRZO_TRUE;
    while (continuetoread)
    { ret=lcrzo_sock_readm(psock, pmaxwaitingtime, &data, &datasize);
      if (ret==LCRZO_ERR_OK)
      { lcrzo_epr(lcrzo_data_appendm_data(data, datasize, bodysize, 
					  pbody, &bodysize));
        lcrzo_epr(lcrzo_data_free2(&data));
      }
      else if (ret==LCRZO_ERR_OKDATAEND)
      { if ( pbodysize != NULL ) *pbodysize=bodysize;
        continuetoread=LCRZO_FALSE;
      }
      else
	return(ret);
    }
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_set_proxysite_userpass(int argc, 
				       char *argv[],
				       int startat,
				       lcrzo_string *pproxyuser, 
				       lcrzo_string *pproxypasswd, 
				       lcrzo_string *psiteuser,
				       lcrzo_string *psitepasswd)
{ lcrzo_string proxyuser, proxypasswd, siteuser, sitepasswd;
  /* Unfortunately, we cannot respect what's described before
     shared_http_url_init because user cannot distinct "" or NULL
     for proxy password for example because he has to enter a 
     site password after.
     So, we do :
       user="u"  passwd="p"    --> user="u" passwd="p"
       user="u"  passwd=""     --> user="u" passwd=""
       user="u"  passwd=NULL   --> user="u" passwd=NULL(ask user)
       user=""   passwd="p"    --> user="" passwd="p"
       user=""   passwd=""     --> user=NULL passwd=NULL -> no auth
       user=""   passwd=NULL   --> user=NULL passwd=NULL -> no auth
       user=NULL passwd=NULL   --> user=NULL passwd=NULL -> no auth
  */

  /* get proxy login */
  proxyuser=NULL;
  if ( argc>=startat+1 ) 
  { lcrzo_epr(shared_para_string_init(argv[startat], "Proxy login ", "",
				      &proxyuser));
  }

  /* get site login */
  siteuser=NULL;
  if ( argc>=startat+2 ) 
  { lcrzo_epr(shared_para_string_init(argv[startat+1], "Site login ", "",
				      &siteuser));
  }

  /* get proxy password */
  proxypasswd=NULL;
  if ( argc>=startat+3 )
  { lcrzo_epr(shared_para_string_init(argv[startat+2], "Proxy password ", 
				      "", &proxypasswd));
  }

  /* get site password */
  sitepasswd=NULL;
  if ( argc>=startat+4 )
  { lcrzo_epr(shared_para_string_init(argv[startat+3], "Site password ", 
				      "", &sitepasswd));
  }

  /* special cases of rewrite */
  if ( proxyuser!=NULL && strlen(proxyuser)==0 )
  { if ( proxypasswd==NULL )
    { lcrzo_er(lcrzo_string_free2(&proxyuser));
      proxyuser=NULL;
    }
    else if ( strlen(proxypasswd)==0 )
    { lcrzo_er(lcrzo_string_free2(&proxyuser));
      lcrzo_er(lcrzo_string_free2(&proxypasswd));
      proxyuser=NULL;
      proxypasswd=NULL;
    }
  }
  if ( siteuser!=NULL && strlen(siteuser)==0 )
  { if ( sitepasswd==NULL )
    { lcrzo_er(lcrzo_string_free2(&siteuser));
      siteuser=NULL;
    }
    else if ( strlen(sitepasswd)==0 )
    { lcrzo_er(lcrzo_string_free2(&siteuser));
      lcrzo_er(lcrzo_string_free2(&sitepasswd));
      siteuser=NULL;
      sitepasswd=NULL;
    }
  }

  /* get missing proxy passwd */
  if ( proxyuser!=NULL && proxypasswd==NULL )
  { lcrzo_epr(lcrzo_string_initm_text_passwd("Proxy passwd", "",
					     &proxypasswd));
  }

  /* get missing site passwd */
  if ( siteuser!=NULL && sitepasswd==NULL )
  { lcrzo_epr(lcrzo_string_initm_text_passwd("Site passwd", "",
					     &sitepasswd));
  }

  /* set output */
  if (pproxyuser==NULL)
  { if (proxyuser!=NULL)
    { lcrzo_er(lcrzo_string_free2(&proxyuser));
    }
  }
  else
  { *pproxyuser=proxyuser;
  }
  if (pproxypasswd==NULL)
  { if (proxypasswd!=NULL)
    { lcrzo_er(lcrzo_string_free2(&proxypasswd));
    }
  }
  else
  { *pproxypasswd=proxypasswd;
  }

  if (psiteuser==NULL)
  { if (siteuser!=NULL)
    { lcrzo_er(lcrzo_string_free2(&siteuser));
    }
  }
  else
  { *psiteuser=siteuser;
  }
  if (psitepasswd==NULL)
  { if (sitepasswd!=NULL)
    { lcrzo_er(lcrzo_string_free2(&sitepasswd));
    }
  }
  else
  { *psitepasswd=sitepasswd;
  }

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_priv_http_url_download_file(lcrzo_conststring server, 
				       lcrzo_uint16 port,
				       lcrzo_conststring url,
				       lcrzo_conststring proxylogin,
				       lcrzo_conststring proxypasswd,
				       lcrzo_conststring sitelogin,
				       lcrzo_conststring sitepasswd, 
				       lcrzo_conststring useragent, 
				       lcrzo_uint16 *perrorcode,
				       lcrzo_conststring localfile);
int shared_priv_http_url_download_file(lcrzo_conststring server, 
				       lcrzo_uint16 port,
				       lcrzo_conststring url,
				       lcrzo_conststring proxylogin,
				       lcrzo_conststring proxypasswd,
				       lcrzo_conststring sitelogin,
				       lcrzo_conststring sitepasswd, 
				       lcrzo_conststring useragent, 
				       lcrzo_uint16 *perrorcode,
				       lcrzo_conststring localfile)
{ lcrzo_ipl ipl;
  lcrzo_sock sock;
  lcrzo_string errormsg, servertype, contenttype, location, fakecontent;
  lcrzo_data header, body;
  lcrzo_int32 headersize, contentlength, bodysize;
  lcrzo_uint16 errorcode;

  /* open the connection */
  lcrzo_er(lcrzo_ipl_init_hs(server, &ipl));
  lcrzo_er(lcrzo_sock_tcpcli_real(ipl, port, &sock));
  
  /* send a request */
  lcrzo_er(shared_http_request_get2(&sock, url, server, port,
				    proxylogin, proxypasswd, 
				    sitelogin, sitepasswd, useragent));

  /* analyze the answer */
  lcrzo_er(shared_http_response_readheader(&sock,
					   &errorcode,
					   &errormsg,
					   &servertype,
					   &contenttype,
					   &contentlength,
					   &location,
					   &header, &headersize));
  if (perrorcode!=NULL) *perrorcode=errorcode;

  /*  printf("Return (%s)\n", errormsg);
  printf("Server: %s\n", servertype);
  printf("Content-Type: %s\n", contenttype);
  lcrzo_er(lcrzo_int32_print("Content-length: ", contentlength, "%d", "\n"));
  lcrzo_er(lcrzo_data_print(header, headersize, LCRZO_PRINTTYPE_DUMP));
  */
  lcrzo_er(lcrzo_string_free2(&errormsg));
  lcrzo_er(lcrzo_string_free2(&servertype));
  lcrzo_er(lcrzo_string_free2(&contenttype));
  lcrzo_er(lcrzo_data_free2(&header));

  /* get the body */
  if (contentlength)
  { lcrzo_er(shared_http_response_readbody(&sock, LCRZO_WAIT_INFINITE,
					    contentlength, 
					    &body, &bodysize));
    lcrzo_er(lcrzo_filename_write(localfile, LCRZO_FALSE, LCRZO_FALSE,
				   body, bodysize));
    lcrzo_er(lcrzo_data_free2(&body));
  }
  else
  { /* eventually create a fake file */
    if ( ( errorcode==SHARED_HTTP_ERRORCODE_CREATED || 
	   errorcode==SHARED_HTTP_ERRORCODE_MOVEDPERMANENTLY || 
	   errorcode==SHARED_HTTP_ERRORCODE_MOVEDTEMPORARILY ) &&
	 strlen(location) )
    { lcrzo_er(lcrzo_string_initm_text("<HTML><BODY>", &fakecontent));
      lcrzo_er(lcrzo_string_appendm_text("Error ", &fakecontent));
      lcrzo_er(lcrzo_string_appendm_uint16(errorcode, "%u", &fakecontent));
      lcrzo_er(lcrzo_string_appendm_text(" was received.<BR>\n", 
					 &fakecontent));
      lcrzo_er(lcrzo_string_appendm_text("Header Location contains ", 
					 &fakecontent));
      lcrzo_er(lcrzo_string_appendm_text("<A HREF=\"", &fakecontent));
      lcrzo_er(lcrzo_string_appendm_text(location, &fakecontent));
      lcrzo_er(lcrzo_string_appendm_text("\">", &fakecontent));
      lcrzo_er(lcrzo_string_appendm_text(location, &fakecontent));
      lcrzo_er(lcrzo_string_appendm_text("</A>.", &fakecontent));
      lcrzo_er(lcrzo_filename_write(localfile, LCRZO_FALSE, LCRZO_FALSE,
				    (lcrzo_data)fakecontent, 
				    strlen(fakecontent)));
      lcrzo_er(lcrzo_string_free2(&fakecontent));
    }
  }
  lcrzo_er(lcrzo_string_free2(&location));

  lcrzo_er(lcrzo_sock_close(&sock));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_priv_http_url_download_ftp(lcrzo_conststring server, 
				      lcrzo_uint16 port,
				      lcrzo_conststring login,
				      lcrzo_conststring passwd, 
				      lcrzo_conststring url,
				      lcrzo_conststring localfile);
int shared_priv_http_url_download_ftp(lcrzo_conststring server, 
				      lcrzo_uint16 port,
				      lcrzo_conststring login,
				      lcrzo_conststring passwd, 
				      lcrzo_conststring url,
				      lcrzo_conststring localfile)
{ shared_sock_address ssa;
  lcrzo_sock sockcontrol, sockdata;
  int ret, ret2;

  /* initialize IP addresses */
  lcrzo_er(lcrzo_ipl_init_hs(server, &ssa.serveripl));
  lcrzo_er(lcrzo_conf_route_to_host_ip(ssa.serveripl, NULL, &ssa.clientipl, 
				       NULL));

  /* initialize ports */
  lcrzo_er(lcrzo_uint16_init_rand(32000, 0xFFFF, &ssa.clientport));
  ssa.serverport=port;
  lcrzo_er(lcrzo_uint16_init_rand(32000, 0xFFFF, &ssa.clientport2));

  /* initialize control connection */
  lcrzo_er(lcrzo_sock_tcpcli_real2(ssa.clientipl, ssa.serveripl,
				   ssa.clientport, ssa.serverport, 
				   NULL, 0, &sockcontrol));

  /* first, we connect */
  if ( login==NULL && passwd==NULL )
  { ret=shared_ftp_login(&sockcontrol, "anonymous", "lcrzoex@s.com");
  }
  else if ( login==NULL && passwd!=NULL )
  { ret=LCRZOEX_ERR_HTTP_LOGNULLPWDNOTNULL;
  }
  else if ( login!=NULL && passwd==NULL )
  { /* we suppose passwd is empty (really, we should ask the user) */
    ret=shared_ftp_login(&sockcontrol, login, "");
  }
  else
  { ret=shared_ftp_login(&sockcontrol, login, passwd);
  }

  if (ret==LCRZO_ERR_OK)
  { /* then, we send a 'PASV' */
    ret=shared_ftp_cmd_pasv(&sockcontrol, &ssa.serveripl,
			    &ssa.serverport2);
    if (ret==LCRZO_ERR_OK)
    { /* initialize data connection */
      ret=lcrzo_sock_tcpcli_real2(ssa.clientipl, ssa.serveripl, 
				  ssa.clientport2, ssa.serverport2,
				  NULL, 0,
				  &sockdata);
      if (ret==LCRZO_ERR_OK)
      { /* we send a 'RETR', and get file */
        ret=shared_ftp_data_retr(&sockcontrol, LCRZO_TRUE,
				 &sockdata, NULL, url, localfile);
      }
    }
  }

  /* then, we send a 'QUIT' */
  ret2=shared_ftp_cmd_quit(&sockcontrol);
  if ( ret==LCRZO_ERR_OK ) ret=ret2;

  /* close the socket */
  lcrzo_er(lcrzo_sock_close(&sockcontrol));

  return(ret);
}

/*-------------------------------------------------------------*/
int shared_http_url_download_file2(lcrzo_conststring siteurl, 
				   lcrzo_conststring proxyurl, 
				   lcrzo_conststring useragent, 
				   lcrzo_uint16 *perrorcode,
				   lcrzo_conststring localfile)
{ lcrzo_string url;
  lcrzo_uint16 siteport, proxyport;
  lcrzo_string siteproto, sitelogin, sitepasswd, siteserver, sitefile;
  lcrzo_string proxyproto, proxylogin, proxypasswd, proxyserver;
  lcrzo_string sitevarval;
  lcrzo_bool useproxy;
  int ret; 

  /* extract parts of the url */
  lcrzo_er(shared_http_url_decode(siteurl, &siteproto, &sitelogin,
				  &sitepasswd, &siteserver, &siteport,
				  &sitefile, &sitevarval, NULL)); 
  ret=shared_http_url_decode(proxyurl, &proxyproto, &proxylogin,
			     &proxypasswd, &proxyserver, &proxyport,
			     NULL, NULL, NULL); 
  useproxy=LCRZO_TRUE;
  if (ret!=LCRZO_ERR_OK)
  { useproxy=LCRZO_FALSE;
  }

  if ( useproxy )
  { /* use proxy */ 
    lcrzo_er(lcrzo_string_initm_text(siteproto, &url));
    lcrzo_er(lcrzo_string_appendm_text("://", &url));
    lcrzo_er(lcrzo_string_appendm_text(siteserver, &url));
    lcrzo_er(lcrzo_string_appendm_text(":", &url));
    lcrzo_er(lcrzo_string_appendm_uint16(siteport, "%u", &url));
    lcrzo_er(lcrzo_string_appendm_text(sitefile, &url));
    if (strlen(sitevarval))
    { lcrzo_er(lcrzo_string_appendm_text("?", &url));
      lcrzo_er(lcrzo_string_appendm_text(sitevarval, &url));
    }
    ret=shared_priv_http_url_download_file(proxyserver, proxyport,
					   url,
					   proxylogin, proxypasswd,
					   sitelogin, sitepasswd, useragent,
					   perrorcode, localfile);
    lcrzo_er(lcrzo_string_free2(&url));
  }
  else
  { if ( !strcmp(siteproto, "http") )
    { /* connect to the server */
      lcrzo_er(lcrzo_string_initm_text(sitefile, &url));
      if (strlen(sitevarval))
      { lcrzo_er(lcrzo_string_appendm_text("?", &url));
        lcrzo_er(lcrzo_string_appendm_text(sitevarval, &url));
      }
      ret=shared_priv_http_url_download_file(siteserver, siteport,
					     url,
					     NULL, NULL,
					     sitelogin, sitepasswd, useragent,
					     perrorcode, localfile);
      lcrzo_er(lcrzo_string_free2(&url));
    }
    else if ( !strcmp(siteproto, "ftp") )
    { /* Note: download of a file is supported, but download of a 
               FTP directory is not yet supported */
      ret=shared_priv_http_url_download_ftp(siteserver, siteport,
					    sitelogin, sitepasswd, 
					    sitefile, localfile);
      if (perrorcode!=NULL) *perrorcode=SHARED_HTTP_ERRORCODE_OK;
    }
    else
    { ret=LCRZO_ERR_IEUNIMPLEMENTED;
    }
  }

  /* free variables */
  lcrzo_er(lcrzo_string_free2(&siteproto));
  if (sitelogin!=NULL) { lcrzo_er(lcrzo_string_free2(&sitelogin)); }
  if (sitepasswd!=NULL) { lcrzo_er(lcrzo_string_free2(&sitepasswd)); }
  lcrzo_er(lcrzo_string_free2(&siteserver));
  lcrzo_er(lcrzo_string_free2(&sitefile));
  lcrzo_er(lcrzo_string_free2(&sitevarval));
  if (useproxy)
  { lcrzo_er(lcrzo_string_free2(&proxyproto));
    if (proxylogin!=NULL) { lcrzo_er(lcrzo_string_free2(&proxylogin)); }
    if (proxypasswd!=NULL) { lcrzo_er(lcrzo_string_free2(&proxypasswd)); }
    lcrzo_er(lcrzo_string_free2(&proxyserver));
  }
  return(ret);
}

/*-------------------------------------------------------------*/
int shared_http_url_download_file(lcrzo_conststring siteurl, 
				  lcrzo_conststring proxyurl, 
				  lcrzo_uint16 *perrorcode,
				  lcrzo_conststring localfile)
{ return(shared_http_url_download_file2(siteurl, proxyurl, NULL,
					perrorcode, localfile));
}

/*-------------------------------------------------------------*/
int shared_priv_http_linktype_get_infos(lcrzo_uint32 linktype,
					lcrzo_string *ptag,
					lcrzo_string *ppara,
					lcrzo_uint16 *pcontain,
					lcrzo_uint16 *pformat);
int shared_priv_http_linktype_get_infos(lcrzo_uint32 linktype,
					lcrzo_string *ptag,
					lcrzo_string *ppara,
					lcrzo_uint16 *pcontain,
					lcrzo_uint16 *pformat)
{ lcrzo_uint16 linkcontain, linkformat;

  switch((int)linktype) {
  case SHARED_HTTP_LINKTYPE_A_HREF :
    lcrzo_er(lcrzo_string_initm_text("a", ptag));
    lcrzo_er(lcrzo_string_initm_text("href", ppara));
    linkcontain=SHARED_HTTP_LINK_CONTAIN_HTML;
    linkformat=SHARED_HTTP_LINK_FORMAT_URL;
    break;
  case SHARED_HTTP_LINKTYPE_APPLET_ARCHIVE :
    lcrzo_er(lcrzo_string_initm_text("applet", ptag));
    lcrzo_er(lcrzo_string_initm_text("archive", ppara));
    linkcontain=SHARED_HTTP_LINK_CONTAIN_CODE;
    linkformat=SHARED_HTTP_LINK_FORMAT_URLLISTCOMMA;
    break;
  case SHARED_HTTP_LINKTYPE_APPLET_CODE :
    lcrzo_er(lcrzo_string_initm_text("applet", ptag));
    lcrzo_er(lcrzo_string_initm_text("code", ppara));
    linkcontain=SHARED_HTTP_LINK_CONTAIN_CODE;
    linkformat=SHARED_HTTP_LINK_FORMAT_URL;
    break;
  case SHARED_HTTP_LINKTYPE_APPLET_CODEBASE :
    lcrzo_er(lcrzo_string_initm_text("applet", ptag));
    lcrzo_er(lcrzo_string_initm_text("codebase", ppara));
    linkcontain=SHARED_HTTP_LINK_CONTAIN_REFERENCE;
    linkformat=SHARED_HTTP_LINK_FORMAT_URL;
    break;
  case SHARED_HTTP_LINKTYPE_AREA_HREF :
    lcrzo_er(lcrzo_string_initm_text("area", ptag));
    lcrzo_er(lcrzo_string_initm_text("href", ppara));
    linkcontain=SHARED_HTTP_LINK_CONTAIN_HTML;
    linkformat=SHARED_HTTP_LINK_FORMAT_URL;
    break;
  case SHARED_HTTP_LINKTYPE_BASE_HREF :
    lcrzo_er(lcrzo_string_initm_text("base", ptag));
    lcrzo_er(lcrzo_string_initm_text("href", ppara));
    linkcontain=SHARED_HTTP_LINK_CONTAIN_REFERENCE;
    linkformat=SHARED_HTTP_LINK_FORMAT_URL;
    break;
  case SHARED_HTTP_LINKTYPE_BGSOUND_SRC :
    lcrzo_er(lcrzo_string_initm_text("bgsound", ptag));
    lcrzo_er(lcrzo_string_initm_text("src", ppara));
    linkcontain=SHARED_HTTP_LINK_CONTAIN_MULTIMEDIA;
    linkformat=SHARED_HTTP_LINK_FORMAT_URL;
    break;
  case SHARED_HTTP_LINKTYPE_BLOCKQUOTE_CITE :
    lcrzo_er(lcrzo_string_initm_text("blockquote", ptag));
    lcrzo_er(lcrzo_string_initm_text("cite", ppara));
    linkcontain=SHARED_HTTP_LINK_CONTAIN_HTML;
    linkformat=SHARED_HTTP_LINK_FORMAT_URL;
    break;
  case SHARED_HTTP_LINKTYPE_BODY_BACKGROUND :
    lcrzo_er(lcrzo_string_initm_text("body", ptag));
    lcrzo_er(lcrzo_string_initm_text("background", ppara));
    linkcontain=SHARED_HTTP_LINK_CONTAIN_IMAGE;
    linkformat=SHARED_HTTP_LINK_FORMAT_URL;
    break;
  case SHARED_HTTP_LINKTYPE_DEL_CITE :
    lcrzo_er(lcrzo_string_initm_text("del", ptag));
    lcrzo_er(lcrzo_string_initm_text("cite", ppara));
    linkcontain=SHARED_HTTP_LINK_CONTAIN_HTML;
    linkformat=SHARED_HTTP_LINK_FORMAT_URL;
    break;
  case SHARED_HTTP_LINKTYPE_EMBED_PLUGINSPAGE :
    lcrzo_er(lcrzo_string_initm_text("embed", ptag));
    lcrzo_er(lcrzo_string_initm_text("pluginspage", ppara));
    linkcontain=SHARED_HTTP_LINK_CONTAIN_HTML;
    linkformat=SHARED_HTTP_LINK_FORMAT_URL;
    break;
  case SHARED_HTTP_LINKTYPE_EMBED_SRC :
    lcrzo_er(lcrzo_string_initm_text("embed", ptag));
    lcrzo_er(lcrzo_string_initm_text("src", ppara));
    linkcontain=SHARED_HTTP_LINK_CONTAIN_MULTIMEDIA;
    linkformat=SHARED_HTTP_LINK_FORMAT_URL;
    break;
  case SHARED_HTTP_LINKTYPE_EMBED_HREF :
    lcrzo_er(lcrzo_string_initm_text("embed", ptag));
    lcrzo_er(lcrzo_string_initm_text("href", ppara));
    linkcontain=SHARED_HTTP_LINK_CONTAIN_HTML;
    linkformat=SHARED_HTTP_LINK_FORMAT_URL;
    break;
  case SHARED_HTTP_LINKTYPE_FIG_IMAGEMAP :
    lcrzo_er(lcrzo_string_initm_text("fig", ptag));
    lcrzo_er(lcrzo_string_initm_text("imagemap", ppara));
    linkcontain=SHARED_HTTP_LINK_CONTAIN_HTML;
    linkformat=SHARED_HTTP_LINK_FORMAT_URL;
    break;
  case SHARED_HTTP_LINKTYPE_FIG_SRC :
    lcrzo_er(lcrzo_string_initm_text("fig", ptag));
    lcrzo_er(lcrzo_string_initm_text("src", ppara));
    linkcontain=SHARED_HTTP_LINK_CONTAIN_IMAGE;
    linkformat=SHARED_HTTP_LINK_FORMAT_URL;
    break;
  case SHARED_HTTP_LINKTYPE_FORM_ACTION :
    lcrzo_er(lcrzo_string_initm_text("form", ptag));
    lcrzo_er(lcrzo_string_initm_text("action", ppara));
    linkcontain=SHARED_HTTP_LINK_CONTAIN_HTML;
    linkformat=SHARED_HTTP_LINK_FORMAT_URL;
    break;
  case SHARED_HTTP_LINKTYPE_FRAME_LONGDESC :
    lcrzo_er(lcrzo_string_initm_text("frame", ptag));
    lcrzo_er(lcrzo_string_initm_text("longdesc", ppara));
    linkcontain=SHARED_HTTP_LINK_CONTAIN_HTML;
    linkformat=SHARED_HTTP_LINK_FORMAT_URL;
    break;
  case SHARED_HTTP_LINKTYPE_FRAME_SRC :
    lcrzo_er(lcrzo_string_initm_text("frame", ptag));
    lcrzo_er(lcrzo_string_initm_text("src", ppara));
    linkcontain=SHARED_HTTP_LINK_CONTAIN_HTML;
    linkformat=SHARED_HTTP_LINK_FORMAT_URL;
    break;
  case SHARED_HTTP_LINKTYPE_HEAD_PROFILE :
    lcrzo_er(lcrzo_string_initm_text("head", ptag));
    lcrzo_er(lcrzo_string_initm_text("profile", ppara));
    linkcontain=SHARED_HTTP_LINK_CONTAIN_HTML;
    linkformat=SHARED_HTTP_LINK_FORMAT_URLLISTSPACE;
    break;
  case SHARED_HTTP_LINKTYPE_IFRAME_LONGDESC :
    lcrzo_er(lcrzo_string_initm_text("iframe", ptag));
    lcrzo_er(lcrzo_string_initm_text("longdesc", ppara));
    linkcontain=SHARED_HTTP_LINK_CONTAIN_HTML;
    linkformat=SHARED_HTTP_LINK_FORMAT_URL;
    break;
  case SHARED_HTTP_LINKTYPE_IFRAME_SRC :
    lcrzo_er(lcrzo_string_initm_text("iframe", ptag));
    lcrzo_er(lcrzo_string_initm_text("src", ppara));
    linkcontain=SHARED_HTTP_LINK_CONTAIN_HTML;
    linkformat=SHARED_HTTP_LINK_FORMAT_URL;
    break;
  case SHARED_HTTP_LINKTYPE_ILAYER_BACKGROUND :
    lcrzo_er(lcrzo_string_initm_text("ilayer", ptag));
    lcrzo_er(lcrzo_string_initm_text("backgroung", ppara));
    linkcontain=SHARED_HTTP_LINK_CONTAIN_IMAGE;
    linkformat=SHARED_HTTP_LINK_FORMAT_URL;
    break;
  case SHARED_HTTP_LINKTYPE_ILAYER_SRC :
    lcrzo_er(lcrzo_string_initm_text("ilayer", ptag));
    lcrzo_er(lcrzo_string_initm_text("src", ppara));
    linkcontain=SHARED_HTTP_LINK_CONTAIN_HTML;
    linkformat=SHARED_HTTP_LINK_FORMAT_URL;
    break;
  case SHARED_HTTP_LINKTYPE_IMG_DYNSRC :
    lcrzo_er(lcrzo_string_initm_text("img", ptag));
    lcrzo_er(lcrzo_string_initm_text("dynsrc", ppara));
    linkcontain=SHARED_HTTP_LINK_CONTAIN_MULTIMEDIA;
    linkformat=SHARED_HTTP_LINK_FORMAT_URL;
    break;
  case SHARED_HTTP_LINKTYPE_IMG_LONGDESC :
    lcrzo_er(lcrzo_string_initm_text("img", ptag));
    lcrzo_er(lcrzo_string_initm_text("longdesc", ppara));
    linkcontain=SHARED_HTTP_LINK_CONTAIN_HTML;
    linkformat=SHARED_HTTP_LINK_FORMAT_URL;
    break;
  case SHARED_HTTP_LINKTYPE_IMG_LOWSRC :
    lcrzo_er(lcrzo_string_initm_text("img", ptag));
    lcrzo_er(lcrzo_string_initm_text("lowsrc", ppara));
    linkcontain=SHARED_HTTP_LINK_CONTAIN_IMAGE;
    linkformat=SHARED_HTTP_LINK_FORMAT_URL;
    break;
  case SHARED_HTTP_LINKTYPE_IMG_SRC :
    lcrzo_er(lcrzo_string_initm_text("img", ptag));
    lcrzo_er(lcrzo_string_initm_text("src", ppara));
    linkcontain=SHARED_HTTP_LINK_CONTAIN_IMAGE;
    linkformat=SHARED_HTTP_LINK_FORMAT_URL;
    break;
  case SHARED_HTTP_LINKTYPE_IMG_USEMAP :
    lcrzo_er(lcrzo_string_initm_text("img", ptag));
    lcrzo_er(lcrzo_string_initm_text("usemap", ppara));
    linkcontain=SHARED_HTTP_LINK_CONTAIN_HTML;
    linkformat=SHARED_HTTP_LINK_FORMAT_URL;
    break;
  case SHARED_HTTP_LINKTYPE_INPUT_SRC :
    lcrzo_er(lcrzo_string_initm_text("input", ptag));
    lcrzo_er(lcrzo_string_initm_text("src", ppara));
    linkcontain=SHARED_HTTP_LINK_CONTAIN_IMAGE;
    linkformat=SHARED_HTTP_LINK_FORMAT_URL;
    break;
  case SHARED_HTTP_LINKTYPE_INPUT_USEMAP :
    lcrzo_er(lcrzo_string_initm_text("input", ptag));
    lcrzo_er(lcrzo_string_initm_text("usemap", ppara));
    linkcontain=SHARED_HTTP_LINK_CONTAIN_HTML;
    linkformat=SHARED_HTTP_LINK_FORMAT_URL;
    break;
  case SHARED_HTTP_LINKTYPE_INS_CITE :
    lcrzo_er(lcrzo_string_initm_text("ins", ptag));
    lcrzo_er(lcrzo_string_initm_text("cite", ppara));
    linkcontain=SHARED_HTTP_LINK_CONTAIN_HTML;
    linkformat=SHARED_HTTP_LINK_FORMAT_URL;
    break;
  case SHARED_HTTP_LINKTYPE_LAYER_BACKGROUND :
    lcrzo_er(lcrzo_string_initm_text("layer", ptag));
    lcrzo_er(lcrzo_string_initm_text("background", ppara));
    linkcontain=SHARED_HTTP_LINK_CONTAIN_IMAGE;
    linkformat=SHARED_HTTP_LINK_FORMAT_URL;
    break;
  case SHARED_HTTP_LINKTYPE_LAYER_SRC :
    lcrzo_er(lcrzo_string_initm_text("layer", ptag));
    lcrzo_er(lcrzo_string_initm_text("src", ppara));
    linkcontain=SHARED_HTTP_LINK_CONTAIN_HTML;
    linkformat=SHARED_HTTP_LINK_FORMAT_URL;
    break;
  case SHARED_HTTP_LINKTYPE_LINK_HREF :
    lcrzo_er(lcrzo_string_initm_text("link", ptag));
    lcrzo_er(lcrzo_string_initm_text("href", ppara));
    linkcontain=SHARED_HTTP_LINK_CONTAIN_HTML;
    linkformat=SHARED_HTTP_LINK_FORMAT_URL;
    break;
  case SHARED_HTTP_LINKTYPE_META_CONTENT :
    lcrzo_er(lcrzo_string_initm_text("meta", ptag));
    lcrzo_er(lcrzo_string_initm_text("content", ppara));
    linkcontain=SHARED_HTTP_LINK_CONTAIN_HTML;
    linkformat=SHARED_HTTP_LINK_FORMAT_REFRESHURL;
    break;
  case SHARED_HTTP_LINKTYPE_OBJECT_ARCHIVE :
    lcrzo_er(lcrzo_string_initm_text("object", ptag));
    lcrzo_er(lcrzo_string_initm_text("archive", ppara));
    linkcontain=SHARED_HTTP_LINK_CONTAIN_CODE;
    linkformat=SHARED_HTTP_LINK_FORMAT_URLLISTSPACE;
    break;
  case SHARED_HTTP_LINKTYPE_OBJECT_CLASSID :
    lcrzo_er(lcrzo_string_initm_text("object", ptag));
    lcrzo_er(lcrzo_string_initm_text("classid", ppara));
    linkcontain=SHARED_HTTP_LINK_CONTAIN_CODE;
    linkformat=SHARED_HTTP_LINK_FORMAT_URL;
    break;
  case SHARED_HTTP_LINKTYPE_OBJECT_CODEBASE :
    lcrzo_er(lcrzo_string_initm_text("object", ptag));
    lcrzo_er(lcrzo_string_initm_text("codebase", ppara));
    linkcontain=SHARED_HTTP_LINK_CONTAIN_REFERENCE;
    linkformat=SHARED_HTTP_LINK_FORMAT_URL;
    break;
  case SHARED_HTTP_LINKTYPE_OBJECT_DATA :
    lcrzo_er(lcrzo_string_initm_text("object", ptag));
    lcrzo_er(lcrzo_string_initm_text("data", ppara));
    linkcontain=SHARED_HTTP_LINK_CONTAIN_CODE;
    linkformat=SHARED_HTTP_LINK_FORMAT_URL;
    break;
  case SHARED_HTTP_LINKTYPE_OBJECT_USEMAP :
    lcrzo_er(lcrzo_string_initm_text("object", ptag));
    lcrzo_er(lcrzo_string_initm_text("usemap", ppara));
    linkcontain=SHARED_HTTP_LINK_CONTAIN_HTML;
    linkformat=SHARED_HTTP_LINK_FORMAT_URL;
    break;
  case SHARED_HTTP_LINKTYPE_OVERLAY_IMAGEMAP :
    lcrzo_er(lcrzo_string_initm_text("overlay", ptag));
    lcrzo_er(lcrzo_string_initm_text("imagemap", ppara));
    linkcontain=SHARED_HTTP_LINK_CONTAIN_HTML;
    linkformat=SHARED_HTTP_LINK_FORMAT_URL;
    break;
  case SHARED_HTTP_LINKTYPE_OVERLAY_SRC :
    lcrzo_er(lcrzo_string_initm_text("overlay", ptag));
    lcrzo_er(lcrzo_string_initm_text("src", ppara));
    linkcontain=SHARED_HTTP_LINK_CONTAIN_IMAGE;
    linkformat=SHARED_HTTP_LINK_FORMAT_URL;
    break;
  case SHARED_HTTP_LINKTYPE_Q_CITE :
    lcrzo_er(lcrzo_string_initm_text("q", ptag));
    lcrzo_er(lcrzo_string_initm_text("cite", ppara));
    linkcontain=SHARED_HTTP_LINK_CONTAIN_HTML;
    linkformat=SHARED_HTTP_LINK_FORMAT_URL;
    break;
  case SHARED_HTTP_LINKTYPE_SCRIPT_FOR :
    lcrzo_er(lcrzo_string_initm_text("script", ptag));
    lcrzo_er(lcrzo_string_initm_text("for", ppara));
    linkcontain=SHARED_HTTP_LINK_CONTAIN_CODE;
    linkformat=SHARED_HTTP_LINK_FORMAT_URL;
    break;
  case SHARED_HTTP_LINKTYPE_SCRIPT_SRC :
    lcrzo_er(lcrzo_string_initm_text("script", ptag));
    lcrzo_er(lcrzo_string_initm_text("src", ppara));
    linkcontain=SHARED_HTTP_LINK_CONTAIN_CODE;
    linkformat=SHARED_HTTP_LINK_FORMAT_URL;
    break;
  case SHARED_HTTP_LINKTYPE_SELECT_SRC :
    lcrzo_er(lcrzo_string_initm_text("select", ptag));
    lcrzo_er(lcrzo_string_initm_text("src", ppara));
    linkcontain=SHARED_HTTP_LINK_CONTAIN_IMAGE;
    linkformat=SHARED_HTTP_LINK_FORMAT_URL;
    break;
  case SHARED_HTTP_LINKTYPE_TABLE_BACKGROUND :
    lcrzo_er(lcrzo_string_initm_text("table", ptag));
    lcrzo_er(lcrzo_string_initm_text("background", ppara));
    linkcontain=SHARED_HTTP_LINK_CONTAIN_IMAGE;
    linkformat=SHARED_HTTP_LINK_FORMAT_URL;
    break;
  case SHARED_HTTP_LINKTYPE_TD_BACKGROUND :
    lcrzo_er(lcrzo_string_initm_text("td", ptag));
    lcrzo_er(lcrzo_string_initm_text("background", ppara));
    linkcontain=SHARED_HTTP_LINK_CONTAIN_IMAGE;
    linkformat=SHARED_HTTP_LINK_FORMAT_URL;
    break;
  case SHARED_HTTP_LINKTYPE_TH_BACKGROUND :
    lcrzo_er(lcrzo_string_initm_text("th", ptag));
    lcrzo_er(lcrzo_string_initm_text("background", ppara));
    linkcontain=SHARED_HTTP_LINK_CONTAIN_IMAGE;
    linkformat=SHARED_HTTP_LINK_FORMAT_URL;
    break;
  case SHARED_HTTP_LINKTYPE_TR_BACKGROUND :
    lcrzo_er(lcrzo_string_initm_text("tr", ptag));
    lcrzo_er(lcrzo_string_initm_text("background", ppara));
    linkcontain=SHARED_HTTP_LINK_CONTAIN_IMAGE;
    linkformat=SHARED_HTTP_LINK_FORMAT_URL;
    break;
  case SHARED_HTTP_LINKTYPE_XML_SRC :
    lcrzo_er(lcrzo_string_initm_text("xml", ptag));
    lcrzo_er(lcrzo_string_initm_text("src", ppara));
    linkcontain=SHARED_HTTP_LINK_CONTAIN_HTML;
    linkformat=SHARED_HTTP_LINK_FORMAT_URL;
    break;
  default:
    lcrzo_er(lcrzo_uint32_print("LinkType ", linktype , "%u",
				" unrecognized\n"));
    return(LCRZO_ERR_PAUNKNOWNTYPE);
  }

  if (pcontain!=NULL) *pcontain=linkcontain;
  if (pformat!=NULL) *pformat=linkformat;
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_linktype_get_tag(lcrzo_uint32 linktype,
				 lcrzo_string *ptag)
{ return(shared_priv_http_linktype_get_infos(linktype, ptag, NULL, 
					     NULL, NULL));
}

/*-------------------------------------------------------------*/
int shared_http_linktype_get_para(lcrzo_uint32 linktype,
				  lcrzo_string *ppara)
{ return(shared_priv_http_linktype_get_infos(linktype, NULL, ppara,
					    NULL, NULL));
}

/*-------------------------------------------------------------*/
int shared_http_linktype_get_contain(lcrzo_uint32 linktype,
				     lcrzo_uint16 *pcontain)
{ return(shared_priv_http_linktype_get_infos(linktype, NULL, NULL, 
					    pcontain, NULL));
}

/*-------------------------------------------------------------*/
int shared_http_linktype_get_format(lcrzo_uint32 linktype,
				    lcrzo_uint16 *pformat)
{ return(shared_priv_http_linktype_get_infos(linktype, NULL, NULL,
					    NULL, pformat));
}

/*-------------------------------------------------------------*/
typedef struct {
  lcrzo_uint32 type;
  lcrzo_string purl;
  lcrzo_int32 startpos;
  lcrzo_int32 endpos;
  lcrzo_uint32 refresh;
} shared_http_urllist;

/*-------------------------------------------------------------*/
int shared_http_urllist_free(lcrzo_ptr pelem);
int shared_http_urllist_free(lcrzo_ptr pelem)
{ shared_http_urllist *purllist;
  purllist=(shared_http_urllist *)pelem;
  lcrzo_er(lcrzo_string_free2(&(purllist->purl)));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_urllist_cmp(lcrzo_constptr peleminf,
			    lcrzo_constptr pelemsup,
			    lcrzo_ptr pinfos,
			    lcrzo_int8 *pcmp);
int shared_http_urllist_cmp(lcrzo_constptr peleminf,
			    lcrzo_constptr pelemsup,
			    lcrzo_ptr pinfos,
			    lcrzo_int8 *pcmp)
{ const shared_http_urllist *plinf=(const shared_http_urllist *)peleminf;
  const shared_http_urllist *plsup=(const shared_http_urllist *)pelemsup;
  lcrzo_int8 cmp;

  if ( plinf->startpos < plsup->startpos ) cmp=-1;
  else if ( plinf->startpos > plsup->startpos ) cmp=+1;
  else cmp=0;
  if (pcmp!=NULL) *pcmp=cmp;
  return(LCRZO_ERR_OK);
  pinfos=pinfos;
}

/*-------------------------------------------------------------*/
int shared_http_urllist_init(lcrzo_list *plist)
{ return(lcrzo_list_init(plist, sizeof(shared_http_urllist),
                         &shared_http_urllist_free));
}

/*-------------------------------------------------------------*/
int shared_http_urllist_add_pos(lcrzo_list *plist,
				lcrzo_int32 pos,
				lcrzo_uint32 type,
				lcrzo_conststring url,
				lcrzo_int32 startpos,
				lcrzo_int32 endpos,
				lcrzo_uint32 refresh)
{ shared_http_urllist urllist;

  /*parameters verification*/
  if (plist==NULL) return(LCRZO_ERR_PANULLPTR);
  if (url==NULL) return(LCRZO_ERR_PANULLPTR);

  /*create the string*/
  urllist.type=type;
  lcrzo_er(lcrzo_string_initm_text(url, &(urllist.purl)));
  urllist.startpos=startpos;
  urllist.endpos=endpos;
  urllist.refresh=refresh;

  /*add the string*/
  lcrzo_er(lcrzo_list_add_pos(plist, pos, &urllist));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_urllist_valuem_pos(lcrzo_list *plist,
				   lcrzo_int32 pos,
				   lcrzo_uint32 *ptype,
				   lcrzo_string *purl,
				   lcrzo_int32 *pstartpos,
				   lcrzo_int32 *pendpos,
				   lcrzo_uint32 *prefresh)
{ shared_http_urllist urllist;

  /*get the value*/
  lcrzo_er(lcrzo_list_value_pos(plist, pos, &urllist));

  if (ptype!=NULL) *ptype=urllist.type;
  lcrzo_er(lcrzo_string_initm_text(urllist.purl, purl));
  if (pstartpos!=NULL) *pstartpos=urllist.startpos;
  if (pendpos!=NULL) *pendpos=urllist.endpos;
  if (prefresh!=NULL) *prefresh=urllist.refresh;

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_priv_http_urllist_add4(lcrzo_constdata data, 
				  lcrzo_int32 datasize, 
				  lcrzo_int32 startat, 
				  lcrzo_int32 *pendat,
				  lcrzo_bool *pfoundonevar,
				  lcrzo_string *pvar,
				  lcrzo_bool *pfoundoneval,
				  lcrzo_string *pval,
				  lcrzo_int32 *pvalstartpos,
				  lcrzo_int32 *pvalendpos);
int shared_priv_http_urllist_add4(lcrzo_constdata data, 
				  lcrzo_int32 datasize, 
				  lcrzo_int32 startat, 
				  lcrzo_int32 *pendat,
				  lcrzo_bool *pfoundonevar,
				  lcrzo_string *pvar,
				  lcrzo_bool *pfoundoneval,
				  lcrzo_string *pval,
				  lcrzo_int32 *pvalstartpos,
				  lcrzo_int32 *pvalendpos)
{ lcrzo_int32 i, lasti;
  lcrzo_uint8 c;
  lcrzo_bool endwithguil, endwithcote;

  /* first, skip leading spaces */
  i=startat;
  while(i<datasize)
  { c=data[i];
    if ( c==' ' || c=='\t' || c==0x0D || c==0x0A )
    { i++;
    }
    else
    { break;
    }
  }
  if (i==datasize)
  { if (pendat!=NULL) *pendat=datasize;
    if (pfoundonevar!=NULL) *pfoundonevar=LCRZO_FALSE;
    return(LCRZO_ERR_OK);
  }

  /* set the var */
  lcrzo_er(lcrzo_string_initm_text("", pvar));
  while(i<datasize)
  { c=data[i];
    if ( c==' ' || c=='\t' || c==0x0D || c==0x0A || c=='=' )
    { /* end of var */
      break;
    }
    lcrzo_er(lcrzo_string_appendm_char(c, 1, pvar));
    i++;
  }
  if (i==datasize)
  { if (pendat!=NULL) *pendat=datasize;
    if (pfoundonevar!=NULL) *pfoundonevar=LCRZO_TRUE;
    if (pfoundoneval!=NULL) *pfoundoneval=LCRZO_FALSE;
    return(LCRZO_ERR_OK);
  }

  /* skip spaces */
  lasti=i;
  while(i<datasize)
  { c=data[i];
    if ( ! ( c==' ' || c=='\t' || c==0x0D || c==0x0A ) )
    { break;
    }
    i++;
  }
  if ( i==datasize )
  { /* var does not have a value */
    if (pendat!=NULL) *pendat=datasize;
    if (pfoundonevar!=NULL) *pfoundonevar=LCRZO_TRUE;
    if (pfoundoneval!=NULL) *pfoundoneval=LCRZO_FALSE;
    return(LCRZO_ERR_OK);
  }
  if ( data[i]!='=' )
  { /* var does not have a value */
    if (pendat!=NULL) *pendat=lasti;
    if (pfoundonevar!=NULL) *pfoundonevar=LCRZO_TRUE;
    if (pfoundoneval!=NULL) *pfoundoneval=LCRZO_FALSE;
    return(LCRZO_ERR_OK);
  }
  else
  { /* skip equal */
    i++;
  }

  /* skip spaces */
  if (pvalstartpos!=NULL) *pvalstartpos=i;
  while(i<datasize)
  { c=data[i];
    if ( ! ( c==' ' || c=='\t' || c==0x0D || c==0x0A ) )
    { break;
    }
    i++;
  }
  if ( i==datasize )
  { /* var does not have a value */
    if (pendat!=NULL) *pendat=datasize;
    if (pfoundonevar!=NULL) *pfoundonevar=LCRZO_TRUE;
    if (pfoundoneval!=NULL) *pfoundoneval=LCRZO_FALSE;
    return(LCRZO_ERR_OK);
  }

  /* set the value */
  lcrzo_er(lcrzo_string_initm_text("", pval));
  endwithguil=LCRZO_FALSE;
  endwithcote=LCRZO_FALSE;
  if (i<datasize)
  { c=data[i];
    if ( c=='"' )
    { endwithguil=LCRZO_TRUE;
      i++;
    }
    if ( c=='\'' )
    { endwithcote=LCRZO_TRUE;
      i++;
    }
  }
  while(i<datasize)
  { c=data[i];
    if ( (endwithguil && c=='"') || (endwithcote && c=='\'') )
    { /* end of value */
      if (pendat!=NULL) *pendat=i+1;
      if (pvalendpos!=NULL) *pvalendpos=i;
      if (pfoundonevar!=NULL) *pfoundonevar=LCRZO_TRUE;
      if (pfoundoneval!=NULL) *pfoundoneval=LCRZO_TRUE;
      return(LCRZO_ERR_OK);
    }
    if ( ( ! (endwithguil||endwithcote) ) && 
	 (c==' ' || c=='\t' || c==0x0D || c==0x0A) )
    { /* end of value */
      if (pendat!=NULL) *pendat=i;
      if (pvalendpos!=NULL) *pvalendpos=i-1;
      if (pfoundonevar!=NULL) *pfoundonevar=LCRZO_TRUE;
      if (pfoundoneval!=NULL) *pfoundoneval=LCRZO_TRUE;
      return(LCRZO_ERR_OK);
    }
    lcrzo_er(lcrzo_string_appendm_char(c, 1, pval));
    i++;
  }
  if (i==datasize)
  { if (pendat!=NULL) *pendat=datasize;
    if (pfoundonevar!=NULL) *pfoundonevar=LCRZO_TRUE;
    if (pfoundoneval!=NULL) *pfoundoneval=LCRZO_TRUE;
    if (pvalendpos!=NULL) *pvalendpos=i-1;
    return(LCRZO_ERR_OK);
  }

  if (pendat!=NULL) *pendat=i;
  if (pfoundonevar!=NULL) *pfoundonevar=LCRZO_TRUE;
  if (pfoundoneval!=NULL) *pfoundoneval=LCRZO_TRUE;
  if (pvalendpos!=NULL) *pvalendpos=i-1;
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_priv_http_urllist_add3_meta(lcrzo_conststring val, 
				       lcrzo_string *pval,
				       lcrzo_uint32 *prefresh);
int shared_priv_http_urllist_add3_meta(lcrzo_conststring val, 
				       lcrzo_string *pval,
				       lcrzo_uint32 *prefresh)
{ lcrzo_string output, pcoutput;
  lcrzo_conststring pc;
  int ret;

  /* val should contain "1234;url=xxx" */
  pc=val;
  
  /* skip spaces */
  while ( *pc==' ' || *pc=='\t' ) pc++;

  /* search integer */
  lcrzo_er(lcrzo_string_initm_text(pc, &output));
  pcoutput=output;
  while ( *pcoutput>='0' && *pcoutput<='9' ) pcoutput++;
  *pcoutput='\0';
  ret=lcrzo_uint32_init_text(output, prefresh);
  if (ret!=LCRZO_ERR_OK)
  { lcrzo_er(lcrzo_string_free2(&output));
    return(LCRZO_ERR_OKNOTDECODED);
  }
  pc+=strlen(output);
  lcrzo_er(lcrzo_string_free2(&output));

  /* skip spaces */
  while ( *pc==' ' || *pc=='\t' ) pc++;

  /* search ";" */
  if ( *pc!=';' ) return(LCRZO_ERR_OKNOTDECODED);
  pc++;

  /* skip spaces */
  while ( *pc==' ' || *pc=='\t' ) pc++;

  /* search "url" */
  if ( *pc!='u' && *pc!='U' ) return(LCRZO_ERR_OKNOTDECODED);
  pc++;
  if ( *pc!='r' && *pc!='R' ) return(LCRZO_ERR_OKNOTDECODED);
  pc++;
  if ( *pc!='l' && *pc!='L' ) return(LCRZO_ERR_OKNOTDECODED);
  pc++;

  /* skip spaces */
  while ( *pc==' ' || *pc=='\t' ) pc++;

  /* search "=" */
  if ( *pc!='=' ) return(LCRZO_ERR_OKNOTDECODED);
  pc++;

  /* skip spaces */
  while ( *pc==' ' || *pc=='\t' ) pc++;

  /* prepare output */
  lcrzo_er(lcrzo_string_initm_text(pc, &output));
  pcoutput=output;
  while ( *pcoutput!=' ' && *pcoutput!='\t' && *pcoutput!='\0' ) pcoutput++;
  if (*pcoutput!='\0')
  { *pcoutput='\0';
    pcoutput++;
    /* ensure it end only with spaces */
    while ( *pcoutput==' ' || *pcoutput=='\t' ) pcoutput++;
    if (*pcoutput!='\0')
    { lcrzo_er(lcrzo_string_free2(&output));
      return(LCRZO_ERR_OKNOTDECODED);
    }
  }

  if (strlen(output)==0)
  { lcrzo_er(lcrzo_string_free2(&output));
    return(LCRZO_ERR_OKNOTDECODED);
  }

  if (pval==NULL) { lcrzo_er(lcrzo_string_free2(&output)); }
  else *pval=output;

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_priv_http_urllist_add3(lcrzo_int32 dataoffset, 
				  lcrzo_constdata data, 
				  lcrzo_int32 datasize, 
				  lcrzo_conststring wantedvar, 
				  lcrzo_uint32 linktype,
				  lcrzo_string *pbasehref,
				  lcrzo_list *purllist);
int shared_priv_http_urllist_add3(lcrzo_int32 dataoffset, 
				  lcrzo_constdata data, 
				  lcrzo_int32 datasize, 
				  lcrzo_conststring wantedvar, 
				  lcrzo_uint32 linktype,
				  lcrzo_string *pbasehref,
				  lcrzo_list *purllist)
{ lcrzo_int32 startat, valstartpos, valendpos;
  lcrzo_int32 valstartposbasehref;
  lcrzo_string var, val, tmpval;
  lcrzo_bool foundonevar, foundoneval;
  lcrzo_uint32 metacontentrefresh;
  int ret;

  /*lcrzo_er(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_DUMP));*/

  startat=0;
  while (startat<datasize)
  { valstartposbasehref=startat;
    metacontentrefresh=0;
    lcrzo_er(shared_priv_http_urllist_add4(data, datasize, startat, &startat,
					   &foundonevar, &var, 
					   &foundoneval, &val, 
					   &valstartpos, &valendpos));
    if ( !foundonevar ) break;
    if ( !foundoneval )
    { lcrzo_er(lcrzo_string_free2(&var));
      continue;
    }
    /*printf("%s=%sZZ\n", var, val);*/
    if ( shared_misc_strcasecmp(wantedvar, var) )
    { lcrzo_er(lcrzo_string_free2(&var));
      lcrzo_er(lcrzo_string_free2(&val));
      continue;
    }

    /* strip spaces, and remove HTML chars */
    lcrzo_er(lcrzo_string_initm_coretext(val, &tmpval));
    lcrzo_er(lcrzo_string_free2(&val));
    lcrzo_er(shared_http_string_initm_he(tmpval, &val));
    lcrzo_er(lcrzo_string_free2(&tmpval));

    /* prepare for adding in the list */
    if (linktype==SHARED_HTTP_LINKTYPE_BASE_HREF)
    { /* store value of base href */
      lcrzo_er(lcrzo_string_free2(pbasehref));
      lcrzo_er(lcrzo_string_initm_text(val, pbasehref));
      valstartpos=valstartposbasehref; /* we will totally suppress this tag */
    }
    else if (linktype==SHARED_HTTP_LINKTYPE_META_CONTENT)
    { /* stored value is special */
      ret=shared_priv_http_urllist_add3_meta(val, &tmpval,
					     &metacontentrefresh);
      if (ret!=LCRZO_ERR_OK)
      { /* do not store it */
	lcrzo_er(lcrzo_string_free2(&var));
	lcrzo_er(lcrzo_string_free2(&val));
	continue;
      }
      lcrzo_er(lcrzo_string_free2(&val));
      lcrzo_er(lcrzo_string_initm_text(tmpval, &val));
      lcrzo_er(lcrzo_string_free2(&tmpval));
    }
    else
    { /* eventually add base href value */
      if (strlen(*pbasehref))
      { ret=shared_http_url_init_dpurl(*pbasehref, val, &tmpval);
        if (ret==LCRZO_ERR_OK)
	{ lcrzo_er(lcrzo_string_free2(&val));
	  lcrzo_er(lcrzo_string_initm_text(tmpval, &val));
	  lcrzo_er(lcrzo_string_free2(&tmpval));
	}
      }
    }
    /* add in the list */
    lcrzo_er(shared_http_urllist_add_pos(purllist, -1, linktype, val,
					 dataoffset+valstartpos,
					 dataoffset+valendpos,
					 metacontentrefresh));
    lcrzo_er(lcrzo_string_free2(&var));
    lcrzo_er(lcrzo_string_free2(&val));
  }

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_priv_http_urllist_add2(lcrzo_int32 dataoffset, 
				  lcrzo_constdata data, 
				  lcrzo_int32 datasize, 
				  lcrzo_string *pbasehref,
				  lcrzo_list *purllist);
int shared_priv_http_urllist_add2(lcrzo_int32 dataoffset, 
				  lcrzo_constdata data, 
				  lcrzo_int32 datasize, 
				  lcrzo_string *pbasehref,
				  lcrzo_list *purllist)
{ lcrzo_string tag, typetag, typepara;
  lcrzo_int32 i, spacepos;
  lcrzo_uint32 type;
  lcrzo_uint8 c;

  /* find the tag */
  spacepos=-1; /* for the compiler warning */
  lcrzo_er(lcrzo_string_initm_text("", &tag));
  for ( i=0 ; i<datasize ; i++ )
  { c=data[i];
    if ( c==' ' || c=='\t' || c==0x0D || c==0x0A )
    { spacepos=i;
      break;
    }
    lcrzo_er(lcrzo_string_appendm_char(c, 1, &tag));
  }

  /* now, try to find parameters corresponding to tag */
  for ( type=SHARED_HTTP_LINKTYPE_BEGIN ;
	type<=SHARED_HTTP_LINKTYPE_END ;
	type++ )
  { lcrzo_er(shared_http_linktype_get_tag(type, &typetag));
    if (!shared_misc_strcasecmp(tag, typetag))
    { lcrzo_er(shared_http_linktype_get_para(type, &typepara));
      lcrzo_er(shared_priv_http_urllist_add3(dataoffset+spacepos,
					     data+spacepos, datasize-spacepos,
					     typepara, type,
					     pbasehref, purllist));
      lcrzo_er(lcrzo_string_free2(&typepara));
    }
    lcrzo_er(lcrzo_string_free2(&typetag));
  }

  /* end */
  lcrzo_er(lcrzo_string_free2(&tag));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_priv_http_urllist_add(lcrzo_int32 dataoffset, 
				 lcrzo_constdata data, 
				 lcrzo_int32 datasize,
				 lcrzo_string *pbasehref,
				 lcrzo_list *purllist,
				 lcrzo_data *pleftdata, 
				 lcrzo_int32 *pleftdatasize);
int shared_priv_http_urllist_add(lcrzo_int32 dataoffset, 
				 lcrzo_constdata data, 
				 lcrzo_int32 datasize, 
				 lcrzo_string *pbasehref,
				 lcrzo_list *purllist,
				 lcrzo_data *pleftdata, 
				 lcrzo_int32 *pleftdatasize)
{ lcrzo_bool foundlt, foundcomment;
  lcrzo_int32 ltpos, i;

  /* separate data in chunks <xxx> */
  foundlt=LCRZO_FALSE;
  foundcomment=LCRZO_FALSE;
  i=0;
  ltpos=-1; /* for the compiler warning */
  while ( i<datasize )
  { if (!foundlt)
    { if (data[i]=='<')
      { foundlt=LCRZO_TRUE;
        ltpos=i;
        foundcomment=LCRZO_FALSE;
	if ( i+3<datasize )
	{ if ( data[i+1]=='!' && data[i+2]=='-' && data[i+3]=='-' )
	    foundcomment=LCRZO_TRUE;
	}
      }
    }
    else
    { if (data[i]=='>')
      { if (foundcomment)
        { if ( i-2>ltpos+3 )
	  { if ( data[i-2]=='-' && data[i-1]=='-' )
	    { /* we found the end of comment */
	      foundlt=LCRZO_FALSE;
	    }
	  }
        }
        else
        { /* so we have the chunk between data[ltpos] and data[i] */
	  lcrzo_er(shared_priv_http_urllist_add2(dataoffset+ltpos+1,
						 data+ltpos+1, i-ltpos-1,
						 pbasehref, purllist));
	  foundlt=LCRZO_FALSE;
	}
      }
    }
    i++;
  }

  /* check if we've left data */
  if (foundlt)
  { lcrzo_er(lcrzo_data_initm_data(data+ltpos, datasize-ltpos,
				   pleftdata, pleftdatasize));
  }
  else
  { *pleftdatasize=0;
    lcrzo_er(lcrzo_data_alloc(*pleftdatasize, pleftdata));
  }
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_urllist_init_file(lcrzo_conststring filename,
				  lcrzo_list *purllist)
{ lcrzo_file file;
  lcrzo_data data, filedata, leftdata;
  lcrzo_int32 datasize, filedatasize, leftdatasize, dataoffset;
  lcrzo_string basehref;
  int ret;

  lcrzo_er(lcrzo_file_open_read(filename, &file));
  lcrzo_er(shared_http_urllist_init(purllist));
  lcrzo_er(lcrzo_string_initm_text("", &basehref));

  leftdatasize=0;
  dataoffset=0;
  lcrzo_er(lcrzo_data_alloc(leftdatasize, &leftdata));
  ret=LCRZO_ERR_OK;
  while (LCRZO_TRUE)
  { ret=lcrzo_file_readm(&file, LCRZO_WAIT_INFINITE, &filedata, &filedatasize);
    if (ret!=LCRZO_ERR_OK)
    { /* suppose there is not left url */
      lcrzo_er(lcrzo_data_free2(&leftdata));
      if (ret==LCRZO_ERR_OKDATAEND) ret=LCRZO_ERR_OK;
      break;
    }
    lcrzo_er(lcrzo_data_initm_data(leftdata, leftdatasize, 
				   &data, &datasize));
    lcrzo_er(lcrzo_data_free2(&leftdata));
    lcrzo_er(lcrzo_data_appendm_data(filedata, filedatasize, 
				     datasize, &data, &datasize));
    lcrzo_er(lcrzo_data_free2(&filedata));
    ret=shared_priv_http_urllist_add(dataoffset, data, datasize, 
				     &basehref, purllist,
				     &leftdata, &leftdatasize);
    dataoffset+=datasize-leftdatasize;
    lcrzo_er(lcrzo_data_free2(&data));
    if (ret!=LCRZO_ERR_OK) break;
  }
  lcrzo_er(lcrzo_string_free2(&basehref));
  lcrzo_er(lcrzo_file_close(&file));
  return(ret);
}

/*-------------------------------------------------------------*/
int shared_http_pt_initm_data(lcrzo_constdata data,
			      lcrzo_int32 datasize,
			      lcrzo_uint16 pttype,
			      lcrzo_string *ptextwithpercent)
{ lcrzo_string textwithpercent;
  lcrzo_uint8 c;
  int i;

  lcrzo_er(lcrzo_string_initm_text("", &textwithpercent));

  switch(pttype)
  { case SHARED_HTTP_PT_TYPE_ENCODENONE:
      for ( i=0 ; i<datasize ; i++ ) 
      { c=data[i];
        lcrzo_er(lcrzo_string_appendm_char(c, 1, &textwithpercent));
      }
      break;
    case SHARED_HTTP_PT_TYPE_ENCODEALL:
      for ( i=0 ; i<datasize ; i++ ) 
      { c=data[i];
        lcrzo_er(lcrzo_string_appendm_char('%', 1, &textwithpercent));
        lcrzo_er(lcrzo_string_appendm_uint8(c, "%02X", &textwithpercent));
      }
      break;
    case SHARED_HTTP_PT_TYPE_PATH:
      for ( i=0 ; i<datasize ; i++ ) 
      { c=data[i];
        if ( ( c>='0' && c<='9' ) || ( c>='a' && c<='z' ) || 
	     ( c>='A' && c<='Z' ) || c=='-' || c=='_' || c=='.' ||
	     c=='/' )
	{ lcrzo_er(lcrzo_string_appendm_char(c, 1, &textwithpercent));
	}
	else
	{ lcrzo_er(lcrzo_string_appendm_char('%', 1, &textwithpercent));
          lcrzo_er(lcrzo_string_appendm_uint8(c, "%02X", &textwithpercent));
	}
      }
      break;
    case SHARED_HTTP_PT_TYPE_VARVAL:
      for ( i=0 ; i<datasize ; i++ ) 
      { c=data[i];
        if ( ( c>='0' && c<='9' ) || ( c>='a' && c<='z' ) || 
	     ( c>='A' && c<='Z' ) || c=='-' || c=='_' || c=='.' ||
	     c=='=' || c=='&' )
	{ lcrzo_er(lcrzo_string_appendm_char(c, 1, &textwithpercent));
	}
	else
	{ lcrzo_er(lcrzo_string_appendm_char('%', 1, &textwithpercent));
          lcrzo_er(lcrzo_string_appendm_uint8(c, "%02X", &textwithpercent));
	}
      }
      break;
    case SHARED_HTTP_PT_TYPE_ANCHOR:
      for ( i=0 ; i<datasize ; i++ ) 
      { c=data[i];
        if ( ( c>='0' && c<='9' ) || ( c>='a' && c<='z' ) || 
	     ( c>='A' && c<='Z' ) || c=='-' || c=='_' || c=='.' )
	{ lcrzo_er(lcrzo_string_appendm_char(c, 1, &textwithpercent));
	}
	else
	{ lcrzo_er(lcrzo_string_appendm_char('%', 1, &textwithpercent));
        lcrzo_er(lcrzo_string_appendm_uint8(c, "%02X", &textwithpercent));
	}
      }
      break;
    case SHARED_HTTP_PT_TYPE_LOGPWD:
      for ( i=0 ; i<datasize ; i++ ) 
      { c=data[i];
        if ( ( c>='0' && c<='9' ) || ( c>='a' && c<='z' ) || 
	     ( c>='A' && c<='Z' ) )
	{ lcrzo_er(lcrzo_string_appendm_char(c, 1, &textwithpercent));
	}
	else
	{ lcrzo_er(lcrzo_string_appendm_char('%', 1, &textwithpercent));
        lcrzo_er(lcrzo_string_appendm_uint8(c, "%02X", &textwithpercent));
	}
      }
      break;
    default:
      lcrzo_er(lcrzo_string_free2(&textwithpercent));
      return(LCRZOEX_ERR_PARA_UNKNOWNTYPE);
  }
  
  if (ptextwithpercent==NULL)
  { lcrzo_er(lcrzo_string_free2(&textwithpercent));
  }
  else
  { *ptextwithpercent=textwithpercent;
  }

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_pt_initm_string(lcrzo_conststring str,
				lcrzo_uint16 pttype,
				lcrzo_string *ptextwithpercent)
{ return(shared_http_pt_initm_data((lcrzo_constdata)str, strlen(str), 
				   pttype, ptextwithpercent));
}

/*-------------------------------------------------------------*/
int shared_http_data_initm_pt(lcrzo_conststring textwithpercent,
			      lcrzo_data *pdata,
			      lcrzo_int32 *pdatasize)
{ lcrzo_int32 datasize, textsize;
  char c, c1, c2;
  int i, number;

  lcrzo_epr(lcrzo_data_initm_text("", pdata, &datasize));
  textsize=strlen(textwithpercent);
  for ( i=0 ; i<textsize ; i++ ) 
  { c=textwithpercent[i];
    if ( c != '%' )
    { if ( c != '+' )
      { lcrzo_epr(lcrzo_data_appendm_char(c, 1, datasize, pdata, &datasize));
      }
      else
      { lcrzo_epr(lcrzo_data_appendm_char(' ', 1, datasize, pdata, &datasize));
      }
    }
    else
    { if ( i+2 >= textsize ) return(LCRZOEX_ERR_HTTP_MISSCHARAFTERPERCENT);
      c1=textwithpercent[++i];
      c2=textwithpercent[++i];
      if ( c1>='0' && c1<='9' )
      { number=c1-'0';
      }
      else if ( c1>='a' && c1<='f') 
      { number=10+c1-'a';
      }
      else if ( c1>='A' && c1<='F' )
      { number=10+c1-'A';
      }
      else
      { return(LCRZOEX_ERR_HTTP_BADCHARAFTERPERCENT);
      }
      number=number<<4;
      if ( c2>='0' && c2<='9' )
      { number+=c2-'0';
      }
      else if ( c2>='a' && c2<='f') 
      { number+=10+c2-'a';
      }
      else if ( c2>='A' && c2<='F' )
      { number+=10+c2-'A';
      }
      else
      { return(LCRZOEX_ERR_HTTP_BADCHARAFTERPERCENT);
      }
      lcrzo_epr(lcrzo_data_appendm_char((char)number, 1, datasize, pdata,&datasize));
    }      
  }

  /* set the output values */
  if (pdatasize!=NULL) *pdatasize=datasize;

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_string_initm_pt(lcrzo_conststring textwithpercent,
				lcrzo_string *pstr)
{ lcrzo_int32 size;
  lcrzo_er(shared_http_data_initm_pt(textwithpercent,
				     (lcrzo_data*)pstr, &size));
  if (pstr!=NULL) (*pstr)[size]='\0';
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_pt_initm_pt(lcrzo_conststring textwithpercent,
			    lcrzo_uint16 pttype,
			    lcrzo_string *ptextwithpercent)
{ lcrzo_data data;
  lcrzo_int32 datasize;

  lcrzo_er(shared_http_data_initm_pt(textwithpercent, &data, &datasize));

  lcrzo_er(shared_http_pt_initm_data(data, datasize, pttype,
				     ptextwithpercent));
  lcrzo_er(lcrzo_data_free2(&data));
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_string_initm_he(lcrzo_conststring texthtmlencoded,
				lcrzo_string *pstr)
{ lcrzo_string str;
  lcrzo_int32 strlentexthtmlencoded, i, j;
  lcrzo_uint8 a1, a2;
  char c, c1, c2, c3, c4, c5;

  strlentexthtmlencoded=strlen(texthtmlencoded);
  lcrzo_er(lcrzo_string_alloc(strlentexthtmlencoded, &str));
  for ( i=0,j=0 ; i<strlentexthtmlencoded ; i++ )
  { c=texthtmlencoded[i];
    if (c!='&')
    { str[j++]=c;
    }
    else
    { c1='\0'; c2='\0'; c3='\0'; c4='\0'; c5='\0';
      if ( i+1<strlentexthtmlencoded ) c1=texthtmlencoded[i+1];
      if ( i+2<strlentexthtmlencoded ) c2=texthtmlencoded[i+2];
      if ( i+3<strlentexthtmlencoded ) c3=texthtmlencoded[i+3];
      if ( i+4<strlentexthtmlencoded ) c4=texthtmlencoded[i+4];
      if ( i+5<strlentexthtmlencoded ) c5=texthtmlencoded[i+5];
      /* check known types */
      if ( c1=='q' && c2=='u' && c3=='o' && c4=='t' && c5==';' )
      { str[j++]='"';
 	i+=5;
      }
      else if ( c1=='a' && c2=='m' && c3=='p' && c4==';' )
      { str[j++]='&';
 	i+=4;
      }
      else if ( c1=='l' && c2=='t' && c3==';' )
      { str[j++]='<';
 	i+=3;
      }
      else if ( c1=='g' && c2=='t' && c3==';' )
      { str[j++]='>';
 	i+=3;
      }
      else if ( c1=='#' && 
		( (c2>='0'&&c2<='9') ||
		  (c2>='a'&&c2<='f') ||
		  (c2>='A'&&c2<='F') ) &&
		( (c3>='0'&&c3<='9') ||
		  (c3>='a'&&c3<='f') ||
		  (c3>='A'&&c3<='F') ) &&
		c4==';' )
      { a1=0;
        if (c2>='0'&&c2<='9') a1=(lcrzo_uint8)(c2-'0');
        else if (c2>='a'&&c2<='f') a1=(lcrzo_uint8)(10+c2-'a');
        else if (c2>='A'&&c2<='F') a1=(lcrzo_uint8)(10+c2-'A');
	a2=0;
        if (c3>='0'&&c3<='9') a2=(lcrzo_uint8)(c3-'0');
        else if (c3>='a'&&c3<='f') a2=(lcrzo_uint8)(10+c3-'a');
        else if (c3>='A'&&c3<='F') a2=(lcrzo_uint8)(10+c3-'A');
	str[j++]=(lcrzo_int8)((a1<<4)|a2);
 	i+=4;
      }
      else
      { str[j++]=c;
      }
    }
  }
  str[j]='\0';

  if (pstr!=NULL) *pstr=str;
  else { lcrzo_er(lcrzo_string_free2(&str)); }

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_string_initm_he_test(void);
int shared_http_string_initm_he_test(void)
{ lcrzo_string val;

  lcrzo_epr(shared_http_string_initm_he("a=b&c=d", &val));
  printf("AA%sZZ\n", val);
  if (strcmp(val, "a=b&c=d")) return(12345);
  lcrzo_er(lcrzo_string_free2(&val));

  lcrzo_epr(shared_http_string_initm_he("a=b&amp;c=d", &val));
  printf("AA%sZZ\n", val);
  if (strcmp(val, "a=b&c=d")) return(12345);
  lcrzo_er(lcrzo_string_free2(&val));

  lcrzo_epr(shared_http_string_initm_he("a=b&quot;c=d", &val));
  printf("AA%sZZ\n", val);
  if (strcmp(val, "a=b\"c=d")) return(12345);
  lcrzo_er(lcrzo_string_free2(&val));

  lcrzo_epr(shared_http_string_initm_he("a=b&lt;c=d", &val));
  printf("AA%sZZ\n", val);
  if (strcmp(val, "a=b<c=d")) return(12345);
  lcrzo_er(lcrzo_string_free2(&val));

  lcrzo_epr(shared_http_string_initm_he("a=b&gt;c=d", &val));
  printf("AA%sZZ\n", val);
  if (strcmp(val, "a=b>c=d")) return(12345);
  lcrzo_er(lcrzo_string_free2(&val));

  lcrzo_epr(shared_http_string_initm_he("a=b&#20;c=d", &val));
  printf("AA%sZZ\n", val);
  if (strcmp(val, "a=b c=d")) return(12345);
  lcrzo_er(lcrzo_string_free2(&val));

  lcrzo_epr(shared_http_string_initm_he("a=b&#2E;c=d", &val));
  printf("AA%sZZ\n", val);
  if (strcmp(val, "a=b.c=d")) return(12345);
  lcrzo_er(lcrzo_string_free2(&val));

  lcrzo_epr(shared_http_string_initm_he("a=b&#2e;c=d", &val));
  printf("AA%sZZ\n", val);
  if (strcmp(val, "a=b.c=d")) return(12345);
  lcrzo_er(lcrzo_string_free2(&val));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_he_initm_string(lcrzo_conststring str,
				lcrzo_string *ptexthtmlencoded)
{ lcrzo_string texthtmlencoded;
  lcrzo_int32 strlenstr, i;
  char c;

  lcrzo_er(lcrzo_string_initm_text("", &texthtmlencoded));
  strlenstr=strlen(str);
  for ( i=0 ; i<strlenstr ; i++ )
  { c=str[i];
    if ( ( c>='0' && c<='9' ) || ( c>='a' && c<='z' ) || 
	 ( c>='A' && c<='Z' ) || 
	 c=='-' || c=='_' || c=='.' ||
	 c=='/' || c=='?' || c=='=' || c=='&' || c=='#' )
    { lcrzo_er(lcrzo_string_appendm_char(c, 1, &texthtmlencoded));
    }
    else
    { lcrzo_er(lcrzo_string_appendm_text("&#", &texthtmlencoded));
      lcrzo_er(lcrzo_string_appendm_uint8(c, "%02X", &texthtmlencoded));
      lcrzo_er(lcrzo_string_appendm_text(";", &texthtmlencoded));
    }
  }

  lcrzo_er(lcrzo_string_initm_text(texthtmlencoded, ptexthtmlencoded));
  lcrzo_er(lcrzo_string_free2(&texthtmlencoded));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_he_initm_string_test(void);
int shared_http_he_initm_string_test(void)
{ lcrzo_string val;

  lcrzo_epr(shared_http_he_initm_string("a=b&c=d", &val));
  printf("AA%sZZ\n", val);
  if (strcmp(val, "a=b&c=d")) return(12345);
  lcrzo_er(lcrzo_string_free2(&val));

  lcrzo_epr(shared_http_he_initm_string("a=b c=d", &val));
  printf("AA%sZZ\n", val);
  if (strcmp(val, "a=b&#20;c=d")) return(12345);
  lcrzo_er(lcrzo_string_free2(&val));

  lcrzo_epr(shared_http_he_initm_string("a=b'c=d", &val));
  printf("AA%sZZ\n", val);
  if (strcmp(val, "a=b&#27;c=d")) return(12345);
  lcrzo_er(lcrzo_string_free2(&val));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_url_init(lcrzo_conststring proto,
			 lcrzo_conststring user,
			 lcrzo_conststring password,
			 lcrzo_conststring server,
			 lcrzo_uint16 port,
			 lcrzo_conststring path,
			 lcrzo_conststring ptvarval,
			 lcrzo_conststring anchor,
			 lcrzo_string *purl)
{ lcrzo_string url, tmp;
  lcrzo_bool ishttp, isftp;

  if ( user==NULL && password!=NULL )
    return(LCRZOEX_ERR_HTTP_LOGNULLPWDNOTNULL);

  ishttp=LCRZO_FALSE;
  isftp=LCRZO_FALSE;
  if ( proto==NULL || strlen(proto)==0 )
  { lcrzo_er(lcrzo_string_initm_text("http", &url));
    ishttp=LCRZO_TRUE;
  }
  else
  { lcrzo_er(lcrzo_string_initm_text(proto, &url));
    if ( !shared_misc_strcasecmp(proto, "http") ) ishttp=LCRZO_TRUE;
    if ( !shared_misc_strcasecmp(proto, "ftp") ) isftp=LCRZO_TRUE;
  }
  lcrzo_er(lcrzo_string_appendm_text("://", &url));

  if ( user!=NULL )
  { if ( strlen(user)!=0 )
    { lcrzo_er(shared_http_pt_initm_string(user, SHARED_HTTP_PT_TYPE_LOGPWD,
					   &tmp));
      lcrzo_er(lcrzo_string_appendm_text(tmp, &url));
      lcrzo_er(lcrzo_string_free2(&tmp));
    }
    if ( password!=NULL )
    { lcrzo_er(lcrzo_string_appendm_text(":", &url));
      if ( strlen(password)!=0 )
      { lcrzo_er(shared_http_pt_initm_string(password, 
					     SHARED_HTTP_PT_TYPE_LOGPWD,
					     &tmp));
        lcrzo_er(lcrzo_string_appendm_text(tmp, &url));
	lcrzo_er(lcrzo_string_free2(&tmp));
      }
    }
    lcrzo_er(lcrzo_string_appendm_text("@", &url));
  }

  if ( server==NULL || strlen(server)==0 )
  { lcrzo_er(lcrzo_string_appendm_text("localhost", &url));
  }
  else
  { lcrzo_er(lcrzo_string_appendm_text(server, &url));
  }
 
  if ( !( ishttp && port==80 ) && !( isftp && port==21 ) )
  { lcrzo_er(lcrzo_string_appendm_text(":", &url));
    lcrzo_er(lcrzo_string_appendm_uint16(port, "%u", &url));
  }

  if ( path==NULL || strlen(path)==0 )
  { lcrzo_er(shared_http_dp_init("/", ptvarval, anchor, &tmp));
    lcrzo_er(lcrzo_string_appendm_text(tmp, &url));
    lcrzo_er(lcrzo_string_free2(&tmp));
  }
  else
  { if ( path[0]=='/' )
    { lcrzo_er(shared_http_dp_init(path, ptvarval, anchor, &tmp));
      lcrzo_er(lcrzo_string_appendm_text(tmp, &url));
      lcrzo_er(lcrzo_string_free2(&tmp));
    }
    else
    { lcrzo_er(shared_http_dp_init(path, ptvarval, anchor, &tmp));
      lcrzo_er(lcrzo_string_appendm_text("/", &url));
      lcrzo_er(lcrzo_string_appendm_text(tmp, &url));
      lcrzo_er(lcrzo_string_free2(&tmp));
    }
  }

  if (purl==NULL) { lcrzo_er(lcrzo_string_free2(&url)); }
  else *purl=url;

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_dp_init(lcrzo_conststring path,
			lcrzo_conststring ptvarval,
			lcrzo_conststring anchor,
			lcrzo_string *pdp)
{ lcrzo_string dp, tmp;

  /* check path */
  if ( path==NULL ) return(LCRZO_ERR_PANULLPTR);
  if ( strlen(path)==0 ) return(LCRZO_ERR_PATOOLOW);

  lcrzo_er(shared_http_pt_initm_string(path, SHARED_HTTP_PT_TYPE_PATH, &dp));

  if (ptvarval!=NULL)
  { if (strlen(ptvarval)!=0)
    { lcrzo_er(lcrzo_string_appendm_text("?", &dp));
      lcrzo_er(shared_http_pt_initm_pt(ptvarval, SHARED_HTTP_PT_TYPE_VARVAL, 
				       &tmp));
      lcrzo_er(lcrzo_string_appendm_text(tmp, &dp));
      lcrzo_er(lcrzo_string_free2(&tmp));
    }
  }

  if (anchor!=NULL)
  { if (strlen(anchor)!=0)
    { lcrzo_er(lcrzo_string_appendm_text("#", &dp));
      lcrzo_er(shared_http_pt_initm_string(anchor, SHARED_HTTP_PT_TYPE_ANCHOR, 
					   &tmp));
      lcrzo_er(lcrzo_string_appendm_text(tmp, &dp));
      lcrzo_er(lcrzo_string_free2(&tmp));
    }
  }

  if (pdp==NULL) { lcrzo_er(lcrzo_string_free2(&dp)); }
  else *pdp=dp;

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_varval_init(lcrzo_conststring var,
                            lcrzo_constdata val,
			    lcrzo_int32 valsize,
			    lcrzo_string *pptvarval)
{ lcrzo_string ptvarval, tmp;

  /* check var */
  if ( var==NULL ) return(LCRZO_ERR_PANULLPTR);
  if ( strlen(var)==0 ) return(LCRZO_ERR_PATOOLOW);
  
  lcrzo_er(lcrzo_string_initm_text(var, &ptvarval));
  lcrzo_er(lcrzo_string_appendm_text("=", &ptvarval));
  lcrzo_er(shared_http_pt_initm_data(val, valsize, SHARED_HTTP_PT_TYPE_VARVAL,
				     &tmp));
  lcrzo_er(lcrzo_string_appendm_text(tmp, &ptvarval));
  lcrzo_er(lcrzo_string_free2(&tmp));

  if (pptvarval==NULL) { lcrzo_er(lcrzo_string_free2(&ptvarval)); }
  else *pptvarval=ptvarval;

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_dpurl_isurl(lcrzo_conststring dpurl,
			    lcrzo_bool *pyes)
{ lcrzo_int32 strlendpurl, i;
  char c;

  /* search [A-Za-z0-9+.-]+: */
  strlendpurl=strlen(dpurl);
  if ( strlendpurl<2 )
  { if (pyes!=NULL) *pyes=LCRZO_FALSE;
    return (LCRZO_ERR_OK);
  }

  for ( i=0 ; i<strlendpurl ; i++ )
  { c=dpurl[i];
    if ( (c>='A'&&c<='Z') || (c>='a'&&c<='z') || (c>='0'&&c<='9') || 
	 c=='+' || c=='.' || c=='-' )
    { /* ok */
    }
    else if ( c==':' )
    { if (pyes!=NULL) *pyes=LCRZO_TRUE;
      return (LCRZO_ERR_OK);
    }
    else
    { if (pyes!=NULL) *pyes=LCRZO_FALSE;
      return (LCRZO_ERR_OK);
    }
  }
  if (pyes!=NULL) *pyes=LCRZO_FALSE;
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_dpurl_isurl_http(lcrzo_conststring dpurl,
				 lcrzo_bool *pyes)
{ lcrzo_bool yes;

  yes=LCRZO_TRUE;
  if (shared_misc_strncasecmp(dpurl, "http://", 7))
  { yes=LCRZO_FALSE;
  }

  if (pyes!=NULL) *pyes=yes;
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_dpurl_isurl_ftp(lcrzo_conststring dpurl,
				lcrzo_bool *pyes)
{ lcrzo_bool yes;

  yes=LCRZO_TRUE;
  if (shared_misc_strncasecmp(dpurl, "ftp://", 6))
  { yes=LCRZO_FALSE;
  }

  if (pyes!=NULL) *pyes=yes;
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_dpurl_isdp(lcrzo_conststring dpurl,
			   lcrzo_bool *pyes)
{ lcrzo_bool yes;

  lcrzo_er(shared_http_dpurl_isurl(dpurl, &yes));
  if (pyes!=NULL)
  { if (yes) *pyes=LCRZO_FALSE;
    else *pyes=LCRZO_TRUE;
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_url_decode(lcrzo_conststring url,
			   lcrzo_string *pproto,
			   lcrzo_string *puser,
			   lcrzo_string *ppassword,
			   lcrzo_string *pserver,
			   lcrzo_uint16 *pport,
			   lcrzo_string *ppath,
			   lcrzo_string *pptvarval,
			   lcrzo_string *panchor)
{ lcrzo_conststring pcupspstart, pcupspend;
  lcrzo_string upsp, userpass, serverport, fullpath, portstr, path, tmpstr;
  lcrzo_bool ishttp, isftp;
  lcrzo_int32 pos;
  int ret;

  /* check if can decode */
  lcrzo_er(shared_http_dpurl_isurl_http(url, &ishttp));
  lcrzo_er(shared_http_dpurl_isurl_ftp(url, &isftp));
  if ( !ishttp && !isftp )
  { return(LCRZO_ERR_OKNOTDECODED);
  }

  /* set proto */
  if (ishttp)
  { lcrzo_er(lcrzo_string_initm_text("http", pproto));
    pcupspstart=url+7;
  }
  else
  { lcrzo_er(lcrzo_string_initm_text("ftp", pproto)); 
    pcupspstart=url+6;
  }
  
  /* if there is nothing after "://", error */
  if ( *pcupspstart=='\0' )
  { if (pproto!=NULL) { lcrzo_er(lcrzo_string_free2(pproto)); }
    return(LCRZO_ERR_OKNOTDECODED);
  }

  /* search pcupspend */
  pcupspend=strchr(pcupspstart, '/');
  if (pcupspend==NULL)
  { lcrzo_er(lcrzo_string_initm_text(pcupspstart, &upsp));
    lcrzo_er(lcrzo_string_initm_text("/", &fullpath));
  }
  else
  { lcrzo_er(shared_misc_string_initm_2ptrs(pcupspstart, 
					    pcupspend-1, 
					    &upsp));
    /* note : rfc 1738 says ftp path does not count '/' :
         ftp://s/%2Fetc/ means directory /etc
         ftp://s/etc/ means directory ./etc
       However, Netscape does not respect that, so I do not bother.*/    
    lcrzo_er(lcrzo_string_initm_text(pcupspend, &fullpath));
  }

  /* decode upsp (contains login:pass@server:port) */
  /* search "@" */
  ret=lcrzo_string_search_char(upsp, LCRZO_FALSE, +1,-1, '@', NULL,&pos);
  if (ret==LCRZO_ERR_OK)
  { lcrzo_er(lcrzo_string_initm_range(upsp, +1, pos, &userpass));
    lcrzo_er(lcrzo_string_initm_range(upsp, pos+1, -1, &serverport));
  }
  else
  { userpass=NULL;
    if (puser!=NULL) *puser=NULL;
    if (ppassword!=NULL) *ppassword=NULL;
    lcrzo_er(lcrzo_string_initm_text(upsp, &serverport));
  }
  lcrzo_er(lcrzo_string_free2(&upsp));

  /* decode login:pass */
  if ( userpass!=NULL )
  { /* search ":" */
    ret=lcrzo_string_search_char(userpass, LCRZO_FALSE, +1,-1, ':', NULL,&pos);
    if (ret==LCRZO_ERR_OK)
    { lcrzo_er(lcrzo_string_initm_range(userpass, +1, pos, &tmpstr));
      lcrzo_er(shared_http_string_initm_pt(tmpstr, puser));
      lcrzo_er(lcrzo_string_free2(&tmpstr));
      lcrzo_er(lcrzo_string_initm_range(userpass, pos+1, -1, &tmpstr));
      lcrzo_er(shared_http_string_initm_pt(tmpstr, ppassword));
      lcrzo_er(lcrzo_string_free2(&tmpstr));
    }
    else
    { lcrzo_er(lcrzo_string_initm_text(userpass, &tmpstr));
      lcrzo_er(shared_http_string_initm_pt(tmpstr, puser));
      lcrzo_er(lcrzo_string_free2(&tmpstr));
      if (ppassword!=NULL) *ppassword=NULL;
    }
    lcrzo_er(lcrzo_string_free2(&userpass));
  }

  /* search ":" */
  ret=lcrzo_string_search_char(serverport, LCRZO_FALSE, +1,-1, ':', NULL,&pos);
  if (ret==LCRZO_ERR_OK)
  { lcrzo_er(lcrzo_string_initm_range(serverport, +1, pos, pserver));
    lcrzo_er(lcrzo_string_initm_range(serverport, pos+1, -1, &portstr));
    ret=lcrzo_uint16_init_text(portstr, pport);
    lcrzo_er(lcrzo_string_free2(&portstr));
    if (ret!=LCRZO_ERR_OK)
    { if (pport!=NULL) *pport=0;
    }
  }
  else
  { lcrzo_er(lcrzo_string_initm_text(serverport, pserver));
    /* try to find the port to use */
    if ( ishttp )
    { if (pport!=NULL) *pport=80;
    }
    else if ( isftp )
    { if (pport!=NULL) *pport=21;
    }
    else return(LCRZO_ERR_IEINTERNALERROR);
  }
  lcrzo_er(lcrzo_string_free2(&serverport));

  /* decode fullpath */
  lcrzo_er(shared_http_dp_decode(fullpath, &path, pptvarval, panchor));
  lcrzo_er(lcrzo_string_free2(&fullpath));

  /* suppress "./" in front of FTP */
  if ( isftp && !strncmp(path, "./", 2) )
  { lcrzo_er(lcrzo_string_initm_text(path+2, ppath));
  }
  else
  { lcrzo_er(lcrzo_string_initm_text(path, ppath));
  }
  lcrzo_er(lcrzo_string_free2(&path));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_dp_decode(lcrzo_conststring dp,
			  lcrzo_string *ppath,
			  lcrzo_string *pptvarval,
			  lcrzo_string *panchor)
{ lcrzo_string ptvarvalanchor, path, ptvarval, anchor, path2;
  lcrzo_int32 pos; 
  lcrzo_bool isdp;
  int ret;

  /* check if can decode */
  lcrzo_er(shared_http_dpurl_isdp(dp, &isdp));
  if ( !isdp )
  { return(LCRZO_ERR_OKNOTDECODED);
  }

  ret=lcrzo_string_search_char(dp, LCRZO_FALSE, +1,-1, '?', NULL, &pos);
  if (ret==LCRZO_ERR_OK)
  { lcrzo_er(lcrzo_string_initm_range(dp, +1, pos, &path));
    lcrzo_er(lcrzo_string_initm_range(dp, pos+1, -1, &ptvarvalanchor));
    ret=lcrzo_string_search_char(ptvarvalanchor, LCRZO_FALSE, +1,-1, '#', 
				 NULL, &pos);
    if (ret==LCRZO_ERR_OK)
    { lcrzo_er(lcrzo_string_initm_range(ptvarvalanchor, +1, pos, &ptvarval));
      lcrzo_er(lcrzo_string_initm_range(ptvarvalanchor, pos+1, -1, &anchor));
    }
    else
    { lcrzo_er(lcrzo_string_initm_text(ptvarvalanchor, &ptvarval));
      lcrzo_er(lcrzo_string_initm_text("", &anchor));
    }
    lcrzo_er(lcrzo_string_free2(&ptvarvalanchor));
  }
  else
  { lcrzo_er(lcrzo_string_initm_text("", &ptvarval));
    ret=lcrzo_string_search_char(dp, LCRZO_FALSE, +1,-1, '#', 
				 NULL, &pos);
    if (ret==LCRZO_ERR_OK)
    { lcrzo_er(lcrzo_string_initm_range(dp, +1, pos, &path));
      lcrzo_er(lcrzo_string_initm_range(dp, pos+1, -1, &anchor));
    }
    else
    { lcrzo_er(lcrzo_string_initm_text(dp, &path));
      lcrzo_er(lcrzo_string_initm_text("", &anchor));
    }
  }

  /* set output */
  lcrzo_er(shared_http_string_initm_pt(path, &path2));
  lcrzo_er(lcrzo_string_free2(&path));
  ret=shared_priv_http_path_clean(path2, ppath);
  if (ret!=LCRZO_ERR_OK)
  { lcrzo_er(lcrzo_string_initm_text(path2, ppath));
  }
  lcrzo_er(lcrzo_string_free2(&path2));
  lcrzo_er(shared_http_pt_initm_pt(ptvarval, SHARED_HTTP_PT_TYPE_VARVAL,
				   pptvarval));
  lcrzo_er(lcrzo_string_free2(&ptvarval));
  lcrzo_er(shared_http_string_initm_pt(anchor, panchor));
  lcrzo_er(lcrzo_string_free2(&anchor));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_varval_decode(lcrzo_conststring ptvarval,
			      lcrzo_conststring var,
			      lcrzo_data *pval,
			      lcrzo_int32 *pvalsize)
{ lcrzo_string ptvarvalclean, varequal;
  int ret, strlenvarequal;

  lcrzo_string pcstart, pcend, varval;

  if (ptvarval==NULL || var==NULL) return(LCRZO_ERR_PANULLPTR);

  lcrzo_er(shared_http_pt_initm_pt(ptvarval, SHARED_HTTP_PT_TYPE_VARVAL,
				   &ptvarvalclean));

  lcrzo_er(lcrzo_string_initm_text(var, &varequal));
  lcrzo_er(lcrzo_string_appendm_text("=", &varequal));
  strlenvarequal=strlen(varequal);

  pcstart=ptvarvalclean;
  while (LCRZO_TRUE)
  { pcend=strchr(pcstart, '&');
    if (pcend==NULL) pcend=ptvarvalclean+strlen(ptvarvalclean);
    lcrzo_er(shared_misc_string_initm_2ptrs(pcstart, pcend-1, &varval));
    if (!strncmp(varval, varequal, strlenvarequal))
    { ret=shared_http_data_initm_pt(varval+strlenvarequal, pval, pvalsize);
      lcrzo_er(lcrzo_string_free2(&varval));
      lcrzo_er(lcrzo_string_free2(&varequal));
      lcrzo_er(lcrzo_string_free2(&ptvarvalclean));
      return(ret);
    }
    lcrzo_er(lcrzo_string_free2(&varval));
    if (*pcend=='\0') break;
    pcstart=pcend+1;
  }

  lcrzo_er(lcrzo_string_free2(&varequal));
  lcrzo_er(lcrzo_string_free2(&ptvarvalclean));
  return (LCRZO_ERR_OKSEARCHNOTFOUND);
}

/*-------------------------------------------------------------*/
int shared_http_varval_decode_test(void);
int shared_http_varval_decode_test(void)
{ lcrzo_string val;
  lcrzo_int32 valsize;

  lcrzo_epr(shared_http_varval_decode("a=b&c=d", "a", 
				      (lcrzo_data*)&val, &valsize));
  val[valsize]='\0'; printf("AA%sZZ\n", val);
  if (strcmp(val, "b")) return(12345);
  lcrzo_er(lcrzo_string_free2(&val));
  
  lcrzo_epr(shared_http_varval_decode("a=b&c=d", "c", 
				      (lcrzo_data*)&val, &valsize));
  val[valsize]='\0'; printf("AA%sZZ\n", val);
  if (strcmp(val, "d")) return(12345);
  lcrzo_er(lcrzo_string_free2(&val));
  
  lcrzo_epr(shared_http_varval_decode("a=&c=", "a", 
				      (lcrzo_data*)&val, &valsize));
  val[valsize]='\0'; printf("AA%sZZ\n", val);
  if (strcmp(val, "")) return(12345);
  lcrzo_er(lcrzo_string_free2(&val));
  
  lcrzo_epr(shared_http_varval_decode("a=&c=", "c", 
				      (lcrzo_data*)&val, &valsize));
  val[valsize]='\0'; printf("AA%sZZ\n", val);
  if (strcmp(val, "")) return(12345);
  lcrzo_er(lcrzo_string_free2(&val));
  
  lcrzo_eipr(shared_http_varval_decode("a=b&c=d", "e", 
				      (lcrzo_data*)&val, &valsize),
	     LCRZO_ERR_OKSEARCHNOTFOUND);
  lcrzo_eipr(shared_http_varval_decode("a=b&c=d&e", "e", 
				      (lcrzo_data*)&val, &valsize),
	     LCRZO_ERR_OKSEARCHNOTFOUND);
  
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_url_init_url(lcrzo_conststring url,
			     lcrzo_string *purl)
{ lcrzo_string proto, user, password, server, path, ptvarval, anchor;
  lcrzo_uint16 port;
  int ret;

  lcrzo_er(shared_http_url_decode(url, &proto, 
				  &user, &password, &server, &port,
				  &path, &ptvarval, &anchor));
  /*printf("A%sB%sC%sD%sE%dF%sG%sH%sI\n", proto, user, password, server, port,
    path, ptvarval, anchor);*/
  ret=shared_http_url_init(proto, user, password, server, port,
			   path, ptvarval, anchor, purl);
  lcrzo_er(lcrzo_string_free2(&proto));
  if (user!=NULL) { lcrzo_er(lcrzo_string_free2(&user)); }
  if (password!=NULL) { lcrzo_er(lcrzo_string_free2(&password)); }
  lcrzo_er(lcrzo_string_free2(&server));
  lcrzo_er(lcrzo_string_free2(&path));
  lcrzo_er(lcrzo_string_free2(&ptvarval));
  lcrzo_er(lcrzo_string_free2(&anchor));
  return(ret);
}

/*-------------------------------------------------------------*/
int shared_priv_http_url_init_url_test(void);
int shared_priv_http_url_init_url_test(void)
{ lcrzo_string str;

  lcrzo_epr(shared_http_url_init_url("http://user:pass@server:81/dir/p.html?pa=s",
				  &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "http://user:pass@server:81/dir/p.html?pa=s")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));

  lcrzo_epr(shared_http_url_init_url("http://user:pass@server:81/dir/p.html?",
				  &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "http://user:pass@server:81/dir/p.html")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));

  lcrzo_epr(shared_http_url_init_url("http://user:pass@s:81/dir/p.html", 
				     &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "http://user:pass@s:81/dir/p.html")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));

  lcrzo_epr(shared_http_url_init_url("http://user:@s:81/dir/p.html", 
				     &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "http://user:@s:81/dir/p.html")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));

  lcrzo_epr(shared_http_url_init_url("http://user@s:81/dir/p.html", 
				     &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "http://user@s:81/dir/p.html")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));

  lcrzo_epr(shared_http_url_init_url("http://s/dir/p.html", &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "http://s/dir/p.html")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));

  lcrzo_epr(shared_http_url_init_url("http://s", &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "http://s/")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));

  lcrzo_epr(shared_http_url_init_url("http://s/di r/p.html", &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "http://s/di%20r/p.html")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));

  lcrzo_epr(shared_http_url_init_url("http://s/di%20r%4e/p.html", &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "http://s/di%20rN/p.html")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));

  lcrzo_epr(shared_http_url_init_url("http://s/di%20r%4e/p.html#2424", &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "http://s/di%20rN/p.html#2424")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));

  lcrzo_epr(shared_http_url_init_url("http://s/di%20r%4e/p.tml?%45=4 5",
				     &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "http://s/di%20rN/p.tml?E=4%205")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));

  lcrzo_epr(shared_http_url_init_url("http://s/di%20r%4e/p.tml?%45=4 5#2%42 4",
				     &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "http://s/di%20rN/p.tml?E=4%205#2B%204")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));

  lcrzo_epr(shared_http_url_init_url("http://s/dir/../p.html", &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "http://s/p.html")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));

  lcrzo_epr(shared_http_url_init_url("http://s/dir/../../p.html", &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "http://s/dir/../../p.html")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));

  lcrzo_epr(shared_http_url_init_url("http://server:90/dir", &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "http://server:90/dir")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));

  lcrzo_epr(shared_http_url_init_url("http://server/", &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "http://server/")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));

  lcrzo_epr(shared_http_url_init_url("ftp://server/dir/../page.html", &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "ftp://server/page.html")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));

  lcrzo_epr(shared_http_url_init_url("ftp://server/page.html", &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "ftp://server/page.html")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));

  lcrzo_epr(shared_http_url_init_url("ftp://server/", &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "ftp://server/")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_url_init_dp(lcrzo_conststring urlwheredpwasfound,
			    lcrzo_conststring dp,
			    lcrzo_string *purl)
{ lcrzo_string proto, user, password, server, path1, path2, ptvarval, anchor;
  lcrzo_string path3, path4;
  lcrzo_uint16 port;
  int ret;

  lcrzo_er(shared_http_url_decode(urlwheredpwasfound, &proto, 
				  &user, &password, &server, &port,
				  &path1, NULL, NULL));
  lcrzo_er(shared_http_dp_decode(dp, &path2, &ptvarval, &anchor));

  /* path2 is "/xxxx" */
  if ( path2[0]=='/' )
  { ret=shared_http_url_init(proto, user, password, server, port,
			     path2, ptvarval, anchor, purl);
  }
  /* path2 is "./file" or "../file" */
  else
  { ret=shared_priv_http_path_dir(path1, &path3);
    if (ret==LCRZO_ERR_OK)
    { ret=lcrzo_string_appendm_text(path2, &path3);
      if (ret==LCRZO_ERR_OK)
      { ret=shared_priv_http_path_clean(path3, &path4);
        if (ret==LCRZO_ERR_OK)
        { ret=shared_http_url_init(proto, user, password, server, port,
				   path4, ptvarval, anchor, purl);
          lcrzo_er(lcrzo_string_free2(&path4));
        }
        lcrzo_er(lcrzo_string_free2(&path3));
      }
    }
  }

  /* free */
  lcrzo_er(lcrzo_string_free2(&proto));
  if (user!=NULL) { lcrzo_er(lcrzo_string_free2(&user)); }
  if (password!=NULL) { lcrzo_er(lcrzo_string_free2(&password)); }
  lcrzo_er(lcrzo_string_free2(&server));
  lcrzo_er(lcrzo_string_free2(&path1));
  lcrzo_er(lcrzo_string_free2(&path2));
  lcrzo_er(lcrzo_string_free2(&ptvarval));
  lcrzo_er(lcrzo_string_free2(&anchor));
  return(ret);
}

/*-------------------------------------------------------------*/
int shared_priv_http_url_init_dp_test(void);
int shared_priv_http_url_init_dp_test(void)
{ lcrzo_string str;

  lcrzo_epr(shared_http_url_init_dp("http://server:81/dir/p.html?pa=s",
				    "toto.html",
				    &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "http://server:81/dir/toto.html")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));

  lcrzo_epr(shared_http_url_init_dp("http://server:81/dir/p.html?pa=s#rer",
				    "toto.html",
				    &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "http://server:81/dir/toto.html")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));

  lcrzo_epr(shared_http_url_init_dp("http://server/dir/p.html",
				    "./toto.html",
				    &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "http://server/dir/toto.html")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));

  lcrzo_epr(shared_http_url_init_dp("http://server/dir/p.html",
				    "../toto.html",
				    &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "http://server/toto.html")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));

  lcrzo_epr(shared_http_url_init_dp("http://server/dir/p.html",
				    "/toto.html",
				    &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "http://server/toto.html")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_url_init_dpurl(lcrzo_conststring urlwheredpurlwasfound,
			       lcrzo_conststring dpurl,
			       lcrzo_string *purl)
{ lcrzo_bool isurl;

  lcrzo_er(shared_http_dpurl_isurl(dpurl, &isurl));
  if (isurl)
  { return(shared_http_url_init_url(dpurl, purl));
  }
  else
  { return(shared_http_url_init_dp(urlwheredpurlwasfound, dpurl, purl));
  }
  return(LCRZO_ERR_OKNOTDECODED);
}

/*-------------------------------------------------------------*/
int shared_http_url_init_alp(lcrzo_conststring alp,
			     lcrzo_string *purl)
{ lcrzo_string url;
  int i, j, strlenalp, number;
  char c, c1, c2;

  strlenalp=strlen(alp);
  lcrzo_er(lcrzo_string_alloc(strlenalp, &url));
  j=0;
  for ( i=0 ; i<strlenalp ; i++ )
  { c=alp[i];
    if ( c!='_' )
    { url[j++]=c;
    }
    else
    { if ( i+1 >= strlenalp )
      { lcrzo_er(lcrzo_string_free2(&url));
        return(LCRZOEX_ERR_HTTP_MISSCHARAFTERPERCENT);
      }
      if (alp[i+1]=='/')
      { /* it is an added '/' */
        i++;
      }
      else if (alp[i+1]=='.')
      { /* it is an added '_', for windows, so ignore is */
      }
      else
      { if ( i+2 >= strlenalp )
        { lcrzo_er(lcrzo_string_free2(&url));
          return(LCRZOEX_ERR_HTTP_MISSCHARAFTERPERCENT);
        }
        c1=alp[++i];
	c2=alp[++i];
	if ( c1>='0' && c1<='9' ) number=c1-'0';
	else if ( c1>='a' && c1<='f')  number=10+c1-'a';
	else if ( c1>='A' && c1<='F' ) number=10+c1-'A';
	else
	{ lcrzo_er(lcrzo_string_free2(&url));
          return(LCRZOEX_ERR_HTTP_BADCHARAFTERPERCENT);
	}
	number=number<<4;
	if ( c2>='0' && c2<='9' ) number+=c2-'0';
	else if ( c2>='a' && c2<='f') number+=10+c2-'a';
	else if ( c2>='A' && c2<='F' ) number+=10+c2-'A';
	else
        { lcrzo_er(lcrzo_string_free2(&url));
          return(LCRZOEX_ERR_HTTP_BADCHARAFTERPERCENT);
	}
	url[j++]=(char)number;
      }
    }
  }
  url[j]='\0';

  lcrzo_er(lcrzo_string_initm_text(url, purl));
  lcrzo_er(lcrzo_string_free2(&url));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_priv_http_url_init_alp_test(void);
int shared_priv_http_url_init_alp_test(void)
{ lcrzo_string str;

  lcrzo_epr(shared_http_url_init_alp("http_3A/_2Fserver/dir/p.html_3Fpa_3Ds",
				     &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "http://server/dir/p.html?pa=s")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));

  lcrzo_epr(shared_http_url_init_alp("http_3A/_2Fu_3Ap_40server_3A50/",
				     &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "http://u:p@server:50/")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));

  lcrzo_epr(shared_http_url_init_alp("ftp_3A/_2Fserver/",
				     &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "ftp://server/")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));

  lcrzo_epr(shared_http_url_init_alp("mailto_3Alaurent_40toto",
				     &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "mailto:laurent@toto")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));

  lcrzo_epr(shared_http_url_init_alp("http_3A/_2Fver/dii_2520ir/p.l_3Fpa_3Ds",
				     &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "http://ver/dii%20ir/p.l?pa=s")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));
  lcrzo_epr(shared_http_url_init_alp("http_3A/_2Fver/dii_2520ir/p.l_3Fpa_3D_/s",
				     &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "http://ver/dii%20ir/p.l?pa=s")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));
  lcrzo_epr(shared_http_url_init_alp("http_3A/_2Fver/dii_2520ir/p.l_3Fpa_/_3Ds",
				     &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "http://ver/dii%20ir/p.l?pa=s")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));
  lcrzo_epr(shared_http_url_init_alp("http_3A/_2Fver/dii_2520i_/r/p.l_3Fpa_/_3Ds",
				     &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "http://ver/dii%20ir/p.l?pa=s")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));
  lcrzo_epr(shared_http_url_init_alp("http_3A/_2Fve_/r/dii_2520_/ir/p.l_3Fpa_/_3Ds",
				     &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "http://ver/dii%20ir/p.l?pa=s")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));
  lcrzo_epr(shared_http_url_init_alp("http_3A/_2Fv_/er/dii_252_/0ir/p.l_3Fp_/a_3Ds",
				     &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "http://ver/dii%20ir/p.l?pa=s")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));

  lcrzo_epr(shared_http_url_init_alp("http_3A/_2Fserver/_.ext",
				     &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "http://server/.ext")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));
  lcrzo_epr(shared_http_url_init_alp("http_3A/_2Fserver_/.ext",
				     &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "http://server.ext")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));
  lcrzo_epr(shared_http_url_init_alp("http_3A/_2Fserver_.ext",
				     &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "http://server.ext")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_url_init_rlp(lcrzo_conststring alpwhererlpwasfound,
			     lcrzo_conststring rlp,
			     lcrzo_string *purl)
{ lcrzo_string alp;
  int ret;

  lcrzo_er(shared_http_alp_init_rlp(alpwhererlpwasfound, rlp, &alp));
  ret=shared_http_url_init_alp(alp, purl);
  lcrzo_er(lcrzo_string_free2(&alp));
  return(ret);
}

/*-------------------------------------------------------------*/
int shared_http_dp_init_dp(lcrzo_conststring urlwheredpwasfound,
			   lcrzo_conststring dp,
			   lcrzo_conststring urlwhichwillcontaindp,
			   lcrzo_string *pdp)
{ lcrzo_string url;
  int ret;

  lcrzo_er(shared_http_url_init_dp(urlwheredpwasfound, dp, &url));
  ret=shared_http_dp_init_url(url, urlwhichwillcontaindp, pdp);
  lcrzo_er(lcrzo_string_free2(&url));
  return(ret);
}

/*-------------------------------------------------------------*/
int shared_http_dp_init_url(lcrzo_conststring url,
			    lcrzo_conststring urlwhichwillcontaindp,
			    lcrzo_string *pdp)
{ lcrzo_string proto1, user1, password1, server1, path1, ptvarval1, anchor1;
  lcrzo_string proto2, user2, password2, server2, path2;
  lcrzo_string path;
  lcrzo_uint16 port1, port2;
  int ret;

  lcrzo_er(shared_http_url_decode(url, &proto1, 
				  &user1, &password1, &server1, &port1,
				  &path1, &ptvarval1, &anchor1));
  lcrzo_er(shared_http_url_decode(urlwhichwillcontaindp, &proto2, 
				  &user2, &password2, &server2, &port2,
				  &path2, NULL, NULL));
  ret=LCRZO_ERR_OK;
  if (strcmp(proto1, proto2)) ret=LCRZO_ERR_OKNOTDECODED;
  if ( user1==NULL && user2!=NULL )
  { ret=LCRZO_ERR_OKNOTDECODED;
  }
  else if ( user1!=NULL && user2==NULL )
  { ret=LCRZO_ERR_OKNOTDECODED;
  }
  else if ( user1!=NULL && user2!=NULL )
  { if (strcmp(user1, user2)) ret=LCRZO_ERR_OKNOTDECODED;
  }
  if ( password1==NULL && password2!=NULL )
  { ret=LCRZO_ERR_OKNOTDECODED;
  }
  else if ( password1!=NULL && password2==NULL )
  { ret=LCRZO_ERR_OKNOTDECODED;
  }
  else if ( password1!=NULL && password2!=NULL )
  { if (strcmp(password1, password2)) ret=LCRZO_ERR_OKNOTDECODED;
  }
  if (strcmp(server1, server2)) ret=LCRZO_ERR_OKNOTDECODED;
  if (port1!=port2) ret=LCRZO_ERR_OKNOTDECODED;
  if (ret==LCRZO_ERR_OK)
  { ret=shared_priv_http_path_relative(path1, path2, &path);
    if (ret==LCRZO_ERR_OK)
    { ret=shared_http_dp_init(path, ptvarval1, anchor1, pdp);
      lcrzo_er(lcrzo_string_free2(&path));
    }
  }
  lcrzo_er(lcrzo_string_free2(&proto1));
  if (user1!=NULL) { lcrzo_er(lcrzo_string_free2(&user1)); }
  if (password1!=NULL) { lcrzo_er(lcrzo_string_free2(&password1)); }
  lcrzo_er(lcrzo_string_free2(&server1));
  lcrzo_er(lcrzo_string_free2(&path1));
  lcrzo_er(lcrzo_string_free2(&ptvarval1));
  lcrzo_er(lcrzo_string_free2(&anchor1));
  lcrzo_er(lcrzo_string_free2(&proto2));
  if (user2!=NULL) { lcrzo_er(lcrzo_string_free2(&user2)); }
  if (password2!=NULL) { lcrzo_er(lcrzo_string_free2(&password2)); }
  lcrzo_er(lcrzo_string_free2(&server2));
  lcrzo_er(lcrzo_string_free2(&path2));
  return(ret);
}

/*-------------------------------------------------------------*/
int shared_priv_http_dp_init_url_test(void);
int shared_priv_http_dp_init_url_test(void)
{ lcrzo_string str;

  lcrzo_epr(shared_http_dp_init_url("http://server/dir/p.html?pa=s",
				    "http://server/dir/index.html",
				    &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "./p.html?pa=s")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));

  lcrzo_epr(shared_http_dp_init_url("http://server/dir/p.html?pa=s",
				    "http://server/dir/tt/index.html",
				    &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "../p.html?pa=s")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));

  lcrzo_epr(shared_http_dp_init_url("http://server/dir/uu/p.html?pa=s",
				    "http://server/dir/tt/index.html",
				    &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "../uu/p.html?pa=s")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));

  lcrzo_epr(shared_http_dp_init_url("http://server/dir/uu/p.html?pa=s",
				    "http://server/dir/index.html",
				    &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "./uu/p.html?pa=s")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_dp_init_dpurl(lcrzo_conststring urlwheredpurlwasfound,
			      lcrzo_conststring dpurl,
			      lcrzo_conststring urlwhichwillcontaindp,
			      lcrzo_string *pdp)
{ lcrzo_bool isurl;

  lcrzo_er(shared_http_dpurl_isurl(dpurl, &isurl));
  if (isurl)
  { return(shared_http_dp_init_url(dpurl, urlwhichwillcontaindp, pdp));
  }
  else
  { return(shared_http_dp_init_dp(urlwheredpurlwasfound, dpurl,
				  urlwhichwillcontaindp, pdp));
  }
  return(LCRZO_ERR_OKNOTDECODED);
}

/*-------------------------------------------------------------*/
int shared_http_dp_init_alp(lcrzo_conststring alp,
			    lcrzo_conststring urlwhichwillcontaindp,
			    lcrzo_string *pdp)
{ lcrzo_string url;
  int ret;

  lcrzo_er(shared_http_url_init_alp(alp, &url));
  ret=shared_http_dp_init_url(url, urlwhichwillcontaindp, pdp);
  lcrzo_er(lcrzo_string_free2(&url));
  return(ret);
}

/*-------------------------------------------------------------*/
int shared_http_dp_init_rlp(lcrzo_conststring alpwhererlpwasfound,
			    lcrzo_conststring rlp,
			    lcrzo_conststring urlwhichwillcontaindp,
			    lcrzo_string *pdp)
{ lcrzo_string alp;
  int ret;

  lcrzo_er(shared_http_alp_init_rlp(alpwhererlpwasfound, rlp, &alp));
  ret=shared_http_dp_init_alp(alp, urlwhichwillcontaindp, pdp);
  lcrzo_er(lcrzo_string_free2(&alp));
  return(ret);
}

/*-------------------------------------------------------------*/
int shared_http_alp_init_alp(lcrzo_conststring alp,
			     lcrzo_uint16 maxfilenamelen,
			     lcrzo_string *palp)
{ lcrzo_string url;
  int ret;

  lcrzo_er(shared_http_url_init_alp(alp, &url));
  ret=shared_http_alp_init_url(url, maxfilenamelen, palp);
  lcrzo_er(lcrzo_string_free2(&url));
  return(ret);
}

/*-------------------------------------------------------------*/
int shared_http_alp_init_url(lcrzo_conststring url,
			     lcrzo_uint16 maxfilenamelen,
			     lcrzo_string *palp)
{ lcrzo_string urlclean, alp;
  lcrzo_bool lastcisslash;
  lcrzo_uint16 filenamelen, leftsize;
  char c, c1, c2;
  int ret, i, j, strlenurlclean;

  /* min is "/_xx_/"==4 or the length of the extension ("x.abcde"==7),
     so we use 8 which is more realistic */
  if (maxfilenamelen<8) return(LCRZO_ERR_PATOOLOW);

  /* clean url */
  ret=shared_http_url_init_url(url, &urlclean);
  if (ret!=LCRZO_ERR_OK)
  { lcrzo_er(lcrzo_string_initm_text(url, &urlclean));
  }

  /* allocate sufficient memory, to not have to bother */
  strlenurlclean=strlen(urlclean);
  lcrzo_er(lcrzo_string_alloc(4*strlenurlclean, &alp));

  /* convert */
  lastcisslash=LCRZO_FALSE;
  j=0;
  filenamelen=0;
  for ( i=0 ; i<strlenurlclean ; i++ )
  { c=urlclean[i];
    if ( c=='#' )
    { /* stop here */
      break;
    }
    else if ( c=='/' && !lastcisslash )
    { alp[j++]=c;
      lastcisslash=LCRZO_TRUE;
      filenamelen=0;
    }
    else if ( (c>='a'&&c<='z') || (c>='A'&&c<='Z') || (c>='0'&&c<='9') || 
	      c=='.' || c=='-' )
    { if ( filenamelen+1>=maxfilenamelen )
      { alp[j++]='_';
        alp[j++]='/';
	lastcisslash=LCRZO_TRUE;
	filenamelen=0;
      }
      if ( lastcisslash && c=='.' )
      { /* because under Windows a file named ".ext" is invalid */
	alp[j++]='_';
	lastcisslash=LCRZO_FALSE;
	filenamelen++;
      }
      leftsize=(lcrzo_uint16)(strlenurlclean-i);
      /*printf("%d %d\n", filenamelen, leftsize);*/
      if ( leftsize==7 )
      { if ( filenamelen+leftsize+1>maxfilenamelen )
        { /* because we do not want to truncate an extension "x.abcde" */
	  if ( urlclean[i+1]=='.' || urlclean[i+2]=='.' ||
	       urlclean[i+3]=='.' || urlclean[i+4]=='.' ||
	       urlclean[i+5]=='.' )
	  { alp[j++]='_';
            alp[j++]='/';
	    lastcisslash=LCRZO_TRUE;
	    filenamelen=0;
	  }
	}
      }
      alp[j++]=c;
      lastcisslash=LCRZO_FALSE;
      filenamelen++;
    }
    else
    { if ( filenamelen+3>=maxfilenamelen )
      { alp[j++]='_';
        alp[j++]='/';
	filenamelen=0;
      }
      alp[j++]='_';
      c1=(char)(c>>4);
      c2=(char)(c&0xF);
      if (c1<10) c1=(char)('0'+c1);
      else c1=(char)('A'+c1-10);
      if (c2<10) c2=(char)('0'+c2);
      else c2=(char)('A'+c2-10);
      alp[j++]=c1;
      alp[j++]=c2;
      lastcisslash=LCRZO_FALSE;
      filenamelen+=3;
    }
  }
  alp[j]='\0';

  lcrzo_er(lcrzo_string_initm_text(alp, palp));
  lcrzo_er(lcrzo_string_free2(&urlclean));
  lcrzo_er(lcrzo_string_free2(&alp));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_priv_http_alp_init_url_test(void);
int shared_priv_http_alp_init_url_test(void)
{ lcrzo_string str;

  lcrzo_epr(shared_http_alp_init_url("http://server/dir/p.html?pa=s",
				     30, &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "http_3A/_2Fserver/dir/p.html_3Fpa_3Ds")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));

  lcrzo_epr(shared_http_alp_init_url("http://server/dir/p.html?pa=s#toto",
				     30, &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "http_3A/_2Fserver/dir/p.html_3Fpa_3Ds")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));

  lcrzo_epr(shared_http_alp_init_url("http://u:p@server:50/",
				     30, &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "http_3A/_2Fu_3Ap_40server_3A50/")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));

  lcrzo_epr(shared_http_alp_init_url("ftp://server/",
				     30, &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "ftp_3A/_2Fserver/")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));

  lcrzo_epr(shared_http_alp_init_url("mailto:laurent@toto",
				     30, &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "mailto_3Alaurent_40toto")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));

  lcrzo_epr(shared_http_alp_init_url("mailto:laur_ent@toto",
				     30, &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "mailto_3Alaur_5Fent_40toto")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));

  lcrzo_epr(shared_http_alp_init_url("http://server/dii ir/p.l?pa=s#toto",
				     13, &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "http_3A/_2Fserver/dii_2520ir/p.l_3Fpa_3Ds")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));
  lcrzo_epr(shared_http_alp_init_url("http://server/dii ir/p.l?pa=s#toto",
				     12, &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "http_3A/_2Fserver/dii_2520ir/p.l_3Fpa_3D_/s")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));
  lcrzo_epr(shared_http_alp_init_url("http://server/dii ir/p.l?pa=s#toto",
				     11, &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "http_3A/_2Fserver/dii_2520ir/p.l_3Fpa_/_3Ds")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));
  lcrzo_epr(shared_http_alp_init_url("http://server/dii ir/p.l?pa=s#toto",
				     10, &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "http_3A/_2Fserver/dii_2520i_/r/p.l_3Fpa_/_3Ds")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));
  lcrzo_epr(shared_http_alp_init_url("http://server/dii ir/p.l?pa=s#toto",
				     9, &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "http_3A/_2Fserve_/r/dii_2520_/ir/p.l_3Fpa_/_3Ds")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));
  lcrzo_epr(shared_http_alp_init_url("http://server/dii ir/p.l?pa=s#toto",
				     8, &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "http_3A/_2Fserv_/er/dii_252_/0ir/p.l_3Fp_/a_3Ds")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));

  lcrzo_epr(shared_http_alp_init_url("http://server/",
				     10, &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "http_3A/_2Fserver/")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));
  lcrzo_epr(shared_http_alp_init_url("http://server/abcdef",
				     9, &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "http_3A/_2Fserve_/r/abcdef")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));
  lcrzo_epr(shared_http_alp_init_url("http://server/abcdef",
				     8, &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "http_3A/_2Fserv_/er/abcdef")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));

  lcrzo_epr(shared_http_alp_init_url("http://seZrv.er/abcdef",
				     10, &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "http_3A/_2FseZrv._/er/abcdef")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));
  lcrzo_epr(shared_http_alp_init_url("http://seZrv.er/abcdef",
				     9, &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "http_3A/_2FseZrv_/_.er/abcdef")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));
  lcrzo_epr(shared_http_alp_init_url("http://seZrv.er/abcdef",
				     8, &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "http_3A/_2FseZr_/v.er/abcdef")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));

  lcrzo_epr(shared_http_alp_init_url("http://ser/abcdefg",
				     8, &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "http_3A/_2Fser/abcdefg")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));

  lcrzo_epr(shared_http_alp_init_url("http://ser/abcdefgh",
				     8, &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "http_3A/_2Fser/abcdefg_/h")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));
  lcrzo_epr(shared_http_alp_init_url("http://ser/bcd.efg",
				     8, &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "http_3A/_2Fser/bcd.efg")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));

  lcrzo_epr(shared_http_alp_init_url("http://ser/bcd.efgh",
				     8, &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "http_3A/_2Fser/b_/cd.efgh")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));

  lcrzo_epr(shared_http_alp_init_url("http://ser/abcdefghijk.",
				     8, &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "http_3A/_2Fser/abcdefg_/hijk.")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));
  lcrzo_epr(shared_http_alp_init_url("http://ser/abcdefghij.k",
				     8, &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "http_3A/_2Fser/abcde_/fghij.k")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));
  lcrzo_epr(shared_http_alp_init_url("http://ser/abcdefghi.jk",
				     8, &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "http_3A/_2Fser/abcde_/fghi.jk")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));
  lcrzo_epr(shared_http_alp_init_url("http://ser/abcdefgh.ijk",
				     8, &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "http_3A/_2Fser/abcde_/fgh.ijk")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));
  lcrzo_epr(shared_http_alp_init_url("http://ser/abcdefg.hijk",
				     8, &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "http_3A/_2Fser/abcde_/fg.hijk")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));
  lcrzo_epr(shared_http_alp_init_url("http://ser/abcdef.ghijk",
				     8, &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "http_3A/_2Fser/abcde_/f.ghijk")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));
  lcrzo_epr(shared_http_alp_init_url("http://ser/abcde.fghijk",
				     8, &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "http_3A/_2Fser/abcde.f_/ghijk")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));

  lcrzo_epr(shared_http_alp_init_url("http://ser/abcdefghijklmnopq.rst",
				     8, &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "http_3A/_2Fser/abcdefg_/hijklmn_/opq.rst")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_alp_init_dp(lcrzo_conststring urlwheredpwasfound,
			    lcrzo_conststring dp,
			    lcrzo_uint16 maxfilenamelen,
			    lcrzo_string *palp)
{ lcrzo_string url;
  int ret;

  lcrzo_er(shared_http_url_init_dp(urlwheredpwasfound, dp, &url));
  ret=shared_http_alp_init_url(url, maxfilenamelen, palp);
  lcrzo_er(lcrzo_string_free2(&url));
  return(ret);
}

/*-------------------------------------------------------------*/
int shared_http_alp_init_dpurl(lcrzo_conststring urlwheredpurlwasfound,
			       lcrzo_conststring dpurl,
			       lcrzo_uint16 maxfilenamelen,
			       lcrzo_string *palp)
{ lcrzo_bool isurl;

  lcrzo_er(shared_http_dpurl_isurl(dpurl, &isurl));
  if (isurl)
  { return(shared_http_alp_init_url(dpurl, maxfilenamelen, palp));
  }
  else
  { return(shared_http_alp_init_dp(urlwheredpurlwasfound, dpurl,
				   maxfilenamelen, palp));
  }
  return(LCRZO_ERR_OKNOTDECODED);
}

/*-------------------------------------------------------------*/
int shared_http_alp_init_rlp(lcrzo_conststring alpwhererlpwasfound,
			     lcrzo_conststring rlp,
			     lcrzo_string *palp)
{ lcrzo_string alp;

  lcrzo_er(shared_priv_http_path_dir(alpwhererlpwasfound, &alp));
  lcrzo_er(lcrzo_string_appendm_text(rlp, &alp));

  lcrzo_er(shared_priv_http_path_clean(alp, palp));
  lcrzo_er(lcrzo_string_free2(&alp));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_priv_http_alp_init_rlp_test(void);
int shared_priv_http_alp_init_rlp_test(void)
{ lcrzo_string str;

  lcrzo_epr(shared_http_alp_init_rlp("/a/b/c/d",
				     "index.html",
				     &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "/a/b/c/index.html")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));

  lcrzo_epr(shared_http_alp_init_rlp("/a/b/c/d",
				     "../index.html",
				     &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "/a/b/index.html")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));

  lcrzo_epr(shared_http_alp_init_rlp("/a/b/c/d",
				     "../e/index.html",
				     &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "/a/b/e/index.html")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_rlp_init_rlp(lcrzo_conststring alpwhererlpwasfound,
			     lcrzo_conststring rlp,
			     lcrzo_conststring alpwhichwillcontainrlp,
			     lcrzo_uint16 maxfilenamelen,
			     lcrzo_string *prlp)
{ lcrzo_string url;
  int ret;

  lcrzo_er(shared_http_url_init_rlp(alpwhererlpwasfound, rlp, &url));
  ret=shared_http_rlp_init_url(url, alpwhichwillcontainrlp, maxfilenamelen, 
			       prlp);
  lcrzo_er(lcrzo_string_free2(&url));
  return(ret);
}

/*-------------------------------------------------------------*/
int shared_http_rlp_init_dp(lcrzo_conststring urlwheredpwasfound,
			    lcrzo_conststring dp,
			    lcrzo_conststring alpwhichwillcontainrlp,
			    lcrzo_uint16 maxfilenamelen,
			    lcrzo_string *prlp)
{ lcrzo_string url;
  int ret;

  lcrzo_er(shared_http_url_init_dp(urlwheredpwasfound, dp, &url));
  ret=shared_http_rlp_init_url(url, alpwhichwillcontainrlp, maxfilenamelen,
			       prlp);
  lcrzo_er(lcrzo_string_free2(&url));
  return(ret);
}

/*-------------------------------------------------------------*/
int shared_http_rlp_init_url(lcrzo_conststring url,
			     lcrzo_conststring alpwhichwillcontainrlp,
			     lcrzo_uint16 maxfilenamelen,
			     lcrzo_string *prlp)
{ lcrzo_string alp;
  int ret;

  lcrzo_er(shared_http_alp_init_url(url, maxfilenamelen, &alp));
  ret=shared_http_rlp_init_alp(alp, alpwhichwillcontainrlp, prlp);
  lcrzo_er(lcrzo_string_free2(&alp));
  return(ret);
}

/*-------------------------------------------------------------*/
int shared_http_rlp_init_dpurl(lcrzo_conststring urlwheredpurlwasfound,
			       lcrzo_conststring dpurl,
			       lcrzo_conststring alpwhichwillcontainrlp,
			       lcrzo_uint16 maxfilenamelen,
			       lcrzo_string *prlp)
{ lcrzo_bool isurl;

  lcrzo_er(shared_http_dpurl_isurl(dpurl, &isurl));
  if (isurl)
  { return(shared_http_rlp_init_url(dpurl, alpwhichwillcontainrlp,
				    maxfilenamelen, prlp));
  }
  else
  { return(shared_http_rlp_init_dp(urlwheredpurlwasfound, dpurl, 
				   alpwhichwillcontainrlp, maxfilenamelen, 
				   prlp));
  }
  return(LCRZO_ERR_OKNOTDECODED);
}

/*-------------------------------------------------------------*/
int shared_http_rlp_init_alp(lcrzo_conststring alp,
			     lcrzo_conststring alpwhichwillcontainrlp,
			     lcrzo_string *prlp)
{
  lcrzo_er(shared_priv_http_path_relative(alp, alpwhichwillcontainrlp, prlp));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_priv_http_rlp_init_alp_test(void);
int shared_priv_http_rlp_init_alp_test(void)
{ lcrzo_string str;

  lcrzo_epr(shared_http_rlp_init_alp("/a/b/c/index.html",
				     "/a/b/c/d",
				     &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "./index.html")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));

  lcrzo_epr(shared_http_rlp_init_alp("/a/b/index.html",
				     "/a/b/c/d",
				     &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "../index.html")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));

  lcrzo_epr(shared_http_rlp_init_alp("/a/b/e/index.html",
				     "/a/b/c/d",
				     &str));
  printf("AA%sZZ\n", str);
  if (strcmp(str, "../e/index.html")) return(12345);
  lcrzo_er(lcrzo_string_free2(&str));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
#define SHARED_PRIV_HTTP_SPIDER_DEFAULTINDEX "indexlcrzoex.html"
#define SHARED_PRIV_HTTP_SPIDER_ERRORINDEX ".errorlcrzoex.html"
#define SHARED_PRIV_HTTP_SPIDER_HTMLINDEX "__.html"

/*-------------------------------------------------------------*/
int shared_http_spider_initdefault(shared_http_spider *pspider)
{
  if (pspider==NULL) return(LCRZO_ERR_PANULLPTR);

  lcrzo_er(lcrzo_string_initm_text("", &(pspider->proxyurl)));
  lcrzo_er(lcrzo_string_initm_text("/tmp/", &(pspider->localrootdir)));
  lcrzo_er(lcrzo_string_initm_text("lcrzoex", &(pspider->useragent)));
  lcrzo_er(lcrzo_stringlist_init(&(pspider->listurltodownload)));
  pspider->maxrecursion=0;
  pspider->millisleep=0;
  lcrzo_er(lcrzo_stringlist_init(&(pspider->listurlmustcontain)));
  lcrzo_er(lcrzo_stringlist_init(&(pspider->listurlmustnotcontain)));
  pspider->getimage=2;
  pspider->getmultimedia=2;
  pspider->getcode=2;
  pspider->maxfilenamelen=64;
  pspider->linkconversion=1;
  pspider->currentrecursionlevel=1;
  pspider->lastisarecurmarker=LCRZO_TRUE;
  pspider->logfileinitialized=LCRZO_FALSE;

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_priv_http_spider_log(shared_http_spider *pspider,
				lcrzo_conststring frtxt,
				lcrzo_conststring entxt);
int shared_priv_http_spider_log(shared_http_spider *pspider,
				lcrzo_conststring frtxt,
				lcrzo_conststring entxt)
{ lcrzo_int16 language;

  lcrzo_er(lcrzo_global_get_language(&language));
  if ( language==LCRZO_GLOBAL_LANG_FR )
  { printf("%s\n", frtxt);
  }
  else
  { printf("%s\n", entxt);
  }
  lcrzo_er(lcrzo_stream_flush(stdout));

  if (pspider->logfileinitialized)
  { if ( language==LCRZO_GLOBAL_LANG_FR )
    { lcrzo_er(lcrzo_file_write(&(pspider->logfile), 
				(lcrzo_constdata)frtxt, strlen(frtxt)));
    }
    else
    { lcrzo_er(lcrzo_file_write(&(pspider->logfile), 
				(lcrzo_constdata)entxt, strlen(entxt)));
    }
    lcrzo_er(lcrzo_file_write(&(pspider->logfile), 
			      (lcrzo_constdata)"\r\n", 2));
    lcrzo_er(lcrzo_file_flush(&(pspider->logfile)));
  }

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_spider_set_proxyurl(shared_http_spider *pspider,
				    lcrzo_conststring proxyurl)
{
  if (pspider==NULL) return(LCRZO_ERR_PANULLPTR);

  lcrzo_er(shared_priv_http_spider_log(pspider,
				       "Affectation de proxyurl",
				       "Setting proxyurl"));

  /* set parameters */
  lcrzo_er(lcrzo_string_free2(&(pspider->proxyurl)));
  if ( proxyurl==NULL )
  { lcrzo_er(lcrzo_string_initm_text("", &(pspider->proxyurl)));
  }
  else
  { lcrzo_er(lcrzo_string_initm_text(proxyurl, &(pspider->proxyurl)));
  }

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_spider_set_localrootdir(shared_http_spider *pspider,
					lcrzo_conststring localrootdir)
{ lcrzo_string logfilename;
  lcrzo_int16 type;
  int ret;

  if (pspider==NULL) return(LCRZO_ERR_PANULLPTR);
  if (localrootdir==NULL) return(LCRZO_ERR_PANULLPTR);

  lcrzo_er(shared_priv_http_spider_log(pspider,
				       "Affectation de localrootdir...",
				       "Setting localrootdir..."));

  ret=lcrzo_path_type(localrootdir, &type);
  if (ret==LCRZO_ERR_OK)
  { if (type!=LCRZO_PATH_TYPE_DIR)
    { lcrzo_er(shared_priv_http_spider_log(pspider,
				      "localrootdir n'est pas un repertoire",
				      "localrootdir is not a directory"));
      return(LCRZO_ERR_OKFILENOTFOUND);
    }
  }
  else
  { lcrzo_er(shared_priv_http_spider_log(pspider,
					 "localrootdir n'existe pas : il sera cree",
					 "localrootdir does not exists : it will be created"));
  }

  lcrzo_er(lcrzo_string_free2(&(pspider->localrootdir)));
  lcrzo_er(lcrzo_string_initm_text(localrootdir, &(pspider->localrootdir)));
  if (pspider->localrootdir[strlen(pspider->localrootdir)-1]!='/')
  { lcrzo_er(lcrzo_string_appendm_text("/", &(pspider->localrootdir)));
  }

  /* open logfile */
  if (pspider->logfileinitialized)
  { lcrzo_er(lcrzo_file_close(&(pspider->logfile)));
  }
  lcrzo_er(lcrzo_string_initm_text(pspider->localrootdir, &logfilename));
  lcrzo_er(lcrzo_string_appendm_text("log.txt", &logfilename));
  lcrzo_er(lcrzo_file_open_write(logfilename, LCRZO_FALSE, LCRZO_FALSE, 
				 &(pspider->logfile)));
  lcrzo_er(lcrzo_string_free2(&logfilename));
  pspider->logfileinitialized=LCRZO_TRUE;

  lcrzo_er(shared_priv_http_spider_log(pspider,
				       "localrootdir affecte",
				       "localrootdir set"));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_spider_set_useragent(shared_http_spider *pspider,
				     lcrzo_conststring useragent)
{
  if (pspider==NULL) return(LCRZO_ERR_PANULLPTR);

  lcrzo_er(shared_priv_http_spider_log(pspider,
				       "Affectation de useragent",
				       "Setting useragent"));

  /* set parameters */
  lcrzo_er(lcrzo_string_free2(&(pspider->useragent)));
  if ( useragent==NULL )
  { lcrzo_er(lcrzo_string_initm_text("lcrzoex", &(pspider->useragent)));
  }
  else
  { lcrzo_er(lcrzo_string_initm_text(useragent, &(pspider->useragent)));
  }

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_spider_set_maxrecursion(shared_http_spider *pspider,
					lcrzo_uint32 maxrecursion)
{
  if (pspider==NULL) return(LCRZO_ERR_PANULLPTR);

  lcrzo_er(shared_priv_http_spider_log(pspider,
				       "Affectation de maxrecursion",
				       "Setting maxrecursion"));

  pspider->maxrecursion=maxrecursion;
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_spider_set_millisleep(shared_http_spider *pspider,
				      lcrzo_uint32 millisleep)
{
  if (pspider==NULL) return(LCRZO_ERR_PANULLPTR);

  lcrzo_er(shared_priv_http_spider_log(pspider,
				       "Affectation de millisleep",
				       "Setting millisleep"));

  pspider->millisleep=millisleep;
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_spider_set_getimage(shared_http_spider *pspider,
				    lcrzo_uint16 getimage)
{
  if (pspider==NULL) return(LCRZO_ERR_PANULLPTR);

  lcrzo_er(shared_priv_http_spider_log(pspider,
				       "Affectation de getimage",
				       "Setting getimage"));

  pspider->getimage=getimage;
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_spider_set_getmultimedia(shared_http_spider *pspider,
					 lcrzo_uint16 getmultimedia)
{
  if (pspider==NULL) return(LCRZO_ERR_PANULLPTR);

  lcrzo_er(shared_priv_http_spider_log(pspider,
				       "Affectation de getmultimedia",
				       "Setting getmultimedia"));

  pspider->getmultimedia=getmultimedia;
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_spider_set_getcode(shared_http_spider *pspider,
				   lcrzo_uint16 getcode)
{
  if (pspider==NULL) return(LCRZO_ERR_PANULLPTR);

  lcrzo_er(shared_priv_http_spider_log(pspider,
				       "Affectation de getcode",
				       "Setting getcode"));

  pspider->getcode=getcode;
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_spider_set_maxfilenamelen(shared_http_spider *pspider,
					  lcrzo_uint16 maxfilenamelen)
{
  if (pspider==NULL) return(LCRZO_ERR_PANULLPTR);

  lcrzo_er(shared_priv_http_spider_log(pspider,
				       "Affectation de maxfilenamelen",
				       "Setting maxfilenamelen"));

  if (maxfilenamelen<4) return(LCRZO_ERR_PATOOLOW);

  pspider->maxfilenamelen=maxfilenamelen;
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_spider_set_linkconversion(shared_http_spider *pspider,
					  lcrzo_uint16 linkconversion)
{
  if (pspider==NULL) return(LCRZO_ERR_PANULLPTR);

  lcrzo_er(shared_priv_http_spider_log(pspider,
				       "Affectation de linkconversion",
				       "Setting linkconversion"));

  pspider->linkconversion=linkconversion;
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_spider_add_urltodownload(shared_http_spider *pspider,
					 lcrzo_uint32 linktype,
					 lcrzo_conststring url)
{ lcrzo_string urlclean;

  if (pspider==NULL) return(LCRZO_ERR_PANULLPTR);
  if (url==NULL) return(LCRZO_ERR_PANULLPTR);

  lcrzo_er(shared_priv_http_spider_log(pspider,
				       "Affectation de urltodownload",
				       "Setting urltodownload"));

  lcrzo_er(shared_http_url_init_url(url, &urlclean));
  lcrzo_er(lcrzo_stringlist_add_pos(&(pspider->listurltodownload), -1, 
                                    linktype, urlclean));
  lcrzo_er(lcrzo_string_free2(&urlclean));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_spider_add_urlmustcontain(shared_http_spider *pspider,
					  lcrzo_conststring urlmustcontain)
{ 
  if (pspider==NULL) return(LCRZO_ERR_PANULLPTR);
  if (urlmustcontain==NULL) return(LCRZO_ERR_PANULLPTR);

  lcrzo_er(shared_priv_http_spider_log(pspider,
				       "Affectation de urlmustcontain",
				       "Setting urlmustcontain"));

  lcrzo_er(lcrzo_stringlist_add_pos(&(pspider->listurlmustcontain), -1, 
                                    0, urlmustcontain));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_spider_add_urlmustnotcontain(shared_http_spider *pspider,
					  lcrzo_conststring urlmustnotcontain)
{ 
  if (pspider==NULL) return(LCRZO_ERR_PANULLPTR);
  if (urlmustnotcontain==NULL) return(LCRZO_ERR_PANULLPTR);

  lcrzo_er(shared_priv_http_spider_log(pspider,
				       "Affectation de urlmustnotcontain",
				       "Setting urlmustnotcontain"));

  lcrzo_er(lcrzo_stringlist_add_pos(&(pspider->listurlmustnotcontain), -1, 
                                    0, urlmustnotcontain));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_spider_init_conffile(shared_http_spider *pspider,
				     lcrzo_conststring conffile)

{ lcrzo_list conflist;
  lcrzo_int32 count, i;
  lcrzo_string var, val;
  lcrzo_bool setproxyurl, setlocalrootdir, setmaxrecursion, setmillisleep;
  lcrzo_bool setgetimage, setgetmultimedia, setgetcode, setmaxfilenamelen;
  lcrzo_bool seturltodownload, seturlmustcontain, seturlmustnotcontain;
  lcrzo_bool setlinkconversion, setuseragent;
  lcrzo_uint32 auint32;
  lcrzo_uint16 auint16;
  lcrzo_int16 language;
  int ret;

  lcrzo_er(shared_conffile_list_init_file(conffile, &conflist));

  lcrzo_er(shared_http_spider_initdefault(pspider));

  setproxyurl=LCRZO_FALSE;
  setlocalrootdir=LCRZO_FALSE;
  setuseragent=LCRZO_FALSE;
  setmaxrecursion=LCRZO_FALSE;
  setmillisleep=LCRZO_FALSE;
  setgetimage=LCRZO_FALSE;
  setgetmultimedia=LCRZO_FALSE;
  setgetcode=LCRZO_FALSE;
  setmaxfilenamelen=LCRZO_FALSE;
  setlinkconversion=LCRZO_FALSE;
  seturltodownload=LCRZO_FALSE;
  seturlmustcontain=LCRZO_FALSE;
  seturlmustnotcontain=LCRZO_FALSE;

  lcrzo_er(lcrzo_global_get_language(&language));

  lcrzo_er(lcrzo_list_count(&conflist, &count));
  for ( i=1 ; i<=count ; i++ )
  { lcrzo_er(shared_conffile_list_valuem_pos(&conflist, i, &var, &val));
    if (!shared_misc_strcasecmp(var, "proxyurl"))
    { setproxyurl=LCRZO_TRUE;
      lcrzo_er(shared_http_spider_set_proxyurl(pspider, val));
    }
    else if (!shared_misc_strcasecmp(var, "localrootdir"))
    { setlocalrootdir=LCRZO_TRUE;
      lcrzo_er(shared_http_spider_set_localrootdir(pspider, val));
    }
    else if (!shared_misc_strcasecmp(var, "useragent"))
    { setuseragent=LCRZO_TRUE;
      lcrzo_er(shared_http_spider_set_useragent(pspider, val));
    }
    else if (!shared_misc_strcasecmp(var, "maxrecursion"))
    { ret=lcrzo_uint32_init_text(val, &auint32);
      if (ret==LCRZO_ERR_OK) 
      { setmaxrecursion=LCRZO_TRUE;
        lcrzo_er(shared_http_spider_set_maxrecursion(pspider, auint32));
      }
    }
    else if (!shared_misc_strcasecmp(var, "millisleep"))
    { ret=lcrzo_uint32_init_text(val, &auint32);
      if (ret==LCRZO_ERR_OK) 
      { setmillisleep=LCRZO_TRUE;
        lcrzo_er(shared_http_spider_set_millisleep(pspider, auint32));
      }
    }
    else if (!shared_misc_strcasecmp(var, "getimage"))
    { ret=lcrzo_uint16_init_text(val, &auint16);
      if (ret==LCRZO_ERR_OK) 
      { setgetimage=LCRZO_TRUE;
        lcrzo_er(shared_http_spider_set_getimage(pspider, auint16));
      }
    }
    else if (!shared_misc_strcasecmp(var, "getmultimedia"))
    { ret=lcrzo_uint16_init_text(val, &auint16);
      if (ret==LCRZO_ERR_OK) 
      { setgetmultimedia=LCRZO_TRUE;
        lcrzo_er(shared_http_spider_set_getmultimedia(pspider, auint16));
      }
    }
    else if (!shared_misc_strcasecmp(var, "getcode"))
    { ret=lcrzo_uint16_init_text(val, &auint16);
      if (ret==LCRZO_ERR_OK) 
      { setgetcode=LCRZO_TRUE;
        lcrzo_er(shared_http_spider_set_getcode(pspider, auint16));
      }
    }
    else if (!shared_misc_strcasecmp(var, "maxfilenamelen"))
    { ret=lcrzo_uint16_init_text(val, &auint16);
      if (ret==LCRZO_ERR_OK) 
      { setmaxfilenamelen=LCRZO_TRUE;
        lcrzo_er(shared_http_spider_set_maxfilenamelen(pspider, auint16));
      }
    }
    else if (!shared_misc_strcasecmp(var, "linkconversion"))
    { ret=lcrzo_uint16_init_text(val, &auint16);
      if (ret==LCRZO_ERR_OK) 
      { setlinkconversion=LCRZO_TRUE;
        lcrzo_er(shared_http_spider_set_linkconversion(pspider, auint16));
      }
    }
    else if (!shared_misc_strcasecmp(var, "urltodownload"))
    { seturltodownload=LCRZO_TRUE;
      lcrzo_er(shared_http_spider_add_urltodownload(pspider, 
						SHARED_HTTP_LINKTYPE_A_HREF,
						    val));
    }
    else if (!shared_misc_strcasecmp(var, "urlmustcontain"))
    { seturlmustcontain=LCRZO_TRUE;
      lcrzo_er(shared_http_spider_add_urlmustcontain(pspider, val));
    }
    else if (!shared_misc_strcasecmp(var, "urlmustnotcontain"))
    { seturlmustnotcontain=LCRZO_TRUE;
      lcrzo_er(shared_http_spider_add_urlmustnotcontain(pspider, val));
    }
    else
    { if ( language==LCRZO_GLOBAL_LANG_FR )
      { printf("Warning : parametre '%s' non reconnu\n", var);
      }
      else
      { printf("Warning : parameter '%s' unrecognized\n", var);
      }
    }
    lcrzo_er(lcrzo_string_free2(&var));
    lcrzo_er(lcrzo_string_free2(&val));
  }
  lcrzo_er(lcrzo_list_close(&conflist));

  if ( !setlocalrootdir )
  { if ( language==LCRZO_GLOBAL_LANG_FR )
    { printf("Warning : parametre localrootdir non affecte\n");
    }
    else
    { printf("Warning : parameter localrootdir not set\n");
    }
  }
  if ( !seturltodownload )
  { if ( language==LCRZO_GLOBAL_LANG_FR )
    { printf("Warning : parametre urltodownload non affecte\n");
    }
    else
    { printf("Warning : parameter urltodownload not set\n");
    }
  }

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_priv_http_spider_localfile_init_url(shared_http_spider *pspider,
					       lcrzo_conststring url,
					       lcrzo_uint16 linkcontain,
					       lcrzo_string *plocalfile);
int shared_priv_http_spider_localfile_init_url(shared_http_spider *pspider,
					       lcrzo_conststring url,
					       lcrzo_uint16 linkcontain,
					       lcrzo_string *plocalfile)
{ lcrzo_string alp, tmp;
  lcrzo_bool addhtml, foundext;
  lcrzo_int32 strlentmp, i;
  char c;

  /* construct final path */
  lcrzo_er(lcrzo_string_initm_text(pspider->localrootdir, &tmp));
  lcrzo_er(lcrzo_string_appendm_text("/", &tmp));
  lcrzo_er(shared_http_alp_init_url(url, pspider->maxfilenamelen, &alp));
  lcrzo_er(lcrzo_string_appendm_text(alp, &tmp));
  lcrzo_er(lcrzo_string_free2(&alp));
  if ( tmp[strlen(tmp)-1]=='/' )
  { lcrzo_er(lcrzo_string_appendm_text(SHARED_PRIV_HTTP_SPIDER_DEFAULTINDEX, 
					&tmp));
  }

  /* eventually add "__.html" */
  if (linkcontain==SHARED_HTTP_LINK_CONTAIN_HTML)
  { addhtml=LCRZO_TRUE;
    strlentmp=strlen(tmp);
    foundext=LCRZO_FALSE;
    for ( i=1 ; i<=5 ; i++ )
    { if ( strlentmp<i+1 ) break;
      c=*(tmp+strlentmp-i);
      if (c=='.')
      { if (foundext) addhtml=LCRZO_FALSE;
        if ( *(tmp+strlentmp-i-1)=='/' )
	{ addhtml=LCRZO_TRUE; /*forbids "/.ext"*/
	}
        break;
      }
      else if ( (c>='A'&&c<='Z') || (c>='a'&&c<='z') || (c>='0'&&c<='9') )
      { foundext=LCRZO_TRUE;
      }
      else
      { break;
      }
    }
    if (addhtml)
    { lcrzo_er(lcrzo_string_appendm_text(SHARED_PRIV_HTTP_SPIDER_HTMLINDEX, 
					 &tmp));
    }
  }

  lcrzo_er(shared_priv_http_path_clean(tmp, plocalfile));
  lcrzo_er(lcrzo_string_free2(&tmp));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_priv_http_spider_url_download(shared_http_spider *pspider,
					 lcrzo_conststring url,
					 lcrzo_uint16 linkcontain,
					 lcrzo_uint16 *perrorcode,
					 lcrzo_string *plocalfile);
int shared_priv_http_spider_url_download(shared_http_spider *pspider,
					 lcrzo_conststring url,
					 lcrzo_uint16 linkcontain,
					 lcrzo_uint16 *perrorcode,
					 lcrzo_string *plocalfile)
{ lcrzo_string localfile, localfile2;
  lcrzo_uint16 errorcode;
  int ret;
    
  lcrzo_er(lcrzo_time_sleep_msec(pspider->millisleep));

  /* compute local file */
  lcrzo_er(shared_priv_http_spider_localfile_init_url(pspider, url, 
						      linkcontain,
						      &localfile));
  ret=shared_http_url_download_file2(url, pspider->proxyurl,
				     pspider->useragent, 
				     &errorcode, localfile);
  if (ret!=LCRZO_ERR_OK)
  { lcrzo_er(lcrzo_string_free2(&localfile));
    return(ret);
  }
  if (perrorcode!=NULL) *perrorcode=errorcode;

  /* on error, rename filename. When a directory is accessed as
     "/dir", it is redirected to "/dir/", so we cannot save a file
     with name "dir", because it should be a directory. */
  if (errorcode!=SHARED_HTTP_ERRORCODE_OK)
  { lcrzo_er(lcrzo_string_initm_text(localfile, &localfile2));
    lcrzo_er(lcrzo_string_appendm_text(SHARED_PRIV_HTTP_SPIDER_ERRORINDEX, 
				       &localfile2));
    lcrzo_er(lcrzo_filename_rename(localfile, localfile2));
    lcrzo_er(lcrzo_string_initm_text(localfile2, plocalfile));
    lcrzo_er(lcrzo_string_free2(&localfile2));
  }
  else
  { lcrzo_er(lcrzo_string_initm_text(localfile, plocalfile));
  }     
  lcrzo_er(lcrzo_string_free2(&localfile));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_priv_http_spider_url_alreadydownloaded(shared_http_spider *pspider,
						  lcrzo_conststring url,
						  lcrzo_string *pfilewereput,
						  lcrzo_bool *pfileexists);
int shared_priv_http_spider_url_alreadydownloaded(shared_http_spider *pspider,
						  lcrzo_conststring url,
						  lcrzo_string *pfilewereput,
						  lcrzo_bool *pfileexists)
{ lcrzo_string localfile;
  lcrzo_bool fileexists;
  int ret;
 
  ret=shared_priv_http_spider_localfile_init_url(pspider, url, 0,
						 &localfile);
  if (ret!=LCRZO_ERR_OK)
  { if (pfileexists!=NULL) *pfileexists=LCRZO_FALSE;
    return(LCRZO_ERR_OK);
  }

  ret=lcrzo_filename_exist(localfile, &fileexists);
  if (ret!=LCRZO_ERR_OK)
  { fileexists=LCRZO_FALSE;
  }

  if (!fileexists)
  { /* also try with ERRORINDEX */
    ret=lcrzo_string_appendm_text(SHARED_PRIV_HTTP_SPIDER_ERRORINDEX, 
				  &localfile);
    if (ret==LCRZO_ERR_OK)
    { ret=lcrzo_filename_exist(localfile, &fileexists);
      if (ret!=LCRZO_ERR_OK)
      { fileexists=LCRZO_FALSE;
      }
    }
  }

  if (!fileexists)
  { /* try with __.html */
    ret=lcrzo_string_free2(&localfile);
    if (ret!=LCRZO_ERR_OK)
    { if (pfileexists!=NULL) *pfileexists=LCRZO_FALSE;
      return(LCRZO_ERR_OK);
    } 
    ret=shared_priv_http_spider_localfile_init_url(pspider, url,
					        SHARED_HTTP_LINK_CONTAIN_HTML,
							&localfile);
    if (ret!=LCRZO_ERR_OK)
    { if (pfileexists!=NULL) *pfileexists=LCRZO_FALSE;
      return(LCRZO_ERR_OK);
    } 
    ret=lcrzo_filename_exist(localfile, &fileexists);
    if (ret!=LCRZO_ERR_OK)
    { fileexists=LCRZO_FALSE;
    }
    if (!fileexists)
    { /* also try with ERRORINDEX */
      ret=lcrzo_string_appendm_text(SHARED_PRIV_HTTP_SPIDER_ERRORINDEX, 
				    &localfile);
      if (ret==LCRZO_ERR_OK)
      { ret=lcrzo_filename_exist(localfile, &fileexists);
        if (ret!=LCRZO_ERR_OK)
        { fileexists=LCRZO_FALSE;
        }
      }
    }
  }

  if (fileexists)
  { ret=lcrzo_string_initm_text(localfile, pfilewereput);
    if (ret!=LCRZO_ERR_OK)
    { fileexists=LCRZO_FALSE;
    }
  }
  ret=lcrzo_string_free2(&localfile);
  if (ret!=LCRZO_ERR_OK) fileexists=LCRZO_FALSE;

  if (pfileexists!=NULL) *pfileexists=fileexists;
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_priv_http_spider_urllist_complete_localfile(shared_http_spider *pspider,
						       lcrzo_uint32 linktype,
						       lcrzo_conststring urlwheredpwas,
						       lcrzo_conststring localfile);
int shared_priv_http_spider_urllist_complete_localfile(shared_http_spider *pspider,
						       lcrzo_uint32 linktype,
						       lcrzo_conststring urlwheredpwas,
						       lcrzo_conststring localfile)
{ lcrzo_list list;
  lcrzo_string dpurl, url;
  lcrzo_uint32 dplinktype;
  lcrzo_int32 count, i;
  lcrzo_uint16 linkcontain;
  int ret;

  /* check if this linktype is html */
  lcrzo_er(shared_http_linktype_get_contain(linktype, &linkcontain));
  if ( linkcontain != SHARED_HTTP_LINK_CONTAIN_HTML )
  { return(LCRZO_ERR_OK);
  }

  /* obtain the list of url contained in this file */
  ret=shared_http_urllist_init_file(localfile, &list);
  if (ret!=LCRZO_ERR_OK)
  { return(LCRZO_ERR_OK);
  }

  /* add them in the list of url to download */
  lcrzo_er(lcrzo_list_count(&list, &count));
  for ( i=1 ; i<=count ; i++ )
  { lcrzo_er(shared_http_urllist_valuem_pos(&list, i, &dplinktype, &dpurl, 
					     NULL, NULL, NULL));
    ret=shared_http_url_init_dpurl(urlwheredpwas, dpurl, &url);
    if ( ret==LCRZO_ERR_OK )
    { lcrzo_er(lcrzo_stringlist_add_pos(&pspider->listurltodownload, -1, 
				         dplinktype, url));
      lcrzo_er(lcrzo_string_free2(&url));
    }
    lcrzo_er(lcrzo_string_free2(&dpurl));
  }
  lcrzo_er(lcrzo_list_close(&list));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_priv_http_spider_recurmarker_add(shared_http_spider *pspider,
					    lcrzo_uint32 level);
int shared_priv_http_spider_recurmarker_add(shared_http_spider *pspider,
					    lcrzo_uint32 level)
{ lcrzo_string str;

#define SHARED_PRIV_HTTP_RECURMARKER "lcrzoexrecurend:"
  lcrzo_er(lcrzo_string_initm_text(SHARED_PRIV_HTTP_RECURMARKER, &str));
  lcrzo_er(lcrzo_string_appendm_uint32(level, "%u", &str));
  lcrzo_er(lcrzo_stringlist_add_pos(&(pspider->listurltodownload), -1, 
                                    0, str));
  lcrzo_er(lcrzo_string_free2(&str));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_priv_http_spider_recurmarker_check(shared_http_spider *pspider,
					      lcrzo_uint32 linktype,
					      lcrzo_conststring url,
					      lcrzo_bool *purlisarecurmarker,
					      lcrzo_bool *pcandownloadthisurl,
					      lcrzo_bool *pmaxrecurreached,
					      lcrzo_bool *pnomoreurl);
int shared_priv_http_spider_recurmarker_check(shared_http_spider *pspider,
					      lcrzo_uint32 linktype,
					      lcrzo_conststring url,
					      lcrzo_bool *purlisarecurmarker,
					      lcrzo_bool *pcandownloadthisurl,
					      lcrzo_bool *pmaxrecurreached,
					      lcrzo_bool *pnomoreurl)
{ lcrzo_uint32 level;
  lcrzo_bool urlisarecurmarker, candownloadthisurl, maxrecurreached;
  lcrzo_uint16 linkcontain;
  int ret, strlenmark;

  urlisarecurmarker=LCRZO_FALSE;
  strlenmark=strlen(SHARED_PRIV_HTTP_RECURMARKER);
  if (pnomoreurl!=NULL) *pnomoreurl=LCRZO_FALSE;
  if ( !strncmp(url, SHARED_PRIV_HTTP_RECURMARKER, strlenmark) )
  { ret=lcrzo_uint32_init_text(url+strlenmark, &level);
    if (ret==LCRZO_ERR_OK)
    { if (pspider->lastisarecurmarker)
      { /* two markers, indicates the end */
	if (purlisarecurmarker!=NULL) *purlisarecurmarker=LCRZO_TRUE;
	if (pcandownloadthisurl!=NULL) *pcandownloadthisurl=LCRZO_FALSE;
	if (pmaxrecurreached!=NULL) *pmaxrecurreached=LCRZO_FALSE;
	if (pnomoreurl!=NULL) *pnomoreurl=LCRZO_TRUE;
	return(LCRZO_ERR_OK);
      }
      pspider->currentrecursionlevel=level+1;
      pspider->lastisarecurmarker=LCRZO_TRUE;
      urlisarecurmarker=LCRZO_TRUE;
      lcrzo_er(shared_priv_http_spider_recurmarker_add(pspider, level+1));
    }
  }
  if (!urlisarecurmarker)
  { pspider->lastisarecurmarker=LCRZO_FALSE;
  }

  if (pspider->maxrecursion)
  { if ( pspider->getimage==2 ||
	 pspider->getmultimedia==2 || 
	 pspider->getcode==2  )
    { if ( pspider->currentrecursionlevel > pspider->maxrecursion+1 )
      { candownloadthisurl=LCRZO_FALSE;
        maxrecurreached=LCRZO_TRUE;
      }
      else if ( pspider->currentrecursionlevel > pspider->maxrecursion )
      { if (!urlisarecurmarker)
        { lcrzo_er(shared_http_linktype_get_contain(linktype, &linkcontain));
	}
	else
	{ linkcontain=0;
	}
        if ( ( pspider->getimage==2 && 
	       linkcontain==SHARED_HTTP_LINK_CONTAIN_IMAGE )
	     ||
	     ( pspider->getmultimedia==2 &&
	       linkcontain==SHARED_HTTP_LINK_CONTAIN_MULTIMEDIA )
	     ||
	     ( pspider->getcode==2 &&
	       linkcontain==SHARED_HTTP_LINK_CONTAIN_CODE )
	     )
        { candownloadthisurl=LCRZO_TRUE;
	}
	else
        { candownloadthisurl=LCRZO_FALSE;
	}
	maxrecurreached=LCRZO_FALSE;
      }
      else
      { candownloadthisurl=LCRZO_TRUE;
        maxrecurreached=LCRZO_FALSE;
      }
    }
    else
    { if ( pspider->currentrecursionlevel > pspider->maxrecursion )
      { candownloadthisurl=LCRZO_FALSE;
        maxrecurreached=LCRZO_TRUE;
      }
      else
      { candownloadthisurl=LCRZO_TRUE;
        maxrecurreached=LCRZO_FALSE;
      }
    }
  }
  else
  { candownloadthisurl=LCRZO_TRUE;
    maxrecurreached=LCRZO_FALSE;
  }
  /*printf("%ld %ld %d\n", pspider->currentrecursionlevel,
    pspider->maxrecursion, maxrecurreached);*/

  if (purlisarecurmarker!=NULL) *purlisarecurmarker=urlisarecurmarker;
  if (pcandownloadthisurl!=NULL) *pcandownloadthisurl=candownloadthisurl;
  if (pmaxrecurreached!=NULL) *pmaxrecurreached=maxrecurreached;

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_priv_http_spider_isinregexplist(lcrzo_list *plist,
					   lcrzo_conststring str,
					   lcrzo_bool *pyes);
int shared_priv_http_spider_isinregexplist(lcrzo_list *plist,
					   lcrzo_conststring str,
					   lcrzo_bool *pyes)
{ lcrzo_int32 i, count;
  lcrzo_bool yes;
  lcrzo_string strre;
  int ret;

  yes=LCRZO_FALSE;
  lcrzo_er(lcrzo_list_count(plist, &count));
  for ( i=1 ; i<=count ; i++ )
  { lcrzo_er(lcrzo_stringlist_valuem_pos(plist, i, NULL, &strre));
    ret=lcrzo_string_search_re(str, LCRZO_FALSE, +1, -1, strre,
			       NULL, NULL, NULL, NULL);
    lcrzo_er(lcrzo_string_free2(&strre));
    if (ret==LCRZO_ERR_OK)
    { yes=LCRZO_TRUE;
      break;
    } 
  }

  if (pyes!=NULL) *pyes=yes;
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_priv_http_spider_urlmustcontain(shared_http_spider *pspider,
					   lcrzo_conststring url,
					   lcrzo_bool *pcandownload);
int shared_priv_http_spider_urlmustcontain(shared_http_spider *pspider,
					   lcrzo_conststring url,
					   lcrzo_bool *pcandownload)
{ lcrzo_int32 count;
  lcrzo_bool yes;

  lcrzo_er(lcrzo_list_count(&pspider->listurlmustcontain, &count));
  if (count)
  { lcrzo_er(shared_priv_http_spider_isinregexplist(&pspider->listurlmustcontain, url, &yes));
    if (!yes)
    { if (pcandownload!=NULL) *pcandownload=LCRZO_FALSE;
      return(LCRZO_ERR_OK);
    }
  }

  if (pcandownload!=NULL) *pcandownload=LCRZO_TRUE;
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_priv_http_spider_urlmustnotcontain(shared_http_spider *pspider,
					      lcrzo_conststring url,
					      lcrzo_bool *pcandownload);
int shared_priv_http_spider_urlmustnotcontain(shared_http_spider *pspider,
					      lcrzo_conststring url,
					      lcrzo_bool *pcandownload)
{ lcrzo_bool yes;

  lcrzo_er(shared_priv_http_spider_isinregexplist(&pspider->listurlmustnotcontain, url, &yes));
  if (yes)
  { if (pcandownload!=NULL) *pcandownload=LCRZO_FALSE;
    return(LCRZO_ERR_OK);
  }
 
  if (pcandownload!=NULL) *pcandownload=LCRZO_TRUE;
  return(LCRZO_ERR_OK);
}


/*-------------------------------------------------------------*/
int shared_priv_http_spider_run_masterindex(shared_http_spider *pspider);
int shared_priv_http_spider_run_masterindex(shared_http_spider *pspider)
{ lcrzo_string filename, url, txt;
  lcrzo_file file;
  lcrzo_int32 count, i;
  lcrzo_int16 language;

  lcrzo_er(shared_priv_http_spider_log(pspider,
				       "Creation du master index",
				       "Creating master index"));

  lcrzo_er(lcrzo_string_initm_text(pspider->localrootdir, &filename));
  lcrzo_er(lcrzo_string_appendm_text("/index.html", &filename));
  lcrzo_er(lcrzo_file_open_write(filename, LCRZO_FALSE, LCRZO_FALSE, &file));
  lcrzo_er(lcrzo_string_free2(&filename));

  lcrzo_er(lcrzo_global_get_language(&language));
  lcrzo_er(lcrzo_string_initm_text("<HTML><BODY><H1>", &txt))
  if ( language==LCRZO_GLOBAL_LANG_FR )
  { lcrzo_er(lcrzo_string_appendm_text("Url ", &txt));
    lcrzo_er(lcrzo_string_appendm_text("t&eacute;l&eacute;charg&eacute;es ", 
				       &txt));
    lcrzo_er(lcrzo_string_appendm_text("avec lcrzoex :", &txt));
  }
  else
  { lcrzo_er(lcrzo_string_appendm_text("Url downloaded with lcrzoex :", 
				       &txt))
  }
  lcrzo_er(lcrzo_string_appendm_text("</H1>\r\n", &txt))
  lcrzo_er(lcrzo_file_write(&file, (lcrzo_constdata)txt, strlen(txt)));
  lcrzo_er(lcrzo_string_free2(&txt));

  lcrzo_er(lcrzo_list_count(&pspider->listurltodownload, &count));
  for ( i=1 ; i<=count ; i++ )
  { lcrzo_er(lcrzo_stringlist_valuem_pos(&pspider->listurltodownload, i, 
					 NULL, &url));
    lcrzo_er(lcrzo_string_initm_text("<A HREF=\"", &txt));
    lcrzo_er(lcrzo_string_appendm_text(url, &txt));
    lcrzo_er(lcrzo_string_appendm_text("\">", &txt));
    lcrzo_er(lcrzo_string_appendm_text(url, &txt));
    lcrzo_er(lcrzo_string_appendm_text("</A><BR>\r\n", &txt));
    lcrzo_er(lcrzo_file_write(&file, (lcrzo_constdata)txt, strlen(txt)));
    lcrzo_er(lcrzo_string_free2(&txt));
    lcrzo_er(lcrzo_string_free2(&url));
  }

  lcrzo_er(lcrzo_string_initm_text("</BODY></HTML>\r\n", &txt))
  lcrzo_er(lcrzo_file_write(&file, (lcrzo_constdata)txt, strlen(txt)));
  lcrzo_er(lcrzo_string_free2(&txt));

  lcrzo_er(lcrzo_file_close(&file));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_priv_http_test(void);
int shared_http_spider_run(shared_http_spider *pspider)
{ lcrzo_string url, localfile, errstr;
  lcrzo_int32 count;
  lcrzo_uint32 linktype;
  lcrzo_uint16 linkcontain, linkformat;
  lcrzo_uint16 errorcode;
  lcrzo_bool urlisarecurmarker, candownloadthisurl, maxrecurreached, nomoreurl;
  lcrzo_bool alreadydownloaded;
  int ret;

  /*lcrzo_epr(shared_priv_http_test());
    return(0);*/

  lcrzo_er(shared_priv_http_spider_log(pspider,
				       "Debut de shared_http_spider_run",
				       "Starting shared_http_spider_run"));

  /* create master index.html */
  lcrzo_er(shared_priv_http_spider_run_masterindex(pspider));

  /* add a marker */
  lcrzo_er(shared_priv_http_spider_recurmarker_add(pspider, 1));

  while (LCRZO_TRUE)
  { /* count the number of url in the list */
    lcrzo_er(lcrzo_list_count(&pspider->listurltodownload, &count));
    if (count==0)
    { /* there is no more url to download */
      return(LCRZO_ERR_OK);
    }
    /* get the url to download */
    lcrzo_er(lcrzo_stringlist_valuem_pos(&pspider->listurltodownload, +1, 
					  &linktype, &url));
    /* we remove it, even if it might not be correctly downloaded */
    lcrzo_er(lcrzo_list_remove_pos(&pspider->listurltodownload, +1));
    lcrzo_er(shared_priv_http_spider_log(pspider, url, url));

    /* check if this url is a recursion marker */
    lcrzo_er(shared_priv_http_spider_recurmarker_check(pspider,
						       linktype, url,
						       &urlisarecurmarker,
						       &candownloadthisurl,
						       &maxrecurreached,
						       &nomoreurl));
    if (nomoreurl)
    { lcrzo_er(shared_priv_http_spider_log(pspider, 
					   "Plus d'url a telecharger", 
					   "No more url to download"));
      lcrzo_er(lcrzo_string_free2(&url));
      break;
    }
    if (maxrecurreached)
    { lcrzo_er(shared_priv_http_spider_log(pspider, 
					   "Valeur maxrecursion atteinte", 
					   "Value maxrecursion reached"));
      lcrzo_er(lcrzo_string_free2(&url));
      break;
    }
    if (urlisarecurmarker)
    { lcrzo_er(shared_priv_http_spider_log(pspider, 
					   "Ignore les marqueurs de recursion", 
					   "Ignore recursion markers"));
      lcrzo_er(lcrzo_string_free2(&url));
      continue;
    }
    if (!candownloadthisurl)
    { lcrzo_er(shared_priv_http_spider_log(pspider, 
					   "On est a la limite de maxrecursion, et le fichier n'est pas un fichier image, multimedia ou script", 
					   "We are at the limit of maxrecursion, and this file is not an image, multimedia or code file"));
      lcrzo_er(lcrzo_string_free2(&url));
      continue;
    }
    /* check if this linktype was not a reference */
    lcrzo_er(shared_http_linktype_get_contain(linktype, &linkcontain));
    if ( linkcontain == SHARED_HTTP_LINK_CONTAIN_REFERENCE )
    { lcrzo_er(lcrzo_string_free2(&url));
      lcrzo_er(shared_priv_http_spider_log(pspider, 
					   "Ne telecharge pas les references", 
					   "Do not download references"));
      continue;
    }
    /* check what user decided to download */
    if ( pspider->getimage==0 && 
	 linkcontain == SHARED_HTTP_LINK_CONTAIN_IMAGE )
    { lcrzo_er(lcrzo_string_free2(&url));
      lcrzo_er(shared_priv_http_spider_log(pspider, 
					   "Ne telecharge pas les images", 
					   "Do not download images"));
      continue;
    }
    if ( pspider->getmultimedia==0 && 
	 linkcontain == SHARED_HTTP_LINK_CONTAIN_MULTIMEDIA )
    { lcrzo_er(lcrzo_string_free2(&url));
      lcrzo_er(shared_priv_http_spider_log(pspider, 
					   "Ne telecharge pas les multimedias", 
					   "Do not download multimedias"));
      continue;
    }
    if ( pspider->getcode==0 && 
	 linkcontain == SHARED_HTTP_LINK_CONTAIN_CODE )
    { lcrzo_er(lcrzo_string_free2(&url));
      lcrzo_er(shared_priv_http_spider_log(pspider, 
					   "Ne telecharge pas les codes", 
					   "Do not download codes"));
      continue;
    }
    /* check urlmustcontain and urlmustnotcontain */
    lcrzo_er(shared_priv_http_spider_urlmustcontain(pspider, url,
						    &candownloadthisurl));
    if (!candownloadthisurl)
    { lcrzo_er(shared_priv_http_spider_log(pspider, 
			  "L'url ne matche pas la liste urlmustcontain", 
			   "Url does not match urlmustcontain list"));
      lcrzo_er(lcrzo_string_free2(&url));
      continue;
    }
    lcrzo_er(shared_priv_http_spider_urlmustnotcontain(pspider, url,
						       &candownloadthisurl));
    if (!candownloadthisurl)
    { lcrzo_er(shared_priv_http_spider_log(pspider, 
			   "L'url matche la liste urlmustnotcontain", 
			   "Url matches urlmustnotcontain list"));
      lcrzo_er(lcrzo_string_free2(&url));
      continue;
    }
    /* check if we've not already downloaded the file */
    lcrzo_er(shared_priv_http_spider_url_alreadydownloaded(pspider, url,
							   NULL,
							  &alreadydownloaded));
    if (alreadydownloaded)
    { /* found it, so do not download it twice */
      lcrzo_er(lcrzo_string_free2(&url));
      lcrzo_er(shared_priv_http_spider_log(pspider, 
					   "Deja telecharge", 
					   "Already downloaded"));
      continue;
    }
    /* check if the linkformat is url */
    lcrzo_er(shared_http_linktype_get_format(linktype, &linkformat));
    if ( linkformat != SHARED_HTTP_LINK_FORMAT_URL &&
	 linkformat != SHARED_HTTP_LINK_FORMAT_REFRESHURL )
    { lcrzo_er(lcrzo_string_free2(&url));
      lcrzo_er(shared_priv_http_spider_log(pspider, 
	       "Seul SHARED_HTTP_LINK_FORMAT_URL est supporte pour le moment", 
	     "Only SHARED_HTTP_LINK_FORMAT_URL is supported for the moment"));
      continue;
    }

    /* compute the local file corresponding to the url */
    ret=shared_priv_http_spider_url_download(pspider, url, linkcontain,
					     &errorcode, &localfile);
    if (ret==LCRZO_ERR_OK)
    { /* search if it contains new urls */
      lcrzo_er(shared_priv_http_spider_urllist_complete_localfile(pspider, 
								  linktype, 
								  url,
								  localfile));
      lcrzo_er(lcrzo_string_free2(&localfile));
      lcrzo_er(lcrzo_string_initm_uint16(errorcode, "%u", &errstr));
      lcrzo_er(shared_priv_http_spider_log(pspider, errstr, errstr));
      lcrzo_er(lcrzo_string_free2(&errstr));
    }
    else
    { lcrzo_er(lcrzo_err_print(ret));
    }
    /* free */
    lcrzo_er(lcrzo_string_free2(&url));
  }

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_priv_http_spider_linkconversion2_url(shared_http_spider *pspider,
						lcrzo_conststring filename,
						lcrzo_string *purl);
int shared_priv_http_spider_linkconversion2_url(shared_http_spider *pspider,
						lcrzo_conststring filename,
						lcrzo_string *purl)
{ lcrzo_string alp;
  int strlenalp, strlenend;

  lcrzo_er(shared_priv_http_path_relative(filename, pspider->localrootdir, 
					 &alp));

  /* if ends with .errorlcrzoex.html or indexlcrzoex.html, remove this */
  strlenalp=strlen(alp);
  strlenend=strlen(SHARED_PRIV_HTTP_SPIDER_ERRORINDEX);
  if ( strlenalp > strlenend )
  { if ( !strcmp(alp+strlenalp-strlenend, SHARED_PRIV_HTTP_SPIDER_ERRORINDEX) )
    { alp[strlenalp-strlenend]='\0';
    }
  }
  strlenalp=strlen(alp);
  strlenend=strlen(SHARED_PRIV_HTTP_SPIDER_HTMLINDEX);
  if ( strlenalp > strlenend )
  { if ( !strcmp(alp+strlenalp-strlenend, 
		 SHARED_PRIV_HTTP_SPIDER_HTMLINDEX) )
    { alp[strlenalp-strlenend]='\0';
    }
  }
  strlenalp=strlen(alp);
  strlenend=strlen(SHARED_PRIV_HTTP_SPIDER_DEFAULTINDEX);
  if ( strlenalp > strlenend )
  { if ( !strcmp(alp+strlenalp-strlenend, 
		 SHARED_PRIV_HTTP_SPIDER_DEFAULTINDEX) )
    { alp[strlenalp-strlenend]='\0';
    }
  }

  /* alp always start with "./" */
  if (strlen(alp)<2)
  { lcrzo_er(lcrzo_string_free2(&alp));
    return(LCRZO_ERR_OKNOTDECODED);
  }

  lcrzo_er(shared_http_url_init_alp(alp+2, purl));
  lcrzo_er(lcrzo_string_free2(&alp));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_priv_http_spider_linkconversion2(shared_http_spider *pspider,
					    lcrzo_conststring filename);
int shared_priv_http_spider_linkconversion2(shared_http_spider *pspider,
					    lcrzo_conststring filename)
{ lcrzo_string urlwheredpwas, dpurl, url, replacewiththeurl, filewereput;
  lcrzo_string alocalfile, newfilename, anchor;
  lcrzo_file oldfile, newfile;
  lcrzo_list list;
  lcrzo_int32 startpos, endpos, lastreadpos, count, i;
  lcrzo_data data;
  lcrzo_int32 datasize;
  lcrzo_uint32 randnum, dpurllinktype, metacontentrefresh;
  lcrzo_uint16 dpurlformat, dpurlcontain;
  lcrzo_bool alreadydownloaded, addanchor;
  int ret;

  lcrzo_er(shared_priv_http_spider_log(pspider, filename, filename));
  
  /* obtain the list of url contained in this file */
  ret=shared_http_urllist_init_file(filename, &list);
  if (ret!=LCRZO_ERR_OK)
  { return(LCRZO_ERR_OK);
  }
  lcrzo_er(lcrzo_list_count(&list, &count));
  if (count==0) 
  { lcrzo_er(lcrzo_list_close(&list));
    return(LCRZO_ERR_OK);
  }

  /* determine the url associated with this file */
  lcrzo_er(shared_priv_http_spider_linkconversion2_url(pspider, filename, 
						       &urlwheredpwas));

  /* open old file */
  lcrzo_er(lcrzo_file_open_read(filename, &oldfile));

  /* create the new file */
  lcrzo_er(lcrzo_string_initm_text(filename, &newfilename));
  lcrzo_er(lcrzo_uint32_init_rand(1, 0xFFFFFFFFu, &randnum));
  lcrzo_er(lcrzo_string_appendm_text(".lcrzoex_", &newfilename));
  lcrzo_er(lcrzo_string_appendm_uint32(randnum, "%08X", &newfilename));
  lcrzo_er(lcrzo_file_open_write(newfilename, LCRZO_FALSE, LCRZO_FALSE, 
				 &newfile));

  /* sort the list */
  lcrzo_er(lcrzo_list_sort_all2(&list, &shared_http_urllist_cmp, NULL));
  /* for each url in the file */
  lastreadpos=0;
  for ( i=1 ; i<=count ; i++ )
  { lcrzo_er(shared_http_urllist_valuem_pos(&list, i, &dpurllinktype, &dpurl, 
					    &startpos, &endpos,
					    &metacontentrefresh));
    /* only recognize this format */
    lcrzo_er(shared_http_linktype_get_format(dpurllinktype, &dpurlformat));
    if ( dpurlformat!=SHARED_HTTP_LINK_FORMAT_URL &&
	 dpurlformat!=SHARED_HTTP_LINK_FORMAT_REFRESHURL )
    { lcrzo_er(lcrzo_string_free2(&dpurl));
      continue;
    }
    /* get the url */
    /*printf("+%s +%s\n", urlwheredpwas, dpurl);*/
    ret=shared_http_url_init_dpurl(urlwheredpwas, dpurl, &url);
    lcrzo_er(lcrzo_string_free2(&dpurl));
    if ( ret!=LCRZO_ERR_OK )
    { /* ignore this dpurl */
      continue;
    }

    /* compute the url which will replace */
    addanchor=LCRZO_FALSE;
    lcrzo_er(shared_http_linktype_get_contain(dpurllinktype, &dpurlcontain));
    if ( dpurlcontain==SHARED_HTTP_LINK_CONTAIN_REFERENCE )
    { lcrzo_er(shared_priv_http_spider_localfile_init_url(pspider, url, 
							  dpurlcontain,
							  &alocalfile));
      lcrzo_er(shared_priv_http_path_relative(alocalfile, filename, 
					     &replacewiththeurl));
      lcrzo_er(lcrzo_string_free2(&alocalfile));
      addanchor=LCRZO_TRUE;
    }
    else
    { /* if this url is already downloaded, use its relative path */
      lcrzo_er(shared_priv_http_spider_url_alreadydownloaded(pspider, url,
							     &filewereput,
							  &alreadydownloaded));
      if (alreadydownloaded)
      { lcrzo_er(shared_http_rlp_init_alp(filewereput, filename,
					  &replacewiththeurl));
        lcrzo_er(lcrzo_string_free2(&filewereput));
	addanchor=LCRZO_TRUE;
      }
      /* else, use a the true link, or a fake one */
      else
      { if (pspider->linkconversion==1) 
	{ lcrzo_er(lcrzo_string_initm_text(url, &replacewiththeurl));
	}
        else
        { lcrzo_er(shared_priv_http_spider_localfile_init_url(pspider, url, 
							      dpurlcontain,
							      &alocalfile));
          lcrzo_er(shared_priv_http_path_relative(alocalfile, filename, 
						 &replacewiththeurl));
	  lcrzo_er(lcrzo_string_free2(&alocalfile));
	  addanchor=LCRZO_TRUE;
	}
      }
    }
    /* eventually add the anchor */
    if ( addanchor )
    { lcrzo_er(shared_http_url_decode(url, NULL, NULL, NULL, NULL, NULL,
				      NULL, NULL, &anchor));
      if (strlen(anchor))
      { lcrzo_er(lcrzo_string_appendm_text("#", &replacewiththeurl));
        lcrzo_er(lcrzo_string_appendm_text(anchor, &replacewiththeurl));
      }
      lcrzo_er(lcrzo_string_free2(&anchor));
    }
    /*printf("%s->%s\n", url, replacewiththeurl);*/
    lcrzo_er(lcrzo_string_free2(&url));

    /* now replace */
    /* read data from old file */
    datasize=startpos-lastreadpos;
    lcrzo_er(lcrzo_file_readm_fixed(&oldfile, LCRZO_WAIT_INFINITE,
				    datasize, &data));
    lcrzo_er(lcrzo_file_write(&newfile, data, datasize));
    lcrzo_er(lcrzo_data_free2(&data));
    /* set new position and skip it */
    lcrzo_er(lcrzo_file_readm_fixed(&oldfile, LCRZO_WAIT_INFINITE,
				    endpos-startpos+1, NULL));
    lastreadpos=endpos+1;
    /* write the url */
    if ( dpurllinktype==SHARED_HTTP_LINKTYPE_BASE_HREF )
    { /* base href is not necessary anymore */
      lcrzo_er(lcrzo_data_initm_text("HREF_SUPPRESSED_BY_LCRZOEX", 
				     &data, &datasize));
    }
    else if ( dpurlformat==SHARED_HTTP_LINK_FORMAT_REFRESHURL )
    { lcrzo_er(lcrzo_data_initm_text("\"", &data, &datasize));
      lcrzo_er(lcrzo_data_appendm_uint32(metacontentrefresh, "%u",
					 datasize, &data, &datasize));
      lcrzo_er(lcrzo_data_appendm_text(";url=", datasize, &data, &datasize));
      lcrzo_er(lcrzo_data_appendm_text(replacewiththeurl, datasize, 
				       &data, &datasize));
      lcrzo_er(lcrzo_data_appendm_text("\"", datasize, &data, &datasize));
    }
    else /*if ( dpurlformat==SHARED_HTTP_LINK_FORMAT_URL ||
	      dpurlformat==SHARED_HTTP_LINK_FORMAT_URLLISTCOMMA ||
	      dpurlformat==SHARED_HTTP_LINK_FORMAT_URLLISTSPACE )*/
    { lcrzo_er(lcrzo_data_initm_text("\"", &data, &datasize));
      lcrzo_er(lcrzo_data_appendm_text(replacewiththeurl, datasize, 
				       &data, &datasize));
      lcrzo_er(lcrzo_data_appendm_text("\"", datasize, &data, &datasize));
    }
    lcrzo_er(lcrzo_file_write(&newfile, data, datasize));
    lcrzo_er(lcrzo_data_free2(&data));

    /* end */
    lcrzo_er(lcrzo_string_free2(&replacewiththeurl));
  }
  lcrzo_er(lcrzo_string_free2(&urlwheredpwas));
  lcrzo_er(lcrzo_list_close(&list));

  /* terminate copy */
  while (LCRZO_TRUE)
  { ret=lcrzo_file_readm(&oldfile, LCRZO_WAIT_INFINITE, &data, &datasize);
    if (ret!=LCRZO_ERR_OK) break;
    lcrzo_er(lcrzo_file_write(&newfile, data, datasize));
    lcrzo_er(lcrzo_data_free2(&data));
  }

  lcrzo_er(lcrzo_file_close(&oldfile));
  lcrzo_er(lcrzo_file_close(&newfile));

  /* rename file */
  lcrzo_er(lcrzo_filename_rename(newfilename, filename));
  lcrzo_er(lcrzo_string_free2(&newfilename));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_priv_http_spider_linkconversion(shared_http_spider *pspider,
					   lcrzo_conststring dirname);
int shared_priv_http_spider_linkconversion(shared_http_spider *pspider,
					   lcrzo_conststring dirname)
{ lcrzo_int32 count, i;
  lcrzo_string filename, tmpfilename;
  lcrzo_int16 filetype;
  int ret;

  lcrzo_er(shared_priv_http_spider_log(pspider, dirname, dirname));

  lcrzo_er(lcrzo_dir_count(dirname, &count));
  for ( i=1 ; i<=count; i++ )
  { lcrzo_er(lcrzo_dir_value_pos(dirname, i, &filename));
    lcrzo_er(lcrzo_string_initm_text(dirname, &tmpfilename));
    lcrzo_er(lcrzo_string_appendm_text("/", &tmpfilename));
    lcrzo_er(lcrzo_string_appendm_text(filename, &tmpfilename));
    lcrzo_er(lcrzo_string_free2(&filename));
    lcrzo_er(shared_priv_http_path_clean(tmpfilename, &filename));
    lcrzo_er(lcrzo_string_free2(&tmpfilename));
    ret=lcrzo_path_type(filename, &filetype);
    if (ret==LCRZO_ERR_OK)
    { if (filetype==LCRZO_PATH_TYPE_REG)
      { lcrzo_er(shared_priv_http_spider_linkconversion2(pspider, filename));
      }
      else if (filetype==LCRZO_PATH_TYPE_DIR)
      { lcrzo_er(shared_priv_http_spider_linkconversion(pspider, filename));
      }
    }
    lcrzo_er(lcrzo_string_free2(&filename));
  }

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_spider_linkconversion(shared_http_spider *pspider)
{

  lcrzo_er(shared_priv_http_spider_log(pspider,
		       	       "Debut de shared_http_spider_linkconversion",
			       "Starting shared_http_spider_linkconversion"));

  /* first empty lists, to free memory */
  lcrzo_er(lcrzo_list_remove_all(&(pspider->listurltodownload)));
  lcrzo_er(lcrzo_list_remove_all(&(pspider->listurlmustcontain)));
  lcrzo_er(lcrzo_list_remove_all(&(pspider->listurlmustnotcontain)));

  /* if (user does not want to convert links) */
  if (pspider->linkconversion==0) return(LCRZO_ERR_OK);

  /* convert files in the directory */
  lcrzo_er(shared_priv_http_spider_linkconversion(pspider, 
						  pspider->localrootdir));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_spider_close(shared_http_spider *pspider)
{

  lcrzo_er(lcrzo_string_free2(&(pspider->proxyurl)));
  lcrzo_er(lcrzo_string_free2(&(pspider->localrootdir)));
  lcrzo_er(lcrzo_string_free2(&(pspider->useragent)));
  lcrzo_er(lcrzo_list_close(&(pspider->listurltodownload)));
  lcrzo_er(lcrzo_list_close(&(pspider->listurlmustcontain)));
  lcrzo_er(lcrzo_list_close(&(pspider->listurlmustnotcontain)));
  lcrzo_er(shared_priv_http_spider_log(pspider, 
				       "Fermeture du spider", 
				       "Closing spider"));
  if (pspider->logfileinitialized)
  { lcrzo_er(lcrzo_file_close(&(pspider->logfile)));
  }

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_priv_http_test(void);
int shared_priv_http_test(void)
{ 
  puts("--> shared_http_string_initm_he_test");
  lcrzo_evr(shared_http_string_initm_he_test());
  puts("--> shared_http_he_initm_string_test");
  lcrzo_evr(shared_http_he_initm_string_test());
  puts("--> shared_http_varval_decode_test");
  lcrzo_evr(shared_http_varval_decode_test());
  puts("--> shared_priv_http_url_init_url_test");
  lcrzo_evr(shared_priv_http_url_init_url_test());
  puts("--> shared_priv_http_url_init_dp_test");
  lcrzo_evr(shared_priv_http_url_init_dp_test());
  puts("--> shared_priv_http_url_init_alp_test");
  lcrzo_evr(shared_priv_http_url_init_alp_test());
  puts("--> shared_priv_http_dp_init_url_test");
  lcrzo_evr(shared_priv_http_dp_init_url_test());
  puts("--> shared_priv_http_alp_init_url_test");
  lcrzo_evr(shared_priv_http_alp_init_url_test());
  puts("--> shared_priv_http_alp_init_rlp_test");
  lcrzo_evr(shared_priv_http_alp_init_rlp_test());
  puts("--> shared_priv_http_rlp_init_alp_test");
  lcrzo_evr(shared_priv_http_rlp_init_alp_test());
  puts("--> shared_priv_http_path_relative_test");
  lcrzo_evr(shared_priv_http_path_relative_test());

  return(LCRZO_ERR_OK);
}
